@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: TestCxxFeatures.cxx,v $
+  Module:    TestCxxFeatures.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -112,6 +112,37 @@ using namespace NamespaceTest;
 
 //----------------------------------------------------------------------------
 
+/* Test nested classes defined outside.  */
+
+class NestedTestOuter
+{
+public:
+  NestedTestOuter();
+  ~NestedTestOuter();
+private:
+  class NestedTestInner;
+  NestedTestInner* Inner;
+};
+
+class NestedTestOuter::NestedTestInner
+{
+public:
+  NestedTestInner() {}
+  ~NestedTestInner() {}
+};
+
+NestedTestOuter::NestedTestOuter()
+{
+  this->Inner = new NestedTestInner;
+}
+
+NestedTestOuter::~NestedTestOuter()
+{
+  delete this->Inner;
+}
+
+//----------------------------------------------------------------------------
+
 /* Test inclusion of some stl headers.  */
 #ifdef _MSC_VER
 #pragma warning (push, 2)
@@ -127,7 +158,7 @@ using namespace NamespaceTest;
 // Fails on kulu.crd IRIX64-6.5-CC-o32 (old SGI compiler).
 void UsingStdVector()
 {
-  using vtkstd::vector;
+  using std::vector;
   vector<int>();
 }
 #endif
@@ -179,6 +210,51 @@ int TestFullySpecializedFunction()
 
 //----------------------------------------------------------------------------
 
+/* Test member template of non-template.  */
+
+class NonTemplate
+{
+  void* Pointer;
+public:
+  template <class T> void Set(T* t) { this->Pointer = t; }
+  template <class T> void Get(T*& t) { t = static_cast<T*>(this->Pointer); }
+};
+
+int TestNonTemplateMemberTemplate()
+{
+  int x = 123;
+  int* px = 0;
+  NonTemplate nt;
+  nt.Set(&x);
+  nt.Get(px);
+  return (*px == 123);
+}
+
+//----------------------------------------------------------------------------
+
+/* Test member template of template.  */
+
+template <class T>
+class OuterTemplate
+{
+  T* Pointer;
+public:
+  template <class U> void Set(U* u) { this->Pointer = u; }
+  template <class U> void Get(U*& u) { u = static_cast<U*>(this->Pointer); }
+};
+
+int TestTemplateMemberTemplate()
+{
+  int x = 123;
+  int* px = 0;
+  OuterTemplate<void> nt;
+  nt.Set(&x);
+  nt.Get(px);
+  return (*px == 123);
+}
+
+//----------------------------------------------------------------------------
+
 /* Test use of standard "bool" type and values.  */
 
 #if !defined(VTK_CXX_SGI_6)
@@ -441,6 +517,126 @@ int TestSafeBoolIdiom()
   return result;
 }
 
+//----------------------------------------------------------------------------
+
+/* Test use of exceptions.  */
+
+#if defined(_MSC_VER)
+# pragma warning (push)
+# pragma warning (disable: 4702) /* Unreachable code. */
+#endif
+
+class TestExceptionUnwind
+{
+  int* pvalue;
+public:
+  TestExceptionUnwind(int* p): pvalue(p) {}
+  ~TestExceptionUnwind() { *pvalue = 1; }
+  void Use() {}
+};
+
+class ExceptionClass {};
+
+void TestThrowException(int* p)
+{
+  TestExceptionUnwind unwind(p);
+  unwind.Use();
+  throw ExceptionClass();
+}
+
+int TestException()
+{
+  int value = 0;
+  try
+    {
+    TestThrowException(&value);
+    }
+  catch(ExceptionClass&)
+    {
+    if(value)
+      {
+      return 1;
+      }
+    else
+      {
+      cerr << "TestExceptionUnwind object not destroyed!" << endl;
+      return 0;
+      }
+    }
+  catch(...)
+    {
+    cerr << "ExceptionClass not caught!" << endl;
+    return 0;
+    }
+  cerr << "No exception caught!" << endl;
+  return 0;
+}
+
+#if defined(_MSC_VER)
+# pragma warning (pop)
+#endif
+
+//----------------------------------------------------------------------------
+
+/* Test void return type syntax.  */
+
+void TestVoidReturnInner() {}
+void TestVoidReturnOuter()
+{
+  // Visual Studio 6 and MIPSpro 7.3 do not support void returns.
+#if !(defined(_MSC_VER) && (_MSC_VER < 1300) || defined(_COMPILER_VERSION) && (_COMPILER_VERSION < 740))
+  return TestVoidReturnInner();
+#endif
+}
+
+// MIPSpro warns about type qualifiers on return types.
+#if defined(_COMPILER_VERSION)
+# pragma set woff 3303 // type qualifier on return is meaningless
+#endif
+// Intel C++ warns about type qualifiers on return types.
+#if defined(__INTEL_COMPILER)
+# pragma warning (push)
+# pragma warning (disable:858) // type qualifier on return is meaningless
+#endif
+
+// clang warns about type qualifiers on return types.
+#if defined(__clang__)
+# pragma clang diagnostic push
+# pragma clang diagnostic ignored "-Wignored-qualifiers"
+#endif
+
+// gcc>=4.3 says type qualifiers ignored on function return type
+#if defined(__GNUC__) && ((__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ >= 6)))
+# pragma GCC diagnostic push
+#endif
+#if defined(__GNUC__) && ((__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ >= 3)))
+# pragma GCC diagnostic ignored "-Wignored-qualifiers"
+#endif
+
+// aCC warns "type qualifier on return type is meaningless" - just omit the
+// function on aCC builds since there is no way to suppress the warning via
+// pragmas...
+#if !defined(__HP_aCC)
+void const TestVoidConstReturn() {}
+#endif
+
+#if defined(__GNUC__) && ((__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ >= 6)))
+# pragma GCC diagnostic pop
+#endif
+
+#if defined(__clang__)
+# pragma clang diagnostic pop
+#endif
+
+#if defined(__INTEL_COMPILER)
+# pragma warning (pop)
+#endif
+
+#if defined(_COMPILER_VERSION)
+# pragma reset woff 3303 // type qualifier on return is meaningless
+#endif
+
+
 //-------------------------------------------------------------------
 // See if the following code works on all platforms
 #if defined(_MSC_VER) && defined(_DEBUG)
@@ -454,6 +650,26 @@ static int TestDriverDebugReport(int type, char* message, int* retVal)
 }
 #endif
 
+
+//----------------------------------------------------------------------------
+
+/* Test setlocale  */
+#include <locale.h>
+int TestSetLocale()
+{
+  char *oldLocale = strdup(setlocale(LC_NUMERIC,NULL));
+  setlocale(LC_NUMERIC,"English");
+  
+  // restore the local
+  if (oldLocale)
+    {
+    setlocale(LC_NUMERIC,oldLocale);
+    free(oldLocale);
+    return 1;
+    }
+  return 0;
+}
+
 //----------------------------------------------------------------------------
 
 #define DO_TEST(x) \
@@ -464,6 +680,8 @@ int main()
 {
   int result = 0;
   DO_TEST(TestFullySpecializedFunction);
+  DO_TEST(TestNonTemplateMemberTemplate);
+  DO_TEST(TestTemplateMemberTemplate);
 #if !defined(VTK_CXX_SGI_6)
   DO_TEST(TestBool);
 #endif
@@ -475,12 +693,17 @@ int main()
 #endif
   DO_TEST(TestBinaryWriting);
   DO_TEST(TestSafeBoolIdiom);
+#ifndef VTK_CXX_GCC_2 // avoid strange exception problem on debian gcc 2.95
+  DO_TEST(TestException);
+#endif
+  DO_TEST(TestSetLocale);
+  
 #if defined(_MSC_VER) && defined(_DEBUG)
   // just call the code to shut up a linker warning
   int retVal = 0;
   if (result)
     {
-    // really shouldn't be calle dunless somehting else failed
+    // really shouldn't be called unless something else failed
     // just want to make the compiler think it might get called
     // all this will be yanked once I see the results of this test
     TestDriverDebugReport(0, "a temp test", &retVal);
