@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: otherArrays.cxx,v $
+  Module:    otherArrays.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -26,6 +26,8 @@
 #include "vtkDoubleArray.h"
 #include "vtkIdTypeArray.h"
 
+#include <vtksys/ios/sstream>
+
 #define SIZE 1000
 
 template <class T, class A, class V>
@@ -121,7 +123,7 @@ int doArrayTest (ostream& strm, T *ptr, A *array, V value, int size)
     }
 
   strm << "\tvtkDataArray::GetTuple(i, double *tuple)...";
-  ptr->vtkDataArray::GetTuple (4, tuple3);
+  static_cast<vtkDataArray*>(ptr)->GetTuple (4, tuple3);
   passed = 1;
   for (i = 0; i < 10; i++)
     {
@@ -666,9 +668,7 @@ int otherArraysTest(ostream& strm)
 
 int otherArrays(int, char *[])
 {
-  vtkDebugLeaks::PromptUserOff();
-
-  ostrstream vtkmsg_with_warning_C4701; 
+  vtksys_ios::ostringstream vtkmsg_with_warning_C4701; 
 //  return otherArraysTest(vtkmsg_with_warning_C4701);
   return otherArraysTest(cerr);
 
