@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: otherTimerLog.cxx,v $
+  Module:    otherTimerLog.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -20,13 +20,17 @@
 #include "vtkTimerLog.h"
 #include "vtkDebugLeaks.h"
 
+#include <vtksys/ios/sstream>
+
 // this is needed for the unlink call
 #if defined(__CYGWIN__)
 #include <sys/unistd.h>
 #elif defined(_WIN32)
-#include <io.h>
+# include <io.h>
 #endif
 
+#include "vtkWindows.h" // for Sleep
+
 void otherTimerLogTest(ostream& strm)
 {
   // actual test
@@ -57,6 +61,7 @@ void otherTimerLogTest(ostream& strm)
   strm << "GetCPUTime: " << timer1->GetCPUTime() << endl;
   timer1->DumpLog( "timing" );
   timer1->ResetLog ();
+  timer1->CleanupLog();
   unlink("timing");
   timer1->Delete();
   strm << "Test vtkTimerLog End" << endl;
@@ -65,9 +70,7 @@ void otherTimerLogTest(ostream& strm)
 
 int otherTimerLog(int,char *[])
 {
-  vtkDebugLeaks::PromptUserOff();
-
-  ostrstream vtkmsg_with_warning_C4701; 
+  vtksys_ios::ostringstream vtkmsg_with_warning_C4701; 
   otherTimerLogTest(vtkmsg_with_warning_C4701);
 
   return 0;
