@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkAmoebaMinimizer.cxx,v $
+  Module:    vtkAmoebaMinimizer.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,7 +15,9 @@
 #include "vtkAmoebaMinimizer.h"
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkAmoebaMinimizer, "$Revision: 1.2 $");
+#define  N_STEPS_NO_VALUE_IMPROVEMENT  2
+#define  N_STEPS_NO_PARAM_IMPROVEMENT  18
+
 vtkStandardNewMacro(vtkAmoebaMinimizer);
 
 //----------------------------------------------------------------------------
@@ -32,7 +34,11 @@ vtkAmoebaMinimizer::vtkAmoebaMinimizer()
 
   this->FunctionValue = 0.0;
 
+  this->ContractionRatio = 0.5;
+  this->ExpansionRatio = 2.0;
+
   this->Tolerance = 1e-4;
+  this->ParameterTolerance = 1e-4;
   this->MaxIterations = 1000;
   this->Iterations = 0;
   this->FunctionEvaluations = 0;
@@ -41,6 +47,8 @@ vtkAmoebaMinimizer::vtkAmoebaMinimizer()
   this->AmoebaVertices = NULL;
   this->AmoebaValues = NULL;
   this->AmoebaSum = NULL;
+  this->AmoebaSize = 0;
+  this->AmoebaHighValue = 0;
   this->AmoebaNStepsNoImprovement = 0;
 }
   
@@ -86,7 +94,7 @@ vtkAmoebaMinimizer::~vtkAmoebaMinimizer()
 //----------------------------------------------------------------------------
 void vtkAmoebaMinimizer::PrintSelf(ostream& os, vtkIndent indent)
 {
-  this->vtkObject::PrintSelf(os, indent);
+  this->Superclass::PrintSelf(os, indent);
   os << indent << "NumberOfParameters: " << this->GetNumberOfParameters() << "\n";
   if (this->NumberOfParameters > 0)
     {
@@ -131,6 +139,9 @@ void vtkAmoebaMinimizer::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Iterations: " << this->GetIterations() << "\n";
   os << indent << "MaxIterations: " << this->GetMaxIterations() << "\n";
   os << indent << "Tolerance: " << this->GetTolerance() << "\n";
+  os << indent << "ParameterTolerance: " << this->GetParameterTolerance() << "\n";
+  os << indent << "ContractionRatio: " << this->GetContractionRatio() << "\n";
+  os << indent << "ExpansionRatio: " << this->GetExpansionRatio() << "\n";
 }
 
 //----------------------------------------------------------------------------
@@ -314,6 +325,7 @@ void vtkAmoebaMinimizer::Initialize()
   this->NumberOfParameters = 0;
   this->Iterations = 0;
   this->FunctionEvaluations = 0;
+  this->AmoebaSize = 0;
 
   this->Modified();
 }
@@ -328,6 +340,40 @@ void vtkAmoebaMinimizer::EvaluateFunction()
   this->FunctionEvaluations++;
 }
 
+//----------------------------------------------------------------------------
+int vtkAmoebaMinimizer::CheckParameterTolerance()
+{
+  int n = this->NumberOfParameters;
+
+  double *vertex0 = this->AmoebaVertices[0];
+  double *scales = this->ParameterScales;
+  double size = 0;
+
+  for (int i = 1; i <= n; i++)
+    {
+    double *vertex = this->AmoebaVertices[i];
+    for (int j = 0; j < n; j++)
+      {
+      double d = fabs((vertex[j] - vertex0[j])/scales[j]);
+      size = ((d < size) ? size : d);
+      }
+    }
+
+  if (size != this->AmoebaSize)
+    {
+    this->AmoebaNStepsNoImprovement = N_STEPS_NO_VALUE_IMPROVEMENT-1;
+    }
+  this->AmoebaSize = size;
+  // if amoeba is static, only make a set number of tries
+  if (this->AmoebaNStepsNoImprovement >
+      (N_STEPS_NO_VALUE_IMPROVEMENT + N_STEPS_NO_PARAM_IMPROVEMENT))
+    {
+    return 1;
+    }
+
+  return (size <= this->ParameterTolerance);
+}
+
 //----------------------------------------------------------------------------
 int vtkAmoebaMinimizer::Iterate()
 {
@@ -342,10 +388,15 @@ int vtkAmoebaMinimizer::Iterate()
     }
 
   int improved = this->PerformAmoeba();
+  int paramsWithinTol = 0;
+  if (!improved)
+    {
+    paramsWithinTol = this->CheckParameterTolerance();
+    }
   this->GetAmoebaParameterValues();
   this->Iterations++;
 
-  return improved;
+  return (improved || !paramsWithinTol);
 }
 
 //----------------------------------------------------------------------------
@@ -366,7 +417,10 @@ void vtkAmoebaMinimizer::Minimize()
     int improved = this->PerformAmoeba();
     if (!improved)
       {
-      break;
+      if (this->CheckParameterTolerance())
+        {
+        break;
+        }
       }
     }
 
@@ -393,10 +447,6 @@ void vtkAmoebaMinimizer::Minimize()
   conventions, and the formatting has been changed.
 */
 
-#define  VTK_AMOEBA_FLIP_RATIO      1.0
-#define  VTK_AMOEBA_CONTRACT_RATIO  0.5
-#define  VTK_AMOEBA_STRETCH_RATIO   2.0
-
 /* ----------------------------- MNI Header -----------------------------------
 @NAME       : vtkAmoebaNumericallyClose
 @INPUT      : n1
@@ -499,7 +549,13 @@ void  vtkAmoebaMinimizer::InitializeAmoeba()
         }
       this->AmoebaSum[j] += this->ParameterValues[j];
       }
-
+    }
+  for( i = 0 ; i < n_parameters+1 ; i++ )
+    {
+    for( j = 0; j < n_parameters; j++ )
+      {
+      this->ParameterValues[j] = this->AmoebaVertices[i][j];
+      }
     this->EvaluateFunction();
     this->AmoebaValues[i] = this->FunctionValue;
     }
@@ -630,12 +686,11 @@ double  vtkAmoebaMinimizer::TryAmoeba(double  sum[],
   return( y_try );
 }
 
-#define  N_STEPS_NO_IMPROVEMENT  20
-
 /* ----------------------------- MNI Header -----------------------------------
 @NAME       : PerformAmoeba
 @INPUT      : 
 @OUTPUT     : 
+          
 @RETURNS    : true if numerically significant improvement
 @DESCRIPTION: Performs one iteration of an amoeba, returning true if a
               numerically significant improvement has been found recently.
@@ -687,12 +742,13 @@ int vtkAmoebaMinimizer::PerformAmoeba()
       }
     }
 
-  if( vtkAmoebaNumericallyClose( this->AmoebaValues[low], 
+  if( this->AmoebaValues[high] == this->AmoebaHighValue ||
+      vtkAmoebaNumericallyClose( this->AmoebaValues[low],
                                  this->AmoebaValues[high],
                                  this->Tolerance ) )
     {
     ++this->AmoebaNStepsNoImprovement;
-    if( this->AmoebaNStepsNoImprovement >= N_STEPS_NO_IMPROVEMENT )
+    if( this->AmoebaNStepsNoImprovement >= N_STEPS_NO_VALUE_IMPROVEMENT )
       {
       improvement_found = 0;
       }
@@ -702,16 +758,18 @@ int vtkAmoebaMinimizer::PerformAmoeba()
     this->AmoebaNStepsNoImprovement = 0;
     }
 
-  y_try = this->TryAmoeba( this->AmoebaSum, high, -VTK_AMOEBA_FLIP_RATIO );
+  this->AmoebaHighValue = this->AmoebaValues[high];
+
+  y_try = this->TryAmoeba( this->AmoebaSum, high, -1.0 );
 
   if( y_try <= this->AmoebaValues[low] )
     {
-    y_try = TryAmoeba( this->AmoebaSum, high, VTK_AMOEBA_STRETCH_RATIO );
+    TryAmoeba( this->AmoebaSum, high, this->ExpansionRatio );
     }
   else if( y_try >= this->AmoebaValues[next_high] )
     {
     y_save = this->AmoebaValues[high];
-    y_try = TryAmoeba( this->AmoebaSum, high, VTK_AMOEBA_CONTRACT_RATIO );
+    y_try = TryAmoeba( this->AmoebaSum, high, this->ContractionRatio );
 
     if( y_try >= y_save )
       {
@@ -722,7 +780,7 @@ int vtkAmoebaMinimizer::PerformAmoeba()
           for( j = 0 ; j < this->NumberOfParameters ; j++ )
             {
             this->ParameterValues[j] = (this->AmoebaVertices[i][j] +
-                                        this->AmoebaVertices[low][j]) / 2.0f;
+                                        this->AmoebaVertices[low][j]) / 2.0;
             this->AmoebaVertices[i][j] = this->ParameterValues[j];
             }
           
