@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkAssemblyPath.cxx,v $
+  Module:    vtkAssemblyPath.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,7 +19,6 @@
 #include "vtkObjectFactory.h"
 #include "vtkProp.h"
 
-vtkCxxRevisionMacro(vtkAssemblyPath, "$Revision: 1.7 $");
 vtkStandardNewMacro(vtkAssemblyPath);
 
 vtkAssemblyPath::vtkAssemblyPath()
@@ -41,7 +40,7 @@ vtkAssemblyPath::~vtkAssemblyPath()
 void vtkAssemblyPath::AddNode(vtkProp *p, vtkMatrix4x4 *m)
 {
   vtkAssemblyNode *n = vtkAssemblyNode::New();
-  n->SetProp(p);
+  n->SetViewProp(p);
   n->SetMatrix(m); //really a copy because we're gonna compute with it
   this->AddNode(n);
   n->Delete(); //ok reference counted
@@ -50,7 +49,7 @@ void vtkAssemblyPath::AddNode(vtkProp *p, vtkMatrix4x4 *m)
 void vtkAssemblyPath::AddNode(vtkAssemblyNode *n) 
 {
   // First add the node to the list
-  this->vtkCollection::AddItem((vtkObject *)n);
+  this->vtkCollection::AddItem(n);
   
   // Grab the matrix, if any, and concatenate it
   this->Transform->Push(); //keep in synch with list of nodes
@@ -64,7 +63,7 @@ void vtkAssemblyPath::AddNode(vtkAssemblyNode *n)
 
 vtkAssemblyNode *vtkAssemblyPath::GetNextNode() 
 { 
-  return (vtkAssemblyNode *)(this->GetNextItemAsObject());
+  return static_cast<vtkAssemblyNode *>(this->GetNextItemAsObject());
 }
 
 vtkAssemblyNode *vtkAssemblyPath::GetFirstNode() 
@@ -75,7 +74,7 @@ vtkAssemblyNode *vtkAssemblyPath::GetFirstNode()
     }
   else
     {
-    return (vtkAssemblyNode *)(this->Top->Item);
+    return static_cast<vtkAssemblyNode *>(this->Top->Item);
     }
 }
 
@@ -87,7 +86,7 @@ vtkAssemblyNode *vtkAssemblyPath::GetLastNode()
     }
   else
     {
-    return (vtkAssemblyNode *)(this->Bottom->Item);
+    return static_cast<vtkAssemblyNode *>(this->Bottom->Item);
     }
 }
 
@@ -106,7 +105,7 @@ void vtkAssemblyPath::ShallowCopy(vtkAssemblyPath *path)
   this->RemoveAllItems();
   for ( path->InitTraversal(); (node = path->GetNextNode()); )
     {
-    this->vtkCollection::AddItem((vtkObject *)node);
+    this->vtkCollection::AddItem(node);
     }
 }
 
@@ -131,5 +130,3 @@ void vtkAssemblyPath::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
 }
-
-
