@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkAssemblyPath.h,v $
+  Module:    vtkAssemblyPath.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -28,8 +28,8 @@
 #define __vtkAssemblyPath_h
 
 #include "vtkCollection.h"
+#include "vtkAssemblyNode.h" // used for inlines
 
-class vtkAssemblyNode;
 class vtkMatrix4x4;
 class vtkTransform;
 class vtkProp;
@@ -37,7 +37,7 @@ class vtkProp;
 class VTK_COMMON_EXPORT vtkAssemblyPath : public vtkCollection
 {
 public:
-  vtkTypeRevisionMacro(vtkAssemblyPath,vtkCollection);
+  vtkTypeMacro(vtkAssemblyPath,vtkCollection);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -86,6 +86,14 @@ public:
   // of the nodes in this path.
   virtual unsigned long GetMTime();
 
+  //BTX
+  // Description: 
+  // Reentrant safe way to get an object in a collection. Just pass the
+  // same cookie back and forth. 
+  vtkAssemblyNode *GetNextNode(vtkCollectionSimpleIterator &cookie) {
+    return static_cast<vtkAssemblyNode *>(this->GetNextItemAsObject(cookie));};
+  //ETX
+
 protected:
   vtkAssemblyPath();
   ~vtkAssemblyPath();
