@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkBitArray.h,v $
+  Module:    vtkBitArray.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -24,11 +24,13 @@
 
 #include "vtkDataArray.h"
 
+class vtkBitArrayLookup;
+
 class VTK_COMMON_EXPORT vtkBitArray : public vtkDataArray
 {
 public:
   static vtkBitArray *New();
-  vtkTypeRevisionMacro(vtkBitArray,vtkDataArray);
+  vtkTypeMacro(vtkBitArray,vtkDataArray);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -48,6 +50,24 @@ public:
   // Set the number of n-tuples in the array.
   void SetNumberOfTuples(vtkIdType number);
 
+  // Description:
+  // Set the tuple at the ith location using the jth tuple in the source array.
+  // This method assumes that the two arrays have the same type
+  // and structure. Note that range checking and memory allocation is not 
+  // performed; use in conjunction with SetNumberOfTuples() to allocate space.
+  virtual void SetTuple(vtkIdType i, vtkIdType j, vtkAbstractArray* source);
+
+  // Description:
+  // Insert the jth tuple in the source array, at ith location in this array. 
+  // Note that memory allocation is performed as necessary to hold the data.
+  virtual void InsertTuple(vtkIdType i, vtkIdType j, vtkAbstractArray* source);
+
+  // Description:
+  // Insert the jth tuple in the source array, at the end in this array. 
+  // Note that memory allocation is performed as necessary to hold the data.
+  // Returns the location at which the data was inserted.
+  virtual vtkIdType InsertNextTuple(vtkIdType j, vtkAbstractArray* source);
+  
   // Description:
   // Get a pointer to a tuple at the ith location. This is a dangerous method
   // (it is not thread safe since a pointer is returned).
@@ -73,6 +93,14 @@ public:
   vtkIdType InsertNextTuple(const float * tuple);
   vtkIdType InsertNextTuple(const double * tuple);
 
+  // Description:
+  // These methods remove tuples from the data array. They shift data and
+  // resize array, so the data array is still valid after this operation. Note,
+  // this operation is fairly slow.
+  virtual void RemoveTuple(vtkIdType id);
+  virtual void RemoveFirstTuple();
+  virtual void RemoveLastTuple();
+
   // Description:
   // Set the data component at the ith tuple and jth component location.
   // Note that i is less then NumberOfTuples and j is less then 
@@ -86,7 +114,7 @@ public:
 
   // Description:
   // Resize the array while conserving the data.
-  virtual void Resize(vtkIdType numTuples);
+  virtual int Resize(vtkIdType numTuples);
 
   // Description:
   // Get the data at a particular index.
@@ -108,6 +136,11 @@ public:
   // Description:
   // Insets values and checks to make sure there is enough memory
   void InsertValue(vtkIdType id, int i);
+
+  // Description:
+  // Set a value in the array from a variant.
+  void SetVariantValue(vtkIdType idx, vtkVariant value);
+
   vtkIdType InsertNextValue(int i);
 
   // Description:
@@ -124,12 +157,18 @@ public:
   // for the number of items requested. Set MaxId according to the number of
   // data values requested.
   unsigned char *WritePointer(vtkIdType id, vtkIdType number);
+  void* WriteVoidPointer(vtkIdType id, vtkIdType number)
+    { return this->WritePointer(id, number); }
   void *GetVoidPointer(vtkIdType id)
-    {return (void *)this->GetPointer(id);};
+    {
+      return static_cast<void *>(this->GetPointer(id));
+    }
 
   // Description:
   // Deep copy of another bit array.
   void DeepCopy(vtkDataArray *da);
+  void DeepCopy(vtkAbstractArray* aa)
+    { this->Superclass::DeepCopy(aa); }
 
   // Description:
   // This method lets the user specify data to be held by the array.  The 
@@ -137,12 +176,40 @@ public:
   // the array supplied by the user.  Set save to 1 to keep the class
   // from deleting the array when it cleans up or reallocates memory.
   // The class uses the actual array provided; it does not copy the data 
-  // from the suppled array.
+  // from the suppled array. If save 0, the array must have been allocated
+  // with new[] not malloc.
   void SetArray(unsigned char* array, vtkIdType size, int save);
   void SetVoidArray(void *array, vtkIdType size, int save) 
-    {this->SetArray((unsigned char *)array, size, save);};
+    {
+      this->SetArray(static_cast<unsigned char *>(array), size, save);
+    }
 
- 
+  // Description:
+  // Returns a new vtkBitArrayIterator instance.
+  vtkArrayIterator* NewIterator();
+  
+  // Description:
+  // Return the indices where a specific value appears.
+  virtual vtkIdType LookupValue(vtkVariant value);
+  virtual void LookupValue(vtkVariant value, vtkIdList* ids);
+  vtkIdType LookupValue(int value);
+  void LookupValue(int value, vtkIdList* ids);
+  
+  // Description:
+  // Tell the array explicitly that the data has changed.
+  // This is only necessary to call when you modify the array contents
+  // without using the array's API (i.e. you retrieve a pointer to the
+  // data and modify the array contents).  You need to call this so that
+  // the fast lookup will know to rebuild itself.  Otherwise, the lookup
+  // functions will give incorrect results.
+  virtual void DataChanged();
+  
+  // Description:
+  // Delete the associated fast lookup data structure on this array,
+  // if it exists.  The lookup will be rebuilt on the next call to a lookup
+  // function.
+  virtual void ClearLookup();
+  
 protected:
   vtkBitArray(vtkIdType numComp=1);
   ~vtkBitArray();
@@ -163,39 +230,33 @@ private:
 private:
   vtkBitArray(const vtkBitArray&);  // Not implemented.
   void operator=(const vtkBitArray&);  // Not implemented.
+  
+  //BTX
+  vtkBitArrayLookup* Lookup;
+  void UpdateLookup();
+  //ETX
 };
 
-inline unsigned char *vtkBitArray::WritePointer(vtkIdType id,
-                                                vtkIdType number)
-{
-  vtkIdType newSize=id+number;
-  if ( newSize > this->Size )
-    {
-    this->ResizeAndExtend(newSize);
-    }
-  if ( (--newSize) > this->MaxId )
-    {
-    this->MaxId = newSize;
-    }
-  return this->Array + id/8;
-}
-
 inline void vtkBitArray::SetNumberOfValues(vtkIdType number) 
 {
   this->Allocate(number);
   this->MaxId = number - 1;
+  this->DataChanged();
 }
 
 inline void vtkBitArray::SetValue(vtkIdType id, int value) 
 {
   if (value)
     {
-    this->Array[id/8] |= (0x80 >> id%8);
+    this->Array[id/8] = static_cast<unsigned char>(
+      this->Array[id/8] | (0x80 >> id%8));
     }
   else
     {
-    this->Array[id/8] &= (~(0x80 >> id%8));
+    this->Array[id/8] = static_cast<unsigned char>(
+      this->Array[id/8] & (~(0x80 >> id%8)));
     }
+  this->DataChanged();
 }
 
 inline void vtkBitArray::InsertValue(vtkIdType id, int i)
@@ -206,21 +267,31 @@ inline void vtkBitArray::InsertValue(vtkIdType id, int i)
     }
   if (i)
     {
-    this->Array[id/8] |= (0x80 >> id%8);
+    this->Array[id/8] = static_cast<unsigned char>(
+      this->Array[id/8] | (0x80 >> id%8));
     }
   else
     {
-    this->Array[id/8] &= (~(0x80 >> id%8));
+    this->Array[id/8] = static_cast<unsigned char>(
+      this->Array[id/8] & (~(0x80 >> id%8)));
     }
   if ( id > this->MaxId )
     {
     this->MaxId = id;
     }
+  this->DataChanged();
+}
+
+inline void vtkBitArray::SetVariantValue(vtkIdType id, vtkVariant value)
+{
+  this->SetValue(id, value.ToInt());
 }
 
 inline vtkIdType vtkBitArray::InsertNextValue(int i)
 {
-  this->InsertValue (++this->MaxId,i); return this->MaxId;
+  this->InsertValue (++this->MaxId,i);
+  this->DataChanged();
+  return this->MaxId;
 }
 
 inline void vtkBitArray::Squeeze() {this->ResizeAndExtend (this->MaxId+1);}
