@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkCollection.cxx,v $
+  Module:    vtkCollection.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -13,13 +13,14 @@
 
 =========================================================================*/
 #include "vtkCollection.h"
-#include "vtkObjectFactory.h"
+
 #include "vtkCollectionIterator.h"
+#include "vtkGarbageCollector.h"
+#include "vtkObjectFactory.h"
 
 #include <stdlib.h>
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkCollection, "$Revision: 1.44 $");
 vtkStandardNewMacro(vtkCollection);
 
 // Construct with empty list.
@@ -31,7 +32,7 @@ vtkCollection::vtkCollection()
   this->Current = NULL;
 }
 
-// Desctructor for the vtkCollection class. This removes all 
+// Destructor for the vtkCollection class. This removes all
 // objects from the collection.
 vtkCollection::~vtkCollection()
 {
@@ -42,7 +43,7 @@ vtkCollection::~vtkCollection()
     elem = this->Top;
     this->Top = elem->Next;
     this->Current = elem->Next;
-    this->NumberOfItems--;    
+    this->NumberOfItems--;
     this->DeleteElement(elem);
     }
 }
@@ -52,7 +53,7 @@ void vtkCollection::DeleteElement(vtkCollectionElement *e)
 {
   if (e->Item != NULL)
     {
-    e->Item->UnRegister(this); 
+    e->Item->UnRegister(this);
     }
   delete e;
 }
@@ -63,7 +64,7 @@ void vtkCollection::AddItem(vtkObject *a)
   vtkCollectionElement *elem;
 
   elem = new vtkCollectionElement;
-  
+
   if (!this->Top)
     {
     this->Top = elem;
@@ -83,6 +84,56 @@ void vtkCollection::AddItem(vtkObject *a)
   this->NumberOfItems++;
 }
 
+// Insert an object into the list. There must be at least one
+// entry pre-existing.
+void vtkCollection::InsertItem(int i, vtkObject *a)
+{
+  if( i >= this->NumberOfItems || !this->Top )
+    {
+    return;
+    }
+
+  vtkCollectionElement *elem;
+
+  elem = new vtkCollectionElement;
+  vtkCollectionElement *curr = this->Top;
+
+  if( i < 0 )
+    {
+    this->Top = elem;
+    elem->Next = curr;
+    }
+  else
+    {
+    vtkCollectionElement *next = curr->Next;
+
+    int j = 0;
+    while( j != i )
+      {
+      curr = next;
+      next = curr->Next;
+      j++;
+      }
+
+    curr->Next = elem;
+    if( curr == this->Bottom )
+      {
+      this->Bottom = elem;
+      }
+    else
+      {
+      elem->Next = next;
+      }
+    }
+
+  a->Register(this);
+  elem->Item = a;
+
+  this->Modified();
+
+  this->NumberOfItems++;
+}
+
 // Remove an object from the list. Removes the first object found, not
 // all occurrences. If no object found, list is unaffected.  See warning
 // in description of RemoveItem(int).
@@ -90,7 +141,7 @@ void vtkCollection::RemoveItem(vtkObject *a)
 {
   int i;
   vtkCollectionElement *elem;
-  
+
   if (!this->Top)
     {
     return;
@@ -117,15 +168,21 @@ void vtkCollection::RemoveAllItems()
 {
   vtkCollectionElement *elem;
 
+  // Don't modify if collection is empty
+  if(this->NumberOfItems == 0)
+    {
+    return;
+    }
+
   while (this->NumberOfItems )
     {
     elem = this->Top;
     this->Top = elem->Next;
     this->Current = elem->Next;
-    this->NumberOfItems--;    
+    this->NumberOfItems--;
     this->DeleteElement(elem);
     }
-  
+
   this->Modified();
 }
 
@@ -135,7 +192,7 @@ int vtkCollection::IsItemPresent(vtkObject *a)
 {
   int i;
   vtkCollectionElement *elem;
-  
+
   if (!this->Top)
     {
     return 0;
@@ -158,13 +215,6 @@ int vtkCollection::IsItemPresent(vtkObject *a)
 }
 
 
-// Return the number of objects in the list.
-int vtkCollection::GetNumberOfItems()
-{
-  return this->NumberOfItems;
-}
-
-
 void vtkCollection::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
@@ -217,7 +267,7 @@ void vtkCollection::ReplaceItem(int i, vtkObject *a)
     {
     return;
     }
-  
+
   elem = this->Top;
   if (i == this->NumberOfItems - 1)
     {
@@ -225,7 +275,7 @@ void vtkCollection::ReplaceItem(int i, vtkObject *a)
     }
   else
     {
-    for (int j = 0; j < i; j++, elem = elem->Next ) 
+    for (int j = 0; j < i; j++, elem = elem->Next )
       {}
     }
 
@@ -235,7 +285,7 @@ void vtkCollection::ReplaceItem(int i, vtkObject *a)
     elem->Item->UnRegister(this);
     }
   a->Register(this);
-  
+
   // j == i
   elem->Item = a;
 
@@ -244,7 +294,7 @@ void vtkCollection::ReplaceItem(int i, vtkObject *a)
 
 
 // Remove the i'th item in the list.
-// Be careful if using this function during traversal of the list using 
+// Be careful if using this function during traversal of the list using
 // GetNextItemAsObject (or GetNextItem in derived class).  The list WILL
 // be shortened if a valid index is given!  If this->Current is equal to the
 // element being removed, have it point to then next element in the list.
@@ -256,7 +306,7 @@ void vtkCollection::RemoveItem(int i)
     {
     return;
     }
-  
+
   this->Modified();
 
   elem = this->Top;
@@ -265,7 +315,7 @@ void vtkCollection::RemoveItem(int i)
     {
     prev = elem;
     elem = elem->Next;
-    }  
+    }
 
   // j == i
   if (prev)
@@ -281,7 +331,7 @@ void vtkCollection::RemoveItem(int i)
     {
     this->Bottom = prev;
     }
-      
+
   if ( this->Current == elem )
     {
     this->Current = elem->Next;
@@ -297,3 +347,25 @@ vtkCollectionIterator* vtkCollection::NewIterator()
   it->SetCollection(this);
   return it;
 }
+
+//----------------------------------------------------------------------------
+void vtkCollection::Register(vtkObjectBase* o)
+{
+  this->RegisterInternal(o, 1);
+}
+
+//----------------------------------------------------------------------------
+void vtkCollection::UnRegister(vtkObjectBase* o)
+{
+  this->UnRegisterInternal(o, 1);
+}
+
+//----------------------------------------------------------------------------
+void vtkCollection::ReportReferences(vtkGarbageCollector* collector)
+{
+  this->Superclass::ReportReferences(collector);
+  for(vtkCollectionElement* elem = this->Top; elem; elem = elem->Next)
+    {
+    vtkGarbageCollectorReport(collector, elem->Item, "Element");
+    }
+}
