@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkCriticalSection.cxx,v $
+  Module:    vtkCriticalSection.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,7 +15,6 @@
 #include "vtkCriticalSection.h"
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkCriticalSection, "$Revision: 1.10 $");
 vtkStandardNewMacro(vtkCriticalSection);
 
 // New for the SimpleCriticalSection
@@ -92,20 +91,6 @@ void vtkSimpleCriticalSection::Unlock()
 #endif
 }
 
-int vtkSimpleCriticalSection::IsA(const char *type)
-{
-  if ( !strcmp(this->vtkSimpleCriticalSection::GetClassName(),type) )
-    {
-    return 1;
-    }
-  return 0;
-}
-
-vtkSimpleCriticalSection *vtkSimpleCriticalSection::SafeDownCast(vtkSimpleCriticalSection *o)
-{
-  return (vtkSimpleCriticalSection *)o;
-}
-  
 void vtkCriticalSection::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
