@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkDebugLeaks.cxx,v $
+  Module:    vtkDebugLeaks.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -13,13 +13,18 @@
 
 =========================================================================*/
 #include "vtkDebugLeaks.h"
-#include "vtkObjectFactory.h"
+
 #include "vtkCriticalSection.h"
+#include "vtkObjectFactory.h"
+#include "vtkWindows.h"
+
+#include <string>
 
 static const char *vtkDebugLeaksIgnoreClasses[] = {
   0
 };
 
+//----------------------------------------------------------------------------
 // return 1 if the class should be ignored
 int vtkDebugLeaksIgnoreClassesCheck(const char* s)
 {
@@ -35,9 +40,9 @@ int vtkDebugLeaksIgnoreClassesCheck(const char* s)
   return 0;
 }
 
-vtkCxxRevisionMacro(vtkDebugLeaks, "$Revision: 1.25 $");
 vtkStandardNewMacro(vtkDebugLeaks);
 
+//----------------------------------------------------------------------------
 // A hash function for converting a string to a long
 inline size_t vtkHashString(const char* s)
 {
@@ -49,6 +54,7 @@ inline size_t vtkHashString(const char* s)
   return size_t(h);
 }
 
+//----------------------------------------------------------------------------
 class vtkDebugLeaksHashNode 
 {
 public:
@@ -58,13 +64,16 @@ public:
       this->Key = 0;
       this->Next =0;
     }
-  void Print(ostream& os)
+  void Print(std::string& os)
     {
       if(this->Count)
         {
-        os << "Class " << this->Key << " has " 
-           << this->Count << ((this->Count==1)? " instance" : " instances")
-           << " still around.\n";
+        char tmp[256];
+        sprintf(tmp,"\" has %i %s still around.\n",this->Count,
+                (this->Count == 1) ? "instance" : "instances");
+        os += "Class \"";
+        os += this->Key;
+        os += tmp;
         }
     }
   ~vtkDebugLeaksHashNode()
@@ -81,6 +90,7 @@ public:
   int Count;
 };
 
+//----------------------------------------------------------------------------
 class vtkDebugLeaksHashTable
 {
 public:
@@ -89,7 +99,7 @@ public:
   void IncrementCount(const char *name);
   unsigned int GetCount(const char *name);
   int DecrementCount(const char* name);
-  void PrintTable(ostream& os);
+  void PrintTable(std::string &os);
   int IsEmpty();
   ~vtkDebugLeaksHashTable()
     {
@@ -107,6 +117,7 @@ private:
   vtkDebugLeaksHashNode* Nodes[64];
 };
 
+//----------------------------------------------------------------------------
 vtkDebugLeaksHashTable::vtkDebugLeaksHashTable()
 {
   int i;
@@ -116,11 +127,13 @@ vtkDebugLeaksHashTable::vtkDebugLeaksHashTable()
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkDebugLeaksHashTable::IncrementCount(const char * name)
 {
   vtkDebugLeaksHashNode *pos;
   vtkDebugLeaksHashNode *newpos;
   int loc;
+
   pos = this->GetNode(name);
   if(pos)
     {
@@ -131,7 +144,7 @@ void vtkDebugLeaksHashTable::IncrementCount(const char * name)
   newpos = new vtkDebugLeaksHashNode;
   newpos->Key = strcpy(new char[strlen(name)+1], name);
 
-  loc = (((unsigned long)vtkHashString(name)) & 0x03f0) / 16;
+  loc = (static_cast<unsigned long>(vtkHashString(name)) & 0x03f0) / 16;
   
   pos = this->Nodes[loc];
   if (!pos)
@@ -146,10 +159,11 @@ void vtkDebugLeaksHashTable::IncrementCount(const char * name)
   pos->Next = newpos;
 }
 
+//----------------------------------------------------------------------------
 vtkDebugLeaksHashNode* vtkDebugLeaksHashTable::GetNode(const char* key)
 {
   vtkDebugLeaksHashNode *pos;
-  int loc = (((unsigned long)vtkHashString(key)) & 0x03f0) / 16;
+  int loc = (static_cast<unsigned long>(vtkHashString(key)) & 0x03f0) / 16;
   
   pos = this->Nodes[loc];
 
@@ -164,10 +178,11 @@ vtkDebugLeaksHashNode* vtkDebugLeaksHashTable::GetNode(const char* key)
   return pos;
 }
 
+//----------------------------------------------------------------------------
 unsigned int vtkDebugLeaksHashTable::GetCount(const char* key)
 {
   vtkDebugLeaksHashNode *pos;
-  int loc = (((unsigned long)vtkHashString(key)) & 0x03f0) / 16;
+  int loc = (static_cast<unsigned long>(vtkHashString(key)) & 0x03f0) / 16;
   
   pos = this->Nodes[loc];
 
@@ -186,6 +201,7 @@ unsigned int vtkDebugLeaksHashTable::GetCount(const char* key)
   return 0;
 }
 
+//----------------------------------------------------------------------------
 int vtkDebugLeaksHashTable::IsEmpty()
 {
   int count = 0;
@@ -211,6 +227,7 @@ int vtkDebugLeaksHashTable::IsEmpty()
   return !count;
 }
 
+//----------------------------------------------------------------------------
 int vtkDebugLeaksHashTable::DecrementCount(const char *key)
 {
   
@@ -226,7 +243,8 @@ int vtkDebugLeaksHashTable::DecrementCount(const char *key)
     }
 }
 
-void vtkDebugLeaksHashTable::PrintTable(ostream& os)
+//----------------------------------------------------------------------------
+void vtkDebugLeaksHashTable::PrintTable(std::string &os)
 {
   for(int i =0; i < 64; i++)
     {
@@ -249,8 +267,7 @@ void vtkDebugLeaksHashTable::PrintTable(ostream& os)
     }
 }
 
-
-
+//----------------------------------------------------------------------------
 #ifdef VTK_DEBUG_LEAKS
 void vtkDebugLeaks::ConstructClass(const char* name)
 {
@@ -264,6 +281,7 @@ void vtkDebugLeaks::ConstructClass(const char*)
 }
 #endif
 
+//----------------------------------------------------------------------------
 #ifdef VTK_DEBUG_LEAKS
 void vtkDebugLeaks::DestructClass(const char* p)
 {
@@ -287,45 +305,103 @@ void vtkDebugLeaks::DestructClass(const char*)
 }
 #endif
 
-void vtkDebugLeaks::PrintCurrentLeaks()
+//----------------------------------------------------------------------------
+void vtkDebugLeaks::SetDebugLeaksObserver(vtkDebugLeaksObserver* observer)
+{
+  vtkDebugLeaks::Observer = observer;
+}
+
+//----------------------------------------------------------------------------
+vtkDebugLeaksObserver* vtkDebugLeaks::GetDebugLeaksObserver()
+{
+  return vtkDebugLeaks::Observer;
+}
+
+//----------------------------------------------------------------------------
+void vtkDebugLeaks::ConstructingObject(vtkObjectBase* object)
+{
+  if (vtkDebugLeaks::Observer)
+    {
+    vtkDebugLeaks::Observer->ConstructingObject(object);
+    }
+}
+
+//----------------------------------------------------------------------------
+void vtkDebugLeaks::DestructingObject(vtkObjectBase* object)
+{
+  if (vtkDebugLeaks::Observer)
+    {
+    vtkDebugLeaks::Observer->DestructingObject(object);
+    }
+}
+
+//----------------------------------------------------------------------------
+int vtkDebugLeaks::PrintCurrentLeaks()
 {
 #ifdef VTK_DEBUG_LEAKS
   if(vtkDebugLeaks::MemoryTable->IsEmpty())
     {
-    return;
+    return 0;
     }
-  // print the table
-  strstream leaks;
+
+  // we must not use stringstream ior strstream due to problems with
+  // finalizers in MSVC
+  std::string leaks;
   vtkDebugLeaks::MemoryTable->PrintTable(leaks);
-  
 #ifdef _WIN32
+  fprintf(stderr,"%s",leaks.c_str());
+  fflush(stderr);
   int cancel=0;
-  while(!cancel && !!leaks)
+  if(getenv("DASHBOARD_TEST_FROM_CTEST") ||
+     getenv("DART_TEST_FROM_DART"))
     {
-    char line[1000];
-    strstream msg;
-    msg << "vtkDebugLeaks has detected LEAKS!\n";
-    int i=0;
-    while((++i <= 10) && !!leaks.getline(line, 1000))
+    // Skip dialogs when running on dashboard.
+    return 1;
+    }
+  std::string::size_type myPos = 0;
+  int count = 0;
+  std::string msg;
+  msg = "vtkDebugLeaks has detected LEAKS!\n";
+  while(!cancel && myPos != leaks.npos)
+    {  
+    std::string::size_type newPos = leaks.find('\n',myPos);
+    if (newPos != leaks.npos)
       {
-      msg << line << "\n";
+      msg += leaks.substr(myPos,newPos-myPos);
+      msg += "\n";
+      myPos = newPos;
+      myPos++;
       }
-    msg << ends;
-    cancel = vtkDebugLeaks::DisplayMessageBox(msg.str());
-    msg.rdbuf()->freeze(0);
+    else
+      {
+      myPos = newPos;
+      }
+    count++;
+    if (count == 10)
+      {
+      count = 0;
+      cancel = vtkDebugLeaks::DisplayMessageBox(msg.c_str());
+      msg = "";
+      }
+    }
+  if (!cancel && count > 0)
+    {
+    vtkDebugLeaks::DisplayMessageBox(msg.c_str());
     }
 #else
   cout << "vtkDebugLeaks has detected LEAKS!\n";
-  cout << leaks.rdbuf() << "\n";
+  cout << leaks.c_str() << endl;
 #endif
 #endif
+  return 1;
 }
 
+//----------------------------------------------------------------------------
 #ifdef _WIN32
 int vtkDebugLeaks::DisplayMessageBox(const char* msg)
 {
 #ifdef UNICODE
-  wchar_t *wmsg = new wchar_t [mbstowcs(NULL, msg, 32000)];
+  wchar_t *wmsg = new wchar_t [mbstowcs(NULL, msg, 32000)+1];
   mbstowcs(wmsg, msg, 32000);
   int result = (MessageBox(NULL, wmsg, L"Error",
                            MB_ICONERROR | MB_OKCANCEL) == IDCANCEL);
@@ -343,18 +419,36 @@ int vtkDebugLeaks::DisplayMessageBox(const char*)
 }
 #endif
 
+//----------------------------------------------------------------------------
+int vtkDebugLeaks::GetExitError()
+{
+  return vtkDebugLeaks::ExitError;
+}
+
+//----------------------------------------------------------------------------
+void vtkDebugLeaks::SetExitError(int flag)
+{
+  vtkDebugLeaks::ExitError = flag;
+}
+
 //----------------------------------------------------------------------------
 void vtkDebugLeaks::ClassInitialize()
 {
 #ifdef VTK_DEBUG_LEAKS
   // Create the hash table.
   vtkDebugLeaks::MemoryTable = new vtkDebugLeaksHashTable;
-  
+
   // Create the lock for the critical sections.
   vtkDebugLeaks::CriticalSection = new vtkSimpleCriticalSection;
+
+  // Default to error when leaks occur while running tests.
+  vtkDebugLeaks::ExitError = 1;
+  vtkDebugLeaks::Observer = 0;
 #else
   vtkDebugLeaks::MemoryTable = 0;
   vtkDebugLeaks::CriticalSection = 0;
+  vtkDebugLeaks::ExitError = 0;
+  vtkDebugLeaks::Observer = 0;
 #endif
 }
 
@@ -362,18 +456,30 @@ void vtkDebugLeaks::ClassInitialize()
 void vtkDebugLeaks::ClassFinalize()
 {
 #ifdef VTK_DEBUG_LEAKS
-  vtkDebugLeaks::PrintCurrentLeaks();
-  
+  // Report leaks.
+  int leaked = vtkDebugLeaks::PrintCurrentLeaks();
+
   // Destroy the hash table.
   delete vtkDebugLeaks::MemoryTable;
   vtkDebugLeaks::MemoryTable = 0;
-  
+
   // Destroy the lock for the critical sections.
   delete vtkDebugLeaks::CriticalSection;
   vtkDebugLeaks::CriticalSection = 0;
+
+  // Exit with error if leaks occured and error mode is on.
+  if(leaked && vtkDebugLeaks::ExitError)
+    {
+    exit(1);
+    }
 #endif
 }
 
+#ifndef VTK_LEGACY_REMOVE
+void vtkDebugLeaks::PromptUserOn() {}
+void vtkDebugLeaks::PromptUserOff() {}
+#endif
+
 //----------------------------------------------------------------------------
 
 // Purposely not initialized.  ClassInitialize will handle it.
@@ -381,3 +487,9 @@ vtkDebugLeaksHashTable* vtkDebugLeaks::MemoryTable;
 
 // Purposely not initialized.  ClassInitialize will handle it.
 vtkSimpleCriticalSection* vtkDebugLeaks::CriticalSection;
+
+// Purposely not initialized.  ClassInitialize will handle it.
+int vtkDebugLeaks::ExitError;
+
+// Purposely not initialized.  ClassInitialize will handle it.
+vtkDebugLeaksObserver* vtkDebugLeaks::Observer;
