@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImplicitFunction.cxx,v $
+  Module:    vtkImplicitFunction.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,8 +16,8 @@
 
 #include "vtkMath.h"
 #include "vtkAbstractTransform.h"
+#include "vtkTransform.h"
 
-vtkCxxRevisionMacro(vtkImplicitFunction, "$Revision: 1.34 $");
 vtkCxxSetObjectMacro(vtkImplicitFunction,Transform,vtkAbstractTransform);
 
 vtkImplicitFunction::vtkImplicitFunction()
@@ -27,7 +27,9 @@ vtkImplicitFunction::vtkImplicitFunction()
 
 vtkImplicitFunction::~vtkImplicitFunction()
 {
-  this->SetTransform(NULL);
+  //static_cast needed since otherwise the
+  //call to SetTransform becomes ambiguous
+  this->SetTransform(static_cast<vtkAbstractTransform*>(NULL));
 }
 
 // Evaluate function at position x-y-z and return value. Point x[3] is
@@ -36,7 +38,7 @@ double vtkImplicitFunction::FunctionValue(const double x[3])
 {
   if ( ! this->Transform )
     {
-    return this->EvaluateFunction((double *)x);
+    return this->EvaluateFunction(const_cast<double *>(x));
     }
   else //pass point through transform
     {
@@ -76,7 +78,7 @@ void vtkImplicitFunction::FunctionGradient(const double x[3], double g[3])
 {
   if ( ! this->Transform )
     {
-    this->EvaluateGradient((double *)x,g);
+    this->EvaluateGradient(const_cast<double *>(x),g);
     }
   else //pass point through transform
     {
@@ -84,7 +86,7 @@ void vtkImplicitFunction::FunctionGradient(const double x[3], double g[3])
     double A[3][3];
     this->Transform->Update();
     this->Transform->InternalTransformDerivative(x,pt,A);
-    this->EvaluateGradient((double *)pt,g);
+    this->EvaluateGradient(static_cast<double *>(pt),g);
 
     // The gradient must be transformed using the same math as is
     // use for a normal to a surface: it must be multiplied by the
@@ -141,4 +143,11 @@ void vtkImplicitFunction::PrintSelf(ostream& os, vtkIndent indent)
     }
 }
 
+void vtkImplicitFunction::SetTransform(const double elements[16])
+{
+  vtkTransform* transform = vtkTransform::New();
+  transform->SetMatrix(elements);
+  this->SetTransform(transform);
+  transform->Delete();
+}
 
