@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkIntArray.h,v $
+  Module:    vtkIntArray.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -22,7 +22,7 @@
 #define __vtkIntArray_h
 
 // Tell the template header how to give our superclass a DLL interface.
-#if !defined(__vtkIntArray_cxx)
+#if !defined(__vtkIntArray_cxx) && (defined(VTK_USE_64BIT_IDS) || !defined(__vtkIdTypeArray_h))
 # define VTK_DATA_ARRAY_TEMPLATE_TYPE int
 #endif
 
@@ -36,7 +36,7 @@ class VTK_COMMON_EXPORT vtkIntArray : public vtkDataArray
 {
 public:
   static vtkIntArray* New();
-  vtkTypeRevisionMacro(vtkIntArray,vtkDataArray);
+  vtkTypeMacro(vtkIntArray,vtkDataArray);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -44,6 +44,27 @@ public:
   int GetDataType()
     { return VTK_INT; }
 
+  // Description:
+  // Copy the tuple value into a user-provided array.
+  void GetTupleValue(vtkIdType i, int* tuple)
+    { this->RealSuperclass::GetTupleValue(i, tuple); }
+
+  // Description:
+  // Set the tuple value at the ith location in the array.
+  void SetTupleValue(vtkIdType i, const int* tuple)
+    { this->RealSuperclass::SetTupleValue(i, tuple); }
+
+  // Description:
+  // Insert (memory allocation performed) the tuple into the ith location
+  // in the array.
+  void InsertTupleValue(vtkIdType i, const int* tuple)
+    { this->RealSuperclass::InsertTupleValue(i, tuple); }
+
+  // Description:
+  // Insert (memory allocation performed) the tuple onto the end of the array.
+  vtkIdType InsertNextTupleValue(const int* tuple)
+    { return this->RealSuperclass::InsertNextTupleValue(tuple); }
+
   // Description:
   // Get the data at a particular index.
   int GetValue(vtkIdType id)
@@ -72,6 +93,34 @@ public:
   vtkIdType InsertNextValue(int f)
     { return this->RealSuperclass::InsertNextValue(f); }
 
+  // Description:
+  // Get the range of array values for the given component in the
+  // native data type.
+  int *GetValueRange(int comp)
+    { return this->RealSuperclass::GetValueRange(comp); }
+//BTX
+  void GetValueRange(int range[2], int comp)
+    { this->RealSuperclass::GetValueRange(range, comp); }
+//ETX
+
+  // Description:
+  // Get the range of array values for the 0th component in the
+  // native data type.
+  int *GetValueRange()
+    { return this->RealSuperclass::GetValueRange(0); }
+//BTX
+  void GetValueRange(int range[2])
+    { this->RealSuperclass::GetValueRange(range, 0); }
+//ETX
+
+  // Description:
+  // Get the minimum data value in its native type.
+  static int GetDataTypeValueMin() { return VTK_INT_MIN; }
+
+  // Description:
+  // Get the maximum data value in its native type.
+  static int GetDataTypeValueMax() { return VTK_INT_MAX; }
+
   // Description:
   // Get the address of a particular data index. Make sure data is allocated
   // for the number of items requested. Set MaxId according to the number of
@@ -94,6 +143,8 @@ public:
   // from the suppled array.
   void SetArray(int* array, vtkIdType size, int save)
     { this->RealSuperclass::SetArray(array, size, save); }
+  void SetArray(int* array, vtkIdType size, int save, int deleteMethod)
+    { this->RealSuperclass::SetArray(array, size, save, deleteMethod); }
 
 protected:
   vtkIntArray(vtkIdType numComp=1);
