@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkJavaAwt.h,v $
+  Module:    vtkJavaAwt.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -28,10 +28,10 @@
   jclass cls = E->GetObjectClass(C);\
   jmethodID mid = E->GetMethodID(cls, "hashCode", "()I");\
   H = E->CallIntMethod(C, mid); }
-#include <windows.h>
+# include "vtkWindows.h"
 int WJLH_init_check = 0;
-#include "vtkstd/map"
-vtkstd::map<int,int> WJLH_lock_map;
+# include <map> // STL Header
+std::map<int,int> WJLH_lock_map;
 #endif
 
 extern "C" JNIEXPORT jint  JNICALL 
@@ -50,7 +50,7 @@ Java_vtk_vtkPanel_RenderCreate(JNIEnv *env, jobject canvas, jobject id0)
 
   // get the render window pointer
   vtkRenderWindow *temp0;
-  temp0 = (vtkRenderWindow *)(vtkJavaGetPointerFromObject(env,id0,(char *) "vtkRenderWindow"));
+  temp0 = (vtkRenderWindow *)(vtkJavaGetPointerFromObject(env,id0));
   
   /* Get the AWT */
   awt.version = JAWT_VERSION_1_3;
@@ -95,6 +95,7 @@ Java_vtk_vtkPanel_RenderCreate(JNIEnv *env, jobject canvas, jobject id0)
   
 // Here is the win32 drawing code
 #if defined(_WIN32) || defined(WIN32)
+  temp0->Finalize();
   JAWT_Win32DrawingSurfaceInfo* dsi_win;
   dsi_win = (JAWT_Win32DrawingSurfaceInfo*)dsi->platformInfo;
   temp0->SetWindowId((void *)dsi_win->hwnd);
@@ -102,10 +103,10 @@ Java_vtk_vtkPanel_RenderCreate(JNIEnv *env, jobject canvas, jobject id0)
   // also set parent id to avoid border sizes being added
   temp0->SetParentId((void *)dsi_win->hdc);
 // use mac code
-#elif defined(VTK_USE_CARBON) || defined(VTK_USE_COCOA)
-  JAWT_MacDrawingSurfaceInfo* dsi_mac;
-  dsi_mac = (JAWT_MacDrawingSurfaceInfo*)dsi->platformInfo;
-  temp0->SetWindowId((void *)dsi_mac->fQDWindow);
+#elif defined(__APPLE__)
+  JAWT_MacOSXDrawingSurfaceInfo* dsi_mac;
+  dsi_mac = (JAWT_MacOSXDrawingSurfaceInfo*)dsi->platformInfo;
+  temp0->SetWindowId(dsi_mac->cocoaViewRef);
 // otherwise use X11 code
 #else
   JAWT_X11DrawingSurfaceInfo* dsi_x11;
