@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkLargeInteger.cxx,v $
+  Module:    vtkLargeInteger.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -70,6 +70,68 @@ vtkLargeInteger::vtkLargeInteger(void)
     this->Sig = 0;
 }
 
+#if defined(VTK_TYPE_USE_LONG_LONG)
+vtkLargeInteger::vtkLargeInteger(long long n)
+{
+  this->Negative = n < 0 ? 1 : 0;
+  n = n < 0 ? -n : n; // strip of sign
+  this->Number = new char[BIT_INCREMENT];
+  for (unsigned int i = 0; i < BIT_INCREMENT; i++)
+    {
+    this->Number[i] = n & 1;
+    n >>= 1;
+    }
+  this->Max = BIT_INCREMENT - 1;
+  this->Sig = BIT_INCREMENT - 1;
+  this->Contract(); // remove leading 0s
+}
+
+vtkLargeInteger::vtkLargeInteger(unsigned long long n)
+{
+  this->Negative = 0;
+  this->Number = new char[BIT_INCREMENT];
+  for (unsigned int i = 0; i < BIT_INCREMENT; i++)
+    {
+    this->Number[i] = n & 1;
+    n >>= 1;
+    }
+  this->Max = BIT_INCREMENT - 1;
+  this->Sig = BIT_INCREMENT - 1;
+  this->Contract(); // remove leading 0s
+}
+#endif
+
+#if defined(VTK_TYPE_USE___INT64)
+vtkLargeInteger::vtkLargeInteger(__int64 n)
+{
+  this->Negative = n < 0 ? 1 : 0;
+  n = n < 0 ? -n : n; // strip of sign
+  this->Number = new char[BIT_INCREMENT];
+  for (unsigned int i = 0; i < BIT_INCREMENT; i++)
+    {
+    this->Number[i] = n & 1;
+    n >>= 1;
+    }
+  this->Max = BIT_INCREMENT - 1;
+  this->Sig = BIT_INCREMENT - 1;
+  this->Contract(); // remove leading 0s
+}
+
+vtkLargeInteger::vtkLargeInteger(unsigned __int64 n)
+{
+  this->Negative = 0;
+  this->Number = new char[BIT_INCREMENT];
+  for (unsigned int i = 0; i < BIT_INCREMENT; i++)
+    {
+    this->Number[i] = n & 1;
+    n >>= 1;
+    }
+  this->Max = BIT_INCREMENT - 1;
+  this->Sig = BIT_INCREMENT - 1;
+  this->Contract(); // remove leading 0s
+}
+#endif
+
 vtkLargeInteger::vtkLargeInteger(long n)
 {
   this->Negative = n < 0 ? 1 : 0;
@@ -537,7 +599,7 @@ vtkLargeInteger& vtkLargeInteger::operator>>=(int n)
 
   // first shift the data
   unsigned int i;
-  if (this->Sig >= (unsigned long)n)
+  if (this->Sig >= static_cast<unsigned long>(n))
     {
     for (i = 0; i <= (this->Sig - n); i++) 
       {
