@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkLookupTable.cxx,v $
+  Module:    vtkLookupTable.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,8 +16,9 @@
 #include "vtkBitArray.h"
 #include "vtkObjectFactory.h"
 #include "vtkMath.h"
+#include "vtkMathConfigure.h"
+#include <assert.h>
 
-vtkCxxRevisionMacro(vtkLookupTable, "$Revision: 1.96 $");
 vtkStandardNewMacro(vtkLookupTable);
 
 // Construct with range=(0,1); and hsv ranges set up for rainbow color table 
@@ -26,6 +27,8 @@ vtkLookupTable::vtkLookupTable(int sze, int ext)
 {
   this->NumberOfColors = sze;
   this->Table = vtkUnsignedCharArray::New();
+  this->Table->Register(this);
+  this->Table->Delete();
   this->Table->SetNumberOfComponents(4);
   this->Table->Allocate(4*sze,4*ext);
 
@@ -42,19 +45,53 @@ vtkLookupTable::vtkLookupTable(int sze, int ext)
   this->AlphaRange[1] = 1.0;
   this->Alpha = 1.0;
 
+  this->NanColor[0] = 0.5;
+  this->NanColor[1] = 0.0;
+  this->NanColor[2] = 0.0;
+  this->NanColor[3] = 1.0;
+
   this->TableRange[0] = 0.0;
   this->TableRange[1] = 1.0;
 
   this->Ramp = VTK_RAMP_SCURVE;
   this->Scale = VTK_SCALE_LINEAR;
+  
+  this->OpaqueFlag=1;
 }
 
+//----------------------------------------------------------------------------
 vtkLookupTable::~vtkLookupTable()
 {
-  this->Table->Delete();
+  this->Table->UnRegister(this);
   this->Table = NULL;
 }
 
+//----------------------------------------------------------------------------
+// Description:
+// Return true if all of the values defining the mapping have an opacity
+// equal to 1. Default implementation return true.
+int vtkLookupTable::IsOpaque()
+{
+  if(this->OpaqueFlagBuildTime<this->GetMTime())
+    {
+    int opaque=1;
+    if (this->NanColor[3] < 1.0) { opaque = 0; }
+    int size=this->Table->GetNumberOfTuples();
+    int i=0;
+    unsigned char *ptr=this->Table->GetPointer(0);
+    while(opaque && i<size)
+      {
+      opaque=ptr[3]==255;
+      ptr+=4;
+      ++i;
+      }
+    this->OpaqueFlag=opaque;
+    this->OpaqueFlagBuildTime.Modified();
+    }
+  return this->OpaqueFlag;
+}
+
+//----------------------------------------------------------------------------
 // Scalar values greater than maximum range value are clamped to maximum
 // range value.
 void vtkLookupTable::SetTableRange(double r[2])
@@ -62,6 +99,7 @@ void vtkLookupTable::SetTableRange(double r[2])
   this->SetTableRange(r[0],r[1]);
 }
 
+//----------------------------------------------------------------------------
 // Set the minimum/maximum scalar values for scalar mapping. Scalar values
 // less than minimum range value are clamped to minimum range value.
 // Scalar values greater than maximum range value are clamped to maximum
@@ -91,6 +129,7 @@ void vtkLookupTable::SetTableRange(double rmin, double rmax)
   this->Modified();
 }
 
+//----------------------------------------------------------------------------
 // Have to be careful about the range if scale is logarithmic
 void vtkLookupTable::SetScale(int scale)
 {
@@ -115,6 +154,7 @@ void vtkLookupTable::SetScale(int scale)
     }
 }
 
+//----------------------------------------------------------------------------
 // Allocate a color table of specified size.
 int vtkLookupTable::Allocate(int sz, int ext)
 {
@@ -124,6 +164,7 @@ int vtkLookupTable::Allocate(int sz, int ext)
   return a;
 }
 
+//----------------------------------------------------------------------------
 // Force the lookup table to rebuild
 void vtkLookupTable::ForceBuild()
 {
@@ -134,10 +175,17 @@ void vtkLookupTable::ForceBuild()
 
   int maxIndex = this->NumberOfColors - 1;
 
-  hinc = (this->HueRange[1] - this->HueRange[0])/maxIndex;
-  sinc = (this->SaturationRange[1] - this->SaturationRange[0])/maxIndex;
-  vinc = (this->ValueRange[1] - this->ValueRange[0])/maxIndex;
-  ainc = (this->AlphaRange[1] - this->AlphaRange[0])/maxIndex;
+  if( maxIndex )
+    {
+    hinc = (this->HueRange[1] - this->HueRange[0])/maxIndex;
+    sinc = (this->SaturationRange[1] - this->SaturationRange[0])/maxIndex;
+    vinc = (this->ValueRange[1] - this->ValueRange[0])/maxIndex;
+    ainc = (this->AlphaRange[1] - this->AlphaRange[0])/maxIndex;
+    }
+  else
+    {
+    hinc = sinc = vinc = ainc = 0.0; 
+    }
 
   for (i = 0; i <= maxIndex; i++) 
     {
@@ -162,38 +210,42 @@ void vtkLookupTable::ForceBuild()
         c_rgba[2] = static_cast<unsigned char> 
           (127.5*(1.0+cos((1.0-static_cast<double>(rgba[2]))*3.141593)));
         c_rgba[3] = static_cast<unsigned char> (alpha*255.0);
-        /* same code, but with rounding 
-           c_rgba[0] = static_cast<unsigned char> 
-           (127.5f*(1.0f + (double)cos(double((1.0f-rgba[0])*3.141593f)))+0.5f);
-           c_rgba[1] = static_cast<unsigned char> 
-           (127.5f*(1.0f + (double)cos(double((1.0f-rgba[1])*3.141593f)))+0.5f);
-           c_rgba[2] = static_cast<unsigned char> 
-           (127.5f*(1.0f + (double)cos(double((1.0f-rgba[2])*3.141593f)))+0.5f);
-           c_rgba[3] = static_cast<unsigned char>(rgba[3]*255.0f + 0.5f);
+        /* same code, but with rounding for correctness
+        c_rgba[0] = static_cast<unsigned char>
+          (127.5*(1.0 + cos((1.0 - rgba[0])*3.141593)) + 0.5);
+        c_rgba[1] = static_cast<unsigned char>
+          (127.5*(1.0 + cos((1.0 - rgba[1])*3.141593)) + 0.5);
+        c_rgba[2] = static_cast<unsigned char>
+          (127.5*(1.0 + cos((1.0 - rgba[2])*3.141593)) + 0.5);
+        c_rgba[3] = static_cast<unsigned char>(alpha*255.0 + 0.5);
         */
         }
         break;
       case VTK_RAMP_LINEAR:
         {
-        c_rgba[0] = static_cast<unsigned char>((rgba[0])*255.0f + 0.5f);
-        c_rgba[1] = static_cast<unsigned char>((rgba[1])*255.0f + 0.5f);
-        c_rgba[2] = static_cast<unsigned char>((rgba[2])*255.0f + 0.5f);
-        c_rgba[3] = static_cast<unsigned char>(rgba[3]*255.0f + 0.5f);
+        c_rgba[0] = static_cast<unsigned char>(rgba[0]*255.0 + 0.5);
+        c_rgba[1] = static_cast<unsigned char>(rgba[1]*255.0 + 0.5);
+        c_rgba[2] = static_cast<unsigned char>(rgba[2]*255.0 + 0.5);
+        c_rgba[3] = static_cast<unsigned char>(rgba[3]*255.0 + 0.5);
         }
         break;
       case VTK_RAMP_SQRT:
         {
-        c_rgba[0] = static_cast<unsigned char>((sqrt(rgba[0]))*255.0f + 0.5f);
-        c_rgba[1] = static_cast<unsigned char>((sqrt(rgba[1]))*255.0f + 0.5f);
-        c_rgba[2] = static_cast<unsigned char>((sqrt(rgba[2]))*255.0f + 0.5f);
-        c_rgba[3] = static_cast<unsigned char>((sqrt(rgba[3]))*255.0f + 0.5f);
+        c_rgba[0] = static_cast<unsigned char>(sqrt(rgba[0])*255.0 + 0.5);
+        c_rgba[1] = static_cast<unsigned char>(sqrt(rgba[1])*255.0 + 0.5);
+        c_rgba[2] = static_cast<unsigned char>(sqrt(rgba[2])*255.0 + 0.5);
+        c_rgba[3] = static_cast<unsigned char>(sqrt(rgba[3])*255.0 + 0.5);
         }
         break;
+      default:
+        assert("check: impossible case." && 0); // reaching this line is a bug.
+        break;
       }
     }
   this->BuildTime.Modified();
 }
 
+//----------------------------------------------------------------------------
 // Generate lookup table from hue, saturation, value, alpha min/max values. 
 // Table is built from linear ramp of each value.
 void vtkLookupTable::Build()
@@ -206,6 +258,7 @@ void vtkLookupTable::Build()
     }
 }
 
+//----------------------------------------------------------------------------
 // get the color for a scalar value
 void vtkLookupTable::GetColor(double v, double rgb[3])
 {
@@ -216,6 +269,7 @@ void vtkLookupTable::GetColor(double v, double rgb[3])
   rgb[2] = rgb8[2]/255.0;
 }
 
+//----------------------------------------------------------------------------
 // get the opacity (alpha) for a scalar value
 double vtkLookupTable::GetOpacity(double v)
 {
@@ -223,11 +277,12 @@ double vtkLookupTable::GetOpacity(double v)
 
   return rgb8[3]/255.0;
 }
-
+ 
+//----------------------------------------------------------------------------
 // There is a little more to this than simply taking the log10 of the
 // two range values: we do conversion of negative ranges to positive
 // ranges, and conversion of zero to a 'very small number'
-void vtkLookupTableLogRange(double range[2], double logRange[2])
+void vtkLookupTableLogRange(const double range[2], double logRange[2])
 {
   double rmin = range[0];
   double rmax = range[1];
@@ -250,19 +305,20 @@ void vtkLookupTableLogRange(double range[2], double logRange[2])
     }
   if (rmin < 0 && rmax < 0)
     {
-    logRange[0] = log10(-(double)rmin);
-    logRange[1] = log10(-(double)rmax);
+    logRange[0] = log10(-static_cast<double>(rmin));
+    logRange[1] = log10(-static_cast<double>(rmax));
     }
   else if (rmin > 0 && rmax > 0)
     {
-    logRange[0] = log10((double)rmin);
-    logRange[1] = log10((double)rmax);
+    logRange[0] = log10(static_cast<double>(rmin));
+    logRange[1] = log10(static_cast<double>(rmax));
     }
 }
 
+//----------------------------------------------------------------------------
 // Apply log to value, with appropriate constraints.
-inline double vtkApplyLogScale(double v, double range[2], 
-                               double logRange[2])
+inline double vtkApplyLogScale(double v, const double range[2], 
+                               const double logRange[2])
 {
   // is the range set for negative numbers?
   if (range[0] < 0)
@@ -296,29 +352,73 @@ inline double vtkApplyLogScale(double v, double range[2],
       }
     }
   return v;
-}                 
-
-// Apply shift/scale to the scalar value v and do table lookup.
-inline unsigned char *vtkLinearLookup(double v,   
-                                      unsigned char *table,
-                                      double maxIndex,
-                                      double shift, double scale)
-{
-  double findx = (v + shift)*scale;
-  if (findx < 0)
-    {
-    findx = 0;
-    }
-  if (findx > maxIndex)
-    {
-    findx = maxIndex;
-    }
-  return &table[4*static_cast<int>(findx)];
-  /* round
-  return &table[4*(int)(findx + 0.5f)];
-  */
 }
 
+//----------------------------------------------------------------------------
+// Apply shift/scale to the scalar value v and do table lookup.
+inline unsigned char *vtkLinearLookupMain(double v,
+                                          unsigned char *table,
+                                          double maxIndex,
+                                          double shift, double scale)
+{
+  double findx = (v + shift)*scale;
+
+  // do not change this code: it compiles into min/max opcodes
+  findx = (findx > 0 ? findx : 0);
+  findx = (findx < maxIndex ? findx : maxIndex);
+
+  return &table[4*static_cast<unsigned int>(findx)];
+}
+
+template<class T>
+unsigned char *vtkLinearLookup(
+  T v, unsigned char *table, double maxIndex, double shift, double scale,
+  unsigned char *vtkNotUsed(nanColor))
+{
+  return vtkLinearLookupMain(v, table, maxIndex, shift, scale);
+}
+
+//----------------------------------------------------------------------------
+// Check for not-a-number when mapping double or float
+inline unsigned char *vtkLinearLookup(
+  double v, unsigned char *table, double maxIndex, double shift, double scale,
+  unsigned char *nanColor)
+{
+  // calling isnan() instead of vtkMath::IsNan() improves performance
+#ifdef VTK_HAS_ISNAN
+  if (isnan(v))
+#else
+  if (vtkMath::IsNan(v))
+#endif
+    {
+    return nanColor;
+    }
+
+  return vtkLinearLookupMain(v, table, maxIndex, shift, scale);
+}
+
+inline unsigned char *vtkLinearLookup(
+  float v, unsigned char *table, double maxIndex, double shift, double scale,
+  unsigned char *nanColor)
+{
+  return vtkLinearLookup(static_cast<double>(v), table, maxIndex, shift, scale,
+                         nanColor);
+}
+
+//----------------------------------------------------------------------------
+void vtkLookupTable::GetLogRange(const double range[2], double log_range[2])
+{
+  vtkLookupTableLogRange(range, log_range);
+}
+
+//----------------------------------------------------------------------------
+double vtkLookupTable::ApplyLogScale(double v, const double range[2],
+  const double log_range[2])
+{
+  return vtkApplyLogScale(v, range, log_range);
+}
+
+//----------------------------------------------------------------------------
 // Given a scalar value v, return an index into the lookup table
 vtkIdType vtkLookupTable::GetIndex(double v)
 {
@@ -336,8 +436,6 @@ vtkIdType vtkLookupTable::GetIndex(double v)
       }
     else
       {
-      /* while this looks like the wrong scale, it is the correct scale
-       * taking into account the truncation to int that happens below. */
       scale = (maxIndex + 1)/(logRange[1] - logRange[0]);
       }
     v = vtkApplyLogScale(v, this->TableRange, logRange);
@@ -351,8 +449,6 @@ vtkIdType vtkLookupTable::GetIndex(double v)
       }
     else
       {
-      /* while this looks like the wrong scale, it is the correct scale
-       * taking into account the truncation to int that happens below. */
       scale = (maxIndex + 1)/(this->TableRange[1] - this->TableRange[0]);
       }
     }
@@ -370,6 +466,34 @@ vtkIdType vtkLookupTable::GetIndex(double v)
   return static_cast<int>(findx);
 }
 
+//----------------------------------------------------------------------------
+// Given a table, set the internal table and set the number of colors.
+void vtkLookupTable::SetTable(vtkUnsignedCharArray *table)
+{
+  if (table != this->Table && table != NULL)
+    {
+    // Check for incorrect arrays.
+    if (table->GetNumberOfComponents() != this->Table->GetNumberOfComponents())
+      {
+      vtkErrorMacro(<<"Number of components in given table (" 
+                    << table->GetNumberOfComponents()
+                    << ") is incorrect, it should have "
+                    << this->Table->GetNumberOfComponents() 
+                    << "." );
+      return;
+      }
+    this->Table->UnRegister(this);
+    this->Table = table;
+    this->Table->Register(this);
+    this->NumberOfColors = this->Table->GetNumberOfTuples();
+    // If InsertTime is not modified the array will be rebuilt.  So we
+    // use the same approach that the SetTableValue function does.
+    this->InsertTime.Modified();
+    this->Modified();
+    }
+}
+
+//----------------------------------------------------------------------------
 // Given a scalar value v, return an rgba color value from lookup table.
 unsigned char *vtkLookupTable::MapValue(double v)
 {
@@ -377,8 +501,7 @@ unsigned char *vtkLookupTable::MapValue(double v)
   return (this->Table->GetPointer(0) + 4*idx);
 }
 
-// accelerate the mapping by copying the data in 32-bit chunks instead
-// of 8-bit chunks
+//----------------------------------------------------------------------------
 template<class T>
 void vtkLookupTableMapData(vtkLookupTable *self, T *input, 
                            unsigned char *output, int length, 
@@ -392,6 +515,16 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
   unsigned char *cptr;
   double alpha;
 
+  unsigned char nanColor[4];
+  const double *nanColord = self->GetNanColor();
+  for (int c = 0; c < 4; c++)
+    {
+    double v = nanColord[c];
+    if (v < 0.0) { v = 0.0; }
+    else if (v > 1.0) { v = 1.0; }
+    nanColor[c] = static_cast<unsigned char>(v*255.0 + 0.5);
+    }
+
   if ( (alpha=self->GetAlpha()) >= 1.0 ) //no blending required 
     {
     if (self->GetScale() == VTK_SCALE_LOG10)
@@ -406,8 +539,6 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         }
       else
         {
-        /* while this looks like the wrong scale, it is the correct scale
-         * taking into account the truncation to int that happens below. */
         scale = (maxIndex + 1)/(logRange[1] - logRange[0]);
         }
       if (outFormat == VTK_RGBA)
@@ -415,12 +546,13 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         while (--i >= 0) 
           {
           val = vtkApplyLogScale(*input, range, logRange);
-          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale); 
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;     
+          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale, nanColor);
+          output[0] = cptr[0];
+          output[1] = cptr[1];
+          output[2] = cptr[2];
+          output[3] = cptr[3];
           input += inIncr;
+          output += 4;
           }
         }
       else if (outFormat == VTK_RGB)
@@ -428,11 +560,12 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         while (--i >= 0) 
           {
           val = vtkApplyLogScale(*input, range, logRange);
-          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale); 
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;
+          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale, nanColor);
+          output[0] = cptr[0];
+          output[1] = cptr[1];
+          output[2] = cptr[2];
           input += inIncr;
+          output += 3;
           }
         }
       else if (outFormat == VTK_LUMINANCE_ALPHA)
@@ -440,11 +573,12 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         while (--i >= 0) 
           {
           val = vtkApplyLogScale(*input, range, logRange);
-          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale); 
-          *output++ = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 + 
+          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale, nanColor);
+          output[0] = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 +
                                                  cptr[2]*0.11 + 0.5);
-          *output++ = cptr[3];
+          output[1] = cptr[3];
           input += inIncr;
+          output += 2;
           }
         }
       else // outFormat == VTK_LUMINANCE
@@ -452,7 +586,7 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         while (--i >= 0) 
           {
           val = vtkApplyLogScale(*input, range, logRange);
-          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale); 
+          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale, nanColor);
           *output++ = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 + 
                                                  cptr[2]*0.11 + 0.5);
           input += inIncr;
@@ -469,8 +603,6 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         }
       else
         {
-        /* while this looks like the wrong scale, it is the correct scale
-         * taking into account the truncation to int that happens below. */
         scale = (maxIndex + 1)/(range[1] - range[0]);
         }
 
@@ -478,41 +610,48 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         {
         while (--i >= 0) 
           {
-          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale); 
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;     
+          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale,
+                                 nanColor);
+          output[0] = cptr[0];
+          output[1] = cptr[1];
+          output[2] = cptr[2];
+          output[3] = cptr[3];
           input += inIncr;
+          output += 4;
           }
         }
       else if (outFormat == VTK_RGB)
         {
         while (--i >= 0) 
           {
-          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale); 
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;
+          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale,
+                                 nanColor);
+          output[0] = cptr[0];
+          output[1] = cptr[1];
+          output[2] = cptr[2];
           input += inIncr;
+          output += 3;
           }
         }
       else if (outFormat == VTK_LUMINANCE_ALPHA)
         {
         while (--i >= 0) 
           {
-          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale); 
-          *output++ = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 + 
+          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale,
+                                 nanColor);
+          output[0] = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 +
                                                  cptr[2]*0.11 + 0.5);
-          *output++ = cptr[3];
+          output[1] = cptr[3];
           input += inIncr;
+          output += 2;
           }
         }
       else // outFormat == VTK_LUMINANCE
         {
         while (--i >= 0) 
           {
-          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale); 
+          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale,
+                                 nanColor);
           *output++ = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 + 
                                                  cptr[2]*0.11 + 0.5);
           input += inIncr;
@@ -535,8 +674,6 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         }
       else
         {
-        /* while this looks like the wrong scale, it is the correct scale
-         * taking into account the truncation to int that happens below. */
         scale = (maxIndex + 1)/(logRange[1] - logRange[0]);
         }
       if (outFormat == VTK_RGBA)
@@ -544,12 +681,13 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         while (--i >= 0) 
           {
           val = vtkApplyLogScale(*input, range, logRange);
-          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale); 
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = static_cast<unsigned char>((*cptr)*alpha); cptr++;
+          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale, nanColor);
+          output[0] = cptr[0];
+          output[1] = cptr[1];
+          output[2] = cptr[2];
+          output[3] = static_cast<unsigned char>(cptr[3]*alpha + 0.5);
           input += inIncr;
+          output += 4;
           }
         }
       else if (outFormat == VTK_RGB)
@@ -557,11 +695,12 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         while (--i >= 0) 
           {
           val = vtkApplyLogScale(*input, range, logRange);
-          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale); 
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;
+          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale, nanColor);
+          output[0] = cptr[0];
+          output[1] = cptr[1];
+          output[2] = cptr[2];
           input += inIncr;
+          output += 3;
           }
         }
       else if (outFormat == VTK_LUMINANCE_ALPHA)
@@ -569,11 +708,12 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         while (--i >= 0) 
           {
           val = vtkApplyLogScale(*input, range, logRange);
-          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale); 
-          *output++ = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 + 
+          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale, nanColor);
+          output[0] = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 +
                                                  cptr[2]*0.11 + 0.5);
-          *output++ = static_cast<unsigned char>(alpha*cptr[3]);
+          output[1] = static_cast<unsigned char>(alpha*cptr[3] + 0.5);
           input += inIncr;
+          output += 2;
           }
         }
       else // outFormat == VTK_LUMINANCE
@@ -581,7 +721,7 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         while (--i >= 0) 
           {
           val = vtkApplyLogScale(*input, range, logRange);
-          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale); 
+          cptr = vtkLinearLookup(val, table, maxIndex, shift, scale, nanColor);
           *output++ = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 + 
                                                  cptr[2]*0.11 + 0.5);
           input += inIncr;
@@ -598,8 +738,6 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         }
       else
         {
-        /* while this looks like the wrong scale, it is the correct scale
-         * taking into account the truncation to int that happens below. */
         scale = (maxIndex + 1)/(range[1] - range[0]);
         }
 
@@ -607,41 +745,48 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
         {
         while (--i >= 0) 
           {
-          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale); 
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = static_cast<unsigned char>((*cptr)*alpha); cptr++;
+          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale,
+                                 nanColor);
+          output[0] = cptr[0];
+          output[1] = cptr[1];
+          output[2] = cptr[2];
+          output[3] = static_cast<unsigned char>(cptr[3]*alpha + 0.5);
           input += inIncr;
+          output += 4;
           }
         }
       else if (outFormat == VTK_RGB)
         {
         while (--i >= 0) 
           {
-          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale); 
-          *output++ = *cptr++;
-          *output++ = *cptr++;
-          *output++ = *cptr++;
+          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale,
+                                 nanColor);
+          output[0] = cptr[0];
+          output[1] = cptr[1];
+          output[2] = cptr[2];
           input += inIncr;
+          output += 3;
           }
         }
       else if (outFormat == VTK_LUMINANCE_ALPHA)
         {
         while (--i >= 0) 
           {
-          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale); 
-          *output++ = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 + 
+          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale,
+                                 nanColor);
+          output[0] = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 +
                                                  cptr[2]*0.11 + 0.5);
-          *output++ = static_cast<unsigned char>(cptr[3]*alpha);
+          output[1] = static_cast<unsigned char>(cptr[3]*alpha + 0.5);
           input += inIncr;
+          output += 2;
           }
         }
       else // outFormat == VTK_LUMINANCE
         {
         while (--i >= 0) 
           {
-          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale); 
+          cptr = vtkLinearLookup(*input, table, maxIndex, shift, scale,
+                                 nanColor);
           *output++ = static_cast<unsigned char>(cptr[0]*0.30 + cptr[1]*0.59 + 
                                                  cptr[2]*0.11 + 0.5);
           input += inIncr;
@@ -651,37 +796,7 @@ void vtkLookupTableMapData(vtkLookupTable *self, T *input,
     }//alpha blending
 }
 
-// Although this is a relatively expensive calculation,
-// it is only done on the first render. Colors are cached
-// for subsequent renders.
-template<class T>
-void vtkLookupTableMapMag(vtkLookupTable *self, T *input, 
-                          unsigned char *output, int length, 
-                          int inIncr, int outFormat)
-{
-  double tmp, sum;
-  double *mag;
-  int i, j;
-
-  mag = new double[length];
-  for (i = 0; i < length; ++i)
-    {
-    sum = 0;
-    for (j = 0; j < inIncr; ++j)
-      {
-      tmp = (double)(*input);  
-      sum += (tmp * tmp);
-      ++input;
-      }
-    mag[i] = sqrt(sum);
-    }
-
-  vtkLookupTableMapData(self, mag, output, length, 1, outFormat);
-
-  delete [] mag;
-}
-
-
+//----------------------------------------------------------------------------
 void vtkLookupTable::MapScalarsThroughTable2(void *input, 
                                              unsigned char *output,
                                              int inputDataType, 
@@ -689,59 +804,6 @@ void vtkLookupTable::MapScalarsThroughTable2(void *input,
                                              int inputIncrement,
                                              int outputFormat)
 {
-  if (this->UseMagnitude && inputIncrement > 1)
-    {
-    switch (inputDataType)
-      {
-      case VTK_BIT:
-        vtkErrorMacro("Cannot comput magnitude of bit array.");
-        break;
-      case VTK_CHAR:
-        vtkLookupTableMapMag(this,static_cast<char *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return; 
-      case VTK_UNSIGNED_CHAR:
-        vtkLookupTableMapMag(this,static_cast<unsigned char *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return;
-      case VTK_SHORT:
-        vtkLookupTableMapMag(this,static_cast<short *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return;
-      case VTK_UNSIGNED_SHORT:
-        vtkLookupTableMapMag(this,static_cast<unsigned short *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return;
-      case VTK_INT:
-        vtkLookupTableMapMag(this,static_cast<int *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return;
-      case VTK_UNSIGNED_INT:
-        vtkLookupTableMapMag(this,static_cast<unsigned int *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return;
-      case VTK_LONG:
-        vtkLookupTableMapMag(this,static_cast<long *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return;
-      case VTK_UNSIGNED_LONG:
-        vtkLookupTableMapMag(this,static_cast<unsigned long *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return;
-      case VTK_FLOAT:
-        vtkLookupTableMapMag(this,static_cast<float *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return;
-      case VTK_DOUBLE:
-        vtkLookupTableMapMag(this,static_cast<double *>(input),output,
-                             numberOfValues,inputIncrement,outputFormat);
-        return;
-      default:
-        vtkErrorMacro(<< "MapImageThroughTable: Unknown input ScalarType");
-        return;
-      }
-    }
-
   switch (inputDataType)
     {
     case VTK_BIT:
@@ -763,68 +825,18 @@ void vtkLookupTable::MapScalarsThroughTable2(void *input,
       bitArray->Delete();
       }
       break;
-      
-    case VTK_CHAR:
-      vtkLookupTableMapData(this,static_cast<char *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
-    case VTK_UNSIGNED_CHAR:
-      vtkLookupTableMapData(this,static_cast<unsigned char *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
-    case VTK_SHORT:
-      vtkLookupTableMapData(this,static_cast<short *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-    break;
-      
-    case VTK_UNSIGNED_SHORT:
-      vtkLookupTableMapData(this,static_cast<unsigned short *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
-    case VTK_ID_TYPE:
-      vtkLookupTableMapData(this,static_cast<vtkIdType *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
-    case VTK_INT:
-      vtkLookupTableMapData(this,static_cast<int *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
-    case VTK_UNSIGNED_INT:
-      vtkLookupTableMapData(this,static_cast<unsigned int *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
-    case VTK_LONG:
-      vtkLookupTableMapData(this,static_cast<long *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
-    case VTK_UNSIGNED_LONG:
-      vtkLookupTableMapData(this,static_cast<unsigned long *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
-    case VTK_FLOAT:
-      vtkLookupTableMapData(this,static_cast<float *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
-    case VTK_DOUBLE:
-      vtkLookupTableMapData(this,static_cast<double *>(input),output,
-                            numberOfValues,inputIncrement,outputFormat);
-      break;
-      
+
+    vtkTemplateMacro(
+      vtkLookupTableMapData(this,static_cast<VTK_TT*>(input),output,
+                            numberOfValues,inputIncrement,outputFormat)
+      );
     default:
       vtkErrorMacro(<< "MapImageThroughTable: Unknown input ScalarType");
       return;
     }
 }  
 
+//----------------------------------------------------------------------------
 // Specify the number of values (i.e., colors) in the lookup
 // table. This method simply allocates memory and prepares the table
 // for use with SetTableValue(). It differs from Build() method in
@@ -840,6 +852,7 @@ void vtkLookupTable::SetNumberOfTableValues(vtkIdType number)
   this->Table->SetNumberOfTuples(number);
 }
 
+//----------------------------------------------------------------------------
 // Directly load color into lookup table. Use [0,1] double values for color
 // component specification. Make sure that you've either used the
 // Build() method or used SetNumberOfTableValues() prior to using this method.
@@ -861,15 +874,16 @@ void vtkLookupTable::SetTableValue(vtkIdType indx, double rgba[4])
 
   unsigned char *_rgba = this->Table->WritePointer(4*indx,4);
 
-  _rgba[0] = static_cast<unsigned char>(rgba[0]*255.0f + 0.5f);
-  _rgba[1] = static_cast<unsigned char>(rgba[1]*255.0f + 0.5f);
-  _rgba[2] = static_cast<unsigned char>(rgba[2]*255.0f + 0.5f);
-  _rgba[3] = static_cast<unsigned char>(rgba[3]*255.0f + 0.5f);
+  _rgba[0] = static_cast<unsigned char>(rgba[0]*255.0 + 0.5);
+  _rgba[1] = static_cast<unsigned char>(rgba[1]*255.0 + 0.5);
+  _rgba[2] = static_cast<unsigned char>(rgba[2]*255.0 + 0.5);
+  _rgba[3] = static_cast<unsigned char>(rgba[3]*255.0 + 0.5);
 
   this->InsertTime.Modified();
   this->Modified();
 }
 
+//----------------------------------------------------------------------------
 // Directly load color into lookup table. Use [0,1] double values for color 
 // component specification.
 void vtkLookupTable::SetTableValue(vtkIdType indx, double r, double g, double b, 
@@ -880,6 +894,7 @@ void vtkLookupTable::SetTableValue(vtkIdType indx, double r, double g, double b,
   this->SetTableValue(indx,rgba);
 }
 
+//----------------------------------------------------------------------------
 // Return a rgba color value for the given index into the lookup Table. Color
 // components are expressed as [0,1] double values.
 void vtkLookupTable::GetTableValue(vtkIdType indx, double rgba[4])
@@ -905,6 +920,7 @@ double *vtkLookupTable::GetTableValue(vtkIdType indx)
   return this->RGBA;
 }
 
+//----------------------------------------------------------------------------
 void vtkLookupTable::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
@@ -921,6 +937,9 @@ void vtkLookupTable::PrintSelf(ostream& os, vtkIndent indent)
      << this->ValueRange[1] << ")\n";
   os << indent << "AlphaRange: (" << this->AlphaRange[0] << ", "
      << this->AlphaRange[1] << ")\n";
+  os << indent << "NanColor: (" << this->NanColor[0] << ", "
+     << this->NanColor[1] << ", " << this->NanColor[2] << ", "
+     << this->NanColor[3] << ")\n";
   os << indent << "NumberOfTableValues: "
      << this->GetNumberOfTableValues() << "\n";
   os << indent << "NumberOfColors: " << this->NumberOfColors << "\n";
@@ -928,17 +947,35 @@ void vtkLookupTable::PrintSelf(ostream& os, vtkIndent indent)
      << (this->Ramp == VTK_RAMP_SCURVE ? "SCurve\n" : "Linear\n");
   os << indent << "InsertTime: " <<this->InsertTime.GetMTime() << "\n";
   os << indent << "BuildTime: " <<this->BuildTime.GetMTime() << "\n";
+  os << indent << "Table: ";
+  if( this->Table )
+    {
+    this->Table->PrintSelf(os << "\n", indent.GetNextIndent());
+    }
+  else
+    {
+    // Should not happen
+    os << "(none)\n";
+    }
 }
 
-void vtkLookupTable::DeepCopy(vtkLookupTable *lut)
+//----------------------------------------------------------------------------
+void vtkLookupTable::DeepCopy(vtkScalarsToColors *obj)
 {
-  if (!lut)
+  if (!obj)
     {
     return;
     }
-  this->Alpha               = lut->Alpha;
-  this->VectorMode          = lut->VectorMode;
-  this->VectorComponent     = lut->VectorMode;
+
+  vtkLookupTable *lut = vtkLookupTable::SafeDownCast(obj);
+
+  if (!lut)
+    {
+    vtkErrorMacro("Cannot DeepCopy a " << obj->GetClassName()
+                  << " into a vtkLookupTable.");
+    return;
+    }
+
   this->Scale               = lut->Scale;
   this->TableRange[0]       = lut->TableRange[0];
   this->TableRange[1]       = lut->TableRange[1];
@@ -952,6 +989,15 @@ void vtkLookupTable::DeepCopy(vtkLookupTable *lut)
   this->AlphaRange[1]       = lut->AlphaRange[1];
   this->NumberOfColors      = lut->NumberOfColors;
   this->Ramp                = lut->Ramp;
+  this->InsertTime          = lut->InsertTime;
+  this->BuildTime           = lut->BuildTime;
   this->Table->DeepCopy(lut->Table);
+
+  this->Superclass::DeepCopy(obj);
 }
 
+//----------------------------------------------------------------------------
+vtkIdType vtkLookupTable::GetNumberOfAvailableColors()
+{
+  return this->Table->GetNumberOfTuples();
+}
