@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkMatrixToHomogeneousTransform.cxx,v $
+  Module:    vtkMatrixToHomogeneousTransform.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,7 +17,6 @@
 #include "vtkMatrix4x4.h"
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkMatrixToHomogeneousTransform, "$Revision: 1.8 $");
 vtkStandardNewMacro(vtkMatrixToHomogeneousTransform);
 vtkCxxSetObjectMacro(vtkMatrixToHomogeneousTransform,Input,vtkMatrix4x4);
 
@@ -73,7 +72,7 @@ void vtkMatrixToHomogeneousTransform::InternalDeepCopy(
                                                 vtkAbstractTransform *gtrans)
 {
   vtkMatrixToHomogeneousTransform *transform = 
-    (vtkMatrixToHomogeneousTransform *)gtrans;
+    static_cast<vtkMatrixToHomogeneousTransform *>(gtrans);
 
   this->SetInput(transform->Input);
 
@@ -105,3 +104,12 @@ unsigned long vtkMatrixToHomogeneousTransform::GetMTime()
     }
   return mtime;
 }
+
+#ifndef VTK_LEGACY_REMOVE
+void vtkMatrixToHomogeneousTransform::SetMatrix(vtkMatrix4x4 *matrix)
+{
+  VTK_LEGACY_REPLACED_BODY(vtkAssemblyNode::SetMatrix, "VTK 5.0",
+                           vtkAssemblyNode::SetInput);
+  this->SetInput(matrix);
+}
+#endif
