@@ -1,17 +1,14 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkMultiThreader.cxx,v $
-  Language:  C++
-  Date:      $Date: 2003/01/29 21:14:52 $
-  Version:   $Revision: 1.43 $
+  Module:    vtkMultiThreader.cxx
 
-  Copyright (c) 1993-2002 Ken Martin, Will Schroeder, Bill Lorensen 
+  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
   See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
 
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
+     This software is distributed WITHOUT ANY WARRANTY; without even
+     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
@@ -19,8 +16,8 @@
 
 #include "vtkMutexLock.h"
 #include "vtkObjectFactory.h"
+#include "vtkWindows.h"
 
-vtkCxxRevisionMacro(vtkMultiThreader, "$Revision: 1.43 $");
 vtkStandardNewMacro(vtkMultiThreader);
 
 // These are the includes necessary for multithreaded rendering on an SGI
@@ -44,7 +41,8 @@ typedef vtkThreadFunctionType vtkExternCThreadFunctionType;
 #endif
 
 #ifdef __APPLE__
-#include <Carbon/Carbon.h>
+#include <sys/types.h>
+#include <sys/sysctl.h>
 #endif
 
 // Initialize static member that controls global maximum number of threads
@@ -101,9 +99,17 @@ int vtkMultiThreader::GetGlobalDefaultNumberOfThreads()
 #endif
 
 #ifdef __APPLE__
-    // MPProcessors returns the physical number of processors present
-    // MPProcessorsScheduled returns number of active processors
-    num = MPProcessors();
+    // Determine the number of CPU cores. Prefer sysctlbyname()
+    // over MPProcessors() because it doesn't require CoreServices
+    // (which is only available in 32bit on Mac OS X 10.4).
+    // hw.logicalcpu takes into account cores/CPUs that are
+    // disabled because of power management.
+    size_t dataLen = sizeof(int); // 'num' is an 'int'
+    int result = sysctlbyname ("hw.logicalcpu", &num, &dataLen, NULL, 0);
+    if (result == -1)
+      {
+      num = 1;
+      }
 #endif
 
 #ifdef _WIN32
@@ -164,6 +170,32 @@ vtkMultiThreader::vtkMultiThreader()
 // Destructor. Nothing allocated so nothing needs to be done here.
 vtkMultiThreader::~vtkMultiThreader()
 {
+  int i;
+  
+  for ( i = 0; i < VTK_MAX_THREADS; i++ )
+    {
+    if ( this->ThreadInfoArray[i].ActiveFlagLock )
+      {
+      this->ThreadInfoArray[i].ActiveFlagLock->Delete();
+      }
+    
+    if ( this->SpawnedThreadActiveFlagLock[i] )
+      {
+      this->SpawnedThreadActiveFlagLock[i]->Delete();
+      }
+    }
+}
+
+//----------------------------------------------------------------------------
+int vtkMultiThreader::GetNumberOfThreads()
+{
+  int num = this->NumberOfThreads;
+  if(vtkMultiThreaderGlobalMaximumNumberOfThreads > 0 &&
+     num > vtkMultiThreaderGlobalMaximumNumberOfThreads)
+    {
+    num = vtkMultiThreaderGlobalMaximumNumberOfThreads;
+    }
+  return num;
 }
 
 // Set the user defined method that will be run on NumberOfThreads threads
@@ -183,9 +215,10 @@ void vtkMultiThreader::SetMultipleMethod( int index,
                                           vtkThreadFunctionType f, void *data )
 { 
   // You can only set the method for 0 through NumberOfThreads-1
-  if ( index >= this->NumberOfThreads ) {
+  if ( index >= this->NumberOfThreads )
+    {
     vtkErrorMacro( << "Can't set method " << index << 
-    " with a thread count of " << this->NumberOfThreads );
+                   " with a thread count of " << this->NumberOfThreads );
     }
   else
     {
@@ -242,12 +275,12 @@ void vtkMultiThreader::SingleMethodExecute()
   // of their process ids for use later in the waitid call
   for (thread_loop = 1; thread_loop < this->NumberOfThreads; thread_loop++ )
     {
-    this->ThreadInfoArray[thread_loop].UserData    = this->SingleData;
+    this->ThreadInfoArray[thread_loop].UserData        = this->SingleData;
     this->ThreadInfoArray[thread_loop].NumberOfThreads = this->NumberOfThreads;
     process_id[thread_loop] = 
       CreateThread(NULL, 0, this->SingleMethod, 
              ((void *)(&this->ThreadInfoArray[thread_loop])), 0, &threadId);
-    if (process_id == NULL)
+    if (process_id[thread_loop] == NULL)
       {
       vtkErrorMacro("Error in thread creation !!!");
       } 
@@ -285,7 +318,7 @@ void vtkMultiThreader::SingleMethodExecute()
 
   for ( thread_loop = 1; thread_loop < this->NumberOfThreads; thread_loop++ )
     {
-    this->ThreadInfoArray[thread_loop].UserData    = this->SingleData;
+    this->ThreadInfoArray[thread_loop].UserData        = this->SingleData;
     this->ThreadInfoArray[thread_loop].NumberOfThreads = this->NumberOfThreads;
     process_id[thread_loop] = 
       sproc( this->SingleMethod, PR_SADDR, 
@@ -297,7 +330,7 @@ void vtkMultiThreader::SingleMethodExecute()
     }
   
   // Now, the parent thread calls this->SingleMethod() itself
-  this->ThreadInfoArray[0].UserData = this->SingleData;
+  this->ThreadInfoArray[0].UserData        = this->SingleData;
   this->ThreadInfoArray[0].NumberOfThreads = this->NumberOfThreads;
   this->SingleMethod((void *)(&this->ThreadInfoArray[0]) );
 
@@ -333,7 +366,7 @@ void vtkMultiThreader::SingleMethodExecute()
   
   for ( thread_loop = 1; thread_loop < this->NumberOfThreads; thread_loop++ )
     {
-    this->ThreadInfoArray[thread_loop].UserData    = this->SingleData;
+    this->ThreadInfoArray[thread_loop].UserData        = this->SingleData;
     this->ThreadInfoArray[thread_loop].NumberOfThreads = this->NumberOfThreads;
 
 #ifdef VTK_HP_PTHREADS
@@ -356,7 +389,7 @@ void vtkMultiThreader::SingleMethodExecute()
     }
   
   // Now, the parent thread calls this->SingleMethod() itself
-  this->ThreadInfoArray[0].UserData = this->SingleData;
+  this->ThreadInfoArray[0].UserData        = this->SingleData;
   this->ThreadInfoArray[0].NumberOfThreads = this->NumberOfThreads;
   this->SingleMethod((void *)(&this->ThreadInfoArray[0]) );
 
@@ -372,7 +405,7 @@ void vtkMultiThreader::SingleMethodExecute()
 #ifndef VTK_USE_SPROC
 #ifndef VTK_USE_PTHREADS
   // There is no multi threading, so there is only one thread.
-  this->ThreadInfoArray[0].UserData    = this->SingleData;
+  this->ThreadInfoArray[0].UserData        = this->SingleData;
   this->ThreadInfoArray[0].NumberOfThreads = this->NumberOfThreads;
   this->SingleMethod( (void *)(&this->ThreadInfoArray[0]) );
 #endif
@@ -437,14 +470,14 @@ void vtkMultiThreader::MultipleMethodExecute()
     process_id[thread_loop] = 
       CreateThread(NULL, 0, this->MultipleMethod[thread_loop], 
              ((void *)(&this->ThreadInfoArray[thread_loop])), 0, &threadId);
-    if (process_id == NULL)
+    if (process_id[thread_loop] == NULL)
       {
       vtkErrorMacro("Error in thread creation !!!");
       } 
     }
   
   // Now, the parent thread calls the last method itself
-  this->ThreadInfoArray[0].UserData = this->MultipleData[0];
+  this->ThreadInfoArray[0].UserData        = this->MultipleData[0];
   this->ThreadInfoArray[0].NumberOfThreads = this->NumberOfThreads;
   (this->MultipleMethod[0])((void *)(&this->ThreadInfoArray[0]) );
 
@@ -485,7 +518,7 @@ void vtkMultiThreader::MultipleMethodExecute()
     }
   
   // Now, the parent thread calls the last method itself
-  this->ThreadInfoArray[0].UserData = this->MultipleData[0];
+  this->ThreadInfoArray[0].UserData        = this->MultipleData[0];
   this->ThreadInfoArray[0].NumberOfThreads = this->NumberOfThreads;
   (this->MultipleMethod[0])((void *)(&this->ThreadInfoArray[0]) );
 
@@ -557,7 +590,7 @@ void vtkMultiThreader::MultipleMethodExecute()
 #ifndef VTK_USE_SPROC
 #ifndef VTK_USE_PTHREADS
   // There is no multi threading, so there is only one thread.
-  this->ThreadInfoArray[0].UserData    = this->MultipleData[0];
+  this->ThreadInfoArray[0].UserData        = this->MultipleData[0];
   this->ThreadInfoArray[0].NumberOfThreads = this->NumberOfThreads;
   (this->MultipleMethod[0])( (void *)(&this->ThreadInfoArray[0]) );
 #endif
@@ -565,21 +598,15 @@ void vtkMultiThreader::MultipleMethodExecute()
 #endif
 }
 
-int vtkMultiThreader::SpawnThread( vtkThreadFunctionType f, void *UserData )
+int vtkMultiThreader::SpawnThread( vtkThreadFunctionType f, void *userdata )
 {
   int id;
 
-  // avoid a warning
-  vtkThreadFunctionType tf;
-  tf = f; tf= tf;
-  
 #ifdef VTK_USE_WIN32_THREADS
   DWORD              threadId;
 #endif
 
-  id = 0;
-
-  while ( id < VTK_MAX_THREADS )
+  for ( id = 0; id < VTK_MAX_THREADS; id++ )
     {
     if ( this->SpawnedThreadActiveFlagLock[id] == NULL )
       {
@@ -594,8 +621,6 @@ int vtkMultiThreader::SpawnThread( vtkThreadFunctionType f, void *UserData )
       break;
       }
     this->SpawnedThreadActiveFlagLock[id]->Unlock();
-      
-    id++;
     }
 
   if ( id >= VTK_MAX_THREADS )
@@ -604,8 +629,7 @@ int vtkMultiThreader::SpawnThread( vtkThreadFunctionType f, void *UserData )
     return -1;
     }
 
-
-  this->SpawnedThreadInfoArray[id].UserData        = UserData;
+  this->SpawnedThreadInfoArray[id].UserData        = userdata;
   this->SpawnedThreadInfoArray[id].NumberOfThreads = 1;
   this->SpawnedThreadInfoArray[id].ActiveFlag = 
     &this->SpawnedThreadActiveFlag[id];
@@ -677,31 +701,52 @@ int vtkMultiThreader::SpawnThread( vtkThreadFunctionType f, void *UserData )
   return id;
 }
 
-void vtkMultiThreader::TerminateThread( int ThreadID )
+void vtkMultiThreader::TerminateThread( int threadID )
 {
-
-  if ( !this->SpawnedThreadActiveFlag[ThreadID] ) {
+  // check if the threadID argument is in range
+  if ( threadID >= VTK_MAX_THREADS )
+    {
+    vtkErrorMacro("ThreadID is out of range. Must be less that " << VTK_MAX_THREADS );
     return;
-  }
-
-  this->SpawnedThreadActiveFlagLock[ThreadID]->Lock();
-  this->SpawnedThreadActiveFlag[ThreadID] = 0;
-  this->SpawnedThreadActiveFlagLock[ThreadID]->Unlock();
+    }
+  
+  // If we don't have a lock, then this thread is definitely not active
+  if ( !this->SpawnedThreadActiveFlag[threadID] )
+    {
+    return;
+    }
+  
+  // If we do have a lock, use it and find out the status of the active flag
+  this->SpawnedThreadActiveFlagLock[threadID]->Lock();
+  int val = this->SpawnedThreadActiveFlag[threadID];
+  this->SpawnedThreadActiveFlagLock[threadID]->Unlock();
+  
+  // If the active flag is 0, return since this thread is not active
+  if ( val == 0 )
+    {
+    return;
+    }
+  
+  // OK - now we know we have an active thread - set the active flag to 0
+  // to indicate to the thread that it should terminate itself
+  this->SpawnedThreadActiveFlagLock[threadID]->Lock();
+  this->SpawnedThreadActiveFlag[threadID] = 0;
+  this->SpawnedThreadActiveFlagLock[threadID]->Unlock();
 
 #ifdef VTK_USE_WIN32_THREADS
-  WaitForSingleObject(this->SpawnedThreadProcessID[ThreadID], INFINITE);
-  CloseHandle(this->SpawnedThreadProcessID[ThreadID]);
+  WaitForSingleObject(this->SpawnedThreadProcessID[threadID], INFINITE);
+  CloseHandle(this->SpawnedThreadProcessID[threadID]);
 #endif
 
 #ifdef VTK_USE_SPROC
   siginfo_t info_ptr;
 
-  waitid( P_PID, (id_t) this->SpawnedThreadProcessID[ThreadID], 
+  waitid( P_PID, (id_t) this->SpawnedThreadProcessID[threadID], 
           &info_ptr, WEXITED );
 #endif
 
 #ifdef VTK_USE_PTHREADS
-  pthread_join( this->SpawnedThreadProcessID[ThreadID], NULL );
+  pthread_join( this->SpawnedThreadProcessID[threadID], NULL );
 #endif
 
 #ifndef VTK_USE_WIN32_THREADS
@@ -714,11 +759,68 @@ void vtkMultiThreader::TerminateThread( int ThreadID )
 #endif
 #endif
 
-  this->SpawnedThreadActiveFlagLock[ThreadID]->Delete();
-  this->SpawnedThreadActiveFlagLock[ThreadID] = NULL;
+  this->SpawnedThreadActiveFlagLock[threadID]->Delete();
+  this->SpawnedThreadActiveFlagLock[threadID] = NULL;
 
 }
 
+//----------------------------------------------------------------------------
+vtkMultiThreaderIDType vtkMultiThreader::GetCurrentThreadID()
+{
+#if defined(VTK_USE_PTHREADS)
+  return pthread_self();
+#elif defined(VTK_USE_WIN32_THREADS)
+  return GetCurrentThreadId();
+#elif defined(VTK_USE_SPROC)
+  return getpid();
+#else
+  // No threading implementation.  Assume all callers are in the same
+  // thread.
+  return 0;
+#endif
+}
+
+int vtkMultiThreader::IsThreadActive( int threadID )
+{
+  // check if the threadID argument is in range
+  if ( threadID >= VTK_MAX_THREADS )
+    {
+    vtkErrorMacro("ThreadID is out of range. Must be less that " << VTK_MAX_THREADS );
+    return 0;
+    }
+  
+  // If we don't have a lock, then this thread is not active
+  if ( this->SpawnedThreadActiveFlagLock[threadID] == NULL )
+    {
+    return 0;
+    }
+  
+  // We have a lock - use it to get the active flag value
+  this->SpawnedThreadActiveFlagLock[threadID]->Lock();
+  int val = this->SpawnedThreadActiveFlag[threadID];
+  this->SpawnedThreadActiveFlagLock[threadID]->Unlock();
+  
+  // now return that value
+  return val;
+}
+
+//----------------------------------------------------------------------------
+int vtkMultiThreader::ThreadsEqual(vtkMultiThreaderIDType t1,
+                                   vtkMultiThreaderIDType t2)
+{
+#if defined(VTK_USE_PTHREADS)
+  return pthread_equal(t1, t2) != 0;
+#elif defined(VTK_USE_WIN32_THREADS)
+  return t1 == t2;
+#elif defined(VTK_USE_SPROC)
+  return t1 == t2;
+#else
+  // No threading implementation.  Assume all callers are in the same
+  // thread.
+  return 1;
+#endif
+}
+
 // Print method for the multithreader
 void vtkMultiThreader::PrintSelf(ostream& os, vtkIndent indent)
 {
