@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkMutexLock.cxx,v $
+  Module:    vtkMutexLock.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,7 +15,10 @@
 #include "vtkMutexLock.h"
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkMutexLock, "$Revision: 1.22 $");
+#ifdef VTK_USE_WIN32_THREADS
+# include "vtkWindows.h"
+#endif
+
 vtkStandardNewMacro(vtkMutexLock);
 
 // New for the SimpleMutex
@@ -89,20 +92,6 @@ void vtkSimpleMutexLock::Unlock()
 #endif
 }
 
-int vtkSimpleMutexLock::IsA(const char *type)
-{
-  if ( !strcmp(this->vtkSimpleMutexLock::GetClassName(),type) )
-    {
-    return 1;
-    }
-  return 0;
-}
-
-vtkSimpleMutexLock *vtkSimpleMutexLock::SafeDownCast(vtkSimpleMutexLock *o)
-{
-  return (vtkSimpleMutexLock *)o;
-}
-  
 void vtkMutexLock::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
