@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkObjectFactory.cxx,v $
+  Module:    vtkObjectFactory.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,16 +15,16 @@
 #include "vtkObjectFactory.h"
 
 #include "vtkDebugLeaks.h"
-#include "vtkDirectory.h"
 #include "vtkDynamicLoader.h"
 #include "vtkObjectFactoryCollection.h"
 #include "vtkOverrideInformation.h"
 #include "vtkOverrideInformationCollection.h"
 #include "vtkVersion.h"
 
+#include "vtksys/Directory.hxx"
+
 #include <ctype.h>
 
-vtkCxxRevisionMacro(vtkObjectFactory, "$Revision: 1.42 $");
 
 vtkObjectFactoryCollection* vtkObjectFactory::RegisteredFactories = 0;
 
@@ -54,9 +54,10 @@ vtkObject* vtkObjectFactory::CreateInstance(const char* vtkclassname)
     }
   
   vtkObjectFactory* factory;
-  
-  for(vtkObjectFactory::RegisteredFactories->InitTraversal();
-      (factory = vtkObjectFactory::RegisteredFactories->GetNextItem());)
+  vtkCollectionSimpleIterator osit;
+  for(vtkObjectFactory::RegisteredFactories->InitTraversal(osit);
+      (factory = vtkObjectFactory::RegisteredFactories->
+       GetNextObjectFactory(osit));)
     {
     vtkObject* newobject = factory->CreateObject(vtkclassname);
     if(newobject)
@@ -130,13 +131,13 @@ void vtkObjectFactory::LoadDynamicFactories()
     SeparatorPosition = strchr(LoadPath, PathSeparator);
     // if not found then use the whole string
     if(SeparatorPosition == 0)
-    {
+      {
       PathLength = strlen(LoadPath);
-    }
+      }
     else
-    {
-      PathLength = SeparatorPosition - LoadPath;
-    }
+      {
+      PathLength = static_cast<size_t>(SeparatorPosition - LoadPath);
+      }
     // copy the path out of LoadPath into CurrentPath
     strncpy(CurrentPath, LoadPath, PathLength);
     // add a null terminator
@@ -154,7 +155,7 @@ void vtkObjectFactory::LoadDynamicFactories()
 // a full path
 static char* CreateFullPath(const char* path, const char* file)
 {
-  int lenpath = static_cast<int>(strlen(path));
+  size_t lenpath = strlen(path);
   char* ret = new char[lenpath + strlen(file)+2];
 #ifdef _WIN32
   const char sep = '\\';
@@ -191,7 +192,7 @@ inline int vtkNameIsSharedLibrary(const char* name)
   
   for(int i = 0; i < len; i++)
     {
-    copy[i] = tolower(name[i]);
+    copy[i] = static_cast<char>(tolower(name[i]));
     }
   copy[len] = 0;
   char* ret = strstr(copy, vtkDynamicLoader::LibExtension());
@@ -201,17 +202,16 @@ inline int vtkNameIsSharedLibrary(const char* name)
 
 void vtkObjectFactory::LoadLibrariesInPath(const char* path)
 {
-  vtkDirectory* dir = vtkDirectory::New();
-  if(!dir->Open(path))
+  vtksys::Directory dir;
+  if(!dir.Load(path))
     {
-    dir->Delete();
     return;
     }
-  
+
   // Attempt to load each file in the directory as a shared library
-  for(int i = 0; i < dir->GetNumberOfFiles(); i++)
+  for(unsigned long i = 0; i < dir.GetNumberOfFiles(); i++)
     {
-    const char* file = dir->GetFile(i);
+    const char* file = dir.GetFile(i);
     // try to make sure the file has at least the extension
     // for a shared library in it.
     if(vtkNameIsSharedLibrary(file))
@@ -223,17 +223,17 @@ void vtkObjectFactory::LoadLibrariesInPath(const char* path)
         // Look for the symbol vtkLoad, vtkGetFactoryCompilerUsed,
         // and vtkGetFactoryVersion in the library
         VTK_LOAD_FUNCTION loadfunction
-          = (VTK_LOAD_FUNCTION)
+          = (VTK_LOAD_FUNCTION)(
           vtkDynamicLoader::GetSymbolAddress(lib,
-                                             "vtkLoad");
+                                             "vtkLoad"));
         VTK_COMPILER_FUNCTION compilerFunction
-          = (VTK_COMPILER_FUNCTION)
+          = (VTK_COMPILER_FUNCTION)(
           vtkDynamicLoader::GetSymbolAddress(lib,
-                                             "vtkGetFactoryCompilerUsed");
+                                             "vtkGetFactoryCompilerUsed"));
         VTK_VERSION_FUNCTION versionFunction
-          = (VTK_VERSION_FUNCTION)
+          = (VTK_VERSION_FUNCTION)(
           vtkDynamicLoader::GetSymbolAddress(lib,
-                                             "vtkGetFactoryVersion");
+                                             "vtkGetFactoryVersion"));
         // if the symbol is found call it to create the factory
         // from the library
         if(loadfunction && compilerFunction && versionFunction)
@@ -259,7 +259,7 @@ void vtkObjectFactory::LoadLibrariesInPath(const char* path)
             newfactory->LibraryCompilerUsed = 
               strcpy(new char[strlen(compiler)+1], compiler);
             // initialize class members if load worked
-            newfactory->LibraryHandle = (void*)lib;
+            newfactory->LibraryHandle = static_cast<void*>(lib);
             newfactory->LibraryPath = 
               strcpy(new char[strlen(fullpath)+1], fullpath);
             vtkObjectFactory::RegisterFactory(newfactory);
@@ -275,7 +275,6 @@ void vtkObjectFactory::LoadLibrariesInPath(const char* path)
       delete [] fullpath;
       }
     }
-  dir->Delete();
 }
 
 
@@ -399,7 +398,7 @@ void vtkObjectFactory::UnRegisterFactory(vtkObjectFactory* factory)
   vtkObjectFactory::RegisteredFactories->RemoveItem(factory);
   if(lib)
     {
-    vtkDynamicLoader::CloseLibrary((vtkLibHandle)lib);
+    vtkDynamicLoader::CloseLibrary(static_cast<vtkLibHandle>(lib));
     }
 }
 
@@ -416,10 +415,11 @@ void vtkObjectFactory::UnRegisterAllFactories()
   // AFTER the factory has been deleted.
   void** libs = new void*[num+1];
   vtkObjectFactory* factory;
-  vtkObjectFactory::RegisteredFactories->InitTraversal();
+  vtkCollectionSimpleIterator osit;
+  vtkObjectFactory::RegisteredFactories->InitTraversal(osit);
   int index = 0;
   while((factory =
-         vtkObjectFactory::RegisteredFactories->GetNextItem()))
+         vtkObjectFactory::RegisteredFactories->GetNextObjectFactory(osit)))
     {
     libs[index++] = factory->LibraryHandle;
     }
@@ -619,8 +619,10 @@ vtkObjectFactoryCollection* vtkObjectFactory::GetRegisteredFactories()
 int vtkObjectFactory::HasOverrideAny(const char* className)
 {
   vtkObjectFactory* factory;
-  for(vtkObjectFactory::RegisteredFactories->InitTraversal();
-      (factory = vtkObjectFactory::RegisteredFactories->GetNextItem());)
+  vtkCollectionSimpleIterator osit;
+  for(vtkObjectFactory::RegisteredFactories->InitTraversal(osit);
+      (factory = 
+       vtkObjectFactory::RegisteredFactories->GetNextObjectFactory(osit));)
     {
     if(factory->HasOverride(className))
       {
@@ -630,49 +632,18 @@ int vtkObjectFactory::HasOverrideAny(const char* className)
   return 0;
 }
 
-// Create a helper class to get around
-// only being able to iterate a list only once at the
-// same time.
-class vtkObjectFactoryCollectionIterator : public vtkCollection
-{
-public:
-  vtkObjectFactoryCollectionIterator(vtkCollection* source)
-    {
-      Source = source;
-#ifdef VTK_DEBUG_LEAKS
-      vtkDebugLeaks::ConstructClass("vtkCollection");
-#endif
-    }
-  vtkObjectFactory* GetNextItem() 
-    {
-      return vtkObjectFactory::SafeDownCast(this->GetNextItemAsObject());
-    }
-  
-  void InitTraversal()
-    { 
-      this->Current = this->Top = 
-        ((vtkObjectFactoryCollectionIterator*)this->Source)->Top;
-    }
-  vtkCollection* Source;
-private:
-  vtkObjectFactoryCollectionIterator(const vtkObjectFactoryCollectionIterator&);
-  void operator=(const vtkObjectFactoryCollectionIterator&);
-};
-
-
 // collect up information about current registered factories  
-void
-vtkObjectFactory::GetOverrideInformation(const char* name,
-                                         vtkOverrideInformationCollection* 
-                                         ret)
+void vtkObjectFactory::GetOverrideInformation(const char* name,
+                                              vtkOverrideInformationCollection* 
+                                              ret)
 {
   // create the collection to return
   vtkOverrideInformation* overInfo; // info object pointer
   vtkObjectFactory* factory; // factory pointer for traversal
-  vtkObjectFactoryCollectionIterator* it = 
-    new vtkObjectFactoryCollectionIterator(vtkObjectFactory::RegisteredFactories);
-  for(it->InitTraversal();
-      (factory = it->GetNextItem());)
+  vtkCollectionSimpleIterator osit;
+  vtkObjectFactory::RegisteredFactories->InitTraversal(osit);
+  for(;(factory = 
+      vtkObjectFactory::RegisteredFactories->GetNextObjectFactory(osit));)
     {
     for(int i =0; i < factory->OverrideArrayLength; i++)
       {
@@ -695,7 +666,6 @@ vtkObjectFactory::GetOverrideInformation(const char* name,
         }
       }
     }
-  it->Delete();
 }
 
 // set enable flag for all registered factories for the given className
@@ -703,8 +673,10 @@ void vtkObjectFactory::SetAllEnableFlags(int flag,
                                          const char* className)
 {
   vtkObjectFactory* factory;
-  for(vtkObjectFactory::RegisteredFactories->InitTraversal();
-      (factory = vtkObjectFactory::RegisteredFactories->GetNextItem());)
+  vtkCollectionSimpleIterator osit;
+  for(vtkObjectFactory::RegisteredFactories->InitTraversal(osit);
+      (factory = 
+       vtkObjectFactory::RegisteredFactories->GetNextObjectFactory(osit));)
     {
     factory->SetEnableFlag(flag, className, 0);
     }
@@ -718,8 +690,10 @@ void vtkObjectFactory::SetAllEnableFlags(int flag,
                                          const char* subclassName)
 {
   vtkObjectFactory* factory;
-  for(vtkObjectFactory::RegisteredFactories->InitTraversal();
-      (factory = vtkObjectFactory::RegisteredFactories->GetNextItem());)
+  vtkCollectionSimpleIterator osit;
+  for(vtkObjectFactory::RegisteredFactories->InitTraversal(osit);
+      (factory = 
+       vtkObjectFactory::RegisteredFactories->GetNextObjectFactory(osit));)
     {
     factory->SetEnableFlag(flag, className, subclassName);
     }
@@ -735,7 +709,9 @@ void vtkObjectFactory::CreateAllInstance(const char* vtkclassname,
   vtkObjectFactory* f;
   vtkObjectFactoryCollection* collection
     = vtkObjectFactory::GetRegisteredFactories();
-  for(collection->InitTraversal(); (f = collection->GetNextItem()); )
+  vtkCollectionSimpleIterator osit;
+  for(collection->InitTraversal(osit); 
+      (f = collection->GetNextObjectFactory(osit)); )
     {
     vtkObject* o = f->CreateObject(vtkclassname);
     if(o)
