@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkObjectFactoryCollection.h,v $
+  Module:    vtkObjectFactoryCollection.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -30,12 +30,15 @@
 class VTK_COMMON_EXPORT vtkObjectFactoryCollection : public vtkCollection
 {
 public:
-  vtkTypeRevisionMacro(vtkObjectFactoryCollection,vtkCollection);
+  vtkTypeMacro(vtkObjectFactoryCollection,vtkCollection);
   static vtkObjectFactoryCollection *New();
 
   // Description:
   // Add an ObjectFactory from the list.
-  void AddItem(vtkObjectFactory *t) { this->vtkCollection::AddItem((vtkObject *)t); }
+  void AddItem(vtkObjectFactory *t)
+    {
+      this->vtkCollection::AddItem(t);
+    }
   
   // Description:
   // Get the next ObjectFactory in the list. Return NULL when the end of the
@@ -43,6 +46,15 @@ public:
   vtkObjectFactory *GetNextItem() 
     { return static_cast<vtkObjectFactory *>(this->GetNextItemAsObject());}
 
+  //BTX
+  // Description: 
+  // Reentrant safe way to get an object in a collection. Just pass the
+  // same cookie back and forth. 
+  vtkObjectFactory *GetNextObjectFactory(vtkCollectionSimpleIterator &cookie) {
+    return static_cast<vtkObjectFactory *>(
+      this->GetNextItemAsObject(cookie));};
+  //ETX
+
 protected:
   vtkObjectFactoryCollection() {};
   ~vtkObjectFactoryCollection() {};
