@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkOverrideInformationCollection.h,v $
+  Module:    vtkOverrideInformationCollection.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -29,7 +29,7 @@
 class VTK_COMMON_EXPORT vtkOverrideInformationCollection : public vtkCollection
 {
 public:
-  vtkTypeRevisionMacro(vtkOverrideInformationCollection,vtkCollection);
+  vtkTypeMacro(vtkOverrideInformationCollection,vtkCollection);
   static vtkOverrideInformationCollection *New();
 
   // Description:
@@ -40,6 +40,16 @@ public:
   // Get the next OverrideInformation in the list.
   vtkOverrideInformation *GetNextItem();
   
+  //BTX
+  // Description: 
+  // Reentrant safe way to get an object in a collection. Just pass the
+  // same cookie back and forth. 
+  vtkOverrideInformation *GetNextOverrideInformation(
+    vtkCollectionSimpleIterator &cookie) {
+    return static_cast<vtkOverrideInformation *>(
+      this->GetNextItemAsObject(cookie));};
+  //ETX
+
 protected:
   vtkOverrideInformationCollection() {};
   ~vtkOverrideInformationCollection() {};
@@ -56,7 +66,7 @@ private:
 
 inline void vtkOverrideInformationCollection::AddItem(vtkOverrideInformation *f) 
 {
-  this->vtkCollection::AddItem((vtkObject *)f);
+  this->vtkCollection::AddItem(f);
 }
 
 inline vtkOverrideInformation *vtkOverrideInformationCollection::GetNextItem() 
