@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkQuadric.cxx,v $
+  Module:    vtkQuadric.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,7 +17,6 @@
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkQuadric, "$Revision: 1.6 $");
 vtkStandardNewMacro(vtkQuadric);
 
 // Construct quadric with all coefficients = 1.
