@@ -1,24 +1,20 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkReferenceCount.cxx,v $
-  Language:  C++
-  Date:      $Date: 2002/01/22 15:26:16 $
-  Version:   $Revision: 1.15 $
+  Module:    vtkReferenceCount.cxx
 
-  Copyright (c) 1993-2002 Ken Martin, Will Schroeder, Bill Lorensen 
+  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
   See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
 
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
+     This software is distributed WITHOUT ANY WARRANTY; without even
+     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
 #include "vtkReferenceCount.h"
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkReferenceCount, "$Revision: 1.15 $");
 vtkStandardNewMacro(vtkReferenceCount);
 
 vtkReferenceCount::vtkReferenceCount() 
