@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkTclUtil.cxx,v $
+  Module:    vtkTclUtil.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -18,9 +18,12 @@
 #include "vtkSetGet.h"
 #include "vtkCallbackCommand.h"
 
+#include <string>
+#include <vtksys/SystemTools.hxx>
+
 extern "C"
 {
-#if (TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION >= 4 && TCL_RELEASE_LEVEL >= TCL_FINAL_RELEASE)
+#if (TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION >= 4)
   typedef int (*vtkTclCommandType)(ClientData, Tcl_Interp *,int, CONST84 char *[]);
 #else
   typedef int (*vtkTclCommandType)(ClientData, Tcl_Interp *,int, char *[]);
@@ -29,7 +32,7 @@ extern "C"
 
 vtkTclInterpStruct *vtkGetInterpStruct(Tcl_Interp *interp)
 {
-  vtkTclInterpStruct *is = (vtkTclInterpStruct *)Tcl_GetAssocData(interp,(char *) "vtk",NULL);
+  vtkTclInterpStruct *is = static_cast<vtkTclInterpStruct *>(Tcl_GetAssocData(interp,(char *)("vtk"),NULL));
   if (!is)
     {
     vtkGenericWarningMacro("unable to find interp struct");
@@ -53,7 +56,7 @@ VTKTCL_EXPORT void vtkTclDeleteObjectFromHash(vtkObject *obj,
                                               unsigned long vtkNotUsed(eventId), 
                                               void *cd, void *)
 {
-  vtkTclCommandArgStruct *as = (vtkTclCommandArgStruct *)cd;
+  vtkTclCommandArgStruct *as = static_cast<vtkTclCommandArgStruct *>(cd);
   char temps[80];
   Tcl_HashEntry *entry;
   char *temp;
@@ -62,9 +65,14 @@ VTKTCL_EXPORT void vtkTclDeleteObjectFromHash(vtkObject *obj,
   // lookup the objects name
   sprintf(temps,"%p",obj);
   entry = Tcl_FindHashEntry(&is->PointerLookup,temps); 
-  temp = (char *)(Tcl_GetHashValue(entry));
-
-  Tcl_DeleteCommand(as->Interp,temp);
+  if (entry)
+    {
+    temp = static_cast<char *>(Tcl_GetHashValue(entry));
+    if (temp)
+      {
+      Tcl_DeleteCommand(as->Interp,temp);
+      }
+    }
 }
 
 // we do no error checking in this.  We assume that if we were called
@@ -78,12 +86,12 @@ VTKTCL_EXPORT void vtkTclGenericDeleteObject(ClientData cd)
   char *temp;
   vtkObject *tobject;
   int error;
-  vtkTclCommandArgStruct *as = (vtkTclCommandArgStruct *)cd;
+  vtkTclCommandArgStruct *as = static_cast<vtkTclCommandArgStruct *>(cd);
   Tcl_Interp *interp = as->Interp;
   vtkTclInterpStruct *is = vtkGetInterpStruct(interp);
 
   /* set up the args */
-  args[1] = (char *) "Delete";
+  args[1] = (char *)("Delete");
 
   // lookup the objects name
   sprintf(temps,"%p",as->Pointer);
@@ -93,21 +101,22 @@ VTKTCL_EXPORT void vtkTclGenericDeleteObject(ClientData cd)
     return;
     }
 
-  temp = (char *)(Tcl_GetHashValue(entry));
+  temp = static_cast<char *>(Tcl_GetHashValue(entry));
   args[0] = temp;
   
   // first we clear the delete callback since we will
   // always remove this object from the hash regardless
   // of if it has really been freed.
-  tobject = (vtkObject *)vtkTclGetPointerFromObject(temp,"vtkObject",
-                                                    interp, error);
+  tobject = static_cast<vtkObject *>(
+    vtkTclGetPointerFromObject(temp,"vtkObject",
+                               interp, error));
   tobject->RemoveObserver(as->Tag);
   as->Tag = 0;
   
   // get the command function and invoke the delete operation
   entry = Tcl_FindHashEntry(&is->CommandLookup,temp);
-  command = (int (*)(ClientData,Tcl_Interp *,int,char *[]))
-    Tcl_GetHashValue(entry);
+  command = (int (*)(ClientData,Tcl_Interp *,int,char *[]))(
+    Tcl_GetHashValue(entry));
   
   // do we need to delete the c++ obj
   if (strncmp(temp,"vtkTemp",7))
@@ -154,7 +163,7 @@ int vtkCreateCommand(ClientData vtkNotUsed(cd), Tcl_Interp *interp, int argc, ch
          entry != NULL;
          entry = Tcl_FirstHashEntry(&is->PointerLookup,&search))
       {
-      tmp = strdup((char *)Tcl_GetHashValue(entry));
+      tmp = strdup(static_cast<char *>(Tcl_GetHashValue(entry)));
       if (tmp)
         {
         Tcl_DeleteCommand(interp,tmp);
@@ -172,7 +181,7 @@ int vtkCreateCommand(ClientData vtkNotUsed(cd), Tcl_Interp *interp, int argc, ch
          entry != NULL; entry = Tcl_NextHashEntry(&search))
       {
       Tcl_AppendResult(interp,
-                       (char *)Tcl_GetHashKey(&is->InstanceLookup,entry),NULL);
+                       static_cast<char *>(Tcl_GetHashKey(&is->InstanceLookup,entry)),NULL);
       Tcl_AppendResult(interp,"\n",NULL);
       }
     return TCL_OK;
@@ -187,6 +196,16 @@ int vtkCreateCommand(ClientData vtkNotUsed(cd), Tcl_Interp *interp, int argc, ch
     is->DebugOn = 0;
     return TCL_OK;
     }
+  if (!strcmp(argv[1],"DeleteExistingObjectOnNewOn"))
+    {
+    is->DeleteExistingObjectOnNew = 1;
+    return TCL_OK;
+    }
+  if (!strcmp(argv[1],"DeleteExistingObjectOnNewOff"))
+    {
+    is->DeleteExistingObjectOnNew = 0;
+    return TCL_OK;
+    }
   if (!strcmp("ListMethods",argv[1]))
     {
     Tcl_AppendResult(interp,"Methods for vtkCommand:\n",NULL);
@@ -194,6 +213,8 @@ int vtkCreateCommand(ClientData vtkNotUsed(cd), Tcl_Interp *interp, int argc, ch
     Tcl_AppendResult(interp,"  DebugOff\n",NULL);
     Tcl_AppendResult(interp,"  DeleteAllObjects\n",NULL);
     Tcl_AppendResult(interp,"  ListAllInstances\n",NULL);
+    Tcl_AppendResult(interp,"  DeleteExistingObjectOnNewOn\n",NULL);
+    Tcl_AppendResult(interp,"  DeleteExistingObjectOnNewOff\n",NULL);
     return TCL_OK;
     }
 
@@ -213,8 +234,8 @@ vtkTclUpdateCommand(Tcl_Interp *interp, char *name,  vtkObject *temp)
     {
     if (cinf.clientData)
       {
-      vtkTclCommandStruct *cs = (vtkTclCommandStruct *)cinf.clientData;
-      command = (Tcl_CmdProc *)cs->CommandFunction;
+      vtkTclCommandStruct *cs = static_cast<vtkTclCommandStruct *>(cinf.clientData);
+      command = reinterpret_cast<Tcl_CmdProc *>(cs->CommandFunction);
       }
     }
   if (tstr)
@@ -236,16 +257,17 @@ vtkTclUpdateCommand(Tcl_Interp *interp, char *name,  vtkObject *temp)
 
   vtkTclInterpStruct *is = vtkGetInterpStruct(interp);
   Tcl_HashEntry *entry = Tcl_FindHashEntry(&is->CommandLookup,name);
-  Tcl_SetHashValue(entry,(ClientData)command);
+  Tcl_SetHashValue(entry,(ClientData)(command));
 }
 
 
 VTKTCL_EXPORT void
 vtkTclGetObjectFromPointer(Tcl_Interp *interp, void *temp1,
-                           int (*command)(ClientData, Tcl_Interp *,int, char *[]))
+                           const char *targetType)
 {
+  int (*command)(ClientData, Tcl_Interp *,int, char *[]) = 0;
   int is_new;
-  vtkObject *temp = (vtkObject *)temp1;
+  vtkObject *temp = static_cast<vtkObject *>(temp1);
   char temps[80];
   char name[80];
   Tcl_HashEntry *entry;
@@ -271,12 +293,12 @@ vtkTclGetObjectFromPointer(Tcl_Interp *interp, void *temp1,
     if (is->DebugOn)
       {
         vtkGenericWarningMacro("Found name: " 
-                               << (char *)(Tcl_GetHashValue(entry)) 
+                               << static_cast<char *>(Tcl_GetHashValue(entry)) 
                                << " for vtk pointer: " << temp);
       }
     
     /* while we are at it store the name since it is required anyhow */
-    Tcl_SetResult(interp, (char *)(Tcl_GetHashValue(entry)), TCL_VOLATILE);
+    Tcl_SetResult(interp, static_cast<char *>(Tcl_GetHashValue(entry)), TCL_VOLATILE);
     return;
     }
 
@@ -297,37 +319,71 @@ vtkTclGetObjectFromPointer(Tcl_Interp *interp, void *temp1,
     {
     if (cinf.clientData)
       {
-      vtkTclCommandStruct *cs = (vtkTclCommandStruct *)cinf.clientData;
+      vtkTclCommandStruct *cs = static_cast<vtkTclCommandStruct *>(cinf.clientData);
       command = cs->CommandFunction;
       }
     }
+  // if the class command wasn;t found try the target return type command
+  if (!command && targetType)
+    {
+    if (tstr)
+      {
+      free(tstr);
+      }
+    tstr = strdup(targetType);
+    if (Tcl_GetCommandInfo(interp,tstr,&cinf))
+      {
+      if (cinf.clientData)
+        {
+        vtkTclCommandStruct *cs = static_cast<vtkTclCommandStruct *>(cinf.clientData);
+        command = cs->CommandFunction;
+        }
+      }
+    }
+  // if we still do not havbe a match then try vtkObject
+  if (!command)
+    {
+    if (tstr)
+      {
+      free(tstr);
+      }
+    tstr = strdup("vtkObject");
+    if (Tcl_GetCommandInfo(interp,tstr,&cinf))
+      {
+      if (cinf.clientData)
+        {
+        vtkTclCommandStruct *cs = static_cast<vtkTclCommandStruct *>(cinf.clientData);
+        command = cs->CommandFunction;
+        }
+      }
+    }
   if (tstr)
     {
     free(tstr);
     }
 
   entry = Tcl_CreateHashEntry(&is->InstanceLookup,name,&is_new);
-  Tcl_SetHashValue(entry,(ClientData)(temp));
+  Tcl_SetHashValue(entry,static_cast<ClientData>(temp));
   entry = Tcl_CreateHashEntry(&is->PointerLookup,temps,&is_new);
-  Tcl_SetHashValue(entry,(ClientData)(strdup(name)));
+  Tcl_SetHashValue(entry,static_cast<ClientData>(strdup(name)));
   vtkTclCommandArgStruct *as = new vtkTclCommandArgStruct;
-  as->Pointer = (void *)temp;
+  as->Pointer = static_cast<void *>(temp);
   as->Interp = interp;
   Tcl_CreateCommand(interp,name,
                     reinterpret_cast<vtkTclCommandType>(command),
-                    (ClientData)(as),
-                    (Tcl_CmdDeleteProc *)vtkTclGenericDeleteObject);
+                    static_cast<ClientData>(as),
+                    reinterpret_cast<Tcl_CmdDeleteProc *>(vtkTclGenericDeleteObject));
   entry = Tcl_CreateHashEntry(&is->CommandLookup,name,&is_new);
-  Tcl_SetHashValue(entry,(ClientData)command);
+  Tcl_SetHashValue(entry,(ClientData)(command));
   
   // setup the delete callback
   vtkCallbackCommand *cbc = vtkCallbackCommand::New();
   cbc->SetCallback(vtkTclDeleteObjectFromHash);
-  cbc->SetClientData((void *)as);
+  cbc->SetClientData(static_cast<void *>(as));
   as->Tag = temp->AddObserver(vtkCommand::DeleteEvent, cbc);
   cbc->Delete();
   
-  Tcl_SetResult(interp, (char *)name, TCL_VOLATILE);
+  Tcl_SetResult(interp, static_cast<char *>(name), TCL_VOLATILE);
 }
       
 VTKTCL_EXPORT void *vtkTclGetPointerFromObject(const char *name,
@@ -357,7 +413,7 @@ VTKTCL_EXPORT void *vtkTclGetPointerFromObject(const char *name,
 
   if ((entry = Tcl_FindHashEntry(&is->InstanceLookup,name)))
     {
-    temp = (ClientData)Tcl_GetHashValue(entry);
+    temp = static_cast<ClientData>(Tcl_GetHashValue(entry));
     }
   else
     {
@@ -370,7 +426,8 @@ VTKTCL_EXPORT void *vtkTclGetPointerFromObject(const char *name,
   /* now handle the typecasting, get the command proc */
   if ((entry = Tcl_FindHashEntry(&is->CommandLookup,name)))
     {
-    command = (int (*)(ClientData,Tcl_Interp *,int,char *[]))Tcl_GetHashValue(entry);
+    command = (int (*)(ClientData,Tcl_Interp *,int,char *[]))(
+      Tcl_GetHashValue(entry));
     }
   else
     {
@@ -381,29 +438,29 @@ VTKTCL_EXPORT void *vtkTclGetPointerFromObject(const char *name,
     }
 
   /* set up the args */
-  args[0] = (char *) "DoTypecasting";
+  args[0] = (char *)("DoTypecasting");
   args[1] = strdup(result_type);
   args[2] = NULL;
   vtkTclCommandArgStruct foo;
   foo.Pointer = temp;
   foo.Interp = interp;
-  if (command((ClientData)&foo,(Tcl_Interp *)NULL,3,args) == TCL_OK)
+  if (command(static_cast<ClientData>(&foo),static_cast<Tcl_Interp *>(NULL),3,args) == TCL_OK)
     {
     free (args[1]);
-    return (void *)(args[2]);
+    return static_cast<void *>(args[2]);
     }
   else
     {
     Tcl_Interp *i;
     i = Tcl_CreateInterp();
     // provide more diagnostic info
-    args[0] = (char *) "Dummy";
+    args[0] = (char *)("Dummy");
     free (args[1]);
-    args[1] = (char *) "GetClassName";
+    args[1] = (char *)("GetClassName");
     args[2] = NULL;
-    command((ClientData)&foo,i,2,args);
+    command(static_cast<ClientData>(&foo),i,2,args);
 
-    sprintf(temps,"vtk bad argument, type conversion failed for object %s.\nCould not type convert %s which is of type %s, to type %s.\n", name, name, i->result, result_type);
+    sprintf(temps,"vtk bad argument, type conversion failed for object %s.\nCould not type convert %s which is of type %s, to type %s.\n", name, name, Tcl_GetStringResult(i), result_type);
     Tcl_AppendResult(interp,temps,NULL);
     error = 1;
     Tcl_DeleteInterp(i);
@@ -418,7 +475,7 @@ VTKTCL_EXPORT void vtkTclVoidFunc(void *arg)
 
   vtkTclVoidFuncArg *arg2;
 
-  arg2 = (vtkTclVoidFuncArg *)arg;
+  arg2 = static_cast<vtkTclVoidFuncArg *>(arg);
 
 #if TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION <= 2
   res = Tcl_GlobalEval(arg2->interp, arg2->command);
@@ -428,11 +485,11 @@ VTKTCL_EXPORT void vtkTclVoidFunc(void *arg)
 
   if (res == TCL_ERROR)
     {
-    if (Tcl_GetVar(arg2->interp,(char *) "errorInfo",0))
+    if (Tcl_GetVar(arg2->interp,(char *)("errorInfo"),0))
       {
       vtkGenericWarningMacro("Error returned from vtk/tcl callback:\n" <<
                              arg2->command << endl <<
-                             Tcl_GetVar(arg2->interp,(char *) "errorInfo",0) <<
+                             Tcl_GetVar(arg2->interp,(char *)("errorInfo"),0) <<
                              " at line number " << arg2->interp->errorLine);
       }
     else
@@ -448,7 +505,7 @@ VTKTCL_EXPORT void vtkTclVoidFuncArgDelete(void *arg)
 {
   vtkTclVoidFuncArg *arg2;
 
-  arg2 = (vtkTclVoidFuncArg *)arg;
+  arg2 = static_cast<vtkTclVoidFuncArg *>(arg);
   
   // free the string and then structure
   delete [] arg2->command;
@@ -496,13 +553,14 @@ int vtkTclNewInstanceCommand(ClientData cd, Tcl_Interp *interp,
   Tcl_HashEntry *entry;
   int is_new;
   char temps[80];
-  vtkTclCommandStruct *cs = (vtkTclCommandStruct *)cd;
+  char name[80];
+  vtkTclCommandStruct *cs = static_cast<vtkTclCommandStruct *>(cd);
   Tcl_CmdInfo cinf;
   vtkTclInterpStruct *is = vtkGetInterpStruct(interp);
 
   if (argc != 2)
     {
-    Tcl_SetResult(interp, (char *) "vtk object creation requires one argument, a name.", TCL_VOLATILE);
+    Tcl_SetResult(interp, (char *)("vtk object creation requires one argument, a name, or the special New keyword to instantiate a new name."), TCL_VOLATILE);
     return TCL_ERROR;
     }
 
@@ -512,14 +570,21 @@ int vtkTclNewInstanceCommand(ClientData cd, Tcl_Interp *interp,
     Tcl_AppendResult(interp, ": vtk object cannot start with a numeric.", NULL);
     return TCL_ERROR;
     }
-
+  
   if (Tcl_FindHashEntry(&is->InstanceLookup,argv[1]))
     { 
-    Tcl_SetResult(interp, argv[1], TCL_VOLATILE);
-    Tcl_AppendResult(interp,
-                     ": a vtk object with that name already exists.",
-                     NULL);
-    return TCL_ERROR;
+    if (is->DeleteExistingObjectOnNew)
+      {
+      Tcl_DeleteCommand(interp, argv[1]);
+      }
+    else
+      {
+      Tcl_SetResult(interp, argv[1], TCL_VOLATILE);
+      Tcl_AppendResult(interp,
+                       ": a vtk object with that name already exists.",
+                       NULL);
+      return TCL_ERROR;
+      }
     }
 
   // Make sure we are not clobbering a built in command
@@ -535,24 +600,33 @@ int vtkTclNewInstanceCommand(ClientData cd, Tcl_Interp *interp,
   ClientData temp;
   if (!strcmp("ListInstances",argv[1]))
     {
-    vtkTclListInstances(interp,(ClientData) cs->CommandFunction);
+    vtkTclListInstances(interp,(ClientData)(cs->CommandFunction));
     return TCL_OK;
     }
+
+  if (!strcmp("New",argv[1]))
+    {
+    sprintf(name,"vtkObj%i",is->Number);
+    is->Number++;
+    argv[1] = name;
+    }
+
   temp = cs->NewCommand();
 
   entry = Tcl_CreateHashEntry(&is->InstanceLookup,argv[1],&is_new);
   Tcl_SetHashValue(entry,temp);
-  sprintf(temps,"%p",(void *)temp);
+  sprintf(temps,"%p",static_cast<void *>(temp));
   entry = Tcl_CreateHashEntry(&is->PointerLookup,temps,&is_new);
-  Tcl_SetHashValue(entry,(ClientData)(strdup(argv[1])));
+  Tcl_SetHashValue(entry,static_cast<ClientData>(strdup(argv[1])));
   
   // check to see if we can find the command function based on class name
-  char *tstr = strdup(((vtkObject *)temp)->GetClassName());
+  char *tstr = strdup(static_cast<vtkObject *>(temp)->GetClassName());
   if (Tcl_GetCommandInfo(interp,tstr,&cinf))
     {
     if (cinf.clientData)
       {
-      vtkTclCommandStruct *cs2 = (vtkTclCommandStruct *)cinf.clientData;
+      vtkTclCommandStruct *cs2 =
+        static_cast<vtkTclCommandStruct *>(cinf.clientData);
       command = cs2->CommandFunction;
       }
     else
@@ -570,20 +644,21 @@ int vtkTclNewInstanceCommand(ClientData cd, Tcl_Interp *interp,
     }
 
   vtkTclCommandArgStruct *as = new vtkTclCommandArgStruct;
-  as->Pointer = (void *)temp;
+  as->Pointer = static_cast<void *>(temp);
   as->Interp = interp;
   Tcl_CreateCommand(interp,argv[1],
                     reinterpret_cast<vtkTclCommandType>(command),
-                    (ClientData)as,
-                    (Tcl_CmdDeleteProc *)vtkTclGenericDeleteObject);
+                    static_cast<ClientData>(as),
+                    reinterpret_cast<Tcl_CmdDeleteProc *>(vtkTclGenericDeleteObject));
   entry = Tcl_CreateHashEntry(&is->CommandLookup,argv[1],&is_new);
   Tcl_SetHashValue(entry,(ClientData)(cs->CommandFunction));
   
   // setup the delete callback
   vtkCallbackCommand *cbc = vtkCallbackCommand::New();
   cbc->SetCallback(vtkTclDeleteObjectFromHash);
-  cbc->SetClientData((void *)as);
-  as->Tag = ((vtkObject *)temp)->AddObserver(vtkCommand::DeleteEvent, cbc);
+  cbc->SetClientData(static_cast<void *>(as));
+  as->Tag =
+    static_cast<vtkObject *>(temp)->AddObserver(vtkCommand::DeleteEvent, cbc);
   cbc->Delete();
 
   Tcl_SetResult(interp, argv[1], TCL_VOLATILE);
@@ -592,7 +667,7 @@ int vtkTclNewInstanceCommand(ClientData cd, Tcl_Interp *interp,
 
 void vtkTclDeleteCommandStruct(ClientData cd)
 {
-  vtkTclCommandStruct *cs = (vtkTclCommandStruct *)cd;
+  vtkTclCommandStruct *cs = static_cast<vtkTclCommandStruct *>(cd);
   delete cs;
 }
 
@@ -605,11 +680,12 @@ void vtkTclCreateNew(Tcl_Interp *interp, const char *cname,
   vtkTclCommandStruct *cs = new vtkTclCommandStruct;
   cs->NewCommand = NewCommand;
   cs->CommandFunction = CommandFunction;
-  Tcl_CreateCommand(interp,(char *) cname,
-                    reinterpret_cast<vtkTclCommandType>(
-                      vtkTclNewInstanceCommand),
-                   (ClientData *)cs,
-                   (Tcl_CmdDeleteProc *)vtkTclDeleteCommandStruct);
+  Tcl_CreateCommand(
+    interp,const_cast<char *>(cname),
+    reinterpret_cast<vtkTclCommandType>(
+      vtkTclNewInstanceCommand),
+    reinterpret_cast<ClientData *>(cs),
+    reinterpret_cast<Tcl_CmdDeleteProc *>(vtkTclDeleteCommandStruct));
 }
 
 
@@ -642,11 +718,11 @@ void vtkTclCommand::Execute(vtkObject *, unsigned long, void *)
 
   if (res == TCL_ERROR)
     {
-    if (Tcl_GetVar(this->Interp,(char *) "errorInfo",0))
+    if (Tcl_GetVar(this->Interp,(char *)("errorInfo"),0))
       {
       vtkGenericWarningMacro("Error returned from vtk/tcl callback:\n" <<
                              this->StringCommand << endl <<
-                             Tcl_GetVar(this->Interp,(char *) "errorInfo",0) <<
+                             Tcl_GetVar(this->Interp,(char *)("errorInfo"),0) <<
                              " at line number " << this->Interp->errorLine);
       }
     else
@@ -657,6 +733,111 @@ void vtkTclCommand::Execute(vtkObject *, unsigned long, void *)
                              this->Interp->errorLine);
       }
     }
+  else if (res == -1)
+    {
+    this->AbortFlagOn();
+    } 
 }
 
+void vtkTclApplicationInitExecutable(int vtkNotUsed(argc),
+                                     const char* const argv[])
+{
+  std::string av0 = argv[0];
 
+  if (vtksys::SystemTools::FileIsFullPath(argv[0]))
+    {
+    av0 = vtksys::SystemTools::CollapseFullPath(argv[0]);
+    }
+  Tcl_FindExecutable(av0.c_str());
+}
+
+// We need two internal Tcl functions.  They usually are declared in
+// tclIntDecls.h, but UNIX builds do not have access to VTK's
+// tkInternals include path.  Since the signature has not changed for
+// years (at least since 8.2), let's just prototype them.
+EXTERN Tcl_Obj* TclGetLibraryPath _ANSI_ARGS_((void));
+EXTERN void TclSetLibraryPath _ANSI_ARGS_((Tcl_Obj * pathPtr));
+
+void vtkTclApplicationInitTclTk(Tcl_Interp* interp,
+                                const char* const relative_dirs[])
+{
+  /*
+    Tcl/Tk requires support files to work (set of tcl files).
+    When an app is linked against Tcl/Tk shared libraries, the path to
+    the libraries is used by Tcl/Tk to search for its support files.
+    For example, on Windows, if bin/tcl84.dll is the shared lib, support
+    files will be searched in bin/../lib/tcl8.4, which is where they are
+    usually installed.
+    If an app is linked against Tcl/Tk *static* libraries, there is no
+    way for Tcl/Tk to find its support files. In that case, it will
+    use the TCL_LIBRARY and TK_LIBRARY environment variable (those should
+    point to the support files dir, ex: c:/tcl/lib/tcl8.4, c:/tk/lib/tcl8.4).
+
+    The above code will also make Tcl/Tk search inside VTK's build/install
+    directory, more precisely inside a TclTk/lib sub dir.
+    ex: [path to vtk.exe]/TclTk/lib/tcl8.4, [path to vtk.exe]/TclTk/lib/tk8.4
+    Support files are copied to that location when
+    VTK_TCL_TK_COPY_SUPPORT_LIBRARY is ON.
+  */
+
+  int has_tcllibpath_env = getenv("TCL_LIBRARY") ? 1 : 0;
+  int has_tklibpath_env = getenv("TK_LIBRARY") ? 1 : 0;
+  std::string selfdir;
+  if(!has_tcllibpath_env || !has_tklibpath_env)
+    {
+    const char* nameofexec = Tcl_GetNameOfExecutable();
+    if(nameofexec && vtksys::SystemTools::FileExists(nameofexec))
+      {
+      std::string name = nameofexec;
+      vtksys::SystemTools::ConvertToUnixSlashes(name);
+      selfdir = vtksys::SystemTools::GetFilenamePath(name);
+      }
+    }
+  if(selfdir.length() > 0)
+    {
+    if(!has_tcllibpath_env)
+      {
+      std::string tdir;
+      for(const char* const* p = relative_dirs; *p; ++p)
+        {
+        tdir = selfdir;
+        tdir += "/";
+        tdir += *p;
+        tdir += "/tcl" TCL_VERSION;
+        tdir = vtksys::SystemTools::CollapseFullPath(tdir.c_str());
+        if(vtksys::SystemTools::FileExists(tdir.c_str()) &&
+           vtksys::SystemTools::FileIsDirectory(tdir.c_str()))
+          {
+          // Set the tcl_library Tcl variable.
+          char tcl_library[1024];
+          strcpy(tcl_library, tdir.c_str());
+          Tcl_SetVar(interp, "tcl_library", tcl_library,
+                     TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG);
+          break;
+          }
+        }
+      }
+    if(!has_tklibpath_env)
+      {
+      std::string tdir;
+      for(const char* const* p = relative_dirs; *p; ++p)
+        {
+        tdir = selfdir;
+        tdir += "/";
+        tdir += *p;
+        tdir += "/tk" TCL_VERSION;
+        tdir = vtksys::SystemTools::CollapseFullPath(tdir.c_str());
+        if(vtksys::SystemTools::FileExists(tdir.c_str()) &&
+           vtksys::SystemTools::FileIsDirectory(tdir.c_str()))
+          {
+          // Set the tk_library Tcl variable.
+          char tk_library[1024];
+          strcpy(tk_library, tdir.c_str());
+          Tcl_SetVar(interp, "tk_library", tk_library,
+                     TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG);
+          break;
+          }
+        }
+      }
+    }
+}
