@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkTimerLog.h,v $
+  Module:    vtkTimerLog.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -71,16 +71,12 @@ typedef struct
 } vtkTimerLogEntry;
 //ETX
 
-// The microsoft compiler defines this as a macro, so
-// undefine it here
-#undef GetCurrentTime
-
 class VTK_COMMON_EXPORT vtkTimerLog : public vtkObject 
 {
 public:
   static vtkTimerLog *New();
 
-  vtkTypeRevisionMacro(vtkTimerLog,vtkObject);
+  vtkTypeMacro(vtkTimerLog,vtkObject);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -145,7 +141,7 @@ public:
   // Description:
   // Returns the elapsed number of seconds since January 1, 1970. This
   // is also called Universal Coordinated Time.
-  static double GetCurrentTime();
+  static double GetUniversalTime();
 
   // Description:
   // Returns the CPU time for this process
@@ -165,9 +161,24 @@ public:
   // a doubleing point value indicating the elapsed time in seconds.
   double GetElapsedTime();
 
+#ifdef VTK_WORKAROUND_WINDOWS_MANGLE
+# define GetTickCount GetCurrentTime
+#endif
+
+  // Description:
+  // @deprecated Replaced by vtkTimerLog::GetUniversalTime() as of VTK 5.0.
+  VTK_LEGACY(static double GetCurrentTime());
+
+#ifdef VTK_WORKAROUND_WINDOWS_MANGLE
+# undef GetTickCount
+  //BTX
+  VTK_LEGACY(static double GetTickCount());
+  //ETX
+#endif
+
 protected:
   vtkTimerLog() {this->StartTime=0; this->EndTime = 0;}; //insure constructor/destructor protected
-  ~vtkTimerLog() {};
+  virtual ~vtkTimerLog() { };
 
   static vtkTimerLogEntry* GetEvent(int i);
 
