@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkTransform.h,v $
+  Module:    vtkTransform.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -51,7 +51,7 @@ class VTK_COMMON_EXPORT vtkTransform : public vtkLinearTransform
 {
  public:
   static vtkTransform *New();
-  vtkTypeRevisionMacro(vtkTransform,vtkLinearTransform);
+  vtkTypeMacro(vtkTransform,vtkLinearTransform);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -160,17 +160,31 @@ class VTK_COMMON_EXPORT vtkTransform : public vtkLinearTransform
   // transformation of a coordinate occurs.  This method is provided
   // to make it possible to decompose a transformation into its
   // constituents, for example to save a transformation to a file.
-  vtkLinearTransform *GetConcatenatedTransform(int i) {
-    if (this->Input == NULL) {
-      return (vtkLinearTransform *)this->Concatenation->GetTransform(i); }
-    else if (i < this->Concatenation->GetNumberOfPreTransforms()) {
-      return (vtkLinearTransform *)this->Concatenation->GetTransform(i); }
-    else if (i > this->Concatenation->GetNumberOfPreTransforms()) {
-      return (vtkLinearTransform *)this->Concatenation->GetTransform(i-1); } 
-    else if (this->GetInverseFlag()) {
-      return (vtkLinearTransform *)this->Input->GetInverse(); }
-    else {
-      return (vtkLinearTransform *)this->Input; } };
+  vtkLinearTransform *GetConcatenatedTransform(int i)
+    {
+      vtkAbstractTransform *t;
+      if (this->Input == NULL)
+        {
+        t=this->Concatenation->GetTransform(i);
+        }
+      else if (i < this->Concatenation->GetNumberOfPreTransforms())
+        {
+        t=this->Concatenation->GetTransform(i);
+        }
+      else if (i > this->Concatenation->GetNumberOfPreTransforms())
+        {
+        t=this->Concatenation->GetTransform(i-1);
+        }
+      else if (this->GetInverseFlag())
+        {
+        t=this->Input->GetInverse();
+        }
+      else
+        {
+        t=this->Input;
+        }
+      return static_cast<vtkLinearTransform *>(t);
+    }
 
   // Description:
   // Get the x, y, z orientation angles from the transformation matrix as an
@@ -184,10 +198,16 @@ class VTK_COMMON_EXPORT vtkTransform : public vtkLinearTransform
   double *GetOrientation() { 
     this->GetOrientation(this->ReturnValue); return this->ReturnValue; };
 
+  // Description:
+  // Convenience function to get the x, y, z orientation angles from 
+  // a transformation matrix as an array of three floating point values.  
+  static void GetOrientation(double orient[3], vtkMatrix4x4 *matrix);
+
   // Description:
   // Return the wxyz angle+axis representing the current orientation.
+  // The angle is in degrees and the axis is a unit vector.
   void GetOrientationWXYZ(double wxyz[4]);
-  void GetOrientationWXYZ(float wxyz[3]) {
+  void GetOrientationWXYZ(float wxyz[4]) {
     double temp[4]; this->GetOrientationWXYZ(temp); 
     wxyz[0]=static_cast<float>(temp[0]); 
     wxyz[1]=static_cast<float>(temp[1]); 
