@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkUnsignedCharArray.h,v $
+  Module:    vtkUnsignedCharArray.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -36,7 +36,7 @@ class VTK_COMMON_EXPORT vtkUnsignedCharArray : public vtkDataArray
 {
 public:
   static vtkUnsignedCharArray* New();
-  vtkTypeRevisionMacro(vtkUnsignedCharArray,vtkDataArray);
+  vtkTypeMacro(vtkUnsignedCharArray,vtkDataArray);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -44,6 +44,27 @@ public:
   int GetDataType()
     { return VTK_UNSIGNED_CHAR; }
 
+  // Description:
+  // Copy the tuple value into a user-provided array.
+  void GetTupleValue(vtkIdType i, unsigned char* tuple)
+    { this->RealSuperclass::GetTupleValue(i, tuple); }
+
+  // Description:
+  // Set the tuple value at the ith location in the array.
+  void SetTupleValue(vtkIdType i, const unsigned char* tuple)
+    { this->RealSuperclass::SetTupleValue(i, tuple); }
+
+  // Description:
+  // Insert (memory allocation performed) the tuple into the ith location
+  // in the array.
+  void InsertTupleValue(vtkIdType i, const unsigned char* tuple)
+    { this->RealSuperclass::InsertTupleValue(i, tuple); }
+
+  // Description:
+  // Insert (memory allocation performed) the tuple onto the end of the array.
+  vtkIdType InsertNextTupleValue(const unsigned char* tuple)
+    { return this->RealSuperclass::InsertNextTupleValue(tuple); }
+
   // Description:
   // Get the data at a particular index.
   unsigned char GetValue(vtkIdType id)
@@ -72,6 +93,34 @@ public:
   vtkIdType InsertNextValue(unsigned char f)
     { return this->RealSuperclass::InsertNextValue(f); }
 
+  // Description:
+  // Get the range of array values for the given component in the
+  // native data type.
+  unsigned char *GetValueRange(int comp)
+    { return this->RealSuperclass::GetValueRange(comp); }
+//BTX
+  void GetValueRange(unsigned char range[2], int comp)
+    { this->RealSuperclass::GetValueRange(range, comp); }
+//ETX
+
+  // Description:
+  // Get the range of array values for the 0th component in the
+  // native data type.
+  unsigned char *GetValueRange()
+    { return this->RealSuperclass::GetValueRange(0); }
+//BTX
+  void GetValueRange(unsigned char range[2])
+    { this->RealSuperclass::GetValueRange(range, 0); }
+//ETX
+
+  // Description:
+  // Get the minimum data value in its native type.
+  static unsigned char GetDataTypeValueMin() { return VTK_UNSIGNED_CHAR_MIN; }
+
+  // Description:
+  // Get the maximum data value in its native type.
+  static unsigned char GetDataTypeValueMax() { return VTK_UNSIGNED_CHAR_MAX; }
+
   // Description:
   // Get the address of a particular data index. Make sure data is allocated
   // for the number of items requested. Set MaxId according to the number of
@@ -94,6 +143,8 @@ public:
   // from the suppled array.
   void SetArray(unsigned char* array, vtkIdType size, int save)
     { this->RealSuperclass::SetArray(array, size, save); }
+  void SetArray(unsigned char* array, vtkIdType size, int save, int deleteMethod)
+    { this->RealSuperclass::SetArray(array, size, save, deleteMethod); }
 
 protected:
   vtkUnsignedCharArray(vtkIdType numComp=1);
