@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkVersion.h,v $
+  Module:    vtkVersion.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -28,14 +28,19 @@
 
 #include "vtkObject.h"
 
-#define VTK_SOURCE_VERSION "vtk version " VTK_VERSION ", vtk source $Revision: 1.1782 $, $Date: 2003/12/17 02:45:35 $ (GMT)"
+#if VTK_MINOR_VERSION & 1
+# include <vtksys/DateStamp.h> // For date stamp
+# define VTK_SOURCE_VERSION "vtk version " VTK_VERSION ", Date: " vtksys_DATE_STAMP_STRING
+#else
+# define VTK_SOURCE_VERSION "vtk version " VTK_VERSION
+#endif
 
 
 class VTK_COMMON_EXPORT vtkVersion : public vtkObject
 {
 public:
   static vtkVersion *New();
-  vtkTypeRevisionMacro(vtkVersion,vtkObject);
+  vtkTypeMacro(vtkVersion,vtkObject);
 
   // Description: 
   // Return the version of vtk this object is a part of.
@@ -56,3 +61,4 @@ private:
 };
 
 #endif 
+
