@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLFileOutputWindow.h,v $
+  Module:    vtkXMLFileOutputWindow.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -26,9 +26,9 @@
 // 
 //   DisplayGenericWarningText - <GenericWarning>
 // 
-//   DisplayDebugText - <Debug.
+//   DisplayDebugText - <Debug>
 // 
-// The method DisplayTag outputs the text unprocessed.  To use this
+// The method DisplayTag outputs the text unprocessed. To use this
 // class, instantiate it and then call SetInstance(this).
 
 
@@ -41,7 +41,7 @@
 class VTK_COMMON_EXPORT vtkXMLFileOutputWindow : public vtkFileOutputWindow
 {
 public:
-  vtkTypeRevisionMacro(vtkXMLFileOutputWindow, vtkFileOutputWindow);
+  vtkTypeMacro(vtkXMLFileOutputWindow, vtkFileOutputWindow);
 
   static vtkXMLFileOutputWindow* New();
 
@@ -59,13 +59,13 @@ public:
   // Put the text into the log file without processing it.
   virtual void DisplayTag(const char*);
 
-
- protected:
+protected:
   vtkXMLFileOutputWindow() {}; 
   virtual ~vtkXMLFileOutputWindow() {}; 
 
   void Initialize();
   virtual void DisplayXML(const char*, const char*);
+
 private:
   vtkXMLFileOutputWindow(const vtkXMLFileOutputWindow&);  // Not implemented.
   void operator=(const vtkXMLFileOutputWindow&);  // Not implemented.
