@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: RGrid.cxx,v $
+  Module:    RGrid.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -26,7 +26,7 @@
 #include "vtkProperty.h"
 #include "vtkCamera.h"
 
-int main( int argc, char *argv[] )
+int main()
 {
   int i;
   static float x[47]={
@@ -80,7 +80,7 @@ int main( int argc, char *argv[] )
     plane->SetExtent(0,46, 16,16, 0,43);
 
   vtkPolyDataMapper *rgridMapper = vtkPolyDataMapper::New();
-      rgridMapper->SetInput(plane->GetOutput());
+      rgridMapper->SetInputConnection(plane->GetOutputPort());
 
   vtkActor *wireActor = vtkActor::New();
       wireActor->SetMapper(rgridMapper);
@@ -96,6 +96,7 @@ int main( int argc, char *argv[] )
 
   renderer->AddActor(wireActor);
   renderer->SetBackground(1,1,1);
+  renderer->ResetCamera();
   renderer->GetActiveCamera()->Elevation(60.0);
   renderer->GetActiveCamera()->Azimuth(30.0);
   renderer->GetActiveCamera()->Zoom(1.0);
@@ -107,6 +108,7 @@ int main( int argc, char *argv[] )
   iren->Start();
 
   // Clean up
+  plane->Delete();
   renderer->Delete();
   renWin->Delete();
   iren->Delete();
