@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: SGrid.cxx,v $
+  Module:    SGrid.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -34,7 +34,7 @@
 #include "vtkRenderer.h"
 #include "vtkStructuredGrid.h"
 
-int main( int argc, char *argv[] )
+int main()
 {
   int i, j, k, kOffset, jOffset, offset;
   float x[3], v[3], rMin=0.5, rMax=1.0, deltaRad, deltaZ;
@@ -66,7 +66,7 @@ int main( int argc, char *argv[] )
       jOffset = j * dims[0];
       for (i=0; i<dims[0]; i++) 
         {
-        theta = i * 15.0 * vtkMath::DegreesToRadians();
+        theta = i * vtkMath::RadiansFromDegrees(15.0);
         x[0] = radius * cos(theta);
         x[1] = radius * sin(theta);
         v[0] = -x[1];
@@ -88,7 +88,7 @@ int main( int argc, char *argv[] )
       hedgehog->SetScaleFactor(0.1);
 
   vtkPolyDataMapper *sgridMapper = vtkPolyDataMapper::New();
-      sgridMapper->SetInput(hedgehog->GetOutput());
+      sgridMapper->SetInputConnection(hedgehog->GetOutputPort());
   vtkActor *sgridActor = vtkActor::New();
       sgridActor->SetMapper(sgridMapper);
       sgridActor->GetProperty()->SetColor(0,0,0);
@@ -103,6 +103,7 @@ int main( int argc, char *argv[] )
 
   renderer->AddActor(sgridActor);
   renderer->SetBackground(1,1,1);
+  renderer->ResetCamera();
   renderer->GetActiveCamera()->Elevation(60.0);
   renderer->GetActiveCamera()->Azimuth(30.0);
   renderer->GetActiveCamera()->Zoom(1.25);
