@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: Example1.cxx,v $
+  Module:    Example1.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -25,13 +25,16 @@
 #include "vtkXRenderWindowInteractor.h"
 
 #include <Xm/PushB.h>
+
 void quit_cb(Widget,XtPointer,XtPointer);
-main (int argc, char *argv[])
+
+int main (int argc, char *argv[])
 {
   // X window stuff
   XtAppContext app;
   Widget toplevel, button;
   Display *display;
+
   // VTK stuff
   vtkRenderWindow *renWin;
   vtkRenderer *ren1;
@@ -42,59 +45,61 @@ main (int argc, char *argv[])
   vtkPolyDataMapper *sphereMapper, *spikeMapper;
   vtkXRenderWindowInteractor *iren;
 
-
   renWin = vtkRenderWindow::New();
   ren1 = vtkRenderer::New();
   renWin->AddRenderer(ren1);
   
   sphere = vtkSphereSource::New();
   sphereMapper = vtkPolyDataMapper::New();
-  sphereMapper->SetInput(sphere->GetOutput());
+  sphereMapper->SetInputConnection(sphere->GetOutputPort());
   sphereActor1 = vtkActor::New();
   sphereActor1->SetMapper(sphereMapper);
   cone = vtkConeSource::New();
   glyph = vtkGlyph3D::New();
-  glyph->SetInput(sphere->GetOutput());
-  glyph->SetSource(cone->GetOutput());
+  glyph->SetInputConnection(sphere->GetOutputPort());
+  glyph->SetSourceConnection(cone->GetOutputPort());
   glyph->SetVectorModeToUseNormal();
   glyph->SetScaleModeToScaleByVector();
   glyph->SetScaleFactor(0.25);
   spikeMapper = vtkPolyDataMapper::New();
-  spikeMapper->SetInput(glyph->GetOutput());
+  spikeMapper->SetInputConnection(glyph->GetOutputPort());
   spikeActor1 = vtkActor::New();
   spikeActor1->SetMapper(spikeMapper);
   ren1->AddActor(sphereActor1);
   ren1->AddActor(spikeActor1);
   ren1->SetBackground(0.4,0.1,0.2);
 
-
-// do the xwindow ui stuff
+  // do the xwindow ui stuff
   XtSetLanguageProc(NULL,NULL,NULL);
   toplevel = XtVaAppInitialize(&app,"Sample",NULL,0,
-                               &argc,argv,NULL, NULL);
-  
+                               &argc,argv,NULL, static_cast<void *>(NULL));
+
   // get the display connection and give it to the renderer
   display = XtDisplay(toplevel);
   renWin->SetDisplayId(display);
-  
-  // we typecast to an X specific interactor
-  // Since we have decided to make this an X program
+
+  // We use an X specific interactor
+  // since we have decided to make this an X program
   iren = vtkXRenderWindowInteractor::New();
   iren->SetRenderWindow(renWin);
   iren->Initialize(app);
-  
+
   button = XtVaCreateManagedWidget("Exit",
-                                   xmPushButtonWidgetClass, 
+                                   xmPushButtonWidgetClass,
                                    toplevel,
-                                   XmNwidth, 50, 
-                                   XmNheight, 50,NULL);
-  
+                                   XmNwidth, 50,
+                                   XmNheight, 50, static_cast<void *>(NULL));
+
   XtRealizeWidget(toplevel);
   XtAddCallback(button,XmNactivateCallback,quit_cb,NULL);
   XtAppMainLoop(app);
+
+  return 0;
 }
+
 // simple quit callback
-void quit_cb(Widget w,XtPointer client_data, XtPointer call_data)
+void quit_cb(Widget vtkNotUsed(w),XtPointer vtkNotUsed(client_data),
+             XtPointer vtkNotUsed(call_data))
 {
   exit(0);
 }
