@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: TaskParallelism.h,v $
+  Module:    TaskParallelism.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,7 +15,7 @@
 #ifndef __TASKPARA_H
 #define __TASKPARA_H
 
-#include "vtkMultiProcessController.h"
+#include "vtkMPIController.h"
 #include "vtkRTAnalyticSource.h"
 #include "vtkFieldDataToAttributeDataFilter.h"
 #include "vtkAttributeDataToFieldDataFilter.h"
