@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: Cone.cxx,v $
+  Module:    Cone.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -27,7 +27,7 @@
 #include "vtkActor.h"
 #include "vtkRenderer.h"
 
-int main( int argc, char *argv[] )
+int main()
 {
   // 
   // Next we create an instance of vtkConeSource and set some of its
@@ -48,7 +48,7 @@ int main( int argc, char *argv[] )
   // connect the output of the cone souece to the input of this mapper.
   //
   vtkPolyDataMapper *coneMapper = vtkPolyDataMapper::New();
-  coneMapper->SetInput( cone->GetOutput() );
+  coneMapper->SetInputConnection( cone->GetOutputPort() );
 
   // 
   // Create an actor to represent the cone. The actor orchestrates rendering
