@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: Cone6.cxx,v $
+  Module:    Cone6.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -46,18 +46,17 @@ class vtkMyCallback : public vtkCommand
 public:
   static vtkMyCallback *New() 
     { return new vtkMyCallback; }
-  void Delete()
-    { delete this; }
   virtual void Execute(vtkObject *caller, unsigned long, void*)
     {
       vtkTransform *t = vtkTransform::New();
       vtkBoxWidget *widget = reinterpret_cast<vtkBoxWidget*>(caller);
       widget->GetTransform(t);
       widget->GetProp3D()->SetUserTransform(t);
+      t->Delete();
     }
 };
 
-int main( int argc, char *argv[] )
+int main()
 {
   // 
   // Next we create an instance of vtkConeSource and set some of its
@@ -78,7 +77,7 @@ int main( int argc, char *argv[] )
   // connect the output of the cone souece to the input of this mapper.
   //
   vtkPolyDataMapper *coneMapper = vtkPolyDataMapper::New();
-  coneMapper->SetInput( cone->GetOutput() );
+  coneMapper->SetInputConnection( cone->GetOutputPort() );
 
   // 
   // Create an actor to represent the cone. The actor orchestrates rendering
