@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkCellLocator.cxx,v $
+  Module:    vtkCellLocator.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -23,7 +23,6 @@
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkCellLocator, "$Revision: 1.81 $");
 vtkStandardNewMacro(vtkCellLocator);
 
 #define VTK_CELL_OUTSIDE 0
@@ -31,6 +30,7 @@ vtkStandardNewMacro(vtkCellLocator);
 
 typedef vtkIdList *vtkIdListPtr;
 
+//----------------------------------------------------------------------------
 class vtkNeighborCells
 {
   public:
@@ -56,22 +56,24 @@ inline int vtkNeighborCells::InsertNextPoint(int *x)
   return id/3;
 }
 
+//----------------------------------------------------------------------------
 // Construct with automatic computation of divisions, averaging
 // 25 cells per bucket.
 vtkCellLocator::vtkCellLocator()
 {
-  this->NumberOfCellsPerBucket = 25;
-  this->Tree = NULL;
-  this->CellHasBeenVisited = NULL;
-  this->QueryNumber = 0;
+  this->MaxLevel             = 8;
+  this->Level                = 8;
+  this->NumberOfCellsPerNode = 25;
+  this->Tree                 = NULL;
+  this->CellHasBeenVisited   = NULL;
+  this->QueryNumber          = 0;
+  this->NumberOfDivisions    = 1;
   this->H[0] = this->H[1] = this->H[2] = 1.0;
-  this->NumberOfDivisions = 1;
 
   this->Buckets = new vtkNeighborCells(10, 10);
-  this->CacheCellBounds = 0;
-  this->CellBounds = NULL;
 }
 
+//----------------------------------------------------------------------------
 vtkCellLocator::~vtkCellLocator()
 {
   if (this->Buckets)
@@ -81,20 +83,16 @@ vtkCellLocator::~vtkCellLocator()
     }
   
   this->FreeSearchStructure();
+  this->FreeCellBounds();
  
   if (this->CellHasBeenVisited)
     {
     delete [] this->CellHasBeenVisited;
     this->CellHasBeenVisited = NULL;
     }
-
-  if (this->CellBounds)
-    {
-    delete [] this->CellBounds;
-    this->CellBounds = NULL;
-    }
 }
 
+//----------------------------------------------------------------------------
 void vtkCellLocator::FreeSearchStructure()
 {
   vtkIdList *cellIds;
@@ -105,7 +103,7 @@ void vtkCellLocator::FreeSearchStructure()
     for (i=0; i<this->NumberOfOctants; i++)
       {
       cellIds = this->Tree[i];
-      if (cellIds == (void *)VTK_CELL_INSIDE)
+      if (cellIds == reinterpret_cast<void *>(VTK_CELL_INSIDE))
         {
         cellIds = 0;
         }
@@ -119,6 +117,7 @@ void vtkCellLocator::FreeSearchStructure()
     }
 }
 
+//----------------------------------------------------------------------------
 // Given an offset into the structure, the number of divisions in the octree,
 // an i,j,k location in the octree; return the index (idx) into the structure.
 // Method returns 1 is the specified i,j,k location is "outside" of the octree.
@@ -137,18 +136,7 @@ int vtkCellLocator::GenerateIndex(int offset, int numDivs, int i, int j,
 }
 
 
-// Return intersection point (if any) of finite line with cells contained
-// in cell locator.
-int vtkCellLocator::IntersectWithLine(double a0[3], double a1[3], double tol,
-                                      double& t, double x[3], double pcoords[3],
-                                      int &subId)
-{
-  vtkIdType cellId = -1;
-  
-  return this->IntersectWithLine( a0, a1, tol, t, x, pcoords,
-                                  subId, cellId);
-}
-
+//----------------------------------------------------------------------------
 void vtkCellLocator::ComputeOctantBounds(int i, int j, int k)
 {
   this->OctantBounds[0] = this->Bounds[0] + i*H[0];
@@ -158,24 +146,8 @@ void vtkCellLocator::ComputeOctantBounds(int i, int j, int k)
   this->OctantBounds[4] = this->Bounds[4] + k*H[2];
   this->OctantBounds[5] = this->OctantBounds[4] + H[2];
 }
-
-// Return intersection point (if any) AND the cell which was intersected by
-// finite line
-int vtkCellLocator::IntersectWithLine(double a0[3], double a1[3], double tol,
-                                      double& t, double x[3], double pcoords[3],
-                                      int &subId, vtkIdType &cellId)
-{
-  vtkGenericCell *cell=vtkGenericCell::New();
-  int returnVal;
-
-  returnVal = this->IntersectWithLine( a0, a1, tol, t, x, pcoords, subId,
-                                       cellId, cell);
-
-  cell->Delete();
-  return returnVal;
-}
-  
     
+//----------------------------------------------------------------------------
 // Return intersection point (if any) AND the cell which was intersected by
 // finite line
 int vtkCellLocator::IntersectWithLine(double a0[3], double a1[3], double tol,
@@ -203,6 +175,8 @@ int vtkCellLocator::IntersectWithLine(double a0[3], double a1[3], double tol,
   double deltaT, pDistance, minPDistance=1.0e38;
   double length, maxLength=0.0;
   
+  this->BuildLocatorIfNeeded();
+
   // convert the line into i,j,k coordinates
   tMax = 0.0;
   for (i=0; i < 3; i++) 
@@ -221,6 +195,8 @@ int vtkCellLocator::IntersectWithLine(double a0[3], double a1[3], double tol,
     tMax += direction2[i]*direction2[i];
     }
   
+  tMax = sqrt(tMax);
+  
   // create a parametric range around the tolerance
   deltaT = tol/maxLength;
 
@@ -259,7 +235,7 @@ int vtkCellLocator::IntersectWithLine(double a0[3], double a1[3], double tol,
     for (loop = 0; loop <3; loop++)
       {
       hitPosition[loop] = hitPosition[loop]*this->NumberOfDivisions + 1.0;
-      pos[loop] = (int)hitPosition[loop];
+      pos[loop] = static_cast<int>(hitPosition[loop]);
       // Adjust right boundary condition: if we intersect from the top, right,
       // or back; then pos must be adjusted to a valid octant index 
       if (pos[loop] > this->NumberOfDivisions)
@@ -390,7 +366,7 @@ int vtkCellLocator::IntersectWithLine(double a0[3], double a1[3], double tol,
       idx = leafStart + pos[0] - 1 + (pos[1]-1)*this->NumberOfDivisions + 
         (pos[2]-1)*prod;
       }
-    }
+    } // if (vtkBox::IntersectBox(...))
     
   if (bestCellId >= 0)
     {
@@ -405,6 +381,7 @@ int vtkCellLocator::IntersectWithLine(double a0[3], double a1[3], double tol,
   return 0;
 }
 
+//----------------------------------------------------------------------------
 // Return closest point (if any) AND the cell on which this closest point lies
 void vtkCellLocator::FindClosestPoint(double x[3], double closestPoint[3], 
                                       vtkGenericCell *cell, vtkIdType &cellId,
@@ -424,7 +401,15 @@ void vtkCellLocator::FindClosestPoint(double x[3], double closestPoint[3],
   double *weights = weightsArray;
   int nWeights = 6, nPoints;
   vtkIdList *cellIds;
+  int stat;
+  //int minStat=0; //save this variable it is used for debugging
   
+  this->BuildLocatorIfNeeded();
+
+  cachedPoint[0] = 0.0;
+  cachedPoint[1] = 0.0;
+  cachedPoint[2] = 0.0;
+
   leafStart = this->NumberOfOctants
     - this->NumberOfDivisions*this->NumberOfDivisions*this->NumberOfDivisions;
   
@@ -447,7 +432,7 @@ void vtkCellLocator::FindClosestPoint(double x[3], double closestPoint[3],
   //
   for (j=0; j<3; j++) 
     {
-    ijk[j] = (int)((x[j] - this->Bounds[2*j]) / this->H[j]);
+    ijk[j] = static_cast<int>((x[j] - this->Bounds[2*j]) / this->H[j]);
     
     if (ijk[j] < 0)
       {
@@ -521,10 +506,17 @@ void vtkCellLocator::FindClosestPoint(double x[3], double closestPoint[3],
                   }
                 
                 // evaluate the position to find the closest point
-                int stat=cell->EvaluatePosition(x, point, subId, pcoords,
-                  dist2, weights);
+                // stat==(-1) is numerical error; stat==0 means outside;
+                // stat=1 means inside. However, for real world performance,
+                // we sometime select stat==0 cells if the distance is close
+                // enough
+                stat = cell->EvaluatePosition(x, point, subId, pcoords,
+                                              dist2, weights);
                 
                 if ( stat != -1 && dist2 < minDist2 ) 
+// This commented out code works better in many cases                                
+//                if ( stat != -1 && ((stat == minStat && dist2 < minDist2) ||
+//                     (stat == 1 && minStat == 0)) )
                   {
                   closestCell = cellId;
                   closestSubCell = subId;
@@ -533,6 +525,7 @@ void vtkCellLocator::FindClosestPoint(double x[3], double closestPoint[3],
                   cachedPoint[1] = point[1];
                   cachedPoint[2] = point[2];
                   refinedRadius2 = dist2;
+//                  minStat = stat;
                   }
                 }
               } // if (!this->CellHasBeenVisited[cellId])
@@ -661,20 +654,8 @@ void vtkCellLocator::FindClosestPoint(double x[3], double closestPoint[3],
     }
 }
 
-
-// Return closest point (if any) AND the cell on which this closest point lies
-void vtkCellLocator::FindClosestPoint(double x[3], double closestPoint[3],
-                                      vtkIdType &cellId, int &subId,
-                                      double& dist2)
-{
-  vtkGenericCell *cell = vtkGenericCell::New();
-  
-  this->FindClosestPoint(x, closestPoint, cell, cellId, subId, dist2);
-  
-  cell->Delete();
-}
-
-int vtkCellLocator::FindClosestPointWithinRadius(double x[3], double radius,
+//----------------------------------------------------------------------------
+vtkIdType vtkCellLocator::FindClosestPointWithinRadius(double x[3], double radius,
                                                  double closestPoint[3],
                                                  vtkGenericCell *cell,
                                                  vtkIdType &cellId, int &subId,
@@ -699,7 +680,13 @@ int vtkCellLocator::FindClosestPointWithinRadius(double x[3], double radius,
   double distance2ToCellBounds, cellBounds[6], currentRadius;
   double distance2ToDataBounds, maxDistance;
   int ii, radiusLevels[3], radiusLevel, prevMinLevel[3], prevMaxLevel[3];
-  
+
+  this->BuildLocatorIfNeeded();
+
+  cachedPoint[0] = 0.0;
+  cachedPoint[1] = 0.0;
+  cachedPoint[2] = 0.0;
+
   leafStart = this->NumberOfOctants
     - this->NumberOfDivisions*this->NumberOfDivisions*this->NumberOfDivisions;
   
@@ -725,7 +712,7 @@ int vtkCellLocator::FindClosestPointWithinRadius(double x[3], double radius,
   //
   for (j=0; j<3; j++) 
     {
-    ijk[j] = (int)((x[j] - this->Bounds[2*j]) / this->H[j]);
+    ijk[j] = static_cast<int>((x[j] - this->Bounds[2*j]) / this->H[j]);
     
     if (ijk[j] < 0)
       {
@@ -829,9 +816,9 @@ int vtkCellLocator::FindClosestPointWithinRadius(double x[3], double radius,
     refinedRadius2 = maxDistance*maxDistance;
     }
   
-  radiusLevels[0] = (int)(refinedRadius/this->H[0]);
-  radiusLevels[1] = (int)(refinedRadius/this->H[1]);
-  radiusLevels[2] = (int)(refinedRadius/this->H[2]);
+  radiusLevels[0] = static_cast<int>(refinedRadius/this->H[0]);
+  radiusLevels[1] = static_cast<int>(refinedRadius/this->H[1]);
+  radiusLevels[2] = static_cast<int>(refinedRadius/this->H[2]);
   
   radiusLevel = radiusLevels[0];
   radiusLevel = radiusLevels[1] > radiusLevel ? radiusLevels[1] : radiusLevel;
@@ -941,7 +928,8 @@ int vtkCellLocator::FindClosestPointWithinRadius(double x[3], double radius,
     // ii appropriately
     if (refinedRadius < currentRadius && ii > 2) //always check ii==1
       {
-      ii = (int)((double)ii * (refinedRadius / currentRadius)) + 1;
+      ii = static_cast<int>(
+        static_cast<double>(ii) * (refinedRadius / currentRadius)) + 1;
       if (ii < 2)
         {
         ii = 2;
@@ -969,35 +957,7 @@ int vtkCellLocator::FindClosestPointWithinRadius(double x[3], double radius,
   return returnVal;
 }
 
-int vtkCellLocator::FindClosestPointWithinRadius(double x[3], double radius,
-                                                 double closestPoint[3],
-                                                 vtkGenericCell *cell,
-                                                 vtkIdType &cellId,
-                                                 int &subId, double& dist2)
-{
-  int inside;
-
-  return 
-    this->FindClosestPointWithinRadius(x, radius, closestPoint,
-                                       cell, cellId, subId, dist2, inside);
-}
-
-int vtkCellLocator::FindClosestPointWithinRadius(double x[3], double radius,
-                                                 double closestPoint[3],
-                                                 vtkIdType &cellId, int &subId,
-                                                 double& dist2)
-{
-  vtkGenericCell *cell = vtkGenericCell::New();
-  int found, inside;
-  
-  found = 
-    this->FindClosestPointWithinRadius(x, radius, closestPoint,
-                                       cell, cellId, subId, dist2, inside);
-  cell->Delete();
-  return found;
-}
-
-//
+//----------------------------------------------------------------------------
 //  Internal function to get bucket neighbors at specified "level". The
 //  bucket neighbors are indices into the "leaf-node" layer of the octree.
 //  These indices must be offset by number of octants before the leaf node
@@ -1010,6 +970,8 @@ void vtkCellLocator::GetBucketNeighbors(int ijk[3], int ndivs, int level)
   int leafStart;
   int numberOfBucketsPerPlane;
   
+  this->BuildLocatorIfNeeded();
+
   numberOfBucketsPerPlane = this->NumberOfDivisions*this->NumberOfDivisions;
   leafStart = this->NumberOfOctants
     - numberOfBucketsPerPlane*this->NumberOfDivisions;
@@ -1065,6 +1027,7 @@ void vtkCellLocator::GetBucketNeighbors(int ijk[3], int ndivs, int level)
   return;
 }
 
+//----------------------------------------------------------------------------
 // Internal method to find those buckets that are within distance specified.
 // Only those buckets outside of level radiuses of ijk are returned. The
 // bucket neighbors are indices into the "leaf-node" layer of the octree.
@@ -1081,6 +1044,8 @@ void vtkCellLocator::GetOverlappingBuckets(double x[3], int vtkNotUsed(ijk)[3],
   int leafStart, kFactor, jFactor;
   int numberOfBucketsPerPlane, jkSkipFlag, kSkipFlag;
 
+  this->BuildLocatorIfNeeded();
+
   numberOfBucketsPerPlane = this->NumberOfDivisions*this->NumberOfDivisions;
   leafStart = this->NumberOfOctants
     - numberOfBucketsPerPlane*this->NumberOfDivisions;
@@ -1091,10 +1056,12 @@ void vtkCellLocator::GetOverlappingBuckets(double x[3], int vtkNotUsed(ijk)[3],
   // Determine the range of indices in each direction
   for (i=0; i < 3; i++)
     {
-    minLevel[i] = (int) ((double) (((x[i]-dist) - this->Bounds[2*i])
-                                  / this->H[i]));
-    maxLevel[i] = (int) ((double) (((x[i]+dist) - this->Bounds[2*i])
-                                  / this->H[i]));
+    minLevel[i] =
+      static_cast<int> (static_cast<double> (((x[i]-dist) - this->Bounds[2*i])
+                                             / this->H[i]));
+    maxLevel[i] =
+      static_cast<int> (static_cast<double> (((x[i]+dist) - this->Bounds[2*i])
+                                             / this->H[i]));
     
     if ( minLevel[i] < 0 )
       {
@@ -1168,6 +1135,7 @@ void vtkCellLocator::GetOverlappingBuckets(double x[3], int vtkNotUsed(ijk)[3],
   prevMaxLevel[2] = maxLevel[2];
 }
 
+//----------------------------------------------------------------------------
 // number of buckets available
 int vtkCellLocator::GetNumberOfBuckets(void) 
 {
@@ -1182,6 +1150,7 @@ int vtkCellLocator::GetNumberOfBuckets(void)
     }
 }
 
+//----------------------------------------------------------------------------
 // Get the cells in a bucket.
 vtkIdList* vtkCellLocator::GetCells(int octantId)
 {
@@ -1189,13 +1158,46 @@ vtkIdList* vtkCellLocator::GetCells(int octantId)
   return this->Tree[octantId];
 }
 
-
+//---------------------------------------------------------------------------
+void vtkCellLocator::BuildLocator()
+{
+  if (this->LazyEvaluation) return;
+  this->ForceBuildLocator();
+}
+//---------------------------------------------------------------------------
+void vtkCellLocator::BuildLocatorIfNeeded()
+{
+  if (this->LazyEvaluation) {
+    if (!this->Tree || (this->Tree && (this->MTime>this->BuildTime))) {
+      this->Modified();
+      vtkDebugMacro(<< "Forcing BuildLocator");
+      this->ForceBuildLocator();
+    }
+  }
+}
+//---------------------------------------------------------------------------
+void vtkCellLocator::ForceBuildLocator()
+{
+  //
+  // don't rebuild if build time is newer than modified and dataset modified time
+  if ( (this->Tree) && (this->BuildTime>this->MTime) && (this->BuildTime>DataSet->GetMTime())) {
+    return;
+  }
+  // don't rebuild if UseExistingSearchStructure is ON and a tree structure already exists
+  if ( (this->Tree) && this->UseExistingSearchStructure) {
+    this->BuildTime.Modified();
+    vtkDebugMacro(<< "BuildLocator exited - UseExistingSearchStructure");
+    return;
+  }
+  this->BuildLocatorInternal();
+}
+//---------------------------------------------------------------------------
 //  Method to form subdivision of space based on the cells provided and
-//  subject to the constraints of levels and NumberOfCellsPerBucket.
+//  subject to the constraints of levels and NumberOfCellsPerNode.
 //  The result is directly addressable and of uniform subdivision.
 //
-void vtkCellLocator::BuildLocator()
-  {
+void vtkCellLocator::BuildLocatorInternal()
+{
   double *bounds, length, cellBounds[6], *boundsPtr;
   vtkIdType numCells;
   int ndivs, product;
@@ -1203,17 +1205,10 @@ void vtkCellLocator::BuildLocator()
   vtkIdType cellId, idx;
   int parentOffset;
   vtkIdList *octant;
-  int numCellsPerBucket = this->NumberOfCellsPerBucket;
-  typedef vtkIdList *vtkIdListPtr;
+  int numCellsPerBucket = this->NumberOfCellsPerNode;
   int prod, numOctants;
   double hTol[3];
   
-  if ( (this->Tree != NULL) && (this->BuildTime > this->MTime)
-    && (this->BuildTime > this->DataSet->GetMTime()) )
-    {
-    return;
-    }
-  
   vtkDebugMacro( << "Subdividing octree..." );
   
   if ( !this->DataSet || (numCells = this->DataSet->GetNumberOfCells()) < 1 )
@@ -1233,11 +1228,7 @@ void vtkCellLocator::BuildLocator()
     delete [] this->CellHasBeenVisited;
     this->CellHasBeenVisited = NULL;
     }
-  if (this->CellBounds)
-    {
-    delete [] this->CellBounds;
-    this->CellBounds = NULL;
-    }
+  this->FreeCellBounds();
   
   //  Size the root cell.  Initialize cell data structure, compute
   //  level and divisions.
@@ -1258,8 +1249,9 @@ void vtkCellLocator::BuildLocator()
   
   if ( this->Automatic ) 
     {
-    this->Level = (int) (ceil(log((double)numCells/numCellsPerBucket) / 
-      (log((double) 8.0))));
+    this->Level = static_cast<int>(
+      ceil(log(static_cast<double>(numCells)/numCellsPerBucket) /
+           (log(static_cast<double>(8.0)))));
     } 
   this->Level =(this->Level > this->MaxLevel ? this->MaxLevel : this->Level);
   
@@ -1282,7 +1274,7 @@ void vtkCellLocator::BuildLocator()
 
   if (this->CacheCellBounds)
     {
-    this->CellBounds = new double [numCells][6];
+    this->StoreCellBounds();
     }
   
   //  Compute width of leaf octant in three directions
@@ -1301,10 +1293,9 @@ void vtkCellLocator::BuildLocator()
   boundsPtr = cellBounds;
   for (cellId=0; cellId<numCells; cellId++) 
     {
-    if (this->CacheCellBounds)
+    if (this->CellBounds)
       {
       boundsPtr = this->CellBounds[cellId];
-      this->DataSet->GetCellBounds(cellId, boundsPtr);
       }
     else
       {
@@ -1314,10 +1305,10 @@ void vtkCellLocator::BuildLocator()
     // find min/max locations of bounding box
     for (i=0; i<3; i++)
       {
-      ijkMin[i] = (int)((boundsPtr[2*i] - this->Bounds[2*i] - hTol[i]) 
-        / this->H[i]);
-      ijkMax[i] = (int)((boundsPtr[2*i+1] - this->Bounds[2*i] + hTol[i])
-        / this->H[i]);
+      ijkMin[i] = static_cast<int>(
+        (boundsPtr[2*i] - this->Bounds[2*i] - hTol[i])/ this->H[i]);
+      ijkMax[i] = static_cast<int>(
+        (boundsPtr[2*i+1] - this->Bounds[2*i] + hTol[i]) / this->H[i]);
       
       if (ijkMin[i] < 0)
         {
@@ -1337,7 +1328,8 @@ void vtkCellLocator::BuildLocator()
         for ( i = ijkMin[0]; i <= ijkMax[0]; i++ )
           {
           idx = parentOffset + i + j*ndivs + k*product;
-          this->MarkParents((void*)VTK_CELL_INSIDE,i,j,k,ndivs,this->Level);
+          this->MarkParents(reinterpret_cast<void*>(VTK_CELL_INSIDE),i,j,k,
+                            ndivs,this->Level);
           octant = this->Tree[idx];
           if ( ! octant )
             {
@@ -1355,6 +1347,7 @@ void vtkCellLocator::BuildLocator()
   this->BuildTime.Modified();
 }
 
+//----------------------------------------------------------------------------
 void vtkCellLocator::MarkParents(void* a, int i, int j, int k, 
                                  int ndivs, int level)
 {
@@ -1383,33 +1376,36 @@ void vtkCellLocator::MarkParents(void* a, int i, int j, int k,
       return;
       }
     
-    this->Tree[parentIdx] = (vtkIdList *)a;
+    this->Tree[parentIdx] = static_cast<vtkIdList *>(a);
     
     prod = prod >> 3;
     offset -= prod;
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkCellLocator::GenerateRepresentation(int level, vtkPolyData *pd)
 {
   vtkPoints *pts;
   vtkCellArray *polys;
   int l, i, j, k, ii, boundary[3];
-  vtkIdType idx;
-  vtkIdList *inside, *Inside[3];
+  vtkIdType idx = 0;
+  vtkIdList *inside, *Inside[3] = {0, 0, 0};
   int numDivs=1;
-  
+
+  this->BuildLocatorIfNeeded();
+
   if ( this->Tree == NULL )
     {
     vtkErrorMacro(<<"No tree to generate representation from");
     return;
     }
-  
+
   pts = vtkPoints::New();
   pts->Allocate(5000);
   polys = vtkCellArray::New();
   polys->Allocate(10000);
-  
+
   // Compute idx into tree at appropriate level; determine if
   // faces of octants are visible.
   //
@@ -1492,6 +1488,7 @@ void vtkCellLocator::GenerateRepresentation(int level, vtkPolyData *pd)
   pd->Squeeze();
 }
   
+//----------------------------------------------------------------------------
 void vtkCellLocator::GenerateFace(int face, int numDivs, int i, int j, int k,
                                   vtkPoints *pts, vtkCellArray *polys)
 {
@@ -1567,6 +1564,7 @@ void vtkCellLocator::GenerateFace(int face, int numDivs, int i, int j, int k,
   polys->InsertNextCell(4,ids);
 }
   
+//----------------------------------------------------------------------------
 void vtkCellLocator::ClearCellHasBeenVisited()
 {
   if (this->CellHasBeenVisited && this->DataSet)
@@ -1575,6 +1573,7 @@ void vtkCellLocator::ClearCellHasBeenVisited()
     }
 }
   
+//----------------------------------------------------------------------------
 void vtkCellLocator::ClearCellHasBeenVisited(int id)
 {
   if (this->CellHasBeenVisited
@@ -1584,6 +1583,7 @@ void vtkCellLocator::ClearCellHasBeenVisited(int id)
     }
 }
   
+//----------------------------------------------------------------------------
 // Calculate the distance between the point x to the bucket "nei".
 //
 // WARNING!!!!! Be very careful altering this routine.  Simple changes to this
@@ -1603,6 +1603,7 @@ double vtkCellLocator::Distance2ToBucket(double x[3], int nei[3])
   return this->Distance2ToBounds(x, bounds);
 }
 
+//----------------------------------------------------------------------------
 // Calculate the distance between the point x and the specified bounds
 //
 // WARNING!!!!! Be very careful altering this routine.  Simple changes to this
@@ -1658,14 +1659,346 @@ double vtkCellLocator::Distance2ToBounds(double x[3], double bounds[6])
   distance = vtkMath::Dot(deltas, deltas);
   return distance;
 }
+//----------------------------------------------------------------------------
+bool vtkCellLocator_Inside(double bounds[6], double point[3]) {
+  if (point[0]<bounds[0] || point[0]>bounds[1] || 
+      point[1]<bounds[2] || point[1]>bounds[3] || 
+      point[2]<bounds[4] || point[2]>bounds[5]) return 0;
+  return 1;
+}
+//----------------------------------------------------------------------------
+vtkIdType vtkCellLocator::FindCell(
+  double x[3], double vtkNotUsed(tol2), vtkGenericCell *cell, 
+  double pcoords[3], double *weights)
+{
+  vtkIdList *cellIds;
+  int ijk[3];
+  int subId;
+  double closestPoint[3], dist2;
+  double cellBounds[6];
+  
+  this->BuildLocatorIfNeeded();
+
+  int leafStart = this->NumberOfOctants
+    - this->NumberOfDivisions*this->NumberOfDivisions*this->NumberOfDivisions;
+
+  // Find bucket point is in.  
+  //
+  for (int j=0; j<3; j++) 
+    {
+    ijk[j] = static_cast<int>((x[j] - this->Bounds[2*j]) / this->H[j]);
+    
+    if (ijk[j] < 0)
+      {
+      ijk[j] = 0;
+      }
+    else if (ijk[j] >= this->NumberOfDivisions)
+      {
+      ijk[j] = this->NumberOfDivisions-1;
+      }
+    }
+  
+  // Search the bucket that the point is in.
+  //
+  if ((cellIds = 
+      this->Tree[leafStart + ijk[0] + ijk[1]*this->NumberOfDivisions + 
+      ijk[2]*this->NumberOfDivisions*this->NumberOfDivisions]) != NULL )
+    {
+    // query each cell
+    for (int j=0; j < cellIds->GetNumberOfIds(); j++) 
+      {
+      // get the cell
+      int cellId = cellIds->GetId(j);
+      // check whether we could be close enough to the cell by
+      // testing the cell bounds
+      if (this->CacheCellBounds)
+        {
+        if (this->InsideCellBounds(x, cellId))
+          {
+          this->DataSet->GetCell(cellId, cell);
+          if (cell->EvaluatePosition(x, closestPoint, subId, pcoords, dist2, weights)==1) 
+            {
+            return cellId;
+            }
+          }
+        }
+      else
+        {
+        this->DataSet->GetCellBounds(cellId, cellBounds);
+        if (vtkCellLocator_Inside(cellBounds, x))
+          {
+          this->DataSet->GetCell(cellId, cell);
+          if (cell->EvaluatePosition(x, closestPoint, subId, pcoords, dist2, weights)==1) 
+            {
+            return cellId;
+            }
+          }
+        }       
+      }
+    }
+
+  return -1;
+}
+
+//----------------------------------------------------------------------------
+void vtkCellLocator::FindCellsWithinBounds(double *bbox, vtkIdList *cells)
+{
+  this->BuildLocatorIfNeeded();
+
+  cells->Reset();
+
+  // Get the locator locations for the two extreme corners of the bounding box
+  double p1[3], p2[3], *p[2];
+  p1[0] = bbox[0];
+  p1[1] = bbox[2];
+  p1[2] = bbox[4];
+  p2[0] = bbox[1];
+  p2[1] = bbox[3];
+  p2[2] = bbox[5];
+  p[0] = p1;
+  p[1] = p2;
+  int ijk[2][3];
+  
+  //  Find bucket the points are in
+  //
+  int i, j, k;
+  for (i=0; i<2; i++)
+    {
+    for (j=0; j<3; j++) 
+      {
+      ijk[i][j] = static_cast<int>((p[i][j] - this->Bounds[2*j]) / this->H[j]);
+    
+      if (ijk[i][j] < 0)
+        {
+        ijk[i][j] = 0;
+        }
+      else if (ijk[i][j] >= this->NumberOfDivisions)
+        {
+        ijk[i][j] = this->NumberOfDivisions-1;
+        }
+      }
+    }
+  
+  // Now loop over block to load in ids
+  int leafStart = this->NumberOfOctants
+    - this->NumberOfDivisions*this->NumberOfDivisions*this->NumberOfDivisions;
+  vtkIdList *cellIds;
+  vtkIdType idx;
+  for (k=ijk[0][2]; k <= ijk[1][2]; k++)
+    {
+    for (j=ijk[0][1]; j <= ijk[1][1]; j++)
+      {
+      for (i=ijk[0][0]; i <= ijk[1][0]; i++)
+        {
+        if ( (cellIds = this->Tree[leafStart + i + j*this->NumberOfDivisions + 
+                                  k*this->NumberOfDivisions*this->NumberOfDivisions]) != NULL )
+          {
+          for ( idx=0; idx < cellIds->GetNumberOfIds(); idx++)
+            {
+            cells->InsertUniqueId( cellIds->GetId(idx) );
+            }
+          }
+        }
+      }
+    }
+}
+
+//----------------------------------------------------------------------------
+void vtkCellLocator::FindCellsAlongLine(double p1[3], double p2[3], double vtkNotUsed(tol),
+                                        vtkIdList *cells)
+{
+  this->BuildLocatorIfNeeded();
+
+  cells->Reset();
+
+  double origin[3];
+  double direction1[3];
+  double direction2[3];
+  double direction3[3];
+  double hitPosition[3];
+  double hitCellBoundsPosition[3], cellBounds[6];
+  int hitCellBounds;
+  double result;
+  double bounds2[6];
+  int i, leafStart, prod, loop;
+  vtkIdType cellId, cId;
+  int idx;
+  double tMax, dist[3];
+  int npos[3];
+  int pos[3];
+  int bestDir;
+  double stopDist, currDist;
+  double length, maxLength=0.0;
+  
+  // convert the line into i,j,k coordinates
+  tMax = 0.0;
+  for (i=0; i < 3; i++) 
+    {
+    direction1[i] = p2[i] - p1[i];
+    length = this->Bounds[2*i+1] - this->Bounds[2*i];
+    if ( length > maxLength )
+      {
+      maxLength = length;
+      }
+    origin[i] = (p1[i] - this->Bounds[2*i]) / length;
+    direction2[i] = direction1[i]/length;
+    
+    bounds2[2*i]   = 0.0;
+    bounds2[2*i+1] = 1.0;
+    tMax += direction2[i]*direction2[i];
+    }
+  tMax = sqrt(tMax);
+
+  // create a parametric range around the tolerance
+  stopDist = tMax*this->NumberOfDivisions;
+  for (i = 0; i < 3; i++) 
+    {
+    direction3[i] = direction2[i]/tMax;
+    }
+  
+  if (vtkBox::IntersectBox(bounds2, origin, direction2, hitPosition, result))
+    {
+    // start walking through the octants
+    prod = this->NumberOfDivisions*this->NumberOfDivisions;
+    leafStart = this->NumberOfOctants - this->NumberOfDivisions*prod;
+    
+    // Clear the array that indicates whether we have visited this cell.
+    // The array is only cleared when the query number rolls over.  This
+    // saves a number of calls to memset.
+    this->QueryNumber++;
+    if (this->QueryNumber == 0)
+      {
+      this->ClearCellHasBeenVisited();
+      this->QueryNumber++;    // can't use 0 as a marker
+      }
+    
+    // set up curr and stop dist
+    currDist = 0;
+    for (i = 0; i < 3; i++)
+      {
+      currDist += (hitPosition[i] - origin[i])*(hitPosition[i] - origin[i]);
+      }
+    currDist = sqrt(currDist)*this->NumberOfDivisions;
+    
+    // add one offset due to the problems around zero
+    for (loop = 0; loop <3; loop++)
+      {
+      hitPosition[loop] = hitPosition[loop]*this->NumberOfDivisions + 1.0;
+      pos[loop] = static_cast<int>(hitPosition[loop]);
+      // Adjust right boundary condition: if we intersect from the top, right,
+      // or back; then pos must be adjusted to a valid octant index 
+      if (pos[loop] > this->NumberOfDivisions)
+        {
+        pos[loop] = this->NumberOfDivisions;
+        }
+      }
+    
+    idx = leafStart + pos[0] - 1 + (pos[1] - 1)*this->NumberOfDivisions 
+      + (pos[2] - 1)*prod;
+    
+    while ( (pos[0] > 0) && (pos[1] > 0) && (pos[2] > 0) &&
+      (pos[0] <= this->NumberOfDivisions) &&
+      (pos[1] <= this->NumberOfDivisions) &&
+      (pos[2] <= this->NumberOfDivisions) &&
+      (currDist < stopDist))
+      {
+      if (this->Tree[idx])
+        {
+        this->ComputeOctantBounds(pos[0]-1,pos[1]-1,pos[2]-1);
+        for (tMax = VTK_DOUBLE_MAX, cellId=0; 
+        cellId < this->Tree[idx]->GetNumberOfIds(); cellId++) 
+          {
+          cId = this->Tree[idx]->GetId(cellId);
+          if (this->CellHasBeenVisited[cId] != this->QueryNumber)
+            {
+            this->CellHasBeenVisited[cId] = this->QueryNumber;
+            hitCellBounds = 0;
+            
+            // check whether we intersect the cell bounds
+            if (this->CacheCellBounds)
+              {
+              hitCellBounds = vtkBox::IntersectBox(this->CellBounds[cId],
+                                                   p1, direction1,
+                                                   hitCellBoundsPosition, result);
+              }
+            else 
+              {
+              this->DataSet->GetCellBounds(cId, cellBounds);
+              hitCellBounds = vtkBox::IntersectBox(cellBounds,
+                                                   p1, direction1,
+                                                   hitCellBoundsPosition, result);
+              }
+
+            if (hitCellBounds)
+              {
+              cells->InsertUniqueId(cId);
+              } // if (hitCellBounds)
+            } // if (!this->CellHasBeenVisited[cId])
+          }
+        }
+      
+      // move to the next octant
+      tMax = VTK_DOUBLE_MAX;
+      bestDir = 0;
+      for (loop = 0; loop < 3; loop++)
+        {
+        if (direction3[loop] > 0)
+          {
+          npos[loop] = pos[loop] + 1;
+          dist[loop] = (1.0 - hitPosition[loop] + pos[loop])/direction3[loop];
+          if (dist[loop] == 0)
+            {
+            dist[loop] = 1.0/direction3[loop];
+            }
+          if (dist[loop] < 0)
+            {
+            dist[loop] = 0;
+            }
+          if (dist[loop] < tMax)
+            {
+            bestDir = loop;
+            tMax = dist[loop];
+            }
+          }
+        if (direction3[loop] < 0)
+          {
+          npos[loop] = pos[loop] - 1;
+          dist[loop] = (pos[loop] - hitPosition[loop])/direction3[loop];
+          if (dist[loop] == 0)
+            {
+            dist[loop] = -0.01/direction3[loop];
+            }
+          if (dist[loop] < 0)
+            {
+            dist[loop] = 0;
+            }
+          if (dist[loop] < tMax)
+            {
+            bestDir = loop;
+            tMax = dist[loop];
+            }
+          }
+        }
+      // update our position
+      for (loop = 0; loop < 3; loop++)
+        {
+        hitPosition[loop] += dist[bestDir]*direction3[loop];
+        }
+      currDist += dist[bestDir];
+      // now make the move, find the smallest distance
+      // only cross one boundry at a time
+      pos[bestDir] = npos[bestDir];
+      
+      idx = leafStart + pos[0] - 1 + (pos[1]-1)*this->NumberOfDivisions + 
+        (pos[2]-1)*prod;
+      }
+    }
+}
 
 
+//----------------------------------------------------------------------------
 void vtkCellLocator::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
-
-  os << indent << "Number of Cells Per Bucket: " 
-     << this->NumberOfCellsPerBucket << "\n";
-  os << indent << "Cache Cell Bounds: " << this->CacheCellBounds << "\n";  
 }
   
