@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkDataSetToDataSetFilter.cxx,v $
+  Module:    vtkDataSetToDataSetFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -13,24 +13,71 @@
 
 =========================================================================*/
 #include "vtkDataSetToDataSetFilter.h"
+
+#include "vtkDemandDrivenPipeline.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkPolyData.h"
 #include "vtkStructuredGrid.h"
 #include "vtkStructuredPoints.h"
 #include "vtkUnstructuredGrid.h"
 #include "vtkRectilinearGrid.h"
 
-vtkCxxRevisionMacro(vtkDataSetToDataSetFilter, "$Revision: 1.64 $");
 
 // Construct object.
 vtkDataSetToDataSetFilter::vtkDataSetToDataSetFilter()
 {
   this->NumberOfRequiredInputs = 1;
+  this->SetNumberOfInputPorts(1);
 }
 
 vtkDataSetToDataSetFilter::~vtkDataSetToDataSetFilter()
 {
 }
 
+int vtkDataSetToDataSetFilter::RequestDataObject(vtkInformation*,
+                                            vtkInformationVector** inputVector,
+                                            vtkInformationVector* outputVector)
+{
+  vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
+  if (!inInfo)
+    {
+    return 0;
+    }
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  
+  if (input)
+    {
+    vtkInformation* info = outputVector->GetInformationObject(0);
+    vtkDataSet *output = vtkDataSet::SafeDownCast(
+      info->Get(vtkDataObject::DATA_OBJECT()));
+    
+    if (!output || !output->IsA(input->GetClassName())) 
+      {
+      output = input->NewInstance();
+      output->SetPipelineInformation(info);
+      output->Delete();
+      }
+    return 1;
+    }
+  return 0;
+}
+
+int vtkDataSetToDataSetFilter::ProcessRequest(vtkInformation* request,
+                                              vtkInformationVector** inputVector,
+                                              vtkInformationVector* outputVector)
+{
+  // Handling REQUEST_DATA_OBJECT because if the filter is connected
+  // with SetInputConnection() as opposed to SetInput(), the output
+  // never gets created
+  if(request->Has(vtkDemandDrivenPipeline::REQUEST_DATA_OBJECT()))
+    {
+    return this->RequestDataObject(request, inputVector, outputVector);
+    }
+  return this->Superclass::ProcessRequest(request, inputVector, outputVector);
+}
+
 //----------------------------------------------------------------------------
 // Specify the input data or filter.
 void vtkDataSetToDataSetFilter::SetInput(vtkDataSet *input)
@@ -46,7 +93,6 @@ void vtkDataSetToDataSetFilter::SetInput(vtkDataSet *input)
       this->SetOutput(NULL);
       }
     }
-  
   if (input != NULL && this->vtkSource::GetOutput(0) == NULL)
     {
     this->vtkSource::SetNthOutput(0, input->NewInstance());
@@ -74,7 +120,7 @@ vtkDataSet *vtkDataSetToDataSetFilter::GetOutput()
     return NULL;
     }
 
-  return (vtkDataSet *)this->Outputs[0];
+  return this->Superclass::GetOutput(0);
 }
 
 // Get the output as vtkPolyData.
@@ -87,7 +133,7 @@ vtkPolyData *vtkDataSetToDataSetFilter::GetPolyDataOutput()
     }
   if (ds->GetDataObjectType() == VTK_POLY_DATA)
     {
-    return (vtkPolyData *)ds;
+    return static_cast<vtkPolyData *>(ds);
     }
   return NULL;
 }
@@ -102,7 +148,7 @@ vtkStructuredPoints *vtkDataSetToDataSetFilter::GetStructuredPointsOutput()
     }
   if (ds->GetDataObjectType() == VTK_STRUCTURED_POINTS)
     {
-    return (vtkStructuredPoints *)ds;
+    return static_cast<vtkStructuredPoints *>(ds);
     }
   return NULL;
 }
@@ -117,7 +163,7 @@ vtkStructuredGrid *vtkDataSetToDataSetFilter::GetStructuredGridOutput()
     }
   if (ds->GetDataObjectType() == VTK_STRUCTURED_GRID)
     {
-    return (vtkStructuredGrid *)ds;
+    return static_cast<vtkStructuredGrid *>(ds);
     }
   return NULL;
 }
@@ -132,7 +178,7 @@ vtkUnstructuredGrid *vtkDataSetToDataSetFilter::GetUnstructuredGridOutput()
     }
   if (ds->GetDataObjectType() == VTK_UNSTRUCTURED_GRID)
     {
-    return (vtkUnstructuredGrid *)ds;
+    return static_cast<vtkUnstructuredGrid *>(ds);
     }
   return NULL;
 }
@@ -147,7 +193,7 @@ vtkRectilinearGrid *vtkDataSetToDataSetFilter::GetRectilinearGridOutput()
     }
   if (ds->GetDataObjectType() == VTK_RECTILINEAR_GRID)
     {
-    return (vtkRectilinearGrid *)ds;
+    return static_cast<vtkRectilinearGrid *>(ds);
     }
   return NULL;
 }
@@ -163,7 +209,7 @@ vtkDataSet *vtkDataSetToDataSetFilter::GetInput()
     return NULL;
     }
   
-  return (vtkDataSet *)(this->Inputs[0]);
+  return static_cast<vtkDataSet *>(this->Inputs[0]);
 }
 
 //----------------------------------------------------------------------------
@@ -171,7 +217,11 @@ vtkDataSet *vtkDataSetToDataSetFilter::GetInput()
 // copy information
 void vtkDataSetToDataSetFilter::ExecuteInformation()
 {
-  this->GetOutput()->CopyTypeSpecificInformation( this->GetInput() );
+  vtkDataSet* output = this->GetOutput();
+  if(output)
+    {
+    output->CopyTypeSpecificInformation( this->GetInput() );
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -191,7 +241,20 @@ void vtkDataSetToDataSetFilter::ComputeInputUpdateExtents(vtkDataObject *output)
 //----------------------------------------------------------------------------
 vtkDataSet *vtkDataSetToDataSetFilter::GetOutput(int idx)
 {
-  return (vtkDataSet *) this->vtkDataSetSource::GetOutput(idx); 
+  return this->vtkDataSetSource::GetOutput(idx);
+}
+
+//----------------------------------------------------------------------------
+int
+vtkDataSetToDataSetFilter
+::FillInputPortInformation(int port, vtkInformation* info)
+{
+  if(!this->Superclass::FillInputPortInformation(port, info))
+    {
+    return 0;
+    }
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
 }
 
 //----------------------------------------------------------------------------
