@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkDataSetToUnstructuredGridFilter.h,v $
+  Module:    vtkDataSetToUnstructuredGridFilter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -32,7 +32,7 @@ class vtkDataSet;
 class VTK_FILTERING_EXPORT vtkDataSetToUnstructuredGridFilter : public vtkUnstructuredGridSource
 {
 public:
-  vtkTypeRevisionMacro(vtkDataSetToUnstructuredGridFilter,vtkUnstructuredGridSource);
+  vtkTypeMacro(vtkDataSetToUnstructuredGridFilter,vtkUnstructuredGridSource);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -41,8 +41,10 @@ public:
   vtkDataSet *GetInput();
   
 protected:
-  vtkDataSetToUnstructuredGridFilter() {this->NumberOfRequiredInputs = 1;};
-  ~vtkDataSetToUnstructuredGridFilter() {};  
+  vtkDataSetToUnstructuredGridFilter();
+  ~vtkDataSetToUnstructuredGridFilter();
+
+  virtual int FillInputPortInformation(int, vtkInformation*);
 
 private:
   vtkDataSetToUnstructuredGridFilter(const vtkDataSetToUnstructuredGridFilter&);  // Not implemented.
