@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageToStructuredPoints.cxx,v $
+  Module:    vtkImageToStructuredPoints.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,23 +15,24 @@
 #include "vtkImageToStructuredPoints.h"
 
 #include "vtkCellData.h"
+#include "vtkDataArray.h"
 #include "vtkFieldData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkStructuredPoints.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImageToStructuredPoints, "$Revision: 1.54 $");
 vtkStandardNewMacro(vtkImageToStructuredPoints);
 
 //----------------------------------------------------------------------------
 vtkImageToStructuredPoints::vtkImageToStructuredPoints()
 {
+  this->SetNumberOfInputPorts(2);
   this->Translate[0] = this->Translate[1] = this->Translate[2] = 0;
-  this->NumberOfRequiredInputs = 1;
-  this->SetNthOutput(0,vtkStructuredPoints::New());
-  this->Outputs[0]->Delete();
 }
 
 //----------------------------------------------------------------------------
@@ -47,57 +48,38 @@ void vtkImageToStructuredPoints::PrintSelf(ostream& os, vtkIndent indent)
 }
 
 //----------------------------------------------------------------------------
-vtkStructuredPoints *vtkImageToStructuredPoints::GetOutput()
+vtkStructuredPoints* vtkImageToStructuredPoints::GetStructuredPointsOutput()
 {
-  if (this->NumberOfOutputs < 1)
-    {
-    return NULL;
-    }
-  
-  return (vtkStructuredPoints *)(this->Outputs[0]);
+  return vtkStructuredPoints::SafeDownCast(this->GetOutputDataObject(0));
 }
 
-//----------------------------------------------------------------------------
-void vtkImageToStructuredPoints::SetInput(vtkImageData *input)
-{
-  this->vtkProcessObject::SetNthInput(0, input);
-}
-
-//----------------------------------------------------------------------------
-vtkImageData *vtkImageToStructuredPoints::GetInput()
-{
-  if (this->NumberOfInputs < 1)
-    {
-    return NULL;
-    }
-  
-  return (vtkImageData *)(this->Inputs[0]);
-}
-
-
 //----------------------------------------------------------------------------
 void vtkImageToStructuredPoints::SetVectorInput(vtkImageData *input)
 {
-  this->vtkProcessObject::SetNthInput(1, input);
+  this->SetInput(1, input);
 }
 
 //----------------------------------------------------------------------------
 vtkImageData *vtkImageToStructuredPoints::GetVectorInput()
 {
-  if (this->NumberOfInputs < 2)
+  if (this->GetNumberOfInputConnections(1) < 1)
     {
     return NULL;
     }
   
-  return (vtkImageData *)(this->Inputs[1]);
+  return vtkImageData::SafeDownCast(this->GetExecutive()->GetInputData(1, 0));
 }
 
-
-
-
 //----------------------------------------------------------------------------
-void vtkImageToStructuredPoints::Execute()
+int vtkImageToStructuredPoints::RequestData(
+  vtkInformation *,
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *vectorInfo = inputVector[1]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
   int uExtent[6];
   int *wExtent;
 
@@ -105,19 +87,22 @@ void vtkImageToStructuredPoints::Execute()
   int maxX = 0;
   int maxY = 0;
   int maxZ = 0;;
-  int inIncX, inIncY, inIncZ, rowLength;
+  vtkIdType inIncX, inIncY, inIncZ;
+  int rowLength;
   unsigned char *inPtr1, *inPtr, *outPtr;
-  vtkStructuredPoints *output = this->GetOutput();
-  vtkImageData *data = this->GetInput();
-  vtkImageData *vData = this->GetVectorInput();
-  
-  if (!data && !vData)
+  vtkStructuredPoints *output = vtkStructuredPoints::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkImageData *data = vtkImageData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkImageData *vData = 0;
+  if (vectorInfo)
     {
-    vtkErrorMacro("Unable to generate data!");
-    return;
+    vData = vtkImageData::SafeDownCast(
+      vectorInfo->Get(vtkDataObject::DATA_OBJECT()));
     }
 
-  output->GetUpdateExtent(uExtent);
+  outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(),
+               uExtent);
   output->SetExtent(uExtent);
 
   uExtent[0] += this->Translate[0];
@@ -151,9 +136,17 @@ void vtkImageToStructuredPoints::Execute()
       }
     else
       {
-      inPtr = (unsigned char *) data->GetScalarPointerForExtent(uExtent);
-      outPtr = (unsigned char *) output->GetScalarPointer();
-      
+      inPtr =
+        static_cast<unsigned char *>(data->GetScalarPointerForExtent(uExtent));
+      outPtr = static_cast<unsigned char *>(output->GetScalarPointer());
+
+      // Make sure there are data.
+      if(!inPtr || !outPtr)
+        {
+        output->Initialize();
+        return 1;
+        }
+
       // Get increments to march through data 
       data->GetIncrements(inIncX, inIncY, inIncZ);
       
@@ -193,8 +186,16 @@ void vtkImageToStructuredPoints::Execute()
     else
       {
       vtkDataArray *fv = vtkDataArray::CreateDataArray(vData->GetScalarType());
-      float *inPtr2 = (float *)(vData->GetScalarPointerForExtent(uExtent));
-      
+      float *inPtr2 =
+        static_cast<float *>(vData->GetScalarPointerForExtent(uExtent));
+
+      // Make sure there are data.
+      if(!inPtr2)
+        {
+        output->Initialize();
+        return 1;
+        }
+
       fv->SetNumberOfComponents(3);
       fv->SetNumberOfTuples((maxZ+1)*(maxY+1)*(maxX+1));
       vData->GetContinuousIncrements(uExtent, inIncX, inIncY, inIncZ);
@@ -220,46 +221,44 @@ void vtkImageToStructuredPoints::Execute()
       fv->Delete();
       }
     }
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
 // Copy WholeExtent, Spacing and Origin.
-void vtkImageToStructuredPoints::ExecuteInformation()
+int vtkImageToStructuredPoints::RequestInformation (
+  vtkInformation * vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkImageData *input = this->GetInput();
-  vtkImageData *vInput = this->GetVectorInput();
-  vtkStructuredPoints *output = this->GetOutput();
+  // get the info objects
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *vInfo = inputVector[1]->GetInformationObject(0);
+  
   int whole[6], *tmp;
   double *spacing, origin[3];
   
-  if (output == NULL)
+  vtkInformation *inScalarInfo = vtkDataObject::GetActiveFieldInformation(inInfo, 
+    vtkDataObject::FIELD_ASSOCIATION_POINTS, vtkDataSetAttributes::SCALARS);
+  if (!inScalarInfo)
     {
-    return;
-    }
-  
-  if (input)
-    {
-    output->SetScalarType(input->GetScalarType());
-    output->SetNumberOfScalarComponents(input->GetNumberOfScalarComponents());
-    input->GetWholeExtent(whole);    
-    spacing = input->GetSpacing();
-    input->GetOrigin(origin);
-    }
-  else if (vInput)
-    {
-    whole[0] = whole[2] = whole[4] = -VTK_LARGE_INTEGER;
-    whole[1] = whole[3] = whole[5] = VTK_LARGE_INTEGER;
-    spacing = vInput->GetSpacing();
-    vInput->GetOrigin(origin);
-    }
-  else
-    {
-    return;
+    vtkErrorMacro("Missing scalar field on input information!");
+    return 0;
     }
+  vtkDataObject::SetPointDataActiveScalarInfo(outInfo, 
+    inScalarInfo->Get( vtkDataObject::FIELD_ARRAY_TYPE() ),
+    inScalarInfo->Get( vtkDataObject::FIELD_NUMBER_OF_COMPONENTS() ) );
+
+  inInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),whole);    
+  spacing = inInfo->Get(vtkDataObject::SPACING());
+  inInfo->Get(vtkDataObject::ORIGIN(), origin);
+
   // intersections for whole extent
-  if (vInput)
+  if (vInfo)
     {
-    tmp = vInput->GetWholeExtent();
+    tmp = vInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT());
     if (tmp[0] > whole[0]) {whole[0] = tmp[0];}
     if (tmp[2] > whole[2]) {whole[2] = tmp[2];}
     if (tmp[4] > whole[4]) {whole[4] = tmp[4];}
@@ -281,27 +280,28 @@ void vtkImageToStructuredPoints::ExecuteInformation()
   whole[5] -= whole[4];
   whole[0] = whole[2] = whole[4] = 0;
   
-  output->SetWholeExtent(whole);
+  outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),whole,6);
   // Now should Origin and Spacing really be part of information?
   // How about xyx arrays in RectilinearGrid of Points in StructuredGrid?
-  output->SetOrigin(origin);
-  output->SetSpacing(spacing);
+  outInfo->Set(vtkDataObject::ORIGIN(),origin,3);
+  outInfo->Set(vtkDataObject::SPACING(),spacing,3);
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
-vtkStructuredPoints *vtkImageToStructuredPoints::GetOutput(int idx)
+int vtkImageToStructuredPoints::RequestUpdateExtent(
+  vtkInformation *,
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  return (vtkStructuredPoints *) this->vtkSource::GetOutput(idx); 
-}
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *vInfo = inputVector[1]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
 
-//----------------------------------------------------------------------------
-void vtkImageToStructuredPoints::ComputeInputUpdateExtents(vtkDataObject *data)
-{
-  vtkStructuredPoints *output = (vtkStructuredPoints*)data;
   int ext[6];
-  vtkImageData *input;
 
-  output->GetUpdateExtent(ext);
+  outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(), ext);
   ext[0] += this->Translate[0];
   ext[1] += this->Translate[0];
   ext[2] += this->Translate[1];
@@ -309,19 +309,39 @@ void vtkImageToStructuredPoints::ComputeInputUpdateExtents(vtkDataObject *data)
   ext[4] += this->Translate[2];
   ext[5] += this->Translate[2];
   
-  input = this->GetInput();
-  if (input)
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(), ext, 6);
+
+  if (vInfo)
     {
-    input->SetUpdateExtent(ext);
+    vInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(), ext, 6);
     }
 
-  input = this->GetVectorInput();
-  if (input)
-    {
-    input->SetUpdateExtent(ext);
-    }
+  return 1;
 }
 
+//----------------------------------------------------------------------------
+int vtkImageToStructuredPoints::FillOutputPortInformation(int port,
+                                                          vtkInformation* info)
+{
+  if(!this->Superclass::FillOutputPortInformation(port, info))
+    {
+    return 0;
+    }
+  info->Set(vtkDataObject::DATA_TYPE_NAME(), "vtkStructuredPoints");
+  return 1;
+}
 
-
-
+//----------------------------------------------------------------------------
+int vtkImageToStructuredPoints::FillInputPortInformation(int port,
+                                                         vtkInformation *info)
+{
+  if (!this->Superclass::FillInputPortInformation(port, info))
+    {
+    return 0;
+    }
+  if (port == 1)
+    {
+    info->Set(vtkAlgorithm::INPUT_IS_OPTIONAL(), 1);
+    }
+  return 1;
+}
