@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImplicitBoolean.cxx,v $
+  Module:    vtkImplicitBoolean.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,7 +19,6 @@
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImplicitBoolean, "$Revision: 1.35 $");
 vtkStandardNewMacro(vtkImplicitBoolean);
 
 // Construct with union operation.
@@ -40,8 +39,9 @@ unsigned long int vtkImplicitBoolean::GetMTime()
   unsigned long int mtime = this->vtkImplicitFunction::GetMTime();
   vtkImplicitFunction *f;
 
-  for (this->FunctionList->InitTraversal(); 
-       (f=this->FunctionList->GetNextItem()); )
+  vtkCollectionSimpleIterator sit;
+  for (this->FunctionList->InitTraversal(sit); 
+       (f=this->FunctionList->GetNextImplicitFunction(sit)); )
     {
     fMtime = f->GetMTime();
     if ( fMtime > mtime )
@@ -84,10 +84,11 @@ double vtkImplicitBoolean::EvaluateFunction(double x[3])
     return value;
     }
   
+  vtkCollectionSimpleIterator sit;
   if ( this->OperationType == VTK_UNION )
     { //take minimum value
-    for (value = VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(); 
-         (f=this->FunctionList->GetNextItem()); )
+    for (value = VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(sit); 
+         (f=this->FunctionList->GetNextImplicitFunction(sit)); )
       {
       if ( (v=f->FunctionValue(x)) < value )
         {
@@ -98,8 +99,8 @@ double vtkImplicitBoolean::EvaluateFunction(double x[3])
 
   else if ( this->OperationType == VTK_INTERSECTION )
     { //take maximum value
-    for (value=-VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(); 
-    (f=this->FunctionList->GetNextItem()); )
+    for (value=-VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(sit); 
+    (f=this->FunctionList->GetNextImplicitFunction(sit)); )
       {
       if ( (v=f->FunctionValue(x)) > value )
         {
@@ -110,8 +111,8 @@ double vtkImplicitBoolean::EvaluateFunction(double x[3])
 
   else if ( this->OperationType == VTK_UNION_OF_MAGNITUDES )
     { //take minimum absolute value
-    for (value = VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(); 
-    (f=this->FunctionList->GetNextItem()); )
+    for (value = VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(sit); 
+    (f=this->FunctionList->GetNextImplicitFunction(sit)); )
       {
       if ( (v=fabs(f->FunctionValue(x))) < value )
         {
@@ -123,14 +124,14 @@ double vtkImplicitBoolean::EvaluateFunction(double x[3])
   else //difference
     {
     vtkImplicitFunction *firstF;
-    this->FunctionList->InitTraversal();
-    if ( (firstF = this->FunctionList->GetNextItem()) != NULL )
+    this->FunctionList->InitTraversal(sit);
+    if ( (firstF = this->FunctionList->GetNextImplicitFunction(sit)) != NULL )
       {
       value = firstF->FunctionValue(x);
       }
 
-    for (this->FunctionList->InitTraversal(); 
-    (f=this->FunctionList->GetNextItem()); )
+    for (this->FunctionList->InitTraversal(sit); 
+    (f=this->FunctionList->GetNextImplicitFunction(sit)); )
       {
       if ( f != firstF )
         {
@@ -151,6 +152,7 @@ void vtkImplicitBoolean::EvaluateGradient(double x[3], double g[3])
   double value = 0;
   double v;
   vtkImplicitFunction *f;
+  vtkCollectionSimpleIterator sit;
 
   if (this->FunctionList->GetNumberOfItems() == 0)
     {
@@ -160,8 +162,8 @@ void vtkImplicitBoolean::EvaluateGradient(double x[3], double g[3])
 
   if ( this->OperationType == VTK_UNION )
     { //take minimum value
-    for (value = VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(); 
-    (f=this->FunctionList->GetNextItem()); )
+    for (value = VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(sit); 
+    (f=this->FunctionList->GetNextImplicitFunction(sit)); )
       {
       if ( (v=f->FunctionValue(x)) < value )
         {
@@ -173,8 +175,8 @@ void vtkImplicitBoolean::EvaluateGradient(double x[3], double g[3])
 
   else if ( this->OperationType == VTK_INTERSECTION )
     { //take maximum value
-    for (value=-VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(); 
-    (f=this->FunctionList->GetNextItem()); )
+    for (value=-VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(sit); 
+    (f=this->FunctionList->GetNextImplicitFunction(sit)); )
       {
       if ( (v=f->FunctionValue(x)) > value ) 
         {
@@ -186,8 +188,8 @@ void vtkImplicitBoolean::EvaluateGradient(double x[3], double g[3])
 
   if ( this->OperationType == VTK_UNION_OF_MAGNITUDES )
     { //take minimum value
-    for (value = VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(); 
-    (f=this->FunctionList->GetNextItem()); )
+    for (value = VTK_DOUBLE_MAX, this->FunctionList->InitTraversal(sit); 
+    (f=this->FunctionList->GetNextImplicitFunction(sit)); )
       {
       if ( (v=fabs(f->FunctionValue(x))) < value )
         {
@@ -201,16 +203,16 @@ void vtkImplicitBoolean::EvaluateGradient(double x[3], double g[3])
     {
     double gTemp[3];
     vtkImplicitFunction *firstF;
-    this->FunctionList->InitTraversal();
-    if ( (firstF = this->FunctionList->GetNextItem()) != NULL )
+    this->FunctionList->InitTraversal(sit);
+    if ( (firstF = this->FunctionList->GetNextImplicitFunction(sit)) != NULL )
       {
       value = firstF->FunctionValue(x);
       firstF->FunctionGradient(x,gTemp); 
       g[0] = -1.0*gTemp[0]; g[1] = -1.0*gTemp[1]; g[2] = -1.0*gTemp[2];
       }
 
-    for (this->FunctionList->InitTraversal(); 
-    (f=this->FunctionList->GetNextItem()); )
+    for (this->FunctionList->InitTraversal(sit); 
+    (f=this->FunctionList->GetNextImplicitFunction(sit)); )
       {
       if ( f != firstF )
         {
