@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkKochanekSpline.cxx,v $
+  Module:    vtkKochanekSpline.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,9 +17,9 @@
 #include "vtkObjectFactory.h"
 #include "vtkPiecewiseFunction.h"
 
-vtkCxxRevisionMacro(vtkKochanekSpline, "$Revision: 1.23 $");
 vtkStandardNewMacro(vtkKochanekSpline);
 
+//----------------------------------------------------------------------------
 // Construct a KochanekSpline wth the following defaults:
 // DefaultBias = 0,
 // DefaultTension = 0,
@@ -31,26 +31,26 @@ vtkKochanekSpline::vtkKochanekSpline ()
   this->DefaultContinuity = 0.0;
 }
 
+//----------------------------------------------------------------------------
 // Evaluate a 1D Spline
 double vtkKochanekSpline::Evaluate (double t)
 {
-  int i, index = 0;
-  int size = this->PiecewiseFunction->GetSize ();
+  int index = 0;
   double *intervals;
   double *coefficients;
 
-  // make sure we have at least 2 points
-  if (size < 2)
-    {
-    vtkErrorMacro("Cannot evaluate a spline with less than 2 points. # of points is: " << size);
-    return 0.0;
-    }
-
   // check to see if we need to recompute the spline
   if (this->ComputeTime < this->GetMTime ())
     {
     this->Compute ();
-    }   
+    }
+
+  // make sure we have at least 2 points
+  int size = this->PiecewiseFunction->GetSize ();
+  if (size < 2)
+    {
+    return 0.0;
+    }
 
   intervals = this->Intervals;
   coefficients = this->Coefficients;
@@ -71,14 +71,7 @@ double vtkKochanekSpline::Evaluate (double t)
     }
 
   // find pointer to cubic spline coefficient
-  for (i = 1; i < size; i++)
-    {
-    index = i - 1;
-    if (t < intervals[i])
-      {
-      break;
-      }
-    }
+  index = this->FindIndex(size,t);
 
   // calculate offset within interval
   t = (t - intervals[index]) / (intervals[index+1] - intervals[index]);
@@ -90,6 +83,7 @@ double vtkKochanekSpline::Evaluate (double t)
                       + *(coefficients + index * 4));
 }
 
+//----------------------------------------------------------------------------
 // Compute Kochanek Spline coefficients.
 void vtkKochanekSpline::Compute ()
 {
@@ -99,9 +93,18 @@ void vtkKochanekSpline::Compute ()
   int size;
   int i;
 
+  // Make sure the function is up to date.
+  this->PiecewiseFunction->Update();
+
   // get the size of the independent variables
   size = this->PiecewiseFunction->GetSize ();
 
+  if(size < 2)
+    {
+    vtkErrorMacro("Spline requires at least 2 points. # of points is: " <<size);
+    return;
+    }
+
   if ( !this->Closed )
     {
     // copy the independent variables
@@ -113,7 +116,7 @@ void vtkKochanekSpline::Compute ()
     ts = this->PiecewiseFunction->GetDataPointer ();  
     for (i = 0; i < size; i++)
       {
-      this->Intervals[i] = *(ts + 2 * i);    
+      this->Intervals[i] = *(ts + 2*i);    
       }
 
     // allocate memory for coefficients
@@ -121,7 +124,7 @@ void vtkKochanekSpline::Compute ()
       {
       delete [] this->Coefficients;
       }
-    this->Coefficients = new double [4 * size];
+    this->Coefficients = new double [4*size];
 
     // allocate memory for dependent variables
     dependent = new double [size];
@@ -150,7 +153,14 @@ void vtkKochanekSpline::Compute ()
       {
       this->Intervals[i] = *(ts + 2 * i);    
       }
-    this->Intervals[size-1] = this->Intervals[size-2] + 1.0;
+    if ( this->ParametricRange[0] != this->ParametricRange[1] )
+      {
+      this->Intervals[size-1] = this->ParametricRange[1];
+      }
+    else
+      {
+      this->Intervals[size-1] = this->Intervals[size-2] + 1.0;
+      }
 
     // allocate memory for coefficients
     if (this->Coefficients)
@@ -191,6 +201,7 @@ void vtkKochanekSpline::Compute ()
 
 #define VTK_EPSILON .0001
 
+//----------------------------------------------------------------------------
 // Compute the coefficients for a 1D spline
 void vtkKochanekSpline::Fit1D (int size, double *x, double *y,
                                double tension, double bias, double continuity,
@@ -206,20 +217,6 @@ void vtkKochanekSpline::Fit1D (int size, double *x, double *y,
   int           N;              /* top point number             */
   int           i;
 
-  if (size == 2)
-    {
-    // two points, set coefficients for a straight line
-    coefficients[0][3] = 0.0;
-    coefficients[1][3] = 0.0;
-    coefficients[0][2] = 0.0;
-    coefficients[1][2] = 0.0;
-    coefficients[0][1] = (y[1] - y[0]) / (x[1] - x[0]);
-    coefficients[1][1] = coefficients[0][1];
-    coefficients[0][0] = y[0];
-    coefficients[1][0] = y[1];
-    return;
-    }
-
   N = size - 1;
 
   for (i=1; i < N; i++)
@@ -248,6 +245,9 @@ void vtkKochanekSpline::Fit1D (int size, double *x, double *y,
   // Calculate the deriviatives at the end points
   coefficients[0][0] = y[0];
   coefficients[N][0] = y[N];
+  coefficients[N][1] = 0.0;
+  coefficients[N][2] = 0.0;
+  coefficients[N][3] = 0.0;
 
   if ( this->Closed ) //the curve is continuous and closed at P0=Pn
     {
@@ -263,8 +263,8 @@ void vtkKochanekSpline::Fit1D (int size, double *x, double *y,
     // adjust deriviatives for non uniform spacing between nodes
     n1  = x[1] - x[0];
     n0  = x[N] - x[N-1];
-    ds *= (2 * n1 / (n0 + n1));
-    dd *= (2 * n0 / (n0 + n1));
+    ds *= (2 * n0 / (n0 + n1));
+    dd *= (2 * n1 / (n0 + n1));
 
     coefficients[0][1] = dd;
     coefficients[0][2] = ds;
@@ -367,6 +367,7 @@ void vtkKochanekSpline::Fit1D (int size, double *x, double *y,
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkKochanekSpline::DeepCopy(vtkSpline *s)
 {
   vtkKochanekSpline *spline = vtkKochanekSpline::SafeDownCast(s);
@@ -382,6 +383,7 @@ void vtkKochanekSpline::DeepCopy(vtkSpline *s)
   this->vtkSpline::DeepCopy(s);
 }
 
+//----------------------------------------------------------------------------
 void vtkKochanekSpline::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
