@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPointSetToPointSetFilter.cxx,v $
+  Module:    vtkPointSetToPointSetFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,13 +17,13 @@
 #include "vtkStructuredGrid.h"
 #include "vtkUnstructuredGrid.h"
 
-vtkCxxRevisionMacro(vtkPointSetToPointSetFilter, "$Revision: 1.60 $");
 
 //----------------------------------------------------------------------------
 // Construct object.
 vtkPointSetToPointSetFilter::vtkPointSetToPointSetFilter()
 {
   this->NumberOfRequiredInputs = 1;
+  this->SetNumberOfInputPorts(1);
 }
 
 //----------------------------------------------------------------------------
@@ -66,7 +66,7 @@ vtkPointSet *vtkPointSetToPointSetFilter::GetInput()
     return NULL;
     }
   
-  return (vtkPointSet *)(this->Inputs[0]);
+  return static_cast<vtkPointSet *>(this->Inputs[0]);
 }
 
 //----------------------------------------------------------------------------
@@ -95,7 +95,7 @@ vtkPolyData *vtkPointSetToPointSetFilter::GetPolyDataOutput()
     }
   if (ds->GetDataObjectType() == VTK_POLY_DATA)
     {
-    return (vtkPolyData *)ds;
+    return static_cast<vtkPolyData *>(ds);
     }
   return NULL;
 }
@@ -111,7 +111,7 @@ vtkStructuredGrid *vtkPointSetToPointSetFilter::GetStructuredGridOutput()
     }
   if (ds->GetDataObjectType() == VTK_STRUCTURED_GRID)
     {
-    return (vtkStructuredGrid *)ds;
+    return static_cast<vtkStructuredGrid *>(ds);
     }
   return NULL;
 }
@@ -127,7 +127,7 @@ vtkUnstructuredGrid *vtkPointSetToPointSetFilter::GetUnstructuredGridOutput()
     }
   if (ds->GetDataObjectType() == VTK_UNSTRUCTURED_GRID)
     {
-    return (vtkUnstructuredGrid *)ds;
+    return static_cast<vtkUnstructuredGrid *>(ds);
     }
   return NULL;
 }
