@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPolyDataToPolyDataFilter.cxx,v $
+  Module:    vtkPolyDataToPolyDataFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,15 +14,17 @@
 =========================================================================*/
 #include "vtkPolyDataToPolyDataFilter.h"
 
+#include "vtkInformation.h"
 #include "vtkPolyData.h"
 
-vtkCxxRevisionMacro(vtkPolyDataToPolyDataFilter, "$Revision: 1.17 $");
 
 //----------------------------------------------------------------------------
 vtkPolyDataToPolyDataFilter::vtkPolyDataToPolyDataFilter() 
 {
   this->NumberOfRequiredInputs = 1;
+  this->SetNumberOfInputPorts(1);
 }
+
 //----------------------------------------------------------------------------
 // Specify the input data or filter.
 void vtkPolyDataToPolyDataFilter::SetInput(vtkPolyData *input)
@@ -39,7 +41,20 @@ vtkPolyData *vtkPolyDataToPolyDataFilter::GetInput()
     return NULL;
     }
   
-  return (vtkPolyData *)(this->Inputs[0]);
+  return static_cast<vtkPolyData *>(this->Inputs[0]);
+}
+
+//----------------------------------------------------------------------------
+int vtkPolyDataToPolyDataFilter::FillInputPortInformation(int port,
+                                                          vtkInformation* info)
+{
+  if(!this->Superclass::FillInputPortInformation(port, info))
+    {
+    return 0;
+    }
+  //info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkPolyData"); HACK
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
 }
 
 //----------------------------------------------------------------------------
