@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkStructuredGridSource.cxx,v $
+  Module:    vtkStructuredGridSource.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,14 +14,17 @@
 =========================================================================*/
 #include "vtkStructuredGridSource.h"
 
+#include "vtkInformation.h"
 #include "vtkObjectFactory.h"
 #include "vtkStructuredGrid.h"
 
-vtkCxxRevisionMacro(vtkStructuredGridSource, "$Revision: 1.28 $");
 
 //----------------------------------------------------------------------------
 vtkStructuredGridSource::vtkStructuredGridSource()
 {
+  // A source has no inputs by default.
+  this->SetNumberOfInputPorts(0);
+
   this->vtkSource::SetNthOutput(0, vtkStructuredGrid::New());
   // Releasing data for pipeline parallism.
   // Filters will know it is empty. 
@@ -37,7 +40,7 @@ vtkStructuredGrid *vtkStructuredGridSource::GetOutput()
     return NULL;
     }
   
-  return (vtkStructuredGrid *)(this->Outputs[0]);
+  return static_cast<vtkStructuredGrid *>(this->Outputs[0]);
 }
 
 //----------------------------------------------------------------------------
@@ -52,6 +55,18 @@ void vtkStructuredGridSource::SetOutput(vtkStructuredGrid *output)
   this->vtkSource::SetNthOutput(0, output);
 }
 
+//----------------------------------------------------------------------------
+int vtkStructuredGridSource::FillOutputPortInformation(int port,
+                                                       vtkInformation* info)
+{
+  if(!this->Superclass::FillOutputPortInformation(port, info))
+    {
+    return 0;
+    }
+  info->Set(vtkDataObject::DATA_TYPE_NAME(), "vtkStructuredGrid");
+  return 1;
+}
+
 //----------------------------------------------------------------------------
 void vtkStructuredGridSource::PrintSelf(ostream& os, vtkIndent indent)
 {
