@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkStructuredGridSource.h,v $
+  Module:    vtkStructuredGridSource.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -30,7 +30,7 @@ class vtkStructuredGrid;
 class VTK_FILTERING_EXPORT vtkStructuredGridSource : public vtkSource
 {
 public:
-  vtkTypeRevisionMacro(vtkStructuredGridSource,vtkSource);
+  vtkTypeMacro(vtkStructuredGridSource,vtkSource);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -43,6 +43,7 @@ protected:
   vtkStructuredGridSource();
   ~vtkStructuredGridSource() {};
 
+  virtual int FillOutputPortInformation(int, vtkInformation*);
 private:
   vtkStructuredGridSource(const vtkStructuredGridSource&);  // Not implemented.
   void operator=(const vtkStructuredGridSource&);  // Not implemented.
