@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkStructuredPointsSource.cxx,v $
+  Module:    vtkStructuredPointsSource.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,15 +15,18 @@
 #include "vtkStructuredPointsSource.h"
 
 #include "vtkDataArray.h"
+#include "vtkInformation.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkStructuredPoints.h"
 
-vtkCxxRevisionMacro(vtkStructuredPointsSource, "$Revision: 1.36 $");
 
 //----------------------------------------------------------------------------
 vtkStructuredPointsSource::vtkStructuredPointsSource()
 {
+  // A source has no inputs by default.
+  this->SetNumberOfInputPorts(0);
+
   this->SetOutput(vtkStructuredPoints::New());
   // Releasing data for pipeline parallism.
   // Filters will know it is empty. 
@@ -45,13 +48,13 @@ vtkStructuredPoints *vtkStructuredPointsSource::GetOutput()
     return NULL;
     }
   
-  return (vtkStructuredPoints *)(this->Outputs[0]);
+  return static_cast<vtkStructuredPoints *>(this->Outputs[0]);
 }
 
 //----------------------------------------------------------------------------
 vtkStructuredPoints *vtkStructuredPointsSource::GetOutput(int idx)
 {
-  return (vtkStructuredPoints *) this->vtkSource::GetOutput(idx); 
+  return static_cast<vtkStructuredPoints *>(this->vtkSource::GetOutput(idx));
 }
 
 //----------------------------------------------------------------------------
@@ -74,6 +77,18 @@ void vtkStructuredPointsSource::ExecuteInformation()
   output->SetWholeExtent(output->GetExtent());
 }
 
+//----------------------------------------------------------------------------
+int vtkStructuredPointsSource::FillOutputPortInformation(int port,
+                                                         vtkInformation* info)
+{
+  if(!this->Superclass::FillOutputPortInformation(port, info))
+    {
+    return 0;
+    }
+  info->Set(vtkDataObject::DATA_TYPE_NAME(), "vtkStructuredPoints");
+  return 1;
+}
+
 //----------------------------------------------------------------------------
 void vtkStructuredPointsSource::PrintSelf(ostream& os, vtkIndent indent)
 {
