@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkStructuredPointsSource.h,v $
+  Module:    vtkStructuredPointsSource.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -30,7 +30,7 @@ class vtkStructuredPoints;
 class VTK_FILTERING_EXPORT vtkStructuredPointsSource : public vtkSource
 {
 public:
-  vtkTypeRevisionMacro(vtkStructuredPointsSource,vtkSource);
+  vtkTypeMacro(vtkStructuredPointsSource,vtkSource);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -47,6 +47,7 @@ protected:
   // structured points sources compute information
   void ExecuteInformation();
 
+  virtual int FillOutputPortInformation(int, vtkInformation*);
 private:
   vtkStructuredPointsSource(const vtkStructuredPointsSource&);  // Not implemented.
   void operator=(const vtkStructuredPointsSource&);  // Not implemented.
