@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkStructuredPointsToStructuredPointsFilter.cxx,v $
+  Module:    vtkStructuredPointsToStructuredPointsFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,11 +15,23 @@
 #include "vtkStructuredPointsToStructuredPointsFilter.h"
 
 #include "vtkImageData.h"
+#include "vtkInformation.h"
 #include "vtkStructuredPoints.h"
 
-vtkCxxRevisionMacro(vtkStructuredPointsToStructuredPointsFilter, "$Revision: 1.29 $");
 
-//----------------------------------------------------------------------------
+// ----------------------------------------------------------------------------
+vtkStructuredPointsToStructuredPointsFilter::vtkStructuredPointsToStructuredPointsFilter()
+{
+  this->NumberOfRequiredInputs = 1;
+  this->SetNumberOfInputPorts(1);
+}
+
+// ----------------------------------------------------------------------------
+vtkStructuredPointsToStructuredPointsFilter::~vtkStructuredPointsToStructuredPointsFilter()
+{
+}
+
+// ----------------------------------------------------------------------------
 // Specify the input data or filter.
 void vtkStructuredPointsToStructuredPointsFilter::SetInput(
                                                    vtkImageData *input)
@@ -27,7 +39,7 @@ void vtkStructuredPointsToStructuredPointsFilter::SetInput(
   this->vtkProcessObject::SetNthInput(0, input);
 }
 
-//----------------------------------------------------------------------------
+// ----------------------------------------------------------------------------
 // Specify the input data or filter.
 vtkImageData *vtkStructuredPointsToStructuredPointsFilter::GetInput()
 {
@@ -36,11 +48,11 @@ vtkImageData *vtkStructuredPointsToStructuredPointsFilter::GetInput()
     return NULL;
     }
   
-  return (vtkStructuredPoints *)(this->Inputs[0]);
+  return static_cast<vtkStructuredPoints *>(this->Inputs[0]);
 }
 
 
-//----------------------------------------------------------------------------
+// ----------------------------------------------------------------------------
 // Copy WholeExtent, Spacing and Origin.
 void vtkStructuredPointsToStructuredPointsFilter::ExecuteInformation()
 {
@@ -58,7 +70,7 @@ void vtkStructuredPointsToStructuredPointsFilter::ExecuteInformation()
 }
 
 
-//----------------------------------------------------------------------------
+// ----------------------------------------------------------------------------
 void vtkStructuredPointsToStructuredPointsFilter::ComputeInputUpdateExtents( 
                                                         vtkDataObject *output)
 {
@@ -68,7 +80,20 @@ void vtkStructuredPointsToStructuredPointsFilter::ComputeInputUpdateExtents(
   this->GetInput()->RequestExactExtentOn();
 }
 
-//----------------------------------------------------------------------------
+// ----------------------------------------------------------------------------
+int
+vtkStructuredPointsToStructuredPointsFilter
+::FillInputPortInformation(int port, vtkInformation* info)
+{
+  if(!this->Superclass::FillInputPortInformation(port, info))
+    {
+    return 0;
+    }
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkImageData");
+  return 1;
+}
+
+// ----------------------------------------------------------------------------
 void vtkStructuredPointsToStructuredPointsFilter::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
