@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: RGrid.cxx,v $
+  Module:    RGrid.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -28,8 +28,6 @@
 
 int RGrid( int argc, char *argv[] )
 {
-  vtkDebugLeaks::PromptUserOff();
-
   int i;
   static float x[47]={-1.22396, -1.17188, -1.11979, -1.06771, -1.01562, -0.963542, 
                       -0.911458, -0.859375, -0.807292, -0.755208, -0.703125, -0.651042, 
@@ -80,7 +78,7 @@ int RGrid( int argc, char *argv[] )
     plane->SetExtent(0,46, 16,16, 0,43);
 
   vtkPolyDataMapper *rgridMapper = vtkPolyDataMapper::New();
-      rgridMapper->SetInput(plane->GetOutput());
+      rgridMapper->SetInputConnection(plane->GetOutputPort());
 
   vtkActor *wireActor = vtkActor::New();
       wireActor->SetMapper(rgridMapper);
@@ -89,6 +87,7 @@ int RGrid( int argc, char *argv[] )
 
   renderer->AddActor(wireActor);
   renderer->SetBackground(1,1,1);
+  renderer->ResetCamera();
   renderer->GetActiveCamera()->Elevation(60.0);
   renderer->GetActiveCamera()->Azimuth(30.0);
   renderer->GetActiveCamera()->Zoom(1.0);
