@@ -12,18 +12,20 @@ vtkRenderWindowInteractor iren
 iren SetRenderWindow renWin
 
 vtkGenericEnSightReader reader
-reader SetCaseFileName $VTK_DATA_ROOT/Data/EnSight/blow1_bin.case
+# Make sure all algorithms use the composite data pipeline
+vtkCompositeDataPipeline cdp
+reader SetDefaultExecutivePrototype cdp
+reader SetCaseFileName "$VTK_DATA_ROOT/Data/EnSight/blow1_bin.case"
 reader SetTimeValue 1
-reader Update
 
 vtkGeometryFilter geom
-geom SetInput [reader GetOutput]
+geom SetInputConnection [reader GetOutputPort]
 
-vtkPolyDataMapper mapper
-mapper SetInput [geom GetOutput]
+vtkHierarchicalPolyDataMapper mapper
+mapper SetInputConnection [geom GetOutputPort]
 mapper SetColorModeToMapScalars
 mapper SetScalarModeToUsePointFieldData
-mapper ColorByArrayComponent displacement 0
+mapper ColorByArrayComponent "displacement" 0
 mapper SetScalarRange 0 2.08
 
 vtkActor actor
@@ -47,3 +49,4 @@ renWin Render
 # prevent the tk window from showing up then start the event loop
 wm withdraw .
 
+reader SetDefaultExecutivePrototype {}
