@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkAppendPolyData.cxx,v $
+  Module:    vtkAppendPolyData.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,14 +14,17 @@
 =========================================================================*/
 #include "vtkAppendPolyData.h"
 
+#include "vtkAlgorithmOutput.h"
 #include "vtkCellArray.h"
 #include "vtkCellData.h"
 #include "vtkDataSetAttributes.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
-vtkCxxRevisionMacro(vtkAppendPolyData, "$Revision: 1.93 $");
 vtkStandardNewMacro(vtkAppendPolyData);
 
 //----------------------------------------------------------------------------
@@ -40,27 +43,33 @@ vtkAppendPolyData::~vtkAppendPolyData()
 // Add a dataset to the list of data to append.
 void vtkAppendPolyData::AddInput(vtkPolyData *ds)
 {
-  if (UserManagedInputs)
+  if (this->UserManagedInputs)
     {
     vtkErrorMacro(<<
       "AddInput is not supported if UserManagedInputs is true");
     return;
     }
-  this->vtkProcessObject::AddInput(ds);
+  this->Superclass::AddInput(ds);
 }
 
 //----------------------------------------------------------------------------
 // Remove a dataset from the list of data to append.
 void vtkAppendPolyData::RemoveInput(vtkPolyData *ds)
 {
-  if (UserManagedInputs)
+  if (this->UserManagedInputs)
     {
     vtkErrorMacro(<<
       "RemoveInput is not supported if UserManagedInputs is true");
     return;
     }
-  this->vtkProcessObject::RemoveInput(ds);
-  this->vtkProcessObject::SqueezeInputArray();
+
+  vtkAlgorithmOutput *algOutput = 0;
+  if (ds)
+    {
+    algOutput = ds->GetProducerPort();
+    }
+
+  this->RemoveInputConnection(0, algOutput);
 }
 
 //----------------------------------------------------------------------------
@@ -68,43 +77,35 @@ void vtkAppendPolyData::RemoveInput(vtkPolyData *ds)
 // should only be used when UserManagedInputs is true.
 void vtkAppendPolyData::SetNumberOfInputs(int num)
 {
-  int overlap;
   if (!this->UserManagedInputs)
     {
     vtkErrorMacro(<<
       "SetNumberOfInputs is not supported if UserManagedInputs is false");
     return;
     }
-  // if the user sets inputs to be less than we already have, we'd better
-  // unregister the tailend ones that are being discarded
-  overlap = this->GetNumberOfInputs() - num;
-  if (overlap>0)
-    {
-    for (int i=0; i<overlap; i++)
-      {
-      this->SetNthInput(i+num,NULL);
-      }
-    }
-  this->vtkProcessObject::SetNumberOfInputs(num);
+
+  // Ask the superclass to set the number of connections.
+  this->SetNumberOfInputConnections(0, num);
 }
 
 //----------------------------------------------------------------------------
 // Set Nth input, should only be used when UserManagedInputs is true.
 void vtkAppendPolyData::SetInputByNumber(int num, vtkPolyData *input)
 {
-  if (!UserManagedInputs)
+  if (!this->UserManagedInputs)
     {
     vtkErrorMacro(<<
       "SetInputByNumber is not supported if UserManagedInputs is false");
     return;
     }
-  this->vtkProcessObject::SetNthInput(num, input);
+
+  // Ask the superclass to connect the input.
+  this->SetNthInputConnection(0, num, input? input->GetProducerPort() : 0);
 }
 
 //----------------------------------------------------------------------------
-// This method is much too long, and has to be broken up!
-// Append data sets into single unstructured grid
-void vtkAppendPolyData::Execute()
+int vtkAppendPolyData::ExecuteAppend(vtkPolyData* output,
+    vtkPolyData* inputs[], int numInputs)
 {
   int idx;
   vtkPolyData *ds;
@@ -118,7 +119,6 @@ void vtkAppendPolyData::Execute()
   vtkIdType numPts, numCells;
   vtkPointData *inPD = NULL;
   vtkCellData *inCD = NULL;
-  vtkPolyData *output = this->GetOutput();
   vtkPointData *outputPD = output->GetPointData();
   vtkCellData *outputCD = output->GetCellData();
   vtkDataArray *newPtScalars = NULL;
@@ -131,7 +131,7 @@ void vtkAppendPolyData::Execute()
   vtkIdType *pPolys;
   vtkIdType npts = 0;
   vtkIdType ptId, cellId;
-  
+
   vtkDebugMacro(<<"Appending polydata");
 
   // loop over all data sets, checking to see what point data is available.
@@ -142,11 +142,13 @@ void vtkAppendPolyData::Execute()
   int countPD=0;
   int countCD=0;
 
+  vtkIdType numVerts = 0, numLines = 0, numStrips = 0;
+
   // These Field lists are very picky.  Count the number of non empty inputs
   // so we can initialize them properly.
-  for (idx = 0; idx < this->NumberOfInputs; ++idx)
+  for (idx = 0; idx < numInputs; ++idx)
     {
-    ds = (vtkPolyData *)(this->Inputs[idx]);
+    ds = inputs[idx];
     if (ds != NULL)
       {
       if ( ds->GetNumberOfPoints() > 0)
@@ -163,11 +165,11 @@ void vtkAppendPolyData::Execute()
   // These are used to determine which fields are available for appending
   vtkDataSetAttributes::FieldList ptList(countPD);
   vtkDataSetAttributes::FieldList cellList(countCD);  
-  
+
   countPD = countCD = 0;
-  for (idx = 0; idx < this->NumberOfInputs; ++idx)
+  for (idx = 0; idx < numInputs; ++idx)
     {
-    ds = (vtkPolyData *)(this->Inputs[idx]);
+    ds = inputs[idx]; 
     if (ds != NULL)
       {
       // Skip points and cells if there are no points.  Empty inputs may have no arrays.
@@ -186,18 +188,24 @@ void vtkAppendPolyData::Execute()
           }
         ++countPD;
         } // for a data set that has points
-        
+
       // Although we cannot have cells without points ... let's not nest.
       if (ds->GetNumberOfCells() > 0 )  
         {
         // keep track of the size of the poly cell array
         if (ds->GetPolys())
           {
-          numPolys += ds->GetPolys()->GetNumberOfCells();
           sizePolys += ds->GetPolys()->GetNumberOfConnectivityEntries();
           }
         numCells += ds->GetNumberOfCells();
-        
+        // Count the cells of each type.
+        // This is used to ensure that cell data is copied at the correct
+        // locations in the output.
+        numVerts += ds->GetNumberOfVerts();
+        numLines += ds->GetNumberOfLines();
+        numPolys += ds->GetNumberOfPolys();
+        numStrips += ds->GetNumberOfStrips();
+
         inCD = ds->GetCellData();
         if ( countCD == 0 )
           {
@@ -214,8 +222,8 @@ void vtkAppendPolyData::Execute()
 
   if ( numPts < 1 || numCells < 1 )
     {
-    //vtkErrorMacro(<<"No data to append!");
-    return;
+    vtkDebugMacro(<<"No data to append!");
+    return 1;
     }
   this->UpdateProgress(0.10);
 
@@ -226,9 +234,9 @@ void vtkAppendPolyData::Execute()
   int pointtype = 0;
 
   // Keep track of types for fast point append
-  for (idx = 0; idx < this->NumberOfInputs; ++idx)
+  for (idx = 0; idx < numInputs; ++idx)
     {
-    ds = (vtkPolyData *)(this->Inputs[idx]);
+    ds = inputs[idx];
     if (ds != NULL && ds->GetNumberOfPoints()>0)
       {
       if ( firstType )
@@ -237,7 +245,7 @@ void vtkAppendPolyData::Execute()
         pointtype = ds->GetPoints()->GetData()->GetDataType();
         }
       ttype = ds->GetPoints()->GetData()->GetDataType();
-      
+
       if ( ttype != pointtype )
         {
         AllSame = 0;
@@ -263,47 +271,84 @@ void vtkAppendPolyData::Execute()
   newPolys = vtkCellArray::New();
   pPolys = newPolys->WritePointer(numPolys, sizePolys);
 
+  if (!pPolys && sizePolys > 0)
+    {
+    vtkErrorMacro(<<"Memory allocation failed in append filter");
+    return 0;
+    }
+
   // These are created manually for faster execution
   // Uses the properties of the last input
+  vtkDataArray *inDA=0;
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::SCALARS) > -1 )
     {
+    inDA=inPD->GetScalars();
     outputPD->CopyScalarsOff();
-    newPtScalars = inPD->GetScalars()->NewInstance();
-    newPtScalars->SetNumberOfComponents(inPD->GetScalars()->GetNumberOfComponents());
-    newPtScalars->SetName(inPD->GetScalars()->GetName());
+    newPtScalars = inDA->NewInstance();
+    newPtScalars->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtScalars->CopyComponentNames( inDA );
+    newPtScalars->SetName(inDA->GetName());
     newPtScalars->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtScalars->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::VECTORS) > -1 )
     {
+    inDA=inPD->GetVectors();
     outputPD->CopyVectorsOff();
-    newPtVectors = inPD->GetVectors()->NewInstance();
-    newPtVectors->SetNumberOfComponents(inPD->GetVectors()->GetNumberOfComponents());
-    newPtVectors->SetName(inPD->GetVectors()->GetName());
+    newPtVectors = inDA->NewInstance();
+    newPtVectors->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtVectors->CopyComponentNames( inDA );
+    newPtVectors->SetName(inDA->GetName());
     newPtVectors->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtVectors->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::TENSORS) > -1 )
     {
+    inDA=inPD->GetTensors();
     outputPD->CopyTensorsOff();
-    newPtTensors = inPD->GetTensors()->NewInstance();
-    newPtTensors->SetNumberOfComponents(inPD->GetTensors()->GetNumberOfComponents());
-    newPtTensors->SetName(inPD->GetTensors()->GetName());
+    newPtTensors = inDA->NewInstance();
+    newPtTensors->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtTensors->CopyComponentNames( inDA );
+    newPtTensors->SetName(inDA->GetName());
     newPtTensors->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtTensors->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::NORMALS) > -1 )
     {
+    inDA=inPD->GetNormals();
     outputPD->CopyNormalsOff();
-    newPtNormals = inPD->GetNormals()->NewInstance();
-    newPtNormals->SetNumberOfComponents(inPD->GetNormals()->GetNumberOfComponents());
-    newPtNormals->SetName(inPD->GetNormals()->GetName());
+    newPtNormals = inDA->NewInstance();
+    newPtNormals->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtNormals->CopyComponentNames( inDA );
+    newPtNormals->SetName(inDA->GetName());
     newPtNormals->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtNormals->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::TCOORDS) > -1 )
     {
+    inDA=inPD->GetTCoords();
     outputPD->CopyTCoordsOff();
-    newPtTCoords = inPD->GetTCoords()->NewInstance();
-    newPtTCoords->SetNumberOfComponents(inPD->GetTCoords()->GetNumberOfComponents());
-    newPtTCoords->SetName(inPD->GetTCoords()->GetName());
+    newPtTCoords = inDA->NewInstance();
+    newPtTCoords->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtTCoords->CopyComponentNames( inDA );
+    newPtTCoords->SetName(inDA->GetName());
     newPtTCoords->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtTCoords->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
 
   // Allocate the point and cell data
@@ -312,12 +357,15 @@ void vtkAppendPolyData::Execute()
 
   // loop over all input sets
   vtkIdType ptOffset = 0;
-  vtkIdType cellOffset = 0;
+  vtkIdType vertOffset = 0;
+  vtkIdType linesOffset = 0;
+  vtkIdType polysOffset = 0;
+  vtkIdType stripsOffset = 0;
   countPD = countCD = 0;
-  for (idx = 0; idx < this->NumberOfInputs; ++idx)
+  for (idx = 0; idx < numInputs; ++idx)
     {
-    this->UpdateProgress(0.2 + 0.8*idx/this->NumberOfInputs);
-    ds = (vtkPolyData *)(this->Inputs[idx]);
+    this->UpdateProgress(0.2 + 0.8*idx/numInputs);
+    ds = inputs[idx];
     // this check is not necessary, but I'll put it in anyway
     if (ds != NULL)
       {
@@ -383,20 +431,53 @@ void vtkAppendPolyData::Execute()
         ++countPD;
         }
 
+
       if (ds->GetNumberOfCells() > 0)
         {
+        // These are the cellIDs at which each of the cell types start.
+        vtkIdType linesIndex = ds->GetNumberOfVerts();
+        vtkIdType polysIndex = linesIndex + ds->GetNumberOfLines();
+        vtkIdType stripsIndex = polysIndex + ds->GetNumberOfPolys();
+
         // cell data could be made efficient like the point data,
         // but I will wait on that.
         // copy cell data
         for (cellId=0; cellId < numCells; cellId++)
           {
-          outputCD->CopyData(cellList,inCD,countCD,cellId,cellId+cellOffset);
+          vtkIdType outCellId = 0;
+          if (cellId < linesIndex)
+            {
+            outCellId = vertOffset;
+            vertOffset++;
+            }
+          else if (cellId < polysIndex)
+            {
+            // outCellId = number of lines we already added + total number of 
+            // verts expected in the output.
+            outCellId = linesOffset + numVerts;
+            linesOffset++;
+            }
+          else if (cellId < stripsIndex)
+            {
+            // outCellId = number of polys we already added + total number of 
+            // verts and lines expected in the output.
+            outCellId = polysOffset + numLines + numVerts;
+            polysOffset++;
+            }
+          else
+            {
+            // outCellId = number of tstrips we already added + total number of 
+            // polys, verts and lines expected in the output.
+            outCellId = stripsOffset + numPolys + numLines + numVerts;
+            stripsOffset++;
+            }
+          outputCD->CopyData(cellList,inCD,countCD,cellId,outCellId);
           }
         ++countCD;
-        
+
         // copy the cells
         pPolys = this->AppendCells(pPolys, inPolys, ptOffset);
-        
+
         // These other cell arrays could be made efficient like polys ...
         for (inVerts->InitTraversal(); inVerts->GetNextCell(npts,pts); )
           {
@@ -406,7 +487,7 @@ void vtkAppendPolyData::Execute()
             newVerts->InsertCellPoint(pts[i]+ptOffset);
             }
           }
-        
+
         for (inLines->InitTraversal(); inLines->GetNextCell(npts,pts); )
           {
           newLines->InsertNextCell(npts);
@@ -426,7 +507,6 @@ void vtkAppendPolyData::Execute()
           }
         }
       ptOffset += numPts;
-      cellOffset += numCells;
       }
     }
   
@@ -488,59 +568,102 @@ void vtkAppendPolyData::Execute()
   // When all optimizations are complete, this squeeze will be unecessary.
   // (But it does not seem to cost much.)
   output->Squeeze();
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
-void vtkAppendPolyData::ComputeInputUpdateExtents(vtkDataObject *data)
+// This method is much too long, and has to be broken up!
+// Append data sets into single polygonal data set.
+int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
+                                   vtkInformationVector **inputVector,
+                                   vtkInformationVector *outputVector)
 {
+  // get the info object
+  // get the ouptut
+  vtkPolyData *output = vtkPolyData::GetData(outputVector, 0);
+
+  int numInputs = inputVector[0]->GetNumberOfInformationObjects();
+  if (numInputs == 1)
+    {
+    output->ShallowCopy(vtkPolyData::GetData(inputVector[0], 0));
+    return 1;
+    }
+
+  vtkPolyData** inputs = new vtkPolyData*[numInputs];
+  for (int idx = 0; idx < numInputs; ++idx)
+    {
+    inputs[idx] = vtkPolyData::GetData(inputVector[0], idx);
+    }
+  int retVal = this->ExecuteAppend(output, inputs, numInputs);
+  delete [] inputs;
+  return retVal;
+}
+
+//----------------------------------------------------------------------------
+int vtkAppendPolyData::RequestUpdateExtent(vtkInformation *vtkNotUsed(request),
+                                           vtkInformationVector **inputVector,
+                                           vtkInformationVector *outputVector)
+{
+  // get the output info object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
   int piece, numPieces, ghostLevel;
-  vtkPolyData *output = (vtkPolyData *)data;
   int idx;
 
-  output->GetUpdateExtent(piece, numPieces, ghostLevel);
+  piece = outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER());
+  numPieces = outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES());
+  ghostLevel = outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS());
   
   // make sure piece is valid
   if (piece < 0 || piece >= numPieces)
     {
-    return;
+    return 0;
     }
 
+  int numInputs = this->GetNumberOfInputConnections(0);
   if (this->ParallelStreaming)
     {
-    piece = piece * this->NumberOfInputs;
-    numPieces = numPieces * this->NumberOfInputs;
+    piece = piece * numInputs;
+    numPieces = numPieces * numInputs;
     }
  
+  vtkInformation *inInfo;
   // just copy the Update extent as default behavior.
-  for (idx = 0; idx < this->NumberOfInputs; ++idx)
+  for (idx = 0; idx < numInputs; ++idx)
     {
-    if (this->Inputs[idx])
+    inInfo = inputVector[0]->GetInformationObject(idx);
+    if (inInfo)
       {
       if (this->ParallelStreaming)
         {
-        this->Inputs[idx]->SetUpdateExtent(piece + idx, numPieces, ghostLevel);
+        inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER(),
+                    piece + idx);
+        inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES(),
+                    numPieces);
+        inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS(),
+                    ghostLevel);
         }
       else
         {
-        this->Inputs[idx]->SetUpdateExtent(piece, numPieces, ghostLevel);
+        inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER(),
+                    piece);
+        inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES(),
+                    numPieces);
+        inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS(),
+                    ghostLevel);
         }
       }
     }
   
-  // Save the piece so execute can use this information.
-  this->ExecutePiece = piece;
-  this->ExecuteNumberOfPieces = numPieces;
+  return 1;
 }
 
 //----------------------------------------------------------------------------
 vtkPolyData *vtkAppendPolyData::GetInput(int idx)
 {
-  if (idx >= this->NumberOfInputs || idx < 0)
-    {
-    return NULL;
-    }
-
-  return (vtkPolyData *)(this->Inputs[idx]);
+  return vtkPolyData::SafeDownCast(
+    this->GetExecutive()->GetInputData(0, idx));
 }
 
 //----------------------------------------------------------------------------
@@ -548,19 +671,18 @@ void vtkAppendPolyData::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
 
-  if ( this->ParallelStreaming )
-    {
-    os << indent << "ParallelStreamingOn\n";
-    }
-  else
-    {
-    os << indent << "ParallelStreamingOff\n";
-    }
-  //
-  os << indent << "UserManagedInputs: "
-     << this->UserManagedInputs << endl;
+  os << "ParallelStreaming:" << (this->ParallelStreaming?"On":"Off") << endl;
+  os << "UserManagedInputs:" << (this->UserManagedInputs?"On":"Off") << endl;
 }
 
+//----------------------------------------------------------------------------
+template <class T>
+size_t vtkAppendPolyDataGetTypeSize(T*)
+{
+  return sizeof(T);
+}
+
+//----------------------------------------------------------------------------
 void vtkAppendPolyData::AppendData(vtkDataArray *dest, vtkDataArray *src,
                                    vtkIdType offset)
 {
@@ -590,36 +712,9 @@ void vtkAppendPolyData::AppendData(vtkDataArray *dest, vtkDataArray *src,
 
   switch (src->GetDataType())
     {
-    case VTK_FLOAT:
-      length *= sizeof(float);
-      break;
-    case VTK_DOUBLE:
-      length *= sizeof(double);
-      break;
-    case VTK_INT:
-      length *= sizeof(int);
-      break;
-    case VTK_UNSIGNED_INT:
-      length *= sizeof(unsigned int);
-      break;
-    case VTK_LONG:
-      length *= sizeof(long);
-      break;
-    case VTK_UNSIGNED_LONG:
-      length *= sizeof(unsigned long);
-      break;
-    case VTK_SHORT:
-      length *= sizeof(short);
-      break;
-    case VTK_UNSIGNED_SHORT:
-      length *= sizeof(unsigned short);
-      break;
-    case VTK_UNSIGNED_CHAR:
-      length *= sizeof(unsigned char);
-      break;
-    case VTK_CHAR:
-      length *= sizeof(char);
-      break;
+    vtkTemplateMacro(
+      length *= vtkAppendPolyDataGetTypeSize(static_cast<VTK_TT*>(0))
+      );
     default:
       vtkErrorMacro("Unknown data type " << src->GetDataType());
     }
@@ -630,6 +725,7 @@ void vtkAppendPolyData::AppendData(vtkDataArray *dest, vtkDataArray *src,
   memcpy(pDest, pSrc, length);
 }
 
+//----------------------------------------------------------------------------
 void vtkAppendPolyData::AppendDifferentPoints(vtkDataArray *dest,
                                               vtkDataArray *src,
                                               vtkIdType offset)
@@ -660,19 +756,20 @@ void vtkAppendPolyData::AppendDifferentPoints(vtkDataArray *dest,
     //
 
     case VTK_DOUBLE:
-      dDest = (double*)(dest->GetVoidPointer(offset*src->GetNumberOfComponents()));
+      dDest = static_cast<double*>(
+        dest->GetVoidPointer(offset*src->GetNumberOfComponents()));
       //
       switch (src->GetDataType())
         {
         case VTK_FLOAT:
-          fSrc = (float*)(src->GetVoidPointer(0));
+          fSrc = static_cast<float*>(src->GetVoidPointer(0));
           for (p=0; p<vals; p++)
             {
-              dDest[p] = (double) fSrc[p];
+            dDest[p] = static_cast<double>(fSrc[p]);
             }
           break;
         case VTK_DOUBLE:
-          dSrc = (double*)(src->GetVoidPointer(0));
+          dSrc = static_cast<double*>(src->GetVoidPointer(0));
           memcpy(dDest, dSrc, vals*sizeof(double));
           break;
         default:
@@ -687,6 +784,7 @@ void vtkAppendPolyData::AppendDifferentPoints(vtkDataArray *dest,
 }
 
 
+//----------------------------------------------------------------------------
 // returns the next pointer in dest
 vtkIdType *vtkAppendPolyData::AppendCells(vtkIdType *pDest, vtkCellArray *src,
                                           vtkIdType offset)
@@ -698,7 +796,7 @@ vtkIdType *vtkAppendPolyData::AppendCells(vtkIdType *pDest, vtkCellArray *src,
     return pDest;
     }
 
-  pSrc = (vtkIdType*)(src->GetPointer());
+  pSrc = src->GetPointer();
   end = pSrc + src->GetNumberOfConnectivityEntries();
   pNum = pSrc;
   
@@ -720,3 +818,14 @@ vtkIdType *vtkAppendPolyData::AppendCells(vtkIdType *pDest, vtkCellArray *src,
   
   return pDest;
 }
+
+//----------------------------------------------------------------------------
+int vtkAppendPolyData::FillInputPortInformation(int port, vtkInformation *info)
+{
+  if (!this->Superclass::FillInputPortInformation(port, info))
+    {
+    return 0;
+    }
+  info->Set(vtkAlgorithm::INPUT_IS_REPEATABLE(), 1);
+  return 1;
+}
