@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkArrayCalculator.cxx,v $
+  Module:    vtkArrayCalculator.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,16 +19,20 @@
 #include "vtkDoubleArray.h"
 #include "vtkFieldData.h"
 #include "vtkFunctionParser.h"
+#include "vtkGraph.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
+#include "vtkPointSet.h"
+#include "vtkPolyData.h"
+#include "vtkUnstructuredGrid.h"
 
-vtkCxxRevisionMacro(vtkArrayCalculator, "$Revision: 1.25.2.1 $");
 vtkStandardNewMacro(vtkArrayCalculator);
 
 vtkArrayCalculator::vtkArrayCalculator()
 {
   this->FunctionParser = vtkFunctionParser::New();
-  
   this->Function = NULL;
   this->ResultArrayName = NULL;
   this->SetResultArrayName("resultArray");
@@ -41,6 +45,17 @@ vtkArrayCalculator::vtkArrayCalculator()
   this->AttributeMode = VTK_ATTRIBUTE_MODE_DEFAULT;
   this->SelectedScalarComponents = NULL;
   this->SelectedVectorComponents = NULL;
+  this->CoordinateScalarVariableNames = NULL;
+  this->CoordinateVectorVariableNames = NULL;
+  this->NumberOfCoordinateScalarArrays = 0;
+  this->NumberOfCoordinateVectorArrays = 0;
+  this->SelectedCoordinateScalarComponents = NULL;
+  this->SelectedCoordinateVectorComponents = NULL;
+  this->CoordinateResults = 0;
+  this->ReplaceInvalidValues = 0;
+  this->ReplacementValue = 0.0;
+
+  this->ResultArrayType=VTK_DOUBLE;
 }
 
 vtkArrayCalculator::~vtkArrayCalculator()
@@ -122,6 +137,45 @@ vtkArrayCalculator::~vtkArrayCalculator()
     delete [] this->SelectedVectorComponents;
     this->SelectedVectorComponents = NULL;
     }
+
+  if (this->CoordinateScalarVariableNames)
+    {
+    for (i = 0; i < this->NumberOfCoordinateScalarArrays; i++)
+      {
+      delete [] this->CoordinateScalarVariableNames[i];
+      this->CoordinateScalarVariableNames[i] = NULL;
+      }
+    delete [] this->CoordinateScalarVariableNames;
+    this->CoordinateScalarVariableNames = NULL;
+    }
+
+  if (this->CoordinateVectorVariableNames)
+    {
+    for (i = 0; i < this->NumberOfCoordinateVectorArrays; i++)
+      {
+      delete [] this->CoordinateVectorVariableNames[i];
+      this->CoordinateVectorVariableNames[i] = NULL;
+      }
+    delete [] this->CoordinateVectorVariableNames;
+    this->CoordinateVectorVariableNames = NULL;
+    }
+
+  if (this->SelectedCoordinateScalarComponents)
+    {
+    delete [] this->SelectedCoordinateScalarComponents;
+    this->SelectedCoordinateScalarComponents = NULL;    
+    }
+
+  if (this->SelectedCoordinateVectorComponents)
+    {
+    for (i = 0; i < this->NumberOfCoordinateVectorArrays; i++)
+      {
+      delete [] this->SelectedCoordinateVectorComponents[i];
+      this->SelectedCoordinateVectorComponents[i] = NULL;
+      }
+    delete [] this->SelectedCoordinateVectorComponents;
+    this->SelectedCoordinateVectorComponents = NULL;
+    }
 }
 
 void vtkArrayCalculator::SetResultArrayName(const char* name)
@@ -136,6 +190,7 @@ void vtkArrayCalculator::SetResultArrayName(const char* name)
     return;
     }
   this->Modified();
+  
   if (this->ResultArrayName)
     {
     delete [] this->ResultArrayName;
@@ -145,65 +200,109 @@ void vtkArrayCalculator::SetResultArrayName(const char* name)
   strcpy(this->ResultArrayName, name);
 }
 
-void vtkArrayCalculator::Execute()
+int vtkArrayCalculator::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  int resultType; // 0 for scalar, 1 for vector
-  int attributeDataType; // 0 for point data, 1 for cell data
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataObject *input = inInfo->Get(vtkDataObject::DATA_OBJECT());
+  vtkDataObject *output = outInfo->Get(vtkDataObject::DATA_OBJECT());
+
+  int resultType = 0; // 0 for scalar, 1 for vector
+  int attributeDataType = 0; // 0 for point data, 1 for cell data
   vtkIdType i;
   int j;
   
-  vtkDataSet* input = this->GetInput();
-  vtkDataSet* output = this->GetOutput();
-  vtkPointData* inPD = input->GetPointData();
-  vtkCellData* inCD = input->GetCellData();
-  vtkFieldData* inFD;
+  vtkDataSetAttributes* inFD = 0;
+  vtkDataSetAttributes* outFD = 0;
   vtkDataArray* currentArray;
-  vtkDoubleArray* resultArray;
-  vtkIdType numTuples;
+  vtkIdType numTuples = 0;
   double scalarResult[1];
-  
-  if (this->AttributeMode == VTK_ATTRIBUTE_MODE_DEFAULT)
+  vtkDataArray* resultArray = 0;
+  vtkPoints* resultPoints = 0;
+
+  this->FunctionParser->SetReplaceInvalidValues(this->ReplaceInvalidValues);
+  this->FunctionParser->SetReplacementValue(this->ReplacementValue);
+
+  vtkDataSet *dsInput = vtkDataSet::SafeDownCast(input);
+  vtkDataSet *dsOutput = vtkDataSet::SafeDownCast(output);
+  vtkGraph *graphInput = vtkGraph::SafeDownCast(input);
+  vtkGraph *graphOutput = vtkGraph::SafeDownCast(output);
+  vtkPointSet* psInput = vtkPointSet::SafeDownCast(input);
+  vtkPointSet* psOutput = vtkPointSet::SafeDownCast(output);
+  if (dsInput)
     {
-    inFD = inPD;
-    attributeDataType = 0;
-    numTuples = input->GetNumberOfPoints();
+    if (this->AttributeMode == VTK_ATTRIBUTE_MODE_DEFAULT ||
+        this->AttributeMode == VTK_ATTRIBUTE_MODE_USE_POINT_DATA)
+      {
+      inFD = dsInput->GetPointData();
+      outFD = dsOutput->GetPointData();
+      attributeDataType = 0;
+      numTuples = dsInput->GetNumberOfPoints();
+      }
+    else
+      {
+      inFD = dsInput->GetCellData();
+      outFD = dsOutput->GetCellData();
+      attributeDataType = 1;
+      numTuples = dsInput->GetNumberOfCells();
+      }
     }
-  else if (this->AttributeMode == VTK_ATTRIBUTE_MODE_USE_POINT_DATA)
+  else if (graphInput)
     {
-    inFD = inPD;
-    attributeDataType = 0;
-    numTuples = input->GetNumberOfPoints();
-    }
-  else
-    {
-    inFD = inCD;
-    attributeDataType = 1;
-    numTuples = input->GetNumberOfCells();
+    if (this->AttributeMode == VTK_ATTRIBUTE_MODE_DEFAULT ||
+        this->AttributeMode == VTK_ATTRIBUTE_MODE_USE_VERTEX_DATA)
+      {
+      inFD = graphInput->GetVertexData();
+      outFD = graphOutput->GetVertexData();
+      attributeDataType = 0;
+      numTuples = graphInput->GetNumberOfVertices();
+      }
+    else
+      {
+      inFD = graphInput->GetEdgeData();
+      outFD = graphOutput->GetEdgeData();
+      attributeDataType = 1;
+      numTuples = graphInput->GetNumberOfEdges();
+      }
     }
   
+  if (numTuples < 1)
+    {
+    vtkDebugMacro("Empty data.");
+    return 1;
+    }
+    
   for (i = 0; i < this->NumberOfScalarArrays; i++)
-    {
+    {   
     currentArray = inFD->GetArray(this->ScalarArrayNames[i]);
+    
     if (currentArray)
       {
       if (currentArray->GetNumberOfComponents() >
           this->SelectedScalarComponents[i])
         {
         this->FunctionParser->
-          SetScalarVariableValue(this->ScalarVariableNames[i],
-                                 currentArray->GetComponent(0, this->SelectedScalarComponents[i]));
+          SetScalarVariableValue(
+            this->ScalarVariableNames[i],
+            currentArray->GetComponent(0, this->SelectedScalarComponents[i]));
         }
       else
         {
         vtkErrorMacro("Array " << this->ScalarArrayNames[i]
                       << " does not contain the selected component.");
-        return;
+        return 1;
         }
       }
     else
       {
       vtkErrorMacro("Invalid array name: " << this->ScalarArrayNames[i]);
-      return;
+      return 1;
       }
     }
 
@@ -220,26 +319,73 @@ void vtkArrayCalculator::Execute()
            this->SelectedVectorComponents[i][2]))
         {
         this->FunctionParser->
-          SetVectorVariableValue(this->VectorVariableNames[i],
-                                 currentArray->GetComponent(0, this->SelectedVectorComponents[i][0]),
-                                 currentArray->GetComponent(0, this->SelectedVectorComponents[i][1]),
-                                 currentArray->GetComponent(0, this->SelectedVectorComponents[i][2]));
+          SetVectorVariableValue(
+            this->VectorVariableNames[i],
+            currentArray->GetComponent(0, this->SelectedVectorComponents[i][0]),
+            currentArray->GetComponent(0, this->SelectedVectorComponents[i][1]),
+            currentArray->GetComponent(0, this->SelectedVectorComponents[i][2]));
         }
       else
         {
         vtkErrorMacro("Array " << this->VectorArrayNames[i]
                       << " does not contain one of the selected components.");
-        return;
+        return 1;
         }
       }
     else
       {
       vtkErrorMacro("Invalid array name: " << this->VectorArrayNames[i]);
-      return;
+      return 1;
       }
     }
 
-  if (this->FunctionParser->IsScalarResult())
+  // we can add points
+  if(attributeDataType == 0)
+    {
+    for (i = 0; i < this->NumberOfCoordinateScalarArrays; i++)
+      {
+      double* pt = 0;
+      if (dsInput)
+        {
+        pt = dsInput->GetPoint(0);
+        }
+      else
+        {
+        pt = graphInput->GetPoint(0);
+        }
+      this->FunctionParser->
+        SetScalarVariableValue(
+          this->CoordinateScalarVariableNames[i],
+          pt[this->SelectedCoordinateScalarComponents[i]]);
+      }
+
+    for (i = 0; i < this->NumberOfCoordinateVectorArrays; i++)
+      {
+      double* pt = 0;
+      if (dsInput)
+        {
+        pt = dsInput->GetPoint(0);
+        }
+      else
+        {
+        pt = graphInput->GetPoint(0);
+        }
+      this->FunctionParser->
+        SetVectorVariableValue(
+          this->CoordinateVectorVariableNames[i],
+          pt[this->SelectedCoordinateVectorComponents[i][0]],
+          pt[this->SelectedCoordinateVectorComponents[i][1]],
+          pt[this->SelectedCoordinateVectorComponents[i][2]]);
+      }
+    }
+
+  if ( !this->Function || strlen(this->Function) == 0)
+    {
+    dsOutput->CopyStructure(dsInput);
+    dsOutput->CopyAttributes(dsInput);
+    return 1;
+    }
+  else if (this->FunctionParser->IsScalarResult())
     {
     resultType = 0;
     }
@@ -249,11 +395,39 @@ void vtkArrayCalculator::Execute()
     }
   else
     {
+    dsOutput->CopyStructure(dsInput);
+    dsOutput->CopyAttributes(dsInput);
     // Error occurred in vtkFunctionParser.
-    return;
+    vtkWarningMacro("An error occured when parsing the calculator's function.  See previous errors.");
+    return 1;
+    }
+  
+  if(resultType == 1 && CoordinateResults != 0 && (psOutput || graphOutput))
+    {
+    resultPoints = vtkPoints::New();
+    resultPoints->SetNumberOfPoints(numTuples);
+    resultArray = resultPoints->GetData();
+    }
+  else if(CoordinateResults != 0)
+    {
+    if(resultType != 1)
+      {
+      vtkErrorMacro("Coordinate output specified, "
+                    "but there are no vector results");
+      }
+    else if(!psOutput)
+      {
+      vtkErrorMacro("Coordinate output specified, "
+                    "but output is not polydata or unstructured grid");
+      }
+    return 1;
+    }
+  else
+    {
+      resultArray=
+        vtkDataArray::SafeDownCast(vtkAbstractArray::CreateArray(this->ResultArrayType));
     }
 
-  resultArray = vtkDoubleArray::New();
   if (resultType == 0)
     {
     resultArray->SetNumberOfComponents(1);
@@ -265,7 +439,7 @@ void vtkArrayCalculator::Execute()
     {
     resultArray->Allocate(numTuples * 3);
     resultArray->SetNumberOfComponents(3);
-    resultArray->SetNumberOfTuples(numTuples);
+    resultArray->SetNumberOfTuples(numTuples);    
     resultArray->SetTuple(0, this->FunctionParser->GetVectorResult());
     }
   
@@ -275,15 +449,45 @@ void vtkArrayCalculator::Execute()
       {
       currentArray = inFD->GetArray(this->ScalarArrayNames[j]);
       this->FunctionParser->
-        SetScalarVariableValue(j, currentArray->GetComponent(i, this->SelectedScalarComponents[j]));
+        SetScalarVariableValue(
+          j, currentArray->GetComponent(i, this->SelectedScalarComponents[j]));
       }
     for (j = 0; j < this->NumberOfVectorArrays; j++)
       {
       currentArray = inFD->GetArray(this->VectorArrayNames[j]);
       this->FunctionParser->
-        SetVectorVariableValue(j, currentArray->GetComponent(i, this->SelectedVectorComponents[j][0]),
-                               currentArray->GetComponent(i, this->SelectedVectorComponents[j][1]),
-                               currentArray->GetComponent(i, this->SelectedVectorComponents[j][2]));      
+        SetVectorVariableValue(
+          j, currentArray->GetComponent(i, this->SelectedVectorComponents[j][0]),
+          currentArray->GetComponent(
+            i, this->SelectedVectorComponents[j][1]),
+          currentArray->GetComponent(i, this->SelectedVectorComponents[j][2]));
+      }
+    if(attributeDataType == 0)
+      {
+      double* pt = 0;
+      if (dsInput)
+        {
+        pt = dsInput->GetPoint(i);
+        }
+      else
+        {
+        pt = graphInput->GetPoint(i);
+        }
+      for (j = 0; j < this->NumberOfCoordinateScalarArrays; j++)
+        {
+        this->FunctionParser->
+          SetScalarVariableValue(
+            j+this->NumberOfScalarArrays, pt[this->SelectedCoordinateScalarComponents[j]]);
+        }
+      for (j = 0; j < this->NumberOfCoordinateVectorArrays; j++)
+        {
+        this->FunctionParser->
+          SetVectorVariableValue(
+            j+this->NumberOfVectorArrays,
+            pt[this->SelectedCoordinateVectorComponents[j][0]],
+            pt[this->SelectedCoordinateVectorComponents[j][1]],
+            pt[this->SelectedCoordinateVectorComponents[j][2]]);
+        }
       }
     if (resultType == 0)
       {
@@ -296,38 +500,67 @@ void vtkArrayCalculator::Execute()
       }
     }
   
-  output->CopyStructure(input);
-  output->GetPointData()->PassData(inPD);
-  output->GetCellData()->PassData(inCD);
-  
-  resultArray->SetName(this->ResultArrayName);
-  if (attributeDataType == 0)
+  if(resultPoints)
     {
-    output->GetPointData()->AddArray(resultArray);
-    if (resultType == 0)
+    if(psInput)
       {
-      output->GetPointData()->SetActiveScalars(this->ResultArrayName);
+      if(attributeDataType == 0)
+        {
+        psOutput->CopyStructure(psInput);
+        }
+      else
+        {
+        vtkPolyData* pd = vtkPolyData::SafeDownCast(psOutput);
+        vtkUnstructuredGrid* ug = vtkUnstructuredGrid::SafeDownCast(psOutput);
+        if(pd)
+          {
+          pd->Reset();
+          pd->Allocate(numTuples);
+          for (i = 1; i < numTuples; i++)
+            {
+            pd->InsertNextCell(VTK_VERTEX, 1, &i);
+            }
+          }
+        else if(ug)
+          {
+          ug->Reset();
+          ug->Allocate(numTuples);
+          for (i = 1; i < numTuples; i++)
+            {
+            ug->InsertNextCell(VTK_VERTEX, 1, &i);
+            }
+          }
+        }
+      psOutput->SetPoints(resultPoints);
+      psOutput->CopyAttributes(dsInput);
       }
     else
       {
-      output->GetPointData()->SetActiveVectors(this->ResultArrayName);
+      graphOutput->CopyStructure(graphInput);
+      outFD->PassData(inFD);
       }
+    resultPoints->Delete();
     }
   else
     {
-    output->GetCellData()->AddArray(resultArray);
+    dsOutput->CopyStructure(dsInput);
+    dsOutput->CopyAttributes(dsInput);
+    
+    resultArray->SetName(this->ResultArrayName);
+    outFD->AddArray(resultArray);
     if (resultType == 0)
       {
-      output->GetCellData()->SetActiveScalars(this->ResultArrayName);
+      outFD->SetActiveScalars(this->ResultArrayName);
       }
     else
       {
-      output->GetCellData()->SetActiveVectors(this->ResultArrayName);
+      outFD->SetActiveVectors(this->ResultArrayName);
       }
+    
+    resultArray->Delete();
     }
-  
-  resultArray->Delete();
-  resultArray = NULL;
+
+  return 1;
 }
 
 void vtkArrayCalculator::SetFunction(const char* function)
@@ -373,7 +606,7 @@ void vtkArrayCalculator::AddScalarArrayName(const char* arrayName,
     strcpy(arrayNames[i], this->ScalarArrayNames[i]);
     delete [] this->ScalarArrayNames[i];
     this->ScalarArrayNames[i] = NULL;
-    varNames[i] = new char[strlen(this->ScalarArrayNames[i]) + 1];
+    varNames[i] = new char[strlen(this->ScalarVariableNames[i]) + 1];
     strcpy(varNames[i], this->ScalarVariableNames[i]);
     delete [] this->ScalarVariableNames[i];
     this->ScalarVariableNames[i] = NULL;
@@ -412,11 +645,8 @@ void vtkArrayCalculator::AddScalarArrayName(const char* arrayName,
     this->SelectedScalarComponents[i] = tempComponents[i];
     }
   delete [] arrayNames;
-  arrayNames = NULL;
   delete [] varNames;
-  varNames = NULL;
   delete [] tempComponents;
-  tempComponents = NULL;
 
   this->ScalarArrayNames[i] = new char[strlen(arrayName) + 1];
   strcpy(this->ScalarArrayNames[i], arrayName);
@@ -497,11 +727,8 @@ void vtkArrayCalculator::AddVectorArrayName(const char* arrayName,
     tempComponents[i] = NULL;
     }
   delete [] arrayNames;
-  arrayNames = NULL;
   delete [] varNames;
-  varNames = NULL;
   delete [] tempComponents;
-  tempComponents = NULL;
 
   this->VectorArrayNames[i] = new char[strlen(arrayName) + 1];
   strcpy(this->VectorArrayNames[i], arrayName);
@@ -574,11 +801,8 @@ void vtkArrayCalculator::AddScalarVariable(const char* variableName,
     this->SelectedScalarComponents[i] = tempComponents[i];
     }
   delete [] arrayNames;
-  arrayNames = NULL;
   delete [] varNames;
-  varNames = NULL;
   delete [] tempComponents;
-  tempComponents = NULL;
 
   this->ScalarArrayNames[i] = new char[strlen(arrayName) + 1];
   strcpy(this->ScalarArrayNames[i], arrayName);
@@ -660,11 +884,8 @@ void vtkArrayCalculator::AddVectorVariable(const char* variableName,
     tempComponents[i] = NULL;
     }
   delete [] arrayNames;
-  arrayNames = NULL;
   delete [] varNames;
-  varNames = NULL;
   delete [] tempComponents;
-  tempComponents = NULL;
 
   this->VectorArrayNames[i] = new char[strlen(arrayName) + 1];
   strcpy(this->VectorArrayNames[i], arrayName);
@@ -678,6 +899,121 @@ void vtkArrayCalculator::AddVectorVariable(const char* variableName,
   this->NumberOfVectorArrays++;
 }
 
+void vtkArrayCalculator::AddCoordinateScalarVariable(const char* variableName,
+                                                     int component)
+{
+  int i;
+  char** varNames = new char *[this->NumberOfCoordinateScalarArrays];
+  int* tempComponents = new int[this->NumberOfCoordinateScalarArrays];
+
+  for (i = 0; i < this->NumberOfCoordinateScalarArrays; i++)
+    {
+    varNames[i] = new char[strlen(this->CoordinateScalarVariableNames[i]) + 1];
+    strcpy(varNames[i], this->CoordinateScalarVariableNames[i]);
+    delete [] this->CoordinateScalarVariableNames[i];
+    this->CoordinateScalarVariableNames[i] = NULL;
+    tempComponents[i] = this->SelectedCoordinateScalarComponents[i];
+    }
+  if (this->CoordinateScalarVariableNames)
+    {
+    delete [] this->CoordinateScalarVariableNames;
+    this->CoordinateScalarVariableNames = NULL;
+    }
+  if (this->SelectedCoordinateScalarComponents)
+    {
+    delete [] this->SelectedCoordinateScalarComponents;
+    this->SelectedCoordinateScalarComponents = NULL;
+    }
+
+  this->CoordinateScalarVariableNames =
+    new char *[this->NumberOfCoordinateScalarArrays + 1];
+  this->SelectedCoordinateScalarComponents =
+    new int[this->NumberOfCoordinateScalarArrays + 1];
+
+  for (i = 0; i < this->NumberOfCoordinateScalarArrays; i++)
+    {
+    this->CoordinateScalarVariableNames[i] = new char[strlen(varNames[i]) + 1];
+    strcpy(this->CoordinateScalarVariableNames[i], varNames[i]);
+    delete [] varNames[i];
+    varNames[i] = NULL;
+    this->SelectedCoordinateScalarComponents[i] = tempComponents[i];
+    }
+  delete [] varNames;
+  delete [] tempComponents;
+
+  this->CoordinateScalarVariableNames[i] = new char[strlen(variableName) + 1];
+  strcpy(this->CoordinateScalarVariableNames[i], variableName);
+  this->SelectedCoordinateScalarComponents[i] = component;
+
+  this->NumberOfCoordinateScalarArrays++;
+}
+
+void vtkArrayCalculator::AddCoordinateVectorVariable(const char* variableName,
+                                                     int component0, 
+                                                     int component1, 
+                                                     int component2)
+{
+  int i;
+  char** varNames = new char *[this->NumberOfCoordinateVectorArrays];
+  int** tempComponents = new int *[this->NumberOfCoordinateVectorArrays];
+
+  for (i = 0; i < this->NumberOfCoordinateVectorArrays; i++)
+    {
+    varNames[i] = new char[strlen(this->CoordinateVectorVariableNames[i])+1];
+    strcpy(varNames[i], this->CoordinateVectorVariableNames[i]);
+    delete [] this->CoordinateVectorVariableNames[i];
+    this->CoordinateVectorVariableNames[i] = NULL;
+    tempComponents[i] = new int[3];
+    tempComponents[i][0] = this->SelectedCoordinateVectorComponents[i][0];
+    tempComponents[i][1] = this->SelectedCoordinateVectorComponents[i][1];
+    tempComponents[i][2] = this->SelectedCoordinateVectorComponents[i][2];
+    delete [] this->SelectedCoordinateVectorComponents[i];
+    this->SelectedCoordinateVectorComponents[i] = NULL;
+    }
+
+  if (this->CoordinateVectorVariableNames)
+    {
+    delete [] this->CoordinateVectorVariableNames;
+    this->CoordinateVectorVariableNames = NULL;
+    }
+  if (this->SelectedCoordinateVectorComponents)
+    {
+    delete [] this->SelectedCoordinateVectorComponents;
+    this->SelectedCoordinateVectorComponents = NULL;
+    }
+
+  this->CoordinateVectorVariableNames = 
+    new char *[this->NumberOfCoordinateVectorArrays + 1];
+  this->SelectedCoordinateVectorComponents = 
+    new int *[this->NumberOfCoordinateVectorArrays + 1];
+
+  for (i = 0; i < this->NumberOfCoordinateVectorArrays; i++)
+    {
+    this->CoordinateVectorVariableNames[i] = new char[strlen(varNames[i]) + 1];
+    strcpy(this->CoordinateVectorVariableNames[i], varNames[i]);
+    delete [] varNames[i];
+    varNames[i] = NULL;
+    this->SelectedCoordinateVectorComponents[i] = new int[3];
+    this->SelectedCoordinateVectorComponents[i][0] = tempComponents[i][0];
+    this->SelectedCoordinateVectorComponents[i][1] = tempComponents[i][1];
+    this->SelectedCoordinateVectorComponents[i][2] = tempComponents[i][2];
+    delete [] tempComponents[i];
+    tempComponents[i] = NULL;
+    }
+  delete [] varNames;
+  delete [] tempComponents;
+
+  this->CoordinateVectorVariableNames[i] = new char[strlen(variableName) + 1];
+  strcpy(this->CoordinateVectorVariableNames[i], variableName);
+  
+  this->SelectedCoordinateVectorComponents[i] = new int[3];
+  this->SelectedCoordinateVectorComponents[i][0] = component0;
+  this->SelectedCoordinateVectorComponents[i][1] = component1;
+  this->SelectedCoordinateVectorComponents[i][2] = component2;
+  
+  this->NumberOfCoordinateVectorArrays++;
+}
+
 const char* vtkArrayCalculator::GetAttributeModeAsString()
 {
   if ( this->AttributeMode == VTK_ATTRIBUTE_MODE_DEFAULT )
@@ -688,13 +1024,21 @@ const char* vtkArrayCalculator::GetAttributeModeAsString()
     {
     return "UsePointData";
     }
-  else 
+  else if ( this->AttributeMode == VTK_ATTRIBUTE_MODE_USE_CELL_DATA )
     {
     return "UseCellData";
     }
+  else if ( this->AttributeMode == VTK_ATTRIBUTE_MODE_USE_VERTEX_DATA )
+    {
+    return "UseVertexData";
+    }
+  else 
+    {
+    return "UseEdgeData";
+    }
 }
 
-void vtkArrayCalculator::RemoveAllVariables()
+void vtkArrayCalculator::RemoveScalarVariables()
 {
   int i;
   
@@ -716,6 +1060,13 @@ void vtkArrayCalculator::RemoveAllVariables()
     }
   this->NumberOfScalarArrays = 0;
   
+  this->FunctionParser->RemoveScalarVariables();
+}
+
+void vtkArrayCalculator::RemoveVectorVariables()
+{
+  int i;
+
   for (i = 0; i < this->NumberOfVectorArrays; i++)
     {
     delete [] this->VectorArrayNames[i];
@@ -735,8 +1086,60 @@ void vtkArrayCalculator::RemoveAllVariables()
     this->SelectedVectorComponents = NULL;
     }
   this->NumberOfVectorArrays = 0;
+  
+  this->FunctionParser->RemoveVectorVariables();
+}
 
-  this->FunctionParser->RemoveAllVariables();
+void vtkArrayCalculator::RemoveCoordinateScalarVariables()
+{
+  int i;
+  
+  for (i = 0; i < this->NumberOfCoordinateScalarArrays; i++)
+    {
+    delete [] this->CoordinateScalarVariableNames[i];
+    this->CoordinateScalarVariableNames[i] = NULL;
+    }
+  if (this->NumberOfCoordinateScalarArrays > 0)
+    {
+    delete [] this->CoordinateScalarVariableNames;
+    this->CoordinateScalarVariableNames = NULL;
+    delete [] this->SelectedCoordinateScalarComponents;
+    this->SelectedCoordinateScalarComponents = NULL;
+    }
+  this->NumberOfCoordinateScalarArrays = 0;
+  
+  this->FunctionParser->RemoveScalarVariables();
+}
+
+void vtkArrayCalculator::RemoveCoordinateVectorVariables()
+{
+  int i;
+
+  for (i = 0; i < this->NumberOfCoordinateVectorArrays; i++)
+    {
+    delete [] this->CoordinateVectorVariableNames[i];
+    this->CoordinateVectorVariableNames[i] = NULL;
+    delete [] this->SelectedCoordinateVectorComponents[i];
+    this->SelectedCoordinateVectorComponents[i] = NULL;
+    }
+  if (this->NumberOfVectorArrays > 0)
+    {
+    delete [] this->CoordinateVectorVariableNames;
+    this->CoordinateVectorVariableNames = NULL;
+    delete [] this->SelectedCoordinateVectorComponents;
+    this->SelectedCoordinateVectorComponents = NULL;
+    }
+  this->NumberOfCoordinateVectorArrays = 0;
+  
+  this->FunctionParser->RemoveVectorVariables();
+}
+
+void vtkArrayCalculator::RemoveAllVariables()
+{
+  this->RemoveScalarVariables();
+  this->RemoveVectorVariables();
+  this->RemoveCoordinateScalarVariables();
+  this->RemoveCoordinateVectorVariables();
 }
 
 char* vtkArrayCalculator::GetScalarArrayName(int i)
@@ -801,9 +1204,20 @@ void vtkArrayCalculator::PrintSelf(ostream& os, vtkIndent indent)
      << (this->Function ? this->Function : "(none)") << endl;
   os << indent << "Result Array Name: "
      << (this->ResultArrayName ? this->ResultArrayName : "(none)") << endl;
+
+  os << indent << "Result Array Type: " << vtkImageScalarTypeNameMacro(this->ResultArrayType) << endl;
+
+  os << indent << "Coordinate Results: " << this->CoordinateResults << endl;
   os << indent << "Attribute Mode: " << this->GetAttributeModeAsString() << endl;
   os << indent << "Number Of Scalar Arrays: " << this->NumberOfScalarArrays
      << endl;
   os << indent << "Number Of Vector Arrays: " << this->NumberOfVectorArrays
      << endl;
+  os << indent << "Number Of Coordinate Scalar Arrays: "
+     << this->NumberOfCoordinateScalarArrays << endl;
+  os << indent << "Number Of Coordinate Vector Arrays: "
+     << this->NumberOfCoordinateVectorArrays << endl;
+  os << indent << "Replace Invalid Values: " 
+     << (this->ReplaceInvalidValues ? "On" : "Off") << endl;
+  os << indent << "Replacement Value: " << this->ReplacementValue << endl;
 }
