@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkClipPolyData.cxx,v $
+  Module:    vtkClipPolyData.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,19 +16,22 @@
 
 #include "vtkCellArray.h"
 #include "vtkCellData.h"
+#include "vtkExecutive.h"
 #include "vtkFloatArray.h"
 #include "vtkGenericCell.h"
 #include "vtkImplicitFunction.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkLine.h"
 #include "vtkMergePoints.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 #include "vtkTriangle.h"
+#include "vtkIncrementalPointLocator.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkClipPolyData, "$Revision: 1.53 $");
 vtkStandardNewMacro(vtkClipPolyData);
 vtkCxxSetObjectMacro(vtkClipPolyData,ClipFunction,vtkImplicitFunction);
 
@@ -43,9 +46,12 @@ vtkClipPolyData::vtkClipPolyData(vtkImplicitFunction *cf)
   this->Value = 0.0;
   this->GenerateClipScalars = 0;
 
+  this->SetNumberOfOutputPorts(2);
+
   this->GenerateClippedOutput = 0;
-  this->vtkSource::SetNthOutput(1,vtkPolyData::New());
-  this->Outputs[1]->Delete();
+  vtkPolyData *output2 = vtkPolyData::New();
+  this->GetExecutive()->SetOutputData(1, output2);
+  output2->Delete();
 }
 
 //----------------------------------------------------------------------------
@@ -64,7 +70,7 @@ vtkClipPolyData::~vtkClipPolyData()
 // then this object is modified as well.
 unsigned long vtkClipPolyData::GetMTime()
 {
-  unsigned long mTime=this->vtkPolyDataToPolyDataFilter::GetMTime();
+  unsigned long mTime=this->Superclass::GetMTime();
   unsigned long time;
 
   if ( this->ClipFunction != NULL )
@@ -83,12 +89,8 @@ unsigned long vtkClipPolyData::GetMTime()
 
 vtkPolyData *vtkClipPolyData::GetClippedOutput()
 {
-  if (this->NumberOfOutputs < 2)
-    {
-    return NULL;
-    }
-  
-  return (vtkPolyData *)(this->Outputs[1]);
+  return vtkPolyData::SafeDownCast(
+    this->GetExecutive()->GetOutputData(1));
 }
 
 
@@ -96,14 +98,21 @@ vtkPolyData *vtkClipPolyData::GetClippedOutput()
 //
 // Clip through data generating surface.
 //
-void vtkClipPolyData::Execute()
+int vtkClipPolyData::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkPolyData *input = this->GetInput();
-  vtkPolyData *output = this->GetOutput();
-  if (!input || !output)
-    {
-    return;
-    }
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkPolyData *input = vtkPolyData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkIdType cellId, i, updateTime;
   vtkPoints *cellPts;
   vtkDataArray *clipScalars;
@@ -129,41 +138,16 @@ void vtkClipPolyData::Execute()
   //
   if ( numPts < 1 || inPts == NULL )
     {
-    //vtkErrorMacro(<<"No data to clip");
-    return;
+    vtkDebugMacro(<<"No data to clip");
+    return 1;
     }
 
   if ( !this->ClipFunction && this->GenerateClipScalars )
     {
     vtkErrorMacro(<<"Cannot generate clip scalars if no clip function defined");
-    return;
+    return 1;
     }
 
-  // Create objects to hold output of clip operation
-  //
-  estimatedSize = numCells;
-  estimatedSize = estimatedSize / 1024 * 1024; //multiple of 1024
-  if (estimatedSize < 1024)
-    {
-    estimatedSize = 1024;
-    }
-
-  newPoints = vtkPoints::New();
-  newPoints->Allocate(numPts,numPts/2);
-  newVerts = vtkCellArray::New();
-  newVerts->Allocate(estimatedSize,estimatedSize/2);
-  newLines = vtkCellArray::New();
-  newLines->Allocate(estimatedSize,estimatedSize/2);
-  newPolys = vtkCellArray::New();
-  newPolys->Allocate(estimatedSize,estimatedSize/2);
-
-  // locator used to merge potentially duplicate points
-  if ( this->Locator == NULL )
-    {
-    this->CreateDefaultLocator();
-    }
-  this->Locator->InitPointInsertion (newPoints, input->GetBounds());
-
   // Determine whether we're clipping with input scalars or a clip function
   // and to necessary setup.
   if ( this->ClipFunction )
@@ -189,10 +173,35 @@ void vtkClipPolyData::Execute()
     if ( !clipScalars )
       {
       vtkErrorMacro(<<"Cannot clip without clip function or input scalars");
-      return;
+      return 1;
       }
     }
-    
+
+  // Create objects to hold output of clip operation
+  //
+  estimatedSize = numCells;
+  estimatedSize = estimatedSize / 1024 * 1024; //multiple of 1024
+  if (estimatedSize < 1024)
+    {
+    estimatedSize = 1024;
+    }
+
+  newPoints = vtkPoints::New();
+  newPoints->Allocate(numPts,numPts/2);
+  newVerts = vtkCellArray::New();
+  newVerts->Allocate(estimatedSize,estimatedSize/2);
+  newLines = vtkCellArray::New();
+  newLines->Allocate(estimatedSize,estimatedSize/2);
+  newPolys = vtkCellArray::New();
+  newPolys->Allocate(estimatedSize,estimatedSize/2);
+
+  // locator used to merge potentially duplicate points
+  if ( this->Locator == NULL )
+    {
+    this->CreateDefaultLocator();
+    }
+  this->Locator->InitPointInsertion (newPoints, input->GetBounds());
+ 
   if ( !this->GenerateClipScalars && !input->GetPointData()->GetScalars())
     {
     outPD->CopyScalarsOff();
@@ -269,7 +278,7 @@ void vtkClipPolyData::Execute()
 
     if ( !(cellId % updateTime) )
       {
-      this->UpdateProgress((double)cellId / numCells);
+      this->UpdateProgress(static_cast<double>(cellId) / numCells);
       abort = this->GetAbortExecute();
       }
     } //for each cell
@@ -348,13 +357,15 @@ void vtkClipPolyData::Execute()
   
   this->Locator->Initialize();//release any extra memory
   output->Squeeze();
+
+  return 1;
 }
 
 
 //----------------------------------------------------------------------------
 // Specify a spatial locator for merging points. By default, 
 // an instance of vtkMergePoints is used.
-void vtkClipPolyData::SetLocator(vtkPointLocator *locator)
+void vtkClipPolyData::SetLocator(vtkIncrementalPointLocator *locator)
 {
   if ( this->Locator == locator)
     {
