@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkConnectivityFilter.cxx,v $
+  Module:    vtkConnectivityFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,21 +19,22 @@
 #include "vtkDataSet.h"
 #include "vtkFloatArray.h"
 #include "vtkIdList.h"
-#include "vtkIntArray.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPointData.h"
 #include "vtkPoints.h"
 #include "vtkUnstructuredGrid.h"
+#include "vtkIdTypeArray.h"
 
-vtkCxxRevisionMacro(vtkConnectivityFilter, "$Revision: 1.71 $");
 vtkStandardNewMacro(vtkConnectivityFilter);
 
 // Construct with default extraction mode to extract largest regions.
 vtkConnectivityFilter::vtkConnectivityFilter()
 {
-  this->RegionSizes = vtkIntArray::New();
+  this->RegionSizes = vtkIdTypeArray::New();
   this->ExtractionMode = VTK_EXTRACT_LARGEST_REGION;
   this->ColorRegions = 0;
 
@@ -51,6 +52,9 @@ vtkConnectivityFilter::vtkConnectivityFilter()
   
   this->Seeds = vtkIdList::New();
   this->SpecifiedRegionIds = vtkIdList::New();
+
+  this->NewScalars = 0;
+  this->NewCellScalars = 0;
 }
 
 vtkConnectivityFilter::~vtkConnectivityFilter()
@@ -62,16 +66,26 @@ vtkConnectivityFilter::~vtkConnectivityFilter()
   this->SpecifiedRegionIds->Delete();
 }
 
-void vtkConnectivityFilter::Execute()
+int vtkConnectivityFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkUnstructuredGrid *output = vtkUnstructuredGrid::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkIdType numPts, numCells, cellId, newCellId, i, j, pt;
   vtkPoints *newPts;
   int id;
   int maxCellsInRegion;
   int largestRegionId = 0;
-  vtkDataSet *input= this->GetInput();
-  if (input == NULL) {return;}
-  vtkUnstructuredGrid *output = this->GetOutput();
   vtkPointData *pd=input->GetPointData(), *outputPD=output->GetPointData();
   vtkCellData *cd=input->GetCellData(), *outputCD=output->GetCellData();
   
@@ -83,7 +97,7 @@ void vtkConnectivityFilter::Execute()
   if ( (numPts=input->GetNumberOfPoints()) < 1 || numCells < 1 )
     {
     vtkDebugMacro(<<"No data to connect!");
-    return;
+    return 1;
     }
   output->Allocate(numCells,numCells);
 
@@ -116,8 +130,14 @@ void vtkConnectivityFilter::Execute()
     this->PointMap[i] = -1;
     }
 
-  this->NewScalars = vtkFloatArray::New();
+  this->NewScalars = vtkIdTypeArray::New();
+  this->NewScalars->SetName("RegionId");
   this->NewScalars->SetNumberOfTuples(numPts);
+
+  this->NewCellScalars = vtkIdTypeArray::New();
+  this->NewCellScalars->SetName("RegionId");
+  this->NewCellScalars->SetNumberOfTuples(numCells);
+
   newPts = vtkPoints::New();
   newPts->Allocate(numPts);
 
@@ -154,7 +174,7 @@ void vtkConnectivityFilter::Execute()
         {
         this->NumCellsInRegion = 0;
         this->Wave->InsertNextId(cellId);
-        this->TraverseAndMark ();
+        this->TraverseAndMark (input);
 
         if ( this->NumCellsInRegion > maxCellsInRegion )
           {
@@ -222,7 +242,7 @@ void vtkConnectivityFilter::Execute()
     this->UpdateProgress (0.5);
 
     //mark all seeded regions
-    this->TraverseAndMark ();
+    this->TraverseAndMark (input);
     this->RegionSizes->InsertValue(this->RegionNumber,this->NumCellsInRegion);
     this->UpdateProgress (0.9);
     }
@@ -252,8 +272,11 @@ void vtkConnectivityFilter::Execute()
     {
     int idx = outputPD->AddArray(this->NewScalars);
     outputPD->SetActiveAttribute(idx, vtkDataSetAttributes::SCALARS);
+    idx = outputCD->AddArray(this->NewCellScalars);
+    outputCD->SetActiveAttribute(idx, vtkDataSetAttributes::SCALARS);
     }
   this->NewScalars->Delete();
+  this->NewCellScalars->Delete();
 
   output->SetPoints(newPts);
   newPts->Delete();
@@ -269,11 +292,23 @@ void vtkConnectivityFilter::Execute()
       {
       if ( this->Visited[cellId] >= 0 )
         {
-        input->GetCellPoints(cellId, this->PointIds);
-        for (i=0; i < this->PointIds->GetNumberOfIds(); i++)
+        // special handling for polyhedron cells
+        if (vtkUnstructuredGrid::SafeDownCast(input) &&
+            input->GetCellType(cellId) == VTK_POLYHEDRON)
           {
-          id = this->PointMap[this->PointIds->GetId(i)];
-          this->PointIds->InsertId(i,id);
+          vtkUnstructuredGrid::SafeDownCast(input)->
+            GetFaceStream(cellId, this->PointIds);
+          vtkUnstructuredGrid::ConvertFaceStreamPointIds(this->PointIds, 
+                                                         this->PointMap);
+          }
+        else
+          {        
+          input->GetCellPoints(cellId, this->PointIds);
+          for (i=0; i < this->PointIds->GetNumberOfIds(); i++)
+            {
+            id = this->PointMap[this->PointIds->GetId(i)];
+            this->PointIds->InsertId(i,id);
+            }
           }
         newCellId = output->InsertNextCell(input->GetCellType(cellId),
                                            this->PointIds);
@@ -298,14 +333,26 @@ void vtkConnectivityFilter::Execute()
           }
         if ( inReg )
           {
-          input->GetCellPoints(cellId, this->PointIds);
-          for (i=0; i < this->PointIds->GetNumberOfIds(); i++)
+          // special handling for polyhedron cells
+          if (vtkUnstructuredGrid::SafeDownCast(input) &&
+              input->GetCellType(cellId) == VTK_POLYHEDRON)
             {
-            id = this->PointMap[this->PointIds->GetId(i)];
-            this->PointIds->InsertId(i,id);
+            vtkUnstructuredGrid::SafeDownCast(input)->
+              GetFaceStream(cellId, this->PointIds);
+            vtkUnstructuredGrid::ConvertFaceStreamPointIds(this->PointIds, 
+                                                           this->PointMap);
             }
-          newCellId =output->InsertNextCell(input->GetCellType(cellId),
-                                            this->PointIds);
+          else
+            {        
+            input->GetCellPoints(cellId, this->PointIds);
+            for (i=0; i < this->PointIds->GetNumberOfIds(); i++)
+              {
+              id = this->PointMap[this->PointIds->GetId(i)];
+              this->PointIds->InsertId(i,id);
+              }
+            }
+          newCellId = output->InsertNextCell(input->GetCellType(cellId),
+                                             this->PointIds);
           outputCD->CopyData(cd,cellId,newCellId);
           }
         }
@@ -317,11 +364,23 @@ void vtkConnectivityFilter::Execute()
       {
       if ( this->Visited[cellId] == largestRegionId )
         {
-        input->GetCellPoints(cellId, this->PointIds);
-        for (i=0; i < this->PointIds->GetNumberOfIds(); i++)
+        // special handling for polyhedron cells
+        if (vtkUnstructuredGrid::SafeDownCast(input) &&
+            input->GetCellType(cellId) == VTK_POLYHEDRON)
           {
-          id = this->PointMap[this->PointIds->GetId(i)];
-          this->PointIds->InsertId(i,id);
+          vtkUnstructuredGrid::SafeDownCast(input)->
+            GetFaceStream(cellId, this->PointIds);
+          vtkUnstructuredGrid::ConvertFaceStreamPointIds(this->PointIds, 
+                                                         this->PointMap);
+          }
+        else
+          {        
+          input->GetCellPoints(cellId, this->PointIds);
+          for (i=0; i < this->PointIds->GetNumberOfIds(); i++)
+            {
+            id = this->PointMap[this->PointIds->GetId(i)];
+            this->PointIds->InsertId(i,id);
+            }
           }
         newCellId = output->InsertNextCell(input->GetCellType(cellId),
                                            this->PointIds);
@@ -351,18 +410,17 @@ void vtkConnectivityFilter::Execute()
   vtkDebugMacro (<< "Total # of cells accounted for: " << count);
   vtkDebugMacro (<< "Extracted " << output->GetNumberOfCells() << " cells");
 
-  return;
+  return 1;
 }
 
 
 // Mark current cell as visited and assign region number.  Note:
 // traversal occurs across shared vertices.
 //
-void vtkConnectivityFilter::TraverseAndMark ()
+void vtkConnectivityFilter::TraverseAndMark (vtkDataSet *input)
 {
-  int i, j, k, cellId, numIds, ptId, numPts, numCells;
+  vtkIdType i, j, k, cellId, numIds, ptId, numPts, numCells;
   vtkIdList *tmpWave;
-  vtkDataSet *input= this->GetInput();
 
   while ( (numIds=this->Wave->GetNumberOfIds()) > 0 )
     {
@@ -371,6 +429,7 @@ void vtkConnectivityFilter::TraverseAndMark ()
       cellId = this->Wave->GetId(i);
       if ( this->Visited[cellId] < 0 )
         {
+        this->NewCellScalars->SetValue(cellId, this->RegionNumber);
         this->Visited[cellId] = this->RegionNumber;
         this->NumCellsInRegion++;
         input->GetCellPoints(cellId, this->PointIds);
@@ -381,8 +440,8 @@ void vtkConnectivityFilter::TraverseAndMark ()
           if ( this->PointMap[ptId=this->PointIds->GetId(j)] < 0 )
             {
             this->PointMap[ptId] = this->PointNumber++;
-            this->NewScalars->SetComponent(this->PointMap[ptId], 0,
-                                           this->RegionNumber);
+            this->NewScalars->SetValue(this->PointMap[ptId],
+                                       this->RegionNumber);
             }
 
           input->GetPointCells(ptId,this->CellIds);
@@ -488,6 +547,12 @@ void vtkConnectivityFilter::DeleteSpecifiedRegion(int id)
   this->SpecifiedRegionIds->DeleteId(id);
 }
 
+int vtkConnectivityFilter::FillInputPortInformation(int, vtkInformation *info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
+}
+
 void vtkConnectivityFilter::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
