@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkCursor3D.cxx,v $
+  Module:    vtkCursor3D.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,12 +15,13 @@
 #include "vtkCursor3D.h"
 
 #include "vtkCellArray.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPolyData.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkCursor3D, "$Revision: 1.43 $");
 vtkStandardNewMacro(vtkCursor3D);
 
 // Construct with model bounds = (-1,1,-1,1,-1,1), focal point = (0,0,0),
@@ -31,10 +32,15 @@ vtkCursor3D::vtkCursor3D()
 
   this->Focus = vtkPolyData::New();
   pts = vtkPoints::New();
-  pts->Allocate(1);
   pts->vtkPoints::InsertPoint(0, 0.0, 0.0, 0.0);
   this->Focus->SetPoints(pts);
   pts->Delete();
+  vtkCellArray* vert = vtkCellArray::New();
+  vert->InsertNextCell(1);
+  vert->InsertCellPoint(0);
+  this->Focus->SetVerts(vert);
+  vert->Delete();
+
 
   this->ModelBounds[0] = -1.0;
   this->ModelBounds[1] = 1.0;
@@ -54,6 +60,8 @@ vtkCursor3D::vtkCursor3D()
   this->ZShadows = 1;
   this->Wrap = 0;
   this->TranslationMode = 0;
+
+  this->SetNumberOfInputPorts(0);
 }
 
 vtkCursor3D::~vtkCursor3D()
@@ -61,17 +69,24 @@ vtkCursor3D::~vtkCursor3D()
   this->Focus->Delete();
 }
 
-void vtkCursor3D::Execute()
+int vtkCursor3D::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
 {
+  // get the info object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the ouptut
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   int i;
   int numPts=0, numLines=0;
   vtkPoints *newPts;
   vtkCellArray *newLines;
   double x[3];
   vtkIdType ptIds[2];
-  vtkPolyData *output = this->GetOutput();
-  
-  vtkDebugMacro(<<"Generating cursor");
 
   // Check bounding box and origin
   //
@@ -80,8 +95,10 @@ void vtkCursor3D::Execute()
     for (i=0; i<3; i++)
       {
       this->FocalPoint[i] = this->ModelBounds[2*i] + 
-             fmod((double)(this->FocalPoint[i]-this->ModelBounds[2*i]), 
-                  (double)(this->ModelBounds[2*i+1]-this->ModelBounds[2*i]));
+             fmod(static_cast<double>(
+                    this->FocalPoint[i]-this->ModelBounds[2*i]), 
+                  static_cast<double>(
+                    this->ModelBounds[2*i+1]-this->ModelBounds[2*i]));
       }
     } 
   else 
@@ -140,7 +157,7 @@ void vtkCursor3D::Execute()
     }
   else
     {
-    return;
+    return 1;
     }
 
   // Create axes
@@ -386,6 +403,8 @@ void vtkCursor3D::Execute()
 
   output->SetLines(newLines);
   newLines->Delete();
+
+  return 1;
 }
 
 // Set the boundary of the 3D cursor.
@@ -437,8 +456,10 @@ void vtkCursor3D::SetFocalPoint(double x[3])
     else if ( this->Wrap ) //wrap
       {
       this->FocalPoint[i] = this->ModelBounds[2*i] + 
-             fmod((double)(this->FocalPoint[i]-this->ModelBounds[2*i]), 
-                  (double)(this->ModelBounds[2*i+1]-this->ModelBounds[2*i]));
+             fmod(static_cast<double>(
+                    this->FocalPoint[i]-this->ModelBounds[2*i]), 
+                  static_cast<double>(
+                    this->ModelBounds[2*i+1]-this->ModelBounds[2*i]));
       }
     else //clamp
       {
