@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkDataSetTriangleFilter.cxx,v $
+  Module:    vtkDataSetTriangleFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,6 +19,8 @@
 #include "vtkGenericCell.h"
 #include "vtkIdList.h"
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkOrderedTriangulator.h"
 #include "vtkPointData.h"
@@ -26,8 +28,8 @@
 #include "vtkStructuredPoints.h"
 #include "vtkUnstructuredGrid.h"
 #include "vtkRectilinearGrid.h"
+#include "vtkUnsignedCharArray.h"
 
-vtkCxxRevisionMacro(vtkDataSetTriangleFilter, "$Revision: 1.22 $");
 vtkStandardNewMacro(vtkDataSetTriangleFilter);
 
 vtkDataSetTriangleFilter::vtkDataSetTriangleFilter()
@@ -35,6 +37,7 @@ vtkDataSetTriangleFilter::vtkDataSetTriangleFilter()
   this->Triangulator = vtkOrderedTriangulator::New();
   this->Triangulator->PreSortedOff();
   this->Triangulator->UseTemplatesOn();
+  this->TetrahedraOnly = 0;
 }
 
 vtkDataSetTriangleFilter::~vtkDataSetTriangleFilter()
@@ -43,32 +46,41 @@ vtkDataSetTriangleFilter::~vtkDataSetTriangleFilter()
   this->Triangulator = NULL;
 }
 
-void vtkDataSetTriangleFilter::Execute()
+int vtkDataSetTriangleFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkDataSet *input = this->GetInput();
-  if (!input)
-    {
-    return;
-    }
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkUnstructuredGrid *output = vtkUnstructuredGrid::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   if (input->IsA("vtkStructuredPoints") ||
       input->IsA("vtkStructuredGrid") || 
       input->IsA("vtkImageData") ||
       input->IsA("vtkRectilinearGrid"))
     {
-    this->StructuredExecute();
+    this->StructuredExecute(input, output);
     }
   else
     {
-    this->UnstructuredExecute();
+    this->UnstructuredExecute(input, output);
     }
 
   vtkDebugMacro(<<"Produced " << this->GetOutput()->GetNumberOfCells() << " cells");
+
+  return 1;
 }
 
-void vtkDataSetTriangleFilter::StructuredExecute()
+void vtkDataSetTriangleFilter::StructuredExecute(vtkDataSet *input,
+                                                 vtkUnstructuredGrid *output)
 {
-  vtkDataSet *input = this->GetInput();
-  vtkUnstructuredGrid *output = this->GetOutput();
   int dimensions[3], i, j, k, l, m;
   vtkIdType newCellId, inId;
   vtkGenericCell *cell = vtkGenericCell::New();
@@ -113,11 +125,11 @@ void vtkDataSetTriangleFilter::StructuredExecute()
   dimensions[1] = dimensions[1] - 1;
   dimensions[2] = dimensions[2] - 1;
 
-  vtkIdType numSlices=dimensions[2];
+  vtkIdType numSlices = ( dimensions[2] > 0 ? dimensions[2] : 1 );
   int abort=0;
-  for (k = 0; k < dimensions[2] && !abort; k++)
+  for (k = 0; k < numSlices && !abort; k++)
     {
-    this->UpdateProgress((double)k / numSlices);
+    this->UpdateProgress(static_cast<double>(k) / numSlices);
     abort = this->GetAbortExecute();
 
     for (j = 0; j < dimensions[1]; j++)
@@ -151,16 +163,19 @@ void vtkDataSetTriangleFilter::StructuredExecute()
           case 4:
             type = VTK_TETRA;     break;
           }
-        for (l = 0; l < numSimplices; l++ )
+        if (!this->TetrahedraOnly || type == VTK_TETRA)
           {
-          for (m = 0; m < dim; m++)
+          for (l = 0; l < numSimplices; l++ )
             {
-            pts[m] = cellPtIds->GetId(dim*l+m);
-            }
-          // copy cell data
-          newCellId = output->InsertNextCell(type, dim, pts);
-          outCD->CopyData(inCD, inId, newCellId);
-          }//for all simplices
+            for (m = 0; m < dim; m++)
+              {
+              pts[m] = cellPtIds->GetId(dim*l+m);
+              }
+            // copy cell data
+            newCellId = output->InsertNextCell(type, dim, pts);
+            outCD->CopyData(inCD, inId, newCellId);
+            }//for all simplices
+          }
         }//i dimension
       }//j dimension
     }//k dimension
@@ -180,10 +195,10 @@ void vtkDataSetTriangleFilter::StructuredExecute()
 // to create templates on the fly. Once the templates are created then they
 // are used to produce the final triangulation.
 //
-void vtkDataSetTriangleFilter::UnstructuredExecute()
+void vtkDataSetTriangleFilter::UnstructuredExecute(vtkDataSet *dataSetInput,
+                                                   vtkUnstructuredGrid *output)
 {
-  vtkPointSet *input = (vtkPointSet*) this->GetInput(); //has to be
-  vtkUnstructuredGrid *output = this->GetOutput();
+  vtkPointSet *input = static_cast<vtkPointSet*>(dataSetInput); //has to be
   vtkIdType numCells = input->GetNumberOfCells();
   vtkGenericCell *cell;
   vtkIdType newCellId, j;
@@ -203,12 +218,52 @@ void vtkDataSetTriangleFilter::UnstructuredExecute()
     return;
     }
 
+  vtkUnstructuredGrid * inUgrid =
+    vtkUnstructuredGrid::SafeDownCast(dataSetInput);
+  if (inUgrid)
+    {
+    //avoid doing cell simplification if all cells are already simplices
+    vtkUnsignedCharArray* cellTypes = inUgrid->GetCellTypesArray();
+    if (cellTypes)
+      {
+      int allsimplices = 1;
+      for (vtkIdType cellId = 0; cellId < cellTypes->GetSize() && allsimplices; cellId++)
+        {
+        switch (cellTypes->GetValue(cellId))
+          {
+          case VTK_TETRA:
+            break;
+          case VTK_VERTEX:
+          case VTK_LINE:
+          case VTK_TRIANGLE:          
+            if (this->TetrahedraOnly)
+              {
+              allsimplices = 0; //don't shallowcopy need to stip non tets
+              }
+            break;
+          default:
+            allsimplices = 0;
+            break;
+          }
+        }
+      if (allsimplices)
+        {
+        output->ShallowCopy(input);
+        return;
+        }
+      }
+    }
+
   cell = vtkGenericCell::New();
   cellPts = vtkPoints::New();
   cellPtIds = vtkIdList::New();
 
   // Create an array of points
-  outCD->CopyAllocate(inCD,input->GetNumberOfCells()*5);
+  vtkCellData *tempCD = vtkCellData::New();
+  tempCD->ShallowCopy(inCD);
+  tempCD->SetActiveGlobalIds(NULL);
+
+  outCD->CopyAllocate(tempCD, input->GetNumberOfCells()*5);
   output->Allocate(input->GetNumberOfCells()*5);
   
   // Points are passed through
@@ -221,14 +276,35 @@ void vtkDataSetTriangleFilter::UnstructuredExecute()
     {
     if ( !(cellId % updateTime) )
       {
-      this->UpdateProgress((double)cellId / numCells);
+      this->UpdateProgress(static_cast<double>(cellId) / numCells);
       abort = this->GetAbortExecute();
       }
 
     input->GetCell(cellId, cell);
     dim = cell->GetCellDimension();
 
-    if ( dim == 3 ) //use ordered triangulation
+    if (cell->GetCellType() == VTK_POLYHEDRON) //polyhedron
+      {
+      dim = 4;
+      cell->Triangulate(0, cellPtIds, cellPts);
+      numPts = cellPtIds->GetNumberOfIds();
+    
+      numSimplices = numPts / dim;
+      type = VTK_TETRA;
+
+      for ( j=0; j < numSimplices; j++ )
+        {
+        for (k=0; k<dim; k++)
+          {
+          pts[k] = cellPtIds->GetId(dim*j+k);
+          }
+        // copy cell data
+        newCellId = output->InsertNextCell(type, dim, pts);
+        outCD->CopyData(tempCD, cellId, newCellId);
+        }
+      }
+
+    else if ( dim == 3 ) //use ordered triangulation
       {
       numPts = cell->GetNumberOfPoints();
       double *p, *pPtr=cell->GetParametricCoords();
@@ -255,11 +331,11 @@ void vtkDataSetTriangleFilter::UnstructuredExecute()
         
       for (j=0; j < numTets; j++)
         {
-        outCD->CopyData(inCD, cellId, ncells+j);
+        outCD->CopyData(tempCD, cellId, ncells+j);
         }
       }
 
-    else //2D or lower dimension
+    else if (!this->TetrahedraOnly) //2D or lower dimension
       {
       dim++;
       cell->Triangulate(0, cellPtIds, cellPts);
@@ -285,7 +361,7 @@ void vtkDataSetTriangleFilter::UnstructuredExecute()
           }
         // copy cell data
         newCellId = output->InsertNextCell(type, dim, pts);
-        outCD->CopyData(inCD, cellId, newCellId);
+        outCD->CopyData(tempCD, cellId, newCellId);
         }
       } //if 2D or less cell
     } //for all cells
@@ -293,13 +369,22 @@ void vtkDataSetTriangleFilter::UnstructuredExecute()
   // Update output
   output->Squeeze();
   
+  tempCD->Delete();
+
   cellPts->Delete();
   cellPtIds->Delete();
   cell->Delete();
 }
 
+int vtkDataSetTriangleFilter::FillInputPortInformation(int, vtkInformation *info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
+}
+
 void vtkDataSetTriangleFilter::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
+  os << indent << "TetrahedraOnly: " << (this->TetrahedraOnly ? "On":"Off") << "\n";
 }
 
