@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkExtractGeometry.cxx,v $
+  Module:    vtkExtractGeometry.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,14 +19,16 @@
 #include "vtkFloatArray.h"
 #include "vtkIdList.h"
 #include "vtkImplicitFunction.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkUnstructuredGrid.h"
 
-vtkCxxRevisionMacro(vtkExtractGeometry, "$Revision: 1.52 $");
 vtkStandardNewMacro(vtkExtractGeometry);
 vtkCxxSetObjectMacro(vtkExtractGeometry,ImplicitFunction,vtkImplicitFunction);
 
+//----------------------------------------------------------------------------
 // Construct object with ExtractInside turned on.
 vtkExtractGeometry::vtkExtractGeometry(vtkImplicitFunction *f)
 {
@@ -38,8 +40,10 @@ vtkExtractGeometry::vtkExtractGeometry(vtkImplicitFunction *f)
     
   this->ExtractInside = 1;
   this->ExtractBoundaryCells = 0;
+  this->ExtractOnlyBoundaryCells = 0;
 }
 
+//----------------------------------------------------------------------------
 vtkExtractGeometry::~vtkExtractGeometry()
 {
   this->SetImplicitFunction(NULL);
@@ -61,8 +65,22 @@ unsigned long vtkExtractGeometry::GetMTime()
   return mTime;
 }
 
-void vtkExtractGeometry::Execute()
+//----------------------------------------------------------------------------
+int vtkExtractGeometry::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkUnstructuredGrid *output = vtkUnstructuredGrid::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkIdType ptId, numPts, numCells, i, cellId, newCellId, newId, *pointMap;
   vtkIdList *cellPts;
   vtkCell *cell;
@@ -71,23 +89,25 @@ void vtkExtractGeometry::Execute()
   double multiplier;
   vtkPoints *newPts;
   vtkIdList *newCellPts;
-  vtkDataSet *input = this->GetInput();
   vtkPointData *pd = input->GetPointData();
   vtkCellData *cd = input->GetCellData();
-  vtkUnstructuredGrid *output = this->GetOutput();
   vtkPointData *outputPD = output->GetPointData();
   vtkCellData *outputCD = output->GetCellData();
   int npts;
-  numCells = input->GetNumberOfCells();
   
   vtkDebugMacro(<< "Extracting geometry");
 
   if ( ! this->ImplicitFunction )
     {
     vtkErrorMacro(<<"No implicit function specified");
-    return;
+    return 1;
     }
 
+  // As this filter is doing a subsetting operation, set the Copy Tuple flag
+  // for GlobalIds array so that, if present, it will be copied to the output.
+  outputPD->CopyGlobalIdsOn();
+  outputCD->CopyGlobalIdsOn();
+
   newCellPts = vtkIdList::New();
   newCellPts->Allocate(VTK_CELL_SIZE);
 
@@ -134,23 +154,20 @@ void vtkExtractGeometry::Execute()
   else
     {
     // To extract boundary cells, we have to create supplemental information
-    if ( this->ExtractBoundaryCells )
-      {
-      double val;
-      newScalars = vtkFloatArray::New();
-      newScalars->SetNumberOfValues(numPts);
+    double val;
+    newScalars = vtkFloatArray::New();
+    newScalars->SetNumberOfValues(numPts);
 
-      for (ptId=0; ptId < numPts; ptId++ )
+    for (ptId=0; ptId < numPts; ptId++ )
+      {
+      input->GetPoint(ptId, x);
+      val = this->ImplicitFunction->FunctionValue(x) * multiplier;
+      newScalars->SetValue(ptId, val);
+      if ( val < 0.0 )
         {
-        input->GetPoint(ptId, x);
-        val = this->ImplicitFunction->FunctionValue(x) * multiplier;
-        newScalars->SetValue(ptId, val);
-        if ( val < 0.0 )
-          {
-          newId = newPts->InsertNextPoint(x);
-          pointMap[ptId] = newId;
-          outputPD->CopyData(pd,ptId,newId);
-          }
+        newId = newPts->InsertNextPoint(x);
+        pointMap[ptId] = newId;
+        outputPD->CopyData(pd,ptId,newId);
         }
       }
     }
@@ -191,7 +208,22 @@ void vtkExtractGeometry::Execute()
           npts++;
           }
         }
-      if ( npts > 0 )
+      int extraction_condition = 0;
+      if ( this->ExtractOnlyBoundaryCells )
+        {
+        if ( ( npts > 0 ) && ( npts != numCellPts ) )
+          {
+          extraction_condition = 1;
+          }
+        }
+      else
+        {
+        if ( npts > 0 )
+          {
+          extraction_condition = 1;
+          }
+        }
+      if ( extraction_condition ) 
         {
         for ( i=0; i < numCellPts; i++ )
           {
@@ -208,8 +240,31 @@ void vtkExtractGeometry::Execute()
         }//a boundary or interior cell
       }//if mapping boundary cells
       
-    if ( npts >= numCellPts || (this->ExtractBoundaryCells && npts > 0) )
+    int extraction_condition = 0;
+    if ( this->ExtractOnlyBoundaryCells )
       {
+      if ( npts != numCellPts && (this->ExtractBoundaryCells && npts > 0) )
+        {
+        extraction_condition = 1;
+        }
+      }
+    else
+      {
+      if ( npts >= numCellPts || (this->ExtractBoundaryCells && npts > 0) )
+        {
+        extraction_condition = 1;
+        }
+      }
+    if ( extraction_condition )
+      {
+      // special handling for polyhedron cells
+      if (vtkUnstructuredGrid::SafeDownCast(input) &&
+          cell->GetCellType() == VTK_POLYHEDRON)
+        {
+        newCellPts->Reset();
+        vtkUnstructuredGrid::SafeDownCast(input)->GetFaceStream(cellId, newCellPts);
+        vtkUnstructuredGrid::ConvertFaceStreamPointIds(newCellPts, pointMap);
+        }
       newCellId = output->InsertNextCell(cell->GetCellType(),newCellPts);
       outputCD->CopyData(cd,cellId,newCellId);
       }
@@ -228,16 +283,28 @@ void vtkExtractGeometry::Execute()
     }
 
   output->Squeeze();
+
+  return 1;
 }
 
+//----------------------------------------------------------------------------
+int vtkExtractGeometry::FillInputPortInformation(int, vtkInformation *info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
+}
+
+//----------------------------------------------------------------------------
 void vtkExtractGeometry::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
 
   os << indent << "Implicit Function: " 
-     << (void *)this->ImplicitFunction << "\n";
+     << static_cast<void *>(this->ImplicitFunction) << "\n";
   os << indent << "Extract Inside: " 
      << (this->ExtractInside ? "On\n" : "Off\n");
   os << indent << "Extract Boundary Cells: " 
      << (this->ExtractBoundaryCells ? "On\n" : "Off\n");
+  os << indent << "Extract Only Boundary Cells: " 
+     << (this->ExtractOnlyBoundaryCells ? "On\n" : "Off\n");
 }
