@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkGlyph3D.cxx,v $
+  Module:    vtkGlyph3D.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,21 +14,27 @@
 =========================================================================*/
 #include "vtkGlyph3D.h"
 
+#include "vtkCellData.h"
 #include "vtkCell.h"
 #include "vtkDataSet.h"
 #include "vtkFloatArray.h"
 #include "vtkIdList.h"
 #include "vtkIdTypeArray.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
+#include "vtkSmartPointer.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkTransform.h"
 #include "vtkUnsignedCharArray.h"
 
-vtkCxxRevisionMacro(vtkGlyph3D, "$Revision: 1.112 $");
 vtkStandardNewMacro(vtkGlyph3D);
+vtkCxxSetObjectMacro(vtkGlyph3D, SourceTransform, vtkTransform);
 
+//----------------------------------------------------------------------------
 // Construct object with scaling on, scaling mode is by scalar value,
 // scale factor = 1.0, the range is (0,1), orient geometry is on, and
 // orientation is by vector. Clamping and indexing are turned off. No
@@ -45,73 +51,114 @@ vtkGlyph3D::vtkGlyph3D()
   this->VectorMode = VTK_USE_VECTOR;
   this->Clamping = 0;
   this->IndexMode = VTK_INDEXING_OFF;
-  this->NumberOfRequiredInputs = 1;
   this->GeneratePointIds = 0;
   this->PointIdsName = NULL;
   this->SetPointIdsName("InputPointIds");
-  this->InputScalarsSelection = NULL;
-  this->InputVectorsSelection = NULL;
-  this->InputNormalsSelection = NULL;
+  this->SetNumberOfInputPorts(2);
+  this->FillCellData = 0;
+  this->SourceTransform = 0;
+
+  // by default process active point scalars
+  this->SetInputArrayToProcess(0,0,0,vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::SCALARS);
+  // by default process active point vectors
+  this->SetInputArrayToProcess(1,0,0,vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::VECTORS);
+  // by default process active point normals
+  this->SetInputArrayToProcess(2,0,0,vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::NORMALS);
+  // by default process active point scalars
+  this->SetInputArrayToProcess(3,0,0,vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::SCALARS);
 }
 
+//----------------------------------------------------------------------------
 vtkGlyph3D::~vtkGlyph3D()
 {
   if (this->PointIdsName)
     {
     delete []PointIdsName;
     }
-  this->SetInputScalarsSelection(NULL);
-  this->SetInputVectorsSelection(NULL);
-  this->SetInputNormalsSelection(NULL);
+  this->SetSourceTransform(NULL);
 }
 
-void vtkGlyph3D::Execute()
+//----------------------------------------------------------------------------
+unsigned long vtkGlyph3D::GetMTime()
 {
+  unsigned long mTime=this->Superclass::GetMTime();
+  unsigned long time;
+  if ( this->SourceTransform != NULL )
+    {
+    time = this->SourceTransform ->GetMTime();
+    mTime = ( time > mTime ? time : mTime );
+    }
+  return mTime;
+}
+
+//----------------------------------------------------------------------------
+int vtkGlyph3D::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
+{
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkPointData *pd;
-  vtkDataArray *inScalars;
+  vtkDataArray *inSScalars; // Scalars for Scaling
+  vtkDataArray *inCScalars; // Scalars for Coloring
   vtkDataArray *inVectors;
   int requestedGhostLevel;
   unsigned char* inGhostLevels=0;
   vtkDataArray *inNormals, *sourceNormals = NULL;
+  vtkDataArray *sourceTCoords = NULL;
   vtkIdType numPts, numSourcePts, numSourceCells, inPtId, i;
   int index;
   vtkPoints *sourcePts = NULL;
+  vtkSmartPointer<vtkPoints> transformedSourcePts = vtkSmartPointer<vtkPoints>::New();
   vtkPoints *newPts;
   vtkDataArray *newScalars=NULL;
   vtkDataArray *newVectors=NULL;
   vtkDataArray *newNormals=NULL;
-  double x[3], v[3], vNew[3], s = 0.0, vMag = 0.0, value;
+  vtkDataArray *newTCoords = NULL;
+  double x[3], v[3], vNew[3], s = 0.0, vMag = 0.0, value, tc[3];
   vtkTransform *trans = vtkTransform::New();
   vtkCell *cell;
   vtkIdList *cellPts;
   int npts;
   vtkIdList *pts;
-  vtkIdType ptIncr, cellId;
-  int haveVectors, haveNormals;
+  vtkIdType ptIncr, cellIncr, cellId;
+  int haveVectors, haveNormals, haveTCoords = 0;
   double scalex,scaley,scalez, den;
-  vtkPolyData *output = this->GetOutput();
-  vtkPointData *outputPD = output->GetPointData();
-  vtkDataSet *input = this->GetInput();
-  int numberOfSources = this->GetNumberOfSources();
+  vtkPointData* outputPD = output->GetPointData();
+  vtkCellData* outputCD = output->GetCellData();
+  int numberOfSources = this->GetNumberOfInputConnections(1);
   vtkPolyData *defaultSource = NULL;
   vtkIdTypeArray *pointIds=0;
+  vtkPolyData *source = 0;
 
   vtkDebugMacro(<<"Generating glyphs");
 
   pts = vtkIdList::New();
   pts->Allocate(VTK_CELL_SIZE);
 
-  if (!input)
-    {
-    vtkErrorMacro(<<"No input");
-    return;
-    }
-
   pd = input->GetPointData();
-  inScalars = pd->GetScalars(this->InputScalarsSelection);
-  inVectors = pd->GetVectors(this->InputVectorsSelection);
-  inNormals = pd->GetNormals(this->InputNormalsSelection);
-
+  inSScalars = this->GetInputArrayToProcess(0,inputVector);
+  inVectors = this->GetInputArrayToProcess(1,inputVector);
+  inNormals = this->GetInputArrayToProcess(2,inputVector);
+  inCScalars = this->GetInputArrayToProcess(3,inputVector);
+  if (inCScalars == NULL)
+    {
+    inCScalars = inSScalars;
+    }
+  
   vtkDataArray* temp = 0;
   if (pd)
     {
@@ -124,11 +171,11 @@ void vtkGlyph3D::Execute()
     }
   else
     {
-    inGhostLevels = ((vtkUnsignedCharArray*)temp)->GetPointer(0);
+    inGhostLevels =static_cast<vtkUnsignedCharArray *>(temp)->GetPointer(0);
     }
 
-  requestedGhostLevel = output->GetUpdateGhostLevel();
-  
+  requestedGhostLevel =
+    outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS());
   
   numPts = input->GetNumberOfPoints();
   if (numPts < 1)
@@ -136,7 +183,7 @@ void vtkGlyph3D::Execute()
     vtkDebugMacro(<<"No points to glyph!");
     pts->Delete();
     trans->Delete();
-    return;
+    return 1;
     }
 
   // Check input for consistency
@@ -156,17 +203,17 @@ void vtkGlyph3D::Execute()
     haveVectors = 0;
     }
 
-  if ( (this->IndexMode == VTK_INDEXING_BY_SCALAR && !inScalars) ||
+  if ( (this->IndexMode == VTK_INDEXING_BY_SCALAR && !inSScalars) ||
        (this->IndexMode == VTK_INDEXING_BY_VECTOR &&
        ((!inVectors && this->VectorMode == VTK_USE_VECTOR) ||
         (!inNormals && this->VectorMode == VTK_USE_NORMAL))) )
     {
-    if ( this->GetSource(0) == NULL )
+    if ( this->GetSource(0, inputVector[1]) == NULL )
       {
       vtkErrorMacro(<<"Indexing on but don't have data to index with");
       pts->Delete();
       trans->Delete();
-      return;
+      return 1;
       }
     else
       {
@@ -179,8 +226,9 @@ void vtkGlyph3D::Execute()
   //
   outputPD->CopyVectorsOff();
   outputPD->CopyNormalsOff();
+  outputPD->CopyTCoordsOff();
 
-  if (!this->GetSource(0))
+  if (!this->GetSource(0, inputVector[1]))
     {
     defaultSource = vtkPolyData::New();
     defaultSource->Allocate();
@@ -207,17 +255,18 @@ void vtkGlyph3D::Execute()
     haveNormals = 1;
     for (numSourcePts=numSourceCells=i=0; i < numberOfSources; i++)
       {
-      if ( this->GetSource(i) != NULL )
+      source = this->GetSource(i, inputVector[1]);
+      if ( source != NULL )
         {
-        if (this->GetSource(i)->GetNumberOfPoints() > numSourcePts)
+        if (source->GetNumberOfPoints() > numSourcePts)
           {
-          numSourcePts = this->GetSource(i)->GetNumberOfPoints();
+          numSourcePts = source->GetNumberOfPoints();
           }
-        if (this->GetSource(i)->GetNumberOfCells() > numSourceCells)
+        if (source->GetNumberOfCells() > numSourceCells)
           {
-          numSourceCells = this->GetSource(i)->GetNumberOfCells();
+          numSourceCells = source->GetNumberOfCells();
           }
-        if ( !(sourceNormals = this->GetSource(i)->GetPointData()->GetNormals()) )
+        if ( !(sourceNormals = source->GetPointData()->GetNormals()) )
           {
           haveNormals = 0;
           }
@@ -226,11 +275,12 @@ void vtkGlyph3D::Execute()
     }
   else
     {
-    sourcePts = this->GetSource(0)->GetPoints();
+    source = this->GetSource(0, inputVector[1]);
+    sourcePts = source->GetPoints();
     numSourcePts = sourcePts->GetNumberOfPoints();
-    numSourceCells = this->GetSource(0)->GetNumberOfCells();
+    numSourceCells = source->GetNumberOfCells();
 
-    sourceNormals = this->GetSource(0)->GetPointData()->GetNormals();
+    sourceNormals = source->GetPointData()->GetNormals();
     if ( sourceNormals )
       {
       haveNormals = 1;
@@ -240,9 +290,23 @@ void vtkGlyph3D::Execute()
       haveNormals = 0;
       }
 
+    sourceTCoords = source->GetPointData()->GetTCoords();
+    if (sourceTCoords)
+      {
+      haveTCoords = 1;
+      }
+    else
+      {
+      haveTCoords = 0;
+      }
+    
     // Prepare to copy output.
-    pd = this->GetSource(0)->GetPointData();
+    pd = input->GetPointData();
     outputPD->CopyAllocate(pd,numPts*numSourcePts);
+    if (this->FillCellData)
+      {
+      outputCD->CopyAllocate(pd,numPts*numSourceCells);
+      }
     }
 
   newPts = vtkPoints::New();
@@ -253,22 +317,23 @@ void vtkGlyph3D::Execute()
     pointIds->SetName(this->PointIdsName);
     pointIds->Allocate(numPts*numSourcePts);
     outputPD->AddArray(pointIds);
+    pointIds->Delete();
     }
-  if ( this->ColorMode == VTK_COLOR_BY_SCALAR && inScalars )
+  if ( this->ColorMode == VTK_COLOR_BY_SCALAR && inCScalars )
     {
-    newScalars = inScalars->NewInstance();
-    newScalars->SetNumberOfComponents(inScalars->GetNumberOfComponents());
-    newScalars->Allocate(inScalars->GetNumberOfComponents()*numPts*numSourcePts);
-    newScalars->SetName(inScalars->GetName());
+    newScalars = inCScalars->NewInstance();
+    newScalars->SetNumberOfComponents(inCScalars->GetNumberOfComponents());
+    newScalars->Allocate(inCScalars->GetNumberOfComponents()*numPts*numSourcePts);
+    newScalars->SetName(inCScalars->GetName());
     }
-  else if ( (this->ColorMode == VTK_COLOR_BY_SCALE) && inScalars)
+  else if ( (this->ColorMode == VTK_COLOR_BY_SCALE) && inSScalars)
     {
     newScalars = vtkFloatArray::New();
     newScalars->Allocate(numPts*numSourcePts);
     newScalars->SetName("GlyphScale");
     if (this->ScaleMode == VTK_SCALE_BY_SCALAR)
       {
-      newScalars->SetName(inScalars->GetName());
+      newScalars->SetName(inSScalars->GetName());
       }
     }
   else if ( (this->ColorMode == VTK_COLOR_BY_VECTOR) && haveVectors)
@@ -291,7 +356,15 @@ void vtkGlyph3D::Execute()
     newNormals->Allocate(3*numPts*numSourcePts);
     newNormals->SetName("Normals");
     }
-
+  if (haveTCoords)
+    {
+    newTCoords = vtkFloatArray::New();
+    int numComps = sourceTCoords->GetNumberOfComponents();
+    newTCoords->SetNumberOfComponents(numComps);
+    newTCoords->Allocate(numComps*numPts*numSourcePts);
+    newTCoords->SetName("TCoords");
+    }
+    
   // Setting up for calls to PolyData::InsertNextCell()
   if (this->IndexMode != VTK_INDEXING_OFF )
     {
@@ -299,19 +372,24 @@ void vtkGlyph3D::Execute()
     }
   else
     {
-    output->Allocate(this->GetSource(0),3*numPts*numSourceCells,numPts*numSourceCells);
+    output->Allocate(this->GetSource(0, inputVector[1]),
+                     3*numPts*numSourceCells, numPts*numSourceCells);
     }
 
+  transformedSourcePts->SetDataTypeToDouble();
+  transformedSourcePts->Allocate(numSourcePts);
+
   // Traverse all Input points, transforming Source points and copying
   // point attributes.
   //
   ptIncr=0;
+  cellIncr=0;
   for (inPtId=0; inPtId < numPts; inPtId++)
     {
     scalex = scaley = scalez = 1.0;
     if ( ! (inPtId % 10000) )
       {
-      this->UpdateProgress ((double)inPtId/numPts);
+      this->UpdateProgress(static_cast<double>(inPtId)/numPts);
       if (this->GetAbortExecute())
         {
         break;
@@ -319,9 +397,9 @@ void vtkGlyph3D::Execute()
       }
 
     // Get the scalar and vector data
-    if ( inScalars )
+    if ( inSScalars )
       {
-      s = inScalars->GetComponent(inPtId, 0);
+      s = inSScalars->GetComponent(inPtId, 0);
       if ( this->ScaleMode == VTK_SCALE_BY_SCALAR ||
            this->ScaleMode == VTK_DATA_SCALING_OFF )
         {
@@ -331,14 +409,27 @@ void vtkGlyph3D::Execute()
     
     if ( haveVectors )
       {
-      if ( this->VectorMode == VTK_USE_NORMAL )
+      vtkDataArray *array3D = this->VectorMode == VTK_USE_NORMAL? inNormals : inVectors;
+      if(array3D->GetNumberOfComponents()>3)
         {
-        inNormals->GetTuple(inPtId, v);
-        }
-      else
-        {
-        inVectors->GetTuple(inPtId, v);
+        vtkErrorMacro(<<"vtkDataArray "<<array3D->GetName()<<" has more than 3 components.\n");
+        pts->Delete();
+        trans->Delete();
+        if(newPts)
+          {
+          newPts->Delete();
+          }
+        if(newVectors)
+          {
+          newVectors->Delete();
+          }
+        return 0;
         }
+
+      v[0] = 0;
+      v[1] = 0;
+      v[2] = 0;
+      array3D->GetTuple(inPtId, v);
       vMag = vtkMath::Norm(v);
       if ( this->ScaleMode == VTK_SCALE_BY_VECTORCOMPONENTS )
         {
@@ -382,21 +473,22 @@ void vtkGlyph3D::Execute()
         value = vMag;
         }
       
-      index = (int) ((double)(value - this->Range[0]) * numberOfSources / den);
+      index = static_cast<int>((value - this->Range[0])*numberOfSources / den);
       index = (index < 0 ? 0 :
               (index >= numberOfSources ? (numberOfSources-1) : index));
       
-      if ( this->GetSource(index) != NULL )
+      source = this->GetSource(index, inputVector[1]);
+      if ( source != NULL )
         {
-        sourcePts = this->GetSource(index)->GetPoints();
-        sourceNormals = this->GetSource(index)->GetPointData()->GetNormals();
+        sourcePts = source->GetPoints();
+        sourceNormals = source->GetPointData()->GetNormals();
         numSourcePts = sourcePts->GetNumberOfPoints();
-        numSourceCells = this->GetSource(index)->GetNumberOfCells();
+        numSourceCells = source->GetNumberOfCells();
         }
       }
-    
+
     // Make sure we're not indexing into empty glyph
-    if ( this->GetSource(index) == NULL )
+    if ( this->GetSource(index, inputVector[1]) == NULL )
       {
       continue;
       }
@@ -407,20 +499,25 @@ void vtkGlyph3D::Execute()
     // ghostLevel > 0.  I am leaving this over glyphing here because
     // it make a nice example (sphereGhost.tcl) to show the 
     // point ghost levels with the glyph filter.  I am not certain 
-    // of the usefullness of point ghost levels over 1, but I will have
+    // of the usefulness of point ghost levels over 1, but I will have
     // to think about it.
     if (inGhostLevels && inGhostLevels[inPtId] > requestedGhostLevel)
       {
       continue;
       }
+
+    if (!this->IsPointVisible(input, inPtId))
+      {
+      continue;
+      }
     
     // Now begin copying/transforming glyph
     trans->Identity();
-    
+
     // Copy all topology (transformation independent)
     for (cellId=0; cellId < numSourceCells; cellId++)
       {
-      cell = this->GetSource(index)->GetCell(cellId);
+      cell = this->GetSource(index, inputVector[1])->GetCell(cellId);
       cellPts = cell->GetPointIds();
       npts = cellPts->GetNumberOfIds();
       for (pts->Reset(), i=0; i < npts; i++) 
@@ -456,28 +553,34 @@ void vtkGlyph3D::Execute()
           vNew[0] = (v[0]+vMag) / 2.0;
           vNew[1] = v[1] / 2.0;
           vNew[2] = v[2] / 2.0;
-          trans->RotateWXYZ((double)180.0,vNew[0],vNew[1],vNew[2]);
+          trans->RotateWXYZ(180.0,vNew[0],vNew[1],vNew[2]);
           }
         }
       }
     
-    // determine scale factor from scalars if appropriate
-    if ( inScalars )
+    if (haveTCoords)
       {
-      // Copy scalar value
-      if (this->ColorMode == VTK_COLOR_BY_SCALE)
+      for (i = 0; i < numSourcePts; i++)
         {
-        for (i=0; i < numSourcePts; i++) 
-          {
-          newScalars->InsertTuple(i+ptIncr, &scalex); // = scaley = scalez
-          }
+        sourceTCoords->GetTuple(i, tc);
+        newTCoords->InsertTuple(i+ptIncr, tc);
         }
-      else if (this->ColorMode == VTK_COLOR_BY_SCALAR)
+      }
+    
+    // determine scale factor from scalars if appropriate
+    // Copy scalar value
+    if (inSScalars && (this->ColorMode == VTK_COLOR_BY_SCALE))
+      {
+      for (i=0; i < numSourcePts; i++)
         {
-        for (i=0; i < numSourcePts; i++)
-          {
-          outputPD->CopyTuple(inScalars, newScalars, inPtId, ptIncr+i);
-          }
+        newScalars->InsertTuple(i+ptIncr, &scalex); // = scaley = scalez
+        }
+      }
+    else if (inCScalars && (this->ColorMode == VTK_COLOR_BY_SCALAR))
+      {
+      for (i=0; i < numSourcePts; i++)
+        {
+        outputPD->CopyTuple(inCScalars, newScalars, inPtId, ptIncr+i);
         }
       }
     if (haveVectors && this->ColorMode == VTK_COLOR_BY_VECTOR)
@@ -516,9 +619,18 @@ void vtkGlyph3D::Execute()
         }
       trans->Scale(scalex,scaley,scalez);
       }
-    
+
     // multiply points and normals by resulting matrix
-    trans->TransformPoints(sourcePts,newPts);
+    if (this->SourceTransform)
+      {
+      transformedSourcePts->Reset();
+      this->SourceTransform->TransformPoints(sourcePts, transformedSourcePts);
+      trans->TransformPoints(transformedSourcePts, newPts);
+      }
+    else
+      {
+      trans->TransformPoints(sourcePts,newPts);
+      }
     
     if ( haveNormals )
       {
@@ -530,7 +642,14 @@ void vtkGlyph3D::Execute()
       {
       for (i=0; i < numSourcePts; i++)
         {
-        outputPD->CopyData(pd,i,ptIncr+i);
+        outputPD->CopyData(pd,inPtId,ptIncr+i);
+        }
+      if (this->FillCellData)
+        {
+        for (i=0; i < numSourceCells; i++)
+          {
+          outputCD->CopyData(pd,inPtId,cellIncr+i);
+          }
         }
       }
 
@@ -544,6 +663,7 @@ void vtkGlyph3D::Execute()
       }
 
     ptIncr += numSourcePts;
+    cellIncr += numSourceCells;
     } 
   
   // Update ourselves and release memory
@@ -570,39 +690,47 @@ void vtkGlyph3D::Execute()
     newNormals->Delete();
     }
 
+  if (newTCoords)
+    {
+    outputPD->SetTCoords(newTCoords);
+    newTCoords->Delete();
+    }
+  
   output->Squeeze();
   trans->Delete();
   pts->Delete();
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
-// Since indexing determines size of outputs, EstimatedWholeMemorySize is
-// truly an estimate.  Ignore Indexing (although for a best estimate we
-// should average the size of the sources instead of using 0).
-void vtkGlyph3D::ExecuteInformation()
+// Specify a source object at a specified table location.
+void vtkGlyph3D::SetSourceConnection(int id, vtkAlgorithmOutput* algOutput)
 {
-  if (this->GetInput() == NULL)
+  if (id < 0)
     {
-    vtkErrorMacro("Missing input");
+    vtkErrorMacro("Bad index " << id << " for source.");
     return;
     }
+
+  int numConnections = this->GetNumberOfInputConnections(1);
+  if (id < numConnections)
+    {
+    this->SetNthInputConnection(1, id, algOutput);
+    }
+  else if (id == numConnections && algOutput)
+    {
+    this->AddInputConnection(1, algOutput);
+    }
+  else if (algOutput)
+    {
+    vtkWarningMacro("The source id provided is larger than the maximum "
+                    "source id, using " << numConnections << " instead.");
+    this->AddInputConnection(1, algOutput);
+    }
 }
 
-
-// Set the number of source objects in the glyph table. This should be
-// done prior to specifying more than one source.
-void vtkGlyph3D::SetNumberOfSources(int num)
-{
-  // one more because input has index 0.
-  this->SetNumberOfInputs(num+1);
-}
-
-int vtkGlyph3D::GetNumberOfSources()
-{
-  // one less because input has index 0.
-  return this->NumberOfInputs - 1;
-}
-
+//----------------------------------------------------------------------------
 // Specify a source object at a specified table location.
 void vtkGlyph3D::SetSource(int id, vtkPolyData *pd)
 {
@@ -611,22 +739,46 @@ void vtkGlyph3D::SetSource(int id, vtkPolyData *pd)
     vtkErrorMacro("Bad index " << id << " for source.");
     return;
     }
-  this->vtkProcessObject::SetNthInput(id + 1, pd);
-}
 
-// Get a pointer to a source object at a specified table location.
-vtkPolyData *vtkGlyph3D::GetSource(int id)
-{
-  if ( id < 0 || id >= this->GetNumberOfSources() )
+  int numConnections = this->GetNumberOfInputConnections(1);
+  vtkAlgorithmOutput *algOutput = 0;
+  if (pd)
     {
-    return NULL;
+    algOutput = pd->GetProducerPort();
     }
   else
     {
-    return (vtkPolyData *)this->Inputs[id+1];
+    vtkErrorMacro("Cannot set NULL source.");
+    return;
+    }
+
+  if (id < numConnections)
+    {
+    if (algOutput)
+      {
+      this->SetNthInputConnection(1, id, algOutput);
+      }
+    }
+  else if (id == numConnections && algOutput)
+    {
+    this->AddInputConnection(1, algOutput);
     }
 }
 
+//----------------------------------------------------------------------------
+// Get a pointer to a source object at a specified table location.
+vtkPolyData *vtkGlyph3D::GetSource(int id)
+{
+  if ( id < 0 || id >= this->GetNumberOfInputConnections(1) )
+    {
+    return NULL;
+    }
+
+  return vtkPolyData::SafeDownCast(
+    this->GetExecutive()->GetInputData(1, id));
+}
+
+//----------------------------------------------------------------------------
 void vtkGlyph3D::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
@@ -639,7 +791,7 @@ void vtkGlyph3D::PrintSelf(ostream& os, vtkIndent indent)
 
   os << indent << "Color Mode: " << this->GetColorModeAsString() << endl;
 
-  if ( this->GetNumberOfSources() < 2 )
+  if ( this->GetNumberOfInputConnections(1) < 2 )
     {
     if ( this->GetSource(0) != NULL )
       {
@@ -652,7 +804,7 @@ void vtkGlyph3D::PrintSelf(ostream& os, vtkIndent indent)
     }
   else
     {
-    os << indent << "A table of " << this->GetNumberOfSources() << " glyphs has been defined\n";
+    os << indent << "A table of " << this->GetNumberOfInputConnections(1) << " glyphs has been defined\n";
     }
 
   os << indent << "Scaling: " << (this->Scaling ? "On\n" : "Off\n");
@@ -690,32 +842,76 @@ void vtkGlyph3D::PrintSelf(ostream& os, vtkIndent indent)
     {
     os << "Indexing off\n";
     }
-  os << indent << "InputScalarsSelection: " 
-     << (this->InputScalarsSelection ? this->InputScalarsSelection : "(none)") << "\n";
-  os << indent << "InputVectorsSelection: " 
-     << (this->InputVectorsSelection ? this->InputVectorsSelection : "(none)") << "\n";
-  os << indent << "InputNormalsSelection: " 
-     << (this->InputNormalsSelection ? this->InputNormalsSelection : "(none)") << "\n";
+
+  os << indent << "Fill Cell Data: " << (this->FillCellData ? "On\n" : "Off\n");
+
+  os << indent << "SourceTransform: ";
+  if (this->SourceTransform)
+    {
+    os << endl;
+    this->SourceTransform->PrintSelf(os, indent.GetNextIndent());
+    }
+  else
+    {
+    os << "(none)" << endl;
+    }
 }
 
-void vtkGlyph3D::ComputeInputUpdateExtents( vtkDataObject *output )
+int vtkGlyph3D::RequestUpdateExtent(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkPolyData *outPd;
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *sourceInfo = inputVector[1]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
 
-  if (this->GetInput() == NULL)
+  if (sourceInfo)
     {
-    vtkErrorMacro("Missing input");
-    return;
+    sourceInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER(),
+                    0);
+    sourceInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES(),
+                    1);
+    sourceInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS(),
+                    0);
     }
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER(),
+              outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER()));
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES(),
+              outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES()));
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS(),
+              outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS()));
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::EXACT_EXTENT(), 1);
 
-  output = output;
-  outPd = this->GetOutput();
-  if (this->GetSource())
+  return 1;
+}
+
+//----------------------------------------------------------------------------
+vtkPolyData* vtkGlyph3D::GetSource(int idx, vtkInformationVector *sourceInfo)
+{
+  vtkInformation *info = sourceInfo->GetInformationObject(idx);
+  if (!info)
     {
-    this->GetSource()->SetUpdateExtent(0, 1, 0);
+    return NULL;
     }
-  this->GetInput()->SetUpdateExtent(outPd->GetUpdatePiece(),
-                                    outPd->GetUpdateNumberOfPieces(),
-                                    outPd->GetUpdateGhostLevel());
-  this->GetInput()->RequestExactExtentOn();
+  return vtkPolyData::SafeDownCast(info->Get(vtkDataObject::DATA_OBJECT()));
+}
+
+//----------------------------------------------------------------------------
+int vtkGlyph3D::FillInputPortInformation(int port, vtkInformation *info)
+{
+  if (port == 0)
+    {
+    info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+    return 1;
+    }
+  else if (port == 1)
+    {
+    info->Set(vtkAlgorithm::INPUT_IS_REPEATABLE(), 1);
+    info->Set(vtkAlgorithm::INPUT_IS_OPTIONAL(), 1);
+    info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkPolyData");
+    return 1;
+    }
+  return 0;
 }
