@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageDataGeometryFilter.cxx,v $
+  Module:    vtkImageDataGeometryFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,11 +17,12 @@
 #include "vtkCellArray.h"
 #include "vtkCellData.h"
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 
-vtkCxxRevisionMacro(vtkImageDataGeometryFilter, "$Revision: 1.11 $");
 vtkStandardNewMacro(vtkImageDataGeometryFilter);
 
 // Construct with initial extent of all the data
@@ -33,14 +34,30 @@ vtkImageDataGeometryFilter::vtkImageDataGeometryFilter()
   this->Extent[3] = VTK_LARGE_INTEGER;
   this->Extent[4] = 0;
   this->Extent[5] = VTK_LARGE_INTEGER;
+  this->ThresholdCells  = 0;
+  this->ThresholdValue  = 0.0;
+  this->OutputTriangles = 0;
 }
 
-void vtkImageDataGeometryFilter::Execute()
+int vtkImageDataGeometryFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkImageData *input = vtkImageData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   int *dims, dimension, dir[3], diff[3];
-  int i, j, k, extent[6];
+  int i, j, k, extent[6], s, threshok;
   vtkIdType idx, startIdx, startCellIdx;
-  vtkIdType ptIds[4], cellId;
+  vtkIdType ptIds[4], cellId, triIds[3];
   vtkPoints *newPts=0;
   vtkCellArray *newVerts=0;
   vtkCellArray *newLines=0;
@@ -49,25 +66,21 @@ void vtkImageDataGeometryFilter::Execute()
   int offset[3], numPolys;
   double x[3];
   vtkPointData *pd, *outPD;
+  vtkDataArray *pointScalars;
   vtkCellData *cd, *outCD;
-  vtkImageData *input = this->GetInput();
-  vtkPolyData *output = this->GetOutput();
-  if (input==NULL)
-    {
-    return;
-    }
   vtkDebugMacro(<< "Extracting structured points geometry");
 
   pd = input->GetPointData();
   outPD = output->GetPointData();
   cd = input->GetCellData();
   outCD = output->GetCellData();
+  pointScalars=pd->GetScalars();
 //  this->PointData.CopyNormalsOff();
   dims = input->GetDimensions();
 
   if (dims[0] <= 0 || dims[1] <= 0 || dims[2] <= 0)
     {
-    return;
+    return 1;
     }
 
 //
@@ -228,7 +241,14 @@ void vtkImageDataGeometryFilter::Execute()
       newPts = vtkPoints::New();
       newPts->Allocate(totPoints);
       newPolys = vtkCellArray::New();
-      newPolys->Allocate(newLines->EstimateSize(numPolys,4));
+      if (this->OutputTriangles)
+        {
+        newPolys->Allocate(2*newLines->EstimateSize(numPolys,3));
+        }
+      else
+        {
+        newPolys->Allocate(newLines->EstimateSize(numPolys,4));
+        }
       outPD->CopyAllocate(pd,totPoints);
       outCD->CopyAllocate(cd,numPolys);
 //
@@ -290,8 +310,41 @@ void vtkImageDataGeometryFilter::Execute()
           ptIds[1] = ptIds[0] + 1;
           ptIds[2] = ptIds[1] + diff[dir[0]] + 1;
           ptIds[3] = ptIds[2] - 1;
-          cellId = newPolys->InsertNextCell(4,ptIds);
-          outCD->CopyData(cd,idx,cellId);
+          if (this->ThresholdCells)
+            {
+            threshok = false;
+            for (s=0; !threshok && s<4; s++)
+              {
+              if (pointScalars->GetComponent(ptIds[s],0)>this->ThresholdValue)
+                {
+                  threshok = true;
+                  break;
+                }
+              }
+            if (threshok)
+              {
+              if (this->OutputTriangles)
+                {
+                triIds[0] = ptIds[0];
+                triIds[1] = ptIds[2];
+                triIds[2] = ptIds[3];
+                cellId = newPolys->InsertNextCell(3,ptIds);
+                outCD->CopyData(cd,idx,cellId);
+                cellId = newPolys->InsertNextCell(3,triIds);
+                outCD->CopyData(cd,idx,cellId);
+                }
+              else
+                {
+                cellId = newPolys->InsertNextCell(4,ptIds);
+                outCD->CopyData(cd,idx,cellId);
+                }
+              }
+            }
+          else
+            {
+            cellId = newPolys->InsertNextCell(4,ptIds);
+            outCD->CopyData(cd,idx,cellId);
+            }
           }
         pos += offset[1];
         }
@@ -320,7 +373,7 @@ void vtkImageDataGeometryFilter::Execute()
       offset[0] = dims[0];
       offset[1] = dims[0]*dims[1];
 
-      for (pos=startIdx, k=0; k < (diff[2]+1); k++) 
+      for (k=0; k < (diff[2]+1); k++) 
         {
         for (j=0; j < (diff[1]+1); j++) 
           {
@@ -364,6 +417,8 @@ void vtkImageDataGeometryFilter::Execute()
     output->SetPolys(newPolys);
     newPolys->Delete();
     }
+
+  return 1;
 }
 
 void vtkImageDataGeometryFilter::SetExtent(int iMin, int iMax, 
@@ -408,13 +463,21 @@ void vtkImageDataGeometryFilter::SetExtent(int extent[6])
     }
 }
 
-void vtkImageDataGeometryFilter::PrintSelf(ostream& os,
-                                                  vtkIndent indent)
+int vtkImageDataGeometryFilter::FillInputPortInformation(int, vtkInformation *info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkImageData");
+  return 1;
+}
+
+void vtkImageDataGeometryFilter::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
 
   os << indent << "Extent: \n";
-  os << indent << "  Imin,Imax: (" << this->Extent[0] << ", " << this->Extent[1] << ")\n";
   os << indent << "  Jmin,Jmax: (" << this->Extent[2] << ", " << this->Extent[3] << ")\n";
   os << indent << "  Kmin,Kmax: (" << this->Extent[4] << ", " << this->Extent[5] << ")\n";
+  os << indent << "  Imin,Imax: (" << this->Extent[0] << ", " << this->Extent[1] << ")\n";
+  os << indent << "OutputTriangles " << this->OutputTriangles << "\n";
+  os << indent << "ThresholdValue " << this->ThresholdValue << "\n";
+  os << indent << "ThresholdCells " << this->ThresholdCells << "\n";
 }
