@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkMaskFields.cxx,v $
+  Module:    vtkMaskFields.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,23 +17,19 @@
 #include "vtkCellData.h"
 #include "vtkDataSet.h"
 #include "vtkDataSetAttributes.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
+#include <ctype.h>
 
-vtkCxxRevisionMacro(vtkMaskFields, "$Revision: 1.6 $");
 vtkStandardNewMacro(vtkMaskFields);
 
 char vtkMaskFields::FieldLocationNames[3][12] 
 = { "OBJECT_DATA",
     "POINT_DATA",
     "CELL_DATA" };
-
-char vtkMaskFields::AttributeNames[vtkDataSetAttributes::NUM_ATTRIBUTES][10] 
-=  { "SCALARS",
-     "VECTORS",
-     "NORMALS",
-     "TCOORDS",
-     "TENSORS" };
+char vtkMaskFields::AttributeNames[vtkDataSetAttributes::NUM_ATTRIBUTES][10]  = { {0} };
 
 vtkMaskFields::vtkMaskFields()
 {
@@ -42,6 +38,19 @@ vtkMaskFields::vtkMaskFields()
   this->NumberOfFieldFlags = 0;
   this->CopyAllOn();
 
+  //convert the attribute names to uppercase for local use
+  if (vtkMaskFields::AttributeNames[0][0] == 0) 
+    {
+    for (int i = 0; i < vtkDataSetAttributes::NUM_ATTRIBUTES; i++)
+      {
+      int l = static_cast<int>(strlen(vtkDataSetAttributes::GetAttributeTypeAsString(i)));
+      for (int c = 0; c < l && c < 10; c++)
+        {
+        vtkMaskFields::AttributeNames[i][c] = 
+          toupper(vtkDataSetAttributes::GetAttributeTypeAsString(i)[c]);
+        }
+      }
+    }
 }
 
 vtkMaskFields::~vtkMaskFields()
@@ -356,10 +365,20 @@ int vtkMaskFields::GetFlag(int arrayType, int loc)
     }
 }
 
-void vtkMaskFields::Execute()
+int vtkMaskFields::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkDataSet *input = static_cast<vtkDataSet*>(this->GetInput());
-  vtkDataSet *output = static_cast<vtkDataSet*>(this->GetOutput());
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkDataSet *output = vtkDataSet::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
 
   // This has to be here because it initializes all field data.
   output->CopyStructure( input );
@@ -373,36 +392,26 @@ void vtkMaskFields::Execute()
 
   } else if (!this->CopyFields && this->CopyAttributes) {
     vtkDebugMacro("Copying only attributes.");
-    output->GetPointData()->CopyAllOff();
-    output->GetPointData()->CopyScalarsOn();
-    output->GetPointData()->CopyVectorsOn();
-    output->GetPointData()->CopyTensorsOn();
-    output->GetPointData()->CopyNormalsOn();
-    output->GetPointData()->CopyTCoordsOn();
 
+    output->GetPointData()->CopyAllOff();
     output->GetCellData()->CopyAllOff();
-    output->GetCellData()->CopyScalarsOn();
-    output->GetCellData()->CopyVectorsOn();
-    output->GetCellData()->CopyTensorsOn();
-    output->GetCellData()->CopyNormalsOn();
-    output->GetCellData()->CopyTCoordsOn();
+    int ai;
+    for (ai = 0; ai < vtkDataSetAttributes::NUM_ATTRIBUTES; ai++)
+      {
+      output->GetPointData()->SetCopyAttribute(1, ai);
+      output->GetCellData()->SetCopyAttribute(1, ai);
+      }
 
   } else if (this->CopyFields && !this->CopyAttributes) {
     vtkDebugMacro("Copying only fields.");
     output->GetPointData()->CopyAllOn();
-    output->GetPointData()->CopyScalarsOff();
-    output->GetPointData()->CopyVectorsOff();
-    output->GetPointData()->CopyTensorsOff();
-    output->GetPointData()->CopyNormalsOff();
-    output->GetPointData()->CopyTCoordsOff();
-
     output->GetCellData()->CopyAllOn();
-    output->GetCellData()->CopyScalarsOff();
-    output->GetCellData()->CopyVectorsOff();
-    output->GetCellData()->CopyTensorsOff();
-    output->GetCellData()->CopyNormalsOff();
-    output->GetCellData()->CopyTCoordsOff();
-
+    int ai;
+    for (ai = 0; ai < vtkDataSetAttributes::NUM_ATTRIBUTES; ai++)
+      {
+      output->GetPointData()->SetCopyAttribute(0, ai);
+      output->GetCellData()->SetCopyAttribute(0, ai);
+      }
     output->GetFieldData()->CopyAllOn();
 
   } else if (!this->CopyFields && !this->CopyAttributes) {
@@ -457,11 +466,15 @@ void vtkMaskFields::Execute()
       }
   }
 
-  // Pass all. (data object's field data is passed by the
-  // superclass after this method)
+  // Pass all.
+  if ( output->GetFieldData() && input->GetFieldData() )
+    {
+    output->GetFieldData()->PassData( input->GetFieldData() );
+    }
   output->GetPointData()->PassData( input->GetPointData() );
   output->GetCellData()->PassData( input->GetCellData() );
 
+  return 1;
 }
 
 void vtkMaskFields::PrintSelf(ostream& os, vtkIndent indent)
