@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkOutlineCornerFilter.cxx,v $
+  Module:    vtkOutlineCornerFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,11 +15,12 @@
 #include "vtkOutlineCornerFilter.h"
 
 #include "vtkDataSet.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkOutlineCornerSource.h"
 #include "vtkPolyData.h"
 
-vtkCxxRevisionMacro(vtkOutlineCornerFilter, "$Revision: 1.9 $");
 vtkStandardNewMacro(vtkOutlineCornerFilter);
 
 vtkOutlineCornerFilter::vtkOutlineCornerFilter ()
@@ -37,38 +38,41 @@ vtkOutlineCornerFilter::~vtkOutlineCornerFilter ()
     }
 }
 
-void vtkOutlineCornerFilter::Execute()
+int vtkOutlineCornerFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkPolyData *output = this->GetOutput();
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
   
   vtkDebugMacro(<< "Creating dataset outline");
 
   //
   // Let OutlineCornerSource do all the work
   //
-  this->OutlineCornerSource->SetBounds(this->GetInput()->GetBounds());
+  this->OutlineCornerSource->SetBounds(input->GetBounds());
   this->OutlineCornerSource->SetCornerFactor(this->GetCornerFactor());
   this->OutlineCornerSource->Update();
 
   output->CopyStructure(this->OutlineCornerSource->GetOutput());
 
+  return 1;
 }
 
-
-void vtkOutlineCornerFilter::ExecuteInformation()
+int vtkOutlineCornerFilter::FillInputPortInformation(int, vtkInformation *info)
 {
-  vtkDebugMacro(<< "Creating dataset outline");
-
-  //
-  // Let OutlineCornerSource do all the work
-  //
-  
-  this->vtkSource::ExecuteInformation();
-
-  this->OutlineCornerSource->UpdateInformation();
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
 }
 
-
 void vtkOutlineCornerFilter::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
