@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPointSource.h,v $
+  Module:    vtkPointSource.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -18,21 +18,22 @@
 // of points within a specified radius about a specified center point. 
 // By default location of the points is random within the sphere. It is
 // also possible to generate random points only on the surface of the
-// sphere.
+// sphere. The output PolyData has the specified number of points and
+// 1 cell - a vtkPolyVertex containing all of the points.
 
 #ifndef __vtkPointSource_h
 #define __vtkPointSource_h
 
-#include "vtkPolyDataSource.h"
+#include "vtkPolyDataAlgorithm.h"
 
 #define VTK_POINT_UNIFORM   1
 #define VTK_POINT_SHELL     0
 
-class VTK_GRAPHICS_EXPORT vtkPointSource : public vtkPolyDataSource 
+class VTK_GRAPHICS_EXPORT vtkPointSource : public vtkPolyDataAlgorithm 
 {
 public:
   static vtkPointSource *New();
-  vtkTypeRevisionMacro(vtkPointSource,vtkPolyDataSource);
+  vtkTypeMacro(vtkPointSource,vtkPolyDataAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
   
   // Description:
@@ -67,8 +68,7 @@ protected:
   vtkPointSource(vtkIdType numPts=10);
   ~vtkPointSource() {};
 
-  void Execute();
-  void ExecuteInformation();
+  int RequestData(vtkInformation *, vtkInformationVector **, vtkInformationVector *);
 
   vtkIdType NumberOfPoints;
   double Center[3];
