@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkProgrammableAttributeDataFilter.cxx,v $
+  Module:    vtkProgrammableAttributeDataFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,10 +16,12 @@
 
 #include "vtkCellData.h"
 #include "vtkDataSetCollection.h"
+#include "vtkGarbageCollector.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 
-vtkCxxRevisionMacro(vtkProgrammableAttributeDataFilter, "$Revision: 1.19 $");
 vtkStandardNewMacro(vtkProgrammableAttributeDataFilter);
 
 vtkProgrammableAttributeDataFilter::vtkProgrammableAttributeDataFilter()
@@ -37,8 +39,11 @@ vtkProgrammableAttributeDataFilter::~vtkProgrammableAttributeDataFilter()
     {
     (*this->ExecuteMethodArgDelete)(this->ExecuteMethodArg);
     }
-  this->InputList->Delete();
-  this->InputList = NULL;
+  if (this->InputList)
+    {
+    this->InputList->Delete();
+    this->InputList = NULL;
+    }
 }
 
 // Add a dataset to the list of data to process.
@@ -63,7 +68,8 @@ void vtkProgrammableAttributeDataFilter::RemoveInput(vtkDataSet *ds)
 
 // Specify the function to use to operate on the point attribute data. Note
 // that the function takes a single (void *) argument.
-void vtkProgrammableAttributeDataFilter::SetExecuteMethod(void (*f)(void *), void *arg)
+void vtkProgrammableAttributeDataFilter::SetExecuteMethod(
+  void (*f)(void *), void *arg)
 {
   if ( f != this->ExecuteMethod || arg != this->ExecuteMethodArg )
     {
@@ -79,7 +85,8 @@ void vtkProgrammableAttributeDataFilter::SetExecuteMethod(void (*f)(void *), voi
 }
 
 // Set the arg delete method. This is used to free user memory.
-void vtkProgrammableAttributeDataFilter::SetExecuteMethodArgDelete(void (*f)(void *))
+void vtkProgrammableAttributeDataFilter::SetExecuteMethodArgDelete(
+  void (*f)(void *))
 {
   if ( f != this->ExecuteMethodArgDelete)
     {
@@ -88,10 +95,20 @@ void vtkProgrammableAttributeDataFilter::SetExecuteMethodArgDelete(void (*f)(voi
     }
 }
 
-void vtkProgrammableAttributeDataFilter::Execute()
+int vtkProgrammableAttributeDataFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkDataSet *input = this->GetInput();
-  vtkDataSet *output = this->GetOutput();
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkDataSet *output = vtkDataSet::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
 
   vtkDebugMacro(<<"Executing programmable point data filter");
 
@@ -107,6 +124,8 @@ void vtkProgrammableAttributeDataFilter::Execute()
     {
     (*this->ExecuteMethod)(this->ExecuteMethodArg);
     }
+
+  return 1;
 }
 
 void vtkProgrammableAttributeDataFilter::PrintSelf(ostream& os, vtkIndent indent)
@@ -126,5 +145,11 @@ void vtkProgrammableAttributeDataFilter::PrintSelf(ostream& os, vtkIndent indent
     }
 }
 
-
-
+//----------------------------------------------------------------------------
+void
+vtkProgrammableAttributeDataFilter
+::ReportReferences(vtkGarbageCollector* collector)
+{
+  this->Superclass::ReportReferences(collector);
+  vtkGarbageCollectorReport(collector, this->InputList, "InputList");
+}
