@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkProjectedTexture.cxx,v $
+  Module:    vtkProjectedTexture.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,11 +16,12 @@
 
 #include "vtkDataSet.h"
 #include "vtkFloatArray.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 
-vtkCxxRevisionMacro(vtkProjectedTexture, "$Revision: 1.28 $");
 vtkStandardNewMacro(vtkProjectedTexture);
 
 // Description:
@@ -77,8 +78,21 @@ void vtkProjectedTexture::SetFocalPoint(double x, double y, double z)
   this->FocalPoint[2] = z;
 }
 
-void vtkProjectedTexture::Execute()
+int vtkProjectedTexture::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkDataSet *output = vtkDataSet::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   double tcoords[2];
   vtkIdType numPts;
   vtkFloatArray *newTCoords;
@@ -87,9 +101,7 @@ void vtkProjectedTexture::Execute()
   double proj;
   double rightv[3], upv[3], diff[3];
   double sScale, tScale, sOffset, tOffset, sSize, tSize, s, t;
-  vtkDataSet *input = this->GetInput();
   double p[3];
-  vtkDataSet *output = this->GetOutput();
 
   vtkDebugMacro(<<"Generating texture coordinates!");
 
@@ -103,6 +115,7 @@ void vtkProjectedTexture::Execute()
   //
 
   newTCoords = vtkFloatArray::New();
+  newTCoords->SetName("ProjectedTextureCoordinates");
   newTCoords->SetNumberOfComponents(2);
   newTCoords->SetNumberOfTuples(numPts);
 
@@ -222,6 +235,8 @@ void vtkProjectedTexture::Execute()
   
   output->GetPointData()->SetTCoords(newTCoords);
   newTCoords->Delete();
+
+  return 1;
 }
 
 void vtkProjectedTexture::PrintSelf(ostream& os, vtkIndent indent)
