@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkProjectedTexture.h,v $
+  Module:    vtkProjectedTexture.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -29,16 +29,16 @@
 #ifndef __vtkProjectedTexture_h
 #define __vtkProjectedTexture_h
 
-#include "vtkDataSetToDataSetFilter.h"
+#include "vtkDataSetAlgorithm.h"
 
 #define VTK_PROJECTED_TEXTURE_USE_PINHOLE 0
 #define VTK_PROJECTED_TEXTURE_USE_TWO_MIRRORS 1
 
-class VTK_GRAPHICS_EXPORT vtkProjectedTexture : public vtkDataSetToDataSetFilter 
+class VTK_GRAPHICS_EXPORT vtkProjectedTexture : public vtkDataSetAlgorithm 
 {
 public:
   static vtkProjectedTexture *New();
-  vtkTypeRevisionMacro(vtkProjectedTexture,vtkDataSetToDataSetFilter);
+  vtkTypeMacro(vtkProjectedTexture,vtkDataSetAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -70,15 +70,20 @@ public:
   // Get the normalized orientation vector of the projector.
   vtkGetVectorMacro(Orientation,double,3);
   
+  // Description:
   // Set/Get the up vector of the projector.
   vtkSetVector3Macro(Up,double);
   vtkGetVectorMacro(Up,double,3);
 
+  // Description:
   // Set/Get the aspect ratio of a perpendicular cross-section of the
   // the projector's frustum.  The aspect ratio consists of three 
   // numbers:  (x, y, z), where x is the width of the 
   // frustum, y is the height, and z is the perpendicular
   // distance from the focus of the projector.
+  //
+  // For example, if the source of the image is a pinhole camera with
+  // view angle A, then you could set x=1, y=1, z=1/tan(A).
   vtkSetVector3Macro(AspectRatio,double);
   vtkGetVectorMacro(AspectRatio,double,3);
 
@@ -96,7 +101,7 @@ protected:
   vtkProjectedTexture();
   ~vtkProjectedTexture() {};
 
-  void Execute();
+  int RequestData(vtkInformation *, vtkInformationVector **, vtkInformationVector *);
   void ComputeNormal();
 
   int CameraMode;
