@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkQuadricClustering.cxx,v $
+  Module:    vtkQuadricClustering.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,18 +16,36 @@
 
 #include "vtkCellArray.h"
 #include "vtkCellData.h"
+#include "vtkExecutive.h"
 #include "vtkFeatureEdges.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 #include "vtkTimerLog.h"
 #include "vtkTriangle.h"
+#include <vtksys/hash_set.hxx> // keep track of inserted triangles
 
-vtkCxxRevisionMacro(vtkQuadricClustering, "$Revision: 1.63 $");
 vtkStandardNewMacro(vtkQuadricClustering);
 
 //----------------------------------------------------------------------------
+// PIMPLd STL set for keeping track of inserted cells
+struct vtkQuadricClusteringIdTypeHash {
+  size_t operator()(vtkIdType val) const { return static_cast<size_t>(val); }
+};
+class vtkQuadricClusteringCellSet : public vtksys::hash_set<vtkIdType, vtkQuadricClusteringIdTypeHash> {};
+typedef vtkQuadricClusteringCellSet::iterator vtkQuadricClusteringCellSetIterator;
+
+
+//----------------------------------------------------------------------------
+// Construct with default NumberOfDivisions to 50, DivisionSpacing to 1
+// in all (x,y,z) directions. AutoAdjustNumberOfDivisions is set to ON.
+// ComputeNumberOfDivisions to OFF. UseFeatureEdges and UseFeaturePoints
+// are set to OFF by default
+// The default behavior is also to compute an optimal position in each
+// bin to produce the output triangles (this is also recommended)
 vtkQuadricClustering::vtkQuadricClustering()
 {
   this->Bounds[0] = this->Bounds[1] = this->Bounds[2] = 0.0;
@@ -55,12 +73,13 @@ vtkQuadricClustering::vtkQuadricClustering()
 
   this->UseInputPoints = 0;
 
+  this->PreventDuplicateCells = 1;
+  this->CellSet = NULL;
+  this->NumberOfBins = 0;
+
   this->OutputTriangleArray = NULL;
   this->OutputLines = NULL;
 
-  // Overide superclass so that append can be called directly.
-  this->NumberOfRequiredInputs = 0;
-
   // Used for matching boundaries.
   this->FeatureEdges = vtkFeatureEdges::New();
   this->FeatureEdges->FeatureEdgesOff();
@@ -69,6 +88,9 @@ vtkQuadricClustering::vtkQuadricClustering()
 
   this->InCellCount = this->OutCellCount = 0;
   this->CopyCellData = 0;
+
+  this->GetInformation()->Set(vtkAlgorithm::PRESERVES_RANGES(), 1);
+  this->GetInformation()->Set(vtkAlgorithm::PRESERVES_BOUNDS(), 1);
 }
 
 //----------------------------------------------------------------------------
@@ -78,6 +100,11 @@ vtkQuadricClustering::~vtkQuadricClustering()
   this->FeatureEdges = NULL;
   this->FeaturePoints->Delete();
   this->FeaturePoints = NULL;
+  if (this->CellSet)
+    {
+    delete this->CellSet;
+    this->CellSet = NULL;
+    }
   if (this->QuadricArray)
     {
     delete [] this->QuadricArray;
@@ -96,21 +123,37 @@ vtkQuadricClustering::~vtkQuadricClustering()
 }
 
 //----------------------------------------------------------------------------
-void vtkQuadricClustering::Execute()
+int vtkQuadricClustering::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkPolyData *input = this->GetInput();
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkPolyData *input = 0;
+  if (inInfo)
+    {
+    input = vtkPolyData::SafeDownCast(
+      inInfo->Get(vtkDataObject::DATA_OBJECT()));
+    }
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkTimerLog *tlog=NULL;
 
-  if (input == NULL || (input->GetNumberOfPoints() == 0))
+  if (!input || (input->GetNumberOfPoints() == 0))
     {
     // The user may be calling StartAppend, Append, and EndAppend explicitly.
-    return;
+    return 1;
     }
   
   if (input->CheckAttributes())
     {
     // avoid crashing if input is not all we expect (is not consistent).
-    return;
+    return 1;
     }
 
   if (this->Debug)
@@ -129,10 +172,13 @@ void vtkQuadricClustering::Execute()
     double factor = pow(((double)numDiv/(double)target),0.33333);
     this->NumberOfDivisions[0] = 
       (int)(0.5+(double)(this->NumberOfXDivisions)/factor);  
+    this->NumberOfDivisions[0] = (this->NumberOfDivisions[0] > 0 ? this->NumberOfDivisions[0] : 1);
     this->NumberOfDivisions[1] = 
       (int)(0.5+(double)(this->NumberOfYDivisions)/factor);  
+    this->NumberOfDivisions[1] = (this->NumberOfDivisions[1] > 0 ? this->NumberOfDivisions[1] : 1);
     this->NumberOfDivisions[2] = 
       (int)(0.5+(double)(this->NumberOfZDivisions)/factor);  
+    this->NumberOfDivisions[2] = (this->NumberOfDivisions[2] > 0 ? this->NumberOfDivisions[2] : 1);
     }
   else
     {
@@ -141,18 +187,21 @@ void vtkQuadricClustering::Execute()
     this->NumberOfDivisions[2] = this->NumberOfZDivisions;
     }
 
+  this->UpdateProgress(.01);
+
   this->StartAppend(input->GetBounds());
+  this->UpdateProgress(.2);
   this->SliceSize = this->NumberOfDivisions[0]*this->NumberOfDivisions[1];
 
   this->Append(input);
   if (this->UseFeatureEdges)
     { // Adjust bin points that contain boundary edges.
-    this->AppendFeatureQuadrics(this->GetInput());
+    this->AppendFeatureQuadrics(input, output);
     }
 
   if (this->UseInputPoints)
     {
-    this->EndAppendUsingPoints(input);
+    this->EndAppendUsingPoints(input, output);
     }
   else
     {
@@ -172,6 +221,8 @@ void vtkQuadricClustering::Execute()
     vtkDebugMacro(<<"Execution took: "<<tlog->GetElapsedTime()<<" seconds.");
     tlog->Delete();
     }  
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -179,6 +230,14 @@ void vtkQuadricClustering::StartAppend(double *bounds)
 {
   vtkIdType i;
 
+  // If there are duplicate triangles. remove them
+  if ( this->PreventDuplicateCells )
+    {
+    this->CellSet = new vtkQuadricClusteringCellSet;
+    this->NumberOfBins = 
+      this->NumberOfDivisions[0]*this->NumberOfDivisions[1]*this->NumberOfDivisions[2];
+    }
+
   // Copy over the bounds.
   for (i = 0; i < 6; ++i)
     {
@@ -201,9 +260,9 @@ void vtkQuadricClustering::StartAppend(double *bounds)
     this->Bounds[1] = this->Bounds[0] + (x * this->DivisionSpacing[0]);
     this->Bounds[3] = this->Bounds[2] + (y * this->DivisionSpacing[1]);
     this->Bounds[5] = this->Bounds[4] + (z * this->DivisionSpacing[2]);
-    this->NumberOfDivisions[0] = (int)x;
-    this->NumberOfDivisions[1] = (int)y;
-    this->NumberOfDivisions[2] = (int)z;
+    this->NumberOfDivisions[0] = (int)x > 0 ? (int)x : 1;
+    this->NumberOfDivisions[1] = (int)y > 0 ? (int)y : 1;
+    this->NumberOfDivisions[2] = (int)z > 0 ? (int)z : 1;
     }
   else
     {
@@ -237,6 +296,10 @@ void vtkQuadricClustering::StartAppend(double *bounds)
   this->YBinSize = (this->Bounds[3]-this->Bounds[2])/this->NumberOfDivisions[1];
   this->ZBinSize = (this->Bounds[5]-this->Bounds[4])/this->NumberOfDivisions[2];
 
+  this->XBinStep = (this->XBinSize > 0.0) ? (1.0/this->XBinSize) : 0.0;
+  this->YBinStep = (this->YBinSize > 0.0) ? (1.0/this->YBinSize) : 0.0;
+  this->ZBinStep = (this->ZBinSize > 0.0) ? (1.0/this->ZBinSize) : 0.0;
+
   this->NumberOfBinsUsed = 0;
   if (this->QuadricArray)
     {
@@ -253,11 +316,22 @@ void vtkQuadricClustering::StartAppend(double *bounds)
     return;
     }
 
-  // Allocate CellData here.
-  if (this->CopyCellData && this->GetInput())
+  vtkInformation *inInfo = this->GetExecutive()->GetInputInformation(0, 0);
+  vtkInformation *outInfo = this->GetExecutive()->GetOutputInformation(0);
+  vtkPolyData *input = 0;
+  if (inInfo)
     {
-    this->GetOutput()->GetCellData()->CopyAllocate(
-            this->GetInput()->GetCellData(), this->NumberOfBinsUsed);
+    input = vtkPolyData::SafeDownCast(
+      inInfo->Get(vtkDataObject::DATA_OBJECT()));
+    }
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
+  // Allocate CellData here.
+  if (this->CopyCellData && input)
+    {
+    output->GetCellData()->CopyAllocate(
+      input->GetCellData(), this->NumberOfBinsUsed);
     this->InCellCount = this->OutCellCount = 0;
     }
 }
@@ -271,38 +345,47 @@ void vtkQuadricClustering::Append(vtkPolyData *pd)
   // Check for mis-use of the Append methods.
   if (this->OutputTriangleArray == NULL || this->OutputLines == NULL)
     {
-    //vtkErrorMacro("Missing Array:  Did you call StartAppend?");
+    vtkDebugMacro("Missing Array:  Did you call StartAppend?");
     return;
     }
 
+  vtkInformation *outInfo =
+    this->GetExecutive()->GetOutputInformation(0);
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   inputVerts = pd->GetVerts();
   if (inputVerts)
     {
-    this->AddVertices(inputVerts, inputPoints, 1);
+    this->AddVertices(inputVerts, inputPoints, 1, pd, output);
     }
+  this->UpdateProgress(.40);
 
   inputLines = pd->GetLines();
   if (inputLines)
     {
-    this->AddEdges(inputLines, inputPoints, 1);
+    this->AddEdges(inputLines, inputPoints, 1, pd, output);
     }
+  this->UpdateProgress(.60);
 
   inputPolys = pd->GetPolys();
   if (inputPolys)
     {
-    this->AddPolygons(inputPolys, inputPoints, 1);
+    this->AddPolygons(inputPolys, inputPoints, 1, pd, output);
     }
+  this->UpdateProgress(.80);
 
   inputStrips = pd->GetStrips();
   if (inputStrips)
     {
-    this->AddStrips(inputStrips, inputPoints, 1);
+    this->AddStrips(inputStrips, inputPoints, 1, pd, output);
     }
 }
 
 //----------------------------------------------------------------------------
 void vtkQuadricClustering::AddPolygons(vtkCellArray *polys, vtkPoints *points,
-                                       int geometryFlag)
+                                       int geometryFlag,
+                                       vtkPolyData *input, vtkPolyData *output)
 {
   int j;
   vtkIdType *ptIds = 0;
@@ -310,6 +393,15 @@ void vtkQuadricClustering::AddPolygons(vtkCellArray *polys, vtkPoints *points,
   double pts0[3], pts1[3], pts2[3];
   vtkIdType binIds[3];
 
+  double total = polys->GetNumberOfCells();
+  double curr = 0;
+  double step = total / 10;
+  if (step < 1000.0)
+    {
+    step = 1000.0;
+    }
+  double cstep = step;
+
   for ( polys->InitTraversal(); polys->GetNextCell(numPts, ptIds); )
     {
     points->GetPoint(ptIds[0],pts0);
@@ -320,15 +412,22 @@ void vtkQuadricClustering::AddPolygons(vtkCellArray *polys, vtkPoints *points,
       binIds[1] = this->HashPoint(pts1);
       points->GetPoint(ptIds[j+2],pts2);
       binIds[2] = this->HashPoint(pts2);
-      this->AddTriangle(binIds, pts0, pts1, pts2, geometryFlag);
+      this->AddTriangle(binIds, pts0, pts1, pts2, geometryFlag, input, output);
       }
     ++this->InCellCount;
+    if ( curr > cstep )
+      {
+      this->UpdateProgress(.6 + .2 * curr / total);
+      cstep += step;
+      }
+    curr += 1;
     }//for all polygons
 }
 
 //----------------------------------------------------------------------------
 void vtkQuadricClustering::AddStrips(vtkCellArray *strips, vtkPoints *points,
-                                     int geometryFlag)
+                                     int geometryFlag,
+                                     vtkPolyData *input, vtkPolyData *output)
 {
   int j;
   vtkIdType *ptIds = 0;
@@ -349,7 +448,8 @@ void vtkQuadricClustering::AddStrips(vtkCellArray *strips, vtkPoints *points,
       {
       points->GetPoint(ptIds[j], pts[2]);
       binIds[2] = this->HashPoint(pts[2]);
-      this->AddTriangle(binIds, pts[0], pts[1], pts[2], geometryFlag);
+      this->AddTriangle(binIds, pts[0], pts[1], pts[2], geometryFlag, input,
+                        output);
       pts[odd][0] = pts[2][0];
       pts[odd][1] = pts[2][1];
       pts[odd][2] = pts[2][2];
@@ -382,11 +482,24 @@ inline void vtkQuadricClustering::InitializeQuadric(double quadric[9])
 // If geomertyFlag is 1 then the triangle is added to the output.  Otherwise,
 // only the quadric is affected.
 void vtkQuadricClustering::AddTriangle(vtkIdType *binIds, double *pt0, double *pt1,
-                                       double *pt2, int geometryFlag)
+                                       double *pt2, int geometryFlag,
+                                       vtkPolyData *input, vtkPolyData *output)
 {
   int i;
   vtkIdType triPtIds[3];
   double quadric[9], quadric4x4[4][4];
+  vtkIdType minIdx, midIdx, maxIdx, idx;
+
+  // Special condition for fast execution.
+  // Only add triangles that traverse three bins to quadrics.
+  if (this->UseInternalTriangles == 0)
+    {
+    if (binIds[0] == binIds[1] || binIds[0] == binIds[2] ||
+        binIds[1] == binIds[2])
+      {
+      return;
+      }
+    }
  
   // Compute the quadric.
   vtkTriangle::ComputeQuadric(pt0, pt1, pt2, quadric4x4);
@@ -400,17 +513,6 @@ void vtkQuadricClustering::AddTriangle(vtkIdType *binIds, double *pt0, double *p
   quadric[7] = quadric4x4[2][2];
   quadric[8] = quadric4x4[2][3];
 
-  // Special condition for fast execution.
-  // Only add triangles that traverse three bins to quadrics.
-  if (this->UseInternalTriangles == 0)
-    {
-    if (binIds[0] == binIds[1] || binIds[0] == binIds[2] ||
-        binIds[1] == binIds[2])
-      {
-      return;
-      }
-    }
-
   // Add the quadric to each of the three corner bins.
   for (i = 0; i < 3; ++i)
     {
@@ -444,20 +546,81 @@ void vtkQuadricClustering::AddTriangle(vtkIdType *binIds, double *pt0, double *p
     if (binIds[0] != binIds[1] && binIds[0] != binIds[2] &&
         binIds[1] != binIds[2])
       {
-      this->OutputTriangleArray->InsertNextCell(3, triPtIds);
-      if (this->CopyCellData && this->GetInput())
+      if ( this->PreventDuplicateCells )
         {
-        this->GetOutput()->GetCellData()->
-          CopyData(this->GetInput()->GetCellData(), this->InCellCount,
-                   this->OutCellCount++);
+        minIdx = ( binIds[0]<binIds[1] ? (binIds[0]<binIds[2] ? 0 : 2) :
+                   (binIds[1]<binIds[2] ? 1 : 2) );
+        midIdx = 0;
+        maxIdx = 0;
+        switch ( minIdx )
+          {
+          case 0:
+            if ( binIds[1] > binIds[2] )
+              {
+              maxIdx = 1;
+              midIdx = 2;
+              }
+            else
+              {
+              maxIdx = 2;
+              midIdx = 1;
+              }
+            break;
+          case 1:
+            if ( binIds[0] > binIds[2] )
+              {
+              maxIdx = 0;
+              midIdx = 2;
+              }
+            else
+              {
+              maxIdx = 2;
+              midIdx = 0;
+              }
+            break;
+          case 2:
+            if ( binIds[0] > binIds[1] )
+              {
+              maxIdx = 0;
+              midIdx = 1;
+              }
+            else
+              {
+              maxIdx = 1;
+              midIdx = 0;
+              }
+            break;
+          }
+        idx = binIds[minIdx] + this->NumberOfBins*binIds[midIdx] + 
+              this->NumberOfBins*this->NumberOfBins*binIds[maxIdx];
+        if ( this->CellSet->find(idx) == this->CellSet->end() )
+          {
+          this->CellSet->insert(idx);
+          this->OutputTriangleArray->InsertNextCell(3, triPtIds);
+          if (this->CopyCellData && input)
+            {
+            output->GetCellData()->
+              CopyData(input->GetCellData(), this->InCellCount,this->OutCellCount++);
+            }//if cell data
+          }//if not a duplicate
         }
-      }
-    }
+      else //don't check for duplicates
+        {
+        this->OutputTriangleArray->InsertNextCell(3, triPtIds);
+        if (this->CopyCellData && input)
+          {
+          output->GetCellData()->
+            CopyData(input->GetCellData(), this->InCellCount,this->OutCellCount++);
+          }//if cell data
+        }//don't check for duplicates
+      }//if not duplicate vertices
+    }//if this should be inserted
 }
 
 //----------------------------------------------------------------------------
 void vtkQuadricClustering::AddEdges(vtkCellArray *edges, vtkPoints *points,
-                                    int geometryFlag)
+                                    int geometryFlag,
+                                    vtkPolyData *input, vtkPolyData *output)
 {
   int j;
   vtkIdType numCells, i;
@@ -472,18 +635,21 @@ void vtkQuadricClustering::AddEdges(vtkCellArray *edges, vtkPoints *points,
   for (i = 0; i < numCells; ++i)
     {
     edges->GetNextCell(numPts, ptIds);
-    points->GetPoint(ptIds[0], pt0);
-    binIds[0] = this->HashPoint(pt0);
-    // This internal loop handles line strips.
-    for (j = 1; j < numPts; ++j)
+    if(numPts != 0)
       {
-      points->GetPoint(ptIds[j], pt1);
-      binIds[1] = this->HashPoint(pt1);
-      this->AddEdge(binIds, pt0, pt1, geometryFlag);
-      pt0[0] = pt1[0];
-      pt0[1] = pt1[1];
-      pt0[2] = pt1[2];
-      binIds[0] = binIds[1];
+      points->GetPoint(ptIds[0], pt0);
+      binIds[0] = this->HashPoint(pt0);
+      // This internal loop handles line strips.
+      for (j = 1; j < numPts; ++j)
+        {
+        points->GetPoint(ptIds[j], pt1);
+        binIds[1] = this->HashPoint(pt1);
+        this->AddEdge(binIds, pt0, pt1, geometryFlag, input, output);
+        pt0[0] = pt1[0];
+        pt0[1] = pt1[1];
+        pt0[2] = pt1[2];
+        binIds[0] = binIds[1];
+        }
       }
     ++this->InCellCount;
     }
@@ -494,7 +660,8 @@ void vtkQuadricClustering::AddEdges(vtkCellArray *edges, vtkPoints *points,
 // If geometryFlag is 1 then the edge is added to the output.  Otherwise,
 // only the quadric is affected.
 void vtkQuadricClustering::AddEdge(vtkIdType *binIds, double *pt0, double *pt1,
-                                   int geometryFlag)
+                                   int geometryFlag,
+                                   vtkPolyData *input, vtkPolyData *output)
 {
   int   i;
   vtkIdType edgePtIds[2];
@@ -586,20 +753,20 @@ void vtkQuadricClustering::AddEdge(vtkIdType *binIds, double *pt0, double *pt1,
     if (binIds[0] != binIds[1])
       {
       this->OutputLines->InsertNextCell(2, edgePtIds);
-      if (this->CopyCellData && this->GetInput())
+      if (this->CopyCellData && input)
         {
-        this->GetOutput()->GetCellData()->
-          CopyData(this->GetInput()->GetCellData(),this->InCellCount,
+        output->GetCellData()->
+          CopyData(input->GetCellData(),this->InCellCount,
                    this->OutCellCount++);
         }
       }
     }
 }
 
-
 //----------------------------------------------------------------------------
 void vtkQuadricClustering::AddVertices(vtkCellArray *verts, vtkPoints *points,
-                                       int geometryFlag)
+                                       int geometryFlag, vtkPolyData *input,
+                                       vtkPolyData *output)
 {
   int j;
   vtkIdType numCells, i;
@@ -609,6 +776,14 @@ void vtkQuadricClustering::AddVertices(vtkCellArray *verts, vtkPoints *points,
   vtkIdType binId;
 
   numCells = verts->GetNumberOfCells();
+  double cstep = (double)numCells / 10.0;
+  if (cstep < 1000.0)
+    {
+    cstep = 1000.0;
+    }
+  double next = cstep;
+  double curr = 0;
+
   verts->InitTraversal();
   for (i = 0; i < numCells; ++i)
     {
@@ -618,18 +793,27 @@ void vtkQuadricClustering::AddVertices(vtkCellArray *verts, vtkPoints *points,
       {
       points->GetPoint(ptIds[j], pt);
       binId = this->HashPoint(pt);
-      this->AddVertex(binId, pt, geometryFlag);
+      this->AddVertex(binId, pt, geometryFlag, input, output);
       }
     ++this->InCellCount;
+
+    if ( curr > next )
+      {
+      this->UpdateProgress(.2 + .2 * curr / (double)numCells);
+      next += cstep;
+      }
+    curr += 1;
     }
 }
+
 //----------------------------------------------------------------------------
 // The error function is the length (point to vert) squared.
 // We ignore constants across all terms.
 // If geomertyFlag is 1 then the vert is added to the output.  Otherwise,
 // only the quadric is affected.
 void vtkQuadricClustering::AddVertex(vtkIdType binId, double *pt, 
-                                     int geometryFlag)
+                                     int geometryFlag,
+                                     vtkPolyData *input, vtkPolyData *output)
 {
   double q[9];
 
@@ -678,19 +862,16 @@ void vtkQuadricClustering::AddVertex(vtkIdType binId, double *pt,
       this->QuadricArray[binId].VertexId = this->NumberOfBinsUsed;
       this->NumberOfBinsUsed++;
 
-
-      if (this->CopyCellData && this->GetInput())
+      if (this->CopyCellData && input)
         {
-        this->GetOutput()->GetCellData()->
-          CopyData(this->GetInput()->GetCellData(), this->InCellCount,
+        output->GetCellData()->
+          CopyData(input->GetCellData(), this->InCellCount,
                    this->OutCellCount++);
         }
       }
     }
 }
 
-
-
 //----------------------------------------------------------------------------
 void vtkQuadricClustering::AddQuadric(vtkIdType binId, double quadric[9])
 {
@@ -698,7 +879,7 @@ void vtkQuadricClustering::AddQuadric(vtkIdType binId, double quadric[9])
   
   for (int i=0; i<9; i++)
     {
-    q[i] += (quadric[i] * 100000000.0f);
+    q[i] += (quadric[i] * 100000000.0);
     }
 }
 
@@ -706,9 +887,11 @@ void vtkQuadricClustering::AddQuadric(vtkIdType binId, double quadric[9])
 vtkIdType vtkQuadricClustering::HashPoint(double point[3])
 {
   vtkIdType binId;
-  int xBinCoord, yBinCoord, zBinCoord;
+  int xBinCoord;
+  int yBinCoord;
+  int zBinCoord;
   
-  xBinCoord = static_cast<int>((point[0] - this->Bounds[0]) / this->XBinSize);
+  xBinCoord = static_cast<int>((point[0] - this->Bounds[0])*this->XBinStep);
   if (xBinCoord < 0)
     {
     xBinCoord = 0;
@@ -717,8 +900,8 @@ vtkIdType vtkQuadricClustering::HashPoint(double point[3])
     {
     xBinCoord = this->NumberOfDivisions[0] - 1;
     }
-
-  yBinCoord = static_cast<int>((point[1] - this->Bounds[2]) / this->YBinSize);
+  
+  yBinCoord = static_cast<int>((point[1] - this->Bounds[2])*this->YBinStep);
   if (yBinCoord < 0)
     {
     yBinCoord = 0;
@@ -728,7 +911,7 @@ vtkIdType vtkQuadricClustering::HashPoint(double point[3])
     yBinCoord = this->NumberOfDivisions[1] - 1;
     }
 
-  zBinCoord = static_cast<int>((point[2] - this->Bounds[4]) / this->ZBinSize);
+  zBinCoord = static_cast<int>((point[2] - this->Bounds[4])*this->ZBinStep);
   if (zBinCoord < 0)
     {
     zBinCoord = 0;
@@ -737,9 +920,9 @@ vtkIdType vtkQuadricClustering::HashPoint(double point[3])
     {
     zBinCoord = this->NumberOfDivisions[2] - 1;
     }
-
+  
   // vary x fastest, then y, then z
-  binId = xBinCoord + yBinCoord*this->NumberOfDivisions[1] + 
+  binId = xBinCoord + yBinCoord*this->NumberOfDivisions[0] + 
     zBinCoord*this->SliceSize;
 
   return binId;
@@ -748,32 +931,56 @@ vtkIdType vtkQuadricClustering::HashPoint(double point[3])
 //----------------------------------------------------------------------------
 void vtkQuadricClustering::EndAppend()
 {
-  vtkIdType i, numBuckets, tenth;
+  vtkInformation *inInfo = this->GetExecutive()->GetInputInformation(0, 0);
+  vtkInformation *outInfo = this->GetExecutive()->GetOutputInformation(0);
+  vtkPolyData *input = 0;
+  if (inInfo)
+    {
+    input = vtkPolyData::SafeDownCast(
+      inInfo->Get(vtkDataObject::DATA_OBJECT()));
+    }
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
+  vtkIdType i, numBuckets;
   int abortExecute=0;
   vtkPoints *outputPoints;
   double newPt[3];
-  vtkPolyData *output = this->GetOutput();
+  numBuckets = this->NumberOfDivisions[0] * this->NumberOfDivisions[1] * 
+                this->NumberOfDivisions[2];
+  double step = (double)numBuckets / 10.0;
+  if (step < 1000.0)
+    {
+    step = 1000.0;
+    }
+  double cstep = 0;
   
   // Check for mis use of the Append methods.
   if (this->OutputTriangleArray == NULL || this->OutputLines == NULL)
     {
-    //vtkErrorMacro("Missing Array:  Did you call StartAppend?");
+    vtkDebugMacro("Missing Array:  Did you call StartAppend?");
     return;
     }
 
-  outputPoints = vtkPoints::New();
+  // Clean up
+  if ( this->PreventDuplicateCells )
+    {
+    delete this->CellSet;
+    this->CellSet = NULL;
+    }
 
-  numBuckets = this->NumberOfDivisions[0] * this->NumberOfDivisions[1] * 
-    this->NumberOfDivisions[2];
-  tenth = numBuckets/10 + 1;
+  // Compute the representative points for each bin
+  outputPoints = vtkPoints::New();
   for (i = 0; !abortExecute && i < numBuckets; i++ )
     {
-    if ( ! (i % tenth) ) 
+    if (cstep > step)
       {
+      cstep = 0;
       vtkDebugMacro(<<"Finding point in bin #" << i);
       this->UpdateProgress (0.8+0.2*i/numBuckets);
       abortExecute = this->GetAbortExecute();
       }
+    ++cstep;
 
     if (this->QuadricArray[i].VertexId != -1)
       {
@@ -800,7 +1007,7 @@ void vtkQuadricClustering::EndAppend()
   this->OutputLines->Delete();
   this->OutputLines = NULL;
 
-  this->EndAppendVertexGeometry(this->GetInput());
+  this->EndAppendVertexGeometry(input, output);
 
   // Tell the data is is up to date 
   // (in case the user calls this method directly).
@@ -864,15 +1071,15 @@ void vtkQuadricClustering::ComputeRepresentativePoint(double quadric[9],
   // Compute diagonal matrix W
   //
 #define VTK_SVTHRESHOLD 1.0E-3
-  double maxW = 0.0;
+  double maxW = 0.0, temp;
   vtkMath::SingularValueDecomposition3x3(A, U, w, VT);
 
-  // Find maximum eigenvalue from SVD
+  // Find maximum (magnitude) eigenvalue from SVD
   for (i = 0; i < 3; i++)
     {
-    if (w[i] > maxW)
+    if ((temp = fabs(w[i])) > maxW)
       {
-      maxW = w[i];
+      maxW = temp;
       }
     }
   // Initialize matrix
@@ -882,19 +1089,19 @@ void vtkQuadricClustering::ComputeRepresentativePoint(double quadric[9],
       {
       if (i == j)
         {
-        if ( (w[i] / maxW) > VTK_SVTHRESHOLD)
+        if ( fabs(w[i] / maxW) > VTK_SVTHRESHOLD)
           {
           // If this is true, then w[i] != 0, so this division is ok.
           W[i][j] = 1.0/w[i];
           }
         else
           {
-          W[i][j] = 0.0f;
+          W[i][j] = 0.0;
           }
         }
       else
         {
-        W[i][j] = 0.0f;
+        W[i][j] = 0.0;
         }
       }
     }
@@ -911,42 +1118,22 @@ void vtkQuadricClustering::ComputeRepresentativePoint(double quadric[9],
     }
   vtkMath::Multiply3x3(tempMatrix, tempVector, tempVector);
 
-  // Make absolutely sure that the point lies in the vicinity of the bin. If
-  // not, then clamp the point to the center of the bin. Currently "vicinity"
-  // is defined as BinSize around the center of the bin. It may be desirable
-  // to increase this to a larger multiple of the bin size.
+  // Make absolutely sure that the point lies in the vicinity (enclosing 
+  // sphere) of the bin.  If not, then clamp the point to the enclosing sphere.
+  double deltaMag = vtkMath::Norm(tempVector);
+  double radius = sqrt(this->XBinSize*this->XBinSize + 
+    this->YBinSize*this->YBinSize +
+    this->ZBinSize*this->ZBinSize) / 2.0;
+  if (deltaMag > radius)
+    {
+    tempVector[0] *= radius / deltaMag;
+    tempVector[1] *= radius / deltaMag;
+    tempVector[2] *= radius / deltaMag;
+    }
+
   point[0] = cellCenter[0] + tempVector[0];
   point[1] = cellCenter[1] + tempVector[1];
   point[2] = cellCenter[2] + tempVector[2];
-
-  if (0)
-    {
-  if ( point[0] < (cellCenter[0] - this->XBinSize) )
-    {
-    point[0] = cellCenter[0];
-    }
-  else if ( point[0] > (cellCenter[0] + this->XBinSize) )
-    {
-    point[0] = cellCenter[0];
-    }
-  if ( point[1] < (cellCenter[1] - this->YBinSize) )
-    {
-    point[1] = cellCenter[1];
-    }
-  else if ( point[1] > (cellCenter[1] + this->YBinSize) )
-    {
-    point[1] = cellCenter[1];
-    }
-  if ( point[2] < (cellCenter[2] - this->ZBinSize) )
-    {
-    point[2] = cellCenter[2];
-    }
-  else if ( point[2] > (cellCenter[2] + this->ZBinSize) )
-    {
-    point[2] = cellCenter[2];
-    }
-    }
-  
 }
 
 //----------------------------------------------------------------------------
@@ -964,9 +1151,9 @@ void vtkQuadricClustering::SetNumberOfXDivisions(int num)
     {
     return;
     }
-  if (num < 2)
+  if (num < 1)
     {
-    vtkErrorMacro("You cannot use less than two divisions.");
+    vtkErrorMacro("You cannot use less than one division.");
     return;
     }
   this->Modified();
@@ -981,9 +1168,9 @@ void vtkQuadricClustering::SetNumberOfYDivisions(int num)
     {
     return;
     }
-  if (num < 2)
+  if (num < 1)
     {
-    vtkErrorMacro("You cannot use less than two divisions.");
+    vtkErrorMacro("You cannot use less than one division.");
     return;
     }
   this->Modified();
@@ -998,9 +1185,9 @@ void vtkQuadricClustering::SetNumberOfZDivisions(int num)
     {
     return;
     }
-  if (num < 2)
+  if (num < 1)
     {
-    vtkErrorMacro("You cannot use less than two divisions.");
+    vtkErrorMacro("You cannot use less than one division.");
     return;
     }
   this->Modified();
@@ -1073,22 +1260,18 @@ void vtkQuadricClustering::SetDivisionSpacing(double x, double y, double z)
 }
 
 //----------------------------------------------------------------------------
-void vtkQuadricClustering::EndAppendUsingPoints(vtkPolyData *input)
+void vtkQuadricClustering::EndAppendUsingPoints(vtkPolyData *input,
+                                                vtkPolyData *output)
 {
   vtkIdType   i;
   vtkIdType   outPtId;
   vtkPoints   *inputPoints;
   vtkPoints   *outputPoints;
-  vtkPolyData *output = this->GetOutput();
   vtkIdType   numPoints, numBins;
   vtkIdType   binId;
   double       *minError, e, pt[3];
   double       *q;
 
-  if (input == NULL || output == NULL)
-    {
-    return;
-    }
   inputPoints = input->GetPoints();
   if (inputPoints == NULL)
     {
@@ -1098,10 +1281,17 @@ void vtkQuadricClustering::EndAppendUsingPoints(vtkPolyData *input)
   // Check for misuse of the Append methods.
   if (this->OutputTriangleArray == NULL || this->OutputLines == NULL)
     {
-    //vtkErrorMacro("Missing Array:  Did you call StartAppend?");
+    vtkDebugMacro("Missing Array:  Did you call StartAppend?");
     return;
     }
 
+  // Clean up
+  if ( this->PreventDuplicateCells )
+    {
+    delete this->CellSet;
+    this->CellSet = NULL;
+    }
+
   outputPoints = vtkPoints::New();
 
   // Prepare to copy point data to output
@@ -1163,7 +1353,7 @@ void vtkQuadricClustering::EndAppendUsingPoints(vtkPolyData *input)
   this->OutputLines->Delete();
   this->OutputLines = NULL;
 
-  this->EndAppendVertexGeometry(input);
+  this->EndAppendVertexGeometry(input, output);
 
   if (this->QuadricArray)
     {
@@ -1174,12 +1364,11 @@ void vtkQuadricClustering::EndAppendUsingPoints(vtkPolyData *input)
   delete [] minError;
 }
 
-
-
 //----------------------------------------------------------------------------
 // This is not a perfect implementation, because it does not determine
 // which vertex cell is the best for a bin.  The first detected is used.
-void vtkQuadricClustering::EndAppendVertexGeometry(vtkPolyData *input)
+void vtkQuadricClustering::EndAppendVertexGeometry(vtkPolyData *input,
+                                                   vtkPolyData *output)
 {
   vtkCellArray *inVerts, *outVerts;
   vtkIdType *tmp = NULL;
@@ -1201,8 +1390,7 @@ void vtkQuadricClustering::EndAppendVertexGeometry(vtkPolyData *input)
       {
       if (tmp)
         {
-        delete tmp;
-        tmp = NULL;
+        delete[] tmp;
         }
       tmp = new vtkIdType[numPts];
       tmpLength = numPts;
@@ -1225,7 +1413,7 @@ void vtkQuadricClustering::EndAppendVertexGeometry(vtkPolyData *input)
       {
       // add poly vertex to output.
       outCellId = outVerts->InsertNextCell(tmpIdx, tmp);
-      this->GetOutput()->GetCellData()->
+      output->GetCellData()->
         CopyData(input->GetCellData(), cellId, outCellId);
       }
     }
@@ -1233,22 +1421,21 @@ void vtkQuadricClustering::EndAppendVertexGeometry(vtkPolyData *input)
   if (tmp)
     {
     delete [] tmp;
-    tmp = NULL;
     }
 
   if (outVerts->GetNumberOfCells() > 0)
     {
-    this->GetOutput()->SetVerts(outVerts);
+    output->SetVerts(outVerts);
     }
   outVerts->Delete();
-  outVerts = NULL;
 }
 
 
 //----------------------------------------------------------------------------
 // This method is called after the execution, but before the vertex array
 // is deleted. It changes some points to be based on the boundary edges.
-void vtkQuadricClustering::AppendFeatureQuadrics(vtkPolyData *pd)
+void vtkQuadricClustering::AppendFeatureQuadrics(vtkPolyData *pd,
+                                                 vtkPolyData *output)
 {
   vtkPolyData *input = vtkPolyData::New();
   vtkPoints *edgePts;
@@ -1266,7 +1453,7 @@ void vtkQuadricClustering::AppendFeatureQuadrics(vtkPolyData *pd)
 
   if (edges && edges->GetNumberOfCells() && edgePts)
     {
-    this->AddEdges(edges, edgePts, 0);
+    this->AddEdges(edges, edgePts, 0, pd, output);
     if (this->UseFeaturePoints)
       {
       this->FindFeaturePoints(edges, edgePts, this->FeaturePointsAngle);
@@ -1274,13 +1461,13 @@ void vtkQuadricClustering::AppendFeatureQuadrics(vtkPolyData *pd)
         {
         this->FeaturePoints->GetPoint(i, featurePt);
         binId = this->HashPoint(featurePt);
-        this->AddVertex(binId, featurePt, 0);
+        this->AddVertex(binId, featurePt, 0, input, output);
         }
       }
     }
 
   // Release data.
-  this->FeatureEdges->SetInput(NULL);
+  this->FeatureEdges->SetInputConnection(0, 0);
   this->FeatureEdges->GetOutput()->ReleaseData();
   input->Delete();
 }
@@ -1306,7 +1493,7 @@ void vtkQuadricClustering::FindFeaturePoints(vtkCellArray *edges,
   double featureEdges[2][3];
   double point1[3], point2[3];
   vtkIdType *cellPointIds;
-  double radAngle = vtkMath::DegreesToRadians() * this->FeaturePointsAngle;
+  double radAngle = vtkMath::RadiansFromDegrees( this->FeaturePointsAngle );
   
   this->FeaturePoints->Allocate(numPts);
   
@@ -1371,14 +1558,20 @@ void vtkQuadricClustering::FindFeaturePoints(vtkCellArray *edges,
     }
 
   pointIdList->Delete();
-  pointIdList = NULL;
   for (i = 0; i < numPts; i++)
     {
     delete [] pointTable[i];
-    pointTable[i] = NULL;
     }
   delete [] pointTable;
-  pointTable = NULL;
+}
+
+//----------------------------------------------------------------------------
+int vtkQuadricClustering::FillInputPortInformation(int port,
+                                                   vtkInformation *info)
+{
+  int retval = this->Superclass::FillInputPortInformation(port, info);
+  info->Set(vtkAlgorithm::INPUT_IS_OPTIONAL(), 1);
+  return retval;
 }
 
 //----------------------------------------------------------------------------
@@ -1425,5 +1618,8 @@ void vtkQuadricClustering::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Use Feature Points: " 
      << (this->UseFeaturePoints ? "On\n" : "Off\n");
   os << indent << "Copy Cell Data : " << this->CopyCellData << endl;
+
+  os << indent << "Prevent Duplicate Cells : " 
+     << (this->PreventDuplicateCells ? "On\n" : "Off\n");
 }
 
