@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkShrinkPolyData.cxx,v $
+  Module:    vtkShrinkPolyData.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,11 +16,12 @@
 
 #include "vtkCellArray.h"
 #include "vtkCellData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 
-vtkCxxRevisionMacro(vtkShrinkPolyData, "$Revision: 1.66 $");
 vtkStandardNewMacro(vtkShrinkPolyData);
 
 vtkShrinkPolyData::vtkShrinkPolyData(double sf)
@@ -29,9 +30,10 @@ vtkShrinkPolyData::vtkShrinkPolyData(double sf)
   this->ShrinkFactor = sf;
 }
 
-
 template <class T>
-void vtkShrinkPolyDataExecute(vtkShrinkPolyData *self, T *inPts, double shrinkFactor)
+void vtkShrinkPolyDataExecute(vtkShrinkPolyData *self, T *inPts,
+                              double shrinkFactor,
+                              vtkInformation *inInfo, vtkInformation *outInfo)
 {
   int j, k;
   T center[3];
@@ -42,11 +44,13 @@ void vtkShrinkPolyDataExecute(vtkShrinkPolyData *self, T *inPts, double shrinkFa
   vtkIdType numNewPts, numNewLines, numNewPolys, polyAllocSize;
   vtkIdType npts = 0;
   vtkIdType *pts = 0;
-  vtkIdType newIds[3];
+  vtkIdType newIds[3] = {0, 0, 0};
   vtkPoints *newPoints;
   T *p1, *p2, *p3;
-  vtkPolyData *input = self->GetInput();
-  vtkPolyData *output= self->GetOutput();
+  vtkPolyData *input = vtkPolyData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output= vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
   vtkPointData *pointData = output->GetPointData(); 
 
   pd = input->GetPointData();
@@ -266,28 +270,42 @@ void vtkShrinkPolyDataExecute(vtkShrinkPolyData *self, T *inPts, double shrinkFa
   output->GetCellData()->PassData(input->GetCellData());
 }
 
-void vtkShrinkPolyData::Execute()
+int vtkShrinkPolyData::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input
+  vtkPolyData *input = vtkPolyData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   // Initialize
   vtkDebugMacro(<<"Shrinking polygonal data");
 
-  if (this->GetInput() == NULL || this->GetInput()->GetPoints() == NULL)
+  if (input == NULL || input->GetPoints() == NULL)
     {
-    return;
+    return 1;
     }
   
   // get the input pointer for templating
-  void *inPtr = this->GetInput()->GetPoints()->GetVoidPointer(0);
+  void *inPtr = input->GetPoints()->GetVoidPointer(0);
 
   // call templated function
-  switch (this->GetInput()->GetPoints()->GetDataType())
+  switch (input->GetPoints()->GetDataType())
     {
-    vtkTemplateMacro3(vtkShrinkPolyDataExecute, this, 
-                      (VTK_TT *)(inPtr), this->ShrinkFactor);
+    vtkTemplateMacro(
+      vtkShrinkPolyDataExecute(this, 
+                               (VTK_TT *)(inPtr), this->ShrinkFactor,
+                               inInfo, outInfo));
     default:
       break;
     }
 
+  return 1;
 }
 
 void vtkShrinkPolyData::PrintSelf(ostream& os, vtkIndent indent)
