@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkSpatialRepresentationFilter.cxx,v $
+  Module:    vtkSpatialRepresentationFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,16 +15,19 @@
 #include "vtkSpatialRepresentationFilter.h"
 
 #include "vtkLocator.h"
+#include "vtkInformation.h"
+#include "vtkGarbageCollector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPolyData.h"
 
-vtkCxxRevisionMacro(vtkSpatialRepresentationFilter, "$Revision: 1.33 $");
 vtkStandardNewMacro(vtkSpatialRepresentationFilter);
 vtkCxxSetObjectMacro(vtkSpatialRepresentationFilter,
                      SpatialRepresentation,vtkLocator);
 
 vtkSpatialRepresentationFilter::vtkSpatialRepresentationFilter()
 {
+  this->NumberOfRequiredInputs = 1;
+  this->SetNumberOfInputPorts(1);
   this->SpatialRepresentation = NULL;
   this->Level = 0;
   this->TerminalNodesRequested = 0;
@@ -180,3 +183,25 @@ vtkDataSet *vtkSpatialRepresentationFilter::GetInput()
   
   return (vtkDataSet *)(this->Inputs[0]);
 }
+
+
+//----------------------------------------------------------------------------
+void vtkSpatialRepresentationFilter::ReportReferences(vtkGarbageCollector* collector)
+{
+  this->Superclass::ReportReferences(collector);
+  // This filter shares our input and is therefore involved in a
+  // reference loop.
+  vtkGarbageCollectorReport(collector, this->SpatialRepresentation,
+                            "SpatialRepresentation");
+}
+
+//----------------------------------------------------------------------------
+int vtkSpatialRepresentationFilter::FillInputPortInformation(int port, vtkInformation* info)
+{
+  if(!this->Superclass::FillInputPortInformation(port, info))
+    {
+    return 0;
+    }
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
+}
