@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkSpatialRepresentationFilter.h,v $
+  Module:    vtkSpatialRepresentationFilter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -64,7 +64,7 @@ class VTK_GRAPHICS_EXPORT vtkSpatialRepresentationFilter : public vtkPolyDataSou
 {
 public:
   static vtkSpatialRepresentationFilter *New();
-  vtkTypeRevisionMacro(vtkSpatialRepresentationFilter,vtkPolyDataSource);
+  vtkTypeMacro(vtkSpatialRepresentationFilter,vtkPolyDataSource);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -104,6 +104,9 @@ protected:
   int TerminalNodesRequested;
 
   vtkLocator *SpatialRepresentation;
+
+  virtual void ReportReferences(vtkGarbageCollector*);
+  virtual int FillInputPortInformation(int, vtkInformation*);
 private:
   vtkSpatialRepresentationFilter(const vtkSpatialRepresentationFilter&);  // Not implemented.
   void operator=(const vtkSpatialRepresentationFilter&);  // Not implemented.
