@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkSphereSource.cxx,v $
+  Module:    vtkSphereSource.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,15 +16,17 @@
 
 #include "vtkCellArray.h"
 #include "vtkFloatArray.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPoints.h"
 #include "vtkPolyData.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkSphereSource, "$Revision: 1.64 $");
 vtkStandardNewMacro(vtkSphereSource);
 
 //----------------------------------------------------------------------------
@@ -45,24 +47,37 @@ vtkSphereSource::vtkSphereSource(int res)
   this->StartPhi = 0.0;
   this->EndPhi = 180.0;
   this->LatLongTessellation = 0;
+
+  this->SetNumberOfInputPorts(0);
 }
 
 //----------------------------------------------------------------------------
-void vtkSphereSource::Execute()
+int vtkSphereSource::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
 {
+  // get the info object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the ouptut
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   int i, j;
   int jStart, jEnd, numOffset;
   int numPts, numPolys;
-  vtkPoints *newPoints; 
+  vtkPoints *newPoints;
   vtkFloatArray *newNormals;
   vtkCellArray *newPolys;
   double x[3], n[3], deltaPhi, deltaTheta, phi, theta, radius, norm;
   double startTheta, endTheta, startPhi, endPhi;
   int base, numPoles=0, thetaResolution, phiResolution;
   vtkIdType pts[4];
-  vtkPolyData *output = this->GetOutput();
-  int piece = output->GetUpdatePiece();
-  int numPieces = output->GetUpdateNumberOfPieces();
+  int piece =
+    outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER());
+  int numPieces =
+    outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES());
 
   if (numPieces > this->ThetaResolution)
     {
@@ -72,11 +87,11 @@ void vtkSphereSource::Execute()
     {
     // Although the super class should take care of this,
     // it cannot hurt to check here.
-    return;
+    return 1;
     }
 
-  // I want to modify the ivars resoultion start theta and end theta, 
-  // so I will make local copies of them.  THese might be able to be merged 
+  // I want to modify the ivars resoultion start theta and end theta,
+  // so I will make local copies of them.  THese might be able to be merged
   // with the other copies of them, ...
   int localThetaResolution = this->ThetaResolution;
   double localStartTheta = this->StartTheta;
@@ -98,7 +113,8 @@ void vtkSphereSource::Execute()
 
   // Set things up; allocate memory
   //
-  vtkDebugMacro("SphereSource Executing");
+  vtkDebugMacro("SphereSource Executing piece index " << piece
+                << " of " << numPieces << " pieces.");
 
   numPts = this->PhiResolution * localThetaResolution + 2;
   // creating triangles
@@ -163,14 +179,16 @@ void vtkSphereSource::Execute()
   deltaTheta = (endTheta - startTheta) / thetaResolution;
 
   jStart = (this->StartPhi <= 0.0 ? 1 : 0);
-  jEnd = (this->EndPhi >= 180.0 ? this->PhiResolution - 1 
+  jEnd = (this->EndPhi >= 180.0 ? this->PhiResolution - 1
         : this->PhiResolution);
 
+  this->UpdateProgress(0.1);
+
   // Create intermediate points
   for (i=0; i < localThetaResolution; i++)
     {
     theta = localStartTheta * vtkMath::Pi() / 180.0 + i*deltaTheta;
-    
+
     for (j=jStart; j<jEnd; j++)
       {
       phi = startPhi + j*deltaPhi;
@@ -187,14 +205,15 @@ void vtkSphereSource::Execute()
         {
         norm = 1.0;
         }
-      n[0] /= norm; n[1] /= norm; n[2] /= norm; 
+      n[0] /= norm; n[1] /= norm; n[2] /= norm;
       newNormals->InsertNextTuple(n);
       }
+    this->UpdateProgress (0.10 + 0.50*i/static_cast<float>(localThetaResolution));
     }
 
   // Generate mesh connectivity
   base = phiResolution * localThetaResolution;
-  
+
   if (fabs(localStartTheta - localEndTheta) < 360.0)
     {
     --localThetaResolution;
@@ -223,6 +242,7 @@ void vtkSphereSource::Execute()
       newPolys->InsertNextCell(3, pts);
       }
     }
+  this->UpdateProgress (0.70);
 
   // bands in-between poles
   for (i=0; i < localThetaResolution; i++)
@@ -245,6 +265,7 @@ void vtkSphereSource::Execute()
         newPolys->InsertNextCell(4, pts);
         }
       }
+    this->UpdateProgress (0.70 + 0.30*i/static_cast<double>(localThetaResolution));
     }
 
   // Update ourselves and release memeory
@@ -260,6 +281,8 @@ void vtkSphereSource::Execute()
   newPolys->Squeeze();
   output->SetPolys(newPolys);
   newPolys->Delete();
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -276,12 +299,29 @@ void vtkSphereSource::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Radius: " << this->Radius << "\n";
   os << indent << "Center: (" << this->Center[0] << ", " 
      << this->Center[1] << ", " << this->Center[2] << ")\n";
-  os << indent 
+  os << indent
      << "LatLong Tessellation: " << this->LatLongTessellation << "\n";
 }
 
 //----------------------------------------------------------------------------
-void vtkSphereSource::ExecuteInformation()
+int vtkSphereSource::RequestInformation(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
 {
-  this->GetOutput()->SetMaximumNumberOfPieces(this->ThetaResolution);
+  // get the info object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  outInfo->Set(vtkStreamingDemandDrivenPipeline::MAXIMUM_NUMBER_OF_PIECES(),
+               -1);
+
+  outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_BOUNDING_BOX(),
+               this->Center[0] - this->Radius,
+               this->Center[0] + this->Radius,
+               this->Center[1] - this->Radius,
+               this->Center[1] + this->Radius,
+               this->Center[2] - this->Radius,
+               this->Center[2] + this->Radius);
+
+  return 1;
 }
