@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkSplitField.h,v $
+  Module:    vtkSplitField.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -51,16 +51,16 @@
 #ifndef __vtkSplitField_h
 #define __vtkSplitField_h
 
-#include "vtkDataSetToDataSetFilter.h"
+#include "vtkDataSetAlgorithm.h"
 
 #include "vtkDataSetAttributes.h" // Needed for NUM_ATTRIBUTES
 
 class vtkFieldData;
 
-class VTK_GRAPHICS_EXPORT vtkSplitField : public vtkDataSetToDataSetFilter
+class VTK_GRAPHICS_EXPORT vtkSplitField : public vtkDataSetAlgorithm
 {
 public:
-  vtkTypeRevisionMacro(vtkSplitField,vtkDataSetToDataSetFilter);
+  vtkTypeMacro(vtkSplitField,vtkDataSetAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -129,7 +129,7 @@ protected:
   vtkSplitField();
   virtual ~vtkSplitField();
 
-  void Execute();
+  int RequestData(vtkInformation *, vtkInformationVector **, vtkInformationVector *);
 
   char* FieldName;
   int FieldType;
