@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkSubPixelPositionEdgels.h,v $
+  Module:    vtkSubPixelPositionEdgels.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -29,21 +29,21 @@
 // orthogonal to its gradient vector.
 
 // .SECTION see also
-// vtkImage vtkImageGradient vtkLinkEdgels
+// vtkImageData vtkImageGradient vtkLinkEdgels
 
 #ifndef __vtkSubPixelPositionEdgels_h
 #define __vtkSubPixelPositionEdgels_h
 
-#include "vtkPolyDataToPolyDataFilter.h"
+#include "vtkPolyDataAlgorithm.h"
 
 class vtkStructuredPoints;
 class vtkDataArray;
 
-class VTK_GRAPHICS_EXPORT vtkSubPixelPositionEdgels : public vtkPolyDataToPolyDataFilter
+class VTK_GRAPHICS_EXPORT vtkSubPixelPositionEdgels : public vtkPolyDataAlgorithm
 {
 public:
   static vtkSubPixelPositionEdgels *New();
-  vtkTypeRevisionMacro(vtkSubPixelPositionEdgels,vtkPolyDataToPolyDataFilter);
+  vtkTypeMacro(vtkSubPixelPositionEdgels,vtkPolyDataAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -65,7 +65,9 @@ protected:
   ~vtkSubPixelPositionEdgels();
 
   // Usual data generation method
-  void Execute();
+  virtual int RequestData(vtkInformation *, vtkInformationVector **, vtkInformationVector *);
+  virtual int FillInputPortInformation(int port, vtkInformation *info);
+
   void Move(int xdim, int ydim, int zdim, int x, int y,
             float *img, vtkDataArray *inVecs, 
             double *result, int z, double *aspect, double *resultNormal);
