@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkTensorGlyph.cxx,v $
+  Module:    vtkTensorGlyph.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,17 +14,21 @@
 =========================================================================*/
 #include "vtkTensorGlyph.h"
 
+
+#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkCell.h"
 #include "vtkCellArray.h"
 #include "vtkDataSet.h"
+#include "vtkExecutive.h"
 #include "vtkFloatArray.h"
 #include "vtkMath.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 #include "vtkTransform.h"
 
-vtkCxxRevisionMacro(vtkTensorGlyph, "$Revision: 1.54 $");
 vtkStandardNewMacro(vtkTensorGlyph);
 
 // Construct object with scaling on and scale factor 1.0. Eigenvalues are 
@@ -42,15 +46,74 @@ vtkTensorGlyph::vtkTensorGlyph()
   this->ThreeGlyphs = 0;
   this->Symmetric = 0;
   this->Length = 1.0;
+
+  this->SetNumberOfInputPorts(2);
+
+  // by default, process active point tensors
+  this->SetInputArrayToProcess(0, 0, 0, vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::TENSORS);
+
+  // by default, process active point scalars
+  this->SetInputArrayToProcess(1, 0, 0, vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::SCALARS);
 }
 
+//----------------------------------------------------------------------------
 vtkTensorGlyph::~vtkTensorGlyph()
 {
-  this->SetSource(NULL);
 }
 
-void vtkTensorGlyph::Execute()
+//----------------------------------------------------------------------------
+int vtkTensorGlyph::RequestUpdateExtent(
+  vtkInformation *vtkNotUsed(request),
+    vtkInformationVector **inputVector,
+      vtkInformationVector *outputVector)
 {
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *sourceInfo = inputVector[1]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  if (sourceInfo)
+    {
+    sourceInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER(),
+                    0);
+    sourceInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES(),
+                    1);
+    sourceInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS(),
+                    0);
+    }
+
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER(),
+  outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER()));
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES(),
+  outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES()));
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS(),
+  outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS()));
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::EXACT_EXTENT(), 1);
+
+  return 1;
+}
+
+//----------------------------------------------------------------------------
+int vtkTensorGlyph::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
+{
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *sourceInfo = inputVector[1]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *source = vtkPolyData::SafeDownCast(
+    sourceInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkDataArray *inTensors;
   double tensor[9];
   vtkDataArray *inScalars;
@@ -58,7 +121,7 @@ void vtkTensorGlyph::Execute()
   int j;
   vtkPoints *sourcePts;
   vtkDataArray *sourceNormals;
-  vtkCellArray *sourceCells, *cells;  
+  vtkCellArray *sourceCells, *cells;
   vtkPoints *newPts;
   vtkFloatArray *newScalars=NULL;
   vtkFloatArray *newNormals=NULL;
@@ -78,50 +141,42 @@ void vtkTensorGlyph::Execute()
   double xv[3], yv[3], zv[3];
   double maxScale;
   vtkPointData *pd, *outPD;
-  vtkDataSet *input = this->GetInput();
-  vtkPolyData *output = this->GetOutput();
-
-  if (this->GetSource() == NULL)
-    {
-    vtkErrorMacro("No source.");
-    return;
-    }
 
   numDirs = (this->ThreeGlyphs?3:1)*(this->Symmetric+1);
-  
-  pts = new vtkIdType[this->GetSource()->GetMaxCellSize()];
+
+  pts = new vtkIdType[source->GetMaxCellSize()];
   trans = vtkTransform::New();
   matrix = vtkMatrix4x4::New();
-  
+
   // set up working matrices
-  m[0] = m0; m[1] = m1; m[2] = m2; 
-  v[0] = v0; v[1] = v1; v[2] = v2; 
+  m[0] = m0; m[1] = m1; m[2] = m2;
+  v[0] = v0; v[1] = v1; v[2] = v2;
 
   vtkDebugMacro(<<"Generating tensor glyphs");
 
   pd = input->GetPointData();
   outPD = output->GetPointData();
-  inTensors = pd->GetTensors();
-  inScalars = pd->GetScalars();
+  inTensors = this->GetInputArrayToProcess(0, inputVector);
+  inScalars = this->GetInputArrayToProcess(1, inputVector);
   numPts = input->GetNumberOfPoints();
 
   if ( !inTensors || numPts < 1 )
     {
     vtkErrorMacro(<<"No data to glyph!");
-    return;
+    return 1;
     }
   //
   // Allocate storage for output PolyData
   //
-  sourcePts = this->GetSource()->GetPoints();
+  sourcePts = source->GetPoints();
   numSourcePts = sourcePts->GetNumberOfPoints();
-  numSourceCells = this->GetSource()->GetNumberOfCells();
+  numSourceCells = source->GetNumberOfCells();
 
   newPts = vtkPoints::New();
   newPts->Allocate(numDirs*numPts*numSourcePts);
 
   // Setting up for calls to PolyData::InsertNextCell()
-  if ( (sourceCells=this->GetSource()->GetVerts())->GetNumberOfCells() > 0 )
+  if ( (sourceCells=source->GetVerts())->GetNumberOfCells() > 0 )
     {
     cells = vtkCellArray::New();
     cells->Allocate(numDirs*numPts*sourceCells->GetSize());
@@ -153,12 +208,20 @@ void vtkTensorGlyph::Execute()
   // only copy scalar data through
   pd = this->GetSource()->GetPointData();
   // generate scalars if eigenvalues are chosen or if scalars exist.
-  if (this->ColorGlyphs && 
-      ((this->ColorMode == COLOR_BY_EIGENVALUES) || 
+  if (this->ColorGlyphs &&
+      ((this->ColorMode == COLOR_BY_EIGENVALUES) ||
        (inScalars && (this->ColorMode == COLOR_BY_SCALARS)) ) )
     {
     newScalars = vtkFloatArray::New();
     newScalars->Allocate(numDirs*numPts*numSourcePts);
+    if (this->ColorMode == COLOR_BY_EIGENVALUES)
+      {
+      newScalars->SetName("MaxEigenvalue");
+      }
+    else
+      {
+      newScalars->SetName(inScalars->GetName());
+      }
     }
   else
     {
@@ -170,6 +233,7 @@ void vtkTensorGlyph::Execute()
     {
     newNormals = vtkFloatArray::New();
     newNormals->SetNumberOfComponents(3);
+    newNormals->SetName("Normals");
     newNormals->Allocate(numDirs*3*numPts*numSourcePts);
     }
   //
@@ -185,7 +249,7 @@ void vtkTensorGlyph::Execute()
       npts = cellPts->GetNumberOfIds();
       for (dir=0; dir < numDirs; dir++)
         {
-        // This variable may be removed, but that 
+        // This variable may be removed, but that
         // will not improve readability
         subIncr = ptIncr + dir*numSourcePts;
         for (i=0; i < npts; i++)
@@ -231,7 +295,7 @@ void vtkTensorGlyph::Execute()
       for (i=0; i<3; i++)
         {
         xv[i] = tensor[i];
-        yv[i] = tensor[i+3]; 
+        yv[i] = tensor[i+3];
         zv[i] = tensor[i+6];
         }
       w[0] = vtkMath::Normalize(xv);
@@ -243,7 +307,7 @@ void vtkTensorGlyph::Execute()
     w[0] *= this->ScaleFactor;
     w[1] *= this->ScaleFactor;
     w[2] *= this->ScaleFactor;
-    
+
     if ( this->ClampScaling )
       {
       for (maxScale=0.0, i=0; i<3; i++)
@@ -287,11 +351,11 @@ void vtkTensorGlyph::Execute()
 
     // Now do the real work for each "direction"
 
-    for (dir=0; dir < numDirs; dir++) 
+    for (dir=0; dir < numDirs; dir++)
       {
       eigen_dir = dir%(this->ThreeGlyphs?3:1);
       symmetric_dir = dir/(this->ThreeGlyphs?3:1);
-        
+
       // Remove previous scales ...
       trans->Identity();
 
@@ -310,8 +374,8 @@ void vtkTensorGlyph::Execute()
       matrix->Element[2][1] = yv[2];
       matrix->Element[2][2] = zv[2];
       trans->Concatenate(matrix);
-        
-      if (eigen_dir == 1) 
+
+      if (eigen_dir == 1)
         {
         trans->RotateZ(90.0);
         }
@@ -321,7 +385,7 @@ void vtkTensorGlyph::Execute()
         trans->RotateY(-90.0);
         }
 
-      if (this->ThreeGlyphs) 
+      if (this->ThreeGlyphs)
         {
         trans->Scale(w[eigen_dir], this->ScaleFactor, this->ScaleFactor);
         }
@@ -337,49 +401,57 @@ void vtkTensorGlyph::Execute()
         }
 
       // if the eigenvalue is negative, shift to reverse direction.
-      // The && is there to ensure that we do not change the 
-      // old behaviour of vtkTensorGlyphs (which only used one dir), 
+      // The && is there to ensure that we do not change the
+      // old behaviour of vtkTensorGlyphs (which only used one dir),
       // in case there is an oriented glyph, e.g. an arrow.
-      if (w[eigen_dir] < 0 && numDirs > 1) 
+      if (w[eigen_dir] < 0 && numDirs > 1)
         {
         trans->Translate(-this->Length, 0., 0.);
         }
-        
+
       // multiply points (and normals if available) by resulting
       // matrix
-      trans->TransformPoints(sourcePts,newPts); 
+      trans->TransformPoints(sourcePts,newPts);
 
-      // Apply the transformation to a series of points, 
+      // Apply the transformation to a series of points,
       // and append the results to outPts.
       if ( newNormals )
         {
+        // a negative determinant means the transform turns the
+        // glyph surface inside out, and its surface normals all
+        // point inward. The following scale corrects the surface
+        // normals to point outward.
+        if (trans->GetMatrix()->Determinant() < 0)
+          {
+          trans->Scale(-1.0,-1.0,-1.0);
+          }
         trans->TransformNormals(sourceNormals,newNormals);
         }
-        
+
         // Copy point data from source
-      if ( this->ColorGlyphs && inScalars && 
+      if ( this->ColorGlyphs && inScalars &&
            (this->ColorMode == COLOR_BY_SCALARS) )
         {
         s = inScalars->GetComponent(inPtId, 0);
-        for (i=0; i < numSourcePts; i++) 
+        for (i=0; i < numSourcePts; i++)
           {
           newScalars->InsertTuple(ptIncr+i, &s);
           }
         }
-      else if (this->ColorGlyphs && 
+      else if (this->ColorGlyphs &&
                (this->ColorMode == COLOR_BY_EIGENVALUES) )
         {
-        // If ThreeGlyphs is false we use the first (largest) 
+        // If ThreeGlyphs is false we use the first (largest)
         // eigenvalue as scalar.
         s = w[eigen_dir];
-        for (i=0; i < numSourcePts; i++) 
+        for (i=0; i < numSourcePts; i++)
           {
           newScalars->InsertTuple(ptIncr+i, &s);
           }
         }
       else
         {
-        for (i=0; i < numSourcePts; i++) 
+        for (i=0; i < numSourcePts; i++)
           {
           outPD->CopyData(pd,i,ptIncr+i);
           }
@@ -412,23 +484,65 @@ void vtkTensorGlyph::Execute()
   output->Squeeze();
   trans->Delete();
   matrix->Delete();
+
+  return 1;
 }
 
+//----------------------------------------------------------------------------
+void vtkTensorGlyph::SetSourceConnection(int id, vtkAlgorithmOutput* algOutput)
+{
+  if (id < 0)
+    {
+    vtkErrorMacro("Bad index " << id << " for source.");
+    return;
+    }
+
+  int numConnections = this->GetNumberOfInputConnections(1);
+  if (id < numConnections)
+    {
+    this->SetNthInputConnection(1, id, algOutput);
+    }
+  else if (id == numConnections && algOutput)
+    {
+    this->AddInputConnection(1, algOutput);
+    }
+  else if (algOutput)
+    {
+    vtkWarningMacro("The source id provided is larger than the maximum "
+                    "source id, using " << numConnections << " instead.");
+    this->AddInputConnection(1, algOutput);
+    }
+}
+
+//----------------------------------------------------------------------------
 void vtkTensorGlyph::SetSource(vtkPolyData *source)
 {
-  this->vtkProcessObject::SetNthInput(1, source);
+  this->SetInput(1, source);
 }
 
+//----------------------------------------------------------------------------
 vtkPolyData *vtkTensorGlyph::GetSource()
 {
-  if (this->NumberOfInputs < 2)
+  if (this->GetNumberOfInputConnections(1) < 1)
     {
     return NULL;
     }
-  return (vtkPolyData *)(this->Inputs[1]);
-  
+  return vtkPolyData::SafeDownCast(this->GetExecutive()->GetInputData(1, 0));
 }
 
+//----------------------------------------------------------------------------
+int vtkTensorGlyph::FillInputPortInformation(int port, vtkInformation *info)
+{
+  if (port == 1)
+    {
+    info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkPolyData");
+    return 1;
+    }
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
+}
+
+//----------------------------------------------------------------------------
 void vtkTensorGlyph::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
