@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkTransformFilter.cxx,v $
+  Module:    vtkTransformFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,12 +16,21 @@
 
 #include "vtkCellData.h"
 #include "vtkFloatArray.h"
+#include "vtkImageData.h"
+#include "vtkImageDataToPointSet.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkLinearTransform.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPointSet.h"
+#include "vtkRectilinearGrid.h"
+#include "vtkRectilinearGridToPointSet.h"
+#include "vtkStructuredGrid.h"
+
+#include "vtkNew.h"
+#include "vtkSmartPointer.h"
 
-vtkCxxRevisionMacro(vtkTransformFilter, "$Revision: 1.44 $");
 vtkStandardNewMacro(vtkTransformFilter);
 vtkCxxSetObjectMacro(vtkTransformFilter,Transform,vtkAbstractTransform);
 
@@ -35,8 +44,81 @@ vtkTransformFilter::~vtkTransformFilter()
   this->SetTransform(NULL);
 }
 
-void vtkTransformFilter::Execute()
+int vtkTransformFilter::FillInputPortInformation(int vtkNotUsed(port),
+                                                 vtkInformation *info)
 {
+  info->Remove(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE());
+  info->Append(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkPointSet");
+  info->Append(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkImageData");
+  info->Append(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkRectilinearGrid");
+  return 1;
+}
+
+int vtkTransformFilter::RequestDataObject(vtkInformation *request,
+                                          vtkInformationVector **inputVector,
+                                          vtkInformationVector *outputVector)
+{
+  vtkImageData *inImage = vtkImageData::GetData(inputVector[0]);
+  vtkRectilinearGrid *inRect = vtkRectilinearGrid::GetData(inputVector[0]);
+
+  if (inImage || inRect)
+    {
+    vtkStructuredGrid *output = vtkStructuredGrid::GetData(outputVector);
+    if (!output)
+      {
+      vtkNew<vtkStructuredGrid> newOutput;
+      newOutput->SetPipelineInformation(outputVector->GetInformationObject(0));
+      }
+    return 1;
+    }
+  else
+    {
+    return this->Superclass::RequestDataObject(request,
+                                               inputVector,
+                                               outputVector);
+    }
+}
+
+int vtkTransformFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
+{
+  vtkSmartPointer<vtkPointSet> input = vtkPointSet::GetData(inputVector[0]);
+  vtkPointSet *output = vtkPointSet::GetData(outputVector);
+
+  if (!input)
+    {
+    // Try converting image data.
+    vtkImageData *inImage = vtkImageData::GetData(inputVector[0]);
+    if (inImage)
+      {
+      vtkNew<vtkImageDataToPointSet> image2points;
+      image2points->SetInput(inImage);
+      image2points->Update();
+      input = image2points->GetOutput();
+      }
+    }
+
+  if (!input)
+    {
+    // Try converting rectilinear grid.
+    vtkRectilinearGrid *inRect = vtkRectilinearGrid::GetData(inputVector[0]);
+    if (inRect)
+      {
+      vtkNew<vtkRectilinearGridToPointSet> rect2points;
+      rect2points->SetInput(inRect);
+      rect2points->Update();
+      input = rect2points->GetOutput();
+      }
+    }
+
+  if (!input)
+    {
+    vtkErrorMacro(<< "Invalid or missing input");
+    return 0;
+    }
+
   vtkPoints *inPts;
   vtkPoints *newPts;
   vtkDataArray *inVectors, *inCellVectors;;
@@ -44,8 +126,6 @@ void vtkTransformFilter::Execute()
   vtkDataArray *inNormals, *inCellNormals;
   vtkFloatArray *newNormals=NULL, *newCellNormals=NULL;
   vtkIdType numPts, numCells;
-  vtkPointSet *input = this->GetInput();
-  vtkPointSet *output = this->GetOutput();
   vtkPointData *pd=input->GetPointData(), *outPD=output->GetPointData();
   vtkCellData *cd=input->GetCellData(), *outCD=output->GetCellData();
 
@@ -59,7 +139,7 @@ void vtkTransformFilter::Execute()
   if ( this->Transform == NULL )
     {
     vtkErrorMacro(<<"No transform defined!");
-    return;
+    return 1;
     }
 
   inPts = input->GetPoints();
@@ -71,7 +151,7 @@ void vtkTransformFilter::Execute()
   if ( !inPts )
     {
     vtkErrorMacro(<<"No input data");
-    return;
+    return 1;
     }
 
   numPts = inPts->GetNumberOfPoints();
@@ -79,14 +159,14 @@ void vtkTransformFilter::Execute()
 
   newPts = vtkPoints::New();
   newPts->Allocate(numPts);
-  if ( inVectors ) 
+  if ( inVectors )
     {
     newVectors = vtkFloatArray::New();
     newVectors->SetNumberOfComponents(3);
     newVectors->Allocate(3*numPts);
     newVectors->SetName(inVectors->GetName());
     }
-  if ( inNormals ) 
+  if ( inNormals )
     {
     newNormals = vtkFloatArray::New();
     newNormals->SetNumberOfComponents(3);
@@ -107,7 +187,7 @@ void vtkTransformFilter::Execute()
   else
     {
     this->Transform->TransformPoints(inPts,newPts);
-    }  
+    }
 
   this->UpdateProgress (.6);
 
@@ -116,18 +196,20 @@ void vtkTransformFilter::Execute()
   vtkLinearTransform* lt=vtkLinearTransform::SafeDownCast(this->Transform);
   if (lt)
     {
-    if ( inCellVectors ) 
+    if ( inCellVectors )
       {
       newCellVectors = vtkFloatArray::New();
       newCellVectors->SetNumberOfComponents(3);
       newCellVectors->Allocate(3*numCells);
+      newCellVectors->SetName( inCellVectors->GetName() );
       lt->TransformVectors(inCellVectors,newCellVectors);
       }
-    if ( inCellNormals ) 
+    if ( inCellNormals )
       {
       newCellNormals = vtkFloatArray::New();
       newCellNormals->SetNumberOfComponents(3);
       newCellNormals->Allocate(3*numCells);
+      newCellNormals->SetName( inCellNormals->GetName() );
       lt->TransformNormals(inCellNormals,newCellNormals);
       }
     }
@@ -184,6 +266,8 @@ void vtkTransformFilter::Execute()
       }
     outFD->PassData(inFD);
     }
+
+  return 1;
 }
 
 unsigned long vtkTransformFilter::GetMTime()
