@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkTransformFilter.h,v $
+  Module:    vtkTransformFilter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,7 +14,7 @@
 =========================================================================*/
 // .NAME vtkTransformFilter - transform points and associated normals and vectors
 // .SECTION Description
-// vtkTransformFilter is a filter to transform point coordinates, and 
+// vtkTransformFilter is a filter to transform point coordinates, and
 // associated point normals and vectors. Other point data is passed
 // through the filter.
 //
@@ -22,8 +22,8 @@
 // to scale, rotate, and translate objects. The difference between the
 // two methods is that vtkActor's transformation simply effects where
 // objects are rendered (via the graphics pipeline), whereas
-// vtkTransformFilter actually modifies point coordinates in the 
-// visualization pipeline. This is necessary for some objects 
+// vtkTransformFilter actually modifies point coordinates in the
+// visualization pipeline. This is necessary for some objects
 // (e.g., vtkProbeFilter) that require point coordinates as input.
 
 // .SECTION See Also
@@ -32,15 +32,15 @@
 #ifndef __vtkTransformFilter_h
 #define __vtkTransformFilter_h
 
-#include "vtkPointSetToPointSetFilter.h"
+#include "vtkPointSetAlgorithm.h"
 
 class vtkAbstractTransform;
 
-class VTK_GRAPHICS_EXPORT vtkTransformFilter : public vtkPointSetToPointSetFilter
+class VTK_GRAPHICS_EXPORT vtkTransformFilter : public vtkPointSetAlgorithm
 {
 public:
   static vtkTransformFilter *New();
-  vtkTypeRevisionMacro(vtkTransformFilter,vtkPointSetToPointSetFilter);
+  vtkTypeMacro(vtkTransformFilter,vtkPointSetAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -52,11 +52,18 @@ public:
   virtual void SetTransform(vtkAbstractTransform*);
   vtkGetObjectMacro(Transform,vtkAbstractTransform);
 
+  virtual int FillInputPortInformation(int port, vtkInformation *info);
+
 protected:
   vtkTransformFilter();
   ~vtkTransformFilter();
 
-  void Execute();
+  int RequestDataObject(vtkInformation *request,
+                        vtkInformationVector **inputVector,
+                        vtkInformationVector *outputVector);
+  int RequestData(vtkInformation *,
+                  vtkInformationVector **,
+                  vtkInformationVector *);
   vtkAbstractTransform *Transform;
 private:
   vtkTransformFilter(const vtkTransformFilter&);  // Not implemented.
