@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkTransformPolyDataFilter.cxx,v $
+  Module:    vtkTransformPolyDataFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -18,11 +18,12 @@
 #include "vtkCellData.h"
 #include "vtkFloatArray.h"
 #include "vtkLinearTransform.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 
-vtkCxxRevisionMacro(vtkTransformPolyDataFilter, "$Revision: 1.31 $");
 vtkStandardNewMacro(vtkTransformPolyDataFilter);
 vtkCxxSetObjectMacro(vtkTransformPolyDataFilter,
                      Transform,vtkAbstractTransform);
@@ -37,8 +38,21 @@ vtkTransformPolyDataFilter::~vtkTransformPolyDataFilter()
   this->SetTransform(NULL);
 }
 
-void vtkTransformPolyDataFilter::Execute()
+int vtkTransformPolyDataFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkPolyData *input = vtkPolyData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkPoints *inPts;
   vtkPoints *newPts;
   vtkDataArray *inVectors, *inCellVectors;
@@ -46,8 +60,6 @@ void vtkTransformPolyDataFilter::Execute()
   vtkDataArray *inNormals, *inCellNormals;
   vtkFloatArray *newNormals=NULL, *newCellNormals=NULL;
   vtkIdType numPts, numCells;
-  vtkPolyData *input = this->GetInput();
-  vtkPolyData *output = this->GetOutput();
   vtkPointData *pd=input->GetPointData(), *outPD=output->GetPointData();
   vtkCellData *cd=input->GetCellData(), *outCD=output->GetCellData();
 
@@ -58,7 +70,7 @@ void vtkTransformPolyDataFilter::Execute()
   if ( this->Transform == NULL )
     {
     vtkErrorMacro(<<"No transform defined!");
-    return;
+    return 1;
     }
 
   inPts = input->GetPoints();
@@ -70,7 +82,7 @@ void vtkTransformPolyDataFilter::Execute()
   if ( !inPts )
     {
     vtkErrorMacro(<<"No input data");
-    return;
+    return 1;
     }
 
   numPts = inPts->GetNumberOfPoints();
@@ -83,12 +95,14 @@ void vtkTransformPolyDataFilter::Execute()
     newVectors = vtkFloatArray::New();
     newVectors->SetNumberOfComponents(3);
     newVectors->Allocate(3*numPts);
+    newVectors->SetName(inVectors->GetName());
     }
   if ( inNormals ) 
     {
     newNormals = vtkFloatArray::New();
     newNormals->SetNumberOfComponents(3);
     newNormals->Allocate(3*numPts);
+    newNormals->SetName(inNormals->GetName());
     }
 
   this->UpdateProgress (.2);
@@ -118,6 +132,7 @@ void vtkTransformPolyDataFilter::Execute()
       newCellVectors = vtkFloatArray::New();
       newCellVectors->SetNumberOfComponents(3);
       newCellVectors->Allocate(3*numCells);
+      newCellVectors->SetName( inCellVectors->GetName() );
       lt->TransformVectors(inCellVectors,newCellVectors);
       }
     if ( inCellNormals ) 
@@ -125,6 +140,7 @@ void vtkTransformPolyDataFilter::Execute()
       newCellNormals = vtkFloatArray::New();
       newCellNormals->SetNumberOfComponents(3);
       newCellNormals->Allocate(3*numCells);
+      newCellNormals->SetName( inCellNormals->GetName() );
       lt->TransformNormals(inCellNormals,newCellNormals);
       }
     }
@@ -171,6 +187,8 @@ void vtkTransformPolyDataFilter::Execute()
 
   outPD->PassData(pd);
   outCD->PassData(cd);
+
+  return 1;
 }
 
 unsigned long vtkTransformPolyDataFilter::GetMTime()
