@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkTubeFilter.cxx,v $
+  Module:    vtkTubeFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -18,12 +18,13 @@
 #include "vtkCellData.h"
 #include "vtkFloatArray.h"
 #include "vtkMath.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 #include "vtkPolyLine.h"
 
-vtkCxxRevisionMacro(vtkTubeFilter, "$Revision: 1.80 $");
 vtkStandardNewMacro(vtkTubeFilter);
 
 // Construct object with radius 0.5, radius variation turned off, the number 
@@ -46,23 +47,42 @@ vtkTubeFilter::vtkTubeFilter()
 
   this->GenerateTCoords = VTK_TCOORDS_OFF;
   this->TextureLength = 1.0;
+
+  // by default process active point scalars
+  this->SetInputArrayToProcess(0,0,0,vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::SCALARS);
+
+  // by default process active point vectors
+  this->SetInputArrayToProcess(1,0,0,vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::VECTORS);
 }
 
-void vtkTubeFilter::Execute()
+int vtkTubeFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkPolyData *input = this->GetInput();
-  vtkPolyData *output = this->GetOutput();
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkPolyData *input = vtkPolyData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkPointData *pd=input->GetPointData();
   vtkPointData *outPD=output->GetPointData();
   vtkCellData *cd=input->GetCellData();
   vtkCellData *outCD=output->GetCellData();
-  vtkCellArray *inLines = NULL;
+  vtkCellArray *inLines;
   vtkDataArray *inNormals;
-  vtkDataArray *inScalars=pd->GetScalars();
-  vtkDataArray *inVectors=pd->GetVectors();
+  vtkDataArray *inScalars=this->GetInputArrayToProcess(0,inputVector);
+  vtkDataArray *inVectors=this->GetInputArrayToProcess(1,inputVector);
 
   vtkPoints *inPts;
-  vtkIdType numPts = 0;
+  vtkIdType numPts;
   vtkIdType numLines;
   vtkIdType numNewPts, numNewCells;
   vtkPoints *newPts;
@@ -87,7 +107,7 @@ void vtkTubeFilter::Execute()
       !(inLines = input->GetLines()) || 
        (numLines = inLines->GetNumberOfCells()) < 1 )
     {
-    return;
+    return 1;
     }
 
   // Create the geometry and topology
@@ -172,7 +192,7 @@ void vtkTubeFilter::Execute()
   //
   numNewCells = inLines->GetNumberOfCells() * this->NumberOfSides + 2;
   outCD->CopyNormalsOff();
-  outPD->CopyAllocate(pd,numNewCells);
+  outCD->CopyAllocate(cd,numNewCells);
 
   //  Create points along each polyline that are connected into NumberOfSides
   //  triangle strips. Texture coordinates are optionally generated.
@@ -200,7 +220,8 @@ void vtkTubeFilter::Execute()
       if ( !lineNormalGenerator->GenerateSlidingNormals(inPts,singlePolyline,
                                                         inNormals) )
         {
-        vtkWarningMacro(<< "No normals for line!");
+        vtkWarningMacro("Could not generate normals for line. "
+                        "Skipping to next.");
         continue; //skip tubing this polyline
         }
       }
@@ -264,6 +285,8 @@ void vtkTubeFilter::Execute()
   lineNormalGenerator->Delete();
 
   output->Squeeze();
+
+  return 1;
 }
 
 int vtkTubeFilter::GeneratePoints(vtkIdType offset, 
@@ -279,12 +302,12 @@ int vtkTubeFilter::GeneratePoints(vtkIdType offset,
   int i, k;
   double p[3];
   double pNext[3];
-  double sNext[3];
+  double sNext[3] = {0.0, 0.0, 0.0};
   double sPrev[3];
   double startCapNorm[3], endCapNorm[3];
   double n[3];
   double s[3];
-  double bevelAngle;
+  //double bevelAngle;
   double w[3];
   double nP[3];
   double sFactor=1.0;
@@ -355,7 +378,7 @@ int vtkTubeFilter::GeneratePoints(vtkIdType offset,
         }
       }
 
-    if ( (bevelAngle = vtkMath::Dot(sNext,sPrev)) > 1.0 )
+/*    if ( (bevelAngle = vtkMath::Dot(sNext,sPrev)) > 1.0 )
       {
       bevelAngle = 1.0;
       }
@@ -370,7 +393,7 @@ int vtkTubeFilter::GeneratePoints(vtkIdType offset,
       }
 
     bevelAngle = this->Radius / bevelAngle; //keep tube constant radius
-
+*/
     vtkMath::Cross(s,n,w);
     if ( vtkMath::Normalize(w) == 0.0)
       {
