@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkWarpLens.cxx,v $
+  Module:    vtkWarpLens.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,13 +14,21 @@
 =========================================================================*/
 #include "vtkWarpLens.h"
 
+#include "vtkImageData.h"
+#include "vtkImageDataToPointSet.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPointSet.h"
 #include "vtkPoints.h"
+#include "vtkRectilinearGrid.h"
+#include "vtkRectilinearGridToPointSet.h"
+
+#include "vtkNew.h"
+#include "vtkSmartPointer.h"
 
-vtkCxxRevisionMacro(vtkWarpLens, "$Revision: 1.28 $");
 vtkStandardNewMacro(vtkWarpLens);
 
 //
@@ -62,37 +70,108 @@ vtkWarpLens::vtkWarpLens()
   this->FormatWidth = 1.0;
   this->FormatHeight = 1.0;
   this->ImageWidth = 1;
-  this->ImageHeight = 1;        
+  this->ImageHeight = 1;
 }
 
-void vtkWarpLens::Execute()
+int vtkWarpLens::FillInputPortInformation(int vtkNotUsed(port),
+                                          vtkInformation *info)
 {
+  info->Remove(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE());
+  info->Append(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkPointSet");
+  info->Append(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkImageData");
+  info->Append(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkRectilinearGrid");
+  return 1;
+}
+
+int vtkWarpLens::RequestDataObject(vtkInformation *request,
+                                   vtkInformationVector **inputVector,
+                                   vtkInformationVector *outputVector)
+{
+  vtkImageData *inImage = vtkImageData::GetData(inputVector[0]);
+  vtkRectilinearGrid *inRect = vtkRectilinearGrid::GetData(inputVector[0]);
+
+  if (inImage || inRect)
+    {
+    vtkStructuredGrid *output = vtkStructuredGrid::GetData(outputVector);
+    if (!output)
+      {
+      vtkNew<vtkStructuredGrid> newOutput;
+      newOutput->SetPipelineInformation(outputVector->GetInformationObject(0));
+      }
+    return 1;
+    }
+  else
+    {
+    return this->Superclass::RequestDataObject(request,
+                                               inputVector,
+                                               outputVector);
+    }
+}
+
+int vtkWarpLens::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
+{
+  vtkSmartPointer<vtkPointSet> input = vtkPointSet::GetData(inputVector[0]);
+  vtkPointSet *output = vtkPointSet::GetData(outputVector);
+
+  if (!input)
+    {
+    // Try converting image data.
+    vtkImageData *inImage = vtkImageData::GetData(inputVector[0]);
+    if (inImage)
+      {
+      vtkNew<vtkImageDataToPointSet> image2points;
+      image2points->SetInput(inImage);
+      image2points->Update();
+      input = image2points->GetOutput();
+      }
+    }
+
+  if (!input)
+    {
+    // Try converting rectilinear grid.
+    vtkRectilinearGrid *inRect = vtkRectilinearGrid::GetData(inputVector[0]);
+    if (inRect)
+      {
+      vtkNew<vtkRectilinearGridToPointSet> rect2points;
+      rect2points->SetInput(inRect);
+      rect2points->Update();
+      input = rect2points->GetOutput();
+      }
+    }
+
+  if (!input)
+    {
+    vtkErrorMacro(<< "Invalid or missing input");
+    return 0;
+    }
+
   vtkPoints *inPts;
   vtkPoints *newPts;
   vtkIdType ptId, numPts;
   double pixel[3], newPixel[3];
-  vtkPointSet *input = this->GetInput();
-  vtkPointSet *output = this->GetOutput();
   double x;
   double y;
   double newX;
   double newY;
   double rSquared;
-  
+
   vtkDebugMacro(<<"Warping data to a point");
 
   // First, copy the input to the output as a starting point
   output->CopyStructure( input );
 
-  inPts = input->GetPoints();  
+  inPts = input->GetPoints();
   if (!inPts )
     {
     vtkErrorMacro(<<"No input data");
-    return;
+    return 1;
     }
 
   numPts = inPts->GetNumberOfPoints();
-  newPts = vtkPoints::New(); 
+  newPts = vtkPoints::New();
   newPts->SetNumberOfPoints(numPts);
 
   //
@@ -130,9 +209,9 @@ void vtkWarpLens::Execute()
     // Convert back to pixels
     //
     newPixel[0] = (newX + this->PrincipalPoint[0]) / this->FormatWidth *
-      this->ImageWidth; 
-    newPixel[1] = (newY - this->PrincipalPoint[1]) / 
-      this->FormatHeight * this->ImageHeight * -1; 
+      this->ImageWidth;
+    newPixel[1] = (newY - this->PrincipalPoint[1]) /
+      this->FormatHeight * this->ImageHeight * -1;
 
     newPixel[2] = pixel[2];             // pixel color
     newPts->SetPoint(ptId, newPixel);
@@ -146,13 +225,15 @@ void vtkWarpLens::Execute()
 
   output->SetPoints(newPts);
   newPts->Delete();
+
+  return 1;
 }
 
 void vtkWarpLens::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
-  
-  os << indent << "PrincipalPoint: (" << this->PrincipalPoint[0] << ", " 
+
+  os << indent << "PrincipalPoint: (" << this->PrincipalPoint[0] << ", "
     << this->PrincipalPoint[1] << ") in mm\n";
   os << indent << "K1: " << this->K1 << "\n";
   os << indent << "K2: " << this->K2 << "\n";
