@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkWarpVector.cxx,v $
+  Module:    vtkWarpVector.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,36 +15,87 @@
 #include "vtkWarpVector.h"
 
 #include "vtkCellData.h"
+#include "vtkImageData.h"
+#include "vtkImageDataToPointSet.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPointSet.h"
 #include "vtkPoints.h"
+#include "vtkRectilinearGrid.h"
+#include "vtkRectilinearGridToPointSet.h"
+#include "vtkStructuredGrid.h"
+
+#include "vtkNew.h"
+#include "vtkSmartPointer.h"
 
-vtkCxxRevisionMacro(vtkWarpVector, "$Revision: 1.44 $");
 vtkStandardNewMacro(vtkWarpVector);
 
+//----------------------------------------------------------------------------
 vtkWarpVector::vtkWarpVector()
 {
   this->ScaleFactor = 1.0;
-  this->InputVectorsSelection = NULL;
+
+  // by default process active point vectors
+  this->SetInputArrayToProcess(0,0,0,vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::VECTORS);
 }
 
+//----------------------------------------------------------------------------
 vtkWarpVector::~vtkWarpVector()
 {
-  this->SetInputVectorsSelection(NULL);
 }
 
+//----------------------------------------------------------------------------
+int vtkWarpVector::FillInputPortInformation(int vtkNotUsed(port),
+                                            vtkInformation *info)
+{
+  info->Remove(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE());
+  info->Append(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkPointSet");
+  info->Append(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkImageData");
+  info->Append(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkRectilinearGrid");
+  return 1;
+}
+
+//----------------------------------------------------------------------------
+int vtkWarpVector::RequestDataObject(vtkInformation *request,
+                                     vtkInformationVector **inputVector,
+                                     vtkInformationVector *outputVector)
+{
+  vtkImageData *inImage = vtkImageData::GetData(inputVector[0]);
+  vtkRectilinearGrid *inRect = vtkRectilinearGrid::GetData(inputVector[0]);
+
+  if (inImage || inRect)
+    {
+    vtkStructuredGrid *output = vtkStructuredGrid::GetData(outputVector);
+    if (!output)
+      {
+      vtkNew<vtkStructuredGrid> newOutput;
+      newOutput->SetPipelineInformation(outputVector->GetInformationObject(0));
+      }
+    return 1;
+    }
+  else
+    {
+    return this->Superclass::RequestDataObject(request,
+                                               inputVector,
+                                               outputVector);
+    }
+}
+
+//----------------------------------------------------------------------------
 template <class T1, class T2>
-void vtkWarpVectorExecute2(vtkWarpVector *self, T1 *inPts, 
+void vtkWarpVectorExecute2(vtkWarpVector *self, T1 *inPts,
                            T1 *outPts, T2 *inVec, vtkIdType max)
 {
   vtkIdType ptId;
   T1 scaleFactor = (T1)self->GetScaleFactor();
-  
+
   // Loop over all points, adjusting locations
   for (ptId=0; ptId < max; ptId++)
     {
-    if (!(ptId & 0xfff)) 
+    if (!(ptId & 0xfff))
       {
       self->UpdateProgress ((double)ptId/(max+1));
       if (self->GetAbortExecute())
@@ -52,7 +103,7 @@ void vtkWarpVectorExecute2(vtkWarpVector *self, T1 *inPts,
         break;
         }
       }
-    
+
     *outPts = *inPts + scaleFactor * (T1)(*inVec);
     outPts++; inPts++; inVec++;
     *outPts = *inPts + scaleFactor * (T1)(*inVec);
@@ -61,39 +112,70 @@ void vtkWarpVectorExecute2(vtkWarpVector *self, T1 *inPts,
     outPts++; inPts++; inVec++;
     }
 }
-          
+
+//----------------------------------------------------------------------------
 template <class T>
-void vtkWarpVectorExecute(vtkWarpVector *self, 
-                          T *inPts, 
-                          T *outPts, 
+void vtkWarpVectorExecute(vtkWarpVector *self,
+                          T *inPts,
+                          T *outPts,
                           vtkIdType max,
-                          const char* vectorsSelection)
+                          vtkDataArray *vectors)
 {
   void *inVec;
-  vtkDataArray *vectors = self->GetInput()->GetPointData()->
-                            GetVectors(vectorsSelection);
-
-  if (vectors == NULL)
-    {
-    return;
-    }
   inVec = vectors->GetVoidPointer(0);
 
   // call templated function
   switch (vectors->GetDataType())
     {
-    vtkTemplateMacro5(vtkWarpVectorExecute2,self, inPts, outPts, 
-                      (VTK_TT *)(inVec), max);
+    vtkTemplateMacro(
+      vtkWarpVectorExecute2(self, inPts, outPts,
+                            (VTK_TT *)(inVec), max));
     default:
       break;
-    }  
+    }
 }
 
 //----------------------------------------------------------------------------
-void vtkWarpVector::Execute()
+int vtkWarpVector::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkPointSet *input = this->GetInput();
-  vtkPointSet *output = this->GetOutput();
+  vtkSmartPointer<vtkPointSet> input = vtkPointSet::GetData(inputVector[0]);
+  vtkPointSet *output = vtkPointSet::GetData(outputVector);
+
+  if (!input)
+    {
+    // Try converting image data.
+    vtkImageData *inImage = vtkImageData::GetData(inputVector[0]);
+    if (inImage)
+      {
+      vtkNew<vtkImageDataToPointSet> image2points;
+      image2points->SetInput(inImage);
+      image2points->Update();
+      input = image2points->GetOutput();
+      }
+    }
+
+  if (!input)
+    {
+    // Try converting rectilinear grid.
+    vtkRectilinearGrid *inRect = vtkRectilinearGrid::GetData(inputVector[0]);
+    if (inRect)
+      {
+      vtkNew<vtkRectilinearGridToPointSet> rect2points;
+      rect2points->SetInput(inRect);
+      rect2points->Update();
+      input = rect2points->GetOutput();
+      }
+    }
+
+  if (!input)
+    {
+    vtkErrorMacro(<< "Invalid or missing input");
+    return 0;
+    }
+
   vtkPoints *points;
   vtkIdType numPts;
 
@@ -102,14 +184,16 @@ void vtkWarpVector::Execute()
 
   if (input == NULL || input->GetPoints() == NULL)
     {
-    return;
+    return 1;
     }
   numPts = input->GetPoints()->GetNumberOfPoints();
-  if ( !input->GetPointData()->GetVectors(this->InputVectorsSelection) 
-          || !numPts)
+
+  vtkDataArray *vectors = this->GetInputArrayToProcess(0,inputVector);
+
+  if ( !vectors || !numPts)
     {
     vtkDebugMacro(<<"No input data");
-    return;
+    return 1;
     }
 
   // SETUP AND ALLOCATE THE OUTPUT
@@ -127,25 +211,24 @@ void vtkWarpVector::Execute()
   // call templated function
   switch (input->GetPoints()->GetDataType())
     {
-    vtkTemplateMacro5(vtkWarpVectorExecute, this, 
-                      (VTK_TT *)(inPtr), (VTK_TT *)(outPtr), numPts, this->InputVectorsSelection);
+    vtkTemplateMacro(
+      vtkWarpVectorExecute( this, (VTK_TT *)(inPtr),
+                            (VTK_TT *)(outPtr), numPts, vectors) );
     default:
       break;
     }
-  
+
   // now pass the data.
   output->GetPointData()->CopyNormalsOff(); // distorted geometry
   output->GetPointData()->PassData(input->GetPointData());
   output->GetCellData()->PassData(input->GetCellData());
+
+  return 1;
 }
 
+//----------------------------------------------------------------------------
 void vtkWarpVector::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
-
-  if (this->InputVectorsSelection)
-    {
-    os << indent << "InputVectorsSelection: " << this->InputVectorsSelection;
-    } 
   os << indent << "Scale Factor: " << this->ScaleFactor << "\n";
 }
