@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkWindowedSincPolyDataFilter.cxx,v $
+  Module:    vtkWindowedSincPolyDataFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -18,6 +18,8 @@
 #include "vtkCellData.h"
 #include "vtkFloatArray.h"
 #include "vtkMath.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
@@ -25,11 +27,11 @@
 #include "vtkTriangle.h"
 #include "vtkTriangleFilter.h"
 
-vtkCxxRevisionMacro(vtkWindowedSincPolyDataFilter, "$Revision: 1.35 $");
 vtkStandardNewMacro(vtkWindowedSincPolyDataFilter);
 
 // Construct object with number of iterations 20; passband .1;
 // feature edge smoothing turned off; feature 
+
 // angle 45 degrees; edge angle 15 degrees; and boundary smoothing turned 
 // on. Error scalars and vectors are not generated (by default). The 
 // convergence criterion is 0.0 of the bounding box diagonal.
@@ -46,6 +48,8 @@ vtkWindowedSincPolyDataFilter::vtkWindowedSincPolyDataFilter()
 
   this->GenerateErrorScalars = 0;
   this->GenerateErrorVectors = 0;
+
+  this->NormalizeCoordinates = 0;
 }
 
 #define VTK_SIMPLE_VERTEX 0
@@ -60,8 +64,21 @@ typedef struct _vtkMeshVertex
   vtkIdList *edges; // connected edges (list of connected point ids)
 } vtkMeshVertex, *vtkMeshVertexPtr;
     
-void vtkWindowedSincPolyDataFilter::Execute()
+int vtkWindowedSincPolyDataFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkPolyData *input = vtkPolyData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkIdType numPts, numCells, numPolys, numStrips, i;
   int j, k;
   vtkIdType npts = 0;
@@ -79,8 +96,6 @@ void vtkWindowedSincPolyDataFilter::Execute()
   vtkCellArray *inVerts, *inLines, *inPolys, *inStrips;
   vtkPoints *newPts[4];
   vtkMeshVertexPtr Verts;
-  vtkPolyData *input = this->GetInput();
-  vtkPolyData *output = this->GetOutput();
 
   // variables specific to windowed sinc interpolation
   double theta_pb, k_pb, sigma, p_x0[3], p_x1[3], p_x3[3];
@@ -95,12 +110,11 @@ void vtkWindowedSincPolyDataFilter::Execute()
   if (numPts < 1 || numCells < 1)
     {
     vtkErrorMacro(<<"No data to smooth!");
-    return;
+    return 1;
     }
 
-  CosFeatureAngle = 
-    cos((double) vtkMath::DegreesToRadians() * this->FeatureAngle);
-  CosEdgeAngle = cos((double) vtkMath::DegreesToRadians() * this->EdgeAngle);
+  CosFeatureAngle = cos( vtkMath::RadiansFromDegrees( this->FeatureAngle) );
+  CosEdgeAngle    = cos( vtkMath::RadiansFromDegrees( this->EdgeAngle) );
 
   vtkDebugMacro(<<"Smoothing " << numPts << " vertices, " << numCells 
                << " cells with:\n"
@@ -124,7 +138,7 @@ void vtkWindowedSincPolyDataFilter::Execute()
     output->GetPointData()->PassData(input->GetPointData());
     output->GetCellData()->PassData(input->GetCellData());
     vtkWarningMacro(<<"Number of iterations == 0: passing data through unchanged");
-    return;
+    return 1;
     }
 //
 // Peform topological analysis. What we're gonna do is build a connectivity
@@ -431,10 +445,30 @@ void vtkWindowedSincPolyDataFilter::Execute()
   newPts[3] = vtkPoints::New();
   newPts[3]->SetNumberOfPoints(numPts);
 
-  
-  for (i=0; i<numPts; i++) //initialize to old coordinates
+  // Get the center and length of the input dataset
+  double *inCenter = input->GetCenter();
+  double inLength = input->GetLength();
+
+  if (!this->NormalizeCoordinates)
     {
-    newPts[zero]->SetPoint(i,inPts->GetPoint(i));
+    for (i=0; i<numPts; i++) //initialize to old coordinates
+      {
+      newPts[zero]->SetPoint(i,inPts->GetPoint(i));
+      }
+    }
+  else
+    {
+    // center the data and scale to be within unit cube [-1, 1]
+    double normalizedPoint[3];
+    for (i=0; i<numPts; i++) //initialize to old coordinates
+      {
+      inPts->GetPoint(i, normalizedPoint);
+      for (j=0; j<3; ++j)
+        {
+        normalizedPoint[j] = (normalizedPoint[j] - inCenter[j]) / inLength;
+        }
+      newPts[zero]->SetPoint(i,normalizedPoint);
+      }
     }
 
   // Smooth with a low pass filter defined as a windowed sinc function.
@@ -658,9 +692,9 @@ void vtkWindowedSincPolyDataFilter::Execute()
 
     // update the pointers. three is always three. all other pointers
     // shift by one and wrap.
-    zero = (++zero)%3;
-    one = (++one)%3;
-    two = (++two)%3;
+    zero = (1+zero)%3;
+    one = (1+one)%3;
+    two = (1+two)%3;
     
     }//for all iterations or until converge
   
@@ -677,6 +711,23 @@ void vtkWindowedSincPolyDataFilter::Execute()
   
   vtkDebugMacro(<<"Performed " << iterationNumber << " smoothing passes");
   
+  // if we scaled the data down to the unit cube, then scale data back
+  // up to the original space
+  if (this->NormalizeCoordinates)
+    {
+    // Re-position the coordinated
+    double repositionedPoint[3];
+    for (i=0; i<numPts; i++) 
+      {
+      newPts[zero]->GetPoint(i, repositionedPoint);
+      for (j=0; j<3; ++j)
+        {
+        repositionedPoint[j] = repositionedPoint[j] * inLength + inCenter[j];
+        }
+      newPts[zero]->SetPoint(i,repositionedPoint);
+      }
+    }
+
 //
 // Update output. Only point coordinates have changed.
 //
@@ -717,7 +768,7 @@ void vtkWindowedSincPolyDataFilter::Execute()
     output->GetPointData()->SetVectors(newVectors);
     newVectors->Delete();
     }
-  
+
   output->SetPoints(newPts[zero]);
   newPts[0]->Delete();
   newPts[1]->Delete();
@@ -738,6 +789,8 @@ void vtkWindowedSincPolyDataFilter::Execute()
     if ( Verts[i].edges != NULL ) {Verts[i].edges->Delete();}
     }
   delete [] Verts;
+
+  return 1;
 }
 
 void vtkWindowedSincPolyDataFilter::PrintSelf(ostream& os, vtkIndent indent)
@@ -746,6 +799,7 @@ void vtkWindowedSincPolyDataFilter::PrintSelf(ostream& os, vtkIndent indent)
 
   os << indent << "Number of Iterations: " << this->NumberOfIterations << "\n";
   os << indent << "Passband: " << this->PassBand << "\n";
+  os << indent << "Normalize Coordinates: " << (this->NormalizeCoordinates ? "On\n" : "Off\n");
   os << indent << "Feature Edge Smoothing: " << (this->FeatureEdgeSmoothing ? "On\n" : "Off\n");
   os << indent << "Feature Angle: " << this->FeatureAngle << "\n";
   os << indent << "Edge Angle: " << this->EdgeAngle << "\n";
