@@ -44,13 +44,17 @@ vtkRenderLargeImage renderLarge
   renderLarge Update
 
 vtkImageViewer viewer
-  viewer SetInput [renderLarge GetOutput]
+  viewer SetInputConnection [renderLarge GetOutputPort]
   viewer SetColorWindow 255
   viewer SetColorLevel 127.5
   viewer Render
 
 # on several opengl X window unix implementations
 # multiple context deletes cause errors
-# so we leak teh renWin1 in this test for unix
-if { $tcl_platform(platform) == "unix" } {renWin1 Register ren1}
-
+# so we leak the renWin1 in this test for unix
+if { $tcl_platform(platform) == "unix" } {
+  renWin1 Register ren1
+  vtkDebugLeaks dl
+  dl SetExitError 0
+  dl Delete
+}
