@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkCubeAxesActor2D.cxx,v $
+  Module:    vtkCubeAxesActor2D.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -22,12 +22,11 @@
 #include "vtkTextProperty.h"
 #include "vtkViewport.h"
 
-vtkCxxRevisionMacro(vtkCubeAxesActor2D, "$Revision: 1.44 $");
 vtkStandardNewMacro(vtkCubeAxesActor2D);
 
 vtkCxxSetObjectMacro(vtkCubeAxesActor2D,Input, vtkDataSet);
-vtkCxxSetObjectMacro(vtkCubeAxesActor2D,Prop, vtkProp);
 vtkCxxSetObjectMacro(vtkCubeAxesActor2D,Camera,vtkCamera);
+vtkCxxSetObjectMacro(vtkCubeAxesActor2D,ViewProp,vtkProp);
 vtkCxxSetObjectMacro(vtkCubeAxesActor2D,AxisLabelTextProperty,vtkTextProperty);
 vtkCxxSetObjectMacro(vtkCubeAxesActor2D,AxisTitleTextProperty,vtkTextProperty);
 
@@ -36,7 +35,7 @@ vtkCxxSetObjectMacro(vtkCubeAxesActor2D,AxisTitleTextProperty,vtkTextProperty);
 vtkCubeAxesActor2D::vtkCubeAxesActor2D()
 {
   this->Input = NULL;
-  this->Prop = NULL;
+  this->ViewProp = NULL;
   this->Bounds[0] = -1.0; this->Bounds[1] = 1.0;
   this->Bounds[2] = -1.0; this->Bounds[3] = 1.0;
   this->Bounds[4] = -1.0; this->Bounds[5] = 1.0;
@@ -81,6 +80,7 @@ vtkCubeAxesActor2D::vtkCubeAxesActor2D()
   this->FontFactor = 1.0;
   this->CornerOffset = 0.05;
   this->Inertia = 1;
+  this->ShowActualBounds = 1;
   this->RenderCount = 0;
 
   this->XAxisVisibility = 1;
@@ -93,6 +93,12 @@ vtkCubeAxesActor2D::vtkCubeAxesActor2D()
   sprintf(this->YLabel,"%s","Y");
   this->ZLabel = new char[2];
   sprintf(this->ZLabel,"%s","Z");
+
+  // Allow the user to specify an origin for the axes. The axes will then run 
+  // from this origin to the bounds and will cross over at this origin.  
+  this->XOrigin = VTK_DOUBLE_MAX;  
+  this->YOrigin = VTK_DOUBLE_MAX;  
+  this->ZOrigin = VTK_DOUBLE_MAX;  
 }
 
 //----------------------------------------------------------------------------
@@ -111,7 +117,7 @@ void vtkCubeAxesActor2D::ShallowCopy(vtkCubeAxesActor2D *actor)
   this->SetZLabel(actor->GetZLabel());
   this->SetFlyMode(actor->GetFlyMode());
   this->SetInput(actor->GetInput());
-  this->SetProp(actor->GetProp());
+  this->SetViewProp(actor->GetViewProp());
   this->SetCamera(actor->GetCamera());
 }
 
@@ -123,9 +129,9 @@ vtkCubeAxesActor2D::~vtkCubeAxesActor2D()
     this->Input->Delete();
     }
 
-  if ( this->Prop )
+  if ( this->ViewProp )
     {
-    this->Prop->Delete();
+    this->ViewProp->Delete();
     }
 
   if ( this->Camera )
@@ -206,8 +212,8 @@ int vtkCubeAxesActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
   double slope = 0.0, minSlope, num, den;
   double pts[8][3], d2, d2Min, min;
   int i, idx = 0;
-  int xIdx, yIdx = 0, zIdx = 0, zIdx2, renderedSomething=0;
-  int xAxes = 0, yAxes, zAxes;
+  int xIdx = 0, yIdx = 0, zIdx = 0, zIdx2 = 0, renderedSomething=0;
+  int xAxes = 0, yAxes = 0, zAxes = 0;
 
   // Initialization
   if ( !this->Camera )
@@ -222,12 +228,27 @@ int vtkCubeAxesActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
   // determine the bounds to use
   this->GetBounds(bounds);
 
+  // Check for user specified origins. By default, these are placed at a corner of the
+  // bounding box of the dataset (corner is based on the fly mode)
+  if (this->XOrigin != VTK_DOUBLE_MAX)
+    {
+    bounds[0] = this->XOrigin;
+    }
+  if (this->YOrigin != VTK_DOUBLE_MAX)
+    {
+    bounds[2] = this->YOrigin;
+    }
+  if (this->ZOrigin != VTK_DOUBLE_MAX)
+    {
+    bounds[4] = this->ZOrigin;
+    }
+
   // Build the axes (almost always needed so we don't check mtime)
   // Transform all points into display coordinates
   this->TransformBounds(viewport, bounds, pts);
 
   // Find the portion of the bounding box that fits within the viewport,
-  if ( this->ClipBounds(viewport, pts, bounds) == 0 )
+  if ( !this->ShowActualBounds && (this->ClipBounds(viewport, pts, bounds) == 0) )
     {
     this->RenderSomething = 0;
     return 0;
@@ -238,7 +259,18 @@ int vtkCubeAxesActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
     {
     // Okay, we have a bounding box, maybe clipped and scaled, that is visible.
     // We setup the axes depending on the fly mode.
-    if ( this->FlyMode == VTK_FLY_CLOSEST_TRIAD )
+    if (this->FlyMode == VTK_FLY_NONE)
+      {
+      idx = 2; // Just use the default axis orientation.
+      xAxes = 0;
+      xIdx = Conn[idx][0];
+      yAxes = 1;
+      yIdx = Conn[idx][1];
+      zAxes = 2;
+      zIdx = idx;
+      zIdx2 = Conn[idx][2];
+      } 
+    else if ( this->FlyMode == VTK_FLY_CLOSEST_TRIAD )
       {
       // Loop over points and find the closest point to the camera
       min = VTK_DOUBLE_MAX;
@@ -260,7 +292,7 @@ int vtkCubeAxesActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
       zIdx = idx;
       zIdx2 = Conn[idx][2];
       }
-    else
+    else if (this->FlyMode == VTK_FLY_OUTER_EDGES)
       {
       double e1[2], e2[2], e3[2];
 
@@ -359,6 +391,10 @@ int vtkCubeAxesActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
   this->AdjustAxes(pts, bounds, idx, xIdx, yIdx, zIdx, zIdx2, 
                    xAxes, yAxes, zAxes, 
                    xCoords, yCoords, zCoords, xRange, yRange, zRange);
+                   
+  // Sorry for ugly hack. I find the fonts slightly too large on the axis. BNW
+  double AxisFontFactor = this->FontFactor*.75;
+  
 
   // Upate axes
   this->Labels[0] = this->XLabel;
@@ -371,7 +407,7 @@ int vtkCubeAxesActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
   this->XAxis->SetTitle(this->Labels[xAxes]);
   this->XAxis->SetNumberOfLabels(this->NumberOfLabels);
   this->XAxis->SetLabelFormat(this->LabelFormat);
-  this->XAxis->SetFontFactor(this->FontFactor);
+  this->XAxis->SetFontFactor(AxisFontFactor); 
   this->XAxis->SetProperty(this->GetProperty());
 
   this->YAxis->GetPositionCoordinate()->SetValue(yCoords[2], yCoords[3]);
@@ -380,7 +416,7 @@ int vtkCubeAxesActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
   this->YAxis->SetTitle(this->Labels[yAxes]);
   this->YAxis->SetNumberOfLabels(this->NumberOfLabels);
   this->YAxis->SetLabelFormat(this->LabelFormat);
-  this->YAxis->SetFontFactor(this->FontFactor);
+  this->YAxis->SetFontFactor(AxisFontFactor);
   this->YAxis->SetProperty(this->GetProperty());
 
   this->ZAxis->GetPositionCoordinate()->SetValue(zCoords[0], zCoords[1]);
@@ -389,7 +425,7 @@ int vtkCubeAxesActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
   this->ZAxis->SetTitle(this->Labels[zAxes]);
   this->ZAxis->SetNumberOfLabels(this->NumberOfLabels);
   this->ZAxis->SetLabelFormat(this->LabelFormat);
-  this->ZAxis->SetFontFactor(this->FontFactor);
+  this->ZAxis->SetFontFactor(AxisFontFactor);
   this->ZAxis->SetProperty(this->GetProperty());
 
   // Rebuid text props
@@ -457,6 +493,14 @@ int vtkCubeAxesActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
   return renderedSomething;
 }
 
+//-----------------------------------------------------------------------------
+// Description:
+// Does this prop have some translucent polygonal geometry?
+int vtkCubeAxesActor2D::HasTranslucentPolygonalGeometry()
+{
+  return 0;
+}
+
 //----------------------------------------------------------------------------
 // Do final adjustment of axes to control offset, etc.
 void vtkCubeAxesActor2D::AdjustAxes(double pts[8][3], double bounds[6],
@@ -544,8 +588,11 @@ void vtkCubeAxesActor2D::AdjustAxes(double pts[8][3], double bounds[6],
     xCoords[3] = xCoords[3] - this->CornerOffset * (xCoords[3] - ave);
 
     ave = (xRange[1] + xRange[0]) / 2.0;
-    xRange[0] = xRange[0] - this->CornerOffset * (xRange[0] - ave);
-    xRange[1] = xRange[1] - this->CornerOffset * (xRange[1] - ave);
+    if (!this->ShowActualBounds)
+      {
+      xRange[0] = xRange[0] - this->CornerOffset * (xRange[0] - ave);
+      xRange[1] = xRange[1] - this->CornerOffset * (xRange[1] - ave);
+      }
     
     // y-axis
     ave = (yCoords[0] + yCoords[2]) / 2.0;
@@ -557,8 +604,11 @@ void vtkCubeAxesActor2D::AdjustAxes(double pts[8][3], double bounds[6],
     yCoords[3] = yCoords[3] - this->CornerOffset * (yCoords[3] - ave);
 
     ave = (yRange[1] + yRange[0]) / 2.0;
-    yRange[0] = yRange[0] - this->CornerOffset * (yRange[0] - ave);
-    yRange[1] = yRange[1] - this->CornerOffset * (yRange[1] - ave);
+    if (!this->ShowActualBounds)
+      {
+      yRange[0] = yRange[0] - this->CornerOffset * (yRange[0] - ave);
+      yRange[1] = yRange[1] - this->CornerOffset * (yRange[1] - ave);
+      }
     
     // z-axis
     ave = (zCoords[0] + zCoords[2]) / 2.0;
@@ -570,8 +620,11 @@ void vtkCubeAxesActor2D::AdjustAxes(double pts[8][3], double bounds[6],
     zCoords[3] = zCoords[3] - this->CornerOffset * (zCoords[3] - ave);
 
     ave = (zRange[1] + zRange[0]) / 2.0;
-    zRange[0] = zRange[0] - this->CornerOffset * (zRange[0] - ave);
-    zRange[1] = zRange[1] - this->CornerOffset * (zRange[1] - ave);
+    if (!this->ShowActualBounds)
+      {
+      zRange[0] = zRange[0] - this->CornerOffset * (zRange[0] - ave);
+      zRange[1] = zRange[1] - this->CornerOffset * (zRange[1] - ave);
+      }
     }
 }
 
@@ -638,8 +691,8 @@ void vtkCubeAxesActor2D::GetBounds(double bounds[6])
       this->Bounds[i] = bounds[i];
       }
     }
-  else if ( this->Prop && 
-            ((propBounds = this->Prop->GetBounds()) && propBounds != NULL) )
+  else if ( this->ViewProp && 
+            ((propBounds = this->ViewProp->GetBounds()) && propBounds != NULL) )
     {
     for (i=0; i< 6; i++)
       {
@@ -693,13 +746,13 @@ void vtkCubeAxesActor2D::PrintSelf(ostream& os, vtkIndent indent)
     os << indent << "Input: (none)\n";
     }
 
-  if ( this->Prop )
+  if ( this->ViewProp )
     {
-    os << indent << "Prop: (" << (void *)this->Prop << ")\n";
+    os << indent << "ViewProp: (" << (void *)this->ViewProp << ")\n";
     }
   else
     {
-    os << indent << "Prop: (none)\n";
+    os << indent << "ViewProp: (none)\n";
     }
 
   os << indent << "Bounds: \n";
@@ -744,11 +797,15 @@ void vtkCubeAxesActor2D::PrintSelf(ostream& os, vtkIndent indent)
     {
     os << indent << "Fly Mode: CLOSEST_TRIAD\n";
     }
-  else
+  else if (this->FlyMode == VTK_FLY_OUTER_EDGES)
     {
     os << indent << "Fly Mode: OUTER_EDGES\n";
     }
-
+  else if (this->FlyMode == VTK_FLY_NONE)
+    {
+    os << indent << "Fly Mode: Disabled\n";
+    }
+  
   os << indent << "Scaling: " << (this->Scaling ? "On\n" : "Off\n");
   os << indent << "UseRanges: " << (this->UseRanges ? "On\n" : "Off\n");
   os << indent << "Ranges: \n";
@@ -777,6 +834,19 @@ void vtkCubeAxesActor2D::PrintSelf(ostream& os, vtkIndent indent)
      << this->Ranges[0] << ", " << this->Ranges[1] << ", "
      << this->Ranges[2] << ", " << this->Ranges[3] << ", "
      << this->Ranges[4] << ", " << this->Ranges[5] << "\n";
+  os << indent << "Show Actual Bounds: " << (this->ShowActualBounds ? "On\n" : "Off\n");
+  if (this->XOrigin != VTK_DOUBLE_MAX)
+    {
+    os << indent << "User specified X Origin: " << this->XOrigin << endl;
+    }
+  if (this->YOrigin != VTK_DOUBLE_MAX)
+    {
+    os << indent << "User specified Y Origin: " << this->YOrigin << endl;
+    }
+  if (this->ZOrigin != VTK_DOUBLE_MAX)
+    {
+    os << indent << "User specified Z Origin: " << this->ZOrigin << endl;
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -1001,3 +1071,56 @@ static int IsInBounds(double x[3], double bounds[6])
     return 1;
     }
 }
+
+//----------------------------------------------------------------------------
+
+// Disable warnings about qualifiers on return types.
+#if defined(_COMPILER_VERSION)
+# pragma set woff 3303
+#endif
+#if defined(__INTEL_COMPILER)
+# pragma warning (disable:858)
+#endif
+
+#ifndef VTK_LEGACY_REMOVE
+# ifdef VTK_WORKAROUND_WINDOWS_MANGLE
+#  undef SetProp
+#  undef GetProp
+void vtkCubeAxesActor2D::SetPropA(vtkProp* prop)
+{
+  VTK_LEGACY_REPLACED_BODY(vtkCubeAxesActor2D::SetProp, "VTK 5.0",
+                           vtkCubeAxesActor2D::SetViewProp);
+  this->SetViewProp(prop);
+}
+void vtkCubeAxesActor2D::SetPropW(vtkProp* prop)
+{
+  VTK_LEGACY_REPLACED_BODY(vtkCubeAxesActor2D::SetProp, "VTK 5.0",
+                           vtkCubeAxesActor2D::SetViewProp);
+  this->SetViewProp(prop);
+}
+vtkProp* vtkCubeAxesActor2D::GetPropA()
+{
+  VTK_LEGACY_REPLACED_BODY(vtkCubeAxesActor2D::GetProp, "VTK 5.0",
+                           vtkCubeAxesActor2D::GetViewProp);
+  return this->GetViewProp();
+}
+vtkProp* vtkCubeAxesActor2D::GetPropW()
+{
+  VTK_LEGACY_REPLACED_BODY(vtkCubeAxesActor2D::GetProp, "VTK 5.0",
+                           vtkCubeAxesActor2D::GetViewProp);
+  return this->GetViewProp();
+}
+# endif
+void vtkCubeAxesActor2D::SetProp(vtkProp* prop)
+{
+  VTK_LEGACY_REPLACED_BODY(vtkCubeAxesActor2D::SetProp, "VTK 5.0",
+                           vtkCubeAxesActor2D::SetViewProp);
+  this->SetViewProp(prop);
+}
+vtkProp* vtkCubeAxesActor2D::GetProp()
+{
+  VTK_LEGACY_REPLACED_BODY(vtkCubeAxesActor2D::GetProp, "VTK 5.0",
+                           vtkCubeAxesActor2D::GetViewProp);
+  return this->GetViewProp();
+}
+#endif
