@@ -1,21 +1,20 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkGreedyTerrainDecimation.cxx,v $
-  Language:  C++
-  Date:      $Date: 2003/11/11 17:51:01 $
-  Version:   $Revision: 1.14.2.1 $
+  Module:    vtkGreedyTerrainDecimation.cxx
 
-  Copyright (c) 1993-2002 Ken Martin, Will Schroeder, Bill Lorensen 
+  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
   See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
 
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
+     This software is distributed WITHOUT ANY WARRANTY; without even
+     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
 #include "vtkGreedyTerrainDecimation.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPriorityQueue.h"
 #include "vtkImageData.h"
@@ -25,10 +24,11 @@
 #include "vtkCellArray.h"
 #include "vtkTriangle.h"
 #include "vtkDoubleArray.h"
+#include "vtkFloatArray.h"
+#include "vtkMath.h"
 
 #include <vector>
 
-vtkCxxRevisionMacro(vtkGreedyTerrainDecimation, "$Revision: 1.14.2.1 $");
 vtkStandardNewMacro(vtkGreedyTerrainDecimation);
 
 // Define some constants describing vertices
@@ -40,6 +40,7 @@ vtkStandardNewMacro(vtkGreedyTerrainDecimation);
 #define VTK_INTERIOR_EDGE 1
 #define VTK_BOUNDARY_EDGE 2
 
+//----------------------------------------------------------------------
 //Supporting classes for points
 class vtkTerrainInfo
 {
@@ -48,24 +49,27 @@ public:
   vtkIdType TriangleId;
 };
 
+//----------------------------------------------------------------------
 //PIMPL STL encapsulation
 //
 // Maps input point ids to owning mesh triangle
-class vtkGreedyTerrainDecimationTerrainInfoType : public vtkstd::vector<vtkTerrainInfo> 
+class vtkGreedyTerrainDecimationTerrainInfoType : public std::vector<vtkTerrainInfo>
 {
 public:
-  typedef vtkstd::vector<vtkTerrainInfo> Superclass;
+  typedef std::vector<vtkTerrainInfo> Superclass;
   typedef Superclass::size_type size_type;
   vtkGreedyTerrainDecimationTerrainInfoType(size_type n, const vtkTerrainInfo& value):
-    vtkstd::vector<vtkTerrainInfo>(n,value) {}
+    std::vector<vtkTerrainInfo>(n,value) {}
 };
 
+//----------------------------------------------------------------------
 // Maps mesh point id to input point id
-class vtkGreedyTerrainDecimationPointInfoType : public vtkstd::vector<vtkIdType> {};
+class vtkGreedyTerrainDecimationPointInfoType : public std::vector<vtkIdType> {};
 
 
 // Begin vtkGreedyTerrainDecimation class implementation-----------------------------------------
 //
+//----------------------------------------------------------------------
 vtkGreedyTerrainDecimation::vtkGreedyTerrainDecimation()
 {
   this->ErrorMeasure = VTK_ERROR_SPECIFIED_REDUCTION;
@@ -74,24 +78,31 @@ vtkGreedyTerrainDecimation::vtkGreedyTerrainDecimation()
   this->AbsoluteError = 1;
   this->RelativeError = 0.01;
   this->BoundaryVertexDeletion = 1;
+
+  this->ComputeNormals = 0;
+  this->Normals = 0;
 }
 
+//----------------------------------------------------------------------
 vtkGreedyTerrainDecimation::~vtkGreedyTerrainDecimation()
 {
 }
 
+//----------------------------------------------------------------------
 inline void vtkGreedyTerrainDecimation::GetTerrainPoint(int i, int j, double x[3])
 {
   x[0] = this->Origin[0] + i*this->Spacing[0];
   x[1] = this->Origin[1] + j*this->Spacing[1];
 }
 
+//----------------------------------------------------------------------
 inline void vtkGreedyTerrainDecimation::ComputeImageCoordinates(vtkIdType inputPtId, int ij[2])
 {
   ij[0] = inputPtId % this->Dimensions[0];
   ij[1] = inputPtId / this->Dimensions[0];
 }
 
+//----------------------------------------------------------------------
 inline vtkIdType vtkGreedyTerrainDecimation::InsertNextPoint(vtkIdType inputPtId, 
                                                              double x[3])
 {
@@ -111,11 +122,13 @@ inline vtkIdType vtkGreedyTerrainDecimation::InsertNextPoint(vtkIdType inputPtId
   return this->CurrentPointId++;
 }
 
+//----------------------------------------------------------------------
 inline double *vtkGreedyTerrainDecimation::GetPoint(vtkIdType id)
 {
   return this->Points->GetPointer(3*id);
 }
 
+//----------------------------------------------------------------------
 inline void vtkGreedyTerrainDecimation::GetPoint(vtkIdType id, double x[3])
 {
   double *ptr = this->Points->GetPointer(3*id);
@@ -124,6 +137,7 @@ inline void vtkGreedyTerrainDecimation::GetPoint(vtkIdType id, double x[3])
   x[2] = *ptr;
 }
 
+//----------------------------------------------------------------------
 void vtkGreedyTerrainDecimation::EstimateOutputSize(const vtkIdType numInputPts,
                                                     vtkIdType &numPts, vtkIdType &numTris)
 {
@@ -144,6 +158,7 @@ void vtkGreedyTerrainDecimation::EstimateOutputSize(const vtkIdType numInputPts,
   return;
 }
 
+//----------------------------------------------------------------------
 int vtkGreedyTerrainDecimation::SatisfiesErrorMeasure(double error)
 {
   switch (this->ErrorMeasure)
@@ -154,7 +169,7 @@ int vtkGreedyTerrainDecimation::SatisfiesErrorMeasure(double error)
 
     case VTK_ERROR_SPECIFIED_REDUCTION:
       {
-      float reduction = (float)this->Mesh->GetNumberOfPolys()/this->MaximumNumberOfTriangles;
+      double reduction = (double)this->Mesh->GetNumberOfPolys()/this->MaximumNumberOfTriangles;
       if ( (1.0 - reduction) <= this->Reduction ) return 1;
       }
       break;
@@ -171,6 +186,7 @@ int vtkGreedyTerrainDecimation::SatisfiesErrorMeasure(double error)
   return 0;
 }
 
+//----------------------------------------------------------------------
 //Update all triangles connected to this mesh point
 void vtkGreedyTerrainDecimation::UpdateTriangles(vtkIdType ptId)
 {
@@ -186,6 +202,7 @@ void vtkGreedyTerrainDecimation::UpdateTriangles(vtkIdType ptId)
     }
 }
 
+//----------------------------------------------------------------------
 //Update all points as to which triangle they lie in. Basically a scanline algorithm.
 void vtkGreedyTerrainDecimation::UpdateTriangle(vtkIdType triId, 
                                                 vtkIdType p1, vtkIdType p2, vtkIdType p3)
@@ -196,14 +213,15 @@ void vtkGreedyTerrainDecimation::UpdateTriangle(vtkIdType triId,
   this->ComputeImageCoordinates(p2, ij2);
   this->ComputeImageCoordinates(p3, ij3);
   
-  float h[4]; //extra entry added for interpolated value
-  h[0] = (float) this->Heights->GetTuple1(p1);
-  h[1] = (float) this->Heights->GetTuple1(p2);
-  h[2] = (float) this->Heights->GetTuple1(p3);
+  double h[4]; //extra entry added for interpolated value
+  h[0] = (double) this->Heights->GetTuple1(p1);
+  h[1] = (double) this->Heights->GetTuple1(p2);
+  h[2] = (double) this->Heights->GetTuple1(p3);
 
   this->UpdateTriangle(triId, ij1, ij2, ij3, h);
 }
 
+//----------------------------------------------------------------------
 void vtkGreedyTerrainDecimation::InsertBoundaryVertices()
 {
   int i, j;
@@ -242,6 +260,7 @@ void vtkGreedyTerrainDecimation::InsertBoundaryVertices()
     }
 }
 
+//----------------------------------------------------------------------
 // Determine whether point x is inside of circumcircle of triangle
 // defined by points (x1, x2, x3). Returns non-zero if inside circle.
 // (Note that z-component is ignored.)
@@ -268,6 +287,7 @@ int vtkGreedyTerrainDecimation::InCircle (double x[3], double x1[3], double x2[3
 
 #define VTK_DEL2D_TOLERANCE 1.0e-014
 
+//----------------------------------------------------------------------
 // Recursive method to locate triangle containing point. Starts with arbitrary
 // triangle (tri) and "walks" towards it. Influenced by some of Guibas and 
 // Stolfi's work. Returns id of enclosing triangle, or -1 if no triangle
@@ -291,10 +311,11 @@ vtkIdType vtkGreedyTerrainDecimation::FindTriangle(double x[3], vtkIdType ptIds[
     this->GetPoint(ptIds[i], p[i]);
     }
 
-  // Randomization (of find edge neighbora) avoids walking in 
+  // Randomization (of find edge neighbors) avoids walking in 
   // circles in certain weird cases
   srand(tri);
   ir = rand() % 3;
+  
   // evaluate in/out of each edge
   for (inside=1, minProj=VTK_DEL2D_TOLERANCE, ic=0; ic<3; ic++)
     {
@@ -378,13 +399,19 @@ vtkIdType vtkGreedyTerrainDecimation::FindTriangle(double x[3], vtkIdType ptIds[
 
 #undef VTK_DEL2D_TOLERANCE
 
+//----------------------------------------------------------------------
 // Recursive method checks whether edge is Delaunay, and if not, swaps edge.
 // Continues until all edges are Delaunay. Points p1 and p2 form the edge in
 // question; x is the coordinates of the inserted point; tri is the current
 // triangle id.
 void vtkGreedyTerrainDecimation::CheckEdge(vtkIdType ptId, double x[3], vtkIdType p1,
-                                           vtkIdType p2, vtkIdType tri)
+                                           vtkIdType p2, vtkIdType tri, int depth)
 {
+  if ( depth > 15 )
+    {
+    return;
+    }
+
   int i;
   vtkIdType *pts, npts, numNei, nei, p3;
   double x1[3], x2[3], x3[3];
@@ -432,8 +459,8 @@ void vtkGreedyTerrainDecimation::CheckEdge(vtkIdType ptId, double x[3], vtkIdTyp
       this->Mesh->ReplaceCell(nei,3,swapTri);
 
       // two new edges become suspect
-      this->CheckEdge(ptId, x, p3, p2, tri);
-      this->CheckEdge(ptId, x, p1, p3, nei);
+      this->CheckEdge(ptId, x, p3, p2, tri, ++depth);
+      this->CheckEdge(ptId, x, p1, p3, nei,   depth);
 
       }//in circle
     }//interior edge
@@ -441,6 +468,7 @@ void vtkGreedyTerrainDecimation::CheckEdge(vtkIdType ptId, double x[3], vtkIdTyp
   neighbors->Delete();
 }
 
+//----------------------------------------------------------------------
 vtkIdType vtkGreedyTerrainDecimation::AddPointToTriangulation(vtkIdType inputPtId)
 {
   vtkIdType ptId, nei[3], tri[4];
@@ -471,6 +499,12 @@ vtkIdType vtkGreedyTerrainDecimation::AddPointToTriangulation(vtkIdType inputPtI
     // Insert the point into the output
     ptId = this->InsertNextPoint(inputPtId, x);
 
+    if (this->Normals)
+      {
+      float n[3];
+      this->ComputePointNormal(ij[0], ij[1], n);
+      this->Normals->InsertNextTuple(n);
+      }
     if ( status == VTK_IN_TRIANGLE ) //in triangle
       {
       //delete this triangle; create three new triangles
@@ -490,9 +524,9 @@ vtkIdType vtkGreedyTerrainDecimation::AddPointToTriangulation(vtkIdType inputPtI
 
       // Check edge neighbors for Delaunay criterion. If not satisfied, flip
       // edge diagonal. (This is done recursively.)
-      this->CheckEdge(ptId, x, pts[0], pts[1], tri[0]);
-      this->CheckEdge(ptId, x, pts[1], pts[2], tri[1]);
-      this->CheckEdge(ptId, x, pts[2], pts[0], tri[2]);
+      this->CheckEdge(ptId, x, pts[0], pts[1], tri[0], 0);
+      this->CheckEdge(ptId, x, pts[1], pts[2], tri[1], 0);
+      this->CheckEdge(ptId, x, pts[2], pts[0], tri[2], 0);
       }
 
     else if ( status == VTK_INTERIOR_EDGE ) // on interior triangle edge; has a neighbor
@@ -520,7 +554,7 @@ vtkIdType vtkGreedyTerrainDecimation::AddPointToTriangulation(vtkIdType inputPtI
       nodes[0][0] = ptId; nodes[0][1] = p2; nodes[0][2] = nei[1];
       this->Mesh->ReplaceCell(tri[0], 3, nodes[0]);
 
-      nodes[1][0] = ptId; nodes[1][1] = p1; nodes[1][2] = nei[1];
+      nodes[1][0] = ptId; nodes[1][1] = nei[1]; nodes[1][2] = p1;
       this->Mesh->ReplaceCell(nei[0], 3, nodes[1]);
 
       this->Mesh->InsertNextLinkedPoint(4);
@@ -530,7 +564,7 @@ vtkIdType vtkGreedyTerrainDecimation::AddPointToTriangulation(vtkIdType inputPtI
       tri[1] = nei[0];
 
       //create two new triangles
-      nodes[2][0] = ptId; nodes[2][1] = p2; nodes[2][2] = nei[2];
+      nodes[2][0] = ptId; nodes[2][1] = nei[2]; nodes[2][2] = p2;
       tri[2] = this->Mesh->InsertNextLinkedCell(VTK_TRIANGLE, 3, nodes[2]);
 
       nodes[3][0] = ptId; nodes[3][1] = p1; nodes[3][2] = nei[2];
@@ -539,7 +573,7 @@ vtkIdType vtkGreedyTerrainDecimation::AddPointToTriangulation(vtkIdType inputPtI
       // Check edge neighbors for Delaunay criterion.
       for ( i=0; i<4; i++ )
         {
-        this->CheckEdge (ptId, x, nodes[i][1], nodes[i][2], tri[i]);
+        this->CheckEdge (ptId, x, nodes[i][1], nodes[i][2], tri[i], 0);
         }
       }
 
@@ -558,20 +592,20 @@ vtkIdType vtkGreedyTerrainDecimation::AddPointToTriangulation(vtkIdType inputPtI
       //replace one triangle
       this->Mesh->RemoveReferenceToCell(nei[2],tri[0]);
 
-      nodes[0][0] = ptId; nodes[0][1] = nei[1]; nodes[0][2] = p1;
+      nodes[0][0] = ptId; nodes[0][1] = p1; nodes[0][2] = nei[1];
       this->Mesh->ReplaceCell(tri[0], 3, nodes[0]);
 
       this->Mesh->InsertNextLinkedPoint(2);
       this->Mesh->AddReferenceToCell(ptId,tri[0]);
 
       //create one new triangles
-      nodes[1][0] = ptId; nodes[1][1] = p1; nodes[1][2] = nei[2];
+      nodes[1][0] = ptId; nodes[1][1] = nei[2]; nodes[1][2] = p1;
       tri[1] = this->Mesh->InsertNextLinkedCell(VTK_TRIANGLE, 3, nodes[1]);
 
       // Check edge neighbors for Delaunay criterion.
       for ( i=0; i<2; i++ )
         {
-        this->CheckEdge (ptId, x, nodes[i][1], nodes[i][2], tri[i]);
+        this->CheckEdge (ptId, x, nodes[i][1], nodes[i][2], tri[i], 0);
         }
       }
     
@@ -584,14 +618,92 @@ vtkIdType vtkGreedyTerrainDecimation::AddPointToTriangulation(vtkIdType inputPtI
   return 0;
 }
 
-void vtkGreedyTerrainDecimation::Execute()
+//----------------------------------------------------------------------
+void vtkGreedyTerrainDecimation::ComputePointNormal(int i, int j, float n[3])
 {
-  vtkImageData *input=this->GetInput();
+  vtkDataArray* scalars;
+  double x0, x1, y0, y1, dx, dy;
+  float vx[3], vy[3];
+
+  scalars = this->InputPD->GetScalars();
+
+  dx = dy = 0;
+  // X
+  if (i > 0)
+    {
+    x0 = scalars->GetTuple1(j*this->Dimensions[0] + i - 1);
+    dx += this->Spacing[0];
+    }
+  else
+    {
+    x0 = scalars->GetTuple1(j*this->Dimensions[0] + i);
+    }
+  if (i < this->Dimensions[0]-1)
+    {
+    x1 = scalars->GetTuple1(j*this->Dimensions[0] + i + 1);
+    dx += this->Spacing[0];
+    }
+  else
+    {
+    x1 = scalars->GetTuple1(j*this->Dimensions[0] + i);
+    }
+  // Y
+  if (j > 0)
+    {
+    y0 = scalars->GetTuple1((j-1)*this->Dimensions[0] + i);
+    dy += this->Spacing[1];
+    }
+  else
+    {
+    y0 = scalars->GetTuple1(j*this->Dimensions[0] + i);
+    }
+  if (j < this->Dimensions[1]-1)
+    {
+    y1 = scalars->GetTuple1((j+1)*this->Dimensions[0] + i);
+    dy += this->Spacing[1];
+    }
+  else
+    {
+    y1 = scalars->GetTuple1(j*this->Dimensions[0] + i);
+    }
+
+  if (dx == 0.0 || dy == 0.0)
+    {
+    // This would only happen if the input was not an XY image.
+    vtkErrorMacro("Could not compute normal.");
+    return;
+    }
+  vx[0] = (float)(dx);
+  vx[1] = 0.0;
+  vx[2] = (float)(x1-x0);
+  vy[0] = 0.0;
+  vy[1] = (float)(dy);
+  vy[2] = (float)(y1-y0);
+  vtkMath::Cross(vx, vy, n);
+  vtkMath::Normalize(n);
+}
+
+//----------------------------------------------------------------------
+int vtkGreedyTerrainDecimation::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
+{
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkImageData *input = vtkImageData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkIdType numInputPts=input->GetNumberOfPoints(), numPts, numTris;
   vtkIdType inputPtId;
-  float error, bounds[6], center[3];
+  double error, bounds[6], center[3];
   vtkCellArray *triangles;
-  this->Mesh = this->GetOutput();
+  this->Mesh = output;
   this->InputPD = input->GetPointData();
   this->OutputPD = this->Mesh->GetPointData();
 
@@ -602,19 +714,19 @@ void vtkGreedyTerrainDecimation::Execute()
   if ( input->GetDataDimension() != 2 )
     {
     vtkWarningMacro(<<"This class treats 2D height fields only");
-    return;
+    return 1;
     }
   if ( (this->Heights = this->InputPD->GetScalars()) == NULL )
     {
     vtkWarningMacro(<<"This class requires height scalars");
-    return;
+    return 1;
     }
 
   input->GetBounds(bounds);
   input->GetCenter(center);
   input->GetDimensions(this->Dimensions);
-  float *origin = input->GetOrigin();
-  float *spacing = input->GetSpacing();
+  double *origin = input->GetOrigin();
+  double *spacing = input->GetSpacing();
   for (int ii=0; ii<3; ii++)
     {
     this->Origin[ii] = (double)origin[ii];
@@ -634,7 +746,7 @@ void vtkGreedyTerrainDecimation::Execute()
   // Top element of VTK's priority queue returns the minimum error value. Since we want the
   // maximum error, we use 1/error relationship to insert errors.
   this->TerrainError = vtkPriorityQueue::New();
-  this->TerrainError->Allocate(numInputPts, (vtkIdType)((float)0.25*numInputPts));
+  this->TerrainError->Allocate(numInputPts, (vtkIdType)((double)0.25*numInputPts));
   
   // Initialize the triangle mesh data structures. Double precision point coordinates
   // are required because of the numerical requirements on the Delaunay algorithm.
@@ -646,6 +758,15 @@ void vtkGreedyTerrainDecimation::Execute()
   newPts->Allocate(numPts);
   this->Points = static_cast<vtkDoubleArray *>(newPts->GetData());
   
+  // initailize the normals
+  if (this->ComputeNormals)
+    {
+    this->Normals = vtkFloatArray::New();
+    this->Normals->SetNumberOfComponents(3);
+    this->Normals->Allocate(numPts*3);
+    this->Normals->SetName("Normals");
+    }  
+  
   // Supplemental arrays used to accelerate the algorithm.
   // TerrainInfo contains the "containing" triangle for each point. PointInfo maps the
   // triangle mesh point id to the input image point id.
@@ -664,29 +785,43 @@ void vtkGreedyTerrainDecimation::Execute()
 
   inputPtId = 0;
   newPts->InsertPoint(0, bounds[0],bounds[2], 
-                      (float)this->Heights->GetTuple1(inputPtId)); //ptId=0
+                      (double)this->Heights->GetTuple1(inputPtId)); //ptId=0
   this->OutputPD->CopyData(this->InputPD,inputPtId,0);
   (*this->PointInfo)[0] = inputPtId;
   
   inputPtId = this->Dimensions[0] - 1;
   newPts->InsertPoint(1, bounds[1],bounds[2],
-                      (float)this->Heights->GetTuple1(inputPtId)); //ptId=1
+                      (double)this->Heights->GetTuple1(inputPtId)); //ptId=1
   this->OutputPD->CopyData(this->InputPD,inputPtId,1);
   (*this->PointInfo)[1] = inputPtId;
 
   inputPtId = this->Dimensions[0]*this->Dimensions[1] - 1;
   newPts->InsertPoint(2, bounds[1],bounds[3], 
-                      (float)this->Heights->GetTuple1(inputPtId)); //ptId=2
+                      (double)this->Heights->GetTuple1(inputPtId)); //ptId=2
   this->OutputPD->CopyData(this->InputPD,inputPtId,2);
   (*this->PointInfo)[2] = inputPtId;
 
   inputPtId = this->Dimensions[0]*(this->Dimensions[1]-1);
   newPts->InsertPoint(3, bounds[0],bounds[3], 
-                      (float)this->Heights->GetTuple1(inputPtId)); //ptId=3
+                      (double)this->Heights->GetTuple1(inputPtId)); //ptId=3
   this->OutputPD->CopyData(this->InputPD,inputPtId,3);
   (*this->PointInfo)[3] = inputPtId;
   this->CurrentPointId = 4;
 
+  // Handle normals of the four corners.
+  if (this->Normals)
+    {
+    float n[3];
+    this->ComputePointNormal(0, 0, n);
+    this->Normals->InsertNextTuple(n);
+    this->ComputePointNormal(this->Dimensions[0]-1, 0, n);
+    this->Normals->InsertNextTuple(n);
+    this->ComputePointNormal(this->Dimensions[0]-1, this->Dimensions[1]-1, n);
+    this->Normals->InsertNextTuple(n);
+    this->ComputePointNormal(0, this->Dimensions[1]-1, n);
+    this->Normals->InsertNextTuple(n);
+    }
+    
   // Insert initial triangles into output mesh
   triangles = vtkCellArray::New();
   triangles->Allocate(numTris,3);
@@ -734,12 +869,19 @@ void vtkGreedyTerrainDecimation::Execute()
       this->AddPointToTriangulation(inputPtId);
       if ( ! (++numInsertedPoints % tenth) )
         {
-        this->UpdateProgress( (float)(numInsertedPoints>numPts?numPts:numInsertedPoints)/numPts);
+        this->UpdateProgress( (double)(numInsertedPoints>numPts?numPts:numInsertedPoints)/numPts);
         abortExecute = this->GetAbortExecute();
         }
       }
     }
 
+  if (this->Normals)
+    {
+    this->OutputPD->SetNormals(this->Normals);
+    this->Normals->Delete();
+    this->Normals = 0;
+    }
+
   vtkDebugMacro(<<"Output TIN contains: " << this->Mesh->GetNumberOfPoints() << " points"
                 <<"and " << this->Mesh->GetNumberOfPolys() << " triangles");
 
@@ -753,6 +895,8 @@ void vtkGreedyTerrainDecimation::Execute()
 
   newPts->Delete();
   triangles->Delete();
+
+  return 1;
 }
 
 /*----------------------------------------------------------------------
@@ -795,14 +939,14 @@ void vtkGreedyTerrainDecimation::Execute()
 // that the point is in, plus computing the error in the height field.
 //
 void vtkGreedyTerrainDecimation::UpdateTriangle(vtkIdType tri, int ij1[2], int ij2[2], int ij3[2],
-                                                float h[3])
+                                                double h[3])
 {
   int *min, *max, *midL, *midR, *mid, mid2[2];
   double t, tt;
   int i, j, xL, xR;
-  float hMin, hMax, hMidL, hMidR, hL, hR;
+  double hMin, hMax, hMidL, hMidR, hL, hR;
   vtkIdType idx, inputPtId, maxInputPtId=0;
-  float error, maxError=0.0;
+  double error, maxError=0.0;
 
   int type = this->CharacterizeTriangle(ij1, ij2, ij3, min, max, midL, midR, mid, mid2, 
                                         h, hMin, hMax, hMidL, hMidR);
@@ -834,7 +978,7 @@ void vtkGreedyTerrainDecimation::UpdateTriangle(vtkIdType tri, int ij1[2], int i
               {
               error = hL;
               }
-            error = fabs( (float)this->Heights->GetTuple1(inputPtId) - error );
+            error = fabs( (double)this->Heights->GetTuple1(inputPtId) - error );
             if ( error > maxError )
               {
               maxError = error;
@@ -874,7 +1018,7 @@ void vtkGreedyTerrainDecimation::UpdateTriangle(vtkIdType tri, int ij1[2], int i
               {
               error = hL;
               }
-            error = fabs( (float)this->Heights->GetTuple1(inputPtId) - error );
+            error = fabs( (double)this->Heights->GetTuple1(inputPtId) - error );
             if ( error > maxError )
               {
               maxError = error;
@@ -898,14 +1042,15 @@ void vtkGreedyTerrainDecimation::UpdateTriangle(vtkIdType tri, int ij1[2], int i
 }
 
 
+//----------------------------------------------------------------------
 // Characterize the configuration of the triangle based on image coordinates
 // (All points in triangulation are from an image).
 //
 int vtkGreedyTerrainDecimation::CharacterizeTriangle(int ij1[2], int ij2[2], int ij3[3],
                                                      int* &min, int* &max, int* &midL, int* &midR,
-                                                     int* &mid, int mid2[2], float h[3], 
-                                                     float &hMin, float &hMax, float &hL, 
-                                                     float &hR)
+                                                     int* &mid, int mid2[2], double h[3], 
+                                                     double &hMin, double &hMax, double &hL, 
+                                                     double &hR)
 {
   // Check for situations where one edge of triangle is horizontal
   //
@@ -1059,7 +1204,7 @@ int vtkGreedyTerrainDecimation::CharacterizeTriangle(int ij1[2], int ij2[2], int
     }
 
   // Find the midL and midR
-  float hMid, hMid2;
+  double hMid, hMid2;
   if ( ij1 != min && ij1 != max)
     {
     mid = ij1;
@@ -1101,6 +1246,14 @@ int vtkGreedyTerrainDecimation::CharacterizeTriangle(int ij1[2], int ij2[2], int
   return VTK_TWO_TRIANGLES;
 }
 
+//----------------------------------------------------------------------
+int vtkGreedyTerrainDecimation::FillInputPortInformation(int, vtkInformation *info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkImageData");
+  return 1;
+}
+
+//----------------------------------------------------------------------
 void vtkGreedyTerrainDecimation::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
@@ -1129,4 +1282,6 @@ void vtkGreedyTerrainDecimation::PrintSelf(ostream& os, vtkIndent indent)
   
   os << indent << "BoundaryVertexDeletion: " 
      << (this->BoundaryVertexDeletion ? "On\n" : "Off\n");
+  os << indent << "ComputeNormals: " 
+     << (this->ComputeNormals ? "On\n" : "Off\n");
 }
