@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkGreedyTerrainDecimation.h,v $
+  Module:    vtkGreedyTerrainDecimation.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -63,13 +63,14 @@
 #ifndef __vtkGreedyTerrainDecimation_h
 #define __vtkGreedyTerrainDecimation_h
 
-#include "vtkStructuredPointsToPolyDataFilter.h"
+#include "vtkPolyDataAlgorithm.h"
 
 class vtkPriorityQueue;
 class vtkDataArray;
 class vtkPointData;
 class vtkIdList;
 class vtkDoubleArray;
+class vtkFloatArray;
 
 //PIMPL Encapsulation for STL containers
 class vtkGreedyTerrainDecimationTerrainInfoType;
@@ -80,10 +81,10 @@ class vtkGreedyTerrainDecimationPointInfoType;
 #define VTK_ERROR_ABSOLUTE            2
 #define VTK_ERROR_RELATIVE            3
 
-class VTK_HYBRID_EXPORT vtkGreedyTerrainDecimation : public vtkStructuredPointsToPolyDataFilter
+class VTK_HYBRID_EXPORT vtkGreedyTerrainDecimation : public vtkPolyDataAlgorithm
 {
 public:
-  vtkTypeRevisionMacro(vtkGreedyTerrainDecimation,vtkStructuredPointsToPolyDataFilter);
+  vtkTypeMacro(vtkGreedyTerrainDecimation,vtkPolyDataAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -111,7 +112,7 @@ public:
   // good idea to make sure this is less than a tessellated mesh
   // at full resolution.) You need to set this value only when
   // the error measure is set to NumberOfTriangles.
-  vtkSetClampMacro(NumberOfTriangles,vtkIdType,2,VTK_LONG_MAX);
+  vtkSetClampMacro(NumberOfTriangles,vtkIdType,2,VTK_LARGE_ID);
   vtkGetMacro(NumberOfTriangles,vtkIdType);
 
   // Description:
@@ -143,18 +144,29 @@ public:
   vtkGetMacro(BoundaryVertexDeletion,int);
   vtkBooleanMacro(BoundaryVertexDeletion,int);
 
+  // Description:
+  // Compute normals based on the input image. Off by default.
+  vtkSetMacro(ComputeNormals, int);
+  vtkGetMacro(ComputeNormals, int);
+  vtkBooleanMacro(ComputeNormals, int);
+
 protected:
   vtkGreedyTerrainDecimation();
   ~vtkGreedyTerrainDecimation();
 
-  void Execute();
+  virtual int RequestData(vtkInformation *, vtkInformationVector **, vtkInformationVector *);
+  virtual int FillInputPortInformation(int port, vtkInformation *info);
+
+  int ComputeNormals;
+  vtkFloatArray* Normals;
+  void ComputePointNormal(int i, int j, float n[3]);
 
   //ivars that the API addresses
   int       ErrorMeasure;
   vtkIdType NumberOfTriangles;
-  double     Reduction;
-  double     AbsoluteError;
-  double     RelativeError;
+  double    Reduction;
+  double    AbsoluteError;
+  double    RelativeError;
   int       BoundaryVertexDeletion; //Can we delete boundary vertices?
 
   //Used for convenience
@@ -170,7 +182,7 @@ protected:
   double         Origin[3];
   double         Spacing[3];
   vtkIdType      MaximumNumberOfTriangles;
-  double          Length;
+  double         Length;
 
   //Bookeeping arrays
   vtkPriorityQueue                          *TerrainError; //errors for each pt in height field
@@ -199,7 +211,7 @@ protected:
   vtkIdType FindTriangle(double x[3], vtkIdType ptIds[3], vtkIdType tri,
                          double tol, vtkIdType nei[3], vtkIdList *neighbors, int& status);
   void CheckEdge(vtkIdType ptId, double x[3], vtkIdType p1, vtkIdType p2,
-                 vtkIdType tri);
+                 vtkIdType tri, int depth);
 
   void UpdateTriangles(vtkIdType meshPtId); //update all points connected to this point
   void UpdateTriangle(vtkIdType triId, vtkIdType p1, vtkIdType p2, vtkIdType p3);
@@ -217,4 +229,3 @@ private:
 };
 
 #endif
-
