@@ -1,17 +1,14 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkGridTransform.cxx,v $
-  Language:  C++
-  Date:      $Date: 2002/06/17 14:10:53 $
-  Version:   $Revision: 1.17 $
+  Module:    vtkGridTransform.cxx
 
-  Copyright (c) 1993-2002 Ken Martin, Will Schroeder, Bill Lorensen 
+  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
   See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
 
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
+     This software is distributed WITHOUT ANY WARRANTY; without even
+     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
@@ -19,36 +16,15 @@
 
 #include "vtkImageData.h"
 #include "vtkMath.h"
+#include "vtkImageInterpolatorInternals.h"
 #include "vtkObjectFactory.h"
 
 #include "math.h"
 
-vtkCxxRevisionMacro(vtkGridTransform, "$Revision: 1.17 $");
 vtkStandardNewMacro(vtkGridTransform);
 
 vtkCxxSetObjectMacro(vtkGridTransform,DisplacementGrid,vtkImageData);
 
-//----------------------------------------------------------------------------
-// fast floor() function for converting a float to an int
-// (the floor() implementation on some computers is much slower than this,
-// because they require some 'exact' behaviour that we don't).
-
-inline int vtkGridFloor(float x, float &f)
-{
-  int ix = int(x);
-  f = x-ix;
-  if (f < 0) { f = x - (--ix); }
-
-  return ix;
-}
-
-inline int vtkGridFloor(float x)
-{
-  int ix = int(x);
-  if (x-ix < 0) { ix--; }
-
-  return ix;
-}
 
 //----------------------------------------------------------------------------
 // Nearest-neighbor interpolation of a displacement grid.
@@ -57,7 +33,7 @@ inline int vtkGridFloor(float x)
 // and one which doesn't.
 
 template <class T>
-inline void vtkNearestHelper(float displacement[3], T *gridPtr, int increment)
+inline void vtkNearestHelper(double displacement[3], T *gridPtr, int increment)
 {
   gridPtr += increment;
   displacement[0] = gridPtr[0];
@@ -65,15 +41,16 @@ inline void vtkNearestHelper(float displacement[3], T *gridPtr, int increment)
   displacement[2] = gridPtr[2];
 }
 
-inline void vtkNearestNeighborInterpolation(float point[3], 
-                                            float displacement[3],
+inline void vtkNearestNeighborInterpolation(double point[3], 
+                                            double displacement[3],
                                             void *gridPtr, int gridType,
-                                            int gridExt[6], int gridInc[3])
+                                            int gridExt[6],
+                                            vtkIdType gridInc[3])
 {
   int gridId[3];
-  gridId[0] = vtkGridFloor(point[0]+0.5f)-gridExt[0];
-  gridId[1] = vtkGridFloor(point[1]+0.5f)-gridExt[2];
-  gridId[2] = vtkGridFloor(point[2]+0.5f)-gridExt[4];
+  gridId[0] = vtkInterpolationMath::Round(point[0])-gridExt[0];
+  gridId[1] = vtkInterpolationMath::Round(point[1])-gridExt[2];
+  gridId[2] = vtkInterpolationMath::Round(point[2])-gridExt[4];
   
   int ext[3];
   ext[0] = gridExt[1]-gridExt[0];
@@ -99,38 +76,25 @@ inline void vtkNearestNeighborInterpolation(float point[3],
     }
 
   // do nearest-neighbor interpolation
-  int increment = gridId[0]*gridInc[0] + 
-                  gridId[1]*gridInc[1] + 
-                  gridId[2]*gridInc[2];
+  vtkIdType increment = gridId[0]*gridInc[0] + 
+                        gridId[1]*gridInc[1] + 
+                        gridId[2]*gridInc[2];
 
   switch (gridType)
     {
-    case VTK_CHAR:
-      vtkNearestHelper(displacement, (char *)gridPtr, increment);
-      break;
-    case VTK_UNSIGNED_CHAR:
-      vtkNearestHelper(displacement, (unsigned char *)gridPtr, increment); 
-      break;
-    case VTK_SHORT:
-      vtkNearestHelper(displacement, (short *)gridPtr, increment);
-      break;
-    case VTK_UNSIGNED_SHORT:
-      vtkNearestHelper(displacement, (unsigned short *)gridPtr, increment);
-      break;
-    case VTK_FLOAT:
-      vtkNearestHelper(displacement, (float *)gridPtr, increment);
-      break;
+    vtkTemplateMacro(
+      vtkNearestHelper(displacement, static_cast<VTK_TT*>(gridPtr), increment));
     }
 }
 
 template <class T>
-inline void vtkNearestHelper(float displacement[3], float derivatives[3][3],
+inline void vtkNearestHelper(double displacement[3], double derivatives[3][3],
                              T *gridPtr, int gridId[3], int gridId0[3], 
-                             int gridId1[3], int gridInc[3])
+                             int gridId1[3], vtkIdType gridInc[3])
 {
-  int incX = gridId[0]*gridInc[0];
-  int incY = gridId[1]*gridInc[1];
-  int incZ = gridId[2]*gridInc[2];
+  vtkIdType incX = gridId[0]*gridInc[0];
+  vtkIdType incY = gridId[1]*gridInc[1];
+  vtkIdType incZ = gridId[2]*gridInc[2];
 
   T *gridPtr0;
   T *gridPtr1 = gridPtr + incX + incY + incZ;
@@ -139,13 +103,13 @@ inline void vtkNearestHelper(float displacement[3], float derivatives[3][3],
   displacement[1] = gridPtr1[1];
   displacement[2] = gridPtr1[2];
 
-  int incX0 = gridId0[0]*gridInc[0];
-  int incX1 = gridId1[0]*gridInc[0];
-  int incY0 = gridId0[1]*gridInc[1];
+  vtkIdType incX0 = gridId0[0]*gridInc[0];
+  vtkIdType incX1 = gridId1[0]*gridInc[0];
+  vtkIdType incY0 = gridId0[1]*gridInc[1];
 
-  int incY1 = gridId1[1]*gridInc[1];
-  int incZ0 = gridId0[2]*gridInc[2];
-  int incZ1 = gridId1[2]*gridInc[2];
+  vtkIdType incY1 = gridId1[1]*gridInc[1];
+  vtkIdType incZ0 = gridId0[2]*gridInc[2];
+  vtkIdType incZ1 = gridId1[2]*gridInc[2];
 
   gridPtr0 = gridPtr + incX0 + incY + incZ;
   gridPtr1 = gridPtr + incX1 + incY + incZ;
@@ -169,9 +133,10 @@ inline void vtkNearestHelper(float displacement[3], float derivatives[3][3],
   derivatives[2][2] = gridPtr1[2] - gridPtr0[2];
 }
 
-void vtkNearestNeighborInterpolation(float point[3], float displacement[3],
-                                     float derivatives[3][3], void *gridPtr, 
-                                     int gridType, int gridExt[6], int gridInc[3])
+void vtkNearestNeighborInterpolation(double point[3], double displacement[3],
+                                     double derivatives[3][3], void *gridPtr, 
+                                     int gridType, int gridExt[6],
+                                     vtkIdType gridInc[3])
 {
   if (derivatives == NULL)
     {
@@ -180,11 +145,11 @@ void vtkNearestNeighborInterpolation(float point[3], float displacement[3],
     return;
     }
 
-  float f[3];
+  double f[3];
   int gridId0[3];
-  gridId0[0] = vtkGridFloor(point[0],f[0])-gridExt[0];
-  gridId0[1] = vtkGridFloor(point[1],f[1])-gridExt[2];
-  gridId0[2] = vtkGridFloor(point[2],f[2])-gridExt[4];
+  gridId0[0] = vtkInterpolationMath::Floor(point[0],f[0])-gridExt[0];
+  gridId0[1] = vtkInterpolationMath::Floor(point[1],f[1])-gridExt[2];
+  gridId0[2] = vtkInterpolationMath::Floor(point[2],f[2])-gridExt[4];
 
   int gridId[3], gridId1[3];
   gridId[0] = gridId1[0] = gridId0[0] + 1;
@@ -234,26 +199,9 @@ void vtkNearestNeighborInterpolation(float point[3], float displacement[3],
   // do nearest-neighbor interpolation
   switch (gridType)
     {
-    case VTK_CHAR:
-      vtkNearestHelper(displacement, derivatives, (char *)gridPtr, 
-                       gridId, gridId0, gridId1, gridInc);
-      break;
-    case VTK_UNSIGNED_CHAR:
-      vtkNearestHelper(displacement, derivatives, (unsigned char *)gridPtr, 
-                       gridId, gridId0, gridId1, gridInc);
-      break;
-    case VTK_SHORT:
-      vtkNearestHelper(displacement, derivatives, (short *)gridPtr, 
-                       gridId, gridId0, gridId1, gridInc);
-      break;
-    case VTK_UNSIGNED_SHORT:
-      vtkNearestHelper(displacement, derivatives, (unsigned short *)gridPtr, 
-                       gridId, gridId0, gridId1, gridInc);
-      break;
-    case VTK_FLOAT:
-      vtkNearestHelper(displacement, derivatives, (float *)gridPtr, 
-                       gridId, gridId0, gridId1, gridInc);
-      break;
+    vtkTemplateMacro(
+      vtkNearestHelper(displacement, derivatives, static_cast<VTK_TT*>(gridPtr),
+                       gridId, gridId0, gridId1, gridInc));
     }
 }
 
@@ -262,28 +210,28 @@ void vtkNearestNeighborInterpolation(float point[3], float displacement[3],
 // The displacement as well as the derivatives are returned.
 
 template <class T>
-inline void vtkLinearHelper(float displacement[3], float derivatives[3][3],
-                            float fx, float fy, float fz, T *gridPtr, 
+inline void vtkLinearHelper(double displacement[3], double derivatives[3][3],
+                            double fx, double fy, double fz, T *gridPtr, 
                             int i000, int i001, int i010, int i011,
                             int i100, int i101, int i110, int i111)
 {
-  float rx = 1 - fx;
-  float ry = 1 - fy;
-  float rz = 1 - fz;
+  double rx = 1 - fx;
+  double ry = 1 - fy;
+  double rz = 1 - fz;
   
-  float ryrz = ry*rz;
-  float ryfz = ry*fz;
-  float fyrz = fy*rz;
-  float fyfz = fy*fz;
+  double ryrz = ry*rz;
+  double ryfz = ry*fz;
+  double fyrz = fy*rz;
+  double fyfz = fy*fz;
 
-  float rxryrz = rx*ryrz;
-  float rxryfz = rx*ryfz;
-  float rxfyrz = rx*fyrz;
-  float rxfyfz = rx*fyfz;
-  float fxryrz = fx*ryrz;
-  float fxryfz = fx*ryfz;
-  float fxfyrz = fx*fyrz;
-  float fxfyfz = fx*fyfz;
+  double rxryrz = rx*ryrz;
+  double rxryfz = rx*ryfz;
+  double rxfyrz = rx*fyrz;
+  double rxfyfz = rx*fyfz;
+  double fxryrz = fx*ryrz;
+  double fxryfz = fx*ryfz;
+  double fxfyrz = fx*fyrz;
+  double fxfyfz = fx*fyfz;
 
   if (!derivatives)
     {
@@ -300,17 +248,17 @@ inline void vtkLinearHelper(float displacement[3], float derivatives[3][3],
     }
   else
     {
-    float rxrz = rx*rz;
-    float rxfz = rx*fz;
-    float fxrz = fx*rz;
-    float fxfz = fx*fz;
+    double rxrz = rx*rz;
+    double rxfz = rx*fz;
+    double fxrz = fx*rz;
+    double fxfz = fx*fz;
     
-    float rxry = rx*ry;
-    float rxfy = rx*fy;
-    float fxry = fx*ry;
-    float fxfy = fx*fy;
+    double rxry = rx*ry;
+    double rxfy = rx*fy;
+    double fxry = fx*ry;
+    double fxfy = fx*fy;
     
-    float *derivative = *derivatives;
+    double *derivative = *derivatives;
 
     int i = 3;
     do
@@ -341,15 +289,15 @@ inline void vtkLinearHelper(float displacement[3], float derivatives[3][3],
     }
 }
 
-void vtkTrilinearInterpolation(float point[3], float displacement[3],
-                               float derivatives[3][3], void *gridPtr, int gridType, 
-                               int gridExt[6], int gridInc[3])
+void vtkTrilinearInterpolation(double point[3], double displacement[3],
+                               double derivatives[3][3], void *gridPtr, int gridType, 
+                               int gridExt[6], vtkIdType gridInc[3])
 {
   // change point into integer plus fraction
-  float f[3];
-  int floorX = vtkGridFloor(point[0],f[0]);
-  int floorY = vtkGridFloor(point[1],f[1]);
-  int floorZ = vtkGridFloor(point[2],f[2]);
+  double f[3];
+  int floorX = vtkInterpolationMath::Floor(point[0],f[0]);
+  int floorY = vtkInterpolationMath::Floor(point[1],f[1]);
+  int floorZ = vtkInterpolationMath::Floor(point[2],f[2]);
 
   int gridId0[3];
   gridId0[0] = floorX - gridExt[0];
@@ -389,50 +337,28 @@ void vtkTrilinearInterpolation(float point[3], float displacement[3],
     }
 
   // do trilinear interpolation
-  int factX0 = gridId0[0]*gridInc[0];
-  int factY0 = gridId0[1]*gridInc[1];
-  int factZ0 = gridId0[2]*gridInc[2];
+  vtkIdType factX0 = gridId0[0]*gridInc[0];
+  vtkIdType factY0 = gridId0[1]*gridInc[1];
+  vtkIdType factZ0 = gridId0[2]*gridInc[2];
 
-  int factX1 = gridId1[0]*gridInc[0];
-  int factY1 = gridId1[1]*gridInc[1];
-  int factZ1 = gridId1[2]*gridInc[2];
+  vtkIdType factX1 = gridId1[0]*gridInc[0];
+  vtkIdType factY1 = gridId1[1]*gridInc[1];
+  vtkIdType factZ1 = gridId1[2]*gridInc[2];
     
-  int i000 = factX0+factY0+factZ0;
-  int i001 = factX0+factY0+factZ1;
-  int i010 = factX0+factY1+factZ0;
-  int i011 = factX0+factY1+factZ1;
-  int i100 = factX1+factY0+factZ0;
-  int i101 = factX1+factY0+factZ1;
-  int i110 = factX1+factY1+factZ0;
-  int i111 = factX1+factY1+factZ1;
+  vtkIdType i000 = factX0+factY0+factZ0;
+  vtkIdType i001 = factX0+factY0+factZ1;
+  vtkIdType i010 = factX0+factY1+factZ0;
+  vtkIdType i011 = factX0+factY1+factZ1;
+  vtkIdType i100 = factX1+factY0+factZ0;
+  vtkIdType i101 = factX1+factY0+factZ1;
+  vtkIdType i110 = factX1+factY1+factZ0;
+  vtkIdType i111 = factX1+factY1+factZ1;
   
   switch (gridType)
     {
-    case VTK_CHAR:
-      vtkLinearHelper(displacement, derivatives, f[0], f[1], f[2], 
-                      (char *)gridPtr,
-                      i000, i001, i010, i011, i100, i101, i110, i111);
-      break;
-    case VTK_UNSIGNED_CHAR:
-      vtkLinearHelper(displacement, derivatives, f[0], f[1], f[2], 
-                      (unsigned char *)gridPtr,
-                      i000, i001, i010, i011, i100, i101, i110, i111);
-      break;
-    case VTK_SHORT:
-      vtkLinearHelper(displacement, derivatives, f[0], f[1], f[2], 
-                      (short *)gridPtr, 
-                      i000, i001, i010, i011, i100, i101, i110, i111);
-      break;
-    case VTK_UNSIGNED_SHORT:
-      vtkLinearHelper(displacement, derivatives, f[0], f[1], f[2], 
-                      (unsigned short *)gridPtr,
-                      i000, i001, i010, i011, i100, i101, i110, i111);
-      break;
-    case VTK_FLOAT:
-      vtkLinearHelper(displacement, derivatives, f[0], f[1], f[2], 
-                      (float *)gridPtr,
-                      i000, i001, i010, i011, i100, i101, i110, i111);
-      break;
+    vtkTemplateMacro(vtkLinearHelper(displacement, derivatives, f[0], f[1], f[2], 
+        static_cast<VTK_TT*>(gridPtr),
+        i000, i001, i010, i011, i100, i101, i110, i111));
     }
 }
 
@@ -450,10 +376,10 @@ void vtkTrilinearInterpolation(float point[3], float displacement[3],
 // helper function: set up the lookup indices and the interpolation 
 // coefficients
 
-void vtkSetTricubicInterpCoeffs(float F[4], int *l, int *m, float f, 
+void vtkSetTricubicInterpCoeffs(double F[4], int *l, int *m, double f, 
                                 int interpMode)
 {   
-  float fp1,fm1,fm2;
+  double fp1,fm1,fm2;
 
   switch (interpMode)
     {
@@ -502,10 +428,10 @@ void vtkSetTricubicInterpCoeffs(float F[4], int *l, int *m, float f,
 }
 
 // set coefficients to be used to find the derivative of the cubic
-void vtkSetTricubicDerivCoeffs(float F[4], float G[4], int *l, int *m, 
-                               float f, int interpMode)
+void vtkSetTricubicDerivCoeffs(double F[4], double G[4], int *l, int *m, 
+                               double f, int interpMode)
 {   
-  float fp1,fm1,fm2;
+  double fp1,fm1,fm2;
 
   switch (interpMode)
     {
@@ -577,22 +503,23 @@ void vtkSetTricubicDerivCoeffs(float F[4], float G[4], int *l, int *m,
 // (set derivatives to NULL to avoid computing them).
 
 template <class T>
-inline void vtkCubicHelper(float displacement[3], float derivatives[3][3],
-                           float fx, float fy, float fz, T *gridPtr,
+inline void vtkCubicHelper(double displacement[3], double derivatives[3][3],
+                           double fx, double fy, double fz, T *gridPtr,
                            int interpModeX, int interpModeY, int interpModeZ,
-                           int factX[4], int factY[4], int factZ[4])
+                           vtkIdType factX[4], vtkIdType factY[4],
+                           vtkIdType factZ[4])
 {
-  float fX[4],fY[4],fZ[4];
-  float gX[4],gY[4],gZ[4];
+  double fX[4],fY[4],fZ[4];
+  double gX[4],gY[4],gZ[4];
   int jl,jm,kl,km,ll,lm;
 
   if (derivatives)
     {
     for (int i = 0; i < 3; i++)
       {
-      derivatives[i][0] = 0.0f; 
-      derivatives[i][1] = 0.0f; 
-      derivatives[i][2] = 0.0f;
+      derivatives[i][0] = 0.0; 
+      derivatives[i][1] = 0.0; 
+      derivatives[i][2] = 0.0;
       }
     vtkSetTricubicDerivCoeffs(fX,gX,&ll,&lm,fx,interpModeX);
     vtkSetTricubicDerivCoeffs(fY,gY,&kl,&km,fy,interpModeY);
@@ -607,7 +534,7 @@ inline void vtkCubicHelper(float displacement[3], float derivatives[3][3],
 
   // Here is the tricubic interpolation
   // (or cubic-cubic-linear, or cubic-nearest-cubic, etc)
-  float vY[3],vZ[3];
+  double vY[3],vZ[3];
   displacement[0] = 0;
   displacement[1] = 0;
   displacement[2] = 0;
@@ -628,7 +555,7 @@ inline void vtkCubicHelper(float displacement[3], float derivatives[3][3],
         for (int l = ll; l < lm; l++)
           {
           T *gridPtr3 = gridPtr2 + factX[l];
-          float f = fX[l];
+          double f = fX[l];
           vY[0] += gridPtr3[0] * f;
           vY[1] += gridPtr3[1] * f;
           vY[2] += gridPtr3[2] * f;
@@ -639,11 +566,11 @@ inline void vtkCubicHelper(float displacement[3], float derivatives[3][3],
         for (int l = ll; l < lm; l++)
           {
           T *gridPtr3 = gridPtr2 + factX[l];
-          float f = fX[l];
-          float gff = gX[l]*fY[k]*fZ[j];
-          float fgf = fX[l]*gY[k]*fZ[j];
-          float ffg = fX[l]*fY[k]*gZ[j];
-          float inVal = gridPtr3[0];
+          double f = fX[l];
+          double gff = gX[l]*fY[k]*fZ[j];
+          double fgf = fX[l]*gY[k]*fZ[j];
+          double ffg = fX[l]*fY[k]*gZ[j];
+          double inVal = gridPtr3[0];
           vY[0] += inVal * f;
           derivatives[0][0] += inVal * gff;
           derivatives[0][1] += inVal * fgf;
@@ -670,17 +597,18 @@ inline void vtkCubicHelper(float displacement[3], float derivatives[3][3],
     }
 }
 
-void vtkTricubicInterpolation(float point[3], float displacement[3], 
-                              float derivatives[3][3], void *gridPtr, 
-                              int gridType, int gridExt[6], int gridInc[3])
+void vtkTricubicInterpolation(double point[3], double displacement[3], 
+                              double derivatives[3][3], void *gridPtr, 
+                              int gridType, int gridExt[6],
+                              vtkIdType gridInc[3])
 {
-  int factX[4],factY[4],factZ[4];
+  vtkIdType factX[4],factY[4],factZ[4];
 
   // change point into integer plus fraction
-  float f[3];
-  int floorX = vtkGridFloor(point[0],f[0]);
-  int floorY = vtkGridFloor(point[1],f[1]);
-  int floorZ = vtkGridFloor(point[2],f[2]);
+  double f[3];
+  int floorX = vtkInterpolationMath::Floor(point[0],f[0]);
+  int floorY = vtkInterpolationMath::Floor(point[1],f[1]);
+  int floorZ = vtkInterpolationMath::Floor(point[2],f[2]);
 
   int gridId0[3];
   gridId0[0] = floorX - gridExt[0];
@@ -753,43 +681,18 @@ void vtkTricubicInterpolation(float point[3], float displacement[3],
 
   switch (gridType)
     {
-    case VTK_CHAR:
+    vtkTemplateMacro(
       vtkCubicHelper(displacement, derivatives, f[0], f[1], f[2],
-                     (char *)gridPtr,
+                     static_cast<VTK_TT*>(gridPtr),
                      interpModeX, interpModeY, interpModeZ,
-                     factX, factY, factZ);
-      break;
-    case VTK_UNSIGNED_CHAR:
-      vtkCubicHelper(displacement, derivatives, f[0], f[1], f[2],
-                     (unsigned char *)gridPtr,
-                     interpModeX, interpModeY, interpModeZ,
-                     factX, factY, factZ);
-      break;
-    case VTK_SHORT:
-      vtkCubicHelper(displacement, derivatives, f[0], f[1], f[2],
-                     (short *)gridPtr,
-                     interpModeX, interpModeY, interpModeZ,
-                     factX, factY, factZ);
-      break;
-    case VTK_UNSIGNED_SHORT:
-      vtkCubicHelper(displacement, derivatives, f[0], f[1], f[2],
-                     (unsigned short *)gridPtr,
-                     interpModeX, interpModeY, interpModeZ,
-                     factX, factY, factZ);
-      break;
-    case VTK_FLOAT:
-      vtkCubicHelper(displacement, derivatives, f[0], f[1], f[2],
-                     (float *)gridPtr,
-                     interpModeX, interpModeY, interpModeZ,
-                     factX, factY, factZ);
-      break;
+                     factX, factY, factZ));
     }
 }                 
 
 //----------------------------------------------------------------------------
 vtkGridTransform::vtkGridTransform()
 {
-  this->InterpolationMode = VTK_GRID_LINEAR;
+  this->InterpolationMode = VTK_LINEAR_INTERPOLATION;
   this->InterpolationFunction = &vtkTrilinearInterpolation;
   this->DisplacementGrid = NULL;
   this->DisplacementScale = 1.0;
@@ -850,13 +753,13 @@ void vtkGridTransform::SetInterpolationMode(int mode)
   this->InterpolationMode = mode;
   switch(mode)
     {
-    case VTK_GRID_NEAREST:
+    case VTK_NEAREST_INTERPOLATION:
       this->InterpolationFunction = &vtkNearestNeighborInterpolation;
       break;
-    case VTK_GRID_LINEAR:
+    case VTK_LINEAR_INTERPOLATION:
       this->InterpolationFunction = &vtkTrilinearInterpolation;
       break;
-    case VTK_GRID_CUBIC:
+    case VTK_CUBIC_INTERPOLATION:
       this->InterpolationFunction = &vtkTricubicInterpolation;
       break;
     default:
@@ -866,8 +769,8 @@ void vtkGridTransform::SetInterpolationMode(int mode)
 }
 
 //----------------------------------------------------------------------------
-void vtkGridTransform::ForwardTransformPoint(const float inPoint[3], 
-                                             float outPoint[3])
+void vtkGridTransform::ForwardTransformPoint(const double inPoint[3], 
+                                             double outPoint[3])
 {
   if (this->DisplacementGrid == NULL)
     {
@@ -877,20 +780,19 @@ void vtkGridTransform::ForwardTransformPoint(const float inPoint[3],
     return;
     }
 
-  vtkImageData *grid = this->DisplacementGrid;
-  void *gridPtr = grid->GetScalarPointer();
-  int gridType = grid->GetScalarType();
+  void *gridPtr = this->GridPointer;
+  int gridType = this->GridScalarType;
 
-  float *spacing = grid->GetSpacing();
-  float *origin = grid->GetOrigin();
-  int *extent = grid->GetExtent();
-  int *increments = grid->GetIncrements();
+  double *spacing = this->GridSpacing;
+  double *origin = this->GridOrigin;
+  int *extent = this->GridExtent;
+  vtkIdType *increments = this->GridIncrements;
 
-  float scale = this->DisplacementScale;
-  float shift = this->DisplacementShift;
-
-  float point[3];
-  float displacement[3];
+  double scale = this->DisplacementScale;
+  double shift = this->DisplacementShift;
+  
+  double point[3];
+  double displacement[3];
 
   // Convert the inPoint to i,j,k indices into the deformation grid 
   // plus fractions
@@ -907,28 +809,28 @@ void vtkGridTransform::ForwardTransformPoint(const float inPoint[3],
 }
 
 //----------------------------------------------------------------------------
-// convert double to float
-void vtkGridTransform::ForwardTransformPoint(const double point[3], 
-                                             double output[3])
+// convert double to double
+void vtkGridTransform::ForwardTransformPoint(const float point[3], 
+                                             float output[3])
 {
-  float fpoint[3];
+  double fpoint[3];
   fpoint[0] = point[0]; 
   fpoint[1] = point[1]; 
   fpoint[2] = point[2];
 
   this->ForwardTransformPoint(fpoint,fpoint);
  
-  output[0] = fpoint[0]; 
-  output[1] = fpoint[1]; 
-  output[2] = fpoint[2];
+  output[0] = static_cast<float>(fpoint[0]); 
+  output[1] = static_cast<float>(fpoint[1]); 
+  output[2] = static_cast<float>(fpoint[2]);
 }
 
 //----------------------------------------------------------------------------
 // calculate the derivative of the grid transform: only cubic interpolation
 // provides well-behaved derivative so we always use that.
-void vtkGridTransform::ForwardTransformDerivative(const float inPoint[3],
-                                                  float outPoint[3],
-                                                  float derivative[3][3])
+void vtkGridTransform::ForwardTransformDerivative(const double inPoint[3],
+                                                  double outPoint[3],
+                                                  double derivative[3][3])
 {
   if (this->DisplacementGrid == NULL)
     {
@@ -939,20 +841,19 @@ void vtkGridTransform::ForwardTransformDerivative(const float inPoint[3],
     return;
     }
 
-  vtkImageData *grid = this->DisplacementGrid;
-  void *gridPtr = grid->GetScalarPointer();
-  int gridType = grid->GetScalarType();
+  void *gridPtr = this->GridPointer;
+  int gridType = this->GridScalarType;
 
-  float *spacing = grid->GetSpacing();
-  float *origin = grid->GetOrigin();
-  int *extent = grid->GetExtent();
-  int *increments = grid->GetIncrements();
+  double *spacing = this->GridSpacing;
+  double *origin = this->GridOrigin;
+  int *extent = this->GridExtent;
+  vtkIdType *increments = this->GridIncrements;
 
-  float scale = this->DisplacementScale;
-  float shift = this->DisplacementShift;
+  double scale = this->DisplacementScale;
+  double shift = this->DisplacementShift;
 
-  float point[3];
-  float displacement[3];
+  double point[3];
+  double displacement[3];
 
   // convert the inPoint to i,j,k indices plus fractions
   point[0] = (inPoint[0] - origin[0])/spacing[0];
@@ -967,7 +868,7 @@ void vtkGridTransform::ForwardTransformDerivative(const float inPoint[3],
     derivative[i][0] = derivative[i][0]*scale/spacing[0];
     derivative[i][1] = derivative[i][1]*scale/spacing[1];
     derivative[i][2] = derivative[i][2]*scale/spacing[2];
-    derivative[i][i] += 1.0f;
+    derivative[i][i] += 1.0;
     }
 
   outPoint[0] = inPoint[0] + (displacement[0]*scale + shift);
@@ -976,13 +877,13 @@ void vtkGridTransform::ForwardTransformDerivative(const float inPoint[3],
 }  
 
 //----------------------------------------------------------------------------
-// convert double to float
-void vtkGridTransform::ForwardTransformDerivative(const double point[3],
-                                                  double output[3],
-                                                  double derivative[3][3])
+// convert double to double
+void vtkGridTransform::ForwardTransformDerivative(const float point[3],
+                                                  float output[3],
+                                                  float derivative[3][3])
 {
-  float fpoint[3];
-  float fderivative[3][3];
+  double fpoint[3];
+  double fderivative[3][3];
   fpoint[0] = point[0];
   fpoint[1] = point[1];
   fpoint[2] = point[2];
@@ -991,10 +892,10 @@ void vtkGridTransform::ForwardTransformDerivative(const double point[3],
 
   for (int i = 0; i < 3; i++)
     {
-    derivative[i][0] = fderivative[i][0];
-    derivative[i][1] = fderivative[i][1];
-    derivative[i][2] = fderivative[i][2];
-    output[i] = fpoint[i];
+    derivative[i][0] = static_cast<float>(fderivative[i][0]);
+    derivative[i][1] = static_cast<float>(fderivative[i][1]);
+    derivative[i][2] = static_cast<float>(fderivative[i][2]);
+    output[i] = static_cast<float>(fpoint[i]);
     }
 }
 
@@ -1004,9 +905,9 @@ void vtkGridTransform::ForwardTransformDerivative(const double point[3],
 // singular.
 // Note that this is similar to vtkWarpTransform::InverseTransformPoint()
 // but has been optimized specifically for grid transforms.
-void vtkGridTransform::InverseTransformDerivative(const float inPoint[3], 
-                                                  float outPoint[3],
-                                                  float derivative[3][3])
+void vtkGridTransform::InverseTransformDerivative(const double inPoint[3], 
+                                                  double outPoint[3],
+                                                  double derivative[3][3])
 {
   if (this->DisplacementGrid == NULL)
     {
@@ -1016,36 +917,35 @@ void vtkGridTransform::InverseTransformDerivative(const float inPoint[3],
     return;
     }
 
-  vtkImageData *grid = this->DisplacementGrid;
-  void *gridPtr = grid->GetScalarPointer();
-  int gridType = grid->GetScalarType();
+  void *gridPtr = this->GridPointer;
+  int gridType = this->GridScalarType;
 
-  float *spacing = grid->GetSpacing();
-  float *origin = grid->GetOrigin();
-  int *extent = grid->GetExtent();
-  int *increments = grid->GetIncrements();
+  double *spacing = this->GridSpacing;
+  double *origin = this->GridOrigin;
+  int *extent = this->GridExtent;
+  vtkIdType *increments = this->GridIncrements;
 
-  float invSpacing[3];
-  invSpacing[0] = 1.0f/spacing[0];
-  invSpacing[1] = 1.0f/spacing[1];
-  invSpacing[2] = 1.0f/spacing[2];
+  double invSpacing[3];
+  invSpacing[0] = 1.0/spacing[0];
+  invSpacing[1] = 1.0/spacing[1];
+  invSpacing[2] = 1.0/spacing[2];
 
-  float shift = this->DisplacementShift;
-  float scale = this->DisplacementScale;
+  double shift = this->DisplacementShift;
+  double scale = this->DisplacementScale;
 
-  float point[3], inverse[3], lastInverse[3];
-  float deltaP[3], deltaI[3];
+  double point[3], inverse[3], lastInverse[3];
+  double deltaP[3], deltaI[3];
 
-  float functionValue = 0;
-  float functionDerivative = 0;
-  float lastFunctionValue = VTK_FLOAT_MAX;
+  double functionValue = 0;
+  double functionDerivative = 0;
+  double lastFunctionValue = VTK_DOUBLE_MAX;
 
-  float errorSquared = 0.0;
-  float toleranceSquared = this->InverseTolerance;
+  double errorSquared = 0.0;
+  double toleranceSquared = this->InverseTolerance;
   toleranceSquared *= toleranceSquared;
 
-  float f = 1.0f;
-  float a;
+  double f = 1.0;
+  double a;
 
   // convert the inPoint to i,j,k indices plus fractions
   point[0] = (inPoint[0] - origin[0])*invSpacing[0];
@@ -1084,7 +984,7 @@ void vtkGridTransform::InverseTransformDerivative(const float inPoint[3],
       derivative[j][0] = derivative[j][0]*scale*invSpacing[0];
       derivative[j][1] = derivative[j][1]*scale*invSpacing[1];
       derivative[j][2] = derivative[j][2]*scale*invSpacing[2];
-      derivative[j][j] += 1.0f;
+      derivative[j][j] += 1.0;
       }
 
     // get the current function value
@@ -1176,13 +1076,13 @@ void vtkGridTransform::InverseTransformDerivative(const float inPoint[3],
 }
 
 //----------------------------------------------------------------------------
-// convert double to float and back again
-void vtkGridTransform::InverseTransformDerivative(const double point[3], 
-                                                  double output[3],
-                                                  double derivative[3][3])
+// convert double to double and back again
+void vtkGridTransform::InverseTransformDerivative(const float point[3], 
+                                                  float output[3],
+                                                  float derivative[3][3])
 {
-  float fpoint[3];
-  float fderivative[3][3];
+  double fpoint[3];
+  double fderivative[3][3];
   fpoint[0] = point[0]; 
   fpoint[1] = point[1]; 
   fpoint[2] = point[2];
@@ -1191,38 +1091,38 @@ void vtkGridTransform::InverseTransformDerivative(const double point[3],
  
   for (int i = 0; i < 3; i++)
     {
-    output[i] = fpoint[i]; 
-    derivative[i][0] = fderivative[i][0];
-    derivative[i][1] = fderivative[i][1];
-    derivative[i][2] = fderivative[i][2];
+    output[i] = static_cast<float>(fpoint[i]); 
+    derivative[i][0] = static_cast<float>(fderivative[i][0]);
+    derivative[i][1] = static_cast<float>(fderivative[i][1]);
+    derivative[i][2] = static_cast<float>(fderivative[i][2]);
     }
 }
 
 //----------------------------------------------------------------------------
-void vtkGridTransform::InverseTransformPoint(const float point[3], 
-                                             float output[3])
+void vtkGridTransform::InverseTransformPoint(const double point[3], 
+                                             double output[3])
 {
   // the derivative won't be used, but it is required for Newton's method
-  float derivative[3][3];
+  double derivative[3][3];
   this->InverseTransformDerivative(point,output,derivative);
 }
 
 //----------------------------------------------------------------------------
-// convert double to float and back again
-void vtkGridTransform::InverseTransformPoint(const double point[3], 
-                                             double output[3])
+// convert double to double and back again
+void vtkGridTransform::InverseTransformPoint(const float point[3], 
+                                             float output[3])
 {
-  float fpoint[3];
-  float fderivative[3][3];
+  double fpoint[3];
+  double fderivative[3][3];
   fpoint[0] = point[0]; 
   fpoint[1] = point[1]; 
   fpoint[2] = point[2];
 
   this->InverseTransformDerivative(fpoint,fpoint,fderivative);
  
-  output[0] = fpoint[0]; 
-  output[1] = fpoint[1]; 
-  output[2] = fpoint[2];
+  output[0] = static_cast<float>(fpoint[0]); 
+  output[1] = static_cast<float>(fpoint[1]); 
+  output[2] = static_cast<float>(fpoint[2]);
 }
 
 //----------------------------------------------------------------------------
@@ -1267,14 +1167,23 @@ void vtkGridTransform::InternalUpdate()
       grid->GetScalarType() != VTK_UNSIGNED_CHAR &&
       grid->GetScalarType() != VTK_SHORT &&
       grid->GetScalarType() != VTK_UNSIGNED_SHORT &&
-      grid->GetScalarType() != VTK_FLOAT)
+      grid->GetScalarType() != VTK_FLOAT &&
+      grid->GetScalarType() != VTK_DOUBLE)
     {
     vtkErrorMacro(<< "TransformPoint: displacement grid is of unsupported numerical type");
     return;
     }
- 
+
   grid->SetUpdateExtent(grid->GetWholeExtent());
   grid->Update();
+
+  this->GridPointer = grid->GetScalarPointer();
+  this->GridScalarType = grid->GetScalarType();
+
+  grid->GetSpacing(this->GridSpacing);
+  grid->GetOrigin(this->GridOrigin);
+  grid->GetExtent(this->GridExtent);
+  grid->GetIncrements(this->GridIncrements);
 }
 
 //----------------------------------------------------------------------------
