@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageToPolyDataFilter.cxx,v $
+  Module:    vtkImageToPolyDataFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,6 +19,8 @@
 #include "vtkCellData.h"
 #include "vtkEdgeTable.h"
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkLine.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
@@ -26,7 +28,6 @@
 #include "vtkScalarsToColors.h"
 #include "vtkUnsignedCharArray.h"
 
-vtkCxxRevisionMacro(vtkImageToPolyDataFilter, "$Revision: 1.26 $");
 vtkStandardNewMacro(vtkImageToPolyDataFilter);
 
 vtkCxxSetObjectMacro(vtkImageToPolyDataFilter,LookupTable,vtkScalarsToColors);
@@ -44,7 +45,6 @@ vtkImageToPolyDataFilter::vtkImageToPolyDataFilter()
 
   this->Table = vtkUnsignedCharArray::New();
   this->LookupTable = NULL;
-
 }
 
 vtkImageToPolyDataFilter::~vtkImageToPolyDataFilter()
@@ -59,10 +59,21 @@ vtkImageToPolyDataFilter::~vtkImageToPolyDataFilter()
 // declare helper functions
 //
 
-void vtkImageToPolyDataFilter::Execute()
+int vtkImageToPolyDataFilter::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  vtkImageData *input=this->GetInput();
-  vtkPolyData *output=this->GetOutput();
+  // get the info objects
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the input and output
+  vtkImageData *input = vtkImageData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkPolyData *tmpOutput;
   vtkPolyData *tmpInput;
   vtkAppendPolyData *append;
@@ -77,14 +88,13 @@ void vtkImageToPolyDataFilter::Execute()
   int i, j, newDims[3], totalPieces, pieceNum, abortExecute=0;
   double newOrigin[3];
 
-
   // Check input and initialize
   vtkDebugMacro(<<"Vectorizing image...");
   
   if ( inScalars == NULL || numPixels < 1 )
     {
     vtkDebugMacro(<<"Not enough input to create output");
-    return;
+    return 1;
     }
 
   append = vtkAppendPolyData::New();
@@ -189,6 +199,8 @@ void vtkImageToPolyDataFilter::Execute()
   append->Delete();
   tmpInput->Delete();
   tmpOutput->Delete();
+
+  return 1;
 }
 
 void vtkImageToPolyDataFilter::PixelizeImage(vtkUnsignedCharArray *pixels, 
@@ -543,6 +555,12 @@ void vtkImageToPolyDataFilter::BuildTable(unsigned char *vtkNotUsed(inPixels))
     }
 }
 
+int vtkImageToPolyDataFilter::FillInputPortInformation(int, vtkInformation *info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkImageData");
+  return 1;
+}
+
 void vtkImageToPolyDataFilter::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
@@ -1249,6 +1267,7 @@ void vtkImageToPolyDataFilter::BuildPolygons(vtkUnsignedCharArray *vtkNotUsed(po
 void vtkImageToPolyDataFilter::SmoothEdges(vtkUnsignedCharArray *pointDescr, 
                                            vtkPolyData *edges)
 {
+  
   vtkPoints *points=edges->GetPoints();
   vtkIdType numPts=points->GetNumberOfPoints(), ptId;
   int i, iterNum;
@@ -1257,6 +1276,7 @@ void vtkImageToPolyDataFilter::SmoothEdges(vtkUnsignedCharArray *pointDescr,
   unsigned short int ncells;
   vtkIdType *cells, *pts, npts;
 
+
   // For each smoothing operation, loop over points. Points that can be 
   // smoothed are moved in the direction of the average of their neighbor
   // points.
@@ -1281,7 +1301,19 @@ void vtkImageToPolyDataFilter::SmoothEdges(vtkUnsignedCharArray *pointDescr,
         for (i=0; i<ncells; i++)
           {
           edges->GetCellPoints(cells[i], npts, pts);
-          connId = (pts[0] != ptId ? pts[0] : pts[1]);
+          if (pts[0] != ptId)
+            {
+            connId = pts[0];
+            }
+          else if (npts > 1)
+            {
+            connId = pts[1];
+            }
+          else
+            {
+            vtkErrorMacro("Bad cell in smoothing operation");
+            connId = pts[0];
+            }
           points->GetPoint(connId, xconn);
           xave[0] += xconn[0]; xave[1] += xconn[1]; xave[2] += xconn[2];
           }
@@ -1340,5 +1372,3 @@ void vtkImageToPolyDataFilter::DecimateEdges(vtkPolyData *edges,
       } //if manifold
     } //for all points
 }
-
-
