@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkTransformToGrid.cxx,v $
+  Module:    vtkTransformToGrid.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,9 +17,11 @@
 #include "vtkAbstractTransform.h"
 #include "vtkIdentityTransform.h"
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
-vtkCxxRevisionMacro(vtkTransformToGrid, "$Revision: 1.13 $");
 vtkStandardNewMacro(vtkTransformToGrid);
 
 vtkCxxSetObjectMacro(vtkTransformToGrid,Input,vtkAbstractTransform);
@@ -34,18 +36,20 @@ vtkTransformToGrid::vtkTransformToGrid()
   for (int i = 0; i < 3; i++)
     {
     this->GridExtent[2*i] = this->GridExtent[2*i+1] = 0;
-    this->GridOrigin[i] = 0.0f;
-    this->GridSpacing[i] = 1.0f;
+    this->GridOrigin[i] = 0.0;
+    this->GridSpacing[i] = 1.0;
     }
 
-  this->DisplacementScale = 1.0f;
-  this->DisplacementShift = 0.0f;
+  this->DisplacementScale = 1.0;
+  this->DisplacementShift = 0.0;
+  this->SetNumberOfInputPorts(0);
+  this->SetNumberOfOutputPorts(1);
 }
 
 //----------------------------------------------------------------------------
 vtkTransformToGrid::~vtkTransformToGrid()
 {
-  this->SetInput(NULL);
+  this->SetInput(static_cast<vtkAbstractTransform*>(0));
 }
 
 //----------------------------------------------------------------------------
@@ -89,21 +93,30 @@ void vtkTransformToGrid::PrintSelf(ostream& os, vtkIndent indent)
 
 //----------------------------------------------------------------------------
 // This method returns the largest data that can be generated.
-void vtkTransformToGrid::ExecuteInformation()
+void vtkTransformToGrid::RequestInformation (
+  vtkInformation * vtkNotUsed(request),
+  vtkInformationVector ** vtkNotUsed( inputVector ),
+  vtkInformationVector *outputVector)
 {
+  // get the info objects
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
+
+
   if (this->GetInput() == NULL)
     {
     vtkErrorMacro("Missing input");
     return;
     }
 
+  // update the transform, maybe in the future make transforms part of the
+  // pipeline
   this->Input->Update();
 
-  this->GetOutput()->SetWholeExtent(this->GridExtent);
-  this->GetOutput()->SetSpacing(this->GridSpacing);
-  this->GetOutput()->SetOrigin(this->GridOrigin);
-  this->GetOutput()->SetScalarType(this->GridScalarType);
-  this->GetOutput()->SetNumberOfScalarComponents(3);
+  outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),
+               this->GridExtent,6);
+  outInfo->Set(vtkDataObject::SPACING(),this->GridSpacing,3);
+  outInfo->Set(vtkDataObject::ORIGIN(),this->GridOrigin,3);
+  vtkDataObject::SetPointDataActiveScalarInfo(outInfo, this->GridScalarType, 3);
 }
 
 //----------------------------------------------------------------------------
@@ -118,8 +131,8 @@ void vtkTransformToGridMinMax(vtkTransformToGrid *self, int extent[6],
 
   if (!transform)
     {
-    minDisplacement = -1.0f;
-    maxDisplacement = +1.0f;
+    minDisplacement = -1.0;
+    maxDisplacement = +1.0;
     return;
     }
 
@@ -170,8 +183,8 @@ void vtkTransformToGrid::UpdateShiftScale()
   // nothing to do for double or double
   if (gridType == VTK_DOUBLE || gridType == VTK_DOUBLE)
     {
-    this->DisplacementShift = 0.0f;
-    this->DisplacementScale = 1.0f;
+    this->DisplacementShift = 0.0;
+    this->DisplacementScale = 1.0;
     vtkDebugMacro(<< "displacement (scale, shift) = (" << 
                   this->DisplacementScale << ", " << 
                   this->DisplacementShift << ")");  
@@ -223,9 +236,9 @@ void vtkTransformToGrid::UpdateShiftScale()
   this->DisplacementShift = ((typeMax*minDisplacement-typeMin*maxDisplacement)/
                              (typeMax - typeMin)); 
 
-  if (this->DisplacementScale == 0.0f)
+  if (this->DisplacementScale == 0.0)
     {
-    this->DisplacementScale = 1.0f;
+    this->DisplacementScale = 1.0;
     }
 
   vtkDebugMacro(<< "displacement (scale, shift) = (" << 
@@ -279,9 +292,9 @@ void vtkTransformToGridExecute(vtkTransformToGrid *self,
 
   double *spacing = grid->GetSpacing();
   double *origin = grid->GetOrigin();
-  int *increments = grid->GetIncrements();
+  vtkIdType *increments = grid->GetIncrements();
 
-  double invScale = 1.0f/scale;
+  double invScale = 1.0/scale;
 
   double point[3];
   double newPoint[3];
@@ -337,9 +350,19 @@ void vtkTransformToGridExecute(vtkTransformToGrid *self,
 }       
 
 //----------------------------------------------------------------------------
-void vtkTransformToGrid::ExecuteData(vtkDataObject *output)
+void vtkTransformToGrid::RequestData(
+  vtkInformation* vtkNotUsed( request ),
+  vtkInformationVector** vtkNotUsed( inputVector ),
+  vtkInformationVector* outputVector)
 {
-  vtkImageData *grid = this->AllocateOutputData(output);
+  // get the data object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+  vtkImageData *grid = vtkImageData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
+  grid->SetExtent(
+    outInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT()));
+  grid->AllocateScalars();
   int *extent = grid->GetExtent();
 
   double *gridPtr = (double *)grid->GetScalarPointerForExtent(extent);
@@ -382,7 +405,7 @@ void vtkTransformToGrid::ExecuteData(vtkDataObject *output)
 //----------------------------------------------------------------------------
 unsigned long vtkTransformToGrid::GetMTime()
 {
-  unsigned long mtime = this->vtkImageSource::GetMTime();
+  unsigned long mtime = this->Superclass::GetMTime();
 
   if (this->Input)
     {
@@ -395,3 +418,59 @@ unsigned long vtkTransformToGrid::GetMTime()
 
   return mtime;
 }
+
+//----------------------------------------------------------------------------
+int vtkTransformToGrid::ProcessRequest(vtkInformation* request,
+                                      vtkInformationVector** inputVector,
+                                      vtkInformationVector* outputVector)
+{
+  // generate the data
+  if(request->Has(vtkDemandDrivenPipeline::REQUEST_DATA()))
+    {
+    this->RequestData(request, inputVector, outputVector);
+    return 1;
+    }
+
+  // execute information
+  if(request->Has(vtkDemandDrivenPipeline::REQUEST_INFORMATION()))
+    {
+    this->RequestInformation(request, inputVector, outputVector);
+    // after executing set the origin and spacing from the
+    // info
+    int i;
+    for (i = 0; i < this->GetNumberOfOutputPorts(); ++i)
+      {
+      vtkInformation* info = outputVector->GetInformationObject(i);
+      vtkImageData *output = 
+        vtkImageData::SafeDownCast(info->Get(vtkDataObject::DATA_OBJECT()));
+      // if execute info didn't set origin and spacing then we set them
+      if (!info->Has(vtkDataObject::ORIGIN()))
+        {
+        info->Set(vtkDataObject::ORIGIN(),0,0,0);
+        info->Set(vtkDataObject::SPACING(),1,1,1);
+        }
+      if (output)
+        {
+        output->SetOrigin(info->Get(vtkDataObject::ORIGIN()));
+        output->SetSpacing(info->Get(vtkDataObject::SPACING()));
+        }
+      }
+    return 1;
+    }
+
+  return this->Superclass::ProcessRequest(request, inputVector, outputVector);
+}
+
+//----------------------------------------------------------------------------
+vtkImageData* vtkTransformToGrid::GetOutput()
+{
+  return vtkImageData::SafeDownCast(this->GetOutputDataObject(0));
+}
+
+int vtkTransformToGrid::FillOutputPortInformation(
+  int vtkNotUsed(port), vtkInformation* info)
+{
+  // now add our info
+  info->Set(vtkDataObject::DATA_TYPE_NAME(), "vtkImageData");
+  return 1;
+}
