@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkWin32VideoSource.cxx,v $
+  Module:    vtkWin32VideoSource.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -21,10 +21,19 @@
 
 #include <ctype.h>
 
-#include <windows.h>
+// because of warnings in windows header push and pop the warning level
+#ifdef _MSC_VER
+#pragma warning (push, 3)
+#endif
+
+#include "vtkWindows.h"
 #include <winuser.h>
 #include <vfw.h>
 
+#ifdef _MSC_VER
+#pragma warning (pop)
+#endif
+
 class vtkWin32VideoSourceInternal
 {
 public:
@@ -40,19 +49,8 @@ public:
 // VFW compressed formats are listed at http://www.webartz.com/fourcc/
 #define VTK_BI_UYVY 0x59565955
 
-vtkCxxRevisionMacro(vtkWin32VideoSource, "$Revision: 1.23 $");
 vtkStandardNewMacro(vtkWin32VideoSource);
 
-#if ( _MSC_VER >= 1300 ) // Visual studio .NET
-#pragma warning ( disable : 4311 )
-#pragma warning ( disable : 4312 )
-#  define vtkGetWindowLong GetWindowLongPtr
-#  define vtkSetWindowLong SetWindowLongPtr
-#else // regular Visual studio 
-#  define vtkGetWindowLong GetWindowLong
-#  define vtkSetWindowLong SetWindowLong
-#endif // 
-
 //----------------------------------------------------------------------------
 vtkWin32VideoSource::vtkWin32VideoSource()
 {
@@ -104,7 +102,7 @@ vtkWin32VideoSourceWinProc(HWND hwnd, UINT message,
                            WPARAM wParam, LPARAM lParam)
 {
   vtkWin32VideoSource *self = (vtkWin32VideoSource *)\
-    (vtkGetWindowLong(hwnd,GWL_USERDATA));
+    (vtkGetWindowLong(hwnd,vtkGWL_USERDATA));
 
   switch(message) {
   
@@ -138,7 +136,7 @@ LRESULT PASCAL vtkWin32VideoSourceCapControlProc(HWND hwndC, int nState)
   if (nState == CONTROLCALLBACK_PREROLL)
     {
     //cerr << "controlcallback preroll\n";
-    self->SetStartTimeStamp(vtkTimerLog::GetCurrentTime());      
+    self->SetStartTimeStamp(vtkTimerLog::GetUniversalTime());      
     }
   else if (nState == CONTROLCALLBACK_CAPTURING)
     {
@@ -152,7 +150,7 @@ LRESULT PASCAL vtkWin32VideoSourceCapControlProc(HWND hwndC, int nState)
 LRESULT PASCAL vtkWin32VideoSourceCallbackProc(HWND hwndC, LPVIDEOHDR lpVHdr)
 {
   vtkWin32VideoSource *self = (vtkWin32VideoSource *)(capGetUserData(hwndC));
-  self->InternalGrab(lpVHdr);
+  self->LocalInternalGrab(lpVHdr);
 
   return 0;
 }
@@ -219,7 +217,7 @@ void vtkWin32VideoSource::Initialize()
   WNDCLASS wc;
   wc.lpszClassName = this->WndClassName;
   wc.hInstance = hinstance;
-  wc.lpfnWndProc = &vtkWin32VideoSourceWinProc;
+  wc.lpfnWndProc = reinterpret_cast<WNDPROC>(&vtkWin32VideoSourceWinProc);
   wc.hCursor = LoadCursor(NULL,IDC_ARROW);
   wc.hIcon = NULL;
   wc.lpszMenuName = NULL;
@@ -238,7 +236,7 @@ void vtkWin32VideoSource::Initialize()
     sprintf(this->WndClassName,"VTKVideo %d",i);
     }
     
-  if (i > 32)
+  if (i > 10)
     {
     vtkErrorMacro(<< "Initialize: failed to register VTKVideo class"\
                     << " (" << GetLastError() << ")");
@@ -276,7 +274,7 @@ void vtkWin32VideoSource::Initialize()
     }
 
   // set the user data to 'this'
-  vtkSetWindowLong(this->Internal->ParentWnd,GWL_USERDATA,(LONG)this);
+  vtkSetWindowLong(this->Internal->ParentWnd,vtkGWL_USERDATA,(vtkLONG)this);
 
   // Create the capture window
   this->Internal->CapWnd = capCreateCaptureWindow("Capture",
@@ -470,7 +468,7 @@ void vtkWin32VideoSource::OnParentWndDestroy()
 //----------------------------------------------------------------------------
 // copy the Device Independent Bitmap from the VFW framebuffer into the
 // vtkVideoSource framebuffer (don't do the unpacking yet)
-void vtkWin32VideoSource::InternalGrab(void* lpptr)
+void vtkWin32VideoSource::LocalInternalGrab(void* lpptr)
 {
   LPVIDEOHDR lpVHdr = static_cast<LPVIDEOHDR>(lpptr);
   // cerr << "Grabbed\n";
@@ -569,7 +567,7 @@ void vtkWin32VideoSource::Grab()
     }
 
   // just do the grab, the callback does the rest
-  this->SetStartTimeStamp(vtkTimerLog::GetCurrentTime());
+  this->SetStartTimeStamp(vtkTimerLog::GetUniversalTime());
   capGrabFrameNoStop(this->Internal->CapWnd);
 }
 
