@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkWin32VideoSource.h,v $
+  Module:    vtkWin32VideoSource.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -36,7 +36,7 @@ class VTK_HYBRID_EXPORT vtkWin32VideoSource : public vtkVideoSource
 {
 public:
   static vtkWin32VideoSource *New();
-  vtkTypeRevisionMacro(vtkWin32VideoSource,vtkVideoSource);
+  vtkTypeMacro(vtkWin32VideoSource,vtkVideoSource);
   void PrintSelf(ostream& os, vtkIndent indent);   
 
   // Description:
@@ -58,6 +58,8 @@ public:
   // Description:
   // Request a particular frame size (set the third value to 1).
   void SetFrameSize(int x, int y, int z);
+  virtual void SetFrameSize(int dim[3]) { 
+    this->SetFrameSize(dim[0], dim[1], dim[2]); };
   
   // Description:
   // Request a particular frame rate (default 30 frames per second).
@@ -93,7 +95,7 @@ public:
 
   // Description:
   // For internal use only
-  void InternalGrab(void*);
+  void LocalInternalGrab(void*);
   void OnParentWndDestroy();
 
 protected:
