@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkAVSucdReader.cxx,v $
+  Module:    vtkAVSucdReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,12 +19,12 @@
 // Aguilera (daniel.aguilera at cea.fr, CEA-DAM, France) who contributed code
 // und advice.  Please address all comments to Jean Favre (jfavre at cscs.ch)
 
-#include <ctype.h>
-
 #include "vtkAVSucdReader.h"
 #include "vtkDataArraySelection.h"
 #include "vtkErrorCode.h"
 #include "vtkUnstructuredGrid.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkFieldData.h"
 #include "vtkPointData.h"
@@ -36,9 +36,9 @@
 #include "vtkByteSwap.h"
 #include "vtkCellArray.h"
 
-vtkCxxRevisionMacro(vtkAVSucdReader, "$Revision: 1.15 $");
 vtkStandardNewMacro(vtkAVSucdReader);
 
+//----------------------------------------------------------------------------
 vtkAVSucdReader::vtkAVSucdReader()
 {
   this->FileName  = NULL;
@@ -49,15 +49,20 @@ vtkAVSucdReader::vtkAVSucdReader()
   this->NumberOfFields = 0;
   this->NumberOfNodeComponents = 0;
   this->NumberOfCellComponents = 0;
-  this->fs = NULL;
+  this->FileStream = NULL;
   this->DecrementNodeIds = 0;
-
+  this->NumberOfNodes = 0;
+  this->NumberOfCells = 0;
+  
   this->NodeDataInfo = NULL;
   this->CellDataInfo = NULL;
   this->PointDataArraySelection = vtkDataArraySelection::New();
   this->CellDataArraySelection = vtkDataArraySelection::New();
+
+  this->SetNumberOfInputPorts(0);
 }
 
+//----------------------------------------------------------------------------
 vtkAVSucdReader::~vtkAVSucdReader()
 {
   if (this->FileName)
@@ -78,17 +83,20 @@ vtkAVSucdReader::~vtkAVSucdReader()
 }
 
 
+//----------------------------------------------------------------------------
 void vtkAVSucdReader::SetByteOrderToBigEndian()
 {
   this->ByteOrder = FILE_BIG_ENDIAN;
 }
 
 
+//----------------------------------------------------------------------------
 void vtkAVSucdReader::SetByteOrderToLittleEndian()
 {
   this->ByteOrder = FILE_LITTLE_ENDIAN;
 }
 
+//----------------------------------------------------------------------------
 const char *vtkAVSucdReader::GetByteOrderAsString()
 {
   if ( this->ByteOrder ==  FILE_LITTLE_ENDIAN)
@@ -101,22 +109,32 @@ const char *vtkAVSucdReader::GetByteOrderAsString()
     }
 }
 
-void vtkAVSucdReader::Execute()
+//----------------------------------------------------------------------------
+int vtkAVSucdReader::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
 {
+  // get the info object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the ouptut
+  vtkUnstructuredGrid *output = vtkUnstructuredGrid::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   vtkDebugMacro( << "Reading AVS UCD file");
 
-  // If ExecuteInformation() failed the fs will be NULL and
+  // If ExecuteInformation() failed FileStream will be NULL and
   // ExecuteInformation() will have spit out an error.
-  if ( this->fs == NULL )
+  if ( this->FileStream )
     {
-    return;
+    this->ReadFile(output);
     }
 
-  this->ReadFile();
-
-  return;
+  return 1;
 }
 
+//----------------------------------------------------------------------------
 void vtkAVSucdReader::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
@@ -142,78 +160,96 @@ void vtkAVSucdReader::PrintSelf(ostream& os, vtkIndent indent)
 }
 
 
-void vtkAVSucdReader::ReadFile()
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::ReadFile(vtkUnstructuredGrid *output)
 {
-  this->ReadGeometry();
+  this->ReadGeometry(output);
 
   if(this->NumberOfNodeFields)
     {
-    this->ReadNodeData();
+    this->ReadNodeData(output);
     }
 
   if(this->NumberOfCellFields)
     {
-    this->ReadCellData(); 
+    this->ReadCellData(output);
     }
 
-  delete this->fs;
-  this->fs = NULL;
+  delete this->FileStream;
+  this->FileStream = NULL;
 }
 
 
-void vtkAVSucdReader::ExecuteInformation()
+//----------------------------------------------------------------------------
+int vtkAVSucdReader::RequestInformation(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *vtkNotUsed(outputVector))
 {
   char magic_number='\0';
-  long TrueFileLength, CalculatedFileLength;
-  int i, k, *ncomp_list;
+  long trueFileLength, calculatedFileLength;
+  int i, j, k, *ncomp_list;
 
   // first open file in binary mode to check the first byte.
   if ( !this->FileName )
     {
-    this->NumberOfNodes = 0;
-    this->NumberOfCells = 0;
-    this->NumberOfNodeFields = 0;
-    this->NumberOfCellFields = 0;
-    this->NumberOfFields = 0;
     vtkErrorMacro("No filename specified");
-    return;
+    return 0;
     }
   
 #ifdef _WIN32
-    this->fs = new ifstream(this->FileName, ios::in | ios::binary);
+    this->FileStream = new ifstream(this->FileName, ios::in | ios::binary);
 #else
-    this->fs = new ifstream(this->FileName, ios::in);
+    this->FileStream = new ifstream(this->FileName, ios::in);
 #endif
-  if (this->fs->fail())
+  if (this->FileStream->fail())
     {
     this->SetErrorCode(vtkErrorCode::FileNotFoundError);
-    delete this->fs;
-    this->fs = NULL;
+    delete this->FileStream;
+    this->FileStream = NULL;
     vtkErrorMacro("Specified filename not found");
-    return;
+    return 0;
     }
 
-  this->fs->get(magic_number);
-  this->fs->putback(magic_number);
+  this->FileStream->get(magic_number);
+  this->FileStream->putback(magic_number);
   if(magic_number != 7)
     { // most likely an ASCII file
     this->BinaryFile = 0;
-    delete this->fs; // close file to reopen it later
-    this->fs = NULL;
+    delete this->FileStream; // close file to reopen it later
+    this->FileStream = NULL;
 
-    this->fs = new ifstream(this->FileName, ios::in);
-    char c='\0', buf[100];
-    while(this->fs->get(c) && c == '#')
+    this->FileStream = new ifstream(this->FileName, ios::in);
+    char c='\0';
+    while (!FileStream->eof())
       {
-      this->fs->get(buf, 100, '\n'); this->fs->get(c);
+      // skip leading whitespace
+      while(isspace(FileStream->peek()))
+        {
+        FileStream->get(c);
+        }
+      // skip comment lines
+      if (FileStream->peek() == '#')
+        {
+        while(this->FileStream->get(c))
+          {
+          if (c == '\n')
+            {
+            break;
+            }
+          }
+        }
+      else
+        {
+        break;
+        }
       }
-    this->fs->putback(c);
 
-    *(this->fs) >> this->NumberOfNodes;
-    *(this->fs) >> this->NumberOfCells;
-    *(this->fs) >> this->NumberOfNodeFields;
-    *(this->fs) >> this->NumberOfCellFields;
-    *(this->fs) >> this->NumberOfFields;
+    *(this->FileStream) >> this->NumberOfNodes;
+    *(this->FileStream) >> this->NumberOfCells;
+    *(this->FileStream) >> this->NumberOfNodeFields;
+    *(this->FileStream) >> this->NumberOfCellFields;
+    *(this->FileStream) >> this->NumberOfFields;
     }
   else
     {
@@ -221,17 +257,17 @@ void vtkAVSucdReader::ExecuteInformation()
 
     // Here we first need to check if the file is little-endian or big-endian
     // We will read the variable once, with the given endian-ness set up in
-    // the class constructor. If TrueFileLength does not match
-    // CalculatedFileLength, then we will toggle the endian-ness and re-swap
+    // the class constructor. If trueFileLength does not match
+    // calculatedFileLength, then we will toggle the endian-ness and re-swap
     // the variables
-    this->fs->seekg(0L, ios::end);
-    TrueFileLength = this->fs->tellg();
-    CalculatedFileLength = 0; // unknown yet
+    this->FileStream->seekg(0L, ios::end);
+    trueFileLength = this->FileStream->tellg();
+    calculatedFileLength = 0; // unknown yet
 
-    while(abs(static_cast<int>(TrueFileLength - CalculatedFileLength))/
-          TrueFileLength > 0.01)
+    while(abs(static_cast<int>(trueFileLength - calculatedFileLength))/
+          trueFileLength > 0.01)
       {
-      if(TrueFileLength != CalculatedFileLength)
+      if(trueFileLength != calculatedFileLength)
         {
         // switch to opposite of what previously set in constructor
         if(this->ByteOrder == FILE_LITTLE_ENDIAN)
@@ -244,82 +280,99 @@ void vtkAVSucdReader::ExecuteInformation()
           }
         }
       // restart at beginning of file
-      this->fs->seekg(0L, ios::beg);
+      this->FileStream->seekg(0L, ios::beg);
 
-      this->fs->read(&magic_number, 1);
+      this->FileStream->read(&magic_number, 1);
 
       this->ReadIntBlock(1, &this->NumberOfNodes);
       this->ReadIntBlock(1, &this->NumberOfCells);
       this->ReadIntBlock(1, &this->NumberOfNodeFields);
       this->ReadIntBlock(1, &this->NumberOfCellFields);
       this->ReadIntBlock(1, &this->NumberOfFields);
-      this->ReadIntBlock(1, &this->nlist_nodes);
+      this->ReadIntBlock(1, &this->NlistNodes);
 
       vtkDebugMacro( << this->NumberOfNodes << " "
                      << this->NumberOfCells << " "
                      << this->NumberOfNodeFields << " "
                      << this->NumberOfCellFields << " "
                      << this->NumberOfFields << " "
-                     << this->nlist_nodes << endl);
+                     << this->NlistNodes << endl);
 
-      CalculatedFileLength  = 1 + 6*4;
-      CalculatedFileLength += 16 * this->NumberOfCells + 4 * this->nlist_nodes;
-      CalculatedFileLength += 3*4 * this->NumberOfNodes;
+      calculatedFileLength  = 1 + 6*4;
+      calculatedFileLength += 16 * this->NumberOfCells + 4 * this->NlistNodes;
+      calculatedFileLength += 3*4 * this->NumberOfNodes;
       if(this->NumberOfNodeFields)
         {
-        CalculatedFileLength += 2052 +
+        calculatedFileLength += 2052 +
           this->NumberOfNodeFields*(12 + 4 * this->NumberOfNodes + 4);
         }
       
       if(this->NumberOfCellFields)
         {
-        CalculatedFileLength += 2052 + 
+        calculatedFileLength += 2052 + 
           this->NumberOfCellFields*(12 + 4 * this->NumberOfCells + 4);
         }
       
       if(this->NumberOfFields)
         {
-        CalculatedFileLength += 2052 + this->NumberOfFields*(4 * 5);
+        calculatedFileLength += 2052 + this->NumberOfFields*(4 * 5);
         }
       
-      vtkDebugMacro( << "TFL = " << TrueFileLength 
-                     << "\tCFL = " << CalculatedFileLength << endl);
+      vtkDebugMacro( << "TFL = " << trueFileLength 
+                     << "\tCFL = " << calculatedFileLength << endl);
 
-      //TrueFileLength = CalculatedFileLength;
+      //trueFileLength = calculatedFileLength;
       } // end of while loop
 
     const long base_offset = 1 + 6*4;
-    char BUFFER2[1024], BUFFER1[1024], label[32];
+    char buffer1[1024], buffer2[1024], label[32];
 
     long offset = base_offset + 16 * this->NumberOfCells + 
-      4 * this->nlist_nodes + 3 * 4 * this->NumberOfNodes;
+      4 * this->NlistNodes + 3 * 4 * this->NumberOfNodes;
 
     if(this->NumberOfNodeFields)
       {
-      this->fs->seekg(offset,ios::beg);
-      this->fs->read(BUFFER1, sizeof(BUFFER1));
-      this->fs->read(BUFFER2, sizeof(BUFFER2)); // read 2nd array of 1024 bytes
+      this->FileStream->seekg(offset,ios::beg);
+      this->FileStream->read(buffer1, sizeof(buffer1));
+      this->FileStream->read(buffer2, sizeof(buffer2)); // read 2nd array of 1024 bytes
       this->ReadIntBlock(1, &this->NumberOfNodeComponents);
 
       ncomp_list = new int[this->NumberOfNodeFields];
       this->ReadIntBlock(this->NumberOfNodeFields, ncomp_list);
-      //for(i=0; i < this->NumberOfNodeFields; i++)
-      //{
-      //cerr << "ncomp_list[" << i << "] = " << ncomp_list[i] << endl;
-      //}
-      //float *mx = new float[this->NumberOfNodeFields];
-      //this->ReadFloatBlock(this->NumberOfNodeFields, mx);
-      //cerr << "Min: " << mx[0] << ", " << mx[1] << ", " << mx[2] << endl;
-      //this->ReadFloatBlock(this->NumberOfNodeFields, mx);
-      //cerr << "Max: " << mx[0] << ", " << mx[1] << ", " << mx[2] << endl;
+
+      this->NodeDataInfo = new DataInfo[this->NumberOfNodeComponents];
+
+      float *mx = new float[this->NumberOfNodeFields];
+      // read now the minimums for node_data
+      this->ReadFloatBlock(this->NumberOfNodeFields, mx);
+      k=0;
+      for(i=0; i < this->NumberOfNodeComponents; i++)
+        {
+        for(j=0; j < ncomp_list[i]; j++)
+          {
+          this->NodeDataInfo[i].min[j] = mx[k];
+          }
+        k++;
+        }
+      // read now the maximums for node_data
+      this->ReadFloatBlock(this->NumberOfNodeFields, mx);
+      k=0;
+      for(i=0; i < this->NumberOfNodeComponents; i++)
+        {
+        for(j=0; j < ncomp_list[i]; j++)
+          {
+          this->NodeDataInfo[i].max[j] = mx[k];
+          }
+        k++;
+        }
+      delete [] mx;
 
       offset +=  1024 + 1024 + 4 + 3 * 4 * this->NumberOfNodeFields;
   
-      this->NodeDataInfo = new DataInfo[this->NumberOfNodeFields];
       k = 0;
       for(i=0; i < this->NumberOfNodeComponents; i++)
         {
-        this->get_label(BUFFER1, i, label);
+        this->GetLabel(buffer1, i, label);
         vtkDebugMacro( << i+1 << " :found ND label = " << label 
                        << " [" << ncomp_list[i] << "]" <<endl);
         this->PointDataArraySelection->AddArray(label);
@@ -334,22 +387,48 @@ void vtkAVSucdReader::ExecuteInformation()
       {
       offset += 4 * this->NumberOfNodes * this->NumberOfNodeFields + 
         4 * this->NumberOfNodeFields;
-      this->fs->seekg(offset,ios::beg);
-      this->fs->read(BUFFER1, sizeof(BUFFER1));
+      this->FileStream->seekg(offset,ios::beg);
+      this->FileStream->read(buffer1, sizeof(buffer1));
 
-      this->fs->read(BUFFER2, sizeof(BUFFER2)); // read 2nd array of 1024 bytes
+      this->FileStream->read(buffer2, sizeof(buffer2)); // read 2nd array of 1024 bytes
       this->ReadIntBlock(1, &this->NumberOfCellComponents);
 
       ncomp_list = new int[this->NumberOfCellFields];
       this->ReadIntBlock(this->NumberOfCellFields, ncomp_list);
 
+      this->CellDataInfo = new DataInfo[this->NumberOfCellComponents];
+
+      float *mx = new float[this->NumberOfCellFields];
+      // read now the minimums for cell_data
+      this->ReadFloatBlock(this->NumberOfCellFields, mx);
+      k=0;
+      for(i=0; i < this->NumberOfCellFields; i++)
+        {
+        for(j=0; j < ncomp_list[i]; j++)
+          {
+          this->CellDataInfo[i].min[j] = mx[k];
+          };
+        k++;
+        }
+      // read now the maximums for cell_data
+      this->ReadFloatBlock(this->NumberOfCellFields, mx);
+      k=0;
+      for(i=0; i < this->NumberOfCellFields; i++)
+        {
+        for(j=0; j < ncomp_list[i]; j++)
+          {
+          this->CellDataInfo[i].max[j] = mx[k];
+          }
+        k++;
+        }
+      delete [] mx;
+
       offset += 1024 + 1024 + 4 + 3 * 4 * this->NumberOfCellFields;
   
-      this->CellDataInfo = new DataInfo[this->NumberOfCellFields];
       k = 0;
       for(i=0; i < this->NumberOfCellComponents; i++)
         {
-        this->get_label(BUFFER1, i, label);
+        this->GetLabel(buffer1, i, label);
         vtkDebugMacro( << i+1 << " :found CD label = " << label << " [" 
                        << ncomp_list[i] << "]" << endl);
         this->CellDataArraySelection->AddArray(label);
@@ -364,15 +443,15 @@ void vtkAVSucdReader::ExecuteInformation()
       {
       offset += 4 * this->NumberOfCells * this->NumberOfCellFields + 
         4 * this->NumberOfCellFields;
-      this->fs->seekg(offset,ios::beg);
-      this->fs->read(BUFFER1, sizeof(BUFFER1));
-      vtkDebugMacro(<< BUFFER1 << endl);
+      this->FileStream->seekg(offset,ios::beg);
+      this->FileStream->read(buffer1, sizeof(buffer1));
+      vtkDebugMacro(<< buffer1 << endl);
 
-      offset += 1024 + 1024 + 4 + 3 * 4 * this->NumberOfFields;
+      //offset += 1024 + 1024 + 4 + 3 * 4 * this->NumberOfFields;
 
       for(i=0; i < this->NumberOfFields; i++)
         {
-        this->get_label(BUFFER1, i, label);
+        this->GetLabel(buffer1, i, label);
         vtkDebugMacro( << "found MD label = " << label << endl);
         }
       }
@@ -396,15 +475,37 @@ void vtkAVSucdReader::ExecuteInformation()
     }
 
   vtkDebugMacro( << "end of ExecuteInformation\n");
+
+  return 1;
 }
 
-void vtkAVSucdReader::ReadGeometry()
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::GetCellDataRange(int cellComp, int index, float *min, float *max)
 {
-  vtkUnstructuredGrid *output = (vtkUnstructuredGrid *)this->GetOutput();
+  if (index >= this->CellDataInfo[cellComp].veclen || index < 0)
+    {
+    index = 0;  // if wrong index, set it to zero
+    }
+  *min = this->CellDataInfo[cellComp].min[index];
+  *max = this->CellDataInfo[cellComp].max[index];
+}
 
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::GetNodeDataRange(int nodeComp, int index, float *min, float *max)
+{
+  if (index >= this->NodeDataInfo[nodeComp].veclen || index < 0)
+    {
+    index = 0;  // if wrong index, set it to zero
+    }
+  *min = this->NodeDataInfo[nodeComp].min[index];
+  *max = this->NodeDataInfo[nodeComp].max[index];
+}
+
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::ReadGeometry(vtkUnstructuredGrid *output)
+{
   // add a material array
   vtkIntArray *materials = vtkIntArray::New();
-  materials->SetNumberOfComponents(1);
   materials->SetNumberOfTuples(this->NumberOfCells);
   materials->SetName("Material Id");
 
@@ -424,7 +525,7 @@ void vtkAVSucdReader::ReadGeometry()
     // this array contains a list of NumberOfCells tuples
     // each tuple is 1 integer, i.e. the number of indices following it (N)
     // followed by these N integers 
-    listcells->SetNumberOfValues(this->NumberOfCells + this->nlist_nodes);
+    listcells->SetNumberOfValues(this->NumberOfCells + this->NlistNodes);
 
     this->ReadBinaryCellTopology(materials, types, listcells);
     this->ReadXYZCoords(coords);
@@ -460,6 +561,7 @@ void vtkAVSucdReader::ReadGeometry()
 }
 
 
+//----------------------------------------------------------------------------
 void vtkAVSucdReader::ReadBinaryCellTopology(vtkIntArray *materials, 
                                              int *types, 
                                              vtkIdTypeArray *listcells)
@@ -467,32 +569,42 @@ void vtkAVSucdReader::ReadBinaryCellTopology(vtkIntArray *materials,
   int i, j, k2=0;
   int *mat = materials->GetPointer(0);
   vtkIdType *list = listcells->GetPointer(0);
-  int *Ctype = new int[4 * this->NumberOfCells];
-  if(Ctype == NULL)
+  int *ctype = new int[4 * this->NumberOfCells];
+  if(ctype == NULL)
     {
-    vtkErrorMacro(<< "Error allocating Ctype memory");
+    vtkErrorMacro(<< "Error allocating ctype memory");
     }
 
-  this->fs->seekg(6*4 + 1,ios::beg);
-  this->ReadIntBlock(4 * this->NumberOfCells, Ctype);
+  this->FileStream->seekg(6*4 + 1,ios::beg);
+  this->ReadIntBlock(4 * this->NumberOfCells, ctype);
   
-  int *topology_list = new int[this->nlist_nodes];
+  int *topology_list = new int[this->NlistNodes];
   if(topology_list == NULL)
     {
     vtkErrorMacro(<< "Error allocating topology_list memory");
     }
 
-  this->ReadIntBlock(this->nlist_nodes, topology_list);
+  this->ReadIntBlock(this->NlistNodes, topology_list);
   this->UpdateProgress(0.25);
 
   for(i=0; i < this->NumberOfCells; i++)
     {
-    //listcells->SetValue(k1++, Ctype[4*i+2]);
-    *list++ = Ctype[4*i+2];
-    for(j=0; j < Ctype[4*i+2]; j++)
+    *list++ = ctype[4*i+2];
+    if(ctype[4*i+3] == vtkAVSucdReader::PYR)
+      { //UCD ordering is 0,1,2,3,4 => VTK ordering is 1,2,3,4,0
+      *list++ = topology_list[++k2] - 1;
+      *list++ = topology_list[++k2] - 1;
+      *list++ = topology_list[++k2] - 1;
+      *list++ = topology_list[++k2] - 1;
+      *list++ = topology_list[k2-4] - 1;
+      k2++;
+      }
+     else
       {
-      //listcells->SetValue(k1++, topology_list[k2++] - 1);
-      *list++ = topology_list[k2++] - 1;
+       for(j=0; j < ctype[4*i+2]; j++)
+         {
+         *list++ = topology_list[k2++] - 1;
+         }
       }
     }
 
@@ -500,8 +612,8 @@ void vtkAVSucdReader::ReadBinaryCellTopology(vtkIntArray *materials,
 
   for(i=0; i < this->NumberOfCells; i++)
     {
-    *mat++ = Ctype[4*i+1];
-    switch(Ctype[4*i+3])
+    *mat++ = ctype[4*i+1];
+    switch(ctype[4*i+3])
       {
       case vtkAVSucdReader::PT:    types[i] = VTK_VERTEX;     break;
       case vtkAVSucdReader::LINE:  types[i] = VTK_LINE;       break;
@@ -512,15 +624,16 @@ void vtkAVSucdReader::ReadBinaryCellTopology(vtkIntArray *materials,
       case vtkAVSucdReader::PRISM: types[i] = VTK_WEDGE;      break;
       case vtkAVSucdReader::HEX:   types[i] = VTK_HEXAHEDRON; break;
       default:
-        vtkErrorMacro( << "cell type: " << Ctype[4*i+3] << "not supported\n");
-        delete [] Ctype;
+        vtkErrorMacro( << "cell type: " << ctype[4*i+3] << "not supported\n");
+        delete [] ctype;
         return;
       }
     }
-  delete [] Ctype;
+  delete [] ctype;
 }
 
 
+//----------------------------------------------------------------------------
 void vtkAVSucdReader::ReadASCIICellTopology(vtkIntArray *materials, 
                                             vtkUnstructuredGrid *output)
 {
@@ -533,15 +646,15 @@ void vtkAVSucdReader::ReadASCIICellTopology(vtkIntArray *materials,
   for(i=0; i < this->NumberOfCells; i++)
     {
     int id;  // no check is done to see that they are monotonously increasing
-    *(this->fs) >> id;
-    *(this->fs) >> mat[i];
-    *(this->fs) >> ctype;
-    //cerr << mat[i] << ", " << ctype << endl;
+    *(this->FileStream) >> id;
+    *(this->FileStream) >> mat[i];
+    *(this->FileStream) >> ctype;
+    vtkDebugMacro( << mat[i] << ", " << ctype );
     if(!strcmp(ctype, "pt"))
       {
       for(k=0; k < 1; k++)
         {
-        *(this->fs) >> list[k];
+        *(this->FileStream) >> list[k];
         if(this->DecrementNodeIds)
           {
           list[k]--;
@@ -553,7 +666,7 @@ void vtkAVSucdReader::ReadASCIICellTopology(vtkIntArray *materials,
       {
       for(k=0; k < 2; k++)
         {
-        *(this->fs) >> list[k];
+        *(this->FileStream) >> list[k];
         if(this->DecrementNodeIds)
           {
           list[k]--;
@@ -565,7 +678,7 @@ void vtkAVSucdReader::ReadASCIICellTopology(vtkIntArray *materials,
       {
       for(k=0; k < 3; k++)
         {
-        *(this->fs) >> list[k];
+        *(this->FileStream) >> list[k];
         if(this->DecrementNodeIds)
           {
           list[k]--;
@@ -577,7 +690,7 @@ void vtkAVSucdReader::ReadASCIICellTopology(vtkIntArray *materials,
       {
       for(k=0; k < 4; k++)
         {
-        *(this->fs) >> list[k];
+        *(this->FileStream) >> list[k];
         if(this->DecrementNodeIds)
           {
           list[k]--;
@@ -589,7 +702,7 @@ void vtkAVSucdReader::ReadASCIICellTopology(vtkIntArray *materials,
       {
       for(k=0; k < 4; k++)
         {
-        *(this->fs) >> list[k];
+        *(this->FileStream) >> list[k];
         if(this->DecrementNodeIds)
           {
           list[k]--;
@@ -601,19 +714,23 @@ void vtkAVSucdReader::ReadASCIICellTopology(vtkIntArray *materials,
       {
       for(k=0; k < 5; k++)
         {
-        *(this->fs) >> list[k];
+        *(this->FileStream) >> list[k];
         if(this->DecrementNodeIds)
           {
           list[k]--;
           }
         }
+      int tmp;
+      tmp = list[0];
+      list[0] = list[1]; list[1] = list[2]; list[2] = list[3];
+      list[3] = list[4]; list[4] = tmp;
       output->InsertNextCell(VTK_PYRAMID, 5, list);
       }
     else if(!strcmp(ctype, "prism"))
       {
       for(k=0; k < 6; k++)
         {
-        *(this->fs) >> list[k];
+        *(this->FileStream) >> list[k];
         if(this->DecrementNodeIds)
           {
           list[k]--;
@@ -625,7 +742,7 @@ void vtkAVSucdReader::ReadASCIICellTopology(vtkIntArray *materials,
       {
       for(k=0; k < 8; k++)
         {
-        *(this->fs) >> list[k];
+        *(this->FileStream) >> list[k];
         if(this->DecrementNodeIds)
           {
           list[k]--;
@@ -642,6 +759,7 @@ void vtkAVSucdReader::ReadASCIICellTopology(vtkIntArray *materials,
 }
 
 
+//----------------------------------------------------------------------------
 void vtkAVSucdReader::ReadXYZCoords(vtkFloatArray *coords)
 {
   int i;
@@ -676,11 +794,11 @@ void vtkAVSucdReader::ReadXYZCoords(vtkFloatArray *coords)
   else
     {
     int id;  // no check is done to see that they are monotonously increasing
-    // read here the first node anc check if its id number is 0
+    // read here the first node and check if its id number is 0
 
-    *(this->fs) >> id;
+    *(this->FileStream) >> id;
     i=0;
-    *(this->fs) >> ptr[3*i] >> ptr[3*i+1] >> ptr[3*i+2];
+    *(this->FileStream) >> ptr[3*i] >> ptr[3*i+1] >> ptr[3*i+2];
     if(id)
       {
       this->DecrementNodeIds = 1;
@@ -688,14 +806,15 @@ void vtkAVSucdReader::ReadXYZCoords(vtkFloatArray *coords)
 
     for(i=1; i < this->NumberOfNodes; i++)
       {
-      *(this->fs) >> id;
-      *(this->fs) >> ptr[3*i] >> ptr[3*i+1] >> ptr[3*i+2];
+      *(this->FileStream) >> id;
+      *(this->FileStream) >> ptr[3*i] >> ptr[3*i+1] >> ptr[3*i+2];
       }
     } // end of ASCII read
 }
 
 
-void vtkAVSucdReader::ReadNodeData()
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::ReadNodeData(vtkUnstructuredGrid *output)
 {
   int i, j, n;
   float *ptr;
@@ -710,7 +829,7 @@ void vtkAVSucdReader::ReadNodeData()
         scalars->SetNumberOfComponents(this->NodeDataInfo[i].veclen);
         scalars->SetNumberOfTuples(this->NumberOfNodes);
         scalars->SetName(PointDataArraySelection->GetArrayName(i));
-        this->fs->seekg(this->NodeDataInfo[i].foffset, ios::beg);
+        this->FileStream->seekg(this->NodeDataInfo[i].foffset, ios::beg);
         if(1) // this->NodeDataInfo[i].veclen == 1)
           {
           ptr = scalars->GetPointer(0);
@@ -731,10 +850,10 @@ void vtkAVSucdReader::ReadNodeData()
           delete [] ptr;
           }
 
-        this->GetOutput()->GetPointData()->AddArray(scalars);
-        if (!this->GetOutput()->GetPointData()->GetScalars())
+        output->GetPointData()->AddArray(scalars);
+        if (!output->GetPointData()->GetScalars())
           {
-          this->GetOutput()->GetPointData()->SetScalars(scalars);
+          output->GetPointData()->SetScalars(scalars);
           }
         scalars->Delete();
         }
@@ -752,26 +871,26 @@ void vtkAVSucdReader::ReadNodeData()
     int id;
     char buf1[128], c='\0', buf2[128];
 
-    this->NodeDataInfo = new DataInfo[this->NumberOfNodeFields];
-    *(this->fs) >> this->NumberOfNodeComponents;
+    *(this->FileStream) >> this->NumberOfNodeComponents;
+    this->NodeDataInfo = new DataInfo[this->NumberOfNodeComponents];
     for(i=0; i < this->NumberOfNodeComponents; i++)
       {
-      *(this->fs) >> this->NodeDataInfo[i].veclen;
+      *(this->FileStream) >> this->NodeDataInfo[i].veclen;
       }
-    this->fs->get(c); // one more newline to catch
+    this->FileStream->get(c); // one more newline to catch
 
     vtkFloatArray **scalars = new 
       vtkFloatArray * [this->NumberOfNodeComponents];
     for(i=0; i < this->NumberOfNodeComponents; i++)
       {
       j=0;
-      while(this->fs->get(c) && c != ',')
+      while(this->FileStream->get(c) && c != ',')
         {
         buf1[j++] = c;
         }
       buf1[j] = '\0';
       // finish here to read the line
-      this->fs->get(buf2, 128, '\n'); this->fs->get(c);
+      this->FileStream->get(buf2, 128, '\n'); this->FileStream->get(c);
 
       scalars[i] = vtkFloatArray::New();
       scalars[i]->SetNumberOfComponents(this->NodeDataInfo[i].veclen);
@@ -780,31 +899,33 @@ void vtkAVSucdReader::ReadNodeData()
       }
     for(n=0; n < this->NumberOfNodes; n++)
       {
-      *(this->fs) >> id;
+      *(this->FileStream) >> id;
       for(i=0; i < this->NumberOfNodeComponents; i++)
         {
         for(j=0; j < this->NodeDataInfo[i].veclen; j++)
           {
-          *(this->fs) >> value;
+          *(this->FileStream) >> value;
           scalars[i]->SetComponent(n, j, value);
           }
         }
       }
     for(i=0; i < this->NumberOfNodeComponents; i++)
       {
-      this->GetOutput()->GetPointData()->AddArray(scalars[i]);
-      if (!this->GetOutput()->GetPointData()->GetScalars())
+      output->GetPointData()->AddArray(scalars[i]);
+      if (!output->GetPointData()->GetScalars())
         {
-        this->GetOutput()->GetPointData()->SetScalars(scalars[i]);
+        output->GetPointData()->SetScalars(scalars[i]);
         }
       scalars[i]->Delete();
       }
+      delete[] scalars;
     } // end of ASCII read
   vtkDebugMacro( << "End of ReadNodeData()\n");
 }
 
 
-void vtkAVSucdReader::ReadCellData()
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::ReadCellData(vtkUnstructuredGrid *output)
 {
   int i, j, n;
   float *ptr;
@@ -819,7 +940,7 @@ void vtkAVSucdReader::ReadCellData()
         scalars->SetNumberOfComponents(this->CellDataInfo[i].veclen);
         scalars->SetNumberOfTuples(this->NumberOfCells);
         scalars->SetName(CellDataArraySelection->GetArrayName(i));
-        this->fs->seekg(this->CellDataInfo[i].foffset, ios::beg);
+        this->FileStream->seekg(this->CellDataInfo[i].foffset, ios::beg);
         if(1) // this->CellDataInfo[i].veclen == 1)
           {
           ptr = scalars->GetPointer(0);
@@ -831,7 +952,7 @@ void vtkAVSucdReader::ReadCellData()
           ptr = new float[this->NumberOfCells];
           for(j=0; j < this->CellDataInfo[i].veclen; j++)
             {
-            this->fs->seekg(this->CellDataInfo[i].foffset + 
+            this->FileStream->seekg(this->CellDataInfo[i].foffset + 
                             j*this->NumberOfCells,
                             ios::beg);
             this->ReadFloatBlock(this->NumberOfCells, ptr);
@@ -844,10 +965,10 @@ void vtkAVSucdReader::ReadCellData()
           delete [] ptr;
           }
 
-        this->GetOutput()->GetCellData()->AddArray(scalars);
-        if (!this->GetOutput()->GetCellData()->GetScalars())
+        output->GetCellData()->AddArray(scalars);
+        if (!output->GetCellData()->GetScalars())
           {
-          this->GetOutput()->GetCellData()->SetScalars(scalars);
+          output->GetCellData()->SetScalars(scalars);
           }
         scalars->Delete();
         }
@@ -859,26 +980,27 @@ void vtkAVSucdReader::ReadCellData()
     int id;
     char buf1[128], c='\0', buf2[128];
 
-    this->CellDataInfo = new DataInfo[this->NumberOfCellFields];
-    *(this->fs) >> this->NumberOfCellComponents;
+    *(this->FileStream) >> this->NumberOfCellComponents;
+    this->CellDataInfo = new DataInfo[this->NumberOfCellComponents];
+
     for(i=0; i < this->NumberOfCellComponents; i++)
       {
-      *(this->fs) >> this->CellDataInfo[i].veclen;
+      *(this->FileStream) >> this->CellDataInfo[i].veclen;
       }
-    this->fs->get(c); // one more newline to catch
+    this->FileStream->get(c); // one more newline to catch
 
     vtkFloatArray **scalars = new 
       vtkFloatArray * [this->NumberOfCellComponents];
     for(i=0; i < this->NumberOfCellComponents; i++)
       {
       j=0;
-      while(this->fs->get(c) && c != ',')
+      while(this->FileStream->get(c) && c != ',')
         {
         buf1[j++] = c;
         }
       buf1[j] = '\0';
       // finish here to read the line
-      this->fs->get(buf2, 128, '\n'); this->fs->get(c);
+      this->FileStream->get(buf2, 128, '\n'); this->FileStream->get(c);
 
       scalars[i] = vtkFloatArray::New();
       scalars[i]->SetNumberOfComponents(this->CellDataInfo[i].veclen);
@@ -887,39 +1009,43 @@ void vtkAVSucdReader::ReadCellData()
       }
     for(n=0; n < this->NumberOfCells; n++)
       {
-      *(this->fs) >> id;
+      *(this->FileStream) >> id;
       for(i=0; i < this->NumberOfCellComponents; i++)
         {
         for(j=0; j < this->CellDataInfo[i].veclen; j++)
           {
-          *(this->fs) >> value;
+          *(this->FileStream) >> value;
           scalars[i]->SetComponent(n, j, value);
           }
         }
       }
     for(i=0; i < this->NumberOfCellComponents; i++)
       {
-      this->GetOutput()->GetCellData()->AddArray(scalars[i]);
-      if (!this->GetOutput()->GetCellData()->GetScalars())
+      output->GetCellData()->AddArray(scalars[i]);
+      if (!output->GetCellData()->GetScalars())
         {
-        this->GetOutput()->GetCellData()->SetScalars(scalars[i]);
+        output->GetCellData()->SetScalars(scalars[i]);
         }
       scalars[i]->Delete();
       }
+      delete[] scalars;
     } // end of ASCII read
   vtkDebugMacro( << "End of ReadCellData()\n");
 }
 
+//----------------------------------------------------------------------------
 const char* vtkAVSucdReader::GetPointArrayName(int index)
 {
   return this->PointDataArraySelection->GetArrayName(index);
 }
 
+//----------------------------------------------------------------------------
 int vtkAVSucdReader::GetPointArrayStatus(const char* name)
 {
   return this->PointDataArraySelection->ArrayIsEnabled(name);
 }
 
+//----------------------------------------------------------------------------
 void vtkAVSucdReader::SetPointArrayStatus(const char* name, int status)
 {
   if(status)
@@ -932,17 +1058,20 @@ void vtkAVSucdReader::SetPointArrayStatus(const char* name, int status)
     }
 }
 
+//----------------------------------------------------------------------------
 const char* vtkAVSucdReader::GetCellArrayName(int index)
 {
   return this->CellDataArraySelection->GetArrayName(index);
 }
 
+//----------------------------------------------------------------------------
 int vtkAVSucdReader::GetCellArrayStatus(const char* name)
 {
   return this->CellDataArraySelection->ArrayIsEnabled(name);
 }
 
 
+//----------------------------------------------------------------------------
 void vtkAVSucdReader::SetCellArrayStatus(const char* name, int status)
 {
   if(status)
@@ -955,17 +1084,44 @@ void vtkAVSucdReader::SetCellArrayStatus(const char* name, int status)
     }
 }
 
+//----------------------------------------------------------------------------
 int vtkAVSucdReader::GetNumberOfCellArrays()
 {
   return this->CellDataArraySelection->GetNumberOfArrays();
 }
 
+//----------------------------------------------------------------------------
 int vtkAVSucdReader::GetNumberOfPointArrays()
 {
   return this->PointDataArraySelection->GetNumberOfArrays();
 }
 
-int vtkAVSucdReader::get_label(char *string, int number, char *label)
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::EnableAllPointArrays()
+{
+    this->PointDataArraySelection->EnableAllArrays();
+}
+
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::DisableAllPointArrays()
+{
+    this->PointDataArraySelection->DisableAllArrays();
+}
+
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::EnableAllCellArrays()
+{
+    this->CellDataArraySelection->EnableAllArrays();
+}
+
+//----------------------------------------------------------------------------
+void vtkAVSucdReader::DisableAllCellArrays()
+{
+    this->CellDataArraySelection->DisableAllArrays();
+}
+
+//----------------------------------------------------------------------------
+int vtkAVSucdReader::GetLabel(char *string, int number, char *label)
 {
   int   i, j, k, len;
   char  current;
@@ -974,12 +1130,13 @@ int vtkAVSucdReader::get_label(char *string, int number, char *label)
   // check to make sure that structure is not NULL
   if (string == NULL)
     {
-    return (0);
+    vtkErrorMacro( << "String is null");
+    return 0;
     }
 
   // search for the appropriate label
   k = 0;
-  len = strlen (string);
+  len = static_cast<int>(strlen(string));
   for(i = 0; i <= number; i++)
     {
     current = string[k++];
@@ -996,7 +1153,7 @@ int vtkAVSucdReader::get_label(char *string, int number, char *label)
         // the nth label was not found, where n = number
         if (i < number)
           {
-          return (0);
+          return 0;
           }
         current = '.';
         }
@@ -1004,17 +1161,18 @@ int vtkAVSucdReader::get_label(char *string, int number, char *label)
     label[j] = '\0';
     }
   // a valid label was found
-  return (1);
+  return 1;
 }
 
 
+//----------------------------------------------------------------------------
 // Read a block of ints (ascii or binary) and return number read.
 int vtkAVSucdReader::ReadIntBlock(int n, int *block)
 {
   if (this->BinaryFile)
     {
-    this->fs->read((char *)block, n * sizeof(int));
-    int retVal = this->fs->gcount() / sizeof(int);
+    this->FileStream->read((char *)block, n * sizeof(int));
+    int retVal = static_cast<int>(this->FileStream->gcount()) / sizeof(int);
 
     if (this->ByteOrder == FILE_LITTLE_ENDIAN)
       {
@@ -1031,7 +1189,7 @@ int vtkAVSucdReader::ReadIntBlock(int n, int *block)
     int count = 0;
     for(int i=0; i<n; i++)
       {
-      if (*(this->fs) >> block[i])
+      if (*(this->FileStream) >> block[i])
         {
         count++;
         }
@@ -1044,12 +1202,13 @@ int vtkAVSucdReader::ReadIntBlock(int n, int *block)
     }
 }
 
+//----------------------------------------------------------------------------
 int vtkAVSucdReader::ReadFloatBlock(int n, float* block)
 {
   if (this->BinaryFile)
     {
-    this->fs->read((char *)block, n * sizeof(float));
-    int retVal = this->fs->gcount() / sizeof(int);
+    this->FileStream->read((char *)block, n * sizeof(float));
+    int retVal = static_cast<int>(this->FileStream->gcount()) / sizeof(int);
     if (this->ByteOrder == FILE_LITTLE_ENDIAN)
       {
       vtkByteSwap::Swap4LERange(block, n);
@@ -1065,7 +1224,7 @@ int vtkAVSucdReader::ReadFloatBlock(int n, float* block)
     int count = 0;
     for(int i=0; i<n; i++)
       {
-      if (*(this->fs) >> block[i])
+      if (*(this->FileStream) >> block[i])
         {
         count++;
         }
