@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkBMPReader.cxx,v $
+  Module:    vtkBMPReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -20,23 +20,14 @@
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 
-vtkCxxRevisionMacro(vtkBMPReader, "$Revision: 1.46 $");
 vtkStandardNewMacro(vtkBMPReader);
 
 #ifdef read
 #undef read
 #endif
 
-//-----  This hack needed to compile using gcc3 on OSX until new stdc++.dylib
-#ifdef __APPLE_CC__
-extern "C"
-{
-  void oft_initIO() 
-  {
-  extern void _ZNSt8ios_base4InitC4Ev();
-  _ZNSt8ios_base4InitC4Ev();
-  }
-}
+#ifdef close
+#undef close
 #endif
 
 vtkBMPReader::vtkBMPReader()
@@ -65,19 +56,14 @@ vtkBMPReader::~vtkBMPReader()
     }
 }
 
-vtkLookupTable *vtkBMPReader::GetLookupTable(void) {
-    return this->LookupTable;
-}
-
+//----------------------------------------------------------------------------
 void vtkBMPReader::ExecuteInformation()
 {
   int xsize, ysize;
   FILE *fp;
-  long tmp;
-  short stmp;
-  long infoSize;
-  int  iinfoSize;  // in case we are on a 64bit machine
-  int  itmp;       // in case we are on a 64bit machine
+  vtkTypeInt32 tmp;
+  vtkTypeInt32 infoSize;
+  vtkTypeInt16 stmp1, stmp2;
 
   // free any old memory
   if (this->Colors)
@@ -85,7 +71,7 @@ void vtkBMPReader::ExecuteInformation()
     delete [] this->Colors;
     this->Colors = NULL;
     }
-      
+
   // if the user has not set the extent, but has set the VOI
   // set the zaxis extent to the VOI z axis
   if (this->DataExtent[4]==0 && this->DataExtent[5] == 0 &&
@@ -111,101 +97,97 @@ void vtkBMPReader::ExecuteInformation()
   // compare magic number to determine file type
   if ((fgetc(fp) != 'B')||(fgetc(fp) != 'M'))
     {
-    vtkErrorMacro(<<"Unknown file type! " << this->InternalFileName 
+    vtkErrorMacro(<<"Unknown file type! " << this->InternalFileName
                   <<" is not a Windows BMP file!");
     fclose(fp);
     return;
     }
 
+  // error indicator
+  bool errorOccurred = false;
+
   // get the size of the file
-  int sizeLong = sizeof(long);
-  if (sizeLong == 4)
+
+  // skip 4 bytes
+  if (fread(&tmp,4,1,fp) != 1)
     {
-    fread(&tmp,4,1,fp);
-    // skip 4 bytes
-    fread(&tmp,4,1,fp);
-    // read the offset
-    fread(&tmp,4,1,fp);
+    errorOccurred = true;
     }
-  else
+  // skip 4 more bytes
+  else if (fread(&tmp,4,1,fp) != 1)
     {
-    fread(&itmp,4,1,fp);
-    // skip 4 bytes
-    fread(&itmp,4,1,fp);
-    // read the offset
-    fread(&itmp,4,1,fp);
+    errorOccurred = true;
+    }
+  // read the offset
+  else if (fread(&tmp,4,1,fp) != 1)
+    {
+    errorOccurred = true;
     }
 
   // get size of header
-  if (sizeLong == 4)   // if we are on a 32 bit machine
+  else if (fread(&infoSize,4,1,fp) != 1)
     {
-    fread(&infoSize,sizeof(long),1,fp);
-    vtkByteSwap::Swap4LE(&infoSize);
-                       
-    // error checking
-    if ((infoSize != 40)&&(infoSize != 12))
-      {
-      vtkErrorMacro(<<"Unknown file type! " << this->InternalFileName 
-                    <<" is not a Windows BMP file!");
-      fclose(fp);
-      return;
-      }
-  
-    // there are two different types of BMP files
-    if (infoSize == 40)
-      {
-      // now get the dimensions
-      fread(&xsize,sizeof(long),1,fp);
-      vtkByteSwap::Swap4LE(&xsize);
-      fread(&ysize,sizeof(long),1,fp);
-      vtkByteSwap::Swap4LE(&ysize);
-      }
-    else
-      {
-      fread(&stmp,sizeof(short),1,fp);
-      vtkByteSwap::Swap2LE(&stmp);
-      xsize = stmp;
-      fread(&stmp,sizeof(short),1,fp);
-      vtkByteSwap::Swap2LE(&stmp);
-      ysize = stmp;
-      }
+    errorOccurred = true;
     }
-  else    // else we are on a 64bit machine
+
+  if (errorOccurred)
     {
-    fread(&iinfoSize,sizeof(int),1,fp);
-    vtkByteSwap::Swap4LE(&iinfoSize);
-    infoSize = iinfoSize;
-    
-    // error checking
-    if ((infoSize != 40)&&(infoSize != 12))
-      {
-      vtkErrorMacro(<<"Unknown file type! " << this->InternalFileName 
-                    <<" is not a Windows BMP file!");
-      fclose(fp);
-      return;
-      }
-  
-    // there are two different types of BMP files
-    if (infoSize == 40)
-      {
-      // now get the dimensions
-      fread(&xsize,sizeof(int),1,fp);
-      vtkByteSwap::Swap4LE(&xsize);
-      fread(&ysize,sizeof(int),1,fp);
-      vtkByteSwap::Swap4LE(&ysize);
-      }
-    else
-      {
-      fread(&stmp,sizeof(short),1,fp);
-      vtkByteSwap::Swap2LE(&stmp);
-      xsize = stmp;
-      fread(&stmp,sizeof(short),1,fp);
-      vtkByteSwap::Swap2LE(&stmp);
-      ysize = stmp;
-      }
+    vtkErrorMacro("Error reading file: " << this->InternalFileName
+                  <<" Premature end of file.");
+    fclose(fp);
+    return;
     }
-  
-  
+
+  vtkByteSwap::Swap4LE(&infoSize);
+
+  // error checking
+  if ((infoSize != 40)&&(infoSize != 12))
+    {
+    vtkErrorMacro("Unknown file type! " << this->InternalFileName
+                  <<" is not a Windows BMP file!");
+    fclose(fp);
+    return;
+    }
+
+  // there are two different types of BMP files
+  if (infoSize == 40)
+    {
+    // now get the dimensions
+    if (fread(&xsize,4,1,fp) != 1)
+      {
+      errorOccurred = true;
+      }
+    else if (fread(&ysize,4,1,fp) != 1)
+      {
+      errorOccurred = true;
+      }
+    vtkByteSwap::Swap4LE(&xsize);
+    vtkByteSwap::Swap4LE(&ysize);
+    }
+  else
+    {
+    if (fread(&stmp1,2,1,fp) != 1)
+      {
+      errorOccurred = true;
+      }
+    else if (fread(&stmp2,2,1,fp) != 1)
+      {
+      errorOccurred = true;
+      }
+    vtkByteSwap::Swap2LE(&stmp1);
+    vtkByteSwap::Swap2LE(&stmp2);
+    xsize = stmp1;
+    ysize = stmp2;
+    }
+
+  if (errorOccurred)
+    {
+    vtkErrorMacro ("BMPReader error reading file: " << this->InternalFileName
+                   << " Premature EOF while reading size.");
+    fclose (fp);
+    return;
+    }
+
   // is corner in upper left or lower left
   if (ysize < 0)
     {
@@ -216,30 +198,50 @@ void vtkBMPReader::ExecuteInformation()
     {
     this->FileLowerLeft = 1;
     }
-    
+
   // ignore planes
-  fread(&stmp,sizeof(short),1,fp);
+  if (fread(&stmp1,2,1,fp) != 1)
+    {
+    errorOccurred = true;
+    }
   // read depth
-  fread(&this->Depth,sizeof(short),1,fp);
-  vtkByteSwap::Swap2LE(&this->Depth);
+  else if (fread(&stmp2,2,1,fp) != 1)
+    {
+    errorOccurred = true;
+    }
+
+  if (errorOccurred)
+    {
+    vtkErrorMacro ("BMPReader error reading file: " << this->InternalFileName
+                   << " Premature EOF while reading depth.");
+    fclose (fp);
+    return;
+    }
+
+  vtkByteSwap::Swap2LE(&stmp2);
+  this->Depth = stmp2;
+
   if ((this->Depth != 8)&&(this->Depth != 24))
     {
-    vtkErrorMacro(<<"Only BMP depths of (8,24) are supported. Not " << this->Depth);
+    vtkErrorMacro("Only BMP depths of (8,24) are supported. Not "
+                  << this->Depth);
     fclose(fp);
     return;
     }
-  
+
   // skip over rest of info for long format
   if (infoSize == 40)
     {
-    fread(&tmp,4,1,fp);
-    fread(&tmp,4,1,fp);
-    fread(&tmp,4,1,fp);
-    fread(&tmp,4,1,fp);
-    fread(&tmp,4,1,fp);
-    fread(&tmp,4,1,fp);
+    int skip[6];
+    if (fread(&skip,4,6,fp) != 6)
+      {
+      vtkErrorMacro ("BMPReader error reading file: " << this->InternalFileName
+                     << " Premature EOF skipping rest of info for long format.");
+      fclose (fp);
+      return;
+      }
     }
-  
+
   // read in color table if required
   if (this->Depth < 24)
     {
@@ -320,7 +322,7 @@ void vtkBMPReader::ExecuteInformation()
 void vtkBMPReader::ComputeDataIncrements()
 {
   int idx;
-  int fileDataLength;
+  vtkIdType fileDataLength;
   
   // Determine the expected length of the data ...
   switch (this->DataScalarType)
@@ -370,10 +372,10 @@ void vtkBMPReader::ComputeDataIncrements()
 template <class OT>
 void vtkBMPReaderUpdate2(vtkBMPReader *self, vtkImageData *data, OT *outPtr)
 {
-  int inIncr[3], outIncr[3];
+  vtkIdType inIncr[3], outIncr[3];
   OT *outPtr0, *outPtr1, *outPtr2;
-  long streamSkip0, streamSkip1;
-  long streamRead;
+  vtkIdType streamSkip0, streamSkip1;
+  vtkIdType streamRead;
   int idx0, idx1, idx2, pixelRead;
   unsigned char *buf;
   int inExtent[6];
@@ -420,16 +422,16 @@ void vtkBMPReaderUpdate2(vtkBMPReader *self, vtkImageData *data, OT *outPtr)
 
   // length of a row, num pixels read at a time
   pixelRead = dataExtent[1] - dataExtent[0] + 1; 
-  streamRead = (long) (pixelRead * self->GetDataIncrements()[0]);  
-  streamSkip0 = (long) (self->GetDataIncrements()[1] - streamRead);
-  streamSkip1 = (long) (self->GetDataIncrements()[2] - 
+  streamRead = (vtkIdType) (pixelRead * self->GetDataIncrements()[0]);  
+  streamSkip0 = (vtkIdType) (self->GetDataIncrements()[1] - streamRead);
+  streamSkip1 = (vtkIdType) (self->GetDataIncrements()[2] - 
     (dataExtent[3] - dataExtent[2] + 1)* self->GetDataIncrements()[1]);
   pixelSkip = self->GetDepth()/8;
     
   // read from the bottom up
   if (!self->GetFileLowerLeft())
     {
-    streamSkip0 = (long) (-streamRead - self->GetDataIncrements()[1]);
+    streamSkip0 = (vtkIdType) (-streamRead - self->GetDataIncrements()[1]);
     }
   
   // create a buffer to hold a row of the data
@@ -442,13 +444,19 @@ void vtkBMPReaderUpdate2(vtkBMPReader *self, vtkImageData *data, OT *outPtr)
   // read the data row by row
   if (self->GetFileDimensionality() == 3)
     {
-    self->OpenAndSeekFile(dataExtent,0);
+    if (!self->OpenAndSeekFile(dataExtent,0))
+      {
+      return;
+      }
     }
   for (idx2 = dataExtent[4]; idx2 <= dataExtent[5]; ++idx2)
     {
     if (self->GetFileDimensionality() == 2)
       {
-      self->OpenAndSeekFile(dataExtent,idx2);
+      if (!self->OpenAndSeekFile(dataExtent,idx2))
+        {
+        return;
+        }
       }
     outPtr1 = outPtr2;
     for (idx1 = dataExtent[2]; 
@@ -464,13 +472,15 @@ void vtkBMPReaderUpdate2(vtkBMPReader *self, vtkImageData *data, OT *outPtr)
       // read the row.
       if ( ! self->GetFile()->read((char *)buf, streamRead))
         {
-        vtkGenericWarningMacro("File operation failed. row = " << idx1
+        vtkErrorWithObjectMacro(self,
+                               "File operation failed. row = " << idx1
                                << ", Read = " << streamRead
                                << ", Skip0 = " << streamSkip0
                                << ", Skip1 = " << streamSkip1
                                << ", FilePos = " << static_cast<vtkIdType>(self->GetFile()->tellg())
                                << ", FileName = " << self->GetInternalFileName()
                                );
+        self->GetFile()->close();
         return;
         }
       
@@ -509,6 +519,8 @@ void vtkBMPReaderUpdate2(vtkBMPReader *self, vtkImageData *data, OT *outPtr)
     outPtr2 += outIncr[2];
     }
 
+  self->GetFile()->close();
+
   // delete the temporary buffer
   delete [] buf;
 }
@@ -521,6 +533,10 @@ void vtkBMPReader::ExecuteData(vtkDataObject *output)
 {
   vtkImageData *data = this->AllocateOutputData(output);
 
+  if (this->UpdateExtentIsEmpty(output))
+    {
+    return;
+    }
   if (this->InternalFileName == NULL)
     {
     vtkErrorMacro(<< "Either a FileName or FilePrefix must be specified.");
@@ -538,41 +554,15 @@ void vtkBMPReader::ExecuteData(vtkDataObject *output)
   outPtr = data->GetScalarPointer();
   switch (data->GetScalarType())
     {
-    case VTK_DOUBLE:
-      vtkBMPReaderUpdate2(this, data, (double *)(outPtr));
-      break;
-    case VTK_FLOAT:
-      vtkBMPReaderUpdate2(this, data, (float *)(outPtr));
-      break;
-    case VTK_UNSIGNED_LONG:
-      vtkBMPReaderUpdate2(this, data, (unsigned long *)(outPtr));
-      break;
-    case VTK_LONG:
-      vtkBMPReaderUpdate2(this, data, (long *)(outPtr));
-      break;
-    case VTK_UNSIGNED_INT:
-      vtkBMPReaderUpdate2(this, data, (unsigned int *)(outPtr));
-      break;
-    case VTK_INT:
-      vtkBMPReaderUpdate2(this, data, (int *)(outPtr));
-      break;
-    case VTK_SHORT:
-      vtkBMPReaderUpdate2(this, data, (short *)(outPtr));
-      break;
-    case VTK_UNSIGNED_SHORT:
-      vtkBMPReaderUpdate2(this, data, (unsigned short *)(outPtr));
-      break;
-    case VTK_UNSIGNED_CHAR:
-      vtkBMPReaderUpdate2(this, data, (unsigned char *)(outPtr));
-      break;
-    case VTK_CHAR:
-      vtkBMPReaderUpdate2(this, data, (char *)(outPtr));
-      break;
+    vtkTemplateMacro(
+      vtkBMPReaderUpdate2(this, data, static_cast<VTK_TT*>(outPtr))
+      );
     default:
       vtkErrorMacro(<< "Execute: Unknown data type");
     }  
 }
 
+//----------------------------------------------------------------------------
 void vtkBMPReader::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
@@ -590,6 +580,7 @@ void vtkBMPReader::PrintSelf(ostream& os, vtkIndent indent)
     }
 }
 
+//----------------------------------------------------------------------------
 int vtkBMPReader::CanReadFile(const char* fname)
 {
   // get the magic number by reading in a file
@@ -606,57 +597,42 @@ int vtkBMPReader::CanReadFile(const char* fname)
     return 0;
     }
 
-  long tmp;
-  long infoSize;
-  int  iinfoSize;  // in case we are on a 64bit machine
-  int  itmp;       // in case we are on a 64bit machine
+  vtkTypeInt32 tmp;
+  vtkTypeInt32 infoSize = 0;
 
-  // get the size of the file
-  int sizeLong = sizeof(long);
-  if (sizeLong == 4)
-    {
-    fread(&tmp,4,1,fp);
-    // skip 4 bytes
-    fread(&tmp,4,1,fp);
-    // read the offset
-    fread(&tmp,4,1,fp);
-    }
-  else
-    {
-    fread(&itmp,4,1,fp);
-    // skip 4 bytes
-    fread(&itmp,4,1,fp);
-    // read the offset
-    fread(&itmp,4,1,fp);
-    }
+  // error indicator
+  bool errorOccurred = false;
 
+  // skip 4 bytes
+  if (fread(&tmp,4,1,fp) != 1)
+    {
+    errorOccurred = true;
+    }
+  // skip 4 more bytes
+  else if (fread(&tmp,4,1,fp) != 1)
+    {
+    errorOccurred = true;
+    }
+  // read the offset
+  else if (fread(&tmp,4,1,fp) != 1)
+    {
+    errorOccurred = true;
+    }
   // get size of header
-  int res = 3;
-  if (sizeLong == 4)   // if we are on a 32 bit machine
+  else if (fread(&infoSize,4,1,fp) != 1)
     {
-    fread(&infoSize,sizeof(long),1,fp);
-    vtkByteSwap::Swap4LE(&infoSize);
-                       
-    // error checking
-    if ((infoSize != 40)&&(infoSize != 12))
-      {
-      fclose(fp);
-      return 0;
-      }
+    infoSize = 0;
+    errorOccurred = true;
     }
-  else    // else we are on a 64bit machine
+
+  vtkByteSwap::Swap4LE(&infoSize);
+
+  // error checking
+  if ((infoSize != 40)&&(infoSize != 12))
     {
-    fread(&iinfoSize,sizeof(int),1,fp);
-    vtkByteSwap::Swap4LE(&iinfoSize);
-    infoSize = iinfoSize;
-    
-    // error checking
-    if ((infoSize != 40)&&(infoSize != 12))
-      {
-      fclose(fp);
-      res = 0;
-      }
+    errorOccurred = true;
     }
+
   fclose(fp);
-  return res;
+  return !errorOccurred;
 }
