@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkBMPReader.h,v $
+  Module:    vtkBMPReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -45,7 +45,7 @@ class VTK_IO_EXPORT vtkBMPReader : public vtkImageReader
 {
 public:
   static vtkBMPReader *New();
-  vtkTypeRevisionMacro(vtkBMPReader,vtkImageReader);
+  vtkTypeMacro(vtkBMPReader,vtkImageReader);
 
   void PrintSelf(ostream& os, vtkIndent indent);
 
@@ -53,8 +53,10 @@ public:
   // Returns the depth of the BMP, either 8 or 24.
   vtkGetMacro(Depth,int);
   
-  // Description: is the given file name a BMP file?
+  // Description:
+  // Is the given file a BMP file?
   virtual int CanReadFile(const char* fname);
+
   // Description:
   // Get the file extensions for this format.
   // Returns a string with a space separated list of extensions in 
@@ -79,7 +81,7 @@ public:
   vtkGetMacro(Allow8BitBMP,int);
   vtkBooleanMacro(Allow8BitBMP,int);
 
-  vtkLookupTable *GetLookupTable(void);
+  vtkGetObjectMacro(LookupTable, vtkLookupTable);
 
 //BTX
   // Description:
