@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkBYUReader.cxx,v $
+  Module:    vtkBYUReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -18,12 +18,13 @@
 #include "vtkCellArray.h"
 #include "vtkFloatArray.h"
 #include "vtkIdList.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPoints.h"
 #include "vtkPolyData.h"
 
-vtkCxxRevisionMacro(vtkBYUReader, "$Revision: 1.50 $");
 vtkStandardNewMacro(vtkBYUReader);
 
 vtkBYUReader::vtkBYUReader()
@@ -38,6 +39,8 @@ vtkBYUReader::vtkBYUReader()
   this->ReadTexture = 1;
 
   this->PartNumber = 0;
+
+  this->SetNumberOfInputPorts(0);
 }
 
 vtkBYUReader::~vtkBYUReader()
@@ -60,34 +63,75 @@ vtkBYUReader::~vtkBYUReader()
     }
 }
 
-void vtkBYUReader::Execute()
+int vtkBYUReader::CanReadFile(const char *filename)
 {
+  int result;
+  FILE *fp = fopen(filename, "r");
+  if (fp == NULL) return 0;
+
+  int numParts, numPts, numPolys, numEdges;
+  result = fscanf(fp, "%d %d %d %d", &numParts, &numPts, &numPolys, &numEdges);
+  if ((result < 4) || (numParts < 1) || (numPts < 1) || (numPolys < 1))
+    {
+    fclose(fp);
+    return 0;
+    }
+
+  for (int part = 0; part < numParts; part++)
+    {
+    int partStart, partEnd;
+    result = fscanf(fp, "%d %d", &partStart, &partEnd);
+    if (   (result < 2)
+        || (partStart < 1) || (partStart > numPolys)
+        || (partEnd < 1) || (partEnd > numPolys)
+        || (partStart >= partEnd) )
+      {
+      fclose(fp);
+      return 0;
+      }
+    }
+
+  fclose(fp);
+  return 1;
+}
+
+int vtkBYUReader::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
+{
+  // get the info object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
   FILE *geomFp;
   int numPts;
 
   if (this->GeometryFileName == NULL || this->GeometryFileName == '\0')
     {
     vtkErrorMacro(<< "No GeometryFileName specified!");
-    return;
+    return 0;
     }
   if ((geomFp = fopen(this->GeometryFileName, "r")) == NULL)
     {
     vtkErrorMacro(<< "Geometry file: " << this->GeometryFileName << " not found");
-    return;
+    return 0;
     }
   else
     {
-    this->ReadGeometryFile(geomFp,numPts);
+    this->ReadGeometryFile(geomFp,numPts,outInfo);
     fclose(geomFp);
     }
 
-  this->ReadDisplacementFile(numPts);
-  this->ReadScalarFile(numPts);
-  this->ReadTextureFile(numPts);
+  this->ReadDisplacementFile(numPts, outInfo);
+  this->ReadScalarFile(numPts, outInfo);
+  this->ReadTextureFile(numPts, outInfo);
   this->UpdateProgress(1.0);
+
+  return 1;
 }
 
-void vtkBYUReader::ReadGeometryFile(FILE *geomFile, int &numPts)
+void vtkBYUReader::ReadGeometryFile(FILE *geomFile, int &numPts,
+                                    vtkInformation *outInfo)
 {
   int numParts, numPolys, numEdges;
   int partStart, partEnd;
@@ -97,7 +141,8 @@ void vtkBYUReader::ReadGeometryFile(FILE *geomFile, int &numPts)
   float x[3];
   vtkIdList *pts;
   int polyId, pt;
-  vtkPolyData *output = this->GetOutput();
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
   
   pts = vtkIdList::New();
   pts->Allocate(VTK_CELL_SIZE);
@@ -105,7 +150,15 @@ void vtkBYUReader::ReadGeometryFile(FILE *geomFile, int &numPts)
   //
   // Read header (not using fixed format! - potential problem in some files.)
   //
-  fscanf (geomFile, "%d %d %d %d", &numParts, &numPts, &numPolys, &numEdges);
+  int cnt;
+  cnt = fscanf (geomFile, "%d %d %d %d", &numParts, &numPts, &numPolys, &numEdges);
+  if (cnt != 4)
+    {
+    vtkErrorMacro(
+      <<"Error reading geometry file: " << this->GeometryFileName
+      << "Expected 4 values in header, but got " << cnt);
+    return;
+    }
 
   if ( this->PartNumber > numParts )
     {
@@ -118,12 +171,33 @@ void vtkBYUReader::ReadGeometryFile(FILE *geomFile, int &numPts)
     vtkDebugMacro(<<"Reading part number: " << this->PartNumber);
     for (i=0; i < (this->PartNumber-1); i++)
       {
-      fscanf (geomFile, "%*d %*d");
+      cnt = fscanf (geomFile, "%*d %*d");
+      if (cnt != 0)
+        {
+        vtkErrorMacro(
+          <<"Error reading geometry file: " << this->GeometryFileName
+          << "Skipping 2 int's, but skipped " << cnt);
+        return;
+        }
+      }
+    cnt = fscanf (geomFile, "%d %d", &partStart, &partEnd);
+    if (cnt != 2)
+      {
+      vtkErrorMacro(
+        <<"Error reading geometry file: " << this->GeometryFileName
+        << "Expected 2 values for partStart and partEnd, but got " << cnt);
+      return;
       }
-    fscanf (geomFile, "%d %d", &partStart, &partEnd);
     for (i=this->PartNumber; i < numParts; i++)
       {
-      fscanf (geomFile, "%*d %*d");
+      cnt = fscanf (geomFile, "%*d %*d");
+      if (cnt != 0)
+        {
+        vtkErrorMacro(
+          <<"Error reading geometry file: " << this->GeometryFileName
+          << "Skipping 2 int's, but skipped " << cnt);
+        return;
+        }
       }
     }
   else // read all parts
@@ -131,7 +205,14 @@ void vtkBYUReader::ReadGeometryFile(FILE *geomFile, int &numPts)
     vtkDebugMacro(<<"Reading all parts.");
     for (i=0; i < numParts; i++)
       {
-      fscanf (geomFile, "%*d %*d");
+      cnt = fscanf (geomFile, "%*d %*d");
+      if (cnt != 0)
+        {
+        vtkErrorMacro(
+          <<"Error reading geometry file: " << this->GeometryFileName
+        << "Skipping 2 int's, but skipped " << cnt);
+        return;
+        }
       }
     partStart = 1;
     partEnd = VTK_LARGE_INTEGER;
@@ -156,7 +237,14 @@ void vtkBYUReader::ReadGeometryFile(FILE *geomFile, int &numPts)
   // read point coordinates
   for (i=0; i<numPts; i++)
     {
-    fscanf(geomFile, "%e %e %e", x, x+1, x+2);
+    cnt = fscanf(geomFile, "%e %e %e", x, x+1, x+2);
+    if (cnt != 3)
+      {
+      vtkErrorMacro(
+        <<"Error reading geometry file: " << this->GeometryFileName
+        << "Expected 3 points, but got " << cnt);
+      return;
+      }
     newPts->InsertPoint(i,x);
     }
   this->UpdateProgress(0.333);
@@ -190,13 +278,14 @@ void vtkBYUReader::ReadGeometryFile(FILE *geomFile, int &numPts)
   pts->Delete();
 }
 
-void vtkBYUReader::ReadDisplacementFile(int numPts)
+void vtkBYUReader::ReadDisplacementFile(int numPts, vtkInformation *outInfo)
 {
   FILE *dispFp;
   int i;
   float v[3];
   vtkFloatArray *newVectors;
-  vtkPolyData *output = this->GetOutput();
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
   
   if ( this->ReadDisplacement && this->DisplacementFileName )
     {
@@ -219,7 +308,14 @@ void vtkBYUReader::ReadDisplacementFile(int numPts)
 
   for (i=0; i<numPts; i++)
     {
-    fscanf(dispFp, "%e %e %e", v, v+1, v+2);
+    int cnt = fscanf(dispFp, "%e %e %e", v, v+1, v+2);
+    if (cnt != 3)
+      {
+      vtkErrorMacro(
+        <<"Error reading displacement file: " << this->DisplacementFileName
+        << "Expected 3 floats, but got " << cnt);
+      return;
+      }
     newVectors->SetTuple(i,v);
     }
 
@@ -230,13 +326,14 @@ void vtkBYUReader::ReadDisplacementFile(int numPts)
   newVectors->Delete();
 }
 
-void vtkBYUReader::ReadScalarFile(int numPts)
+void vtkBYUReader::ReadScalarFile(int numPts, vtkInformation *outInfo)
 {
   FILE *scalarFp;
   int i;
   float s;
   vtkFloatArray *newScalars;
-  vtkPolyData *output = this->GetOutput();
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
   
   if ( this->ReadScalar && this->ScalarFileName )
     {
@@ -258,7 +355,15 @@ void vtkBYUReader::ReadScalarFile(int numPts)
 
   for (i=0; i<numPts; i++)
     {
-    fscanf(scalarFp, "%e", &s);
+    int cnt = fscanf(scalarFp, "%e", &s);
+    if (cnt != 1)
+      {
+      vtkErrorMacro(
+        <<"Error reading scalar file: " << this->ScalarFileName
+        << "Expected 1 float, but got " << cnt);
+      return;
+      }
+
     newScalars->SetTuple(i,&s);
     }
 
@@ -269,13 +374,14 @@ void vtkBYUReader::ReadScalarFile(int numPts)
   newScalars->Delete();
 }
 
-void vtkBYUReader::ReadTextureFile(int numPts)
+void vtkBYUReader::ReadTextureFile(int numPts, vtkInformation *outInfo)
 {
   FILE *textureFp;
   int i;
   float t[2];
   vtkFloatArray *newTCoords;
-  vtkPolyData *output = this->GetOutput();
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
 
   if ( this->ReadTexture && this->TextureFileName )
     {
@@ -298,7 +404,14 @@ void vtkBYUReader::ReadTextureFile(int numPts)
 
   for (i=0; i<numPts; i++)
     {
-    fscanf(textureFp, "%e %e", t, t+1);
+    int cnt = fscanf(textureFp, "%e %e", t, t+1);
+    if (cnt != 2)
+      {
+      vtkErrorMacro(
+        <<"Error reading texture file: " << this->TextureFileName
+        << "Expected 2 texture coordibates, but got " << cnt);
+      return;
+      }
     newTCoords->SetTuple(i,t);
     }
 
@@ -343,4 +456,3 @@ void vtkBYUReader::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Texture File Name: " 
      << (this->TextureFileName ? this->TextureFileName : "(none)") << "\n";
 }
-
