@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkBYUReader.h,v $
+  Module:    vtkBYUReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -22,14 +22,14 @@
 #ifndef __vtkBYUReader_h
 #define __vtkBYUReader_h
 
-#include "vtkPolyDataSource.h"
+#include "vtkPolyDataAlgorithm.h"
 
-class VTK_IO_EXPORT vtkBYUReader : public vtkPolyDataSource 
+class VTK_IO_EXPORT vtkBYUReader : public vtkPolyDataAlgorithm 
 {
 public:
   static vtkBYUReader *New();
 
-  vtkTypeRevisionMacro(vtkBYUReader,vtkPolyDataSource);
+  vtkTypeMacro(vtkBYUReader,vtkPolyDataAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -81,11 +81,18 @@ public:
   vtkSetClampMacro(PartNumber,int,1,VTK_LARGE_INTEGER);
   vtkGetMacro(PartNumber,int);
 
+  // Description:
+  // Returns 1 if this file can be read and 0 if the file cannot be read.
+  // Because BYU files do not have anything in the header specifying the file
+  // type, the result is not definitive.  Invalid files may still return 1
+  // although a valid file will never return 0.
+  static int CanReadFile(const char *filename);
+
 protected:
   vtkBYUReader();
   ~vtkBYUReader();
 
-  void Execute();
+  int RequestData(vtkInformation *, vtkInformationVector **, vtkInformationVector *);
   // This source does not know how to generate pieces yet.
   int ComputeDivisionExtents(vtkDataObject *output, 
                              int idx, int numDivisions);
@@ -99,15 +106,13 @@ protected:
   int ReadTexture;
   int PartNumber;
 
-  void ReadGeometryFile(FILE *fp, int &numPts);
-  void ReadDisplacementFile(int numPts);
-  void ReadScalarFile(int numPts);
-  void ReadTextureFile(int numPts);
+  void ReadGeometryFile(FILE *fp, int &numPts, vtkInformation *outInfo);
+  void ReadDisplacementFile(int numPts, vtkInformation *outInfo);
+  void ReadScalarFile(int numPts, vtkInformation *outInfo);
+  void ReadTextureFile(int numPts, vtkInformation *outInfo);
 private:
   vtkBYUReader(const vtkBYUReader&);  // Not implemented.
   void operator=(const vtkBYUReader&);  // Not implemented.
 };
 
 #endif
-
-
