@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkDEMReader.h,v $
+  Module:    vtkDEMReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,7 +16,7 @@
 // .SECTION Description
 // vtkDEMReader reads digital elevation files and creates image data.
 // Digital elevation files are produced by the
-// <A HREF="http://www.usgs.org">US Geological Survey</A>. 
+// <A HREF="http://www.usgs.gov">US Geological Survey</A>. 
 // A complete description of the DEM file is located at the USGS site.
 // The reader reads the entire dem file and create a vtkImageData that
 // contains a single scalar component that is the elevation in meters.
@@ -25,13 +25,13 @@
 #ifndef __vtkDEMReader_h
 #define __vtkDEMReader_h
 
-#include "vtkImageSource.h"
+#include "vtkImageAlgorithm.h"
 
-class VTK_IO_EXPORT vtkDEMReader : public vtkImageSource
+class VTK_IO_EXPORT vtkDEMReader : public vtkImageAlgorithm
 {
 public:
   static vtkDEMReader *New();
-  vtkTypeRevisionMacro(vtkDEMReader,vtkImageSource);
+  vtkTypeMacro(vtkDEMReader,vtkImageAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -39,6 +39,23 @@ public:
   vtkSetStringMacro(FileName);
   vtkGetStringMacro(FileName);
 
+//BTX
+  enum {REFERENCE_SEA_LEVEL=0,REFERENCE_ELEVATION_BOUNDS};
+//ETX
+
+  // Description:
+  // Specify the elevation origin to use. By default, the elevation origin
+  // is equal to ElevationBounds[0]. A more convenient origin is to use sea
+  // level (i.e., a value of 0.0).
+  vtkSetClampMacro(ElevationReference,int,REFERENCE_SEA_LEVEL,
+                   REFERENCE_ELEVATION_BOUNDS);
+  vtkGetMacro(ElevationReference,int);
+  void SetElevationReferenceToSeaLevel() 
+    {this->SetElevationReference(REFERENCE_SEA_LEVEL);}
+  void SetElevationReferenceToElevationBounds() 
+    {this->SetElevationReference(REFERENCE_ELEVATION_BOUNDS);}
+  const char *GetElevationReferenceAsString(void);
+
   // Description:
   // An ASCII description of the map
   vtkGetStringMacro(MapLabel);
@@ -106,7 +123,8 @@ public:
   // Reads the DEM Type A record to compute the extent, origin and
   // spacing of the image data. The number of scalar components is set
   // to 1 and the output scalar type is VTK_FLOAT. 
-  void ExecuteInformation();
+  virtual int RequestInformation (vtkInformation *, vtkInformationVector **, 
+                                  vtkInformationVector *);
 
 protected:
   vtkDEMReader();
@@ -133,12 +151,17 @@ protected:
   float SpatialResolution[3];
   int ProfileDimension[2];
   int ProfileSeekOffset;
+  int ElevationReference;
+
   void ComputeExtentOriginAndSpacing (int extent[6], 
                                       double origin[6], 
                                       double spacing[6]);
   int ReadTypeARecord ();
   int ReadProfiles (vtkImageData *data);
-  void ExecuteData(vtkDataObject *out);
+  virtual int RequestData(  vtkInformation* request,
+                            vtkInformationVector** inputVector,
+                            vtkInformationVector* outputVector);
+
 private:
   vtkDEMReader(const vtkDEMReader&);  // Not implemented.
   void operator=(const vtkDEMReader&);  // Not implemented.
