@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkDataReader.h,v $
+  Module:    vtkDataReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -21,29 +21,32 @@
 //
 // .SECTION See Also
 // vtkPolyDataReader vtkStructuredPointsReader vtkStructuredGridReader
-// vtkUnsutructuredGridReader vtkRectilinearGridReader
+// vtkUnstructuredGridReader vtkRectilinearGridReader
 
 #ifndef __vtkDataReader_h
 #define __vtkDataReader_h
 
-#include "vtkSource.h"
+#include "vtkAlgorithm.h"
+#include "vtkStdString.h" // For API using strings
 
 #define VTK_ASCII 1
 #define VTK_BINARY 2
 
+class vtkAbstractArray;
 class vtkCharArray;
-class vtkDataArray;
 class vtkDataSet;
 class vtkDataSetAttributes;
 class vtkFieldData;
+class vtkGraph;
 class vtkPointSet;
 class vtkRectilinearGrid;
+class vtkTable;
 
-class VTK_IO_EXPORT vtkDataReader : public vtkSource
+class VTK_IO_EXPORT vtkDataReader : public vtkAlgorithm
 {
 public:
   static vtkDataReader *New();
-  vtkTypeRevisionMacro(vtkDataReader,vtkSource);
+  vtkTypeMacro(vtkDataReader,vtkAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -76,10 +79,12 @@ public:
   void SetInputString(const char *in, int len);
   vtkGetMacro(InputStringLength, int);
   void SetBinaryInputString(const char *, int len);
+  void SetInputString(const vtkStdString& input)
+    { this->SetBinaryInputString(input.c_str(), static_cast<int>(input.length())); }
 
   // Description:
   // Specify the vtkCharArray to be used  when reading from a string.
-  // If set, this array has precendence over InputString.
+  // If set, this array has precedence over InputString.
   // Use this instead of InputString to avoid the extra memory copy.
   // It should be noted that if the underlying char* is owned by the
   // user ( vtkCharArray::SetArray(array, 1); ) and is deleted before
@@ -175,6 +180,48 @@ public:
   vtkSetStringMacro(FieldDataName);
   vtkGetStringMacro(FieldDataName);
 
+  // Description:
+  // Enable reading all scalars. 
+  vtkSetMacro(ReadAllScalars,int);
+  vtkGetMacro(ReadAllScalars,int);
+  vtkBooleanMacro(ReadAllScalars,int);
+
+  // Description:
+  // Enable reading all vectors. 
+  vtkSetMacro(ReadAllVectors,int);
+  vtkGetMacro(ReadAllVectors,int);
+  vtkBooleanMacro(ReadAllVectors,int);
+
+  // Description:
+  // Enable reading all normals. 
+  vtkSetMacro(ReadAllNormals,int);
+  vtkGetMacro(ReadAllNormals,int);
+  vtkBooleanMacro(ReadAllNormals,int);
+
+  // Description:
+  // Enable reading all tensors. 
+  vtkSetMacro(ReadAllTensors,int);
+  vtkGetMacro(ReadAllTensors,int);
+  vtkBooleanMacro(ReadAllTensors,int);
+
+  // Description:
+  // Enable reading all color scalars. 
+  vtkSetMacro(ReadAllColorScalars,int);
+  vtkGetMacro(ReadAllColorScalars,int);
+  vtkBooleanMacro(ReadAllColorScalars,int);
+
+  // Description:
+  // Enable reading all tcoords. 
+  vtkSetMacro(ReadAllTCoords,int);
+  vtkGetMacro(ReadAllTCoords,int);
+  vtkBooleanMacro(ReadAllTCoords,int);
+
+  // Description:
+  // Enable reading all fields. 
+  vtkSetMacro(ReadAllFields,int);
+  vtkGetMacro(ReadAllFields,int);
+  vtkBooleanMacro(ReadAllFields,int);
+
   // Description:
   // Open a vtk data file. Returns zero if error.
   int OpenVTKFile();
@@ -199,6 +246,26 @@ public:
   // Read point coordinates. Return 0 if error.
   int ReadPoints(vtkPointSet *ps, int numPts);
 
+  // Description:
+  // Read point coordinates. Return 0 if error.
+  int ReadPoints(vtkGraph *g, int numPts);
+
+  // Description:
+  // Read the vertex data of a vtk data file. The number of vertices (from the
+  // graph) must match the number of vertices defined in vertex attributes
+  // (unless no geometry was defined).
+  int ReadVertexData(vtkGraph *g, int numVertices);
+
+  // Description:
+  // Read the edge data of a vtk data file. The number of edges (from the
+  // graph) must match the number of edges defined in edge attributes
+  // (unless no geometry was defined).
+  int ReadEdgeData(vtkGraph *g, int numEdges);
+
+  // Description:
+  // Read the row data of a vtk data file.
+  int ReadRowData(vtkTable *t, int numEdges);
+
   // Description:
   // Read a bunch of "cells". Return 0 if error.
   int ReadCells(int size, int *data);
@@ -214,9 +281,10 @@ public:
 
   // Description:
   // Helper functions for reading data.
-  vtkDataArray *ReadArray(const char *dataType, int numTuples, int numComp);
+  vtkAbstractArray *ReadArray(const char *dataType, int numTuples, int numComp);
   vtkFieldData *ReadFieldData();
 
+//BTX  
   // Description:
   // Internal function to read in a value.  Returns zero if there was an
   // error.
@@ -228,8 +296,17 @@ public:
   int Read(unsigned int *);
   int Read(long *);
   int Read(unsigned long *);
+#if defined(VTK_TYPE_USE___INT64)
+  int Read(__int64 *result);
+  int Read(unsigned __int64 *result);
+#endif
+#if defined(VTK_TYPE_USE_LONG_LONG)
+  int Read(long long *result);
+  int Read(unsigned long long *result);
+#endif
   int Read(float *);
-  int Read(double *);
+  int Read(double *);  
+//ETX
 
   // Description:
   // Close the vtk file.
@@ -255,6 +332,11 @@ public:
   istream *GetIStream() {return this->IS;};
 //ETX
 
+  // Description:
+  // Read the meta information from the file.  This needs to be public to it
+  // can be accessed by vtkDataSetReader.
+  virtual int ReadMetaData(vtkInformation *) { return 1; }
+
 protected:
   vtkDataReader();
   ~vtkDataReader();
@@ -277,7 +359,7 @@ protected:
   int InputStringLength;
   int InputStringPos;
 
-  vtkSetStringMacro(ScalarLut);
+  void SetScalarLut(const char* lut);
   vtkGetStringMacro(ScalarLut);
 
   char *Header;
@@ -289,6 +371,8 @@ protected:
   int ReadCoScalarData(vtkDataSetAttributes *a, int num);
   int ReadLutData(vtkDataSetAttributes *a);
   int ReadTCoordsData(vtkDataSetAttributes *a, int num);
+  int ReadGlobalIds(vtkDataSetAttributes *a, int num);
+  int ReadPedigreeIds(vtkDataSetAttributes *a, int num);
 
   int ReadDataSetData(vtkDataSet *ds);
 
@@ -312,6 +396,14 @@ protected:
   char **FieldDataNameInFile;
   int FieldDataNameAllocSize;
   vtkTimeStamp CharacteristicsTime;
+ 
+  int ReadAllScalars;
+  int ReadAllVectors;
+  int ReadAllNormals;
+  int ReadAllTensors;
+  int ReadAllColorScalars;
+  int ReadAllTCoords;
+  int ReadAllFields;
 
   void InitializeCharacteristics();
   int CharacterizeFile(); //read entire file, storing important characteristics
@@ -321,9 +413,22 @@ protected:
   vtkCharArray* InputArray;
 
   // Description:
-  // Decode the name of array. This method is the inverse of 
-  // vtkWriter::EncodeName.
-  void DecodeArrayName(char *resname, const char* name);
+  // Decode a string. This method is the inverse of 
+  // vtkWriter::EncodeString.  Returns the length of the
+  // result string.
+  int DecodeString(char *resname, const char* name);
+
+  virtual int ProcessRequest(vtkInformation *, vtkInformationVector **,
+                             vtkInformationVector *);
+  virtual int RequestData(vtkInformation *, vtkInformationVector **,
+                          vtkInformationVector *)
+    { return 1; }
+  virtual int RequestUpdateExtent(vtkInformation *, vtkInformationVector **,
+                                  vtkInformationVector *)
+    { return 1; }
+  virtual int RequestInformation(vtkInformation *, vtkInformationVector **,
+                                 vtkInformationVector *)
+    { return 1; }
 
 private:
   vtkDataReader(const vtkDataReader&);  // Not implemented.
