@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkDataWriter.h,v $
+  Module:    vtkDataWriter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -28,16 +28,18 @@
 
 #include "vtkWriter.h"
 
-class vtkDataSet;
-class vtkPoints;
 class vtkCellArray;
 class vtkDataArray;
+class vtkDataSet;
 class vtkFieldData;
+class vtkGraph;
+class vtkPoints;
+class vtkTable;
 
 class VTK_IO_EXPORT vtkDataWriter : public vtkWriter
 {
 public:
-  vtkTypeRevisionMacro(vtkDataWriter,vtkWriter);
+  vtkTypeMacro(vtkDataWriter,vtkWriter);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -62,8 +64,15 @@ public:
   // the next call to write ...
   vtkGetMacro(OutputStringLength, int);  
   vtkGetStringMacro(OutputString);
-  unsigned char *GetBinaryOutputString() {
-      return (unsigned char *)this->OutputString;};
+  unsigned char *GetBinaryOutputString()
+    {
+      return reinterpret_cast<unsigned char *>(this->OutputString);
+    }
+
+  // Description:
+  // When WriteToOutputString is on, this method returns a copy of the
+  // output string in a vtkStdString.
+  vtkStdString GetOutputStdString();
       
   // Description:
   // This convenience method returns the string, sets the IVAR to NULL,
@@ -113,6 +122,18 @@ public:
   vtkSetStringMacro(TCoordsName);
   vtkGetStringMacro(TCoordsName);
 
+  // Description:
+  // Give a name to the global ids data. If not specified, uses 
+  // default name "global_ids".
+  vtkSetStringMacro(GlobalIdsName);
+  vtkGetStringMacro(GlobalIdsName);
+
+  // Description:
+  // Give a name to the pedigree ids data. If not specified, uses 
+  // default name "pedigree_ids".
+  vtkSetStringMacro(PedigreeIdsName);
+  vtkGetStringMacro(PedigreeIdsName);
+
   // Description:
   // Give a name to the lookup table. If not specified, uses default
   // name "lookupTable".
@@ -155,6 +176,21 @@ public:
   // Returns 0 if error.
   int WritePointData(ostream *fp, vtkDataSet *ds);
 
+  // Description:
+  // Write the edge data (e.g., scalars, vectors, ...) of a vtk graph.
+  // Returns 0 if error.
+  int WriteEdgeData(ostream *fp, vtkGraph *g);
+
+  // Description:
+  // Write the vertex data (e.g., scalars, vectors, ...) of a vtk graph.
+  // Returns 0 if error.
+  int WriteVertexData(ostream *fp, vtkGraph *g);
+
+  // Description:
+  // Write the row data (e.g., scalars, vectors, ...) of a vtk table.
+  // Returns 0 if error.
+  int WriteRowData(ostream *fp, vtkTable *g);
+
   // Description:
   // Write out the field data.
   int WriteFieldData(ostream *fp, vtkFieldData *f);
@@ -176,7 +212,6 @@ protected:
   int WriteToOutputString;
   char *OutputString;
   int OutputStringLength;
-  int OutputStringAllocatedLength;
   
   void WriteData(); //dummy method to allow this class to be instantiated and delegated to
 
@@ -191,14 +226,18 @@ protected:
   char *NormalsName;
   char *LookupTableName;
   char *FieldDataName;
+  char* GlobalIdsName;
+  char* PedigreeIdsName;
 
-  int WriteArray(ostream *fp, int dataType, vtkDataArray *data, const char *format, 
+  int WriteArray(ostream *fp, int dataType, vtkAbstractArray *data, const char *format, 
                  int num, int numComp);
   int WriteScalarData(ostream *fp, vtkDataArray *s, int num);
   int WriteVectorData(ostream *fp, vtkDataArray *v, int num);
   int WriteNormalData(ostream *fp, vtkDataArray *n, int num);
   int WriteTCoordData(ostream *fp, vtkDataArray *tc, int num);
   int WriteTensorData(ostream *fp, vtkDataArray *t, int num);
+  int WriteGlobalIdData(ostream *fp, vtkDataArray *g, int num);
+  int WritePedigreeIdData(ostream *fp, vtkAbstractArray *p, int num);
 
 private:
   vtkDataWriter(const vtkDataWriter&);  // Not implemented.
@@ -206,5 +245,3 @@ private:
 };
 
 #endif
-
-
