@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkEnSightGoldBinaryReader.h,v $
+  Module:    vtkEnSightGoldBinaryReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -31,101 +31,118 @@
 // what types they will be.
 // This reader can only handle static EnSight datasets (both static geometry
 // and variables).
+// .SECTION Thanks
+// Thanks to Yvan Fournier for providing the code to support nfaced elements.
 
 #ifndef __vtkEnSightGoldBinaryReader_h
 #define __vtkEnSightGoldBinaryReader_h
 
 #include "vtkEnSightReader.h"
 
+class vtkMultiBlockDataSet;
+
 class VTK_IO_EXPORT vtkEnSightGoldBinaryReader : public vtkEnSightReader
 {
 public:
   static vtkEnSightGoldBinaryReader *New();
-  vtkTypeRevisionMacro(vtkEnSightGoldBinaryReader, vtkEnSightReader);
+  vtkTypeMacro(vtkEnSightGoldBinaryReader, vtkEnSightReader);
   virtual void PrintSelf(ostream& os, vtkIndent indent);
- 
+
 protected:
   vtkEnSightGoldBinaryReader();
   ~vtkEnSightGoldBinaryReader();
-  
+
   // Returns 1 if successful.  Sets file size as a side action.
   int OpenFile(const char* filename);
 
+
+  // Returns 1 if successful.  Handles constructing the filename, opening the file and checking
+  // if it's binary
+  int InitializeFile(const char* filename);
+
   // Description:
   // Read the geometry file.  If an error occurred, 0 is returned; otherwise 1.
-  virtual int ReadGeometryFile(char* fileName, int timeStep);
+  virtual int ReadGeometryFile(const char* fileName, int timeStep,
+    vtkMultiBlockDataSet *output);
 
   // Description:
   // Read the measured geometry file.  If an error occurred, 0 is returned;
   // otherwise 1.
-  virtual int ReadMeasuredGeometryFile(char* fileName, int timeStep);
+  virtual int ReadMeasuredGeometryFile(const char* fileName, int timeStep,
+    vtkMultiBlockDataSet *output);
 
   // Description:
   // Read scalars per node for this dataset.  If an error occurred, 0 is
   // returned; otherwise 1.  If there will be more than one component in
   // the data array, it is assumed that 0 is the first component added.
-  virtual int ReadScalarsPerNode(char* fileName, char* description,
-                                 int timeStep, int measured = 0,
-                                 int numberOfComponents = 1,
-                                 int component = 0);
-  
+  virtual int ReadScalarsPerNode(const char* fileName, const char* description,
+    int timeStep, vtkMultiBlockDataSet *output,
+    int measured = 0, int numberOfComponents = 1,
+    int component = 0);
+
   // Description:
   // Read vectors per node for this dataset.  If an error occurred, 0 is
   // returned; otherwise 1.
-  virtual int ReadVectorsPerNode(char* fileName, char* description,
-                                 int timeStep, int measured = 0);
+  virtual int ReadVectorsPerNode(const char* fileName, const char* description,
+    int timeStep, vtkMultiBlockDataSet *output,
+    int measured = 0);
 
   // Description:
   // Read tensors per node for this dataset.  If an error occurred, 0 is
   // returned; otherwise 1.
-  virtual int ReadTensorsPerNode(char* fileName, char* description,
-                                 int timeStep);
+  virtual int ReadTensorsPerNode(const char* fileName, const char* description,
+    int timeStep, vtkMultiBlockDataSet *output);
 
   // Description:
   // Read scalars per element for this dataset.  If an error occurred, 0 is
   // returned; otherwise 1.  If there will be more than one componenet in the
   // data array, it is assumed that 0 is the first component added.
-  virtual int ReadScalarsPerElement(char* fileName, char* description,
-                                    int timeStep, int numberOfComponents = 1,
-                                    int component = 0);
+  virtual int ReadScalarsPerElement(const char* fileName, const char* description,
+    int timeStep, vtkMultiBlockDataSet *output,
+    int numberOfComponents = 1,
+    int component = 0);
 
   // Description:
   // Read vectors per element for this dataset.  If an error occurred, 0 is
   // returned; otherwise 1.
-  virtual int ReadVectorsPerElement(char* fileName, char* description,
-                                    int timeStep);
+  virtual int ReadVectorsPerElement(const char* fileName, const char* description,
+    int timeStep, vtkMultiBlockDataSet *output);
 
   // Description:
   // Read tensors per element for this dataset.  If an error occurred, 0 is
   // returned; otherwise 1.
-  virtual int ReadTensorsPerElement(char* fileName, char* description,
-                                    int timeStep);
+  virtual int ReadTensorsPerElement(const char* fileName, const char* description,
+    int timeStep, vtkMultiBlockDataSet *output);
 
   // Description:
   // Read an unstructured part (partId) from the geometry file and create a
   // vtkUnstructuredGrid output.  Return 0 if EOF reached. Return -1 if
   // an error occurred.
-  virtual int CreateUnstructuredGridOutput(int partId, 
-                                           char line[80],
-                                           const char* name);
-  
+  virtual int CreateUnstructuredGridOutput(int partId,
+    char line[80],
+    const char* name,
+    vtkMultiBlockDataSet *output);
+
   // Description:
   // Read a structured part from the geometry file and create a
   // vtkStructuredGrid output.  Return 0 if EOF reached.
-  virtual int CreateStructuredGridOutput(int partId, 
-                                         char line[256],
-                                         const char* name);
-  
+  virtual int CreateStructuredGridOutput(int partId,
+    char line[256],
+    const char* name,
+    vtkMultiBlockDataSet *output);
+
   // Description:
   // Read a structured part from the geometry file and create a
   // vtkRectilinearGrid output.  Return 0 if EOF reached.
-  int CreateRectilinearGridOutput(int partId, char line[256], const char* name);
-  
+  int CreateRectilinearGridOutput(int partId, char line[256], const char* name,
+    vtkMultiBlockDataSet *output);
+
   // Description:
   // Read a structured part from the geometry file and create a
   // vtkImageData output.  Return 0 if EOF reached.
-  int CreateImageDataOutput(int partId, char line[80], const char* name);
-  
+  int CreateImageDataOutput(int partId, char line[80], const char* name,
+    vtkMultiBlockDataSet *output);
+
   // Description:
   // Internal function to read in a line up to 80 characters.
   // Returns zero if there was an error.
@@ -147,22 +164,31 @@ protected:
   // Returns zero if there was an error.
   int ReadFloatArray(float *result, int numFloats);
 
+  // Description:
+  // Counts the number of timesteps in the geometry file
+  // This function assumes the file is already open and returns the
+  // number of timesteps remaining in the file
+  // The file will be closed after calling this method
+  int CountTimeSteps();
+
   // Description:
   // Read to the next time step in the geometry file.
-  void SkipTimeStep();
+  int SkipTimeStep();
   int SkipStructuredGrid(char line[256]);
   int SkipUnstructuredGrid(char line[256]);
   int SkipRectilinearGrid(char line[256]);
   int SkipImageData(char line[256]);
-  
+
   int NodeIdsListed;
   int ElementIdsListed;
-  
+  int Fortran;
+
   ifstream *IFile;
   // The size of the file could be used to choose byte order.
-  int FileSize;
+  vtkIdType FileSize;
 
 private:
+  int SizeOfInt;
   vtkEnSightGoldBinaryReader(const vtkEnSightGoldBinaryReader&);  // Not implemented.
   void operator=(const vtkEnSightGoldBinaryReader&);  // Not implemented.
 };
