@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageReader.cxx,v $
+  Module:    vtkImageReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,12 +15,15 @@
 #include "vtkImageReader.h"
 
 #include "vtkByteSwap.h"
+#include "vtkDataArray.h"
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkTransform.h"
 
-vtkCxxRevisionMacro(vtkImageReader, "$Revision: 1.115 $");
 vtkStandardNewMacro(vtkImageReader);
 
 vtkCxxSetObjectMacro(vtkImageReader,Transform,vtkTransform);
@@ -42,9 +45,8 @@ vtkImageReader::vtkImageReader()
     {
     this->DataVOI[idx*2] = this->DataVOI[idx*2 + 1] = 0;
     }
-  
-  // Left over from short reader
-  this->DataMask = 0xffff;
+
+  this->DataMask = static_cast<vtkTypeUInt64>(~0UL);
   this->Transform = NULL;
   
   this->ScalarArrayName = NULL;
@@ -90,16 +92,21 @@ void vtkImageReader::PrintSelf(ostream& os, vtkIndent indent)
 }
 
 
-//----------------------------------------------------------------------------
 // This method returns the largest data that can be generated.
-void vtkImageReader::ExecuteInformation()
+int vtkImageReader::RequestInformation (
+  vtkInformation       * vtkNotUsed( request ),
+  vtkInformationVector** vtkNotUsed( inputVector ),
+  vtkInformationVector * outputVector)
 {
-  vtkImageData *output = this->GetOutput();
+  // call the old method to help with backwards compatiblity
+  this->ExecuteInformation();
+
+  // get the info objects
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
   double spacing[3];
   int extent[6];
   double origin[3];
   
-  
   // set the extent, if the VOI has not been set then default to
   // the DataExtent
   if (this->DataVOI[0] || this->DataVOI[1] || 
@@ -107,28 +114,26 @@ void vtkImageReader::ExecuteInformation()
       this->DataVOI[4] || this->DataVOI[5])
     {
     this->ComputeTransformedExtent(this->DataVOI,extent);
-    output->SetWholeExtent(extent);
     }
   else
     {
     this->ComputeTransformedExtent(this->DataExtent,extent);
-    output->SetWholeExtent(extent);
     }
+  outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),extent, 6);
     
   // set the spacing
   this->ComputeTransformedSpacing(spacing);
-  output->SetSpacing(spacing);
+  outInfo->Set(vtkDataObject::SPACING(), this->DataSpacing, 3);
 
   // set the origin.
   this->ComputeTransformedOrigin(origin);
-  output->SetOrigin(origin);
+  outInfo->Set(vtkDataObject::ORIGIN(),  this->DataOrigin, 3);
 
-  output->SetScalarType(this->DataScalarType);
-  output->SetNumberOfScalarComponents(this->NumberOfScalarComponents);
+  vtkDataObject::SetPointDataActiveScalarInfo(outInfo, this->DataScalarType,
+    this->NumberOfScalarComponents);
+  return 1;
 }
 
-
-
 int vtkImageReader::OpenAndSeekFile(int dataExtent[6], int idx)
 {
   unsigned long streamStart;
@@ -193,7 +198,7 @@ template <class IT, class OT>
 void vtkImageReaderUpdate2(vtkImageReader *self, vtkImageData *data,
                            IT *inPtr, OT *outPtr)
 {
-  int inIncr[3], outIncr[3];
+  vtkIdType inIncr[3], outIncr[3];
   OT *outPtr0, *outPtr1, *outPtr2;
   long streamSkip0, streamSkip1;
   unsigned long streamRead;
@@ -204,7 +209,7 @@ void vtkImageReaderUpdate2(vtkImageReader *self, vtkImageData *data,
   int comp, pixelSkip;
   long filePos, correction = 0;
   unsigned long count = 0;
-  unsigned short DataMask;
+  vtkTypeUInt64 DataMask;
   unsigned long target;
   
   // Get the requested extents.
@@ -320,7 +325,7 @@ void vtkImageReaderUpdate2(vtkImageReader *self, vtkImageData *data,
       for (idx0 = dataExtent[0]; idx0 <= dataExtent[1]; ++idx0)
         {
         // Copy pixel into the output.
-        if (DataMask == 0xffff)
+        if (DataMask == static_cast<vtkTypeUInt64>(~0UL))
           {
           for (comp = 0; comp < pixelSkip; comp++)
             {
@@ -329,10 +334,9 @@ void vtkImageReaderUpdate2(vtkImageReader *self, vtkImageData *data,
           }
         else
           {
-          // left over from short reader (what about other types.
           for (comp = 0; comp < pixelSkip; comp++)
             {
-            outPtr0[comp] = (OT)((short)(inPtr[comp]) & DataMask);
+            outPtr0[comp] = (OT)((vtkTypeUInt64)(inPtr[comp]) & DataMask);
             }
           }
         // move to next pixel
@@ -402,8 +406,8 @@ void vtkImageReaderUpdate1(vtkImageReader *self, vtkImageData *data, T *inPtr)
   outPtr = data->GetScalarPointer();
   switch (data->GetScalarType())
     {
-    vtkTemplateMacro4(vtkImageReaderUpdate2, self, data, inPtr, 
-                      (VTK_TT *)(outPtr));
+    vtkTemplateMacro(vtkImageReaderUpdate2(self, data, inPtr, 
+                                           (VTK_TT *)(outPtr)));
     default:
       vtkGenericWarningMacro("Update1: Unknown data type\n");
     }  
@@ -416,7 +420,6 @@ void vtkImageReader::ExecuteData(vtkDataObject *output)
   vtkImageData *data = this->AllocateOutputData(output);
   
   void *ptr = NULL;
-  int *ext;
 
   if (!this->FileName && !this->FilePattern)
     {
@@ -424,9 +427,15 @@ void vtkImageReader::ExecuteData(vtkDataObject *output)
     return;
     }
 
-  ext = data->GetExtent();
+  if (!data->GetPointData()->GetScalars())
+    {
+    return;
+    }
   data->GetPointData()->GetScalars()->SetName(this->ScalarArrayName);
 
+#ifndef NDEBUG
+  int *ext = data->GetExtent();
+#endif
   vtkDebugMacro("Reading extent: " << ext[0] << ", " << ext[1] << ", " 
         << ext[2] << ", " << ext[3] << ", " << ext[4] << ", " << ext[5]);
   
@@ -435,7 +444,7 @@ void vtkImageReader::ExecuteData(vtkDataObject *output)
   // Call the correct templated function for the output
   switch (this->GetDataScalarType())
     {
-    vtkTemplateMacro3(vtkImageReaderUpdate1, this, data, (VTK_TT *)(ptr));
+    vtkTemplateMacro(vtkImageReaderUpdate1(this, data, (VTK_TT *)(ptr)));
     default:
       vtkErrorMacro(<< "UpdateFromFile: Unknown data type");
     }   
@@ -674,14 +683,14 @@ void vtkImageReader::ComputeInverseTransformedExtent(int inExtent[6],
     << outExtent[4] << ", " << outExtent[5]);
 }
 
-void vtkImageReader::ComputeTransformedIncrements(int inIncr[3],
-                                                  int outIncr[3])
+void vtkImageReader::ComputeTransformedIncrements(vtkIdType inIncr[3],
+                                                  vtkIdType outIncr[3])
 {
   double transformedIncr[3];
   
   if (!this->Transform)
     {
-    memcpy (outIncr, inIncr, 3 * sizeof (int));
+    memcpy (outIncr, inIncr, 3 * sizeof (vtkIdType));
     }
   else
     {
@@ -689,23 +698,23 @@ void vtkImageReader::ComputeTransformedIncrements(int inIncr[3],
     transformedIncr[1] = inIncr[1];
     transformedIncr[2] = inIncr[2];
     this->Transform->TransformVector(transformedIncr, transformedIncr);
-    outIncr[0] = (int) transformedIncr[0];
-    outIncr[1] = (int) transformedIncr[1];
-    outIncr[2] = (int) transformedIncr[2];
+    outIncr[0] = (vtkIdType) transformedIncr[0];
+    outIncr[1] = (vtkIdType) transformedIncr[1];
+    outIncr[2] = (vtkIdType) transformedIncr[2];
     vtkDebugMacro(<< "Transformed Incr are:" 
     << outIncr[0] << ", " << outIncr[1] << ", " << outIncr[2]);
     }
 }
 
 
-void vtkImageReader::ComputeInverseTransformedIncrements(int inIncr[3],
-                                                         int outIncr[3])
+void vtkImageReader::ComputeInverseTransformedIncrements(vtkIdType inIncr[3],
+                                                         vtkIdType outIncr[3])
 {
   double transformedIncr[3];
   
   if (!this->Transform)
     {
-    memcpy (outIncr, inIncr, 3 * sizeof (int));
+    memcpy (outIncr, inIncr, 3 * sizeof (vtkIdType));
     }
   else
     {
@@ -714,9 +723,9 @@ void vtkImageReader::ComputeInverseTransformedIncrements(int inIncr[3],
     transformedIncr[2] = inIncr[2];
     this->Transform->GetLinearInverse()->TransformVector(transformedIncr,
                                                          transformedIncr);
-    outIncr[0] = (int) transformedIncr[0];
-    outIncr[1] = (int) transformedIncr[1];
-    outIncr[2] = (int) transformedIncr[2];
+    outIncr[0] = (vtkIdType) transformedIncr[0];
+    outIncr[1] = (vtkIdType) transformedIncr[1];
+    outIncr[2] = (vtkIdType) transformedIncr[2];
     vtkDebugMacro(<< "Inverse Transformed Incr are:" 
     << outIncr[0] << ", " << outIncr[1] << ", " << outIncr[2]);
     }
