@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageWriter.cxx,v $
+  Module:    vtkImageWriter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,28 +16,17 @@
 
 #include "vtkCommand.h"
 #include "vtkErrorCode.h"
-#include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
+#include "vtkImageData.h"
 
-#if !defined(_WIN32) || defined(__CYGWIN__)
-# include <unistd.h> /* unlink */
-#else
-# include <io.h> /* unlink */
-#endif
+#include <vtksys/SystemTools.hxx>
 
-vtkCxxRevisionMacro(vtkImageWriter, "$Revision: 1.53 $");
 vtkStandardNewMacro(vtkImageWriter);
 
-#ifdef write
-#undef write
-#endif
-
-#ifdef close
-#undef close
-#endif
-
-
 //----------------------------------------------------------------------------
 vtkImageWriter::vtkImageWriter()
 {
@@ -55,6 +44,7 @@ vtkImageWriter::vtkImageWriter()
   
   this->MinimumFileNumber = this->MaximumFileNumber = 0;
   this->FilesDeleted = 0;
+  this->SetNumberOfOutputPorts(0);
 }
 
 
@@ -97,124 +87,40 @@ void vtkImageWriter::PrintSelf(ostream& os, vtkIndent indent)
 }
 
 
-//----------------------------------------------------------------------------
-void vtkImageWriter::SetInput(vtkImageData *input)
-{
-  this->vtkProcessObject::SetNthInput(0, input);
-}
-
 //----------------------------------------------------------------------------
 vtkImageData *vtkImageWriter::GetInput()
 {
-  if (this->NumberOfInputs < 1)
+  if (this->GetNumberOfInputConnections(0) < 1)
     {
-    return NULL;
+    return 0;
     }
-  
-  return (vtkImageData *)(this->Inputs[0]);
+  return vtkImageData::SafeDownCast(
+    this->GetExecutive()->GetInputData(0, 0));
 }
-
-
-
 //----------------------------------------------------------------------------
-// This function sets the name of the file. 
-void vtkImageWriter::SetFileName(const char *name)
-{
-  if ( this->FileName && name && (!strcmp(this->FileName,name)))
-    {
-    return;
-    }
-  if (!name && !this->FileName)
-    {
-    return;
-    }
-  if (this->FileName)
-    {
-    delete [] this->FileName;
-    }
-  if (this->FilePrefix)
-    {
-    delete [] this->FilePrefix;
-    this->FilePrefix = NULL;
-    }  
-  this->FileName = new char[strlen(name) + 1];
-  strcpy(this->FileName, name);
-  this->Modified();
-}
 
-//----------------------------------------------------------------------------
-// This function sets the prefix of the file name. "image" would be the
-// name of a series: image.1, image.2 ...
-void vtkImageWriter::SetFilePrefix(char *prefix)
-{
-  if ( this->FilePrefix && prefix && (!strcmp(this->FilePrefix,prefix)))
-    {
-    return;
-    }
-  if (!prefix && !this->FilePrefix)
-    {
-    return;
-    }
-  if (this->FilePrefix)
-    {
-    delete [] this->FilePrefix;
-    }
-  if (this->FileName)
-    {
-    delete [] this->FileName;
-    this->FileName = NULL;
-    }  
-  this->FilePrefix = new char[strlen(prefix) + 1];
-  strcpy(this->FilePrefix, prefix);
-  this->Modified();
-}
-
-//----------------------------------------------------------------------------
-// This function sets the pattern of the file name which turn a prefix
-// into a file name. "%s.%3d" would be the
-// pattern of a series: image.001, image.002 ...
-void vtkImageWriter::SetFilePattern(const char *pattern)
-{
-  if ( this->FilePattern && pattern && 
-       (!strcmp(this->FilePattern,pattern)))
-    {
-    return;
-    }
-  if (!pattern && !this->FilePattern)
-    {
-    return;
-    }
-  if (this->FilePattern)
-    {
-    delete [] this->FilePattern;
-    }
-  if (this->FileName)
-    {
-    delete [] this->FileName;
-    this->FileName = NULL;
-    }
-  this->FilePattern = new char[strlen(pattern) + 1];
-  strcpy(this->FilePattern, pattern);
-  this->Modified();
-}
-
-//----------------------------------------------------------------------------
-// Writes all the data from the input.
-void vtkImageWriter::Write()
+int vtkImageWriter::RequestData(
+  vtkInformation* vtkNotUsed( request ),
+  vtkInformationVector** inputVector,
+  vtkInformationVector* vtkNotUsed( outputVector) )
 {
   this->SetErrorCode(vtkErrorCode::NoError);
+
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkImageData *input = 
+    vtkImageData::SafeDownCast(inInfo->Get(vtkDataObject::DATA_OBJECT()));
   
   // Error checking
-  if ( this->GetInput() == NULL )
+  if (input == NULL )
     {
     vtkErrorMacro(<<"Write:Please specify an input!");
-    return;
+    return 0;
     }
-  if ( ! this->FileName && !this->FilePattern)
+  if ( !this->FileName && !this->FilePattern)
     {
     vtkErrorMacro(<<"Write:Please specify either a FileName or a file prefix and pattern");
     this->SetErrorCode(vtkErrorCode::NoFileNameError);
-    return;
+    return 0;
     }
   
   // Make sure the file name is allocated
@@ -224,17 +130,15 @@ void vtkImageWriter::Write()
             (this->FilePattern ? strlen(this->FilePattern) : 1) + 10];
   
   // Fill in image information.
-  this->GetInput()->UpdateInformation();
-  this->GetInput()->SetUpdateExtent(this->GetInput()->GetWholeExtent());
-  this->FileNumber = this->GetInput()->GetWholeExtent()[4];
+  int *wExt = inInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT());
+  this->FileNumber = wExt[4];
   this->MinimumFileNumber = this->MaximumFileNumber = this->FileNumber;
   this->FilesDeleted = 0;
-
+  
   // Write
-
   this->InvokeEvent(vtkCommand::StartEvent);
   this->UpdateProgress(0.0);
-  this->RecursiveWrite(2, this->GetInput(), NULL);
+  this->RecursiveWrite(2, input, NULL);
 
   if (this->ErrorCode == vtkErrorCode::OutOfDiskSpaceError)
     {
@@ -246,6 +150,19 @@ void vtkImageWriter::Write()
 
   delete [] this->InternalFileName;
   this->InternalFileName = NULL;
+
+  return 1;
+}
+
+//----------------------------------------------------------------------------
+// Writes all the data from the input.
+void vtkImageWriter::Write()
+{
+  // we always write, even if nothing has changed, so send a modified
+  this->Modified();
+  this->UpdateInformation();
+  this->GetInput()->SetUpdateExtent(this->GetInput()->GetWholeExtent());
+  this->Update();
 }
 
 //----------------------------------------------------------------------------
@@ -255,7 +172,6 @@ void vtkImageWriter::RecursiveWrite(int axis, vtkImageData *cache,
 {
   vtkImageData    *data;
   int             fileOpenedHere = 0;
-  int             *ext;
 
   // if we need to open another slice, do it
   if (!file && (axis + 1) == this->FileDimensionality)
@@ -318,7 +234,9 @@ void vtkImageWriter::RecursiveWrite(int axis, vtkImageData *cache,
   this->GetInput()->PropagateUpdateExtent();
 
   // just get the data and write it out
-  ext = cache->GetUpdateExtent();
+#ifndef NDEBUG
+  int *ext = cache->GetUpdateExtent();
+#endif
   vtkDebugMacro("Getting input extent: " << ext[0] << ", " << 
                 ext[1] << ", " << ext[2] << ", " << ext[3] << ", " << 
                 ext[4] << ", " << ext[5] << endl);
@@ -340,7 +258,6 @@ void vtkImageWriter::RecursiveWrite(int axis, vtkImageData *cache,
       }
     file->close();
     delete file;
-    file = NULL;
     }
   return;
 }
@@ -482,6 +399,13 @@ void vtkImageWriter::RecursiveWrite(int axis, vtkImageData *cache,
 }
   
 
+//----------------------------------------------------------------------------
+template <class T>
+unsigned long vtkImageWriterGetSize(T*)
+{
+  return sizeof(T);
+}
+
 //----------------------------------------------------------------------------
 // Writes a region in a file.  Subclasses can override this method
 // to produce a header. This method only hanldes 3d data (plus components).
@@ -507,36 +431,9 @@ void vtkImageWriter::WriteFile(ofstream *file, vtkImageData *data,
   // take into consideration the scalar type
   switch (data->GetScalarType())
     {
-    case VTK_DOUBLE:
-      rowLength = sizeof(double);
-      break;
-    case VTK_FLOAT:
-      rowLength = sizeof(float);
-      break;
-    case VTK_LONG:
-      rowLength = sizeof(long);
-      break;
-    case VTK_UNSIGNED_LONG:
-      rowLength = sizeof(unsigned long); 
-      break;
-    case VTK_INT:
-      rowLength = sizeof(int);
-      break;
-    case VTK_UNSIGNED_INT:
-      rowLength = sizeof(unsigned int); 
-      break;
-    case VTK_SHORT:
-      rowLength = sizeof(short);
-      break;
-    case VTK_UNSIGNED_SHORT:
-      rowLength = sizeof(unsigned short); 
-      break;
-    case VTK_CHAR:
-      rowLength = sizeof(char);
-      break;
-    case VTK_UNSIGNED_CHAR:
-      rowLength = sizeof(unsigned char); 
-      break;
+    vtkTemplateMacro(
+      rowLength = vtkImageWriterGetSize(static_cast<VTK_TT*>(0))
+      );
     default:
       vtkErrorMacro(<< "Execute: Unknown output ScalarType");
       return; 
@@ -597,7 +494,7 @@ void vtkImageWriter::DeleteFiles()
   
   if (this->FileName)
     {
-    unlink(this->FileName);
+    vtksys::SystemTools::RemoveFile(this->FileName);
     }
   else
     {
@@ -609,7 +506,7 @@ void vtkImageWriter::DeleteFiles()
       for (i = this->MinimumFileNumber; i <= this->MaximumFileNumber; i++)
         {
         sprintf(fileName, this->FilePattern, this->FilePrefix, i);
-        unlink(fileName);
+        vtksys::SystemTools::RemoveFile(fileName);
         }
       delete [] fileName;
       }
@@ -620,7 +517,7 @@ void vtkImageWriter::DeleteFiles()
       for (i = this->MinimumFileNumber; i <= this->MaximumFileNumber; i++)
         {
         sprintf(fileName, this->FilePattern, i);
-        unlink(fileName);
+        vtksys::SystemTools::RemoveFile(fileName);
         }
       delete [] fileName;
       }
