@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkMCubesReader.cxx,v $
+  Module:    vtkMCubesReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -18,14 +18,16 @@
 #include "vtkCellArray.h"
 #include "vtkFloatArray.h"
 #include "vtkMergePoints.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
+#include "vtkIncrementalPointLocator.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
 
-vtkCxxRevisionMacro(vtkMCubesReader, "$Revision: 1.63 $");
 vtkStandardNewMacro(vtkMCubesReader);
 
 // Construct object with FlipNormals turned off and Normals set to true.
@@ -44,6 +46,8 @@ vtkMCubesReader::vtkMCubesReader()
   this->HeaderSize = 0;
   this->FlipNormals = 0;
   this->Normals = 1;
+
+  this->SetNumberOfInputPorts(0);
 }
 
 vtkMCubesReader::~vtkMCubesReader()
@@ -63,10 +67,20 @@ vtkMCubesReader::~vtkMCubesReader()
     }
 }
 
-void vtkMCubesReader::Execute()
+int vtkMCubesReader::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
 {
+  // get the info object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the ouptut
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   FILE *fp;
-  FILE *limitp = NULL;
+  FILE *limitp;
   vtkPoints *newPts;
   vtkCellArray *newPolys;
   vtkFloatArray *newNormals = NULL;
@@ -78,7 +92,6 @@ void vtkMCubesReader::Execute()
   int numDegenerate=0;
   vtkIdType nodes[3];
   float direction, n[3], dummy[2];
-  vtkPolyData *output = this->GetOutput();
   int byteOrder = this->GetDataByteOrder();
   
   vtkDebugMacro(<<"Reading marching cubes file");
@@ -90,12 +103,12 @@ void vtkMCubesReader::Execute()
   if ( this->FileName == NULL )
     {
     vtkErrorMacro(<< "Please specify input FileName");
-    return;
+    return 0;
     }
   if ( (fp = fopen(this->FileName, "rb")) == NULL)
     {
     vtkErrorMacro(<< "File " << this->FileName << " not found");
-    return;
+    return 0;
     }
 
   // Try to read limits file to get bounds. Otherwise, read data.
@@ -103,17 +116,40 @@ void vtkMCubesReader::Execute()
   (limitp = fopen (this->LimitsFileName, "rb")) != NULL &&
   stat (this->FileName, &buf) == 0 )
     {
+    bool errorOccurred = false;
+
     // skip first three pairs
     float fbounds[6];
-    fread (dummy, sizeof(float), 2, limitp);
-    fread (dummy, sizeof(float), 2, limitp);
-    fread (dummy, sizeof(float), 2, limitp);
+    if (fread (dummy, sizeof(float), 2, limitp) != 2)
+      {
+      errorOccurred = true;
+      }
+    else if (fread (dummy, sizeof(float), 2, limitp) != 2)
+      {
+      errorOccurred = true;
+      }
+    if (fread (dummy, sizeof(float), 2, limitp) != 2)
+      {
+      errorOccurred = true;
+      }
 
     // next three pairs are x, y, z limits
-    for (i = 0; i < 6; i++) 
+    for (i = 0; i < 6 && !errorOccurred; i++)
       {
-      fread (&fbounds[i], sizeof (float), 1, limitp);
+      if (fread (&fbounds[i], sizeof (float), 1, limitp) != 1)
+        {
+        errorOccurred = true;
+        }
       }
+
+    if (errorOccurred)
+      {
+      vtkErrorMacro ("MCubesReader error reading file: " << this->LimitsFileName
+                     << " Premature EOF while reading limits.");
+      fclose (limitp);
+      return 0;
+      }
+
     // do swapping if necc
     if (byteOrder == VTK_FILE_BYTE_ORDER_BIG_ENDIAN)
       {
@@ -139,8 +175,8 @@ void vtkMCubesReader::Execute()
     {
     fseek (fp, this->HeaderSize, 0);
     // cannot use vtkMath uninitialze bounds for this computation
-    bounds[0] = bounds[2] = bounds[4] = VTK_FLOAT_MAX;
-    bounds[1] = bounds[3] = bounds[5] = -VTK_FLOAT_MAX;
+    bounds[0] = bounds[2] = bounds[4] = VTK_DOUBLE_MAX;
+    bounds[1] = bounds[3] = bounds[5] = VTK_DOUBLE_MIN;
     for (i=0; fread(&point, sizeof(pointType), 1, fp); i++) 
       {
       // swap bytes if necc
@@ -267,11 +303,13 @@ void vtkMCubesReader::Execute()
     {
     this->Locator->Initialize(); //free storage
     }
+
+  return 1;
 }
 
 // Specify a spatial locator for merging points. By default, 
 // an instance of vtkMergePoints is used.
-void vtkMCubesReader::SetLocator(vtkPointLocator *locator)
+void vtkMCubesReader::SetLocator(vtkIncrementalPointLocator *locator)
 {
   if ( this->Locator == locator ) 
     {
@@ -399,7 +437,7 @@ void vtkMCubesReader::PrintSelf(ostream& os, vtkIndent indent)
 
 unsigned long int vtkMCubesReader::GetMTime()
 {
-  unsigned long mTime=this-> vtkPolyDataSource::GetMTime();
+  unsigned long mTime=this->Superclass::GetMTime();
   unsigned long time;
 
   if ( this->Locator != NULL )
