@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkMedicalImageReader2.h,v $
+  Module:    vtkMedicalImageReader2.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,50 +17,53 @@
 // vtkMedicalImageReader2 is a parent class for medical image readers.
 // It provides a place to store patient information that may be stored
 // in the image header.
-
 // .SECTION See Also
-// vtkImageReader2 vtkGESignaReader
+// vtkImageReader2 vtkGESignaReader vtkMedicalImageProperties
 
 #ifndef __vtkMedicalImageReader2_h
 #define __vtkMedicalImageReader2_h
 
 #include "vtkImageReader2.h"
 
+class vtkMedicalImageProperties;
 
 class VTK_IO_EXPORT vtkMedicalImageReader2 : public vtkImageReader2
 {
 public:
   static vtkMedicalImageReader2 *New();
-  vtkTypeRevisionMacro(vtkMedicalImageReader2,vtkImageReader2);
+  vtkTypeMacro(vtkMedicalImageReader2,vtkImageReader2);
   void PrintSelf(ostream& os, vtkIndent indent);   
 
   // Description:
-  // Methods to set/get the patient information data.
-  vtkSetStringMacro(PatientName);
-  vtkGetStringMacro(PatientName);
-  vtkSetStringMacro(PatientID);
-  vtkGetStringMacro(PatientID);
-  vtkSetStringMacro(Date);
-  vtkGetStringMacro(Date);
-  vtkSetStringMacro(Series);
-  vtkGetStringMacro(Series);
-  vtkSetStringMacro(Study);
-  vtkGetStringMacro(Study);
-  vtkSetStringMacro(ImageNumber);
-  vtkGetStringMacro(ImageNumber);
+  // Get the medical image properties object
+  vtkGetObjectMacro(MedicalImageProperties, vtkMedicalImageProperties);
+  
+  // Description:
+  // For backward compatibility, propagate calls to the MedicalImageProperties
+  // object.
+  virtual void SetPatientName(const char*);
+  virtual const char* GetPatientName();
+  virtual void SetPatientID(const char*);
+  virtual const char* GetPatientID();
+  virtual void SetDate(const char*);
+  virtual const char* GetDate();
+  virtual void SetSeries(const char*);
+  virtual const char* GetSeries();
+  virtual void SetStudy(const char*);
+  virtual const char* GetStudy();
+  virtual void SetImageNumber(const char*);
+  virtual const char* GetImageNumber();
+  virtual void SetModality(const char*);
+  virtual const char* GetModality();
   
 protected:
   vtkMedicalImageReader2();
   ~vtkMedicalImageReader2();
 
-  // store header info
-  char *PatientName;
-  char *PatientID;
-  char *Date;
-  char *ImageNumber;
-  char *Study;
-  char *Series;
-  
+  // Description:
+  // Medical Image properties
+  vtkMedicalImageProperties *MedicalImageProperties;
+
 private:
   vtkMedicalImageReader2(const vtkMedicalImageReader2&); // Not implemented.
   void operator=(const vtkMedicalImageReader2&); // Not implemented.
