@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkOBJReader.h,v $
+  Module:    vtkOBJReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -22,13 +22,13 @@
 #ifndef __vtkOBJReader_h
 #define __vtkOBJReader_h
 
-#include "vtkPolyDataSource.h"
+#include "vtkPolyDataAlgorithm.h"
 
-class VTK_IO_EXPORT vtkOBJReader : public vtkPolyDataSource 
+class VTK_IO_EXPORT vtkOBJReader : public vtkPolyDataAlgorithm 
 {
 public:
   static vtkOBJReader *New();
-  vtkTypeRevisionMacro(vtkOBJReader,vtkPolyDataSource);
+  vtkTypeMacro(vtkOBJReader,vtkPolyDataAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -40,7 +40,7 @@ protected:
   vtkOBJReader();
   ~vtkOBJReader();
   
-  void Execute();
+  int RequestData(vtkInformation *, vtkInformationVector **, vtkInformationVector *);
 
   char *FileName;
 private:
@@ -49,5 +49,3 @@ private:
 };
 
 #endif
-
-
