@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPLYReader.h,v $
+  Module:    vtkPLYReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,7 +16,7 @@
 // .SECTION Description
 // vtkPLYReader is a source object that reads polygonal data in
 // Stanford University PLY file format (see 
-// http://graphics.stanford.edu/data/3Dscanrep/). It requires that
+// http://graphics.stanford.edu/data/3Dscanrep). It requires that
 // the elements "vertex" and "face" are defined. The "vertex" element
 // must have the properties "x", "y", and "z". The "face" element must
 // have the property "vertex_indices" defined. Optionally, if the "face"
@@ -30,12 +30,12 @@
 #ifndef __vtkPLYReader_h
 #define __vtkPLYReader_h
 
-#include "vtkPolyDataSource.h"
+#include "vtkPolyDataAlgorithm.h"
 
-class VTK_IO_EXPORT vtkPLYReader : public vtkPolyDataSource 
+class VTK_IO_EXPORT vtkPLYReader : public vtkPolyDataAlgorithm 
 {
 public:
-  vtkTypeRevisionMacro(vtkPLYReader,vtkPolyDataSource);
+  vtkTypeMacro(vtkPLYReader,vtkPolyDataAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -47,18 +47,20 @@ public:
   vtkSetStringMacro(FileName);
   vtkGetStringMacro(FileName);
 
+  // Description:
+  // A simple, non-exhaustive check to see if a file is a valid ply file.
+  static int CanReadFile(const char *filename);
+
 protected:
   vtkPLYReader();
   ~vtkPLYReader();
 
   char *FileName;
 
-  void Execute();
+  int RequestData(vtkInformation *, vtkInformationVector **, vtkInformationVector *);
 private:
   vtkPLYReader(const vtkPLYReader&);  // Not implemented.
   void operator=(const vtkPLYReader&);  // Not implemented.
 };
 
 #endif
-
-
