@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPolyDataReader.h,v $
+  Module:    vtkPolyDataReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -36,32 +36,30 @@ class VTK_IO_EXPORT vtkPolyDataReader : public vtkDataReader
 {
 public:
   static vtkPolyDataReader *New();
-  vtkTypeRevisionMacro(vtkPolyDataReader,vtkDataReader);
+  vtkTypeMacro(vtkPolyDataReader,vtkDataReader);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
   // Get the output of this reader.
   vtkPolyData *GetOutput();
-  vtkPolyData *GetOutput(int idx)
-    {return (vtkPolyData *) this->vtkSource::GetOutput(idx); };
+  vtkPolyData *GetOutput(int idx);
   void SetOutput(vtkPolyData *output);
 
 protected:
   vtkPolyDataReader();
   ~vtkPolyDataReader();
 
-  void Execute();
+  int RequestData(vtkInformation *, vtkInformationVector **,
+                  vtkInformationVector *);
 
   // Update extent of PolyData is specified in pieces.  
   // Since all DataObjects should be able to set UpdateExent as pieces,
   // just copy output->UpdateExtent  all Inputs.
-  void ComputeInputUpdateExtents(vtkDataObject *output);
+  int RequestUpdateExtent(vtkInformation *, vtkInformationVector **,
+                          vtkInformationVector *);
   
-  // Used by streaming: The extent of the output being processed
-  // by the execute method. Set in the ComputeInputUpdateExtents method.
-  int ExecutePiece;
-  int ExecuteNumberOfPieces;
-  int ExecuteGhostLevel;
+  int FillOutputPortInformation(int, vtkInformation*);
+
 private:
   vtkPolyDataReader(const vtkPolyDataReader&);  // Not implemented.
   void operator=(const vtkPolyDataReader&);  // Not implemented.
