@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPolyDataWriter.cxx,v $
+  Module:    vtkPolyDataWriter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,6 +14,7 @@
 =========================================================================*/
 #include "vtkPolyDataWriter.h"
 
+#include "vtkInformation.h"
 #include "vtkObjectFactory.h"
 #include "vtkPolyData.h"
 
@@ -23,28 +24,8 @@
 # include <io.h> /* unlink */
 #endif
 
-vtkCxxRevisionMacro(vtkPolyDataWriter, "$Revision: 1.23 $");
 vtkStandardNewMacro(vtkPolyDataWriter);
 
-//----------------------------------------------------------------------------
-// Specify the input data or filter.
-void vtkPolyDataWriter::SetInput(vtkPolyData *input)
-{
-  this->vtkProcessObject::SetNthInput(0, input);
-}
-
-//----------------------------------------------------------------------------
-// Specify the input data or filter.
-vtkPolyData *vtkPolyDataWriter::GetInput()
-{
-  if (this->NumberOfInputs < 1)
-    {
-    return NULL;
-    }
-  
-  return (vtkPolyData *)(this->Inputs[0]);
-}
-
 void vtkPolyDataWriter::WriteData()
 {
   ostream *fp;
@@ -56,10 +37,18 @@ void vtkPolyDataWriter::WriteData()
     {
     if (fp)
       {
-      vtkErrorMacro("Ran out of disk space; deleting file: "
-                    << this->FileName);
-      this->CloseVTKFile(fp);
-      unlink(this->FileName);
+      if(this->FileName)
+        {
+        vtkErrorMacro("Ran out of disk space; deleting file: "
+                      << this->FileName);
+        this->CloseVTKFile(fp);
+        unlink(this->FileName);
+        }
+      else
+        {
+        this->CloseVTKFile(fp);
+        vtkErrorMacro("Could not read memory header. ");
+        }
       }
     return;
     }
@@ -70,85 +59,88 @@ void vtkPolyDataWriter::WriteData()
   
   //
   // Write data owned by the dataset
+  int errorOccured = 0;
   if (!this->WriteDataSetData(fp, input))
     {
-    vtkErrorMacro("Ran out of disk space; deleting file: " << this->FileName);
-    this->CloseVTKFile(fp);
-    unlink(this->FileName);
-    return;
+    errorOccured = 1;
     }
-
-  if (!this->WritePoints(fp, input->GetPoints()))
+  if (!errorOccured && !this->WritePoints(fp, input->GetPoints()))
     {
-    vtkErrorMacro("Ran out of disk space; deleting file: " << this->FileName);
-    this->CloseVTKFile(fp);
-    unlink(this->FileName);
-    return;
+    errorOccured = 1;
     }
 
-  if (input->GetVerts())
+  if (!errorOccured && input->GetVerts())
     {
     if (!this->WriteCells(fp, input->GetVerts(),"VERTICES"))
       {
-      vtkErrorMacro("Ran out of disk space; deleting file: "
-                    << this->FileName);
-      this->CloseVTKFile(fp);
-      unlink(this->FileName);
-      return;
+      errorOccured = 1;
       }
     }
-  if (input->GetLines())
+  if (!errorOccured && input->GetLines())
     {
     if (!this->WriteCells(fp, input->GetLines(),"LINES"))
       {
-      vtkErrorMacro("Ran out of disk space; deleting file: "
-                    << this->FileName);
-      this->CloseVTKFile(fp);
-      unlink(this->FileName);
-      return;
+      errorOccured = 1;
       }
     }
-  if (input->GetPolys())
+  if (!errorOccured && input->GetPolys())
     {
     if (!this->WriteCells(fp, input->GetPolys(),"POLYGONS"))
       {
-      vtkErrorMacro("Ran out of disk space; deleting file: "
-                    << this->FileName);
-      this->CloseVTKFile(fp);
-      unlink(this->FileName);
-      return;
+      errorOccured = 1;
       }
     }
-  if (input->GetStrips())
+  if (!errorOccured && input->GetStrips())
     {
     if (!this->WriteCells(fp, input->GetStrips(),"TRIANGLE_STRIPS"))
       {
-      vtkErrorMacro("Ran out of disk space; deleting file: "
-                    << this->FileName);
-      this->CloseVTKFile(fp);
-      unlink(this->FileName);
-      return;
+      errorOccured = 1;
       }
     }
 
-  if (!this->WriteCellData(fp, input))
+  if (!errorOccured && !this->WriteCellData(fp, input))
     {
-    vtkErrorMacro("Ran out of disk space; deleting file: " << this->FileName);
-    this->CloseVTKFile(fp);
-    unlink(this->FileName);
-    return;
+    errorOccured = 1;
     }
-  if (!this->WritePointData(fp, input))
+  if (!errorOccured && !this->WritePointData(fp, input))
     {
-    vtkErrorMacro("Ran out of disk space; deleting file: " << this->FileName);
-    this->CloseVTKFile(fp);
-    unlink(this->FileName);
-    return;
+    errorOccured = 1;
     }
 
+  if(errorOccured)
+    {
+    if(this->FileName)
+      {
+      vtkErrorMacro("Ran out of disk space; deleting file: " << this->FileName);
+      this->CloseVTKFile(fp);
+      unlink(this->FileName);
+      }
+    else
+      {
+      vtkErrorMacro("Error writing data set to memory");
+      this->CloseVTKFile(fp);
+      }
+    return;
+    }
   this->CloseVTKFile(fp);
 }
 
+int vtkPolyDataWriter::FillInputPortInformation(int, vtkInformation *info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkPolyData");
+  return 1;
+}
+
+vtkPolyData* vtkPolyDataWriter::GetInput()
+{
+  return vtkPolyData::SafeDownCast(this->Superclass::GetInput());
+}
+
+vtkPolyData* vtkPolyDataWriter::GetInput(int port)
+{
+  return vtkPolyData::SafeDownCast(this->Superclass::GetInput(port));
+}
+
 void vtkPolyDataWriter::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
