@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkSLCReader.cxx,v $
+  Module:    vtkSLCReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,13 +14,13 @@
 =========================================================================*/
 #include "vtkSLCReader.h"
 
+#include "vtkDataArray.h"
 #include "vtkImageData.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 
 #include <ctype.h>
 
-vtkCxxRevisionMacro(vtkSLCReader, "$Revision: 1.48 $");
 vtkStandardNewMacro(vtkSLCReader);
 
 // Constructor for a vtkSLCReader.
@@ -81,62 +81,130 @@ unsigned char* vtkSLCReader::Decode8BitData( unsigned char *in_ptr,
 
 
 // This will be needed when we make this an imaging filter.
-void vtkSLCReader::ExecuteInformation()
+int vtkSLCReader::RequestInformation (
+  vtkInformation       * request,
+  vtkInformationVector** inputVector,
+  vtkInformationVector * outputVector)
 {
   FILE *fp;
   int   temp;
   double f[3];
   int   size[3];
   int   magic_num;
-
   this->Error = 1;
-  vtkImageData *output = this->GetOutput();
 
   if (!this->FileName)
     {
     vtkErrorMacro(<<"A FileName must be specified.");
-    return;
+    return 0;
     }
 
   // Initialize
   if ((fp = fopen(this->FileName, "rb")) == NULL)
     {
     vtkErrorMacro(<< "File " << this->FileName << " not found");
-    return;
+    return 0;
     }
   this->FileDimensionality = 3;
-  fscanf( fp, "%d", &magic_num );
+  if (fscanf( fp, "%d", &magic_num ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to read magic number");
+    return 1;
+    }
   if( magic_num != 11111 )
     {
     vtkErrorMacro(<< "SLC magic number is not correct");
-    return;
+    return 1;
     }
 
   f[0] = f[1] = f[2] = 0.0;
-  output->SetOrigin(f);
+  this->SetDataOrigin(f);
 
-  fscanf( fp, "%d", size );
-  fscanf( fp, "%d", size+1 );
-  fscanf( fp, "%d", size+2 );
-  output->SetWholeExtent(0, size[0]-1, 0, size[1]-1, 0, size[2]-1);
+  if (fscanf( fp, "%d", size ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to read size[0]");
+    return 1;
+    }
+  if (fscanf( fp, "%d", size+1 ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to read size[1]");
+    return 1;
+    }
+  if (fscanf( fp, "%d", size+2 ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to read size[2]");
+    return 1;
+    }
+  this->SetDataExtent(0, size[0]-1, 0, size[1]-1, 0, size[2]-1);
 
   // Skip Over bits_per_voxel Field */
-  fscanf( fp, "%d",   &temp );
+  if (fscanf( fp, "%d",   &temp ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to skip over bits per pixel");
+    return 1;
+    }
 
-  fscanf( fp, "%lf", f );
-  fscanf( fp, "%lf", f+1 );
-  fscanf( fp, "%lf", f+2 );
-  output->SetSpacing(f);
+  if (fscanf( fp, "%lf", f ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to spacing[0]");
+    return 1;
+    }
+  if (fscanf( fp, "%lf", f+1 ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to spacing[1]");
+    return 1;
+    }
+  if (fscanf( fp, "%lf", f+2 ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to spacing[2]");
+    return 1;
+    }
+  this->SetDataSpacing(f);
 
   // Skip Over unit_type, data_origin, and data_modification 
-  fscanf( fp, "%d", &temp );
-  fscanf( fp, "%d", &temp );
-  fscanf( fp, "%d", &temp );
+  if (fscanf( fp, "%d", &temp ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to skip over unit type");
+    return 1;
+    }
+  if (fscanf( fp, "%d", &temp ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to skip over data origin");
+    return 1;
+    }
+  if (fscanf( fp, "%d", &temp ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to skip over data modification");
+    return 1;
+    }
 
-  output->SetScalarType(VTK_UNSIGNED_CHAR);
-  output->SetNumberOfScalarComponents(1);
+  this->SetDataScalarType(VTK_UNSIGNED_CHAR);
+  this->SetNumberOfScalarComponents(1);
 
   fclose( fp );
+  return this->Superclass::RequestInformation(request, inputVector, outputVector);
 }
           
 // Reads an SLC file and creates a vtkStructuredPoints dataset.
@@ -146,6 +214,11 @@ void vtkSLCReader::ExecuteData(vtkDataObject* )
   
   output->SetExtent(output->GetWholeExtent());
   output->AllocateScalars();
+
+  if (!output->GetPointData()->GetScalars())
+    {
+    return;
+    }
   output->GetPointData()->GetScalars()->SetName("SLCImage");
 
   FILE *fp;
@@ -153,7 +226,6 @@ void vtkSLCReader::ExecuteData(vtkDataObject* )
   int   temp;
   int   data_compression;
   int   plane_size;
-  int   volume_size;
   double f[3];
   int   size[3];
   int   magic_num;
@@ -161,8 +233,8 @@ void vtkSLCReader::ExecuteData(vtkDataObject* )
   int   icon_width, icon_height;
   int   compressed_size;
 
-  unsigned char *icon_ptr = NULL;
-  unsigned char *compressed_ptr = NULL;
+  unsigned char *icon_ptr;
+  unsigned char *compressed_ptr;
   unsigned char *scan_ptr = NULL;
 
   this->Error = 1;
@@ -180,7 +252,13 @@ void vtkSLCReader::ExecuteData(vtkDataObject* )
     return;
     }
 
-  fscanf( fp, "%d", &magic_num );
+  if (fscanf( fp, "%d", &magic_num ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to read magic number");
+    return;
+    }
   if( magic_num != 11111 )
     {
     vtkErrorMacro(<< "SLC magic number is not correct");
@@ -190,36 +268,121 @@ void vtkSLCReader::ExecuteData(vtkDataObject* )
   f[0] = f[1] = f[2] = 0.0;
   output->SetOrigin(f);
 
-  fscanf( fp, "%d", size );
-  fscanf( fp, "%d", size+1 );
-  fscanf( fp, "%d", size+2 );
+  if (fscanf( fp, "%d", size ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed read size[0]");
+    return;
+    }
+  if (fscanf( fp, "%d", size+1 ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed read size[1]");
+    return;
+    }
+  if (fscanf( fp, "%d", size+2 ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed read size[2]");
+    return;
+    }
   output->SetDimensions(size);
 
   // Skip Over bits_per_voxel Field */
-  fscanf( fp, "%d",   &temp );
-
-  fscanf( fp, "%lf", f );
-  fscanf( fp, "%lf", f+1 );
-  fscanf( fp, "%lf", f+2 );
+  if (fscanf( fp, "%d",   &temp ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to skip over bits per voxel");
+    return;
+    }
+  if (fscanf( fp, "%lf", f ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed read spacing[0]");
+    return;
+    }
+  if (fscanf( fp, "%lf", f+1 ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed read spacing[1]");
+    return;
+    }
+  if (fscanf( fp, "%lf", f+2 ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed read spacing[2]");
+    return;
+    }
   output->SetSpacing(f);
 
   // Skip Over unit_type, data_origin, and data_modification 
-  fscanf( fp, "%d", &temp );
-  fscanf( fp, "%d", &temp );
-  fscanf( fp, "%d", &temp );
+  if (fscanf( fp, "%d", &temp ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to skip over unit type");
+    return;
+    }
+  if (fscanf( fp, "%d", &temp ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to skip over data origin");
+    return;
+    }
+  if (fscanf( fp, "%d", &temp ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to skip over data modification");
+    return;
+    }
 
-  fscanf( fp, "%d\n", &data_compression );
+  if (fscanf( fp, "%d\n", &data_compression ) != 1)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to read data compression");
+    return;
+    }
 
   plane_size = size[0] * size[1];
-  volume_size = plane_size * size[2];
+#ifndef NDEBUG
+  int   volume_size = plane_size * size[2];
+#endif
 
   // Skip Over Icon
-  fscanf( fp, "%d %d X", &icon_width,  &icon_height );
+  if (fscanf( fp, "%d %d X", &icon_width,  &icon_height ) != 2)
+    {
+    vtkErrorMacro(
+      <<"Error reading file: " << this->FileName
+      << "Failed to skip over icon");
+    return;
+    }
   icon_ptr = new unsigned char[(icon_width*icon_height)];
 
-  fread( icon_ptr, 1, (icon_width*icon_height), fp );
-  fread( icon_ptr, 1, (icon_width*icon_height), fp );
-  fread( icon_ptr, 1, (icon_width*icon_height), fp );
+  if (fread( icon_ptr, (icon_width*icon_height), 1, fp ) != 1)
+    {
+    vtkErrorMacro ("SLCReader error reading file: " << this->FileName
+                   << " Premature EOF while reading icon.");
+    return;
+    }
+  if (fread( icon_ptr, (icon_width*icon_height), 1, fp ) != 1)
+    {
+    vtkErrorMacro ("SLCReader error reading file: " << this->FileName
+                   << " Premature EOF while reading icon.");
+    return;
+    }
+  if (fread( icon_ptr, (icon_width*icon_height), 1, fp ) != 1)
+    {
+    }
 
   delete [] icon_ptr;
 
@@ -257,7 +420,13 @@ void vtkSLCReader::ExecuteData(vtkDataObject* )
           delete [] scan_ptr;
           }
 
-        fscanf( fp, "%d X", &compressed_size );
+        if (fscanf( fp, "%d X", &compressed_size ) != 1)
+          {
+          vtkErrorMacro(
+            <<"Error reading file: " << this->FileName
+            << "Failed to read compressed size");
+          return;
+          }
 
         compressed_ptr = new unsigned char[compressed_size];
 
@@ -293,13 +462,17 @@ void vtkSLCReader::ExecuteData(vtkDataObject* )
 int vtkSLCReader::CanReadFile(const char* fname)
 {
   FILE* fp;
-  int   magic_num;
+  int   magic_num = 0;
   if ((fp = fopen(fname, "rb")) == NULL)
     {
     return 0;
     }
 
-  fscanf( fp, "%d", &magic_num );
+  if (fscanf( fp, "%d", &magic_num ) != 1)
+    {
+    fclose(fp);
+    return 0;
+    }
   if( magic_num != 11111 )
     {
     fclose(fp);
