@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkSTLReader.cxx,v $
+  Module:    vtkSTLReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,19 +17,27 @@
 #include "vtkByteSwap.h"
 #include "vtkCellArray.h"
 #include "vtkCellData.h"
+#include "vtkErrorCode.h"
 #include "vtkFloatArray.h"
+#include "vtkIncrementalPointLocator.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkMergePoints.h"
 #include "vtkObjectFactory.h"
 #include "vtkPolyData.h"
+#include "vtkSmartPointer.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include <ctype.h>
+#include <vtksys/SystemTools.hxx>
 
-vtkCxxRevisionMacro(vtkSTLReader, "$Revision: 1.69 $");
 vtkStandardNewMacro(vtkSTLReader);
 
 #define VTK_ASCII 0
 #define VTK_BINARY 1
 
+vtkCxxSetObjectMacro(vtkSTLReader,Locator,vtkIncrementalPointLocator);
+
 // Construct object with merging set to true.
 vtkSTLReader::vtkSTLReader()
 {
@@ -37,28 +45,23 @@ vtkSTLReader::vtkSTLReader()
   this->Merging = 1;
   this->ScalarTags = 0;
   this->Locator = NULL;
+
+  this->SetNumberOfInputPorts(0);
 }
 
 vtkSTLReader::~vtkSTLReader()
 {
-  if (this->FileName)
-    {
-    delete [] this->FileName;
-    }
-  if ( this->Locator )
-    {
-    this->Locator->UnRegister(this);
-    this->Locator = NULL;
-    }
+  this->SetFileName(0);
+  this->SetLocator(0);
 }
 
 // Overload standard modified time function. If locator is modified,
 // then this object is modified as well.
 unsigned long vtkSTLReader::GetMTime()
 {
-  unsigned long mTime1=this->vtkPolyDataSource::GetMTime();
+  unsigned long mTime1 = this->Superclass::GetMTime();
   unsigned long mTime2;
-  
+
   if (this->Locator)
     {
     mTime2 = this->Locator->GetMTime();
@@ -68,25 +71,34 @@ unsigned long vtkSTLReader::GetMTime()
   return mTime1;
 }
 
-
-void vtkSTLReader::Execute()
+int vtkSTLReader::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
 {
+  // get the info object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+  // get the ouptut
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
   FILE *fp;
   vtkPoints *newPts, *mergedPts;
   vtkCellArray *newPolys, *mergedPolys;
   vtkFloatArray *newScalars=0, *mergedScalars=0;
-  vtkPolyData *output = this->GetOutput();
-  
+
   // All of the data in the first piece.
-  if (output->GetUpdatePiece() > 0)
+  if (outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER()) > 0)
     {
-    return;
+    return 0;
     }
-  
-  if (!this->FileName)
+
+  if (!this->FileName || (this->FileName && (0==strlen(this->FileName))))
     {
     vtkErrorMacro(<<"A FileName must be specified.");
-    return;
+    this->SetErrorCode( vtkErrorCode::NoFileNameError );
+    return 0;
     }
 
   // Initialize
@@ -94,26 +106,27 @@ void vtkSTLReader::Execute()
   if ((fp = fopen(this->FileName, "r")) == NULL)
     {
     vtkErrorMacro(<< "File " << this->FileName << " not found");
-    return;
+    this->SetErrorCode( vtkErrorCode::CannotOpenFileError );
+    return 0;
     }
 
   newPts = vtkPoints::New();
-  newPts->Allocate(5000,10000);
   newPolys = vtkCellArray::New();
-  newPolys->Allocate(10000,20000);
 
   // Depending upon file type, read differently
   //
-  if ( this->GetSTLFileType(fp) == VTK_ASCII )
+  if ( this->GetSTLFileType(this->FileName) == VTK_ASCII )
     {
-    if (ScalarTags) 
+    newPts->Allocate(5000,10000);
+    newPolys->Allocate(10000,20000);
+    if (this->ScalarTags)
       {
       newScalars = vtkFloatArray::New();
       newScalars->Allocate(5000,10000);
       }
     if ( this->ReadASCIISTL(fp,newPts,newPolys,newScalars) )
       {
-      return;
+      return 1;
       }
     }
   else
@@ -122,25 +135,24 @@ void vtkSTLReader::Execute()
     fp = fopen(this->FileName, "rb");
     if ( this->ReadBinarySTL(fp,newPts,newPolys) )
       {
-      return;
+      return 1;
       }
     }
 
-  vtkDebugMacro(<< "Read: " 
-  << newPts->GetNumberOfPoints() << " points, "
-  << newPolys->GetNumberOfCells() << " triangles");
+  vtkDebugMacro(<< "Read: "
+                << newPts->GetNumberOfPoints() << " points, "
+                << newPolys->GetNumberOfCells() << " triangles");
 
   fclose(fp);
   //
   // If merging is on, create hash table and merge points/triangles.
   //
-//  if (0)
-  if ( this->Merging )  
+  if ( this->Merging )
     {
     int i;
     vtkIdType *pts = 0;
     vtkIdType nodes[3];
-    vtkIdType npts = 0;
+    vtkIdType npts;
     double x[3];
     int nextCell=0;
 
@@ -148,34 +160,35 @@ void vtkSTLReader::Execute()
     mergedPts->Allocate(newPts->GetNumberOfPoints()/2);
     mergedPolys = vtkCellArray::New();
     mergedPolys->Allocate(newPolys->GetSize());
-    if (newScalars) 
+    if (newScalars)
       {
       mergedScalars = vtkFloatArray::New();
       mergedScalars->Allocate(newPolys->GetSize());
       }
-    
-    if ( this->Locator == NULL )
+
+    vtkSmartPointer<vtkIncrementalPointLocator> locator = this->Locator;
+    if (this->Locator == NULL)
       {
-      this->CreateDefaultLocator();
+      locator.TakeReference(this->NewDefaultLocator());
       }
-    this->Locator->InitPointInsertion (mergedPts, newPts->GetBounds());
+    locator->InitPointInsertion (mergedPts, newPts->GetBounds());
 
     for (newPolys->InitTraversal(); newPolys->GetNextCell(npts,pts); )
       {
-      for (i=0; i < 3; i++) 
+      for (i=0; i < 3; i++)
         {
         newPts->GetPoint(pts[i],x);
-        this->Locator->InsertUniquePoint(x, nodes[i]);
+        locator->InsertUniquePoint(x, nodes[i]);
         }
 
       if ( nodes[0] != nodes[1] &&
-           nodes[0] != nodes[2] && 
+           nodes[0] != nodes[2] &&
            nodes[1] != nodes[2] )
         {
         mergedPolys->InsertNextCell(3,nodes);
-        if (newScalars) 
+        if (newScalars)
           {
-          mergedScalars->InsertNextValue(newScalars->GetValue(nextCell));       
+          mergedScalars->InsertNextValue(newScalars->GetValue(nextCell));
           }
         }
       nextCell++;
@@ -187,9 +200,9 @@ void vtkSTLReader::Execute()
       {
       newScalars->Delete();
       }
-      
-    vtkDebugMacro(<< "Merged to: " 
-    << mergedPts->GetNumberOfPoints() << " points, " 
+
+    vtkDebugMacro(<< "Merged to: "
+    << mergedPts->GetNumberOfPoints() << " points, "
     << mergedPolys->GetNumberOfCells() << " triangles");
     }
   else
@@ -209,6 +222,7 @@ void vtkSTLReader::Execute()
 
   if (mergedScalars) 
     {
+    mergedScalars->SetName("STLSolidLabeling");
     output->GetCellData()->SetScalars(mergedScalars);
     mergedScalars->Delete();
     }
@@ -219,9 +233,11 @@ void vtkSTLReader::Execute()
     }
 
   output->Squeeze();
+
+  return 1;
 }
 
-int vtkSTLReader::ReadBinarySTL(FILE *fp, vtkPoints *newPts, 
+int vtkSTLReader::ReadBinarySTL(FILE *fp, vtkPoints *newPts,
                                 vtkCellArray *newPolys)
 {
   int i, numTris;
@@ -236,8 +252,20 @@ int vtkSTLReader::ReadBinarySTL(FILE *fp, vtkPoints *newPts,
 
   //  File is read to obtain raw information as well as bounding box
   //
-  fread (header, 1, 80, fp);
-  fread (&ulint, 1, 4, fp);
+  if (fread (header, 1, 80, fp) != 80)
+    {
+    vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                   << " Premature EOF while reading header.");
+    fclose(fp);
+    return 0;
+    }
+  if (fread (&ulint, 1, 4, fp) != 4)
+    {
+    vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                   << " Premature EOF while reading header.");
+    fclose(fp);
+    return 0;
+    }
   vtkByteSwap::Swap4LE(&ulint);
 
   // Many .stl files contain bogus count.  Hence we will ignore and read 
@@ -249,9 +277,27 @@ int vtkSTLReader::ReadBinarySTL(FILE *fp, vtkPoints *newPts,
     << numTris << ")");
     }
 
+  // Verify the numTris with the length of the file
+  unsigned long ulFileLength = vtksys::SystemTools::FileLength(this->FileName);
+  ulFileLength -= (80 + 4); // 80 byte - header, 4 byte - tringle count
+  ulFileLength /= 50;       // 50 byte - twelve 32-bit-floating point numbers + 2 byte for attribute byte count
+
+  if (numTris < static_cast<int>(ulFileLength))
+    numTris = static_cast<int>(ulFileLength);
+
+  // now we can allocate the memory we need for this STL file
+  newPts->Allocate(numTris*3,10000);
+  newPolys->Allocate(numTris,20000);
+
   for ( i=0; fread(&facet,48,1,fp) > 0; i++ )
     {
-    fread(&ibuff2,2,1,fp); //read extra junk
+    if (fread(&ibuff2,2,1,fp) != 1) //read extra junk
+      {
+      vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                     << " Premature EOF while reading extra junk.");
+      fclose(fp);
+      return 0;
+      }
 
     vtkByteSwap::Swap4LE (facet.n);
     vtkByteSwap::Swap4LE (facet.n+1);
@@ -277,14 +323,14 @@ int vtkSTLReader::ReadBinarySTL(FILE *fp, vtkPoints *newPts,
     if ( (i % 5000) == 0 && i != 0 )
       {
       vtkDebugMacro(<< "triangle# " << i);
-      this->UpdateProgress((i%50000)/50000.0);
+      this->UpdateProgress(static_cast<double>(i)/numTris);
       }
     }
 
   return 0;
 }
 
-int vtkSTLReader::ReadASCIISTL(FILE *fp, vtkPoints *newPts, 
+int vtkSTLReader::ReadASCIISTL(FILE *fp, vtkPoints *newPts,
                                vtkCellArray *newPolys, vtkFloatArray *scalars)
 {
   char line[256];
@@ -297,28 +343,76 @@ int vtkSTLReader::ReadASCIISTL(FILE *fp, vtkPoints *newPts,
 
   //  Ingest header and junk to get to first vertex
   //
-  fgets (line, 255, fp);
+  if (!fgets (line, 255, fp))
+    {
+    vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                   << " Premature EOF while reading header.");
+    return 0;
+    }
 
   done = (fscanf(fp,"%s %*s %f %f %f\n", line, x, x+1, x+2)==EOF);
+  if ((strcmp(line, "COLOR") == 0) || (strcmp(line, "color") == 0))
+    {
+      done = (fscanf(fp,"%s %*s %f %f %f\n", line, x, x+1, x+2)==EOF);
+    }
 
   //  Go into loop, reading  facet normal and vertices
   //
-//  while (fscanf(fp,"%*s %*s %f %f %f\n", x, x+1, x+2)!=EOF) 
+//  while (fscanf(fp,"%*s %*s %f %f %f\n", x, x+1, x+2)!=EOF)
   while (!done)
     {
-//if (ctr>=253840) { 
+//if (ctr>=253840) {
 //    fprintf(stdout, "Reading record %d\n", ctr);
 //}
 //ctr += 7;
-    fgets (line, 255, fp);
-    fscanf (fp, "%*s %f %f %f\n", x,x+1,x+2);
+    if (!fgets (line, 255, fp))
+      {
+      vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                     << " Premature EOF while reading header.");
+      return 0;
+      }
+
+    if (fscanf (fp, "%*s %f %f %f\n", x,x+1,x+2) != 3)
+      {
+      vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                     << " Premature EOF while reading point.");
+      fclose(fp);
+      return 0;
+      }
+
     pts[0] = newPts->InsertNextPoint(x);
-    fscanf (fp, "%*s %f %f %f\n", x,x+1,x+2);
+    if (fscanf (fp, "%*s %f %f %f\n", x,x+1,x+2) != 3)
+      {
+      vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                     << " Premature EOF while reading point.");
+      fclose(fp);
+      return 0;
+      }
+
     pts[1] = newPts->InsertNextPoint(x);
-    fscanf (fp, "%*s %f %f %f\n", x,x+1,x+2);
+    if (fscanf (fp, "%*s %f %f %f\n", x,x+1,x+2) != 3)
+      {
+      vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                     << " Premature EOF while reading point.");
+      fclose(fp);
+      return 0;
+      }
+
     pts[2] = newPts->InsertNextPoint(x);
-    fgets (line, 255, fp); // end loop
-    fgets (line, 255, fp); // end facet
+    if (!fgets (line, 255, fp)) // end loop
+      {
+      vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                     << " Premature EOF while reading end loop.");
+      fclose(fp);
+      return 0;
+      }
+    if (!fgets (line, 255, fp)) // end facet
+      {
+      vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                     << " Premature EOF while reading end facet.");
+      fclose(fp);
+      return 0;
+      }
 
     newPolys->InsertNextCell(3,pts);
     if (scalars) 
@@ -335,15 +429,39 @@ int vtkSTLReader::ReadASCIISTL(FILE *fp, vtkPoints *newPts,
     if ((strcmp(line, "ENDSOLID") == 0) || (strcmp(line, "endsolid") == 0)) 
       {
       currentSolid++;
-      fgets(line, 255, fp);
+      if (!fgets(line, 255, fp))
+        {
+        vtkErrorMacro ("STLReader error reading file: " << this->FileName
+                       << " Premature EOF while reading solid.");
+        fclose(fp);
+        return 0;
+        }
+
       done = feof(fp);
       while ((strstr(line, "SOLID") == 0) && (strstr(line, "solid") == 0) && !done) 
         {
-        fgets(line, 255, fp);
+        if (!fgets(line, 255, fp))
+          {
+          // if fgets() returns an error, it may be due to the fact that the EOF
+          // is reached (BUG #13101) hence we test again.
+          done = feof(fp);
+          if (!done)
+            {
+            vtkErrorMacro ("STLReader error reading file: " << this->FileName
+              << " Premature EOF while reading end solid.");
+            fclose(fp);
+            return 0;
+            }
+          }
         done = feof(fp);
         }
 
       done = (fscanf(fp,"%s", line)==EOF);
+      if ((strstr(line, "COLOR") == 0) || (strstr(line, "color") == 0))
+        {
+          done = (fscanf(fp,"%f %f %f\n", x,x+1,x+2)==EOF);
+          done = (fscanf(fp,"%s", line)==EOF);
+        }
       }
     if (!done) {
     done = (fscanf(fp,"%*s %f %f %f\n", x, x+1, x+2)==EOF);
@@ -353,76 +471,53 @@ int vtkSTLReader::ReadASCIISTL(FILE *fp, vtkPoints *newPts,
   return 0;
 }
 
-int vtkSTLReader::GetSTLFileType(FILE *fp)
+int vtkSTLReader::GetSTLFileType(const char *filename)
 {
-  unsigned char header[256];
-  int type, i;
-  int numChars;
-
-  //  Read a little from the file to figure what type it is.
-  //
-  // skip 255 characters so we are past any first line comment */
-  numChars = static_cast<int>(fread ((unsigned char *)header, 1, 255, fp));
-  for (i = 0, type=VTK_ASCII; i< numChars && type == VTK_ASCII; i++) // don't test \0
+  int type;
+  vtksys::SystemTools::FileTypeEnum ft =
+    vtksys::SystemTools::DetectFileType(filename);
+  switch(ft)
     {
-    if (header[i] > 127)
-      {
-      type = VTK_BINARY;
-      }
+  case vtksys::SystemTools::FileTypeBinary:
+    type = VTK_BINARY;
+    break;
+  case vtksys::SystemTools::FileTypeText:
+    type = VTK_ASCII;
+    break;
+  case vtksys::SystemTools::FileTypeUnknown:
+    vtkWarningMacro( "File type not recognized attempting binary" );
+    type = VTK_BINARY;
+    break;
+  default:
+    vtkErrorMacro( "Case not handled" );
+    type = VTK_BINARY; // should not happen
     }
-
-  // Reset file for reading
-  //
-  rewind (fp);
   return type;
 }
 
 // Specify a spatial locator for merging points. By
 // default an instance of vtkMergePoints is used.
-void vtkSTLReader::SetLocator(vtkPointLocator *locator)
+vtkIncrementalPointLocator* vtkSTLReader::NewDefaultLocator()
 {
-  if ( this->Locator == locator ) 
-    {
-    return;
-    }
-  if ( this->Locator )
-    {
-    this->Locator->UnRegister(this);
-    this->Locator = NULL;
-    }
-  if ( locator )
-    {
-    locator->Register(this);
-    }
-  this->Locator = locator;
-  this->Modified();
-}
-
-void vtkSTLReader::CreateDefaultLocator()
-{
-  if ( this->Locator == NULL )
-    {
-    this->Locator = vtkMergePoints::New();
-    this->Locator->Register(this);
-    this->Locator->Delete();
-    }
+  return vtkMergePoints::New();
 }
 
 void vtkSTLReader::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
 
-  os << indent << "File Name: " 
+  os << indent << "File Name: "
      << (this->FileName ? this->FileName : "(none)") << "\n";
 
   os << indent << "Merging: " << (this->Merging ? "On\n" : "Off\n");
   os << indent << "ScalarTags: " << (this->ScalarTags ? "On\n" : "Off\n");
+  os << indent << "Locator: ";
   if ( this->Locator )
     {
-    os << indent << "Locator: " << this->Locator << "\n";
+    this->Locator->PrintSelf(os<<endl, indent.GetNextIndent());
     }
   else
     {
-    os << indent << "Locator: (none)\n";
+    os << "(none)\n";
     }
 }
