@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLDataSetWriter.cxx,v $
+  Module:    vtkXMLDataSetWriter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,19 +16,21 @@
 
 #include "vtkCallbackCommand.h"
 #include "vtkDataSet.h"
+#include "vtkHyperOctree.h"
 #include "vtkImageData.h"
+#include "vtkInformation.h"
 #include "vtkObjectFactory.h"
 #include "vtkPolyData.h"
 #include "vtkRectilinearGrid.h"
 #include "vtkStructuredGrid.h"
 #include "vtkUnstructuredGrid.h"
+#include "vtkXMLHyperOctreeWriter.h"
 #include "vtkXMLImageDataWriter.h"
 #include "vtkXMLPolyDataWriter.h"
 #include "vtkXMLRectilinearGridWriter.h"
 #include "vtkXMLStructuredGridWriter.h"
 #include "vtkXMLUnstructuredGridWriter.h"
 
-vtkCxxRevisionMacro(vtkXMLDataSetWriter, "$Revision: 1.6 $");
 vtkStandardNewMacro(vtkXMLDataSetWriter);
 
 //----------------------------------------------------------------------------
@@ -52,27 +54,16 @@ void vtkXMLDataSetWriter::PrintSelf(ostream& os, vtkIndent indent)
   this->Superclass::PrintSelf(os, indent);
 }
 
-//----------------------------------------------------------------------------
-void vtkXMLDataSetWriter::SetInput(vtkDataSet* input)
-{
-  this->vtkProcessObject::SetNthInput(0, input);
-}
-
 //----------------------------------------------------------------------------
 vtkDataSet* vtkXMLDataSetWriter::GetInput()
 {
-  if(this->NumberOfInputs < 1)
-    {
-    return 0;
-    }
-  
-  return static_cast<vtkDataSet*>(this->Inputs[0]);
+  return static_cast<vtkDataSet*>(this->Superclass::GetInput());
 }
 
 //----------------------------------------------------------------------------
 int vtkXMLDataSetWriter::WriteInternal()
 {  
-  vtkDataSet* input = this->GetInput();
+  vtkDataSet* input = vtkDataSet::SafeDownCast(this->GetInput());
   vtkXMLWriter* writer = 0;
   
   // Create a writer based on the data set type.
@@ -109,6 +100,12 @@ int vtkXMLDataSetWriter::WriteInternal()
       w->SetInput(static_cast<vtkPolyData*>(input));
       writer = w;
       } break;
+    case VTK_HYPER_OCTREE:
+      {
+      vtkXMLHyperOctreeWriter* w = vtkXMLHyperOctreeWriter::New();
+      w->SetInput(static_cast<vtkHyperOctree*>(input));
+      writer = w;
+      } break;
     }
   
   // Make sure we got a valid writer for the data set.
@@ -138,12 +135,6 @@ int vtkXMLDataSetWriter::WriteInternal()
   return result;
 }
 
-//----------------------------------------------------------------------------
-int vtkXMLDataSetWriter::WriteData()
-{
-  return 0;
-}
-
 //----------------------------------------------------------------------------
 const char* vtkXMLDataSetWriter::GetDataSetName()
 {
@@ -161,7 +152,7 @@ void vtkXMLDataSetWriter::ProgressCallbackFunction(vtkObject* caller,
                                                    unsigned long,
                                                    void* clientdata, void*)
 {
-  vtkProcessObject* w = vtkProcessObject::SafeDownCast(caller);
+  vtkAlgorithm* w = vtkAlgorithm::SafeDownCast(caller);
   if(w)
     {
     reinterpret_cast<vtkXMLDataSetWriter*>(clientdata)->ProgressCallback(w);
@@ -169,7 +160,7 @@ void vtkXMLDataSetWriter::ProgressCallbackFunction(vtkObject* caller,
 }
 
 //----------------------------------------------------------------------------
-void vtkXMLDataSetWriter::ProgressCallback(vtkProcessObject* w)
+void vtkXMLDataSetWriter::ProgressCallback(vtkAlgorithm* w)
 {
   float width = this->ProgressRange[1]-this->ProgressRange[0];
   float internalProgress = w->GetProgress();
@@ -180,3 +171,11 @@ void vtkXMLDataSetWriter::ProgressCallback(vtkProcessObject* w)
     w->SetAbortExecute(1);
     }
 }
+
+//----------------------------------------------------------------------------
+int vtkXMLDataSetWriter::FillInputPortInformation(
+  int vtkNotUsed(port), vtkInformation* info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
+}
