@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLImageDataReader.h,v $
+  Module:    vtkXMLImageDataReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -33,27 +33,40 @@ class vtkImageData;
 class VTK_IO_EXPORT vtkXMLImageDataReader : public vtkXMLStructuredDataReader
 {
 public:
-  vtkTypeRevisionMacro(vtkXMLImageDataReader,vtkXMLStructuredDataReader);
+  vtkTypeMacro(vtkXMLImageDataReader,vtkXMLStructuredDataReader);
   void PrintSelf(ostream& os, vtkIndent indent);
   static vtkXMLImageDataReader *New();
   
   // Description:
-  // Get/Set the reader's output.
-  void SetOutput(vtkImageData *output);
+  // Get the reader's output.
   vtkImageData *GetOutput();
   vtkImageData *GetOutput(int idx);
-  
+
+  // Description:
+  // For the specified port, copy the information this reader sets up in
+  // SetupOutputInformation to outInfo
+  virtual void CopyOutputInformation(vtkInformation *outInfo, int port);
+
 protected:
   vtkXMLImageDataReader();
   ~vtkXMLImageDataReader();  
   
   double Origin[3];
   double Spacing[3];
+  int PieceExtent[6];
   
   const char* GetDataSetName();
   void SetOutputExtent(int* extent);
+  virtual void SetupUpdateExtentInformation(vtkInformation *outInfo);
+ 
   int ReadPrimaryElement(vtkXMLDataElement* ePrimary);
-  void SetupOutputInformation();
+
+  // Setup the output's information.
+  void SetupOutputInformation(vtkInformation *outInfo);
+
+  virtual int FillOutputPortInformation(int, vtkInformation*);
+
+
   
 private:
   vtkXMLImageDataReader(const vtkXMLImageDataReader&);  // Not implemented.
