@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLImageDataWriter.h,v $
+  Module:    vtkXMLImageDataWriter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -34,13 +34,14 @@ class VTK_IO_EXPORT vtkXMLImageDataWriter : public vtkXMLStructuredDataWriter
 {
 public:
   static vtkXMLImageDataWriter* New();
-  vtkTypeRevisionMacro(vtkXMLImageDataWriter,vtkXMLStructuredDataWriter);
+  vtkTypeMacro(vtkXMLImageDataWriter,vtkXMLStructuredDataWriter);
   void PrintSelf(ostream& os, vtkIndent indent);
   
+  //BTX
   // Description:
   // Get/Set the writer's input.
-  void SetInput(vtkImageData* input);
   vtkImageData* GetInput();
+  //ETX
   
   // Description:
   // Get the default file extension for files written by this writer.
@@ -50,7 +51,10 @@ protected:
   vtkXMLImageDataWriter();
   ~vtkXMLImageDataWriter();  
   
-  void WritePrimaryElementAttributes();
+  // see algorithm for more info
+  virtual int FillInputPortInformation(int port, vtkInformation* info);
+
+  void WritePrimaryElementAttributes(ostream &os, vtkIndent indent);
   void GetInputExtent(int* extent);
   const char* GetDataSetName();
   
