@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPDataSetWriter.h,v $
+  Module:    vtkXMLPDataSetWriter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -31,19 +31,23 @@
 class VTK_IO_EXPORT vtkXMLPDataSetWriter : public vtkXMLPDataWriter
 {
 public:
-  vtkTypeRevisionMacro(vtkXMLPDataSetWriter,vtkXMLPDataWriter);
+  vtkTypeMacro(vtkXMLPDataSetWriter,vtkXMLPDataWriter);
   void PrintSelf(ostream& os, vtkIndent indent);
   static vtkXMLPDataSetWriter* New();
   
+  //BTX
   // Description:
   // Get/Set the writer's input.
-  void SetInput(vtkDataSet* input);
   vtkDataSet* GetInput();
+  //ETX
   
 protected:
   vtkXMLPDataSetWriter();
   ~vtkXMLPDataSetWriter();
   
+  // see algorithm for more info
+  virtual int FillInputPortInformation(int port, vtkInformation* info);
+
   // Override writing method from superclass.
   virtual int WriteInternal();
   
