@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPImageDataWriter.cxx,v $
+  Module:    vtkXMLPImageDataWriter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -13,12 +13,13 @@
 
 =========================================================================*/
 #include "vtkXMLPImageDataWriter.h"
-#include "vtkObjectFactory.h"
-#include "vtkXMLImageDataWriter.h"
+
 #include "vtkErrorCode.h"
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkObjectFactory.h"
+#include "vtkXMLImageDataWriter.h"
 
-vtkCxxRevisionMacro(vtkXMLPImageDataWriter, "$Revision: 1.3 $");
 vtkStandardNewMacro(vtkXMLPImageDataWriter);
 
 //----------------------------------------------------------------------------
@@ -37,21 +38,10 @@ void vtkXMLPImageDataWriter::PrintSelf(ostream& os, vtkIndent indent)
   this->Superclass::PrintSelf(os,indent);
 }
 
-//----------------------------------------------------------------------------
-void vtkXMLPImageDataWriter::SetInput(vtkImageData* input)
-{
-  this->vtkProcessObject::SetNthInput(0, input);
-}
-
 //----------------------------------------------------------------------------
 vtkImageData* vtkXMLPImageDataWriter::GetInput()
 {
-  if(this->NumberOfInputs < 1)
-    {
-    return 0;
-    }
-  
-  return static_cast<vtkImageData*>(this->Inputs[0]);
+  return static_cast<vtkImageData*>(this->Superclass::GetInput());
 }
 
 //----------------------------------------------------------------------------
@@ -67,9 +57,9 @@ const char* vtkXMLPImageDataWriter::GetDefaultFileExtension()
 }
 
 //----------------------------------------------------------------------------
-void vtkXMLPImageDataWriter::WritePrimaryElementAttributes()
+void vtkXMLPImageDataWriter::WritePrimaryElementAttributes(ostream &os, vtkIndent indent)
 {
-  this->Superclass::WritePrimaryElementAttributes();
+  this->Superclass::WritePrimaryElementAttributes(os, indent);
   if (this->ErrorCode == vtkErrorCode::OutOfDiskSpaceError)
     {
     return;
@@ -94,3 +84,11 @@ vtkXMLPImageDataWriter::CreateStructuredPieceWriter()
   pWriter->SetInput(this->GetInput());
   return pWriter;
 }
+
+//----------------------------------------------------------------------------
+int vtkXMLPImageDataWriter::FillInputPortInformation(
+  int vtkNotUsed(port), vtkInformation* info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkImageData");
+  return 1;
+}
