@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPPolyDataReader.cxx,v $
+  Module:    vtkXMLPPolyDataReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,19 +19,14 @@
 #include "vtkPolyData.h"
 #include "vtkUnsignedCharArray.h"
 #include "vtkCellArray.h"
+#include "vtkInformation.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
-vtkCxxRevisionMacro(vtkXMLPPolyDataReader, "$Revision: 1.4 $");
 vtkStandardNewMacro(vtkXMLPPolyDataReader);
 
 //----------------------------------------------------------------------------
 vtkXMLPPolyDataReader::vtkXMLPPolyDataReader()
 {
-  // Copied from vtkPolyDataReader constructor:
-  this->SetOutput(vtkPolyData::New());
-  // Releasing data for pipeline parallism.
-  // Filters will know it is empty. 
-  this->Outputs[0]->ReleaseData();
-  this->Outputs[0]->Delete();  
 }
 
 //----------------------------------------------------------------------------
@@ -45,26 +40,16 @@ void vtkXMLPPolyDataReader::PrintSelf(ostream& os, vtkIndent indent)
   this->Superclass::PrintSelf(os, indent);
 }
 
-//----------------------------------------------------------------------------
-void vtkXMLPPolyDataReader::SetOutput(vtkPolyData *output)
-{
-  this->Superclass::SetNthOutput(0, output);
-}
-
 //----------------------------------------------------------------------------
 vtkPolyData* vtkXMLPPolyDataReader::GetOutput()
 {
-  if(this->NumberOfOutputs < 1)
-    {
-    return 0;
-    }
-  return static_cast<vtkPolyData*>(this->Outputs[0]);
+  return this->GetOutput(0);
 }
 
 //----------------------------------------------------------------------------
 vtkPolyData* vtkXMLPPolyDataReader::GetOutput(int idx)
 {
-  return static_cast<vtkPolyData*>(this->Superclass::GetOutput(idx));
+  return vtkPolyData::SafeDownCast( this->GetOutputDataObject(idx) );
 }
 
 //----------------------------------------------------------------------------
@@ -78,7 +63,13 @@ void vtkXMLPPolyDataReader::GetOutputUpdateExtent(int& piece,
                                                   int& numberOfPieces,
                                                   int& ghostLevel)
 {
-  this->GetOutput()->GetUpdateExtent(piece, numberOfPieces, ghostLevel);
+  vtkInformation* outInfo = this->GetCurrentOutputInformation();
+  piece = outInfo->Get(
+      vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER());
+  numberOfPieces= outInfo->Get(
+      vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES());
+  ghostLevel = outInfo->Get(
+      vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS());
 }
 
 //----------------------------------------------------------------------------
@@ -171,7 +162,7 @@ void vtkXMLPPolyDataReader::SetupOutputData()
 {
   this->Superclass::SetupOutputData();
   
-  vtkPolyData* output = this->GetOutput();
+  vtkPolyData* output = vtkPolyData::SafeDownCast(this->GetCurrentOutput());
   
   // Setup the output's cell arrays.  
   vtkCellArray* outVerts = vtkCellArray::New();
@@ -207,7 +198,7 @@ int vtkXMLPPolyDataReader::ReadPieceData()
   
   vtkPointSet* ips = this->GetPieceInputAsPointSet(this->Piece);
   vtkPolyData* input = static_cast<vtkPolyData*>(ips);
-  vtkPolyData* output = this->GetOutput();
+  vtkPolyData* output = vtkPolyData::SafeDownCast(this->GetCurrentOutput());
   
   // Copy the Verts.
   this->CopyCellArray(this->TotalNumberOfVerts, input->GetVerts(),
@@ -280,3 +271,12 @@ vtkXMLDataReader* vtkXMLPPolyDataReader::CreatePieceReader()
 {
   return vtkXMLPolyDataReader::New();
 }
+
+
+//----------------------------------------------------------------------------
+int vtkXMLPPolyDataReader::FillOutputPortInformation(int,
+                                                 vtkInformation* info)
+{
+  info->Set(vtkDataObject::DATA_TYPE_NAME(), "vtkPolyData");
+  return 1;
+}
