@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPPolyDataWriter.cxx,v $
+  Module:    vtkXMLPPolyDataWriter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -13,11 +13,12 @@
 
 =========================================================================*/
 #include "vtkXMLPPolyDataWriter.h"
+
 #include "vtkObjectFactory.h"
 #include "vtkXMLPolyDataWriter.h"
 #include "vtkPolyData.h"
+#include "vtkInformation.h"
 
-vtkCxxRevisionMacro(vtkXMLPPolyDataWriter, "$Revision: 1.2 $");
 vtkStandardNewMacro(vtkXMLPPolyDataWriter);
 
 //----------------------------------------------------------------------------
@@ -36,21 +37,10 @@ void vtkXMLPPolyDataWriter::PrintSelf(ostream& os, vtkIndent indent)
   this->Superclass::PrintSelf(os,indent);
 }
 
-//----------------------------------------------------------------------------
-void vtkXMLPPolyDataWriter::SetInput(vtkPolyData* input)
-{
-  this->vtkProcessObject::SetNthInput(0, input);
-}
-
 //----------------------------------------------------------------------------
 vtkPolyData* vtkXMLPPolyDataWriter::GetInput()
 {
-  if(this->NumberOfInputs < 1)
-    {
-    return 0;
-    }
-  
-  return static_cast<vtkPolyData*>(this->Inputs[0]);
+  return static_cast<vtkPolyData*>(this->Superclass::GetInput());
 }
 
 //----------------------------------------------------------------------------
@@ -74,3 +64,11 @@ vtkXMLPPolyDataWriter::CreateUnstructuredPieceWriter()
   pWriter->SetInput(this->GetInput());
   return pWriter;
 }
+
+//----------------------------------------------------------------------------
+int vtkXMLPPolyDataWriter::FillInputPortInformation(
+  int vtkNotUsed(port), vtkInformation* info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkPolyData");
+  return 1;
+}
