@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPPolyDataWriter.h,v $
+  Module:    vtkXMLPPolyDataWriter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -34,13 +34,14 @@ class VTK_IO_EXPORT vtkXMLPPolyDataWriter : public vtkXMLPUnstructuredDataWriter
 {
 public:
   static vtkXMLPPolyDataWriter* New();
-  vtkTypeRevisionMacro(vtkXMLPPolyDataWriter,vtkXMLPUnstructuredDataWriter);
+  vtkTypeMacro(vtkXMLPPolyDataWriter,vtkXMLPUnstructuredDataWriter);
   void PrintSelf(ostream& os, vtkIndent indent);
-  
+
+  //BTX
   // Description:
   // Get/Set the writer's input.
-  void SetInput(vtkPolyData* input);
   vtkPolyData* GetInput();
+  //ETX
   
   // Description:
   // Get the default file extension for files written by this writer.
@@ -50,6 +51,9 @@ protected:
   vtkXMLPPolyDataWriter();
   ~vtkXMLPPolyDataWriter();
   
+  // see algorithm for more info
+  virtual int FillInputPortInformation(int port, vtkInformation* info);
+
   const char* GetDataSetName();
   vtkXMLUnstructuredDataWriter* CreateUnstructuredPieceWriter(); 
   
