@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPRectilinearGridReader.cxx,v $
+  Module:    vtkXMLPRectilinearGridReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,19 +19,14 @@
 #include "vtkRectilinearGrid.h"
 #include "vtkXMLDataElement.h"
 #include "vtkXMLRectilinearGridReader.h"
+#include "vtkInformation.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
-vtkCxxRevisionMacro(vtkXMLPRectilinearGridReader, "$Revision: 1.8 $");
 vtkStandardNewMacro(vtkXMLPRectilinearGridReader);
 
 //----------------------------------------------------------------------------
 vtkXMLPRectilinearGridReader::vtkXMLPRectilinearGridReader()
 {
-  // Copied from vtkRectilinearGridReader constructor:
-  this->SetOutput(vtkRectilinearGrid::New());
-  // Releasing data for pipeline parallism.
-  // Filters will know it is empty. 
-  this->Outputs[0]->ReleaseData();
-  this->Outputs[0]->Delete();
 }
 
 //----------------------------------------------------------------------------
@@ -46,25 +41,21 @@ void vtkXMLPRectilinearGridReader::PrintSelf(ostream& os, vtkIndent indent)
 }
 
 //----------------------------------------------------------------------------
-void vtkXMLPRectilinearGridReader::SetOutput(vtkRectilinearGrid *output)
+void vtkXMLPRectilinearGridReader::SetupEmptyOutput()
 {
-  this->Superclass::SetNthOutput(0, output);
+  this->GetCurrentOutput()->Initialize();
 }
 
 //----------------------------------------------------------------------------
 vtkRectilinearGrid* vtkXMLPRectilinearGridReader::GetOutput()
 {
-  if(this->NumberOfOutputs < 1)
-    {
-    return 0;
-    }
-  return static_cast<vtkRectilinearGrid*>(this->Outputs[0]);
+  return this->GetOutput(0);
 }
 
 //----------------------------------------------------------------------------
 vtkRectilinearGrid* vtkXMLPRectilinearGridReader::GetOutput(int idx)
 {
-  return static_cast<vtkRectilinearGrid*>(this->Superclass::GetOutput(idx));
+  return vtkRectilinearGrid::SafeDownCast( this->GetOutputDataObject(idx) );
 }
 
 //----------------------------------------------------------------------------
@@ -84,7 +75,7 @@ const char* vtkXMLPRectilinearGridReader::GetDataSetName()
 //----------------------------------------------------------------------------
 void vtkXMLPRectilinearGridReader::SetOutputExtent(int* extent)
 {
-  this->GetOutput()->SetExtent(extent);
+  vtkRectilinearGrid::SafeDownCast(this->GetCurrentOutput())->SetExtent(extent);
 }
 
 //----------------------------------------------------------------------------
@@ -117,7 +108,9 @@ vtkXMLPRectilinearGridReader::ReadPrimaryElement(vtkXMLDataElement* ePrimary)
   if(!this->PCoordinatesElement)
     {
     int extent[6];
-    this->GetOutput()->GetWholeExtent(extent);
+    vtkInformation* outInfo = this->GetCurrentOutputInformation();
+    outInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),
+                 extent);
     if((extent[0] <= extent[1]) && (extent[2] <= extent[3]) &&
        (extent[4] <= extent[5]))
       {
@@ -129,27 +122,39 @@ vtkXMLPRectilinearGridReader::ReadPrimaryElement(vtkXMLDataElement* ePrimary)
   return 1;
 }
 
+
 //----------------------------------------------------------------------------
-void vtkXMLPRectilinearGridReader::SetupOutputInformation()
+void vtkXMLPRectilinearGridReader::SetupOutputData()
 {
-  this->Superclass::SetupOutputInformation();  
-  vtkRectilinearGrid* output = this->GetOutput();  
+  this->Superclass::SetupOutputData();
   
   if(!this->PCoordinatesElement)
     {
     // Empty volume.
     return;
     }
-  
-  // Create the coordinate arrays.
+
+  // Allocate the coordinate arrays.
+  vtkRectilinearGrid* output = vtkRectilinearGrid::SafeDownCast(
+      this->GetCurrentOutput());  
+
   vtkXMLDataElement* xc = this->PCoordinatesElement->GetNestedElement(0);
   vtkXMLDataElement* yc = this->PCoordinatesElement->GetNestedElement(1);
   vtkXMLDataElement* zc = this->PCoordinatesElement->GetNestedElement(2);
-  vtkDataArray* x = this->CreateDataArray(xc);
-  vtkDataArray* y = this->CreateDataArray(yc);
-  vtkDataArray* z = this->CreateDataArray(zc);
+  
+  // Create the coordinate arrays (all are data arrays).
+  vtkAbstractArray* ax = this->CreateArray(xc); 
+  vtkAbstractArray* ay = this->CreateArray(yc);
+  vtkAbstractArray* az = this->CreateArray(zc);
+  
+  vtkDataArray* x = vtkDataArray::SafeDownCast(ax);
+  vtkDataArray* y = vtkDataArray::SafeDownCast(ay);
+  vtkDataArray* z = vtkDataArray::SafeDownCast(az);
   if(x && y && z)
     {
+    x->SetNumberOfTuples(this->PointDimensions[0]);
+    y->SetNumberOfTuples(this->PointDimensions[1]);
+    z->SetNumberOfTuples(this->PointDimensions[2]);
     output->SetXCoordinates(x);
     output->SetYCoordinates(y);
     output->SetZCoordinates(z);
@@ -159,25 +164,13 @@ void vtkXMLPRectilinearGridReader::SetupOutputInformation()
     }
   else
     {
-    if(x) { x->Delete(); }
-    if(y) { y->Delete(); }
-    if(z) { z->Delete(); }
-    this->InformationError = 1;
+    if (ax) { ax->Delete(); }
+    if (ay) { ay->Delete(); }
+    if (az) { az->Delete(); }
+    this->DataError = 1;
     }
 }
 
-//----------------------------------------------------------------------------
-void vtkXMLPRectilinearGridReader::SetupOutputData()
-{
-  this->Superclass::SetupOutputData();
-  
-  // Allocate the coordinate arrays.
-  vtkRectilinearGrid* output = this->GetOutput();  
-  output->GetXCoordinates()->SetNumberOfTuples(this->PointDimensions[0]);
-  output->GetYCoordinates()->SetNumberOfTuples(this->PointDimensions[1]);
-  output->GetZCoordinates()->SetNumberOfTuples(this->PointDimensions[2]);  
-}
-
 //----------------------------------------------------------------------------
 int vtkXMLPRectilinearGridReader::ReadPieceData()
 {
@@ -185,7 +178,8 @@ int vtkXMLPRectilinearGridReader::ReadPieceData()
   
   // Copy the coordinates arrays from the input piece.
   vtkRectilinearGrid* input = this->GetPieceInput(this->Piece);
-  vtkRectilinearGrid* output = this->GetOutput();
+  vtkRectilinearGrid* output = vtkRectilinearGrid::SafeDownCast(
+      this->GetCurrentOutput());
   this->CopySubCoordinates(this->SubPieceExtent, this->UpdateExtent,
                            this->SubExtent, input->GetXCoordinates(),
                            output->GetXCoordinates());
@@ -223,3 +217,12 @@ void vtkXMLPRectilinearGridReader::CopySubCoordinates(int* inBounds,
          inArray->GetVoidPointer(sourceStartIndex*components),
          length*tupleSize);
 }
+
+
+int vtkXMLPRectilinearGridReader::FillOutputPortInformation(int, vtkInformation *info)
+  {
+  info->Set(vtkDataObject::DATA_TYPE_NAME(), "vtkRectilinearGrid");
+  return 1;
+  }
+
+
