@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPRectilinearGridReader.h,v $
+  Module:    vtkXMLPRectilinearGridReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -33,13 +33,12 @@ class vtkRectilinearGrid;
 class VTK_IO_EXPORT vtkXMLPRectilinearGridReader : public vtkXMLPStructuredDataReader
 {
 public:
-  vtkTypeRevisionMacro(vtkXMLPRectilinearGridReader,vtkXMLPStructuredDataReader);
+  vtkTypeMacro(vtkXMLPRectilinearGridReader,vtkXMLPStructuredDataReader);
   void PrintSelf(ostream& os, vtkIndent indent);
   static vtkXMLPRectilinearGridReader *New();
   
   // Description:
-  // Get/Set the reader's output.
-  void SetOutput(vtkRectilinearGrid *output);
+  // Get the reader's output.
   vtkRectilinearGrid *GetOutput();
   vtkRectilinearGrid *GetOutput(int idx);
   
@@ -48,17 +47,18 @@ protected:
   ~vtkXMLPRectilinearGridReader();
   
   vtkRectilinearGrid* GetPieceInput(int index);
-  
+
+  void SetupEmptyOutput();
   const char* GetDataSetName();
   void SetOutputExtent(int* extent);
   void GetPieceInputExtent(int index, int* extent);
   int ReadPrimaryElement(vtkXMLDataElement* ePrimary);
-  void SetupOutputInformation();
   void SetupOutputData();
   int ReadPieceData();
   vtkXMLDataReader* CreatePieceReader();
   void CopySubCoordinates(int* inBounds, int* outBounds, int* subBounds,
                           vtkDataArray* inArray, vtkDataArray* outArray);
+  virtual int FillOutputPortInformation(int, vtkInformation*);
 
   // The PCoordinates element with coordinate information.
   vtkXMLDataElement* PCoordinatesElement;
