@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPRectilinearGridWriter.h,v $
+  Module:    vtkXMLPRectilinearGridWriter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -35,13 +35,14 @@ class VTK_IO_EXPORT vtkXMLPRectilinearGridWriter : public vtkXMLPStructuredDataW
 {
 public:
   static vtkXMLPRectilinearGridWriter* New();
-  vtkTypeRevisionMacro(vtkXMLPRectilinearGridWriter,vtkXMLPStructuredDataWriter);
+  vtkTypeMacro(vtkXMLPRectilinearGridWriter,vtkXMLPStructuredDataWriter);
   void PrintSelf(ostream& os, vtkIndent indent);
-  
+
+  //BTX
   // Description:
   // Get/Set the writer's input.
-  void SetInput(vtkRectilinearGrid* input);
   vtkRectilinearGrid* GetInput();
+  //ETX
   
   // Description:
   // Get the default file extension for files written by this writer.
@@ -51,6 +52,9 @@ protected:
   vtkXMLPRectilinearGridWriter();
   ~vtkXMLPRectilinearGridWriter();
   
+  // see algorithm for more info
+  virtual int FillInputPortInformation(int port, vtkInformation* info);
+
   const char* GetDataSetName();
   vtkXMLStructuredDataWriter* CreateStructuredPieceWriter(); 
   void WritePData(vtkIndent indent);
