@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPStructuredGridWriter.cxx,v $
+  Module:    vtkXMLPStructuredGridWriter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -13,12 +13,13 @@
 
 =========================================================================*/
 #include "vtkXMLPStructuredGridWriter.h"
-#include "vtkObjectFactory.h"
-#include "vtkXMLStructuredGridWriter.h"
-#include "vtkErrorCode.h"
-#include "vtkStructuredGrid.h"
 
-vtkCxxRevisionMacro(vtkXMLPStructuredGridWriter, "$Revision: 1.4 $");
+#include "vtkErrorCode.h"
+#include "vtkInformation.h"
+#include "vtkObjectFactory.h"
+#include "vtkStructuredGrid.h"
+#include "vtkXMLStructuredGridWriter.h"
+
 vtkStandardNewMacro(vtkXMLPStructuredGridWriter);
 
 //----------------------------------------------------------------------------
@@ -37,21 +38,11 @@ void vtkXMLPStructuredGridWriter::PrintSelf(ostream& os, vtkIndent indent)
   this->Superclass::PrintSelf(os,indent);
 }
 
-//----------------------------------------------------------------------------
-void vtkXMLPStructuredGridWriter::SetInput(vtkStructuredGrid* input)
-{
-  this->vtkProcessObject::SetNthInput(0, input);
-}
 
 //----------------------------------------------------------------------------
 vtkStructuredGrid* vtkXMLPStructuredGridWriter::GetInput()
 {
-  if(this->NumberOfInputs < 1)
-    {
-    return 0;
-    }
-  
-  return static_cast<vtkStructuredGrid*>(this->Inputs[0]);
+  return static_cast<vtkStructuredGrid*>(this->Superclass::GetInput());
 }
 
 //----------------------------------------------------------------------------
@@ -87,3 +78,10 @@ void vtkXMLPStructuredGridWriter::WritePData(vtkIndent indent)
   vtkStructuredGrid* input = this->GetInput();  
   this->WritePPoints(input->GetPoints(), indent);
 }
+
+int vtkXMLPStructuredGridWriter::FillInputPortInformation(
+  int vtkNotUsed(port), vtkInformation* info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkStructuredGrid");
+  return 1;
+}
