@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLPolyDataReader.h,v $
+  Module:    vtkXMLPolyDataReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -33,13 +33,12 @@ class vtkPolyData;
 class VTK_IO_EXPORT vtkXMLPolyDataReader : public vtkXMLUnstructuredDataReader
 {
 public:
-  vtkTypeRevisionMacro(vtkXMLPolyDataReader,vtkXMLUnstructuredDataReader);
+  vtkTypeMacro(vtkXMLPolyDataReader,vtkXMLUnstructuredDataReader);
   void PrintSelf(ostream& os, vtkIndent indent);  
   static vtkXMLPolyDataReader *New();
   
   // Description:
-  // Get/Set the reader's output.
-  void SetOutput(vtkPolyData *output);
+  // Get the reader's output.
   vtkPolyData *GetOutput();
   vtkPolyData *GetOutput(int idx);
   
@@ -66,12 +65,15 @@ protected:
   int ReadPieceData();
   
   // Read a data array whose tuples coorrespond to cells.
-  int ReadArrayForCells(vtkXMLDataElement* da, vtkDataArray* outArray);
+  virtual int ReadArrayForCells(vtkXMLDataElement* da, 
+    vtkAbstractArray* outArray);
   
   // Get the number of cells in the given piece.  Valid after
   // UpdateInformation.
   virtual vtkIdType GetNumberOfCellsInPiece(int piece);
 
+  virtual int FillOutputPortInformation(int, vtkInformation*);
+
   // The size of the UpdatePiece.
   int TotalNumberOfVerts;
   int TotalNumberOfLines;
@@ -91,7 +93,17 @@ protected:
   vtkIdType* NumberOfLines;
   vtkIdType* NumberOfStrips;
   vtkIdType* NumberOfPolys;
-  
+
+  // For TimeStep support
+  int VertsTimeStep;
+  unsigned long VertsOffset;
+  int LinesTimeStep;
+  unsigned long LinesOffset;
+  int StripsTimeStep;
+  unsigned long StripsOffset;
+  int PolysTimeStep;
+  unsigned long PolysOffset;
+
 private:
   vtkXMLPolyDataReader(const vtkXMLPolyDataReader&);  // Not implemented.
   void operator=(const vtkXMLPolyDataReader&);  // Not implemented.
