@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLReader.h,v $
+  Module:    vtkXMLReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -21,20 +21,22 @@
 #ifndef __vtkXMLReader_h
 #define __vtkXMLReader_h
 
-#include "vtkSource.h"
+#include "vtkAlgorithm.h"
 
+class vtkAbstractArray;
 class vtkCallbackCommand;
-class vtkDataArray;
 class vtkDataArraySelection;
 class vtkDataSet;
 class vtkDataSetAttributes;
 class vtkXMLDataElement;
 class vtkXMLDataParser;
+class vtkInformationVector;
+class vtkInformation;
 
-class VTK_IO_EXPORT vtkXMLReader : public vtkSource
+class VTK_IO_EXPORT vtkXMLReader : public vtkAlgorithm
 {
 public:
-  vtkTypeRevisionMacro(vtkXMLReader,vtkSource);
+  vtkTypeMacro(vtkXMLReader,vtkAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
   
   // Description:
@@ -76,19 +78,35 @@ public:
   int GetCellArrayStatus(const char* name);
   void SetPointArrayStatus(const char* name, int status);  
   void SetCellArrayStatus(const char* name, int status);  
+
+  // For the specified port, copy the information this reader sets up in
+  // SetupOutputInformation to outInfo
+  virtual void CopyOutputInformation(vtkInformation *vtkNotUsed(outInfo),
+                                   int vtkNotUsed(port)) {}
+
+  // Description:
+  // Which TimeStep to read.    
+  vtkSetMacro(TimeStep, int);
+  vtkGetMacro(TimeStep, int);
+ 
+  vtkGetMacro(NumberOfTimeSteps, int);
+  // Description:
+  // Which TimeStepRange to read
+  vtkGetVector2Macro(TimeStepRange, int);
+  vtkSetVector2Macro(TimeStepRange, int);
+
+  virtual int ProcessRequest(vtkInformation *request,
+                             vtkInformationVector **inputVector,
+                             vtkInformationVector *outputVector);
   
 protected:
   vtkXMLReader();
   ~vtkXMLReader();
-  
-  // Standard pipeline exectution methods.
-  void ExecuteInformation();
-  void ExecuteData(vtkDataObject* output);
-  
+   
   // Pipeline execution methods to be defined by subclass.  Called by
-  // corresponding Execute methods after appropriate setup has been
+  // corresponding RequestData methods after appropriate setup has been
   // done.
-  virtual void ReadXMLInformation();
+  virtual int ReadXMLInformation();
   virtual void ReadXMLData();
   
   // Get the name of the data set being read.
@@ -100,10 +118,14 @@ protected:
   // Setup the output with no data available.  Used in error cases.
   virtual void SetupEmptyOutput()=0;
   
-  // Setup the output's information and data without allocation.
-  virtual void SetupOutputInformation();
+  // Setup the output's information.
+  virtual void SetupOutputInformation(vtkInformation *vtkNotUsed(outInfo)) {}
   
-  // Setup the output's information and data with allocation.
+  // Setup the output's information for the update extent
+  virtual void SetupUpdateExtentInformation
+  (vtkInformation *vtkNotUsed(outInfo)) {}
+
+  // Setup the output's data with allocation.
   virtual void SetupOutputData();
   
   // Read the primary element from the file.  This is the element
@@ -114,26 +136,38 @@ protected:
   // VTKFile element.
   int ReadVTKFile(vtkXMLDataElement* eVTKFile);  
   
-  // Create a vtkDataArray from its cooresponding XML representation.
+  // Create a vtkAbstractArray from its cooresponding XML representation.
   // Does not allocate.
-  vtkDataArray* CreateDataArray(vtkXMLDataElement* da);
-  
+  vtkAbstractArray* CreateArray(vtkXMLDataElement* da);
+
+  // Create a vtkInformationKey from its coresponding XML representation.
+  // Stores it in the instance of vtkInformationProvided. Does not allocate.
+  int CreateInformationKey(vtkXMLDataElement *eInfoKey, vtkInformation *info);
+
   // Internal utility methods.
-  int OpenVTKFile();
-  void CloseVTKFile();
+  virtual int OpenVTKFile();
+  virtual void CloseVTKFile();
   virtual void CreateXMLParser();
   virtual void DestroyXMLParser();
   void SetupCompressor(const char* type);
   int CanReadFileVersionString(const char* version);
+
+  // Returns the major version for the file being read. -1 when invalid.
+  vtkGetMacro(FileMajorVersion, int);
+
+  // Returns the minor version for the file being read. -1 when invalid.
+  vtkGetMacro(FileMinorVersion, int);
   
   // Utility methods for subclasses.
   int IntersectExtents(int* extent1, int* extent2, int* result);
   int Min(int a, int b);
   int Max(int a, int b);
-  void ComputeDimensions(int* extent, int* dimensions, int isPoint);
-  void ComputeIncrements(int* extent, int* increments, int isPoint);
-  unsigned int GetStartTuple(int* extent, int* increments,
-                             int i, int j, int k);
+  void ComputePointDimensions(int* extent, int* dimensions);
+  void ComputePointIncrements(int* extent, vtkIdType* increments);
+  void ComputeCellDimensions(int* extent, int* dimensions);
+  void ComputeCellIncrements(int* extent, vtkIdType* increments);
+  vtkIdType GetStartTuple(int* extent, vtkIdType* increments,
+                          int i, int j, int k);
   void ReadAttributeIndices(vtkXMLDataElement* eDSA,
                             vtkDataSetAttributes* dsa);
   char** CreateStringArray(int numStrings);
@@ -142,7 +176,12 @@ protected:
   // Setup the data array selections for the input's set of arrays.
   void SetDataArraySelections(vtkXMLDataElement* eDSA,
                               vtkDataArraySelection* sel);
-  
+
+//BTX
+  int SetFieldDataInfo(vtkXMLDataElement *eDSA, int association,  
+  int numTuples, vtkInformationVector *(&infoVector));
+//ETX
+
   // Check whether the given array element is an enabled array.
   int PointDataArrayIsEnabled(vtkXMLDataElement* ePDA);
   int CellDataArrayIsEnabled(vtkXMLDataElement* eCDA);
@@ -153,7 +192,10 @@ protected:
   
   // The vtkXMLDataParser instance used to hide XML reading details.
   vtkXMLDataParser* XMLParser;
-  
+
+  // The FieldData element representation.
+  vtkXMLDataElement* FieldDataElement;
+
   // The input file's name.
   char* FileName;
   
@@ -168,27 +210,76 @@ protected:
   // modified.
   vtkCallbackCommand* SelectionObserver;
   
-  // Whether there was an error reading the file in ExecuteInformation.
+  // Whether there was an error reading the file in RequestInformation.
   int InformationError;
   
-  // Whether there was an error reading the file in ExecuteData.
+  // Whether there was an error reading the file in RequestData.
   int DataError;
   
-  // The index of the output on which ExecuteData is currently
-  // running.
-  int CurrentOutput;
-  
-  // The current range over which progress is moving.  This allows for
   // incrementally fine-tuned progress updates.
   virtual void GetProgressRange(float* range);
   virtual void SetProgressRange(float* range, int curStep, int numSteps);
   virtual void SetProgressRange(float* range, int curStep, float* fractions);
   virtual void UpdateProgressDiscrete(float progress);
   float ProgressRange[2];
+  
+  virtual int RequestData(vtkInformation *request,
+                          vtkInformationVector **inputVector,
+                          vtkInformationVector *outputVector);
+  virtual int RequestDataObject(vtkInformation *vtkNotUsed(request),
+                                vtkInformationVector **vtkNotUsed(inputVector),
+                                vtkInformationVector *vtkNotUsed(outputVector))
+    { return 1; }
+  virtual int RequestInformation(vtkInformation *request,
+                                 vtkInformationVector **inputVector,
+                                 vtkInformationVector *outputVector);
+  virtual int RequestUpdateExtentInformation
+  (vtkInformation *request,
+   vtkInformationVector **inputVector,
+   vtkInformationVector *outputVector);
 
+  vtkTimeStamp ReadMTime;
+
+  // Whether there was an error reading the XML.
+  int ReadError;
+
+  // For structured data keep track of dimensions empty of cells.  For
+  // unstructured data these are always zero.  This is used to support
+  // 1-D and 2-D cell data.
+  int AxesEmpty[3];
+
+  // The timestep currently being read.
+  int TimeStep;
+  int CurrentTimeStep;
+  int NumberOfTimeSteps;
+  void SetNumberOfTimeSteps(int num);
+  // buffer for reading timestep from the XML file the length is of 
+  // NumberOfTimeSteps and therefore is always long enough
+  int *TimeSteps; 
+  // Store the range of time steps
+  int TimeStepRange[2];
+
+  // Now we need to save what was the last time read for each kind of 
+  // data to avoid rereading it that is to say we need a var for 
+  // e.g. PointData/CellData/Points/Cells...
+  // See SubClass for details with member vars like PointsTimeStep/PointsOffset
+
+  // Helper function useful to know if a timestep is found in an array of timestep
+  static int IsTimeStepInArray(int timestep, int* timesteps, int length);
+
+  vtkDataObject* GetCurrentOutput();
+  vtkInformation* GetCurrentOutputInformation();
+  
 private:
   // The stream used to read the input if it is in a file.
   ifstream* FileStream;  
+  int TimeStepWasReadOnce;
+
+  int FileMajorVersion;
+  int FileMinorVersion;
+  
+  vtkDataObject* CurrentOutput;
+  vtkInformation* CurrentOutputInformation;
   
 private:
   vtkXMLReader(const vtkXMLReader&);  // Not implemented.
