@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLRectilinearGridWriter.cxx,v $
+  Module:    vtkXMLRectilinearGridWriter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -18,21 +18,26 @@
 #include "vtkErrorCode.h"
 #include "vtkExtentTranslator.h"
 #include "vtkFloatArray.h"
+#include "vtkInformation.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkRectilinearGrid.h"
+#define vtkOffsetsManager_DoNotInclude
+#include "vtkOffsetsManagerArray.h"
+#undef  vtkOffsetsManager_DoNotInclude
 
-vtkCxxRevisionMacro(vtkXMLRectilinearGridWriter, "$Revision: 1.6 $");
 vtkStandardNewMacro(vtkXMLRectilinearGridWriter);
 
 //----------------------------------------------------------------------------
 vtkXMLRectilinearGridWriter::vtkXMLRectilinearGridWriter()
 {
+  this->CoordinateOM = new OffsetsManagerArray;
 }
 
 //----------------------------------------------------------------------------
 vtkXMLRectilinearGridWriter::~vtkXMLRectilinearGridWriter()
 {
+  delete this->CoordinateOM;
 }
 
 //----------------------------------------------------------------------------
@@ -41,21 +46,10 @@ void vtkXMLRectilinearGridWriter::PrintSelf(ostream& os, vtkIndent indent)
   this->Superclass::PrintSelf(os,indent);
 }
 
-//----------------------------------------------------------------------------
-void vtkXMLRectilinearGridWriter::SetInput(vtkRectilinearGrid* input)
-{
-  this->vtkProcessObject::SetNthInput(0, input);
-}
-
 //----------------------------------------------------------------------------
 vtkRectilinearGrid* vtkXMLRectilinearGridWriter::GetInput()
 {
-  if(this->NumberOfInputs < 1)
-    {
-    return 0;
-    }
-  
-  return static_cast<vtkRectilinearGrid*>(this->Inputs[0]);
+  return static_cast<vtkRectilinearGrid*>(this->Superclass::GetInput());
 }
 
 //----------------------------------------------------------------------------
@@ -83,6 +77,8 @@ vtkXMLRectilinearGridWriter::CreateExactCoordinates(vtkDataArray* a, int xyz)
   int inExtent[6];
   int outExtent[6];
   this->GetInput()->GetExtent(inExtent);
+  this->ExtentTranslator->SetPiece(this->CurrentPiece);
+  this->ExtentTranslator->PieceToExtent();
   this->ExtentTranslator->GetExtent(outExtent);
   int* inBounds = inExtent+xyz*2;
   int* outBounds = outExtent+xyz*2;
@@ -116,24 +112,17 @@ vtkXMLRectilinearGridWriter::CreateExactCoordinates(vtkDataArray* a, int xyz)
 }
 
 //----------------------------------------------------------------------------
-int vtkXMLRectilinearGridWriter::WriteAppendedMode(vtkIndent indent)
+void vtkXMLRectilinearGridWriter::AllocatePositionArrays()
 {
-  int i;
-  this->CoordinatePositions = new unsigned long*[this->NumberOfPieces];
-  for(i=0;i < this->NumberOfPieces;++i) { this->CoordinatePositions[i] = 0; }
-  
-  int result = this->Superclass::WriteAppendedMode(indent);
-  
-  for(i=0;i < this->NumberOfPieces;++i)
-    {
-    if(this->CoordinatePositions[i])
-      {
-      delete [] this->CoordinatePositions[i];
-      }
-    }
-  delete [] this->CoordinatePositions;
+  this->Superclass::AllocatePositionArrays();
 
-  return result;
+  this->CoordinateOM->Allocate(this->NumberOfPieces);
+}
+
+//----------------------------------------------------------------------------
+void vtkXMLRectilinearGridWriter::DeletePositionArrays()
+{
+  this->Superclass::DeletePositionArrays();
 }
 
 //----------------------------------------------------------------------------
@@ -146,12 +135,10 @@ void vtkXMLRectilinearGridWriter::WriteAppendedPiece(int index,
     return;
     }
   
-  this->CoordinatePositions[index] =
-    this->WriteCoordinatesAppended(this->GetInput()->GetXCoordinates(),
-                                   this->GetInput()->GetYCoordinates(),
-                                   this->GetInput()->GetZCoordinates(),
-                                   indent);
-  
+  this->WriteCoordinatesAppended(this->GetInput()->GetXCoordinates(),
+                                 this->GetInput()->GetYCoordinates(),
+                                 this->GetInput()->GetZCoordinates(),
+                                 indent, &this->CoordinateOM->GetPiece(index));
 }
 
 //----------------------------------------------------------------------------
@@ -181,12 +168,13 @@ void vtkXMLRectilinearGridWriter::WriteAppendedPieceData(int index)
   this->WriteCoordinatesAppendedData(this->GetInput()->GetXCoordinates(),
                                      this->GetInput()->GetYCoordinates(),
                                      this->GetInput()->GetZCoordinates(),
-                                     this->CoordinatePositions[index]);
-  this->CoordinatePositions[index] = 0;
+                                     this->CurrentTimeIndex,
+                                     &this->CoordinateOM->GetPiece(index));
+  this->CoordinateOM->GetPiece(index).Allocate(0); //mark it invalid
 }
 
 //----------------------------------------------------------------------------
-void vtkXMLRectilinearGridWriter::WriteInlinePiece(int index, vtkIndent indent)
+void vtkXMLRectilinearGridWriter::WriteInlinePiece(vtkIndent indent)
 {
   // Split progress range by the approximate fractions of data written
   // by each step in this method.
@@ -199,7 +187,7 @@ void vtkXMLRectilinearGridWriter::WriteInlinePiece(int index, vtkIndent indent)
   this->SetProgressRange(progressRange, 0, fractions);
   
   // Let the superclass write its data.
-  this->Superclass::WriteInlinePiece(index, indent);
+  this->Superclass::WriteInlinePiece(indent);
   if (this->ErrorCode == vtkErrorCode::OutOfDiskSpaceError)
     {
     return;
@@ -219,6 +207,8 @@ void vtkXMLRectilinearGridWriter::WriteInlinePiece(int index, vtkIndent indent)
 void vtkXMLRectilinearGridWriter::CalculateSuperclassFraction(float* fractions)
 {
   int extent[6];
+  this->ExtentTranslator->SetPiece(this->CurrentPiece);
+  this->ExtentTranslator->PieceToExtent();
   this->ExtentTranslator->GetExtent(extent);
   int dims[3] = {extent[1]-extent[0]+1,
                  extent[3]-extent[2]+1,
@@ -241,3 +231,11 @@ void vtkXMLRectilinearGridWriter::CalculateSuperclassFraction(float* fractions)
   fractions[1] = fractions[0] + float(superclassPieceSize)/totalPieceSize;
   fractions[2] = 1;
 }
+
+//----------------------------------------------------------------------------
+int vtkXMLRectilinearGridWriter::FillInputPortInformation(
+  int vtkNotUsed(port), vtkInformation* info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkRectilinearGrid");
+  return 1;
+}
