@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLUnstructuredGridReader.h,v $
+  Module:    vtkXMLUnstructuredGridReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -29,20 +29,20 @@
 #include "vtkXMLUnstructuredDataReader.h"
 
 class vtkUnstructuredGrid;
+class vtkIdTypeArray;
 
 class VTK_IO_EXPORT vtkXMLUnstructuredGridReader : public vtkXMLUnstructuredDataReader
 {
 public:
-  vtkTypeRevisionMacro(vtkXMLUnstructuredGridReader,vtkXMLUnstructuredDataReader);
+  vtkTypeMacro(vtkXMLUnstructuredGridReader,vtkXMLUnstructuredDataReader);
   void PrintSelf(ostream& os, vtkIndent indent);  
   static vtkXMLUnstructuredGridReader *New();
   
   // Description:
-  // Get/Set the reader's output.
-  void SetOutput(vtkUnstructuredGrid *output);
+  // Get the reader's output.
   vtkUnstructuredGrid *GetOutput();
   vtkUnstructuredGrid *GetOutput(int idx);
-  
+
 protected:
   vtkXMLUnstructuredGridReader();
   ~vtkXMLUnstructuredGridReader();
@@ -58,13 +58,16 @@ protected:
   void SetupNextPiece();
   int ReadPieceData();
   
-  // Read a data array whose tuples coorrespond to cells.
-  int ReadArrayForCells(vtkXMLDataElement* da, vtkDataArray* outArray);
+  // Read a data array whose tuples correspond to cells.
+  virtual int ReadArrayForCells(vtkXMLDataElement* da, 
+    vtkAbstractArray* outArray);
   
   // Get the number of cells in the given piece.  Valid after
   // UpdateInformation.
   virtual vtkIdType GetNumberOfCellsInPiece(int piece);
 
+  virtual int FillOutputPortInformation(int, vtkInformation*);
+
   // The index of the cell in the output where the current piece
   // begins.
   vtkIdType StartCell;
@@ -73,6 +76,9 @@ protected:
   vtkXMLDataElement** CellElements;
   vtkIdType* NumberOfCells;
   
+  int CellsTimeStep;
+  unsigned long CellsOffset;
+
 private:
   vtkXMLUnstructuredGridReader(const vtkXMLUnstructuredGridReader&);  // Not implemented.
   void operator=(const vtkXMLUnstructuredGridReader&);  // Not implemented.
