@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkZLibDataCompressor.cxx,v $
+  Module:    vtkZLibDataCompressor.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,10 +14,8 @@
 =========================================================================*/
 #include "vtkZLibDataCompressor.h"
 #include "vtkObjectFactory.h"
+#include "vtk_zlib.h"
 
-#include <zlib.h>
-
-vtkCxxRevisionMacro(vtkZLibDataCompressor, "$Revision: 1.3 $");
 vtkStandardNewMacro(vtkZLibDataCompressor);
 
 //----------------------------------------------------------------------------
