@@ -7,29 +7,29 @@ package require vtk
 # Image pipeline
 
 vtkPNGReader reader
-reader SetFileName $VTK_DATA_ROOT/Data/fullhead15.png
+reader SetFileName "$VTK_DATA_ROOT/Data/fullhead15.png"
 
 vtkImageGaussianSmooth smooth
   smooth SetDimensionality 2
   smooth SetStandardDeviations 1 1
-  smooth SetInput [reader GetOutput]
+  smooth SetInputConnection [reader GetOutputPort]
 
 vtkImageAppendComponents imageAppend
   imageAppend AddInput [reader GetOutput]
   imageAppend AddInput [smooth GetOutput]
 
 vtkImageClip clip
-  clip SetInput [imageAppend GetOutput]
+  clip SetInputConnection [imageAppend GetOutputPort]
   clip SetOutputWholeExtent 0 255 0 255 20 22
 
 vtkImageAccumulate accum
-  accum SetInput [clip GetOutput]
+  accum SetInputConnection [clip GetOutputPort]
   accum SetComponentExtent 0 255 0 255 0 0
   accum SetComponentSpacing 12 12 0.0
 
 
 vtkImageViewer viewer
-	viewer SetInput [accum GetOutput]
+	viewer SetInputConnection [accum GetOutputPort]
 	viewer SetColorWindow 4
 	viewer SetColorLevel 2
 
