@@ -10,11 +10,11 @@ reader SetFilePrefix "$VTK_DATA_ROOT/Data/headsq/quarter"
 reader SetDataMask 0x7fff
 
 vtkImageRange3D range
-range SetInput [reader GetOutput]
+range SetInputConnection [reader GetOutputPort]
 range SetKernelSize 5 5 5
 
 vtkImageViewer viewer
-viewer SetInput [range GetOutput]
+viewer SetInputConnection [range GetOutputPort]
 viewer SetZSlice 22
 viewer SetColorWindow 1000
 viewer SetColorLevel 500
