@@ -1,7 +1,7 @@
 package require vtk
 
 # Shift and scale an image (in that order)
-# This filter is usefull for converting to a lower precision data type.
+# This filter is useful for converting to a lower precision data type.
 
 vtkImageReader reader
 [reader GetOutput] ReleaseDataFlagOff
@@ -11,23 +11,23 @@ reader SetFilePrefix "$VTK_DATA_ROOT/Data/headsq/quarter"
 reader SetDataMask 0x7fff
 
 vtkImageShiftScale shiftScale
-shiftScale SetInput [reader GetOutput]
+shiftScale SetInputConnection [reader GetOutputPort]
 shiftScale SetShift 0
 shiftScale SetScale 0.5
 shiftScale SetOutputScalarTypeToDouble
 
 vtkImageShiftScale shiftScale2
-shiftScale2 SetInput [shiftScale GetOutput]
+shiftScale2 SetInputConnection [shiftScale GetOutputPort]
 shiftScale2 SetShift 0
 shiftScale2 SetScale 2.0
 
 vtkImageMagnify mag
-  mag SetInput [shiftScale2 GetOutput]
+  mag SetInputConnection [shiftScale2 GetOutputPort]
   mag SetMagnificationFactors 4 4 1
   mag InterpolateOff
 
 vtkImageViewer viewer
-viewer SetInput [mag GetOutput]
+viewer SetInputConnection [mag GetOutputPort]
 viewer SetColorWindow 1024
 viewer SetColorLevel 1024
 
@@ -35,7 +35,7 @@ viewer SetColorLevel 1024
 source [file join [file dirname [info script]] WindowLevelInterface.tcl]
 
 
-vtkPNMWriter w
-w SetFileName D:/vtknew/vtk/graphics/examplesTcl/mace2.ppm
-w SetInput [shiftScale GetOutput]
+#vtkPNMWriter w
+#w SetFileName "D:/vtknew/vtk/graphics/examplesTcl/mace2.ppm"
+#w SetInputConnection [shiftScale GetOutputPort]
 #w Write
