@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkBooleanTexture.cxx,v $
+  Module:    vtkBooleanTexture.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,27 +15,18 @@
 #include "vtkBooleanTexture.h"
 
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkPointData.h"
 #include "vtkUnsignedCharArray.h"
 
-vtkCxxRevisionMacro(vtkBooleanTexture, "$Revision: 1.37 $");
 vtkStandardNewMacro(vtkBooleanTexture);
 
-//-----  This hack needed to compile using gcc3 on OSX until new stdc++.dylib
-#ifdef __APPLE_CC__
-extern "C"
-{
-  void oft_initImaging() 
-  {
-  extern void _ZNSt8ios_base4InitC4Ev();
-  _ZNSt8ios_base4InitC4Ev();
-  }
-}
-#endif
-
 vtkBooleanTexture::vtkBooleanTexture()
 {
+  this->SetNumberOfInputPorts(0);
   this->Thickness = 0;
 
   this->XSize = this->YSize = 12;
@@ -52,13 +43,25 @@ vtkBooleanTexture::vtkBooleanTexture()
 }
 
 //----------------------------------------------------------------------------
-void vtkBooleanTexture::ExecuteInformation()
+int vtkBooleanTexture::RequestInformation (
+  vtkInformation * vtkNotUsed(request),
+  vtkInformationVector** vtkNotUsed( inputVector ),
+  vtkInformationVector *outputVector)
 {
-  vtkImageData *output = this->GetOutput();
+  // get the info objects
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
   
-  output->SetWholeExtent(0,this->XSize -1, 0, this->YSize - 1, 0,0);
-  output->SetScalarType(VTK_UNSIGNED_CHAR);
-  output->SetNumberOfScalarComponents(2);
+  int wExt[6];
+  wExt[0] = 0;
+  wExt[1] = this->XSize - 1;
+  wExt[2] = 0;
+  wExt[3] = this->YSize - 1;
+  wExt[4] = 0;
+  wExt[5] = 0;
+  
+  outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),wExt,6);
+  vtkDataObject::SetPointDataActiveScalarInfo(outInfo, VTK_UNSIGNED_CHAR, 2);
+  return 1;
 }
 
 void vtkBooleanTexture::ExecuteData(vtkDataObject *outp)
@@ -78,10 +81,14 @@ void vtkBooleanTexture::ExecuteData(vtkDataObject *outp)
 //
 // Compute size of various regions
 //
-  midILower = (int) ((float)(this->XSize - 1) / 2.0 - this->Thickness / 2.0);
-  midJLower = (int) ((float)(this->YSize - 1) / 2.0 - this->Thickness / 2.0);
-  midIUpper = (int) ((float)(this->XSize - 1) / 2.0 + this->Thickness / 2.0);
-  midJUpper = (int) ((float)(this->YSize - 1) / 2.0 + this->Thickness / 2.0);
+  midILower = static_cast<int>((this->XSize - 1) / 2.0
+                               - this->Thickness / 2.0);
+  midJLower = static_cast<int>((this->YSize - 1) / 2.0
+                               - this->Thickness / 2.0);
+  midIUpper = static_cast<int>((this->XSize - 1) / 2.0
+                               + this->Thickness / 2.0);
+  midJUpper = static_cast<int>((this->YSize - 1) / 2.0
+                               + this->Thickness / 2.0);
 //
 // Create texture map
 //
