@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkGaussianSplatter.cxx,v $
+  Module:    vtkGaussianSplatter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,12 +16,14 @@
 
 #include "vtkDoubleArray.h"
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkPointData.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkGaussianSplatter, "$Revision: 1.54 $");
 vtkStandardNewMacro(vtkGaussianSplatter);
 
 // Construct object with dimensions=(50,50,50); automatic computation of 
@@ -56,21 +58,30 @@ vtkGaussianSplatter::vtkGaussianSplatter()
   this->NullValue = 0.0;
 }
 
-void vtkGaussianSplatter::ExecuteInformation()
+//----------------------------------------------------------------------------
+int vtkGaussianSplatter::RequestInformation (
+  vtkInformation * vtkNotUsed(request),
+  vtkInformationVector ** vtkNotUsed( inputVector ),
+  vtkInformationVector *outputVector)
 {
-  vtkImageData *output = this->GetOutput();
+  // get the info objects
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
 
   // use model bounds if set
+  this->Origin[0] = 0;
+  this->Origin[1] = 0;
+  this->Origin[2] = 0;
   if ( this->ModelBounds[0] < this->ModelBounds[1] &&
        this->ModelBounds[2] < this->ModelBounds[3] &&
        this->ModelBounds[4] < this->ModelBounds[5] )
     {
-    output->SetOrigin(this->ModelBounds[0],this->ModelBounds[2],
-                      this->ModelBounds[4]);
+    this->Origin[0] = this->ModelBounds[0];
+    this->Origin[1] = this->ModelBounds[2];
+    this->Origin[2] = this->ModelBounds[4];
     }
 
-  // Set volume origin and data spacing
-  output->GetOrigin(this->Origin);
+  outInfo->Set(vtkDataObject::ORIGIN(), this->Origin, 3);
+
   int i;  
   for (i=0; i<3; i++)
     {
@@ -81,28 +92,44 @@ void vtkGaussianSplatter::ExecuteInformation()
       this->Spacing[i] = 1.0;
       }
     }
-  output->SetSpacing(this->Spacing);
+  outInfo->Set(vtkDataObject::SPACING(),this->Spacing,3);
   
-  output->SetWholeExtent(0, this->SampleDimensions[0] - 1, 
-                         0, this->SampleDimensions[1] - 1, 
-                         0, this->SampleDimensions[2] - 1);
-  output->SetScalarType(VTK_DOUBLE);
-  output->SetNumberOfScalarComponents(1);
+  outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),
+               0, this->SampleDimensions[0] - 1, 
+               0, this->SampleDimensions[1] - 1, 
+               0, this->SampleDimensions[2] - 1);
+  vtkDataObject::SetPointDataActiveScalarInfo(outInfo, VTK_DOUBLE, 1);
+  return 1;
 }
 
-void vtkGaussianSplatter::ExecuteData(vtkDataObject *outp)
+//----------------------------------------------------------------------------
+int vtkGaussianSplatter::RequestData(
+  vtkInformation* vtkNotUsed( request ),
+  vtkInformationVector** inputVector,
+  vtkInformationVector* outputVector)
 {
+  // get the data object
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+  vtkImageData *output = vtkImageData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+  
+  output->SetExtent(
+    outInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT()));
+  output->AllocateScalars();
+  
   vtkIdType numPts, numNewPts, ptId, idx, i;
   int j, k;
   int min[3], max[3];
   vtkPointData *pd;
   vtkDataArray *inNormals=NULL;
-  vtkDataArray *inScalars=NULL;
   double loc[3], dist2, cx[3];
-  vtkImageData *output = this->AllocateOutputData(outp);
   vtkDoubleArray *newScalars = 
-    vtkDoubleArray::SafeDownCast(output->GetPointData()->GetScalars());
-  vtkDataSet *input= this->GetInput();
+    vtkDoubleArray::SafeDownCast(output->GetPointData()->GetScalars());  
+  newScalars->SetName("SplatterValues");
+  
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  vtkDataSet *input = vtkDataSet::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
   int sliceSize=this->SampleDimensions[0]*this->SampleDimensions[1];
   
   vtkDebugMacro(<< "Splatting data");
@@ -111,8 +138,8 @@ void vtkGaussianSplatter::ExecuteData(vtkDataObject *outp)
   //
   if ( (numPts=input->GetNumberOfPoints()) < 1 )
     {
-    vtkErrorMacro(<<"No points to splat!");
-    return;
+    vtkDebugMacro(<<"No points to splat!");
+    return 1;
     }
 
   //  Compute the radius of influence of the points.  If an
@@ -134,11 +161,20 @@ void vtkGaussianSplatter::ExecuteData(vtkDataObject *outp)
     }
 
   output->SetDimensions(this->GetSampleDimensions());
-  this->ComputeModelBounds();
+  this->ComputeModelBounds(input,output, outInfo);
+
+  //decide which array to splat, if any
+  pd = input->GetPointData();
+  int association = vtkDataObject::FIELD_ASSOCIATION_POINTS;
+  vtkDataArray *inScalars=this->GetInputArrayToProcess
+    (0, inputVector, association);
+  if (!inScalars)
+    {
+    inScalars = pd->GetScalars();
+    }
 
   //  Set up function pointers to sample functions
   //
-  pd = input->GetPointData();
   if ( this->NormalWarping && (inNormals=pd->GetNormals()) != NULL )
     {
     this->Sample = &vtkGaussianSplatter::EccentricGaussian;
@@ -148,7 +184,7 @@ void vtkGaussianSplatter::ExecuteData(vtkDataObject *outp)
     this->Sample = &vtkGaussianSplatter::Gaussian;
     }
 
-  if ( this->ScalarWarping && (inScalars=pd->GetScalars()) != NULL )
+  if ( this->ScalarWarping && inScalars != NULL )
     {
     this->SampleFactor = &vtkGaussianSplatter::ScalarSampling;
     }
@@ -170,7 +206,7 @@ void vtkGaussianSplatter::ExecuteData(vtkDataObject *outp)
     if ( ! (ptId % progressInterval) )
       {
       vtkDebugMacro(<<"Inserting point #" << ptId);
-      this->UpdateProgress ((double)ptId/numPts);
+      this->UpdateProgress (static_cast<double>(ptId)/numPts);
       abortExecute = this->GetAbortExecute();
       }
 
@@ -193,8 +229,8 @@ void vtkGaussianSplatter::ExecuteData(vtkDataObject *outp)
     // Determine splat footprint
     for (i=0; i<3; i++)
       {
-      min[i] = (int) floor((double)loc[i]-this->SplatDistance[i]);
-      max[i] = (int) ceil((double)loc[i]+this->SplatDistance[i]);
+      min[i] = static_cast<int>(floor(static_cast<double>(loc[i])-this->SplatDistance[i]));
+      max[i] = static_cast<int>(ceil(static_cast<double>(loc[i])+this->SplatDistance[i]));
       if ( min[i] < 0 )
         {
         min[i] = 0;
@@ -239,16 +275,19 @@ void vtkGaussianSplatter::ExecuteData(vtkDataObject *outp)
   // Update self and release memeory
   //
   delete [] this->Visited;
+
+  return 1;
 }
 
+//----------------------------------------------------------------------------
 // Compute the size of the sample bounding box automatically from the
 // input data.
-void vtkGaussianSplatter::ComputeModelBounds()
+void vtkGaussianSplatter::ComputeModelBounds(vtkDataSet *input, 
+                                             vtkImageData *output,
+                                             vtkInformation *outInfo)
 {
   double *bounds, maxDist;
   int i, adjustBounds=0;
-  vtkImageData *output = this->GetOutput();
-  vtkDataSet *input= this->GetInput();
   
   // compute model bounds if not set previously
   if ( this->ModelBounds[0] >= this->ModelBounds[1] ||
@@ -284,9 +323,11 @@ void vtkGaussianSplatter::ComputeModelBounds()
     }
 
   // Set volume origin and data spacing
-  output->SetOrigin(this->ModelBounds[0],this->ModelBounds[2],
-                    this->ModelBounds[4]);
-  output->GetOrigin(this->Origin);
+  outInfo->Set(vtkDataObject::ORIGIN(),
+               this->ModelBounds[0],this->ModelBounds[2],
+               this->ModelBounds[4]);
+  memcpy(this->Origin,outInfo->Get(vtkDataObject::ORIGIN()), sizeof(double)*3);
+  output->SetOrigin(this->Origin);
   
   for (i=0; i<3; i++)
     {
@@ -297,6 +338,7 @@ void vtkGaussianSplatter::ComputeModelBounds()
       this->Spacing[i] = 1.0;
       }
     }
+  outInfo->Set(vtkDataObject::SPACING(),this->Spacing,3);
   output->SetSpacing(this->Spacing);
   
   // Determine the splat propagation distance...used later
@@ -318,6 +360,7 @@ void vtkGaussianSplatter::SetSampleDimensions(int i, int j, int k)
   this->SetSampleDimensions(dim);
 }
 
+//----------------------------------------------------------------------------
 void vtkGaussianSplatter::SetSampleDimensions(int dim[3])
 {
   int dataDim, i;
@@ -358,6 +401,7 @@ void vtkGaussianSplatter::SetSampleDimensions(int dim[3])
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkGaussianSplatter::Cap(vtkDoubleArray *s)
 {
   int i,j,k;
@@ -365,7 +409,7 @@ void vtkGaussianSplatter::Cap(vtkDoubleArray *s)
   int d01=this->SampleDimensions[0]*this->SampleDimensions[1];
 
   // i-j planes
-  k = 0;
+  //k = 0;
   for (j=0; j<this->SampleDimensions[1]; j++)
     {
     for (i=0; i<this->SampleDimensions[0]; i++)
@@ -383,7 +427,7 @@ void vtkGaussianSplatter::Cap(vtkDoubleArray *s)
       }
     }
   // j-k planes
-  i = 0;
+  //i = 0;
   for (k=0; k<this->SampleDimensions[2]; k++)
     {
     for (j=0; j<this->SampleDimensions[1]; j++)
@@ -400,7 +444,7 @@ void vtkGaussianSplatter::Cap(vtkDoubleArray *s)
       }
     }
   // i-k planes
-  j = 0;
+  //j = 0;
   for (k=0; k<this->SampleDimensions[2]; k++)
     {
     for (i=0; i<this->SampleDimensions[0]; i++)
@@ -419,6 +463,7 @@ void vtkGaussianSplatter::Cap(vtkDoubleArray *s)
     }
 }
 
+//----------------------------------------------------------------------------
 //
 //  Gaussian sampling
 //
@@ -428,6 +473,7 @@ double vtkGaussianSplatter::Gaussian (double cx[3])
           (cx[2]-P[2])*(cx[2]-P[2]) );
 }
     
+//----------------------------------------------------------------------------
 //
 //  Ellipsoidal Gaussian sampling
 //
@@ -451,7 +497,7 @@ double vtkGaussianSplatter::EccentricGaussian (double cx[3])
       }
     else
       {
-      mag = sqrt((double)mag);
+      mag = sqrt(mag);
       }
     }
 
@@ -463,11 +509,13 @@ double vtkGaussianSplatter::EccentricGaussian (double cx[3])
   return (rxy2/this->Eccentricity2 + z2);
 }
     
+//----------------------------------------------------------------------------
 void vtkGaussianSplatter::SetScalar(int idx, double dist2, 
                                     vtkDoubleArray *newScalars)
 {
-  double v = (this->*SampleFactor)(this->S) * exp((double)
-            (this->ExponentFactor*(dist2)/(this->Radius2)));
+  double v = (this->*SampleFactor)(this->S) * exp(
+    static_cast<double>
+    (this->ExponentFactor*(dist2)/(this->Radius2)));
 
   if ( ! this->Visited[idx] )
     {
@@ -493,6 +541,7 @@ void vtkGaussianSplatter::SetScalar(int idx, double dist2,
     }//not first visit
 }
 
+//----------------------------------------------------------------------------
 const char *vtkGaussianSplatter::GetAccumulationModeAsString()
 {
   if ( this->AccumulationMode == VTK_ACCUMULATION_MODE_MIN )
@@ -509,6 +558,7 @@ const char *vtkGaussianSplatter::GetAccumulationModeAsString()
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkGaussianSplatter::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
@@ -546,3 +596,10 @@ void vtkGaussianSplatter::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Null Value: " << this->NullValue << "\n";
 }
 
+//----------------------------------------------------------------------------
+int vtkGaussianSplatter::FillInputPortInformation(
+  int vtkNotUsed(port), vtkInformation* info)
+{
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  return 1;
+}
