@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageCast.cxx,v $
+  Module:    vtkImageCast.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,14 +16,18 @@
 
 #include "vtkImageData.h"
 #include "vtkImageProgressIterator.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
-vtkCxxRevisionMacro(vtkImageCast, "$Revision: 1.45 $");
 vtkStandardNewMacro(vtkImageCast);
 
 //----------------------------------------------------------------------------
 vtkImageCast::vtkImageCast()
 {
+  this->SetNumberOfInputPorts(1);
+  this->SetNumberOfOutputPorts(1);
   this->OutputScalarType = VTK_FLOAT;
   this->ClampOverflow = 0;
 }
@@ -31,25 +35,15 @@ vtkImageCast::vtkImageCast()
 
 //----------------------------------------------------------------------------
 // Just change the Image type.
-void vtkImageCast::ExecuteInformation(vtkImageData *vtkNotUsed(inData), 
-                                      vtkImageData *outData)
+int vtkImageCast::RequestInformation (
+  vtkInformation       * vtkNotUsed( request ),
+  vtkInformationVector** vtkNotUsed( inputVector ),
+  vtkInformationVector * outputVector)
 {
-  outData->SetScalarType(this->OutputScalarType);
-}
-
-//----------------------------------------------------------------------------
-// The update method first checks to see is a cast is necessary.
-void vtkImageCast::UpdateData(vtkDataObject *data)
-{
-  
-  if (! this->GetInput() || ! this->GetOutput())
-    {
-    vtkErrorMacro("Update: Input or output is not set.");
-    return;
-    }
-  
-  // call the superclass update which will cause an execute.
-  this->vtkImageToImageFilter::UpdateData(data);
+  // get the info objects
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
+  vtkDataObject::SetPointDataActiveScalarInfo(outInfo, this->OutputScalarType, -1);
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -122,9 +116,10 @@ void vtkImageCastExecute(vtkImageCast *self,
 {
   switch (outData->GetScalarType())
     {
-    vtkTemplateMacro7(vtkImageCastExecute, self, 
-                      inData, outData, outExt, id,
-                      static_cast<T *>(0), static_cast<VTK_TT *>(0));
+    vtkTemplateMacro(vtkImageCastExecute(self, 
+                                         inData, outData, outExt, id,
+                                         static_cast<T *>(0), 
+                                         static_cast<VTK_TT *>(0)));
     default:
       vtkGenericWarningMacro("Execute: Unknown output ScalarType");
       return;
@@ -139,23 +134,23 @@ void vtkImageCastExecute(vtkImageCast *self,
 // algorithm to fill the output from the input.
 // It just executes a switch statement to call the correct function for
 // the regions data types.
-void vtkImageCast::ThreadedExecute(vtkImageData *inData, 
+void vtkImageCast::ThreadedExecute (vtkImageData *inData, 
                                    vtkImageData *outData,
                                    int outExt[6], int id)
 {
-  vtkDebugMacro(<< "Execute: inData = " << inData 
-                << ", outData = " << outData);
-  
   switch (inData->GetScalarType())
     {
-    vtkTemplateMacro6(vtkImageCastExecute, this, inData, 
-                      outData, outExt, id, static_cast<VTK_TT *>(0));
+    vtkTemplateMacro(
+      vtkImageCastExecute(this, inData, 
+                          outData, outExt, id, 
+                          static_cast<VTK_TT *>(0)));
     default:
       vtkErrorMacro(<< "Execute: Unknown input ScalarType");
       return;
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkImageCast::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
