@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageCityBlockDistance.h,v $
+  Module:    vtkImageCityBlockDistance.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -36,16 +36,21 @@ class VTK_IMAGING_EXPORT vtkImageCityBlockDistance : public vtkImageDecomposeFil
 {
 public:
   static vtkImageCityBlockDistance *New();
-  vtkTypeRevisionMacro(vtkImageCityBlockDistance,vtkImageDecomposeFilter);
+  vtkTypeMacro(vtkImageCityBlockDistance,vtkImageDecomposeFilter);
   
 protected:
   vtkImageCityBlockDistance();
   ~vtkImageCityBlockDistance() {};
 
-  void ComputeInputUpdateExtent(int inExt[6], int outExt[6]);
-  void IterativeExecuteData(vtkImageData *inData, vtkImageData *outData);
+  virtual int IterativeRequestUpdateExtent(vtkInformation* in,
+                                           vtkInformation* out);
+  virtual int IterativeRequestData(vtkInformation*,
+                                    vtkInformationVector**,
+                                    vtkInformationVector*);
 
-  void AllocateOutputScalars(vtkImageData *outData);
+  void AllocateOutputScalars(vtkImageData *outData, 
+                             int* updateExtent,
+                             int* wholeExtent);
 
 private:
   vtkImageCityBlockDistance(const vtkImageCityBlockDistance&);  // Not implemented.
