@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageClip.h,v $
+  Module:    vtkImageClip.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -27,18 +27,18 @@
 // I did not make this a subclass of in place filter because
 // the references on the data do not matter. I make no modifications
 // to the data.
-#include "vtkImageToImageFilter.h"
+#include "vtkImageAlgorithm.h"
 
-class VTK_IMAGING_EXPORT vtkImageClip : public vtkImageToImageFilter
+class VTK_IMAGING_EXPORT vtkImageClip : public vtkImageAlgorithm
 {
 public:
   static vtkImageClip *New();
-  vtkTypeRevisionMacro(vtkImageClip,vtkImageToImageFilter);
+  vtkTypeMacro(vtkImageClip,vtkImageAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
   // The whole extent of the output has to be set explicitly.
-  void SetOutputWholeExtent(int extent[6]);
+  void SetOutputWholeExtent(int extent[6], vtkInformation *outInfo=0);
   void SetOutputWholeExtent(int minX, int maxX, int minY, int maxY, 
                             int minZ, int maxZ);
   void GetOutputWholeExtent(int extent[6]);
@@ -69,13 +69,16 @@ protected:
 
   int ClipData;
   
-  void ExecuteInformation(vtkImageData *inData, vtkImageData *outData);
-  void ExecuteInformation(){this->vtkImageToImageFilter::ExecuteInformation();};
+  virtual int RequestInformation (vtkInformation *,
+                                  vtkInformationVector **,
+                                  vtkInformationVector *);
+
   void CopyData(vtkImageData *inData, vtkImageData *outData, int *ext);
 
-  int SplitExtentTmp(int piece, int numPieces, int *ext);
+  virtual int RequestData(vtkInformation *,
+                          vtkInformationVector **,
+                          vtkInformationVector *);
 
-  virtual void ExecuteData(vtkDataObject *out);
 private:
   vtkImageClip(const vtkImageClip&);  // Not implemented.
   void operator=(const vtkImageClip&);  // Not implemented.
