@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageContinuousDilate3D.cxx,v $
+  Module:    vtkImageContinuousDilate3D.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,13 +14,15 @@
 =========================================================================*/
 #include "vtkImageContinuousDilate3D.h"
 
+#include "vtkDataArray.h"
 #include "vtkImageData.h"
 #include "vtkImageEllipsoidSource.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
-#include "vtkDataArray.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
-vtkCxxRevisionMacro(vtkImageContinuousDilate3D, "$Revision: 1.26 $");
 vtkStandardNewMacro(vtkImageContinuousDilate3D);
 
 //----------------------------------------------------------------------------
@@ -38,7 +40,6 @@ vtkImageContinuousDilate3D::vtkImageContinuousDilate3D()
   this->SetKernelSize(1, 1, 1);
 }
 
-
 //----------------------------------------------------------------------------
 vtkImageContinuousDilate3D::~vtkImageContinuousDilate3D()
 {
@@ -49,16 +50,10 @@ vtkImageContinuousDilate3D::~vtkImageContinuousDilate3D()
     }
 }
 
-
 //----------------------------------------------------------------------------
 void vtkImageContinuousDilate3D::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
-  if (this->InputScalarsSelection)
-    {
-    os << indent << "InputScalarsSelection: " 
-       << this->InputScalarsSelection << endl;
-    }
 }
 
 //----------------------------------------------------------------------------
@@ -93,17 +88,20 @@ void vtkImageContinuousDilate3D::SetKernelSize(int size0, int size1, int size2)
     this->Ellipse->SetWholeExtent(0, this->KernelSize[0]-1, 
                                   0, this->KernelSize[1]-1, 
                                   0, this->KernelSize[2]-1);
-    this->Ellipse->SetCenter((float)(this->KernelSize[0]-1)*0.5,
-                             (float)(this->KernelSize[1]-1)*0.5,
-                             (float)(this->KernelSize[2]-1)*0.5);
-    this->Ellipse->SetRadius((float)(this->KernelSize[0])*0.5,
-                             (float)(this->KernelSize[1])*0.5,
-                             (float)(this->KernelSize[2])*0.5);
+    this->Ellipse->SetCenter(static_cast<float>(this->KernelSize[0]-1)*0.5,
+                             static_cast<float>(this->KernelSize[1]-1)*0.5,
+                             static_cast<float>(this->KernelSize[2]-1)*0.5);
+    this->Ellipse->SetRadius(static_cast<float>(this->KernelSize[0])*0.5,
+                             static_cast<float>(this->KernelSize[1])*0.5,
+                             static_cast<float>(this->KernelSize[2])*0.5);
 
     // make sure scalars have been allocated (needed if multithreaded is used)
-    this->Ellipse->GetOutput()->SetUpdateExtent(0, this->KernelSize[0]-1, 
-                                                0, this->KernelSize[1]-1, 
-                                                0, this->KernelSize[2]-1);
+    vtkInformation *ellipseOutInfo =
+      this->Ellipse->GetExecutive()->GetOutputInformation(0);
+    ellipseOutInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(),
+                        0, this->KernelSize[0]-1, 
+                        0, this->KernelSize[1]-1, 
+                        0, this->KernelSize[2]-1);
     this->Ellipse->GetOutput()->Update();
     }
 }
@@ -119,14 +117,15 @@ void vtkImageContinuousDilate3DExecute(vtkImageContinuousDilate3D *self,
                                        vtkImageData *inData, T *inPtr, 
                                        vtkImageData *outData, 
                                        int *outExt, T *outPtr, int id,
-                                       const char* inputScalars)
+                                       vtkDataArray *inArray,
+                                       vtkInformation *inInfo)
 {
   int *kernelMiddle, *kernelSize;
   // For looping though output (and input) pixels.
   int outMin0, outMax0, outMin1, outMax1, outMin2, outMax2;
   int outIdx0, outIdx1, outIdx2;
-  int inInc0, inInc1, inInc2;
-  int outInc0, outInc1, outInc2;
+  vtkIdType inInc0, inInc1, inInc2;
+  vtkIdType outInc0, outInc1, outInc2;
   T *inPtr0, *inPtr1, *inPtr2;
   T *outPtr0, *outPtr1, *outPtr2;
   int numComps, outIdxC;
@@ -136,31 +135,34 @@ void vtkImageContinuousDilate3DExecute(vtkImageContinuousDilate3D *self,
   T *hoodPtr0, *hoodPtr1, *hoodPtr2;
   // For looping through the mask.
   unsigned char *maskPtr, *maskPtr0, *maskPtr1, *maskPtr2;
-  int maskInc0, maskInc1, maskInc2;
+  vtkIdType maskInc0, maskInc1, maskInc2;
   // The extent of the whole input image
   int inImageMin0, inImageMin1, inImageMin2;
   int inImageMax0, inImageMax1, inImageMax2;
+  int inImageExt[6];
   // to compute the range
   T pixelMax;
   unsigned long count = 0;
   unsigned long target;
-  vtkDataArray *inArray;
   int *inExt;
 
-  inArray=inData->GetPointData()->GetScalars(inputScalars);
   inExt = inData->GetExtent();
 
   // Get information to march through data
   inData->GetIncrements(inInc0, inInc1, inInc2); 
-  self->GetInput()->GetUpdateExtent(inImageMin0, inImageMax0, inImageMin1,
-                                   inImageMax1, inImageMin2, inImageMax2);
+  inInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(), inImageExt);
+  inImageMin0 = inImageExt[0];
+  inImageMax0 = inImageExt[1];
+  inImageMin1 = inImageExt[2];
+  inImageMax1 = inImageExt[3];
+  inImageMin2 = inImageExt[4];
+  inImageMax2 = inImageExt[5];
   outData->GetIncrements(outInc0, outInc1, outInc2); 
   outMin0 = outExt[0];   outMax0 = outExt[1];
   outMin1 = outExt[2];   outMax1 = outExt[3];
   outMin2 = outExt[4];   outMax2 = outExt[5];
   numComps = outData->GetNumberOfScalarComponents();
-  
-  
+
   // Get ivars of this object (easier than making friends)
   kernelSize = self->GetKernelSize();
   kernelMiddle = self->GetKernelMiddle();
@@ -172,16 +174,16 @@ void vtkImageContinuousDilate3DExecute(vtkImageContinuousDilate3D *self,
   hoodMax2 = hoodMin2 + kernelSize[2] - 1;
 
   // Setup mask info
-  maskPtr = (unsigned char *)(mask->GetScalarPointer());
+  maskPtr = static_cast<unsigned char *>(mask->GetScalarPointer());
   mask->GetIncrements(maskInc0, maskInc1, maskInc2);
   
   // in and out should be marching through corresponding pixels.
-  inPtr = (T *)(inArray->GetVoidPointer((outMin0-inExt[0])*inInc0 +
-                                        (outMin1-inExt[2])*inInc1 +
-                                        (outMin2-inExt[4])*inInc2));
+  inPtr = static_cast<T *>(inArray->GetVoidPointer((outMin0-inExt[0])*inInc0 +
+                                                   (outMin1-inExt[2])*inInc1 +
+                                                   (outMin2-inExt[4])*inInc2));
 
-  target = (unsigned long)(numComps*(outMax2-outMin2+1)*
-                           (outMax1-outMin1+1)/50.0);
+  target = static_cast<unsigned long>(numComps*(outMax2-outMin2+1)*
+                                      (outMax1-outMin1+1)/50.0);
   target++;
   
   // loop through components
@@ -269,30 +271,40 @@ void vtkImageContinuousDilate3DExecute(vtkImageContinuousDilate3D *self,
     }
 }
 
-
-                
-
 //----------------------------------------------------------------------------
 // This method contains the first switch statement that calls the correct
 // templated function for the input and output Data types.
 // It handles image boundaries, so the image does not shrink.
-void vtkImageContinuousDilate3D::ThreadedExecute(vtkImageData *inData, 
-                                                 vtkImageData *outData, 
-                                                 int outExt[6], int id)
+void vtkImageContinuousDilate3D::ThreadedRequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *vtkNotUsed(outputVector),
+  vtkImageData ***inData, 
+  vtkImageData **outData, 
+  int outExt[6], int id)
 {
-  int inExt[6];
-  this->ComputeInputUpdateExtent(inExt,outExt);
-  void *inPtr;
-  void *outPtr = outData->GetScalarPointerForExtent(outExt);
-  vtkImageData *mask;
-  vtkDataArray *inArray;
+  // return if nothing to do
+  if (outExt[1] < outExt[0] ||
+      outExt[3] < outExt[2] ||
+      outExt[5] < outExt[4])
+    {
+    return;
+    }
 
-  inArray = inData->GetPointData()->GetScalars(this->InputScalarsSelection);
+  int inExt[6], wholeExt[6];
+
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+  inInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), wholeExt);
+  this->InternalRequestUpdateExtent(inExt,outExt,wholeExt);
+  void *inPtr;
+  void *outPtr = outData[0]->GetScalarPointerForExtent(outExt);
+  vtkImageData *mask;
+
+  vtkDataArray *inArray = this->GetInputArrayToProcess(0,inputVector);
   // Reset later.
   inPtr = inArray->GetVoidPointer(0);
 
   // Error checking on mask
-  this->Ellipse->GetOutput()->Update();
   mask = this->Ellipse->GetOutput();
   if (mask->GetScalarType() != VTK_UNSIGNED_CHAR)
     {
@@ -301,24 +313,35 @@ void vtkImageContinuousDilate3D::ThreadedExecute(vtkImageData *inData,
     }
 
   // this filter expects the output type to be same as input
-  if (outData->GetScalarType() != inArray->GetDataType())
+  if (outData[0]->GetScalarType() != inArray->GetDataType())
     {
     vtkErrorMacro(<< "Execute: output ScalarType, " 
-      << vtkImageScalarTypeNameMacro(outData->GetScalarType())
+      << vtkImageScalarTypeNameMacro(outData[0]->GetScalarType())
       << " must match input array data type");
     return;
     }
 
   switch (inArray->GetDataType())
     {
-    vtkTemplateMacro9(vtkImageContinuousDilate3DExecute, this, 
-                      mask, inData, (VTK_TT *)(inPtr), 
-                      outData, outExt, (VTK_TT *)(outPtr), id,
-                      this->InputScalarsSelection);
+    vtkTemplateMacro(
+      vtkImageContinuousDilate3DExecute(this, 
+                                        mask, inData[0][0], 
+                                        static_cast<VTK_TT *>(inPtr), 
+                                        outData[0], outExt, 
+                                        static_cast<VTK_TT *>(outPtr), id,
+                                        inArray, inInfo) );
     default:
       vtkErrorMacro(<< "Execute: Unknown ScalarType");
       return;
     }
 }
 
-
+//----------------------------------------------------------------------------
+int vtkImageContinuousDilate3D::RequestData(
+  vtkInformation *request,
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
+{
+  this->Ellipse->GetOutput()->Update();
+  return this->Superclass::RequestData(request, inputVector, outputVector);
+}
