@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageDecomposeFilter.cxx,v $
+  Module:    vtkImageDecomposeFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,7 +16,6 @@
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImageDecomposeFilter, "$Revision: 1.19 $");
 
 //----------------------------------------------------------------------------
 // Construct an instance of vtkImageDecomposeFilter fitler.
@@ -56,8 +55,10 @@ void vtkImageDecomposeFilter::SetDimensionality(int dim)
 
 
 //----------------------------------------------------------------------------
-void vtkImageDecomposeFilter::PermuteIncrements(int *increments, int &inc0, 
-                                                int &inc1, int &inc2)
+void vtkImageDecomposeFilter::PermuteIncrements(vtkIdType *increments,
+                                                vtkIdType &inc0, 
+                                                vtkIdType &inc1,
+                                                vtkIdType &inc2)
 {
   switch (this->Iteration)
     {
