@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageEllipsoidSource.cxx,v $
+  Module:    vtkImageEllipsoidSource.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -13,11 +13,13 @@
 
 =========================================================================*/
 #include "vtkImageEllipsoidSource.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include "vtkImageData.h"
 
-vtkCxxRevisionMacro(vtkImageEllipsoidSource, "$Revision: 1.27 $");
 vtkStandardNewMacro(vtkImageEllipsoidSource);
 
 //----------------------------------------------------------------------------
@@ -39,6 +41,7 @@ vtkImageEllipsoidSource::vtkImageEllipsoidSource()
   this->OutValue = 0.0;
   
   this->OutputScalarType = VTK_UNSIGNED_CHAR;
+  this->SetNumberOfInputPorts(0);
 }
 
 //----------------------------------------------------------------------------
@@ -101,35 +104,35 @@ void vtkImageEllipsoidSource::GetWholeExtent(int extent[6])
 }
 
 //----------------------------------------------------------------------------
-void vtkImageEllipsoidSource::ExecuteInformation()
+int vtkImageEllipsoidSource::RequestInformation (
+  vtkInformation * vtkNotUsed(request),
+  vtkInformationVector** vtkNotUsed( inputVector ),
+  vtkInformationVector *outputVector)
 {
-  vtkImageData *data = this->GetOutput();
+  // get the info objects
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
   
-  data->SetSpacing(1.0, 1.0, 1.0);
-  data->SetWholeExtent(this->WholeExtent);
-  data->SetNumberOfScalarComponents(1);
-  data->SetScalarType(this->OutputScalarType);
+  outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),
+               this->WholeExtent, 6);
+  vtkDataObject::SetPointDataActiveScalarInfo(outInfo, this->OutputScalarType, -1);
+  return 1;
 }
 
-
-
-
-
 template <class T>
 void vtkImageEllipsoidSourceExecute(vtkImageEllipsoidSource *self,
                                     vtkImageData *data, int ext[6], T *ptr)
 {
   int min0, max0;
   int idx0, idx1, idx2;
-  int inc0, inc1, inc2;
+  vtkIdType inc0, inc1, inc2;
   double s0, s1, s2, temp;
   T outVal, inVal;
   double *center, *radius;
   unsigned long count = 0;
   unsigned long target;
 
-  outVal = (T)(self->GetOutValue());
-  inVal = (T)(self->GetInValue());
+  outVal = static_cast<T>(self->GetOutValue());
+  inVal = static_cast<T>(self->GetInValue());
   center = self->GetCenter();
   radius = self->GetRadius();
 
@@ -137,7 +140,7 @@ void vtkImageEllipsoidSourceExecute(vtkImageEllipsoidSource *self,
   max0 = ext[1];
   data->GetContinuousIncrements(ext, inc0, inc1, inc2);
 
-  target = (unsigned long)((ext[5]-ext[4]+1)*(ext[3]-ext[2]+1)/50.0);
+  target = static_cast<unsigned long>((ext[5]-ext[4]+1)*(ext[3]-ext[2]+1)/50.0);
   target++;
 
   for (idx2 = ext[4]; idx2 <= ext[5]; ++idx2)
@@ -145,11 +148,11 @@ void vtkImageEllipsoidSourceExecute(vtkImageEllipsoidSource *self,
     // handle divide by zero
     if (radius[2] != 0.0)
       {
-      temp = ((double)idx2 - (double)(center[2])) / (double)(radius[2]);
+      temp = (static_cast<double>(idx2) - center[2]) / radius[2];
       }
     else
       {
-      if ((double)idx2 - center[2] == 0.0)
+      if (static_cast<double>(idx2) - center[2] == 0.0)
         {
         temp = 0.0;
         }
@@ -172,11 +175,11 @@ void vtkImageEllipsoidSourceExecute(vtkImageEllipsoidSource *self,
       // handle divide by zero
       if (radius[1] != 0.0)
         {
-        temp = ((double)idx1 - (double)(center[1])) / (double)(radius[1]);
+        temp = (static_cast<double>(idx1) - center[1]) / radius[1];
         }
       else
         {
-        if ((double)idx1 - center[1] == 0.0)
+        if (static_cast<double>(idx1) - center[1] == 0.0)
           {
           temp = 0.0;
           }
@@ -192,11 +195,11 @@ void vtkImageEllipsoidSourceExecute(vtkImageEllipsoidSource *self,
         // handle divide by zero
         if (radius[0] != 0.0)
           {
-          temp = ((double)idx0 - (double)(center[0])) / (double)(radius[0]);
+          temp = (static_cast<double>(idx0) - center[0]) / radius[0];
           }
         else
           {
-          if ((double)idx0 - center[0] == 0.0)
+          if (static_cast<double>(idx0) - center[0] == 0.0)
             {
             temp = 0.0;
             }
@@ -225,28 +228,32 @@ void vtkImageEllipsoidSourceExecute(vtkImageEllipsoidSource *self,
 }
 
 //----------------------------------------------------------------------------
-void vtkImageEllipsoidSource::ExecuteData(vtkDataObject *output)
+int vtkImageEllipsoidSource::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector ** vtkNotUsed( inputVector ),
+  vtkInformationVector *outputVector)
 {
-  int *extent;
-  void *ptr;
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);  
+  vtkImageData *data = vtkImageData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+  int extent[6];
+
+  outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(),extent);
   
-  vtkImageData *data = this->AllocateOutputData(output);
+  data->SetExtent(extent);
+  data->AllocateScalars();
   
-  extent = this->GetOutput()->GetUpdateExtent();
+  void *ptr;  
   ptr = data->GetScalarPointerForExtent(extent);
   
   switch (data->GetScalarType())
     {
-    vtkTemplateMacro4(vtkImageEllipsoidSourceExecute, this, data, 
-                      extent, (VTK_TT *)ptr);
+    vtkTemplateMacro(
+      vtkImageEllipsoidSourceExecute(this, data, extent,
+                                     static_cast<VTK_TT *>(ptr)));
     default:
       vtkErrorMacro("Execute: Unknown output ScalarType");
     }
+
+  return 1;
 }
-
-
-
-
-
-
-
