@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageEuclideanDistance.cxx,v $
+  Module:    vtkImageEuclideanDistance.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,11 +15,13 @@
 #include "vtkImageEuclideanDistance.h"
 
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImageEuclideanDistance, "$Revision: 1.16 $");
 vtkStandardNewMacro(vtkImageEuclideanDistance);
 
 //----------------------------------------------------------------------------
@@ -34,30 +36,23 @@ vtkImageEuclideanDistance::vtkImageEuclideanDistance()
 
 //----------------------------------------------------------------------------
 // This extent of the components changes to real and imaginary values.
-void vtkImageEuclideanDistance::ExecuteInformation(
-  vtkImageData *vtkNotUsed(input), vtkImageData *output)
+int vtkImageEuclideanDistance::IterativeRequestInformation(
+  vtkInformation* vtkNotUsed(input), vtkInformation* output)
 {
-  output->SetNumberOfScalarComponents(1);
-  output->SetScalarType(VTK_DOUBLE);
+  vtkDataObject::SetPointDataActiveScalarInfo(output, VTK_DOUBLE, 1);
+  return 1;
 }
 
 //----------------------------------------------------------------------------
 // This method tells the superclass that the whole input array is needed
 // to compute any output region.
-void vtkImageEuclideanDistance::ComputeInputUpdateExtent(int inExt[6], 
-                                                         int outExt[6])
+int vtkImageEuclideanDistance::IterativeRequestUpdateExtent(
+  vtkInformation* input, vtkInformation* vtkNotUsed(output) )
 {
-  memcpy(inExt, outExt, 6 * sizeof(int));
+  int *wExt = input->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT());
+  input->Set(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(),wExt,6);
 
-  // Assumes that the input update extent has been initialized to output ...
-  if ( this->GetInput() != NULL ) 
-    {
-    this->GetInput()->GetWholeExtent(inExt);
-    } 
-  else 
-    {
-    vtkErrorMacro( "Input is NULL" );
-    }
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -69,11 +64,11 @@ void vtkImageEuclideanDistanceCopyData(vtkImageEuclideanDistance *self,
                                        vtkImageData *outData, int outExt[6], 
                                        double *outPtr )
 {
-  int inInc0, inInc1, inInc2;
+  vtkIdType inInc0, inInc1, inInc2;
   TT *inPtr0, *inPtr1, *inPtr2;
 
   int outMin0, outMax0, outMin1, outMax1, outMin2, outMax2;
-  int outInc0, outInc1, outInc2;
+  vtkIdType outInc0, outInc1, outInc2;
   double *outPtr0, *outPtr1, *outPtr2;
   
   int idx0, idx1, idx2;
@@ -117,11 +112,11 @@ void vtkImageEuclideanDistanceInitialize(vtkImageEuclideanDistance *self,
                                          vtkImageData *outData, 
                                          int outExt[6], double *outPtr )
 {
-  int inInc0, inInc1, inInc2;
+  vtkIdType inInc0, inInc1, inInc2;
   T *inPtr0, *inPtr1, *inPtr2;
 
   int outMin0, outMax0, outMin1, outMax1, outMin2, outMax2;
-  int outInc0, outInc1, outInc2;
+  vtkIdType outInc0, outInc1, outInc2;
   double *outPtr0, *outPtr1, *outPtr2;
   
   int idx0, idx1, idx2;
@@ -169,9 +164,10 @@ void vtkImageEuclideanDistanceInitialize(vtkImageEuclideanDistance *self,
   else   
     // No initialization required. We just copy inData to outData.
     {
-    vtkImageEuclideanDistanceCopyData( self, 
-                                       inData, (T *)(inPtr), 
-                                       outData, outExt, (double *)(outPtr) );
+    vtkImageEuclideanDistanceCopyData( self,
+                                       inData, static_cast<T *>(inPtr),
+                                       outData, outExt,
+                                       static_cast<double *>(outPtr) );
     }
 }
 
@@ -190,7 +186,7 @@ void vtkImageEuclideanDistanceExecuteSaito(vtkImageEuclideanDistance *self,
 {
   
   int outMin0, outMax0, outMin1, outMax1, outMin2, outMax2;
-  int outInc0, outInc1, outInc2;
+  vtkIdType outInc0, outInc1, outInc2;
   double *outPtr0, *outPtr1, *outPtr2;
   int idx0, idx1, idx2, inSize0;
   double maxDist;
@@ -207,10 +203,10 @@ void vtkImageEuclideanDistanceExecuteSaito(vtkImageEuclideanDistance *self,
   inSize0 = outMax0 - outMin0 + 1;  
   maxDist = self->GetMaximumDistance();
 
-  buff= (double *)calloc(outMax0+1,sizeof(double));
+  buff= static_cast<double *>(calloc(outMax0+1,sizeof(double)));
     
   // precompute sq[]. Anisotropy is handled here by using Spacing information
-  sq = (double *)calloc(inSize0*2+2,sizeof(double));
+  sq = static_cast<double *>(calloc(inSize0*2+2,sizeof(double)));
   for(df=2*inSize0+1;df>inSize0;df--)
     {
     sq[df]=maxDist;
@@ -304,7 +300,7 @@ void vtkImageEuclideanDistanceExecuteSaito(vtkImageEuclideanDistance *self,
           if(a>0) {a--;}
           if(buff[idx0]>buffer+sq[1]) 
             {
-            b=(int)(floor)((((buff[idx0]-buffer)/spacing)-1)/2); 
+            b=static_cast<int>(floor((((buff[idx0]-buffer)/spacing)-1)/2));
             if((idx0+b)>outMax0) {b=(outMax0)-idx0;}
                       
             for(n=a;n<=b;n++) 
@@ -333,7 +329,7 @@ void vtkImageEuclideanDistanceExecuteSaito(vtkImageEuclideanDistance *self,
           if(a>0) {a--;}
           if(buff[idx0]>buffer+sq[1]) 
             {
-            b=(int)(floor)((((buff[idx0]-buffer)/spacing)-1)/2); 
+            b=static_cast<int>(floor((((buff[idx0]-buffer)/spacing)-1)/2));
             if((idx0-b)<outMin0) {b=idx0-outMin0;}
                     
             for(n=a;n<=b;n++) 
@@ -372,7 +368,7 @@ void vtkImageEuclideanDistanceExecuteSaitoCached(
 {
   
   int outMin0, outMax0, outMin1, outMax1, outMin2, outMax2;
-  int outInc0, outInc1, outInc2;
+  vtkIdType outInc0, outInc1, outInc2;
   double *outPtr0, *outPtr1, *outPtr2;
   //
   int idx0, idx1, idx2, inSize0;
@@ -393,11 +389,11 @@ void vtkImageEuclideanDistanceExecuteSaitoCached(
   inSize0 = outMax0 - outMin0 + 1;  
   maxDist = self->GetMaximumDistance();
 
-  buff= (double *)calloc(outMax0+1,sizeof(double));
-  temp= (double *)calloc(outMax0+1,sizeof(double));
+  buff= static_cast<double *>(calloc(outMax0+1,sizeof(double)));
+  temp= static_cast<double *>(calloc(outMax0+1,sizeof(double)));
     
   // precompute sq[]. Anisotropy is handled here by using Spacing information
-  sq = (double *)calloc(inSize0*2+2,sizeof(double));
+  sq = static_cast<double *>(calloc(inSize0*2+2,sizeof(double)));
   for(df=2*inSize0+1;df>inSize0;df--) 
     {
     sq[df]=maxDist;
@@ -491,7 +487,7 @@ void vtkImageEuclideanDistanceExecuteSaitoCached(
           
           if(buff[idx0]>buffer+sq[1]) 
             {
-            b=(int)(floor)((((buff[idx0]-buffer)/spacing)-1)/2); 
+            b=static_cast<int>(floor((((buff[idx0]-buffer)/spacing)-1)/2));
             if((idx0+b)>outMax0) {b=(outMax0)-idx0;}
                       
             for(n=a;n<=b;n++) 
@@ -521,7 +517,7 @@ void vtkImageEuclideanDistanceExecuteSaitoCached(
           if(a>0) {a--;}
           if(buff[idx0]>buffer+sq[1]) 
             {
-            b=(int)(floor)((((buff[idx0]-buffer)/spacing)-1)/2); 
+            b=static_cast<int>(floor((((buff[idx0]-buffer)/spacing)-1)/2));
             if((idx0-b)<outMin0) b=idx0-outMin0;
                     
             for(n=a;n<=b;n++) 
@@ -564,12 +560,24 @@ void vtkImageEuclideanDistance::AllocateOutputScalars(vtkImageData *outData)
   outData->SetExtent(outData->GetWholeExtent());
   outData->AllocateScalars();
 }
+
 //----------------------------------------------------------------------------
 // This method is passed input and output Datas, and executes the
 // EuclideanDistance algorithm to fill the output from the input.
-void vtkImageEuclideanDistance::IterativeExecuteData(vtkImageData *inData,
-                                                     vtkImageData *outData)
+int vtkImageEuclideanDistance::IterativeRequestData(
+  vtkInformation* vtkNotUsed( request ),
+  vtkInformationVector** inputVector,
+  vtkInformationVector* outputVector)
 {
+  vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
+  vtkImageData *inData = vtkImageData::SafeDownCast(
+    inInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+  vtkImageData *outData = vtkImageData::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
+  this->AllocateOutputScalars(outData);
+  
   void *inPtr;
   void *outPtr;
 
@@ -578,34 +586,44 @@ void vtkImageEuclideanDistance::IterativeExecuteData(vtkImageData *inData,
   int outExt[6];
   outData->GetWholeExtent( outExt );
   
-  inPtr = inData->GetScalarPointerForExtent(inData->GetUpdateExtent());
+  inPtr = inData->GetScalarPointerForExtent(
+    inInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT()));
   outPtr = outData->GetScalarPointer();
+   
+  if (!inPtr)
+    {
+    vtkErrorMacro(<< "Execute: No scalars for update extent.")
+    return 1;
+    }
+   
   
   // this filter expects that the output be doubles.
   if (outData->GetScalarType() != VTK_DOUBLE)
     {
     vtkErrorMacro(<< "Execute: Output must be be type double.");
-    return;
+    return 1;
     }
   
   // this filter expects input to have 1 components
   if (outData->GetNumberOfScalarComponents() != 1 )
     {
     vtkErrorMacro(<< "Execute: Cannot handle more than 1 components");
-    return;
+    return 1;
     }
   
   if ( this->GetIteration() == 0 )
     {
     switch (inData->GetScalarType())
       {
-      vtkTemplateMacro6(vtkImageEuclideanDistanceInitialize,
-                        this, 
-                        inData, (VTK_TT *)(inPtr), 
-                        outData, outExt, (double *)(outPtr) );
+      vtkTemplateMacro(
+        vtkImageEuclideanDistanceInitialize(this, 
+                                            inData,
+                                            static_cast<VTK_TT *>(inPtr), 
+                                            outData, outExt, 
+                                            static_cast<double *>(outPtr) ));
       default:
         vtkErrorMacro(<< "Execute: Unknown ScalarType");
-        return;
+        return 1;
       } 
     } 
   else 
@@ -613,10 +631,12 @@ void vtkImageEuclideanDistance::IterativeExecuteData(vtkImageData *inData,
     if( inData != outData )
       switch (inData->GetScalarType())
         {    
-        vtkTemplateMacro6(vtkImageEuclideanDistanceCopyData,
-                          this, 
-                          inData, (VTK_TT *)(inPtr), 
-                          outData, outExt, (double *)(outPtr) );
+        vtkTemplateMacro(
+          vtkImageEuclideanDistanceCopyData(this, 
+                                            inData,
+                                            static_cast<VTK_TT *>(inPtr), 
+                                            outData, outExt, 
+                                            static_cast<double *>(outPtr) ));
         }
     }
   
@@ -625,17 +645,19 @@ void vtkImageEuclideanDistance::IterativeExecuteData(vtkImageData *inData,
     {
     case VTK_EDT_SAITO:
       vtkImageEuclideanDistanceExecuteSaito( this, outData, outExt, 
-                                             (double *)(outPtr) );
+                                             static_cast<double *>(outPtr) );
       break;
     case VTK_EDT_SAITO_CACHED:
       vtkImageEuclideanDistanceExecuteSaitoCached( this, outData, outExt, 
-                                                   (double *)(outPtr) );
+                                                   static_cast<double *>(outPtr) );
       break;
     default:
       vtkErrorMacro(<< "Execute: Unknown Algorithm");
     }
   
   this->UpdateProgress((this->GetIteration()+1.0)/3.0);
+
+  return 1;
 }
 
 
@@ -702,7 +724,6 @@ int vtkImageEuclideanDistance::SplitExtent(int splitExt[6], int startExt[6],
   vtkDebugMacro("  Split Piece: ( " <<splitExt[0]<< ", " <<splitExt[1]<< ", "
                 << splitExt[2] << ", " << splitExt[3] << ", "
                 << splitExt[4] << ", " << splitExt[5] << ")");
-  fflush(stderr);
 
   return total;
 }
