@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageEuclideanDistance.h,v $
+  Module:    vtkImageEuclideanDistance.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -52,7 +52,7 @@ class VTK_IMAGING_EXPORT vtkImageEuclideanDistance : public vtkImageDecomposeFil
 {
 public:
   static vtkImageEuclideanDistance *New();
-  vtkTypeRevisionMacro(vtkImageEuclideanDistance,vtkImageDecomposeFilter);
+  vtkTypeMacro(vtkImageEuclideanDistance,vtkImageDecomposeFilter);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -100,7 +100,9 @@ public:
   void SetAlgorithmToSaitoCached () 
     { this->SetAlgorithm(VTK_EDT_SAITO_CACHED); }   
 
-  void IterativeExecuteData(vtkImageData *in, vtkImageData *out);
+  virtual int IterativeRequestData(vtkInformation*,
+                                   vtkInformationVector**,
+                                   vtkInformationVector*);
   
 protected:
   vtkImageEuclideanDistance();
@@ -114,10 +116,11 @@ protected:
   // Replaces "EnlargeOutputUpdateExtent"
   virtual void AllocateOutputScalars(vtkImageData *outData);
   
-  void ExecuteInformation(vtkImageData *input, vtkImageData *output);
-  void ExecuteInformation()
-    {this->vtkImageIterateFilter::ExecuteInformation();}
-  void ComputeInputUpdateExtent(int inExt[6], int outExt[6]);
+  virtual int IterativeRequestInformation(vtkInformation* in,
+                                          vtkInformation* out);
+  virtual int IterativeRequestUpdateExtent(vtkInformation* in,
+                                           vtkInformation* out);
+
 private:
   vtkImageEuclideanDistance(const vtkImageEuclideanDistance&);  // Not implemented.
   void operator=(const vtkImageEuclideanDistance&);  // Not implemented.
