@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageExport.cxx,v $
+  Module:    vtkImageExport.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,13 +14,13 @@
 =========================================================================*/
 #include "vtkImageExport.h"
 
-#include "vtkImageData.h"
+#include "vtkExecutive.h"
 #include "vtkObjectFactory.h"
+#include "vtkImageData.h"
 
 #include <ctype.h>
 #include <string.h>
 
-vtkCxxRevisionMacro(vtkImageExport, "$Revision: 1.26 $");
 vtkStandardNewMacro(vtkImageExport);
 
 //----------------------------------------------------------------------------
@@ -31,6 +31,8 @@ vtkImageExport::vtkImageExport()
   this->DataDimensions[0] = this->DataDimensions[1] =
     this->DataDimensions[2] = 0;
   this->LastPipelineMTime = 0;
+
+  this->SetNumberOfOutputPorts(0);
 }
 
 //----------------------------------------------------------------------------
@@ -47,25 +49,18 @@ void vtkImageExport::PrintSelf(ostream& os, vtkIndent indent)
      << (this->ImageLowerLeft ? "On\n" : "Off\n");
 }
 
-//----------------------------------------------------------------------------
-void vtkImageExport::SetInput(vtkImageData *input)
-{
-  this->vtkProcessObject::SetNthInput(0, input);
-}
-
-//----------------------------------------------------------------------------
 vtkImageData *vtkImageExport::GetInput()
 {
-  if (this->NumberOfInputs < 1)
+  if (this->GetNumberOfInputConnections(0) < 1)
     {
-    return NULL;
+    return 0;
     }
-  
-  return (vtkImageData *)(this->Inputs[0]);
+  return vtkImageData::SafeDownCast(
+    this->GetExecutive()->GetInputData(0, 0));
 }
 
 //----------------------------------------------------------------------------
-int vtkImageExport::GetDataMemorySize()
+vtkIdType vtkImageExport::GetDataMemorySize()
 {
   vtkImageData *input = this->GetInput();
   if (input == NULL)
@@ -75,7 +70,7 @@ int vtkImageExport::GetDataMemorySize()
 
   input->UpdateInformation();
   int *extent = input->GetWholeExtent();
-  int size = input->GetScalarSize();
+  vtkIdType size = input->GetScalarSize();
   size *= input->GetNumberOfScalarComponents();
   size *= (extent[1] - extent[0] + 1);
   size *= (extent[3] - extent[2] + 1);
@@ -113,10 +108,28 @@ void vtkImageExport::SetExportVoidPointer(void *ptr)
   this->Modified();
 }
 
+//----------------------------------------------------------------------------
+// This is the superclasses style of Execute method.  Convert it into
+// an imaging style Execute method.
+int vtkImageExport::RequestData(
+  vtkInformation* vtkNotUsed( request ),
+  vtkInformationVector** vtkNotUsed( inputVector ),
+  vtkInformationVector* vtkNotUsed( outputVector ))
+{
+  // we are the end of the pipeline, we do nothing
+  return 1;
+}
+
 //----------------------------------------------------------------------------
 // Exports all the data from the input.
 void vtkImageExport::Export(void *output)
 {
+  if (!this->GetPointerToData())
+    {
+    // GetPointerToData() outputs an error message.
+    return;
+    }
+  
   if (this->ImageLowerLeft)
     {
     memcpy(output,this->GetPointerToData(),this->GetDataMemorySize());
@@ -125,22 +138,23 @@ void vtkImageExport::Export(void *output)
     { // flip the image when it is output
     void *ptr = this->GetPointerToData();
     int *extent = this->GetInput()->GetWholeExtent();
-    int xsize = extent[1]-extent[0]+1;
-    int ysize = extent[3]-extent[2]+1;
-    int zsize = extent[5]-extent[4]+1;
+    vtkIdType xsize = extent[1]-extent[0]+1;
+    vtkIdType ysize = extent[3]-extent[2]+1;
+    vtkIdType zsize = extent[5]-extent[4]+1;
     int csize = this->GetInput()->GetScalarSize()* \
                 this->GetInput()->GetNumberOfScalarComponents();
 
-    for (int i = 0; i < zsize; i++)
+    for (vtkIdType i = 0; i < zsize; i++)
       {
-      ptr = (void *)(((char *)ptr) + ysize*xsize*csize);
-      for (int j = 0; j < ysize; j++)
+      ptr = static_cast<void *>(static_cast<char *>(ptr) + ysize*xsize*csize);
+      for (vtkIdType j = 0; j < ysize; j++)
         {
-        ptr = (void *)(((char *)ptr) - xsize*csize);
+        ptr = static_cast<void *>(static_cast<char *>(ptr) - xsize*csize);
         memcpy(output, ptr, xsize*csize);
-        output = (void *)(((char *)output) + xsize*csize);
+        output = static_cast<void *>(
+          static_cast<char *>(output) + xsize*csize);
         }
-      ptr = (void *)(((char *)ptr) + ysize*xsize*csize);
+      ptr = static_cast<void *>(static_cast<char *>(ptr) + ysize*xsize*csize);
       }
     }
 }
@@ -314,11 +328,19 @@ void* vtkImageExport::BufferPointerCallbackFunction(void* userData)
 //----------------------------------------------------------------------------
 void vtkImageExport::UpdateInformationCallback()
 {
-  this->GetInput()->UpdateInformation();
+  if (this->GetInput())
+    {
+    this->GetInput()->UpdateInformation();
+    }
 }
 
 int vtkImageExport::PipelineModifiedCallback()
 {
+  if (!this->GetInput())
+    {
+    return 0;
+    }
+  
   unsigned long mtime = this->GetInput()->GetPipelineMTime();
   if(mtime > this->LastPipelineMTime)
     {
@@ -330,21 +352,50 @@ int vtkImageExport::PipelineModifiedCallback()
 
 int* vtkImageExport::WholeExtentCallback()
 {
-  return this->GetInput()->GetWholeExtent();
+  static int defaultextent[6] = {0,0,0,0,0,0};
+  if (!this->GetInput())
+    {
+    return defaultextent;
+    }
+  else
+    {
+    return this->GetInput()->GetWholeExtent();
+    }
 }
 
 double* vtkImageExport::SpacingCallback()
 {
+  static double defaultspacing[6] = {0.0,0.0,0.0};
+  if (!this->GetInput())
+    {
+    return defaultspacing;
+    }
+  else
+    {
   return this->GetInput()->GetSpacing();
+    }
 }
 
 double* vtkImageExport::OriginCallback()
 {
-  return this->GetInput()->GetOrigin();
+  static double defaultorigin[3] = {0.0,0.0,0.0};
+  if (!this->GetInput())
+    {
+    return defaultorigin;
+    }
+  else
+    {
+    return this->GetInput()->GetOrigin();
+    }
 }
 
 const char* vtkImageExport::ScalarTypeCallback()
 {
+  if (!this->GetInput())
+    {
+    return "unsigned char";
+    }
+  
   switch (this->GetInput()->GetScalarType())
     {
     case VTK_DOUBLE:
@@ -367,6 +418,8 @@ const char* vtkImageExport::ScalarTypeCallback()
       { return "char"; }
     case VTK_UNSIGNED_CHAR:
       { return "unsigned char"; }
+    case VTK_SIGNED_CHAR:
+      { return "signed char"; }
     default:
       { return "<unsupported>"; }
     }
@@ -374,27 +427,55 @@ const char* vtkImageExport::ScalarTypeCallback()
   
 int vtkImageExport::NumberOfComponentsCallback()
 {
-  return this->GetInput()->GetNumberOfScalarComponents();
+  if (!this->GetInput())
+    {
+    return 1;
+    }
+  else
+    {
+    return this->GetInput()->GetNumberOfScalarComponents();
+    }
 }
 
 void vtkImageExport::PropagateUpdateExtentCallback(int* extent)
 {
-  this->GetInput()->SetUpdateExtent(extent);
+  if (this->GetInput())
+    {
+    this->GetInput()->SetUpdateExtent(extent);
+    }
 }
 
 void vtkImageExport::UpdateDataCallback()
 {
-  this->GetInput()->Update();
+  if (this->GetInput())
+    {
+    this->GetInput()->Update();
+    }
 }
 
 int* vtkImageExport::DataExtentCallback()
 {
-  return this->GetInput()->GetExtent();
+  static int defaultextent[6] = {0,0,0,0,0,0};
+  if (!this->GetInput())
+    {
+    return defaultextent;
+    }
+  else
+    {
+    return this->GetInput()->GetExtent();
+    }
 }
 
 void* vtkImageExport::BufferPointerCallback()
 {
-  return this->GetInput()->GetScalarPointer();
+  if (!this->GetInput())
+    {
+    return static_cast<void *>(NULL);
+    }
+  else
+    {
+    return this->GetInput()->GetScalarPointer();
+    }
 }
 
 int vtkImageExport::GetDataNumberOfScalarComponents() 
@@ -414,7 +495,7 @@ int vtkImageExport::GetDataScalarType()
     return VTK_UNSIGNED_CHAR; 
     }
   this->GetInput()->UpdateInformation();
-  return this->GetInput()->GetScalarType(); 
+  return this->GetInput()->GetScalarType();
 }
 
 int *vtkImageExport::GetDataExtent() 
