@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageExtractComponents.cxx,v $
+  Module:    vtkImageExtractComponents.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,16 +15,20 @@
 #include "vtkImageExtractComponents.h"
 
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImageExtractComponents, "$Revision: 1.28 $");
 vtkStandardNewMacro(vtkImageExtractComponents);
 
 //----------------------------------------------------------------------------
 vtkImageExtractComponents::vtkImageExtractComponents()
 {
+  this->SetNumberOfInputPorts(1);
+  this->SetNumberOfOutputPorts(1);
   this->Components[0] = 0;
   this->Components[1] = 1;
   this->Components[2] = 2;
@@ -102,10 +106,14 @@ void vtkImageExtractComponents::SetComponents(int c1)
 
 //----------------------------------------------------------------------------
 // This method tells the superclass that only one component will remain.
-void vtkImageExtractComponents::ExecuteInformation(
-                   vtkImageData *vtkNotUsed(inData), vtkImageData *outData)
+int vtkImageExtractComponents::RequestInformation (
+  vtkInformation       * vtkNotUsed( request ),
+  vtkInformationVector ** vtkNotUsed( inputVector ), 
+  vtkInformationVector * outputVector)
 {
-  outData->SetNumberOfScalarComponents(this->NumberOfComponents);
+  vtkDataObject::SetPointDataActiveScalarInfo(
+    outputVector->GetInformationObject(0), -1, this->NumberOfComponents);
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -117,8 +125,8 @@ void vtkImageExtractComponentsExecute(vtkImageExtractComponents *self,
 {
   int idxR, idxY, idxZ;
   int maxX, maxY, maxZ;
-  int inIncX, inIncY, inIncZ;
-  int outIncX, outIncY, outIncZ;
+  vtkIdType inIncX, inIncY, inIncZ;
+  vtkIdType outIncX, outIncY, outIncZ;
   int cnt, inCnt;
   int offset1, offset2, offset3;
   unsigned long count = 0;
@@ -128,7 +136,7 @@ void vtkImageExtractComponentsExecute(vtkImageExtractComponents *self,
   maxX = outExt[1] - outExt[0];
   maxY = outExt[3] - outExt[2]; 
   maxZ = outExt[5] - outExt[4];
-  target = (unsigned long)((maxZ+1)*(maxY+1)/50.0);
+  target = static_cast<unsigned long>((maxZ+1)*(maxY+1)/50.0);
   target++;
   
   // Get increments to march through data 
@@ -203,7 +211,7 @@ void vtkImageExtractComponentsExecute(vtkImageExtractComponents *self,
 //----------------------------------------------------------------------------
 // This method is passed input and output datas, and executes the
 // ExtractComponents function on each line.  
-void vtkImageExtractComponents::ThreadedExecute(vtkImageData *inData, 
+void vtkImageExtractComponents::ThreadedExecute (vtkImageData *inData, 
                                                 vtkImageData *outData,
                                                 int outExt[6], int id)
 {
@@ -211,14 +219,13 @@ void vtkImageExtractComponents::ThreadedExecute(vtkImageData *inData,
   void *inPtr = inData->GetScalarPointerForExtent(outExt);
   void *outPtr = outData->GetScalarPointerForExtent(outExt);
   
-  vtkDebugMacro(<< "Execute: inData = " << inData 
-                << ", outData = " << outData);
-  
   // this filter expects that input is the same type as output.
   if (inData->GetScalarType() != outData->GetScalarType())
     {
-    vtkErrorMacro(<< "Execute: input ScalarType, " << inData->GetScalarType()
-    << ", must match out ScalarType " << outData->GetScalarType());
+    vtkErrorMacro(<< "Execute: input ScalarType, " 
+                  << inData->GetScalarType()
+                  << ", must match out ScalarType " 
+                  << outData->GetScalarType());
     return;
     }
   
@@ -226,7 +233,7 @@ void vtkImageExtractComponents::ThreadedExecute(vtkImageData *inData,
   max = inData->GetNumberOfScalarComponents();
   for (idx = 0; idx < this->NumberOfComponents; ++idx)
     {
-    if (this->Components[idx] > max)
+    if (this->Components[idx] >= max || this->Components[idx] < 0)
       {
       vtkErrorMacro("Execute: Component " << this->Components[idx]
                     << " is not in input.");
@@ -237,9 +244,11 @@ void vtkImageExtractComponents::ThreadedExecute(vtkImageData *inData,
   // choose which templated function to call.
   switch (inData->GetScalarType())
     {
-    vtkTemplateMacro7(vtkImageExtractComponentsExecute, this, inData, 
-                      (VTK_TT *)(inPtr), outData, (VTK_TT *)(outPtr),
-                      outExt, id);
+    vtkTemplateMacro(
+      vtkImageExtractComponentsExecute(this, inData, 
+                                       static_cast<VTK_TT *>(inPtr), outData, 
+                                       static_cast<VTK_TT *>(outPtr),
+                                       outExt, id));
     default:
       vtkErrorMacro(<< "Execute: Unknown ScalarType");
       return;
