@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageFourierFilter.cxx,v $
+  Module:    vtkImageFourierFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,7 +16,6 @@
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImageFourierFilter, "$Revision: 1.17 $");
 
 
 /*=========================================================================
@@ -62,7 +61,8 @@ void vtkImageFourierFilter::ExecuteFftStep2(vtkImageComplex *p_in,
   fact1.Real = 1.0;
   fact1.Imag = 0.0;
   q.Real = 0.0;
-  q.Imag = -2.0 * 3.141592654 * (float)(fb) / (float)(bsize * 2);
+  q.Imag = -2.0 * 3.141592654 * static_cast<float>(fb)
+    / static_cast<float>(bsize * 2);
   vtkImageComplexExponential(q, q);
   p3 = p_out;
   for(i1 = 0; i1 < N / (bsize * 2); ++i1)
@@ -114,11 +114,11 @@ void vtkImageFourierFilter::ExecuteFftStepN(vtkImageComplex *p_in,
     }
   
   p1 = p_in;
-  p3 = p_out;
   for(i0 = 0; i0 < n; ++i0)
     {
     q.Real = 0.0;
-    q.Imag = -2.0 * 3.141592654 * (float)(i0) * (float)(fb) / (float)(bsize*n);
+    q.Imag = -2.0 * 3.141592654 * static_cast<float>(i0) *
+      static_cast<float>(fb) / static_cast<float>(bsize*n);
     vtkImageComplexExponential(q, q);
     p3 = p_out;
     for(i1 = 0; i1 < N / (bsize * n); ++i1)
@@ -166,8 +166,8 @@ void vtkImageFourierFilter::ExecuteFftForwardBackward(vtkImageComplex *in,
     p1 = in;
     for(idx = 0; idx < N; ++idx)
       {
-      p1->Real = p1->Real / (float)(N);
-      p1->Imag = p1->Imag / (float)(N);
+      p1->Real = p1->Real / static_cast<float>(N);
+      p1->Imag = p1->Imag / static_cast<float>(N);
       ++p1;
       }
     }
