@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageHybridMedian2D.cxx,v $
+  Module:    vtkImageHybridMedian2D.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,12 +15,14 @@
 #include "vtkImageHybridMedian2D.h"
 
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
-#include <vtkstd/vector>
-#include <vtkstd/algorithm>
-#include <vtkstd/numeric>
+#include "vtkStreamingDemandDrivenPipeline.h"
+#include <vector>
+#include <algorithm>
+#include <numeric>
 
-vtkCxxRevisionMacro(vtkImageHybridMedian2D, "$Revision: 1.23 $");
 vtkStandardNewMacro(vtkImageHybridMedian2D);
 
 //----------------------------------------------------------------------------
@@ -35,38 +37,39 @@ vtkImageHybridMedian2D::vtkImageHybridMedian2D()
   this->HandleBoundaries = 1;
 }
 
-
 template <class T>
 void vtkImageHybridMedian2DExecute(vtkImageHybridMedian2D *self,
-                             vtkImageData *inData, T *inPtr2,
-                             vtkImageData *outData, T *outPtr2,
-                             int outExt[6], int id)
+                                   vtkImageData *inData, T *inPtr2,
+                                   vtkImageData *outData, T *outPtr2,
+                                   int outExt[6], int id,
+                                   vtkInformation *inInfo)
 {
   int idx0, idx1, idx2, idxC;
-  int inInc0, inInc1, inInc2;
-  int outInc0, outInc1, outInc2;
+  vtkIdType inInc0, inInc1, inInc2;
+  vtkIdType outInc0, outInc1, outInc2;
   int min0, max0, min1, max1, min2, max2, numComps;
-  int wholeMin0, wholeMax0, wholeMin1, wholeMax1, wholeMin2, wholeMax2;
+  int wholeMin0, wholeMax0, wholeMin1, wholeMax1;
+  int wholeExt[6];
   T *inPtr0, *inPtr1, *inPtrC;
   T *outPtr0, *outPtr1, *outPtrC, *ptr;
   T median1, median2, temp;
-  vtkstd::vector<T> array;
+  std::vector<T> array;
   unsigned long count = 0;
   unsigned long target;
 
-  id = id;
-
-
   inData->GetIncrements(inInc0, inInc1, inInc2);
-  self->GetInput()->GetWholeExtent(wholeMin0, wholeMax0, wholeMin1, wholeMax1,
-                              wholeMin2, wholeMax2);
+  inInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), wholeExt);
+  wholeMin0 = wholeExt[0];
+  wholeMax0 = wholeExt[1];
+  wholeMin1 = wholeExt[2];
+  wholeMax1 = wholeExt[3];
   numComps = inData->GetNumberOfScalarComponents();
   outData->GetIncrements(outInc0, outInc1, outInc2);
   min0 = outExt[0];   max0 = outExt[1];
   min1 = outExt[2];   max1 = outExt[3];
   min2 = outExt[4];   max2 = outExt[5];
 
-  target = (unsigned long)((max2-min2+1)*(max1-min1+1)/50.0);
+  target = static_cast<unsigned long>((max2-min2+1)*(max1-min1+1)/50.0);
   target++;
 
   for (idx2 = min2; idx2 <= max2; ++idx2)
@@ -148,7 +151,7 @@ void vtkImageHybridMedian2DExecute(vtkImageHybridMedian2D *self,
             array.push_back( *ptr );
             }
 
-          vtkstd::sort(array.begin(),array.end());
+          std::sort(array.begin(),array.end());
           median1 = array[static_cast<unsigned int>(0.5*array.size())];
 
           // compute median of x neighborhood
@@ -206,7 +209,7 @@ void vtkImageHybridMedian2DExecute(vtkImageHybridMedian2D *self,
             array.push_back( *ptr );
             }
 
-          vtkstd::sort(array.begin(),array.end());
+          std::sort(array.begin(),array.end());
           median2 = array[static_cast<unsigned int>(0.5*array.size())];
 
           // Compute the median of the three. (med1, med2 and center)
@@ -245,34 +248,38 @@ void vtkImageHybridMedian2DExecute(vtkImageHybridMedian2D *self,
 // This method contains the first switch statement that calls the correct
 // templated function for the input and output Data types.
 // It hanldes image boundaries, so the image does not shrink.
-void vtkImageHybridMedian2D::ThreadedExecute(vtkImageData *inData,
-                                       vtkImageData *outData,
-                                       int outExt[6], int id)
+void vtkImageHybridMedian2D::ThreadedRequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *vtkNotUsed(outputVector),
+  vtkImageData ***inData,
+  vtkImageData **outData,
+  int outExt[6], int id)
 {
-  void *inPtr = inData->GetScalarPointerForExtent(outExt);
-  void *outPtr = outData->GetScalarPointerForExtent(outExt);
+  void *inPtr = inData[0][0]->GetScalarPointerForExtent(outExt);
+  void *outPtr = outData[0]->GetScalarPointerForExtent(outExt);
+
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
 
   // this filter expects the output type to be same as input
-  if (outData->GetScalarType() != inData->GetScalarType())
+  if (outData[0]->GetScalarType() != inData[0][0]->GetScalarType())
     {
     vtkErrorMacro(<< "Execute: output ScalarType, "
-      << vtkImageScalarTypeNameMacro(outData->GetScalarType())
+      << vtkImageScalarTypeNameMacro(outData[0]->GetScalarType())
       << " must match input scalar type");
     return;
     }
 
-  switch (inData->GetScalarType())
+  switch (inData[0][0]->GetScalarType())
     {
-    vtkTemplateMacro7(vtkImageHybridMedian2DExecute, this, inData,
-                      (VTK_TT *)(inPtr), outData, (VTK_TT *)(outPtr),
-                      outExt, id);
+    vtkTemplateMacro(
+      vtkImageHybridMedian2DExecute( this, inData[0][0],
+                                     static_cast<VTK_TT *>(inPtr), outData[0], 
+                                     static_cast<VTK_TT *>(outPtr),
+                                     outExt, id, inInfo));
 
     default:
       vtkErrorMacro(<< "Execute: Unknown ScalarType");
       return;
     }
 }
-
-
-
-
