@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageIdealLowPass.cxx,v $
+  Module:    vtkImageIdealLowPass.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,11 +14,13 @@
 =========================================================================*/
 #include "vtkImageIdealLowPass.h"
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImageIdealLowPass, "$Revision: 1.19 $");
 vtkStandardNewMacro(vtkImageIdealLowPass);
 
 //----------------------------------------------------------------------------
@@ -61,18 +63,22 @@ void vtkImageIdealLowPass::SetZCutOff(double cutOff)
 
 
 //----------------------------------------------------------------------------
-void vtkImageIdealLowPass::ThreadedExecute(vtkImageData *inData, 
-                                           vtkImageData *outData,
-                                           int ext[6], int id)
+void vtkImageIdealLowPass::ThreadedRequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *vtkNotUsed(outputVector),
+  vtkImageData ***inData, 
+  vtkImageData **outData,
+  int ext[6], int id)
 {
   int idx0, idx1, idx2;
   int min0, max0;
   double *inPtr;
   double *outPtr;
-  int *wholeExtent;
-  double *spacing;
-  int inInc0, inInc1, inInc2;
-  int outInc0, outInc1, outInc2;
+  int wholeExtent[6];
+  double spacing[3];
+  vtkIdType inInc0, inInc1, inInc2;
+  vtkIdType outInc0, outInc1, outInc2;
   double temp0, temp1, temp2, mid0, mid1, mid2;
   // normalization factors
   double norm0, norm1, norm2;
@@ -80,34 +86,36 @@ void vtkImageIdealLowPass::ThreadedExecute(vtkImageData *inData,
   unsigned long count = 0;
   unsigned long target;
 
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+
   // Error checking
-  if (inData->GetNumberOfScalarComponents() != 2)
+  if (inData[0][0]->GetNumberOfScalarComponents() != 2)
     {
     vtkErrorMacro("Expecting 2 components not " 
-                  << inData->GetNumberOfScalarComponents());
+                  << inData[0][0]->GetNumberOfScalarComponents());
     return;
     }
-  if (inData->GetScalarType() != VTK_DOUBLE || 
-      outData->GetScalarType() != VTK_DOUBLE)
+  if (inData[0][0]->GetScalarType() != VTK_DOUBLE || 
+      outData[0]->GetScalarType() != VTK_DOUBLE)
     {
     vtkErrorMacro("Expecting input and output to be of type double");
     return;
     }
   
-  wholeExtent = this->GetInput()->GetWholeExtent();
-  spacing = inData->GetSpacing();
+  inInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), wholeExtent);
+  inData[0][0]->GetSpacing(spacing);
 
-  inPtr = (double *)(inData->GetScalarPointerForExtent(ext));
-  outPtr = (double *)(outData->GetScalarPointerForExtent(ext));
+  inPtr = static_cast<double *>(inData[0][0]->GetScalarPointerForExtent(ext));
+  outPtr = static_cast<double *>(outData[0]->GetScalarPointerForExtent(ext));
 
-  inData->GetContinuousIncrements(ext, inInc0, inInc1, inInc2);
-  outData->GetContinuousIncrements(ext, outInc0, outInc1, outInc2);  
+  inData[0][0]->GetContinuousIncrements(ext, inInc0, inInc1, inInc2);
+  outData[0]->GetContinuousIncrements(ext, outInc0, outInc1, outInc2);  
   
   min0 = ext[0];
   max0 = ext[1];
-  mid0 = (double)(wholeExtent[0] + wholeExtent[1] + 1) / 2.0;
-  mid1 = (double)(wholeExtent[2] + wholeExtent[3] + 1) / 2.0;
-  mid2 = (double)(wholeExtent[4] + wholeExtent[5] + 1) / 2.0;
+  mid0 = static_cast<double>(wholeExtent[0] + wholeExtent[1] + 1) / 2.0;
+  mid1 = static_cast<double>(wholeExtent[2] + wholeExtent[3] + 1) / 2.0;
+  mid2 = static_cast<double>(wholeExtent[4] + wholeExtent[5] + 1) / 2.0;
   if ( this->CutOff[0] == 0.0)
     {
     norm0 = VTK_DOUBLE_MAX;
@@ -133,14 +141,15 @@ void vtkImageIdealLowPass::ThreadedExecute(vtkImageData *inData,
     norm2 = 1.0 / ((spacing[2] * 2.0 * mid2) * this->CutOff[2]);
     }
   
-  target = (unsigned long)((ext[5]-ext[4]+1)*(ext[3]-ext[2]+1)/50.0);
+  target = static_cast<unsigned long>(
+    (ext[5]-ext[4]+1)*(ext[3]-ext[2]+1)/50.0);
   target++;
 
   // loop over all the pixels (keeping track of normalized distance to origin.
   for (idx2 = ext[4]; idx2 <= ext[5]; ++idx2)
     {
     // distance to min (this axis' contribution)
-    temp2 = (double)idx2;
+    temp2 = static_cast<double>(idx2);
     // Wrap back to 0.
     if (temp2 > mid2)
       {
@@ -160,7 +169,7 @@ void vtkImageIdealLowPass::ThreadedExecute(vtkImageData *inData,
         count++;
         }
       // distance to min (this axis' contribution)
-      temp1 = (double)idx1;
+      temp1 = static_cast<double>(idx1);
       // Wrap back to 0.
       if (temp1 > mid1)
         {
@@ -173,7 +182,7 @@ void vtkImageIdealLowPass::ThreadedExecute(vtkImageData *inData,
       for (idx0 = min0; idx0 <= max0; ++idx0)
         {
         // distance to min (this axis' contribution)
-        temp0 = (double)idx0;
+        temp0 = static_cast<double>(idx0);
         // Wrap back to 0.
         if (temp0 > mid0)
           {
