@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageLuminance.cxx,v $
+  Module:    vtkImageLuminance.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,19 +16,32 @@
 
 #include "vtkImageData.h"
 #include "vtkImageProgressIterator.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImageLuminance, "$Revision: 1.22 $");
 vtkStandardNewMacro(vtkImageLuminance);
 
 //----------------------------------------------------------------------------
-// This method overrides information set by parent's ExecuteInformation.
-void vtkImageLuminance::ExecuteInformation(vtkImageData *vtkNotUsed(inData), 
-                                           vtkImageData *outData)
+vtkImageLuminance::vtkImageLuminance()
 {
-  outData->SetNumberOfScalarComponents(1);
+  this->SetNumberOfInputPorts(1);
+  this->SetNumberOfOutputPorts(1);
+}
+
+//----------------------------------------------------------------------------
+// This method overrides information set by parent's ExecuteInformation.
+int vtkImageLuminance::RequestInformation (
+  vtkInformation       * vtkNotUsed( request ),
+  vtkInformationVector** vtkNotUsed( inputVector ),
+  vtkInformationVector * outputVector)
+{
+  vtkDataObject::SetPointDataActiveScalarInfo(
+    outputVector->GetInformationObject(0), -1, 1);
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -69,7 +82,7 @@ void vtkImageLuminanceExecute(vtkImageLuminance *self, vtkImageData *inData,
 // This method contains a switch statement that calls the correct
 // templated function for the input data type.  The output data
 // must match input type.  This method does handle boundary conditions.
-void vtkImageLuminance::ThreadedExecute(vtkImageData *inData, 
+void vtkImageLuminance::ThreadedExecute (vtkImageData *inData, 
                                         vtkImageData *outData,
                                         int outExt[6], int id)
 {
@@ -79,7 +92,8 @@ void vtkImageLuminance::ThreadedExecute(vtkImageData *inData,
   // this filter expects that input is the same type as output.
   if (inData->GetNumberOfScalarComponents() != 3)
     {
-    vtkErrorMacro(<< "Execute: input must have 3 components, but has " << inData->GetNumberOfScalarComponents());
+    vtkErrorMacro(<< "Execute: input must have 3 components, but has " 
+                  << inData->GetNumberOfScalarComponents());
     return;
     }
   
@@ -93,8 +107,9 @@ void vtkImageLuminance::ThreadedExecute(vtkImageData *inData,
   
   switch (inData->GetScalarType())
     {
-    vtkTemplateMacro6(vtkImageLuminanceExecute, this, inData, outData, 
-                      outExt, id, static_cast<VTK_TT *>(0));
+    vtkTemplateMacro(
+      vtkImageLuminanceExecute( this, inData, outData, 
+                                outExt, id, static_cast<VTK_TT *>(0)));
     default:
       vtkErrorMacro(<< "Execute: Unknown ScalarType");
       return;
