@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageLuminance.h,v $
+  Module:    vtkImageLuminance.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -20,22 +20,24 @@
 #define __vtkImageLuminance_h
 
 
-#include "vtkImageToImageFilter.h"
+#include "vtkThreadedImageAlgorithm.h"
 
-class VTK_IMAGING_EXPORT vtkImageLuminance : public vtkImageToImageFilter
+class VTK_IMAGING_EXPORT vtkImageLuminance : public vtkThreadedImageAlgorithm
 {
 public:
   static vtkImageLuminance *New();
-  vtkTypeRevisionMacro(vtkImageLuminance,vtkImageToImageFilter);
+  vtkTypeMacro(vtkImageLuminance,vtkThreadedImageAlgorithm);
 
 protected:
-  vtkImageLuminance() {};
+  vtkImageLuminance();
   ~vtkImageLuminance() {};
   
-  void ExecuteInformation(vtkImageData *inData, vtkImageData *outData);
-  void ExecuteInformation(){this->vtkImageToImageFilter::ExecuteInformation();};
-  void ThreadedExecute(vtkImageData *inData, vtkImageData *outData, 
-                       int ext[6], int id);
+  virtual int RequestInformation (vtkInformation *, vtkInformationVector**,
+                                  vtkInformationVector *);
+
+  void ThreadedExecute (vtkImageData *inData, vtkImageData *outData,
+                        int outExt[6], int id);
+
 private:
   vtkImageLuminance(const vtkImageLuminance&);  // Not implemented.
   void operator=(const vtkImageLuminance&);  // Not implemented.
