@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageMagnify.cxx,v $
+  Module:    vtkImageMagnify.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,9 +15,11 @@
 #include "vtkImageMagnify.h"
 
 #include "vtkImageData.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
-vtkCxxRevisionMacro(vtkImageMagnify, "$Revision: 1.44 $");
 vtkStandardNewMacro(vtkImageMagnify);
 
 //----------------------------------------------------------------------------
@@ -31,20 +33,25 @@ vtkImageMagnify::vtkImageMagnify()
   this->MagnificationFactors[2] = 1;
 }
 
-
 //----------------------------------------------------------------------------
 // Computes any global image information associated with regions.
-void vtkImageMagnify::ExecuteInformation(vtkImageData *inData, 
-                                         vtkImageData *outData)
+int vtkImageMagnify::RequestInformation (
+  vtkInformation * vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
-  double *spacing;
+  // get the info objects
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+
+  double spacing[3];
   int idx;
-  int *inExt;
   double outSpacing[3];
-  int outExt[6];
+  int inExt[6], outExt[6];
+  
+  inInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), inExt);
+  inInfo->Get(vtkDataObject::SPACING(), spacing);
   
-  inExt = inData->GetWholeExtent();
-  spacing = inData->GetSpacing();
   for (idx = 0; idx < 3; idx++)
     {
     // Scale the output extent
@@ -53,35 +60,54 @@ void vtkImageMagnify::ExecuteInformation(vtkImageData *inData,
       (inExt[idx*2+1] - inExt[idx*2] + 1)*this->MagnificationFactors[idx] - 1;
     
     // Change the data spacing
-    outSpacing[idx] = spacing[idx] / (double)(this->MagnificationFactors[idx]);
+    outSpacing[idx] = spacing[idx] /
+      static_cast<double>(this->MagnificationFactors[idx]);
     }
   
-  outData->SetWholeExtent(outExt);
-  outData->SetSpacing(outSpacing);
+  outInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),outExt,6);
+  outInfo->Set(vtkDataObject::SPACING(),outSpacing,3);
 
+  return 1;
 }
 
 //----------------------------------------------------------------------------
 // This method computes the Region of input necessary to generate outRegion.
 // It assumes offset and size are multiples of Magnify Factors.
-void vtkImageMagnify::ComputeInputUpdateExtent(int inExt[6],
-                                               int outExt[6])
+int vtkImageMagnify::RequestUpdateExtent (
+  vtkInformation * vtkNotUsed(request),
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
+{
+  // get the info objects
+  vtkInformation* outInfo = outputVector->GetInformationObject(0);
+  vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+
+  int outExt[6], inExt[6];
+  outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(), outExt);
+
+  this->InternalRequestUpdateExtent(inExt, outExt);
+  
+  inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT(), inExt, 6);
+
+  return 1;
+}
+
+void vtkImageMagnify::InternalRequestUpdateExtent(int *inExt, int *outExt)
 {
   int idx;
-  
+
   for (idx = 0; idx < 3; idx++)
     {
     // For Min. Round Down
-    inExt[idx*2] = (int)(floor((double)(outExt[idx*2]) / 
-                               (double)(this->MagnificationFactors[idx])));
-    inExt[idx*2+1] = (int)(floor((double)(outExt[idx*2+1]) / 
-                                 (double)(this->MagnificationFactors[idx])));
+    inExt[idx*2] = static_cast<int>(
+      floor(static_cast<double>(outExt[idx*2]) / 
+            static_cast<double>(this->MagnificationFactors[idx])));
+    inExt[idx*2+1] = static_cast<int>(
+      floor(static_cast<double>(outExt[idx*2+1]) / 
+            static_cast<double>(this->MagnificationFactors[idx])));
     }
 }
 
-
-
-
 //----------------------------------------------------------------------------
 // The templated execute function handles all the data types.
 // 2d even though operation is 1d.
@@ -96,8 +122,8 @@ void vtkImageMagnifyExecute(vtkImageMagnify *self,
   int inIdxX, inIdxY, inIdxZ;
   int inMaxX, inMaxY, inMaxZ;
   int maxC, maxX, maxY, maxZ;
-  int inIncX, inIncY, inIncZ;
-  int outIncX, outIncY, outIncZ;
+  vtkIdType inIncX, inIncY, inIncZ;
+  vtkIdType outIncX, outIncY, outIncZ;
   unsigned long count = 0;
   unsigned long target;
   int interpolate;
@@ -121,7 +147,7 @@ void vtkImageMagnifyExecute(vtkImageMagnify *self,
   maxX = outExt[1] - outExt[0];
   maxY = outExt[3] - outExt[2]; 
   maxZ = outExt[5] - outExt[4];
-  target = (unsigned long)(maxC*(maxZ+1)*(maxY+1)/50.0);
+  target = static_cast<unsigned long>(maxC*(maxZ+1)*(maxY+1)/50.0);
   target++;
   
   // Get increments to march through data 
@@ -226,15 +252,15 @@ void vtkImageMagnifyExecute(vtkImageMagnify *self,
               dataPXYZ = *(inPtrX + tiX + tiY + tiZ); 
               interpSetup = 1;
               }
-            *outPtrC = (T)
-              ((double)dataP*(magXIdx + 1)*iMagP + 
-               (double)dataPX*(magX - magXIdx - 1)*iMagP +
-               (double)dataPY*(magXIdx + 1)*iMagPY + 
-               (double)dataPXY*(magX - magXIdx - 1)*iMagPY +
-               (double)dataPZ*(magXIdx + 1)*iMagPZ + 
-               (double)dataPXZ*(magX - magXIdx - 1)*iMagPZ +
-               (double)dataPYZ*(magXIdx + 1)*iMagPYZ + 
-               (double)dataPXYZ*(magX - magXIdx - 1)*iMagPYZ);
+            *outPtrC = static_cast<T>
+              (static_cast<double>(dataP)*(magXIdx + 1)*iMagP + 
+               static_cast<double>(dataPX)*(magX - magXIdx - 1)*iMagP +
+               static_cast<double>(dataPY)*(magXIdx + 1)*iMagPY + 
+               static_cast<double>(dataPXY)*(magX - magXIdx - 1)*iMagPY +
+               static_cast<double>(dataPZ)*(magXIdx + 1)*iMagPZ + 
+               static_cast<double>(dataPXZ)*(magX - magXIdx - 1)*iMagPZ +
+               static_cast<double>(dataPYZ)*(magXIdx + 1)*iMagPYZ + 
+               static_cast<double>(dataPXYZ)*(magX - magXIdx - 1)*iMagPYZ);
             }
           outPtrC += maxC;
           if (!magXIdx) 
@@ -264,30 +290,37 @@ void vtkImageMagnifyExecute(vtkImageMagnify *self,
     }
 }
 
-void vtkImageMagnify::ThreadedExecute(vtkImageData *inData, 
-                                      vtkImageData *outData,
-                                      int outExt[6], int id)
+void vtkImageMagnify::ThreadedRequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *vtkNotUsed(outputVector),
+  vtkImageData ***inData,
+  vtkImageData **outData,
+  int outExt[6], int id)
 {
   int inExt[6];
-  this->ComputeInputUpdateExtent(inExt,outExt);
-  void *inPtr = inData->GetScalarPointerForExtent(inExt);
-  void *outPtr = outData->GetScalarPointerForExtent(outExt);
-  
-  vtkDebugMacro(<< "Execute: inData = " << inData 
-  << ", outData = " << outData);
+  this->InternalRequestUpdateExtent(inExt, outExt);
+
+  void *inPtr = inData[0][0]->GetScalarPointerForExtent(inExt);
+  void *outPtr = outData[0]->GetScalarPointerForExtent(outExt);
   
   // this filter expects that input is the same type as output.
-  if (inData->GetScalarType() != outData->GetScalarType())
+  if (inData[0][0]->GetScalarType() != outData[0]->GetScalarType())
     {
-    vtkErrorMacro(<< "Execute: input ScalarType, " << inData->GetScalarType()
-    << ", must match out ScalarType " << outData->GetScalarType());
+    vtkErrorMacro("Execute: input ScalarType, "
+                  << inData[0][0]->GetScalarType()
+                  << ", must match out ScalarType "
+                  << outData[0]->GetScalarType());
     return;
     }
   
-  switch (inData->GetScalarType())
+  switch (inData[0][0]->GetScalarType())
     {
-    vtkTemplateMacro8(vtkImageMagnifyExecute, this, inData, (VTK_TT *)(inPtr),
-                      inExt, outData, (VTK_TT *)(outPtr), outExt, id);
+    vtkTemplateMacro(
+      vtkImageMagnifyExecute(this, inData[0][0],
+                             static_cast<VTK_TT *>(inPtr), inExt, outData[0], 
+                             static_cast<VTK_TT *>(outPtr),
+                             outExt, id));
     default:
       vtkErrorMacro(<< "Execute: Unknown ScalarType");
       return;
@@ -304,6 +337,4 @@ void vtkImageMagnify::PrintSelf(ostream& os, vtkIndent indent)
      << this->MagnificationFactors[2] << " )\n";
 
   os << indent << "Interpolate: " << (this->Interpolate ? "On\n" : "Off\n");
-
 }
-
