@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkImageMagnitude.cxx,v $
+  Module:    vtkImageMagnitude.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,22 +14,34 @@
 =========================================================================*/
 #include "vtkImageMagnitude.h"
 
+#include "vtkDataArray.h"
 #include "vtkImageData.h"
 #include "vtkImageProgressIterator.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImageMagnitude, "$Revision: 1.36 $");
 vtkStandardNewMacro(vtkImageMagnitude);
 
 //----------------------------------------------------------------------------
-// This method tells the superclass that the first axis will collapse.
-void vtkImageMagnitude::ExecuteInformation(vtkImageData *vtkNotUsed(inData), 
-                                           vtkImageData *outData)
+vtkImageMagnitude::vtkImageMagnitude()
 {
-  outData->SetNumberOfScalarComponents(1);
+  this->SetNumberOfInputPorts(1);
+  this->SetNumberOfOutputPorts(1);
+}
+
+int vtkImageMagnitude::RequestInformation (
+  vtkInformation       * vtkNotUsed( request ),
+  vtkInformationVector ** vtkNotUsed( inputVector ),
+  vtkInformationVector * outputVector)
+{
+  vtkDataObject::SetPointDataActiveScalarInfo(
+    outputVector->GetInformationObject(0), -1, 1);
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -78,7 +90,7 @@ void vtkImageMagnitudeExecute(vtkImageMagnitude *self,
 // This method contains a switch statement that calls the correct
 // templated function for the input data type.  The output data
 // must match input type.  This method does handle boundary conditions.
-void vtkImageMagnitude::ThreadedExecute(vtkImageData *inData, 
+void vtkImageMagnitude::ThreadedExecute (vtkImageData *inData, 
                                         vtkImageData *outData,
                                         int outExt[6], int id)
 {
@@ -101,8 +113,9 @@ void vtkImageMagnitude::ThreadedExecute(vtkImageData *inData,
   
   switch (inData->GetScalarType())
     {
-    vtkTemplateMacro6(vtkImageMagnitudeExecute, this, inData, outData, 
-                      outExt, id, static_cast<VTK_TT *>(0));
+    vtkTemplateMacro(
+      vtkImageMagnitudeExecute( this, inData, outData, 
+                                outExt, id, static_cast<VTK_TT *>(0)));
     default:
       vtkErrorMacro(<< "Execute: Unknown ScalarType");
       return;
